/* Generated by Cython 0.29.26 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/core/include/numpy/arrayobject.h",
            "/tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/core/include/numpy/ufuncobject.h"
        ],
        "include_dirs": [
            "/tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/core/include",
            "/tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/ms_peak_picker/_c"
        ],
        "name": "ms_deisotope._c.peak_set",
        "sources": [
            "ms_deisotope/_c/peak_set.pyx"
        ]
    },
    "module_name": "ms_deisotope._c.peak_set"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_26"
#define CYTHON_HEX_VERSION 0x001D1AF0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030B00A1)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__ms_deisotope___c__peak_set
#define __PYX_HAVE_API__ms_deisotope___c__peak_set
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"

    /* NumPy API declarations from "numpy/__init__.pxd" */
    
#include <math.h>
#include <stdlib.h>
#include "pythread.h"
#include "pystate.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "ms_deisotope/_c/peak_set.pyx",
  "ms_deisotope/_c/peak_set.pxd",
  "__init__.pxd",
  "stringsource",
  "composition.pxd",
  "isotopic_distribution.pxd",
  "peak_set.pxd",
  "type.pxd",
  "peak_index.pxd",
  "ms_deisotope/_c/averagine.pxd",
  "ms_deisotope/_c/deconvoluter_base.pxd",
  "ms_deisotope/_c/scoring.pxd",
};
/* BufferFormatStructs.proto */
#define IS_UNSIGNED(type) (((type) -1) > 0)
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* Atomics.proto */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __pyx_atomic_int_type int
#if CYTHON_ATOMICS && __GNUC__ >= 4 && (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL >= 2)) &&\
                    !defined(__i386__)
    #define __pyx_atomic_incr_aligned(value, lock) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_aligned(value, lock) __sync_fetch_and_sub(value, 1)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER) && 0
    #include <Windows.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type LONG
    #define __pyx_atomic_incr_aligned(value, lock) InterlockedIncrement(value)
    #define __pyx_atomic_decr_aligned(value, lock) InterlockedDecrement(value)
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#elif CYTHON_ATOMICS && (defined(__ICC) || defined(__INTEL_COMPILER)) && 0
    #define __pyx_atomic_incr_aligned(value, lock) _InterlockedIncrement(value)
    #define __pyx_atomic_decr_aligned(value, lock) _InterlockedDecrement(value)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using Intel atomics"
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif
typedef volatile __pyx_atomic_int_type __pyx_atomic_int;
#if CYTHON_ATOMICS
    #define __pyx_add_acquisition_count(memview)\
             __pyx_atomic_incr_aligned(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_atomic_decr_aligned(__pyx_get_slice_count_pointer(memview), memview->lock)
#else
    #define __pyx_add_acquisition_count(memview)\
            __pyx_add_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_sub_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
#endif

/* MemviewSliceStruct.proto */
struct __pyx_memoryview_obj;
typedef struct {
  struct __pyx_memoryview_obj *memview;
  char *data;
  Py_ssize_t shape[8];
  Py_ssize_t strides[8];
  Py_ssize_t suboffsets[8];
} __Pyx_memviewslice;
#define __Pyx_MemoryView_Len(m)  (m.shape[0])


/* "brainpy/_c/composition.pxd":3
 * from brainpy.mass_dict import nist_mass as __nist_mass
 * 
 * ctypedef long count_type             # <<<<<<<<<<<<<<
 * 
 * cdef dict nist_mass
 */
typedef long __pyx_t_7brainpy_2_c_11composition_count_type;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":689
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":690
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":691
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":692
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":696
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":697
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":698
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":699
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":703
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":704
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":713
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":714
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":715
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":717
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":718
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":719
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":721
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":722
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":724
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":725
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":726
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/
struct __pyx_obj_7brainpy_2_c_11composition_PyComposition;
struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak;
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase;
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak;
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet;
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed;
struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex;
struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine;
struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern;
struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache;
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase;
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase;
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase;
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_LeastSquaresFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_GTestFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledGTestFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_MSDeconVFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_FunctionScorer;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_InterferenceDetection;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_DistinctPatternFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_DotProductFitter;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr;
struct __pyx_array_obj;
struct __pyx_MemviewEnum_obj;
struct __pyx_memoryview_obj;
struct __pyx_memoryviewslice_obj;
struct __pyx_opt_args_7brainpy_2_c_11composition_neutral_mass;
struct __pyx_opt_args_7brainpy_2_c_11composition_mass_charge_ratio;
struct __pyx_t_7brainpy_2_c_11composition_Isotope;
struct __pyx_t_7brainpy_2_c_11composition_IsotopeMap;
struct __pyx_t_7brainpy_2_c_11composition_Element;
struct __pyx_t_7brainpy_2_c_11composition_ElementHashBucket;
struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable;
struct __pyx_t_7brainpy_2_c_11composition_Composition;
struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition__set_impl;
struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition_pop;

/* "brainpy/_c/composition.pxd":10
 * cdef double PROTON = nist_mass["H+"][0][0]
 * 
 * cdef double neutral_mass(double mz,  int z, double charge_carrier=*) nogil             # <<<<<<<<<<<<<<
 * cdef double mass_charge_ratio(double neutral_mass, int z, double charge_carrier=*) nogil
 * cdef char* _parse_isotope_string(char* label, int* isotope_num, char* element_name) nogil
 */
struct __pyx_opt_args_7brainpy_2_c_11composition_neutral_mass {
  int __pyx_n;
  double charge_carrier;
};

/* "brainpy/_c/composition.pxd":11
 * 
 * cdef double neutral_mass(double mz,  int z, double charge_carrier=*) nogil
 * cdef double mass_charge_ratio(double neutral_mass, int z, double charge_carrier=*) nogil             # <<<<<<<<<<<<<<
 * cdef char* _parse_isotope_string(char* label, int* isotope_num, char* element_name) nogil
 * 
 */
struct __pyx_opt_args_7brainpy_2_c_11composition_mass_charge_ratio {
  int __pyx_n;
  double charge_carrier;
};

/* "brainpy/_c/composition.pxd":17
 * # Isotope and IsotopeMap Declarations
 * 
 * cdef struct Isotope:             # <<<<<<<<<<<<<<
 *     double mass
 *     double abundance
 */
struct __pyx_t_7brainpy_2_c_11composition_Isotope {
  double mass;
  double abundance;
  int neutrons;
  int neutron_shift;
};

/* "brainpy/_c/composition.pxd":23
 *     int neutron_shift
 * 
 * cdef struct IsotopeMap:             # <<<<<<<<<<<<<<
 *     Isotope* bins
 *     size_t size
 */
struct __pyx_t_7brainpy_2_c_11composition_IsotopeMap {
  struct __pyx_t_7brainpy_2_c_11composition_Isotope *bins;
  size_t size;
};

/* "brainpy/_c/composition.pxd":36
 * # Element Declarations
 * 
 * cdef struct Element:             # <<<<<<<<<<<<<<
 *     char* symbol
 *     IsotopeMap* isotopes
 */
struct __pyx_t_7brainpy_2_c_11composition_Element {
  char *symbol;
  struct __pyx_t_7brainpy_2_c_11composition_IsotopeMap *isotopes;
  int monoisotopic_isotope_index;
};

/* "brainpy/_c/composition.pxd":56
 * # ElementHashTable and ElementHashBucket Declarations
 * 
 * cdef struct ElementHashBucket:             # <<<<<<<<<<<<<<
 *     Element** elements
 *     size_t used
 */
struct __pyx_t_7brainpy_2_c_11composition_ElementHashBucket {
  struct __pyx_t_7brainpy_2_c_11composition_Element **elements;
  size_t used;
  size_t size;
};

/* "brainpy/_c/composition.pxd":65
 * 
 * 
 * cdef struct ElementHashTable:             # <<<<<<<<<<<<<<
 *     ElementHashBucket* buckets
 *     size_t size
 */
struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable {
  struct __pyx_t_7brainpy_2_c_11composition_ElementHashBucket *buckets;
  size_t size;
};

/* "brainpy/_c/composition.pxd":94
 * # Composition Declarations
 * 
 * cdef struct Composition:             # <<<<<<<<<<<<<<
 *     char** elements
 *     count_type* counts
 */
struct __pyx_t_7brainpy_2_c_11composition_Composition {
  char **elements;
  __pyx_t_7brainpy_2_c_11composition_count_type *counts;
  size_t size;
  size_t used;
  int max_variants;
};

/* "brainpy/_c/composition.pxd":130
 *     @staticmethod
 *     cdef PyComposition _create(Composition* base)
 *     cdef void _set_impl(self, Composition* composition, bint free_existing=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef void _initialize_from_formula(self, str formula)
 */
struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition__set_impl {
  int __pyx_n;
  int free_existing;
};

/* "brainpy/_c/composition.pxd":144
 *     cpdef list items(self)
 * 
 *     cpdef pop(self, str key, object default=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef count_type getitem(self, str key)
 */
struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition_pop {
  int __pyx_n;
  PyObject *__pyx_default;
};
struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector;

/* "brainpy/_c/double_vector.pxd":1
 * cdef struct DoubleVector:             # <<<<<<<<<<<<<<
 *     double* v
 *     size_t used
 */
struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector {
  double *v;
  size_t used;
  size_t size;
};
struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters;
struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants;
struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants;

/* "brainpy/_c/isotopic_constants.pxd":6
 * from brainpy._c.double_vector cimport DoubleVector
 * 
 * ctypedef DoubleVector dvec             # <<<<<<<<<<<<<<
 * 
 * 
 */
typedef struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector __pyx_t_7brainpy_2_c_18isotopic_constants_dvec;

/* "brainpy/_c/isotopic_constants.pxd":9
 * 
 * 
 * cdef struct PolynomialParameters:             # <<<<<<<<<<<<<<
 *     dvec* elementary_symmetric_polynomial
 *     dvec* power_sum
 */
struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters {
  __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *elementary_symmetric_polynomial;
  __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *power_sum;
};

/* "brainpy/_c/isotopic_constants.pxd":14
 * 
 * 
 * cdef struct PhiConstants:             # <<<<<<<<<<<<<<
 *     int order
 *     Element* element
 */
struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants {
  int order;
  struct __pyx_t_7brainpy_2_c_11composition_Element *element;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *element_coefficients;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *mass_coefficients;
};

/* "brainpy/_c/isotopic_constants.pxd":21
 * 
 * 
 * cdef struct IsotopicConstants:             # <<<<<<<<<<<<<<
 *     int order
 *     PhiConstants** constants
 */
struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants {
  int order;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants **constants;
  size_t size;
  size_t used;
};
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_Peak;
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_PeakList;
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_ElementCache;
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_IsotopicDistribution;
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_make_isotopic_distribution;
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_guess_npeaks;
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution__isotopic_variants;
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_isotopic_variants;
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_max_variants_approx;

/* "brainpy/_c/isotopic_distribution.pxd":7
 * 
 * 
 * cdef struct Peak:             # <<<<<<<<<<<<<<
 *     double mz
 *     double intensity
 */
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_Peak {
  double mz;
  double intensity;
  int charge;
};

/* "brainpy/_c/isotopic_distribution.pxd":13
 * 
 * 
 * cdef struct PeakList:             # <<<<<<<<<<<<<<
 *     Peak* peaks
 *     size_t used
 */
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_PeakList {
  struct __pyx_t_7brainpy_2_c_21isotopic_distribution_Peak *peaks;
  size_t used;
  size_t size;
};

/* "brainpy/_c/isotopic_distribution.pxd":35
 * 
 * 
 * cdef struct ElementCache:             # <<<<<<<<<<<<<<
 *     Element** elements
 *     size_t used
 */
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_ElementCache {
  struct __pyx_t_7brainpy_2_c_11composition_Element **elements;
  size_t used;
  size_t size;
  struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *source;
};

/* "brainpy/_c/isotopic_distribution.pxd":49
 * 
 * 
 * cdef struct IsotopicDistribution:             # <<<<<<<<<<<<<<
 *     Composition* composition
 *     IsotopicConstants* _isotopic_constants
 */
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_IsotopicDistribution {
  struct __pyx_t_7brainpy_2_c_11composition_Composition *composition;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *_isotopic_constants;
  int order;
  double average_mass;
  struct __pyx_t_7brainpy_2_c_21isotopic_distribution_Peak *monoisotopic_peak;
  struct __pyx_t_7brainpy_2_c_21isotopic_distribution_ElementCache *cache;
};

/* "brainpy/_c/isotopic_distribution.pxd":58
 * 
 * 
 * cdef IsotopicDistribution* make_isotopic_distribution(Composition* composition, int order, ElementCache* cache=*) nogil             # <<<<<<<<<<<<<<
 * cdef void free_isotopic_distribution(IsotopicDistribution* distribution)  nogil
 * 
 */
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_make_isotopic_distribution {
  int __pyx_n;
  struct __pyx_t_7brainpy_2_c_21isotopic_distribution_ElementCache *cache;
};

/* "brainpy/_c/isotopic_distribution.pxd":77
 * 
 * cdef int max_variants(Composition* composition, ElementCache* cache) nogil
 * cdef int guess_npeaks(Composition* composition_struct, size_t max_npeaks, ElementCache* cache=*) nogil             # <<<<<<<<<<<<<<
 * 
 * cpdef list _isotopic_variants(object composition, object npeaks=*, int charge=*, double charge_carrier=*)
 */
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_guess_npeaks {
  int __pyx_n;
  struct __pyx_t_7brainpy_2_c_21isotopic_distribution_ElementCache *cache;
};

/* "brainpy/_c/isotopic_distribution.pxd":79
 * cdef int guess_npeaks(Composition* composition_struct, size_t max_npeaks, ElementCache* cache=*) nogil
 * 
 * cpdef list _isotopic_variants(object composition, object npeaks=*, int charge=*, double charge_carrier=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution__isotopic_variants {
  int __pyx_n;
  PyObject *npeaks;
  int charge;
  double charge_carrier;
};

/* "brainpy/_c/isotopic_distribution.pxd":82
 * 
 * 
 * cdef PeakList* isotopic_variants(Composition* composition, int npeaks, int charge=*, double charge_carrier=*) nogil             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_isotopic_variants {
  int __pyx_n;
  int charge;
  double charge_carrier;
};

/* "brainpy/_c/isotopic_distribution.pxd":85
 * 
 * 
 * cdef size_t max_variants_approx(double mass, double lambda_factor=*, size_t maxiter=*, double threshold=*) nogil             # <<<<<<<<<<<<<<
 */
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_max_variants_approx {
  int __pyx_n;
  double lambda_factor;
  size_t maxiter;
  double threshold;
};
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet__has_peak;
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet_all_peaks_for;
struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_cell;
struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_list;
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_14PeakSetIndexed_has_peak_hinted;

/* "ms_peak_picker/_c/peak_set.pxd":42
 *     cdef FittedPeak _get_nearest_peak(self, double mz, double* errout)
 * 
 *     cdef FittedPeak _has_peak(self, double mz, double tolerance=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef PeakSet clone(self)
 */
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet__has_peak {
  int __pyx_n;
  double tolerance;
};

/* "ms_peak_picker/_c/peak_set.pxd":50
 *     cdef int _between_bounds(self, double m1, double m2, size_t* startp, size_t* endp)
 * 
 *     cpdef tuple all_peaks_for(self, double mz, double error_tolerance=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef FittedPeak getitem(self, size_t i)
 */
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet_all_peaks_for {
  int __pyx_n;
  double error_tolerance;
};

/* "ms_peak_picker/_c/peak_set.pxd":64
 * 
 * 
 * cdef struct index_cell:             # <<<<<<<<<<<<<<
 *     double center_value
 *     size_t start
 */
struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_cell {
  double center_value;
  size_t start;
  size_t end;
};

/* "ms_peak_picker/_c/peak_set.pxd":70
 * 
 * 
 * cdef struct index_list:             # <<<<<<<<<<<<<<
 *     index_cell* index
 *     size_t size
 */
struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_list {
  struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_cell *index;
  size_t size;
  double low;
  double high;
};

/* "ms_peak_picker/_c/peak_set.pxd":90
 *     cpdef _allocate_index(self, size_t interval_index_size)
 * 
 *     cpdef FittedPeak has_peak_hinted(self, double mz, double tolerance=*, size_t hint_start=*, size_t hint_end=*)             # <<<<<<<<<<<<<<
 * 
 * cdef size_t double_binary_search_ppm(double* array, double value, double tolerance, size_t n)
 */
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_14PeakSetIndexed_has_peak_hinted {
  int __pyx_n;
  double tolerance;
  size_t hint_start;
  size_t hint_end;
};

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":728
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":729
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":730
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":732
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
struct __pyx_opt_args_14ms_peak_picker_2_c_10peak_index_9PeakIndex__has_peak;
struct __pyx_opt_args_14ms_peak_picker_2_c_10peak_index_9PeakIndex_all_peaks_for;

/* "ms_peak_picker/_c/peak_index.pxd":11
 *         public PeakSet peaks
 * 
 *     cdef FittedPeak _has_peak(self, double mz, double tolerance=*)             # <<<<<<<<<<<<<<
 *     cdef PeakSet _between(self, double start, double stop)
 * 
 */
struct __pyx_opt_args_14ms_peak_picker_2_c_10peak_index_9PeakIndex__has_peak {
  int __pyx_n;
  double tolerance;
};

/* "ms_peak_picker/_c/peak_index.pxd":14
 *     cdef PeakSet _between(self, double start, double stop)
 * 
 *     cpdef tuple all_peaks_for(self, double mz, double error_tolerance=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t get_size(self)
 */
struct __pyx_opt_args_14ms_peak_picker_2_c_10peak_index_9PeakIndex_all_peaks_for {
  int __pyx_n;
  double error_tolerance;
};
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_mass_charge_ratio;
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_neutral_mass;
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine_scale;
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine_isotopic_cluster;
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine__isotopic_cluster;
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_ignore_below;
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_truncate_after;
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_scale;
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern__scale;
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_14AveragineCache_has_mz_charge_pair;
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_14AveragineCache_isotopic_cluster;
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_isotopic_shift;
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_poisson_approximate;

/* "ms_deisotope/_c/averagine.pxd":7
 * cdef double PROTON
 * 
 * cdef double mass_charge_ratio(double neutral_mass, int z, double charge_carrier=*)             # <<<<<<<<<<<<<<
 * cdef double neutral_mass(double mz,  int z, double charge_carrier=*)
 * cdef void slide(double mz, list peaklist)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_mass_charge_ratio {
  int __pyx_n;
  double charge_carrier;
};

/* "ms_deisotope/_c/averagine.pxd":8
 * 
 * cdef double mass_charge_ratio(double neutral_mass, int z, double charge_carrier=*)
 * cdef double neutral_mass(double mz,  int z, double charge_carrier=*)             # <<<<<<<<<<<<<<
 * cdef void slide(double mz, list peaklist)
 * cdef dict scale_dict(dict data, double factor)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_neutral_mass {
  int __pyx_n;
  double charge_carrier;
};

/* "ms_deisotope/_c/averagine.pxd":17
 *         public dict base_composition
 * 
 *     cpdef dict scale(self, double mz, int charge=*, double charge_carrier=*)             # <<<<<<<<<<<<<<
 *     cpdef TheoreticalIsotopicPattern isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)
 *     cdef TheoreticalIsotopicPattern _isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine_scale {
  int __pyx_n;
  int charge;
  double charge_carrier;
};

/* "ms_deisotope/_c/averagine.pxd":18
 * 
 *     cpdef dict scale(self, double mz, int charge=*, double charge_carrier=*)
 *     cpdef TheoreticalIsotopicPattern isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)             # <<<<<<<<<<<<<<
 *     cdef TheoreticalIsotopicPattern _isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine_isotopic_cluster {
  int __pyx_n;
  int charge;
  double charge_carrier;
  double truncate_after;
  double ignore_below;
};

/* "ms_deisotope/_c/averagine.pxd":19
 *     cpdef dict scale(self, double mz, int charge=*, double charge_carrier=*)
 *     cpdef TheoreticalIsotopicPattern isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)
 *     cdef TheoreticalIsotopicPattern _isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine__isotopic_cluster {
  int __pyx_n;
  int charge;
  double charge_carrier;
  double truncate_after;
  double ignore_below;
};

/* "ms_deisotope/_c/averagine.pxd":39
 *     cdef bint _eq_inst(self, TheoreticalIsotopicPattern other)
 * 
 *     cpdef TheoreticalIsotopicPattern ignore_below(self, double ignore_below=*)             # <<<<<<<<<<<<<<
 *     cpdef TheoreticalIsotopicPattern truncate_after(self, double truncate_after=*)
 *     cpdef TheoreticalIsotopicPattern shift(self, double mz)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_ignore_below {
  int __pyx_n;
  double ignore_below;
};

/* "ms_deisotope/_c/averagine.pxd":40
 * 
 *     cpdef TheoreticalIsotopicPattern ignore_below(self, double ignore_below=*)
 *     cpdef TheoreticalIsotopicPattern truncate_after(self, double truncate_after=*)             # <<<<<<<<<<<<<<
 *     cpdef TheoreticalIsotopicPattern shift(self, double mz)
 *     cpdef TheoreticalIsotopicPattern scale(self, list experimental_distribution, str method=*)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_truncate_after {
  int __pyx_n;
  double truncate_after;
};

/* "ms_deisotope/_c/averagine.pxd":42
 *     cpdef TheoreticalIsotopicPattern truncate_after(self, double truncate_after=*)
 *     cpdef TheoreticalIsotopicPattern shift(self, double mz)
 *     cpdef TheoreticalIsotopicPattern scale(self, list experimental_distribution, str method=*)             # <<<<<<<<<<<<<<
 *     cpdef TheoreticalIsotopicPattern scale_raw(self, double scale_factor)
 *     cpdef double drop_last_peak(self)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_scale {
  int __pyx_n;
  PyObject *method;
};

/* "ms_deisotope/_c/averagine.pxd":53
 * 
 *     @cython.final
 *     cdef inline TheoreticalIsotopicPattern _scale(self, list experimental_distribution, str method=*)             # <<<<<<<<<<<<<<
 * 
 *     @cython.final
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern__scale {
  int __pyx_n;
  PyObject *method;
};

/* "ms_deisotope/_c/averagine.pxd":88
 *     cdef double _make_cache_key(self, double mz)
 * 
 *     cdef TheoreticalIsotopicPattern has_mz_charge_pair(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)             # <<<<<<<<<<<<<<
 *     cpdef TheoreticalIsotopicPattern isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_14AveragineCache_has_mz_charge_pair {
  int __pyx_n;
  int charge;
  double charge_carrier;
  double truncate_after;
  double ignore_below;
};

/* "ms_deisotope/_c/averagine.pxd":89
 * 
 *     cdef TheoreticalIsotopicPattern has_mz_charge_pair(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)
 *     cpdef TheoreticalIsotopicPattern isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_14AveragineCache_isotopic_cluster {
  int __pyx_n;
  int charge;
  double charge_carrier;
  double truncate_after;
  double ignore_below;
};

/* "ms_deisotope/_c/averagine.pxd":92
 * 
 * 
 * cpdef double isotopic_shift(int charge=*)             # <<<<<<<<<<<<<<
 * 
 * cpdef TheoreticalIsotopicPattern poisson_approximate(double mass, size_t n_peaks, double lambda_factor=*, int charge=*)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_isotopic_shift {
  int __pyx_n;
  int charge;
};

/* "ms_deisotope/_c/averagine.pxd":94
 * cpdef double isotopic_shift(int charge=*)
 * 
 * cpdef TheoreticalIsotopicPattern poisson_approximate(double mass, size_t n_peaks, double lambda_factor=*, int charge=*)             # <<<<<<<<<<<<<<
 */
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_poisson_approximate {
  int __pyx_n;
  double lambda_factor;
  int charge;
};
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase_match_theoretical_isotopic_distribution;
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase_subtraction;
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_next_putative_peak;
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_previous_putative_peak;
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_next_putative_peak_inplace;
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_previous_putative_peak_inplace;
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_25AveragineDeconvoluterBase_fit_theoretical_distribution;
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_25AveragineDeconvoluterBase__fit_peaks_at_charges;
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_30MultiAveragineDeconvoluterBase_fit_theoretical_distribution;
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_30MultiAveragineDeconvoluterBase__fit_peaks_at_charges;
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base__get_all_peak_charge_pairs;

/* "ms_deisotope/_c/deconvoluter_base.pxd":42
 *     cdef FittedPeak _has_peak(self, double mz, double error_tolerance)
 * 
 *     cpdef list match_theoretical_isotopic_distribution(self, list theoretical_distribution, double error_tolerance=*)             # <<<<<<<<<<<<<<
 *     cpdef scale_theoretical_distribution(self, TheoreticalIsotopicPattern theoretical_distribution, list experimental_distribution)
 *     cpdef IsotopicFitRecord _evaluate_theoretical_distribution(self, list experimental, TheoreticalIsotopicPattern theoretical, FittedPeak peak, int charge)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase_match_theoretical_isotopic_distribution {
  int __pyx_n;
  double error_tolerance;
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":45
 *     cpdef scale_theoretical_distribution(self, TheoreticalIsotopicPattern theoretical_distribution, list experimental_distribution)
 *     cpdef IsotopicFitRecord _evaluate_theoretical_distribution(self, list experimental, TheoreticalIsotopicPattern theoretical, FittedPeak peak, int charge)
 *     cpdef subtraction(self, TheoreticalIsotopicPattern isotopic_cluster, double error_tolerance=*)             # <<<<<<<<<<<<<<
 *     cpdef list fit_incremental_truncation(self, IsotopicFitRecord seed_fit, double lower_bound)
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase_subtraction {
  int __pyx_n;
  double error_tolerance;
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":50
 *     cpdef bint _check_fit(self, IsotopicFitRecord fit)
 * 
 *     cpdef list _find_next_putative_peak(self, double mz, int charge, int step=*, double tolerance=*)             # <<<<<<<<<<<<<<
 *     cpdef list _find_previous_putative_peak(self, double mz, int charge, int step=*, double tolerance=*)
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_next_putative_peak {
  int __pyx_n;
  int step;
  double tolerance;
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":51
 * 
 *     cpdef list _find_next_putative_peak(self, double mz, int charge, int step=*, double tolerance=*)
 *     cpdef list _find_previous_putative_peak(self, double mz, int charge, int step=*, double tolerance=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef int _find_next_putative_peak_inplace(self, double mz, int charge, set result, int step=*, double tolerance=*)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_previous_putative_peak {
  int __pyx_n;
  int step;
  double tolerance;
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":53
 *     cpdef list _find_previous_putative_peak(self, double mz, int charge, int step=*, double tolerance=*)
 * 
 *     cdef int _find_next_putative_peak_inplace(self, double mz, int charge, set result, int step=*, double tolerance=*)             # <<<<<<<<<<<<<<
 *     cdef int _find_previous_putative_peak_inplace(self, double mz, int charge, set result, int step=*, double tolerance=*)
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_next_putative_peak_inplace {
  int __pyx_n;
  int step;
  double tolerance;
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":54
 * 
 *     cdef int _find_next_putative_peak_inplace(self, double mz, int charge, set result, int step=*, double tolerance=*)
 *     cdef int _find_previous_putative_peak_inplace(self, double mz, int charge, set result, int step=*, double tolerance=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_previous_putative_peak_inplace {
  int __pyx_n;
  int step;
  double tolerance;
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":61
 *         public AveragineCache averagine
 * 
 *     cpdef IsotopicFitRecord fit_theoretical_distribution(self, FittedPeak peak, double error_tolerance, int charge,             # <<<<<<<<<<<<<<
 *                                                          double charge_carrier=*, double truncate_after=*,
 *                                                          double ignore_below=*)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_25AveragineDeconvoluterBase_fit_theoretical_distribution {
  int __pyx_n;
  double charge_carrier;
  double truncate_after;
  double ignore_below;
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":65
 *                                                          double ignore_below=*)
 * 
 *     cpdef set _fit_peaks_at_charges(self, set peak_charge_set, double error_tolerance, double charge_carrier=*, double truncate_after=*,             # <<<<<<<<<<<<<<
 *                                     double ignore_below=*)
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_25AveragineDeconvoluterBase__fit_peaks_at_charges {
  int __pyx_n;
  double charge_carrier;
  double truncate_after;
  double ignore_below;
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":73
 *         public list averagines
 * 
 *     cpdef IsotopicFitRecord fit_theoretical_distribution(self, FittedPeak peak, double error_tolerance, int charge,             # <<<<<<<<<<<<<<
 *                                                          AveragineCache  averagine, double charge_carrier=*, double truncate_after=*,
 *                                                          double ignore_below=*)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_30MultiAveragineDeconvoluterBase_fit_theoretical_distribution {
  int __pyx_n;
  double charge_carrier;
  double truncate_after;
  double ignore_below;
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":76
 *                                                          AveragineCache  averagine, double charge_carrier=*, double truncate_after=*,
 *                                                          double ignore_below=*)
 *     cpdef set _fit_peaks_at_charges(self, set peak_charge_set, double error_tolerance, double charge_carrier=*, double truncate_after=*,             # <<<<<<<<<<<<<<
 *                                     double ignore_below=*)
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_30MultiAveragineDeconvoluterBase__fit_peaks_at_charges {
  int __pyx_n;
  double charge_carrier;
  double truncate_after;
  double ignore_below;
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":83
 * 
 * 
 * cpdef set _get_all_peak_charge_pairs(DeconvoluterBase self, FittedPeak peak, double error_tolerance=*,             # <<<<<<<<<<<<<<
 *                                      object charge_range=*,
 *                                      int left_search_limit=*, int right_search_limit=*,
 */
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base__get_all_peak_charge_pairs {
  int __pyx_n;
  double error_tolerance;
  PyObject *charge_range;
  int left_search_limit;
  int right_search_limit;
  int recalculate_starting_peak;
  int use_quick_charge;
};
struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_17IsotopicFitRecord__create;
struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_14MSDeconVFitter_score_peak;
struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_21InterferenceDetection_detect_interference;

/* "ms_deisotope/_c/scoring.pxd":29
 * 
 *     @staticmethod
 *     cdef IsotopicFitRecord _create(FittedPeak seed_peak, double score, int charge, TheoreticalIsotopicPattern theoretical,             # <<<<<<<<<<<<<<
 *                                    list experimental, object data=*, int missed_peaks=*)
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_17IsotopicFitRecord__create {
  int __pyx_n;
  PyObject *data;
  int missed_peaks;
};

/* "ms_deisotope/_c/scoring.pxd":75
 *     cdef public double mass_error_tolerance
 * 
 *     cdef double score_peak(self, FittedPeak obs, TheoreticalPeak theo, double mass_error_tolerance=*, double minimum_signal_to_noise=*) nogil             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_14MSDeconVFitter_score_peak {
  int __pyx_n;
  double mass_error_tolerance;
  double minimum_signal_to_noise;
};

/* "ms_deisotope/_c/scoring.pxd":92
 *     cdef public PeakSet peaklist
 * 
 *     cdef double detect_interference(self, list experimental_peaks, double lower=*, double upper=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_21InterferenceDetection_detect_interference {
  int __pyx_n;
  double lower;
  double upper;
};
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak;
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak;
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for;
struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell;
struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list;
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set__Index_reconstructor;
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak;
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for;

/* "ms_deisotope/_c/peak_set.pxd":87
 *     cpdef DeconvolutedPeakSet copy(self)
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=*, bint use_mz=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef DeconvolutedPeak has_peak(self, double neutral_mass, double error_tolerance=*, bint use_mz=*)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak {
  int __pyx_n;
  double error_tolerance;
  int use_mz;
};

/* "ms_deisotope/_c/peak_set.pxd":89
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=*, bint use_mz=*)
 * 
 *     cpdef DeconvolutedPeak has_peak(self, double neutral_mass, double error_tolerance=*, bint use_mz=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=*)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak {
  int __pyx_n;
  double error_tolerance;
  int use_mz;
};

/* "ms_deisotope/_c/peak_set.pxd":91
 *     cpdef DeconvolutedPeak has_peak(self, double neutral_mass, double error_tolerance=*, bint use_mz=*)
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef DeconvolutedPeak _get_nearest_peak(self, double neutral_mass, double* errout)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for {
  int __pyx_n;
  double tolerance;
};

/* "ms_deisotope/_c/peak_set.pxd":118
 * cdef size_t INTERVAL_INDEX_SIZE
 * 
 * cdef struct index_cell:             # <<<<<<<<<<<<<<
 *     double center_value
 *     size_t start
 */
struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell {
  double center_value;
  size_t start;
  size_t end;
};

/* "ms_deisotope/_c/peak_set.pxd":124
 * 
 * 
 * cdef struct index_list:             # <<<<<<<<<<<<<<
 *     index_cell* index
 *     size_t size
 */
struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list {
  struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell *index;
  size_t size;
  double low;
  double high;
};

/* "ms_deisotope/_c/peak_set.pyx":45
 * 
 * 
 * cpdef _Index_reconstructor(object neutral_mass, object mz=None):             # <<<<<<<<<<<<<<
 *     if isinstance(neutral_mass, int):
 *         return _Index._create(neutral_mass, mz)
 */
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set__Index_reconstructor {
  int __pyx_n;
  PyObject *mz;
};

/* "ms_deisotope/_c/peak_set.pyx":1209
 *         return self
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak {
  int __pyx_n;
  double error_tolerance;
  int use_mz;
};

/* "ms_deisotope/_c/peak_set.pyx":1264
 *         return status, start, end
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         '''Find all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for {
  int __pyx_n;
  double tolerance;
};

/* "brainpy/_c/composition.pxd":123
 * 
 * 
 * cdef class PyComposition(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Composition* impl
 */
struct __pyx_obj_7brainpy_2_c_11composition_PyComposition {
  PyObject_HEAD
  struct __pyx_vtabstruct_7brainpy_2_c_11composition_PyComposition *__pyx_vtab;
  struct __pyx_t_7brainpy_2_c_11composition_Composition *impl;
  double cached_mass;
  int _clean;
};


/* "brainpy/_c/isotopic_distribution.pxd":63
 * 
 * @freelist(100000)
 * cdef class TheoreticalPeak(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mz
 */
struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak {
  PyObject_HEAD
  struct __pyx_vtabstruct_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *__pyx_vtab;
  double mz;
  double intensity;
  int charge;
};


/* "ms_peak_picker/_c/peak_set.pxd":2
 * 
 * cdef class PeakBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mz
 */
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_vtab;
  double mz;
  double intensity;
  double area;
};


/* "ms_peak_picker/_c/peak_set.pxd":12
 * 
 * 
 * cdef class FittedPeak(PeakBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double signal_to_noise
 */
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak {
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase __pyx_base;
  double signal_to_noise;
  double full_width_at_half_max;
  double left_width;
  double right_width;
  long peak_count;
  long index;
};


/* "ms_peak_picker/_c/peak_set.pxd":29
 *                             double area)
 * 
 * cdef class PeakSet(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public tuple peaks
 */
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet {
  PyObject_HEAD
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet *__pyx_vtab;
  PyObject *peaks;
  int indexed;
};


/* "ms_peak_picker/_c/peak_set.pxd":83
 * 
 * 
 * cdef class PeakSetIndexed(PeakSet):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* mz_index
 */
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed {
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet __pyx_base;
  double *mz_index;
  struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_list *interval_index;
};


/* "ms_peak_picker/_c/peak_index.pxd":5
 * 
 * 
 * cdef class PeakIndex(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public np.ndarray mz_array
 */
struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex {
  PyObject_HEAD
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_10peak_index_PeakIndex *__pyx_vtab;
  PyArrayObject *mz_array;
  PyArrayObject *intensity_array;
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *peaks;
};


/* "ms_deisotope/_c/averagine.pxd":12
 * cdef dict scale_dict(dict data, double factor)
 * 
 * cdef class Averagine(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double base_mass
 */
struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_Averagine *__pyx_vtab;
  double base_mass;
  PyObject *base_composition;
};


/* "ms_deisotope/_c/averagine.pxd":23
 * 
 * @cython.final
 * cdef class TheoreticalIsotopicPattern(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list peaklist
 */
struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *__pyx_vtab;
  PyObject *peaklist;
  double origin;
  double offset;
};


/* "ms_deisotope/_c/averagine.pxd":79
 * 
 * @cython.final
 * cdef class AveragineCache(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public dict backend
 */
struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_AveragineCache *__pyx_vtab;
  PyObject *backend;
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine *averagine;
  double cache_truncation;
  int enabled;
};


/* "ms_deisotope/_c/deconvoluter_base.pxd":25
 * 
 * 
 * cdef class DeconvoluterBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public bint use_subtraction
 */
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *__pyx_vtab;
  int use_subtraction;
  PyObject *scale_method;
  int merge_isobaric_peaks;
  double minimum_intensity;
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *peaklist;
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *scorer;
  int verbose;
  PyObject *_slice_cache;
};


/* "ms_deisotope/_c/deconvoluter_base.pxd":57
 * 
 * 
 * cdef class AveragineDeconvoluterBase(DeconvoluterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public AveragineCache averagine
 */
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase {
  struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase __pyx_base;
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache *averagine;
};


/* "ms_deisotope/_c/deconvoluter_base.pxd":69
 * 
 * 
 * cdef class MultiAveragineDeconvoluterBase(DeconvoluterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list averagines
 */
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase {
  struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase __pyx_base;
  PyObject *averagines;
};


/* "ms_deisotope/_c/deconvoluter_base.pxd":91
 * 
 * 
 * cdef class ChargeIterator(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public int lower
 */
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *__pyx_vtab;
  int lower;
  int upper;
  int sign;
  int *values;
  size_t size;
  size_t index;
};


/* "ms_deisotope/_c/scoring.pxd":16
 * 
 * 
 * cdef class IsotopicFitRecord(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public FittedPeak seed_peak
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *__pyx_vtab;
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *seed_peak;
  double score;
  int charge;
  PyObject *experimental;
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *theoretical;
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *monoisotopic_peak;
  int missed_peaks;
  PyObject *data;
  Py_hash_t _hash;
};


/* "ms_deisotope/_c/scoring.pxd":38
 * 
 * 
 * cdef class FitSelectorBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double minimum_score
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FitSelectorBase *__pyx_vtab;
  double minimum_score;
};


/* "ms_deisotope/_c/scoring.pxd":50
 * 
 * 
 * cdef class IsotopicFitterBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public FitSelectorBase select
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *__pyx_vtab;
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase *select;
};


/* "ms_deisotope/_c/scoring.pxd":62
 * 
 * 
 * cdef class LeastSquaresFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_LeastSquaresFitter {
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
};


/* "ms_deisotope/_c/scoring.pxd":66
 * 
 * 
 * cdef class GTestFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_GTestFitter {
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
};


/* "ms_deisotope/_c/scoring.pxd":69
 *     pass
 * 
 * cdef class ScaledGTestFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledGTestFitter {
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
};


/* "ms_deisotope/_c/scoring.pxd":72
 *     pass
 * 
 * cdef class MSDeconVFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef public double mass_error_tolerance
 * 
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_MSDeconVFitter {
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
  double mass_error_tolerance;
};


/* "ms_deisotope/_c/scoring.pxd":78
 * 
 * 
 * cdef class PenalizedMSDeconVFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mass_error_tolerance
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter {
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
  double mass_error_tolerance;
  double penalty_factor;
};


/* "ms_deisotope/_c/scoring.pxd":84
 * 
 * 
 * cdef class FunctionScorer(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         object function
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_FunctionScorer {
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
  PyObject *function;
};


/* "ms_deisotope/_c/scoring.pxd":89
 * 
 * 
 * cdef class InterferenceDetection(object):             # <<<<<<<<<<<<<<
 *     cdef public PeakSet peaklist
 * 
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_InterferenceDetection {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_InterferenceDetection *__pyx_vtab;
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *peaklist;
};


/* "ms_deisotope/_c/scoring.pxd":95
 * 
 * 
 * cdef class DistinctPatternFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public InterferenceDetection interference_detector
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_DistinctPatternFitter {
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_InterferenceDetection *interference_detector;
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledGTestFitter *g_test_scaled;
  double peak_count_scale;
  double domain_scale;
};


/* "ms_deisotope/_c/scoring.pxd":102
 *         public double domain_scale
 * 
 * cdef class ScaledPenalizedMSDeconvFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double scale_factor
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter {
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
  double scale_factor;
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter *scorer;
};


/* "ms_deisotope/_c/scoring.pxd":112
 * 
 * 
 * cdef class DotProductFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 */
struct __pyx_obj_12ms_deisotope_2_c_7scoring_DotProductFitter {
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
};


/* "ms_deisotope/_c/peak_set.pxd":5
 * 
 * 
 * cdef class _Index:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public size_t neutral_mass
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set__Index *__pyx_vtab;
  size_t neutral_mass;
  size_t mz;
};


/* "ms_deisotope/_c/peak_set.pxd":14
 * 
 * 
 * cdef class EnvelopePair:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mz
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_vtab;
  double mz;
  double intensity;
};


/* "ms_deisotope/_c/peak_set.pxd":25
 * 
 * 
 * cdef class Envelope:             # <<<<<<<<<<<<<<
 *     cdef public tuple pairs
 * 
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_vtab;
  PyObject *pairs;
};


/* "ms_deisotope/_c/peak_set.pxd":40
 * 
 * 
 * cdef class DeconvolutedPeak(PeakBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double neutral_mass
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak {
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase __pyx_base;
  double neutral_mass;
  double signal_to_noise;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *_index;
  double full_width_at_half_max;
  int charge;
  double a_to_a2_ratio;
  double most_abundant_mass;
  double average_mass;
  double score;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *envelope;
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *fit;
  int chosen_for_msms;
};


/* "ms_deisotope/_c/peak_set.pxd":66
 * 
 * 
 * cdef class IonMobilityDeconvolutedPeak(DeconvolutedPeak):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double drift_time
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak __pyx_base;
  double drift_time;
};


/* "ms_deisotope/_c/peak_set.pxd":71
 * 
 * 
 * cdef class DeconvolutedPeakSolution(DeconvolutedPeak):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public object solution
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak __pyx_base;
  PyObject *solution;
};


/* "ms_deisotope/_c/peak_set.pxd":75
 *         public object solution
 * 
 * cdef class DeconvolutedPeakSet(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public tuple peaks
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet {
  PyObject_HEAD
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_vtab;
  PyObject *peaks;
  PyObject *_mz_ordered;
  int indexed;
};


/* "ms_deisotope/_c/peak_set.pxd":104
 * 
 * 
 * cdef class DeconvolutedPeakSetIndexed(DeconvolutedPeakSet):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* neutral_mass_array
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet __pyx_base;
  double *neutral_mass_array;
  double *mz_array;
  struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *interval_index;
  size_t _size;
};


/* "ms_deisotope/_c/peak_set.pyx":143
 *             raise IndexError(i)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.mz
 *         yield self.intensity
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ {
  PyObject_HEAD
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self;
};


/* "ms_deisotope/_c/peak_set.pyx":259
 *         return iter(self.pairs)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))
 * 
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ {
  PyObject_HEAD
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self;
};


/* "ms_deisotope/_c/peak_set.pyx":260
 * 
 *     def __repr__(self):
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr {
  PyObject_HEAD
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *__pyx_outer_scope;
  PyObject *__pyx_v_t;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* "ms_deisotope/_c/peak_set.pyx":529
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.solution
 *         yield self
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ {
  PyObject_HEAD
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self;
};


/* "ms_deisotope/_c/peak_set.pyx":599
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ {
  PyObject_HEAD
  PyObject *__pyx_v_item;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self;
};


/* "ms_deisotope/_c/peak_set.pyx":601
 *     def __getitem__(self, item):
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))             # <<<<<<<<<<<<<<
 *         return self.peaks[item]
 * 
 */
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr {
  PyObject_HEAD
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *__pyx_outer_scope;
  PyObject *__pyx_v_p;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* "View.MemoryView":105
 * 
 * @cname("__pyx_array")
 * cdef class array:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_array_obj {
  PyObject_HEAD
  struct __pyx_vtabstruct_array *__pyx_vtab;
  char *data;
  Py_ssize_t len;
  char *format;
  int ndim;
  Py_ssize_t *_shape;
  Py_ssize_t *_strides;
  Py_ssize_t itemsize;
  PyObject *mode;
  PyObject *_format;
  void (*callback_free_data)(void *);
  int free_data;
  int dtype_is_object;
};


/* "View.MemoryView":279
 * 
 * @cname('__pyx_MemviewEnum')
 * cdef class Enum(object):             # <<<<<<<<<<<<<<
 *     cdef object name
 *     def __init__(self, name):
 */
struct __pyx_MemviewEnum_obj {
  PyObject_HEAD
  PyObject *name;
};


/* "View.MemoryView":330
 * 
 * @cname('__pyx_memoryview')
 * cdef class memoryview(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef object obj
 */
struct __pyx_memoryview_obj {
  PyObject_HEAD
  struct __pyx_vtabstruct_memoryview *__pyx_vtab;
  PyObject *obj;
  PyObject *_size;
  PyObject *_array_interface;
  PyThread_type_lock lock;
  __pyx_atomic_int acquisition_count[2];
  __pyx_atomic_int *acquisition_count_aligned_p;
  Py_buffer view;
  int flags;
  int dtype_is_object;
  __Pyx_TypeInfo *typeinfo;
};


/* "View.MemoryView":965
 * 
 * @cname('__pyx_memoryviewslice')
 * cdef class _memoryviewslice(memoryview):             # <<<<<<<<<<<<<<
 *     "Internal class for passing memoryview slices to Python"
 * 
 */
struct __pyx_memoryviewslice_obj {
  struct __pyx_memoryview_obj __pyx_base;
  __Pyx_memviewslice from_slice;
  PyObject *from_object;
  PyObject *(*to_object_func)(char *);
  int (*to_dtype_func)(char *, PyObject *);
};



/* "brainpy/_c/composition.pxd":123
 * 
 * 
 * cdef class PyComposition(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Composition* impl
 */

struct __pyx_vtabstruct_7brainpy_2_c_11composition_PyComposition {
  struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *(*_create)(struct __pyx_t_7brainpy_2_c_11composition_Composition *);
  void (*_set_impl)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, struct __pyx_t_7brainpy_2_c_11composition_Composition *, struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition__set_impl *__pyx_optional_args);
  void (*_initialize_from_formula)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *);
  double (*mass)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);
  int (*__pyx___equality_pycomposition)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);
  int (*__pyx___equality_dict)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *(*copy)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);
  PyObject *(*update)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*keys)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);
  PyObject *(*values)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);
  PyObject *(*items)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);
  PyObject *(*pop)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition_pop *__pyx_optional_args);
  __pyx_t_7brainpy_2_c_11composition_count_type (*getitem)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *);
  void (*setitem)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *, __pyx_t_7brainpy_2_c_11composition_count_type);
  void (*increment)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *, __pyx_t_7brainpy_2_c_11composition_count_type);
  void (*add_from)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *);
  void (*subtract_from)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *);
  void (*scale_by)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, long);
};
static struct __pyx_vtabstruct_7brainpy_2_c_11composition_PyComposition *__pyx_vtabptr_7brainpy_2_c_11composition_PyComposition;


/* "brainpy/_c/isotopic_distribution.pxd":63
 * 
 * @freelist(100000)
 * cdef class TheoreticalPeak(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mz
 */

struct __pyx_vtabstruct_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak {
  struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *(*_create)(double, double, int);
  int (*_eq)(struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *, struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *, int __pyx_skip_dispatch);
  struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *(*clone)(struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *__pyx_vtabptr_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak;


/* "ms_peak_picker/_c/peak_set.pxd":2
 * 
 * cdef class PeakBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mz
 */

struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase {
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*clone)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch);
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*copy)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakBase;


/* "ms_peak_picker/_c/peak_set.pxd":12
 * 
 * 
 * cdef class FittedPeak(PeakBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double signal_to_noise
 */

struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_FittedPeak {
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase __pyx_base;
  int (*_eq)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, int __pyx_skip_dispatch);
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_create)(double, double, double, double, double, double, long, long, double);
};
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_FittedPeak *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_FittedPeak;


/* "ms_peak_picker/_c/peak_set.pxd":29
 *                             double area)
 * 
 * cdef class PeakSet(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public tuple peaks
 */

struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet {
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*_create)(PyObject *);
  PyObject *(*reindex)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);
  size_t (*_index)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_get_nearest_peak)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, double *);
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_has_peak)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet__has_peak *__pyx_optional_args);
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*clone)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*copy)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*_between)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, double);
  int (*_between_bounds)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, double, size_t *, size_t *);
  PyObject *(*all_peaks_for)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet_all_peaks_for *__pyx_optional_args);
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*getitem)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, size_t);
  size_t (*get_size)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *);
  int (*_eq)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSet;


/* "ms_peak_picker/_c/peak_set.pxd":83
 * 
 * 
 * cdef class PeakSetIndexed(PeakSet):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* mz_index
 */

struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed {
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet __pyx_base;
  PyObject *(*_allocate_index)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed *, size_t, int __pyx_skip_dispatch);
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*has_peak_hinted)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_14PeakSetIndexed_has_peak_hinted *__pyx_optional_args);
};
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed;


/* "ms_peak_picker/_c/peak_index.pxd":5
 * 
 * 
 * cdef class PeakIndex(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public np.ndarray mz_array
 */

struct __pyx_vtabstruct_14ms_peak_picker_2_c_10peak_index_PeakIndex {
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_has_peak)(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *, double, struct __pyx_opt_args_14ms_peak_picker_2_c_10peak_index_9PeakIndex__has_peak *__pyx_optional_args);
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*_between)(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *, double, double);
  PyObject *(*all_peaks_for)(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_14ms_peak_picker_2_c_10peak_index_9PeakIndex_all_peaks_for *__pyx_optional_args);
  size_t (*get_size)(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *);
  struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *(*clone)(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *, int __pyx_skip_dispatch);
  struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *(*copy)(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_10peak_index_PeakIndex *__pyx_vtabptr_14ms_peak_picker_2_c_10peak_index_PeakIndex;


/* "ms_deisotope/_c/averagine.pxd":12
 * cdef dict scale_dict(dict data, double factor)
 * 
 * cdef class Averagine(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double base_mass
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_Averagine {
  PyObject *(*scale)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine_scale *__pyx_optional_args);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*isotopic_cluster)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine_isotopic_cluster *__pyx_optional_args);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*_isotopic_cluster)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine *, double, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine__isotopic_cluster *__pyx_optional_args);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_Averagine *__pyx_vtabptr_12ms_deisotope_2_c_9averagine_Averagine;


/* "ms_deisotope/_c/averagine.pxd":23
 * 
 * @cython.final
 * cdef class TheoreticalIsotopicPattern(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list peaklist
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern {
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*_create)(PyObject *, double, double);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*clone)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch);
  int (*_eq)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, PyObject *, int __pyx_skip_dispatch);
  int (*_eq_inst)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*ignore_below)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_ignore_below *__pyx_optional_args);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*truncate_after)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_truncate_after *__pyx_optional_args);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*shift)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, double, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*scale)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_scale *__pyx_optional_args);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*scale_raw)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, double, int __pyx_skip_dispatch);
  double (*drop_last_peak)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch);
  double (*total)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*normalize)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*_scale)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, PyObject *, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern__scale *__pyx_optional_args);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*clone_shift)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, double);
  struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *(*get)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, Py_ssize_t);
  size_t (*get_size)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *);
  double (*get_monoisotopic_mz)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *);
  PyObject *(*get_processed_peaks)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *);
  size_t (*basepeak_index)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch);
  PyObject *(*incremental_truncation)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, double, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*clone_drop_last)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *__pyx_vtabptr_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern;


/* "ms_deisotope/_c/averagine.pxd":79
 * 
 * @cython.final
 * cdef class AveragineCache(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public dict backend
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_AveragineCache {
  double (*_make_cache_key)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache *, double);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*has_mz_charge_pair)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache *, double, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_14AveragineCache_has_mz_charge_pair *__pyx_optional_args);
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*isotopic_cluster)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_14AveragineCache_isotopic_cluster *__pyx_optional_args);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_AveragineCache *__pyx_vtabptr_12ms_deisotope_2_c_9averagine_AveragineCache;


/* "ms_deisotope/_c/deconvoluter_base.pxd":25
 * 
 * 
 * cdef class DeconvoluterBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public bint use_subtraction
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase {
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*between)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, double, int __pyx_skip_dispatch);
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*has_peak)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, double, int __pyx_skip_dispatch);
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_has_peak)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, double);
  PyObject *(*match_theoretical_isotopic_distribution)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase_match_theoretical_isotopic_distribution *__pyx_optional_args);
  PyObject *(*scale_theoretical_distribution)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *(*_evaluate_theoretical_distribution)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, PyObject *, struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, int, int __pyx_skip_dispatch);
  PyObject *(*subtraction)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase_subtraction *__pyx_optional_args);
  PyObject *(*fit_incremental_truncation)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, double, int __pyx_skip_dispatch);
  int (*_check_fit)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);
  PyObject *(*_find_next_putative_peak)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, int, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_next_putative_peak *__pyx_optional_args);
  PyObject *(*_find_previous_putative_peak)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, int, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_previous_putative_peak *__pyx_optional_args);
  int (*_find_next_putative_peak_inplace)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, int, PyObject *, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_next_putative_peak_inplace *__pyx_optional_args);
  int (*_find_previous_putative_peak_inplace)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, int, PyObject *, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_previous_putative_peak_inplace *__pyx_optional_args);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase;


/* "ms_deisotope/_c/deconvoluter_base.pxd":57
 * 
 * 
 * cdef class AveragineDeconvoluterBase(DeconvoluterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public AveragineCache averagine
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase __pyx_base;
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *(*fit_theoretical_distribution)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, double, int, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_25AveragineDeconvoluterBase_fit_theoretical_distribution *__pyx_optional_args);
  PyObject *(*_fit_peaks_at_charges)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase *, PyObject *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_25AveragineDeconvoluterBase__fit_peaks_at_charges *__pyx_optional_args);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase *__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase;


/* "ms_deisotope/_c/deconvoluter_base.pxd":69
 * 
 * 
 * cdef class MultiAveragineDeconvoluterBase(DeconvoluterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list averagines
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase __pyx_base;
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *(*fit_theoretical_distribution)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, double, int, struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_30MultiAveragineDeconvoluterBase_fit_theoretical_distribution *__pyx_optional_args);
  PyObject *(*_fit_peaks_at_charges)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase *, PyObject *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_30MultiAveragineDeconvoluterBase__fit_peaks_at_charges *__pyx_optional_args);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase *__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase;


/* "ms_deisotope/_c/deconvoluter_base.pxd":91
 * 
 * 
 * cdef class ChargeIterator(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public int lower
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator {
  struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *(*_create)(int, int);
  struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *(*from_tuple)(PyObject *);
  struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *(*from_quickcharge)(PyObject *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *);
  PyObject *(*sequence_from_quickcharge)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, int __pyx_skip_dispatch);
  PyObject *(*make_sequence)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *, int __pyx_skip_dispatch);
  void (*release_sequence)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *);
  void (*set_bounds)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *, int, int);
  int (*has_more)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *, int __pyx_skip_dispatch);
  PyObject *(*reset)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *, int __pyx_skip_dispatch);
  int (*get_next_value)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *, int __pyx_skip_dispatch);
  size_t (*get_size)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator;


/* "ms_deisotope/_c/scoring.pxd":16
 * 
 * 
 * cdef class IsotopicFitRecord(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public FittedPeak seed_peak
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitRecord {
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *(*_create)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, double, int, struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, PyObject *, struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_17IsotopicFitRecord__create *__pyx_optional_args);
  int (*_eq)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);
  int (*_ne)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);
  int (*_lt)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);
  int (*_gt)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_IsotopicFitRecord;


/* "ms_deisotope/_c/scoring.pxd":38
 * 
 * 
 * cdef class FitSelectorBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double minimum_score
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FitSelectorBase {
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *(*best)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *(*_best_from_set)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase *, PyObject *);
  int (*reject)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);
  int (*reject_score)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase *, double, int __pyx_skip_dispatch);
  int (*is_maximizing)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FitSelectorBase *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_FitSelectorBase;


/* "ms_deisotope/_c/scoring.pxd":50
 * 
 * 
 * cdef class IsotopicFitterBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public FitSelectorBase select
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase {
  double (*_evaluate)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  int (*reject)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);
  int (*reject_score)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *, double, int __pyx_skip_dispatch);
  int (*is_maximizing)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *(*_configure)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *, struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_IsotopicFitterBase;


/* "ms_deisotope/_c/scoring.pxd":62
 * 
 * 
 * cdef class LeastSquaresFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_LeastSquaresFitter {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_LeastSquaresFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_LeastSquaresFitter;


/* "ms_deisotope/_c/scoring.pxd":66
 * 
 * 
 * cdef class GTestFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_GTestFitter {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_GTestFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_GTestFitter;


/* "ms_deisotope/_c/scoring.pxd":69
 *     pass
 * 
 * cdef class ScaledGTestFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_ScaledGTestFitter {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_ScaledGTestFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_ScaledGTestFitter;


/* "ms_deisotope/_c/scoring.pxd":72
 *     pass
 * 
 * cdef class MSDeconVFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef public double mass_error_tolerance
 * 
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_MSDeconVFitter {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
  double (*score_peak)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_MSDeconVFitter *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *, struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_14MSDeconVFitter_score_peak *__pyx_optional_args);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_MSDeconVFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_MSDeconVFitter;


/* "ms_deisotope/_c/scoring.pxd":78
 * 
 * 
 * cdef class PenalizedMSDeconVFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mass_error_tolerance
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter;


/* "ms_deisotope/_c/scoring.pxd":84
 * 
 * 
 * cdef class FunctionScorer(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         object function
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FunctionScorer {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FunctionScorer *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_FunctionScorer;


/* "ms_deisotope/_c/scoring.pxd":89
 * 
 * 
 * cdef class InterferenceDetection(object):             # <<<<<<<<<<<<<<
 *     cdef public PeakSet peaklist
 * 
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_InterferenceDetection {
  double (*detect_interference)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_InterferenceDetection *, PyObject *, struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_21InterferenceDetection_detect_interference *__pyx_optional_args);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_InterferenceDetection *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_InterferenceDetection;


/* "ms_deisotope/_c/scoring.pxd":95
 * 
 * 
 * cdef class DistinctPatternFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public InterferenceDetection interference_detector
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_DistinctPatternFitter {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_DistinctPatternFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_DistinctPatternFitter;


/* "ms_deisotope/_c/scoring.pxd":102
 *         public double domain_scale
 * 
 * cdef class ScaledPenalizedMSDeconvFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double scale_factor
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
  double (*_calculate_scale_factor)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);
  void (*scale_fitted_peaks)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter *, PyObject *, double);
  void (*scale_theoretical_peaks)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter *, PyObject *, double);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter;


/* "ms_deisotope/_c/scoring.pxd":112
 * 
 * 
 * cdef class DotProductFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_DotProductFitter {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_DotProductFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_DotProductFitter;


/* "ms_deisotope/_c/peak_set.pyx":93
 * 
 * @cython.freelist(100000)
 * cdef class _Index(object):             # <<<<<<<<<<<<<<
 *     """
 *     Stores the ordered index of an object under sort by `mz` and
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set__Index {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *(*_create)(size_t, size_t);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set__Index *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set__Index;


/* "ms_deisotope/_c/peak_set.pyx":129
 * 
 * @cython.freelist(1000000)
 * cdef class EnvelopePair(object):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, mz, intensity):
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_EnvelopePair {
  int (*_eq)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *(*_create)(double, double);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_EnvelopePair;


/* "ms_deisotope/_c/peak_set.pyx":237
 * 
 * @cython.freelist(100000)
 * cdef class Envelope(object):             # <<<<<<<<<<<<<<
 *     """
 *     Represents a sequence of (mz, intensity) pairs which store peak positions
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_Envelope {
  size_t (*get_size)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *(*getitem)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, size_t);
  int (*_eq)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *(*clone)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *(*_create)(PyObject *);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_Envelope;
static CYTHON_INLINE size_t __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *);
static CYTHON_INLINE struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, size_t);


/* "ms_deisotope/_c/peak_set.pyx":323
 * 
 * 
 * cdef class DeconvolutedPeak(PeakBase):             # <<<<<<<<<<<<<<
 *     """
 *     Represent a single deconvoluted peak which represents an aggregated isotopic
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak {
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase __pyx_base;
  int (*_eq)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*_create_simple)(double, double, int, double, double, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;


/* "ms_deisotope/_c/peak_set.pyx":455
 * 
 * 
 * cdef class IonMobilityDeconvolutedPeak(DeconvolutedPeak):             # <<<<<<<<<<<<<<
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak __pyx_base;
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak;


/* "ms_deisotope/_c/peak_set.pyx":497
 * 
 * 
 * cdef class DeconvolutedPeakSolution(DeconvolutedPeak):             # <<<<<<<<<<<<<<
 *     """
 *     Extends :class:`DeconvolutedPeak` to also include a reference to
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak __pyx_base;
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution;


/* "ms_deisotope/_c/peak_set.pyx":544
 * 
 * 
 * cdef class DeconvolutedPeakSet:             # <<<<<<<<<<<<<<
 *     """
 *     Represents a collection of :class:`DeconvolutedPeak` instances under multiple orderings.
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet {
  PyObject *(*reindex)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *(*clone)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *(*copy)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*_has_peak)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak *__pyx_optional_args);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*has_peak)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak *__pyx_optional_args);
  PyObject *(*all_peaks_for)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for *__pyx_optional_args);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*_get_nearest_peak)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, double *);
  size_t (*get_size)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*getitem)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, size_t);
  PyObject *(*getslice)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, size_t, size_t);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;


/* "ms_deisotope/_c/peak_set.pyx":1139
 * INTERVAL_INDEX_SIZE = 0
 * 
 * cdef class DeconvolutedPeakSetIndexed(DeconvolutedPeakSet):             # <<<<<<<<<<<<<<
 *     def __init__(self, peaks):
 *         self.neutral_mass_array = NULL
 */

struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed {
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet __pyx_base;
  void (*_build_index_arrays)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *);
  int (*_interval_for)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *, double, double, size_t *, size_t *);
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed;


/* "View.MemoryView":105
 * 
 * @cname("__pyx_array")
 * cdef class array:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

struct __pyx_vtabstruct_array {
  PyObject *(*get_memview)(struct __pyx_array_obj *);
};
static struct __pyx_vtabstruct_array *__pyx_vtabptr_array;


/* "View.MemoryView":330
 * 
 * @cname('__pyx_memoryview')
 * cdef class memoryview(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef object obj
 */

struct __pyx_vtabstruct_memoryview {
  char *(*get_item_pointer)(struct __pyx_memoryview_obj *, PyObject *);
  PyObject *(*is_slice)(struct __pyx_memoryview_obj *, PyObject *);
  PyObject *(*setitem_slice_assignment)(struct __pyx_memoryview_obj *, PyObject *, PyObject *);
  PyObject *(*setitem_slice_assign_scalar)(struct __pyx_memoryview_obj *, struct __pyx_memoryview_obj *, PyObject *);
  PyObject *(*setitem_indexed)(struct __pyx_memoryview_obj *, PyObject *, PyObject *);
  PyObject *(*convert_item_to_object)(struct __pyx_memoryview_obj *, char *);
  PyObject *(*assign_item_from_object)(struct __pyx_memoryview_obj *, char *, PyObject *);
};
static struct __pyx_vtabstruct_memoryview *__pyx_vtabptr_memoryview;


/* "View.MemoryView":965
 * 
 * @cname('__pyx_memoryviewslice')
 * cdef class _memoryviewslice(memoryview):             # <<<<<<<<<<<<<<
 *     "Internal class for passing memoryview slices to Python"
 * 
 */

struct __pyx_vtabstruct__memoryviewslice {
  struct __pyx_vtabstruct_memoryview __pyx_base;
};
static struct __pyx_vtabstruct__memoryviewslice *__pyx_vtabptr__memoryviewslice;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

/* StringJoin.proto */
#if PY_MAJOR_VERSION < 3
#define __Pyx_PyString_Join __Pyx_PyBytes_Join
#define __Pyx_PyBaseString_Join(s, v) (PyUnicode_CheckExact(s) ? PyUnicode_Join(s, v) : __Pyx_PyBytes_Join(s, v))
#else
#define __Pyx_PyString_Join PyUnicode_Join
#define __Pyx_PyBaseString_Join PyUnicode_Join
#endif
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION < 3
    #define __Pyx_PyBytes_Join _PyString_Join
    #else
    #define __Pyx_PyBytes_Join _PyBytes_Join
    #endif
#else
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* py_abs.proto */
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject *__Pyx_PyLong_AbsNeg(PyObject *num);
#define __Pyx_PyNumber_Absolute(x)\
    ((likely(PyLong_CheckExact(x))) ?\
         (likely(Py_SIZE(x) >= 0) ? (Py_INCREF(x), (x)) : __Pyx_PyLong_AbsNeg(x)) :\
         PyNumber_Absolute(x))
#else
#define __Pyx_PyNumber_Absolute(x)  PyNumber_Absolute(x)
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* pyobject_as_double.proto */
static double __Pyx__PyObject_AsDouble(PyObject* obj);
#if CYTHON_COMPILING_IN_PYPY
#define __Pyx_PyObject_AsDouble(obj)\
(likely(PyFloat_CheckExact(obj)) ? PyFloat_AS_DOUBLE(obj) :\
 likely(PyInt_CheckExact(obj)) ?\
 PyFloat_AsDouble(obj) : __Pyx__PyObject_AsDouble(obj))
#else
#define __Pyx_PyObject_AsDouble(obj)\
((likely(PyFloat_CheckExact(obj))) ?\
 PyFloat_AS_DOUBLE(obj) : __Pyx__PyObject_AsDouble(obj))
#endif

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* UnicodeAsUCS4.proto */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject*);

/* object_ord.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyObject_Ord(c)\
    (likely(PyUnicode_Check(c)) ? (long)__Pyx_PyUnicode_AsPy_UCS4(c) : __Pyx__PyObject_Ord(c))
#else
#define __Pyx_PyObject_Ord(c) __Pyx__PyObject_Ord(c)
#endif
static long __Pyx__PyObject_Ord(PyObject* c);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
#define __Pyx_unpack_tuple2(tuple, value1, value2, is_tuple, has_known_size, decref_tuple)\
    (likely(is_tuple || PyTuple_Check(tuple)) ?\
        (likely(has_known_size || PyTuple_GET_SIZE(tuple) == 2) ?\
            __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple) :\
            (__Pyx_UnpackTupleError(tuple, 2), -1)) :\
        __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple))
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* IsLittleEndian.proto */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void);

/* BufferFormatCheck.proto */
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts);
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type);

/* BufferGetAndValidate.proto */
#define __Pyx_GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack)\
    ((obj == Py_None || obj == NULL) ?\
    (__Pyx_ZeroBuffer(buf), 0) :\
    __Pyx__GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack))
static int  __Pyx__GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static void __Pyx_ZeroBuffer(Py_buffer* buf);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);
static Py_ssize_t __Pyx_minusones[] = { -1, -1, -1, -1, -1, -1, -1, -1 };
static Py_ssize_t __Pyx_zeros[] = { 0, 0, 0, 0, 0, 0, 0, 0 };

#define __Pyx_BufPtrCContig1d(type, buf, i0, s0) ((type)buf + i0)
/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* None.proto */
static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t, Py_ssize_t);

/* UnaryNegOverflows.proto */
#define UNARY_NEG_WOULD_OVERFLOW(x)\
        (((x) < 0) & ((unsigned long)(x) == 0-(unsigned long)(x)))

static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static PyObject *__pyx_array_get_memview(struct __pyx_array_obj *); /*proto*/
/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* decode_c_string_utf16.proto */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_string.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* ListExtend.proto */
static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

/* None.proto */
static CYTHON_INLINE long __Pyx_div_long(long, long);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* ClassMethod.proto */
#include "descrobject.h"
static CYTHON_UNUSED PyObject* __Pyx_Method_ClassMethod(PyObject *method);

/* GetNameInClass.proto */
#define __Pyx_GetNameInClass(var, nmspace, name)  (var) = __Pyx__GetNameInClass(nmspace, name)
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name);

/* FusedFunction.proto */
typedef struct {
    __pyx_CyFunctionObject func;
    PyObject *__signatures__;
    PyObject *type;
    PyObject *self;
} __pyx_FusedFunctionObject;
static PyObject *__pyx_FusedFunction_New(PyMethodDef *ml, int flags,
                                         PyObject *qualname, PyObject *closure,
                                         PyObject *module, PyObject *globals,
                                         PyObject *code);
static int __pyx_FusedFunction_clear(__pyx_FusedFunctionObject *self);
static PyTypeObject *__pyx_FusedFunctionType = NULL;
static int __pyx_FusedFunction_init(void);
#define __Pyx_FusedFunction_USED

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


/* BufferStructDeclare.proto */
typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

/* MemviewSliceIsContig.proto */
static int __pyx_memviewslice_is_contig(const __Pyx_memviewslice mvs, char order, int ndim);

/* OverlappingSlices.proto */
static int __pyx_slices_overlap(__Pyx_memviewslice *slice1,
                                __Pyx_memviewslice *slice2,
                                int ndim, size_t itemsize);

/* Capsule.proto */
static CYTHON_INLINE PyObject *__pyx_capsule_create(void *p, const char *sig);

/* TypeInfoCompare.proto */
static int __pyx_typeinfo_cmp(__Pyx_TypeInfo *a, __Pyx_TypeInfo *b);

/* MemviewSliceValidateAndInit.proto */
static int __Pyx_ValidateAndInit_memviewslice(
                int *axes_specs,
                int c_or_f_flag,
                int buf_flags,
                int ndim,
                __Pyx_TypeInfo *dtype,
                __Pyx_BufFmt_StackElem stack[],
                __Pyx_memviewslice *memviewslice,
                PyObject *original_obj);

/* ObjectToMemviewSlice.proto */
static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_float(PyObject *, int writable_flag);

/* ObjectToMemviewSlice.proto */
static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_double(PyObject *, int writable_flag);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell;
static PyObject* __pyx_convert__to_py_struct____pyx_t_12ms_deisotope_2_c_8peak_set_index_cell(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell s);
/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* MemviewSliceCopyTemplate.proto */
static __Pyx_memviewslice
__pyx_memoryview_copy_new_contig(const __Pyx_memviewslice *from_mvs,
                                 const char *mode, int ndim,
                                 size_t sizeof_dtype, int contig_flag,
                                 int dtype_is_object);

/* MemviewSliceInit.proto */
#define __Pyx_BUF_MAX_NDIMS %(BUF_MAX_NDIMS)d
#define __Pyx_MEMVIEW_DIRECT   1
#define __Pyx_MEMVIEW_PTR      2
#define __Pyx_MEMVIEW_FULL     4
#define __Pyx_MEMVIEW_CONTIG   8
#define __Pyx_MEMVIEW_STRIDED  16
#define __Pyx_MEMVIEW_FOLLOW   32
#define __Pyx_IS_C_CONTIG 1
#define __Pyx_IS_F_CONTIG 2
static int __Pyx_init_memviewslice(
                struct __pyx_memoryview_obj *memview,
                int ndim,
                __Pyx_memviewslice *memviewslice,
                int memview_is_new_reference);
static CYTHON_INLINE int __pyx_add_acquisition_count_locked(
    __pyx_atomic_int *acquisition_count, PyThread_type_lock lock);
static CYTHON_INLINE int __pyx_sub_acquisition_count_locked(
    __pyx_atomic_int *acquisition_count, PyThread_type_lock lock);
#define __pyx_get_slice_count_pointer(memview) (memview->acquisition_count_aligned_p)
#define __pyx_get_slice_count(memview) (*__pyx_get_slice_count_pointer(memview))
#define __PYX_INC_MEMVIEW(slice, have_gil) __Pyx_INC_MEMVIEW(slice, have_gil, __LINE__)
#define __PYX_XDEC_MEMVIEW(slice, have_gil) __Pyx_XDEC_MEMVIEW(slice, have_gil, __LINE__)
static CYTHON_INLINE void __Pyx_INC_MEMVIEW(__Pyx_memviewslice *, int, int);
static CYTHON_INLINE void __Pyx_XDEC_MEMVIEW(__Pyx_memviewslice *, int, int);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_uint32(npy_uint32 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* BytesContains.proto */
static CYTHON_INLINE int __Pyx_BytesContains(PyObject* bytes, char character);

/* ImportNumPyArray.proto */
static PyObject *__pyx_numpy_ndarray = NULL;
static PyObject* __Pyx_ImportNumPyArrayTypeIfAvailable(void);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE char __Pyx_PyInt_As_char(PyObject *);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* CoroutineBase.proto */
typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* VoidPtrExport.proto */
static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(size_t __pyx_v_neutral_mass, size_t __pyx_v_mz); /* proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create(double __pyx_v_mz, double __pyx_v_intensity); /* proto*/
static CYTHON_INLINE size_t __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto*/
static CYTHON_INLINE struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, size_t __pyx_v_i); /* proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(PyObject *__pyx_v_pairs); /* proto*/
static struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__create_simple(double __pyx_v_neutral_mass, double __pyx_v_intensity, int __pyx_v_charge, double __pyx_v_score, double __pyx_v_mz, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_envelope); /* proto*/
static struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_f_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_get_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_copy(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak *__pyx_optional_args); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak *__pyx_optional_args); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_getitem(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, size_t __pyx_v_i); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_getslice(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, size_t __pyx_v_start, size_t __pyx_v_end); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for *__pyx_optional_args); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__get_nearest_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, double *__pyx_v_errout); /* proto*/
static void __pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__build_index_arrays(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for *__pyx_optional_args); /* proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__interval_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance, size_t *__pyx_v_start, size_t *__pyx_v_end); /* proto*/
static PyObject *__pyx_array_get_memview(struct __pyx_array_obj *__pyx_v_self); /* proto*/
static char *__pyx_memoryview_get_item_pointer(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index); /* proto*/
static PyObject *__pyx_memoryview_is_slice(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj); /* proto*/
static PyObject *__pyx_memoryview_setitem_slice_assignment(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_dst, PyObject *__pyx_v_src); /* proto*/
static PyObject *__pyx_memoryview_setitem_slice_assign_scalar(struct __pyx_memoryview_obj *__pyx_v_self, struct __pyx_memoryview_obj *__pyx_v_dst, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview_setitem_indexed(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview_convert_item_to_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp); /* proto*/
static PyObject *__pyx_memoryview_assign_item_from_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryviewslice_convert_item_to_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp); /* proto*/
static PyObject *__pyx_memoryviewslice_assign_item_from_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value); /* proto*/

/* Module declarations from 'brainpy._c.composition' */
static PyTypeObject *__pyx_ptype_7brainpy_2_c_11composition_PyComposition = 0;
static PyObject **__pyx_vp_7brainpy_2_c_11composition_nist_mass = 0;
#define __pyx_v_7brainpy_2_c_11composition_nist_mass (*__pyx_vp_7brainpy_2_c_11composition_nist_mass)
static double *__pyx_vp_7brainpy_2_c_11composition_PROTON = 0;
#define __pyx_v_7brainpy_2_c_11composition_PROTON (*__pyx_vp_7brainpy_2_c_11composition_PROTON)
static struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable **__pyx_vp_7brainpy_2_c_11composition__ElementTable = 0;
#define __pyx_v_7brainpy_2_c_11composition__ElementTable (*__pyx_vp_7brainpy_2_c_11composition__ElementTable)

/* Module declarations from 'brainpy._c.double_vector' */

/* Module declarations from 'brainpy._c.isotopic_constants' */
static size_t *__pyx_vp_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE = 0;
#define __pyx_v_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE (*__pyx_vp_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE)

/* Module declarations from 'brainpy._c.isotopic_distribution' */
static PyTypeObject *__pyx_ptype_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak = 0;

/* Module declarations from 'ms_peak_picker._c.peak_set' */
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase = 0;
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_8peak_set_FittedPeak = 0;
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSet = 0;
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed = 0;
static size_t *__pyx_vp_14ms_peak_picker_2_c_8peak_set_INTERVAL_INDEX_SIZE = 0;
#define __pyx_v_14ms_peak_picker_2_c_8peak_set_INTERVAL_INDEX_SIZE (*__pyx_vp_14ms_peak_picker_2_c_8peak_set_INTERVAL_INDEX_SIZE)

/* Module declarations from 'cython' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/

/* Module declarations from 'ms_peak_picker._c.peak_index' */
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_10peak_index_PeakIndex = 0;

/* Module declarations from 'ms_deisotope._c.averagine' */
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_9averagine_Averagine = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_9averagine_AveragineCache = 0;
static double *__pyx_vp_12ms_deisotope_2_c_9averagine_PROTON = 0;
#define __pyx_v_12ms_deisotope_2_c_9averagine_PROTON (*__pyx_vp_12ms_deisotope_2_c_9averagine_PROTON)
static double (*__pyx_f_12ms_deisotope_2_c_9averagine_mass_charge_ratio)(double, int, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_mass_charge_ratio *__pyx_optional_args); /*proto*/

/* Module declarations from 'ms_deisotope._c.deconvoluter_base' */
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator = 0;

/* Module declarations from 'ms_deisotope._c.scoring' */
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_FitSelectorBase = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitterBase = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_LeastSquaresFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_GTestFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledGTestFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_MSDeconVFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_FunctionScorer = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_InterferenceDetection = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_DistinctPatternFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_DotProductFitter = 0;
static double *__pyx_vp_12ms_deisotope_2_c_7scoring_INFINITY = 0;
#define __pyx_v_12ms_deisotope_2_c_7scoring_INFINITY (*__pyx_vp_12ms_deisotope_2_c_7scoring_INFINITY)

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'libc.math' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'ms_deisotope._c.peak_set' */
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set__Index = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr = 0;
static PyTypeObject *__pyx_array_type = 0;
static PyTypeObject *__pyx_MemviewEnum_type = 0;
static PyTypeObject *__pyx_memoryview_type = 0;
static PyTypeObject *__pyx_memoryviewslice_type = 0;
static size_t __pyx_v_12ms_deisotope_2_c_8peak_set_INTERVAL_INDEX_SIZE;
static PyObject *__pyx_v_12ms_deisotope_2_c_8peak_set_neutral_mass_getter = 0;
static PyObject *__pyx_v_12ms_deisotope_2_c_8peak_set_mz_getter = 0;
static double __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon;
static double __pyx_v_12ms_deisotope_2_c_8peak_set_INF;
static PyObject *generic = 0;
static PyObject *strided = 0;
static PyObject *indirect = 0;
static PyObject *contiguous = 0;
static PyObject *indirect_contiguous = 0;
static int __pyx_memoryview_thread_locks_used;
static PyThread_type_lock __pyx_memoryview_thread_locks[8];
static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval(double *, double, double, size_t, size_t *, size_t *); /*proto*/
static double __pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(double, double); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set__Index_reconstructor(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set__Index_reconstructor *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set__Index_to_bytes(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set__Index_from_bytes(PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_to_bytes(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_from_bytes(PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_reconstructor(PyObject *, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_solution_neutral_mass(PyObject *, double, size_t, double); /*proto*/
static double __pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(double, double); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search(double *, double, double, size_t, size_t *); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_neutral_mass(PyObject *, double, double); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_solution_mz(PyObject *, double, size_t, double); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_mz(PyObject *, double, double); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_nearest_match_neutral_mass(PyObject *, double, size_t, size_t, double *); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_nearest_neutral_mass(PyObject *, double, double *); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_with_hint(double *, double, double, size_t, size_t, size_t *); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval_with_hint(double *, double, double, size_t, size_t, size_t *, size_t *); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_build_interval_index(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *, size_t); /*proto*/
static double *__pyx_f_12ms_deisotope_2_c_8peak_set_build_linear_spaced_array(double, double, size_t); /*proto*/
static void __pyx_f_12ms_deisotope_2_c_8peak_set_free_index_list(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_check_index(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *); /*proto*/
static size_t __pyx_f_12ms_deisotope_2_c_8peak_set_interpolate_index(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *, double); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_find_search_interval(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *, double, size_t *, size_t *); /*proto*/
static struct __pyx_array_obj *__pyx_array_new(PyObject *, Py_ssize_t, char *, char *, char *); /*proto*/
static void *__pyx_align_pointer(void *, size_t); /*proto*/
static PyObject *__pyx_memoryview_new(PyObject *, int, int, __Pyx_TypeInfo *); /*proto*/
static CYTHON_INLINE int __pyx_memoryview_check(PyObject *); /*proto*/
static PyObject *_unellipsify(PyObject *, int); /*proto*/
static PyObject *assert_direct_dimensions(Py_ssize_t *, int); /*proto*/
static struct __pyx_memoryview_obj *__pyx_memview_slice(struct __pyx_memoryview_obj *, PyObject *); /*proto*/
static int __pyx_memoryview_slice_memviewslice(__Pyx_memviewslice *, Py_ssize_t, Py_ssize_t, Py_ssize_t, int, int, int *, Py_ssize_t, Py_ssize_t, Py_ssize_t, int, int, int, int); /*proto*/
static char *__pyx_pybuffer_index(Py_buffer *, char *, Py_ssize_t, Py_ssize_t); /*proto*/
static int __pyx_memslice_transpose(__Pyx_memviewslice *); /*proto*/
static PyObject *__pyx_memoryview_fromslice(__Pyx_memviewslice, int, PyObject *(*)(char *), int (*)(char *, PyObject *), int); /*proto*/
static __Pyx_memviewslice *__pyx_memoryview_get_slice_from_memoryview(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static void __pyx_memoryview_slice_copy(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static PyObject *__pyx_memoryview_copy_object(struct __pyx_memoryview_obj *); /*proto*/
static PyObject *__pyx_memoryview_copy_object_from_slice(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static Py_ssize_t abs_py_ssize_t(Py_ssize_t); /*proto*/
static char __pyx_get_best_slice_order(__Pyx_memviewslice *, int); /*proto*/
static void _copy_strided_to_strided(char *, Py_ssize_t *, char *, Py_ssize_t *, Py_ssize_t *, Py_ssize_t *, int, size_t); /*proto*/
static void copy_strided_to_strided(__Pyx_memviewslice *, __Pyx_memviewslice *, int, size_t); /*proto*/
static Py_ssize_t __pyx_memoryview_slice_get_size(__Pyx_memviewslice *, int); /*proto*/
static Py_ssize_t __pyx_fill_contig_strides_array(Py_ssize_t *, Py_ssize_t *, Py_ssize_t, int, char); /*proto*/
static void *__pyx_memoryview_copy_data_to_temp(__Pyx_memviewslice *, __Pyx_memviewslice *, char, int); /*proto*/
static int __pyx_memoryview_err_extents(int, Py_ssize_t, Py_ssize_t); /*proto*/
static int __pyx_memoryview_err_dim(PyObject *, char *, int); /*proto*/
static int __pyx_memoryview_err(PyObject *, char *); /*proto*/
static int __pyx_memoryview_copy_contents(__Pyx_memviewslice, __Pyx_memviewslice, int, int, int); /*proto*/
static void __pyx_memoryview_broadcast_leading(__Pyx_memviewslice *, int, int); /*proto*/
static void __pyx_memoryview_refcount_copying(__Pyx_memviewslice *, int, int, int); /*proto*/
static void __pyx_memoryview_refcount_objects_in_slice_with_gil(char *, Py_ssize_t *, Py_ssize_t *, int, int); /*proto*/
static void __pyx_memoryview_refcount_objects_in_slice(char *, Py_ssize_t *, Py_ssize_t *, int, int); /*proto*/
static void __pyx_memoryview_slice_assign_scalar(__Pyx_memviewslice *, int, size_t, void *, int); /*proto*/
static void __pyx_memoryview__slice_assign_scalar(char *, Py_ssize_t *, Py_ssize_t *, int, size_t, void *); /*proto*/
static PyObject *__pyx_unpickle_Enum__set_state(struct __pyx_MemviewEnum_obj *, PyObject *); /*proto*/
static __Pyx_TypeInfo __Pyx_TypeInfo_float = { "float", NULL, sizeof(float), { 0 }, 0, 'R', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_double = { "double", NULL, sizeof(double), { 0 }, 0, 'R', 0, 0 };
#define __Pyx_MODULE_NAME "ms_deisotope._c.peak_set"
extern int __pyx_module_is_main_ms_deisotope___c__peak_set;
int __pyx_module_is_main_ms_deisotope___c__peak_set = 0;

/* Implementation of 'ms_deisotope._c.peak_set' */
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_IndexError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_map;
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_sorted;
static PyObject *__pyx_builtin_AttributeError;
static PyObject *__pyx_builtin_ImportError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_Ellipsis;
static PyObject *__pyx_builtin_id;
static const char __pyx_k_[] = ", ";
static const char __pyx_k_O[] = "O";
static const char __pyx_k_a[] = "a";
static const char __pyx_k_b[] = "b";
static const char __pyx_k_c[] = "c";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_n[] = "n";
static const char __pyx_k_s[] = "[%s]";
static const char __pyx_k__2[] = "";
static const char __pyx_k__3[] = "()";
static const char __pyx_k__4[] = "|";
static const char __pyx_k_eq[] = "_eq";
static const char __pyx_k_id[] = "id";
static const char __pyx_k_m1[] = "m1";
static const char __pyx_k_m2[] = "m2";
static const char __pyx_k_mz[] = "mz";
static const char __pyx_k_d_d[] = "%d|%d";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_fit[] = "fit";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_inf[] = "inf";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_map[] = "map";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_obj[] = "obj";
static const char __pyx_k_area[] = "area";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_base[] = "base";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_iter[] = "__iter__";
static const char __pyx_k_join[] = "join";
static const char __pyx_k_kind[] = "kind";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mode[] = "mode";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_ndim[] = "ndim";
static const char __pyx_k_pack[] = "pack";
static const char __pyx_k_peak[] = "peak";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_step[] = "step";
static const char __pyx_k_stop[] = "stop";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_ASCII[] = "ASCII";
static const char __pyx_k_Index[] = "_Index";
static const char __pyx_k_array[] = "array";
static const char __pyx_k_class[] = "__class__";
static const char __pyx_k_clone[] = "clone";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_error[] = "error";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_float[] = "float";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_pairs[] = "pairs";
static const char __pyx_k_peaks[] = "peaks";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_score[] = "score";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_strip[] = "strip";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_charge[] = "charge";
static const char __pyx_k_double[] = "double";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_kwargs[] = "kwargs";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_sorted[] = "sorted";
static const char __pyx_k_struct[] = "struct";
static const char __pyx_k_unpack[] = "unpack";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_use_mz[] = "use_mz";
static const char __pyx_k_convert[] = "convert";
static const char __pyx_k_fortran[] = "fortran";
static const char __pyx_k_genexpr[] = "genexpr";
static const char __pyx_k_indexed[] = "indexed";
static const char __pyx_k_memview[] = "memview";
static const char __pyx_k_reindex[] = "reindex";
static const char __pyx_k_Ellipsis[] = "Ellipsis";
static const char __pyx_k_Envelope[] = "Envelope";
static const char __pyx_k_defaults[] = "defaults";
static const char __pyx_k_envelope[] = "envelope";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_has_peak[] = "has_peak";
static const char __pyx_k_itemsize[] = "itemsize";
static const char __pyx_k_operator[] = "operator";
static const char __pyx_k_pyx_capi[] = "__pyx_capi__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_solution[] = "solution";
static const char __pyx_k_0_4f_0_2f[] = "(%0.4f, %0.2f)";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_converted[] = "converted";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_from_peak[] = "from_peak";
static const char __pyx_k_intensity[] = "intensity";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_tolerance[] = "tolerance";
static const char __pyx_k_IndexError[] = "IndexError";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_attrgetter[] = "attrgetter";
static const char __pyx_k_drift_time[] = "drift_time";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_signatures[] = "signatures";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_EnvelopePair[] = "EnvelopePair";
static const char __pyx_k_average_mass[] = "average_mass";
static const char __pyx_k_center_value[] = "center_value";
static const char __pyx_k_envelope_obj[] = "envelope_obj";
static const char __pyx_k_neutral_mass[] = "neutral_mass";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_a_to_a2_ratio[] = "a_to_a2_ratio";
static const char __pyx_k_all_peaks_for[] = "all_peaks_for";
static const char __pyx_k_envelope_list[] = "envelope_list";
static const char __pyx_k_pyx_getbuffer[] = "__pyx_getbuffer";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_AttributeError[] = "AttributeError";
static const char __pyx_k_View_MemoryView[] = "View.MemoryView";
static const char __pyx_k_allocate_buffer[] = "allocate_buffer";
static const char __pyx_k_chosen_for_msms[] = "chosen_for_msms";
static const char __pyx_k_dtype_is_object[] = "dtype_is_object";
static const char __pyx_k_error_tolerance[] = "error_tolerance";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_release_buffers[] = "_release_buffers";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_signal_to_noise[] = "signal_to_noise";
static const char __pyx_k_DeconvolutedPeak[] = "DeconvolutedPeak";
static const char __pyx_k_current_envelope[] = "current_envelope";
static const char __pyx_k_decode_envelopes[] = "decode_envelopes";
static const char __pyx_k_pyx_unpickle_Enum[] = "__pyx_unpickle_Enum";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_most_abundant_mass[] = "most_abundant_mass";
static const char __pyx_k_strided_and_direct[] = "<strided and direct>";
static const char __pyx_k_DeconvolutedPeakSet[] = "DeconvolutedPeakSet";
static const char __pyx_k_EnvelopePair___iter[] = "EnvelopePair.__iter__";
static const char __pyx_k_INTERVAL_INDEX_SIZE[] = "INTERVAL_INDEX_SIZE";
static const char __pyx_k_Index_reconstructor[] = "_Index_reconstructor";
static const char __pyx_k_init___locals_lambda[] = "__init__.<locals>.<lambda>";
static const char __pyx_k_strided_and_indirect[] = "<strided and indirect>";
static const char __pyx_k_contiguous_and_direct[] = "<contiguous and direct>";
static const char __pyx_k_repr___locals_genexpr[] = "__repr__.<locals>.genexpr";
static const char __pyx_k_Envelope_reconstructor[] = "Envelope_reconstructor";
static const char __pyx_k_MemoryView_of_r_object[] = "<MemoryView of %r object>";
static const char __pyx_k_full_width_at_half_max[] = "full_width_at_half_max";
static const char __pyx_k_MemoryView_of_r_at_0x_x[] = "<MemoryView of %r at 0x%x>";
static const char __pyx_k_contiguous_and_indirect[] = "<contiguous and indirect>";
static const char __pyx_k_Cannot_index_with_type_s[] = "Cannot index with type '%s'";
static const char __pyx_k_DeconvolutedPeakSolution[] = "DeconvolutedPeakSolution";
static const char __pyx_k_getitem___locals_genexpr[] = "__getitem__.<locals>.genexpr";
static const char __pyx_k_ms_deisotope__c_peak_set[] = "ms_deisotope._c.peak_set";
static const char __pyx_k_Invalid_shape_in_axis_d_d[] = "Invalid shape in axis %d: %d.";
static const char __pyx_k_DeconvolutedPeakSetIndexed[] = "DeconvolutedPeakSetIndexed";
static const char __pyx_k_DeconvolutedPeakSet_d_Peaks[] = "<DeconvolutedPeakSet %d Peaks>";
static const char __pyx_k_IonMobilityDeconvolutedPeak[] = "IonMobilityDeconvolutedPeak";
static const char __pyx_k_No_matching_signature_found[] = "No matching signature found";
static const char __pyx_k_itemsize_0_for_cython_array[] = "itemsize <= 0 for cython.array";
static const char __pyx_k_ms_deisotope__c_peak_set_pyx[] = "ms_deisotope/_c/peak_set.pyx";
static const char __pyx_k_unable_to_allocate_array_data[] = "unable to allocate array data.";
static const char __pyx_k_strided_and_direct_or_indirect[] = "<strided and direct or indirect>";
static const char __pyx_k_DeconvolutedPeakSolution___iter[] = "DeconvolutedPeakSolution.__iter__";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_self___class_____name___a_to_a2[] = "{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, envelope={self.envelope}, full_width_at_half_max={self.full_width_at_half_max}, index={self.index}, intensity={self.intensity}, most_abundant_mass={self.most_abundant_mass}, mz={self.mz}, neutral_mass={self.neutral_mass}, score={self.score}, signal_to_noise={self.signal_to_noise})";
static const char __pyx_k_Buffer_view_does_not_expose_stri[] = "Buffer view does not expose strides";
static const char __pyx_k_Can_only_create_a_buffer_that_is[] = "Can only create a buffer that is contiguous in memory.";
static const char __pyx_k_Cannot_assign_to_read_only_memor[] = "Cannot assign to read-only memoryview";
static const char __pyx_k_Cannot_create_writable_memory_vi[] = "Cannot create writable memory view from read-only memoryview";
static const char __pyx_k_Empty_shape_tuple_for_cython_arr[] = "Empty shape tuple for cython.array";
static const char __pyx_k_Expected_at_least_d_argument_s_g[] = "Expected at least %d argument%s, got %d";
static const char __pyx_k_Function_call_with_ambiguous_arg[] = "Function call with ambiguous argument types";
static const char __pyx_k_Incompatible_checksums_s_vs_0xb0[] = "Incompatible checksums (%s vs 0xb068931 = (name))";
static const char __pyx_k_Indirect_dimensions_not_supporte[] = "Indirect dimensions not supported";
static const char __pyx_k_Invalid_mode_expected_c_or_fortr[] = "Invalid mode, expected 'c' or 'fortran', got %s";
static const char __pyx_k_Out_of_bounds_on_buffer_access_a[] = "Out of bounds on buffer access (axis %d)";
static const char __pyx_k_Unable_to_convert_item_to_object[] = "Unable to convert item to object";
static const char __pyx_k_got_differing_extents_in_dimensi[] = "got differing extents in dimension %d (got %d and %d)";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_unable_to_allocate_shape_and_str[] = "unable to allocate shape and strides.";
static const char __pyx_k_self___class_____name___a_to_a2_2[] = "{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, envelope={self.envelope}, full_width_at_half_max={self.full_width_at_half_max}, index={self.index}, intensity={self.intensity}, most_abundant_mass={self.most_abundant_mass}, mz={self.mz}, neutral_mass={self.neutral_mass}, score={self.score}, signal_to_noise={self.signal_to_noise},drift_time={self.drift_time})";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_kp_s_0_4f_0_2f;
static PyObject *__pyx_n_s_ASCII;
static PyObject *__pyx_n_s_AttributeError;
static PyObject *__pyx_kp_s_Buffer_view_does_not_expose_stri;
static PyObject *__pyx_kp_s_Can_only_create_a_buffer_that_is;
static PyObject *__pyx_kp_s_Cannot_assign_to_read_only_memor;
static PyObject *__pyx_kp_s_Cannot_create_writable_memory_vi;
static PyObject *__pyx_kp_s_Cannot_index_with_type_s;
static PyObject *__pyx_n_s_DeconvolutedPeak;
static PyObject *__pyx_n_s_DeconvolutedPeakSet;
static PyObject *__pyx_n_s_DeconvolutedPeakSetIndexed;
static PyObject *__pyx_kp_s_DeconvolutedPeakSet_d_Peaks;
static PyObject *__pyx_n_s_DeconvolutedPeakSolution;
static PyObject *__pyx_n_s_DeconvolutedPeakSolution___iter;
static PyObject *__pyx_n_s_Ellipsis;
static PyObject *__pyx_kp_s_Empty_shape_tuple_for_cython_arr;
static PyObject *__pyx_n_s_Envelope;
static PyObject *__pyx_n_s_EnvelopePair;
static PyObject *__pyx_n_s_EnvelopePair___iter;
static PyObject *__pyx_n_s_Envelope_reconstructor;
static PyObject *__pyx_kp_s_Expected_at_least_d_argument_s_g;
static PyObject *__pyx_kp_s_Function_call_with_ambiguous_arg;
static PyObject *__pyx_n_s_INTERVAL_INDEX_SIZE;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xb0;
static PyObject *__pyx_n_s_Index;
static PyObject *__pyx_n_s_IndexError;
static PyObject *__pyx_n_s_Index_reconstructor;
static PyObject *__pyx_kp_s_Indirect_dimensions_not_supporte;
static PyObject *__pyx_kp_s_Invalid_mode_expected_c_or_fortr;
static PyObject *__pyx_kp_s_Invalid_shape_in_axis_d_d;
static PyObject *__pyx_n_s_IonMobilityDeconvolutedPeak;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_s_MemoryView_of_r_at_0x_x;
static PyObject *__pyx_kp_s_MemoryView_of_r_object;
static PyObject *__pyx_kp_s_No_matching_signature_found;
static PyObject *__pyx_n_b_O;
static PyObject *__pyx_kp_s_Out_of_bounds_on_buffer_access_a;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_Unable_to_convert_item_to_object;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_View_MemoryView;
static PyObject *__pyx_kp_s__2;
static PyObject *__pyx_kp_s__3;
static PyObject *__pyx_kp_s__4;
static PyObject *__pyx_n_s_a;
static PyObject *__pyx_n_s_a_to_a2_ratio;
static PyObject *__pyx_n_s_all_peaks_for;
static PyObject *__pyx_n_s_allocate_buffer;
static PyObject *__pyx_n_s_area;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_array;
static PyObject *__pyx_n_s_attrgetter;
static PyObject *__pyx_n_s_average_mass;
static PyObject *__pyx_n_s_b;
static PyObject *__pyx_n_s_base;
static PyObject *__pyx_n_s_c;
static PyObject *__pyx_n_u_c;
static PyObject *__pyx_n_s_center_value;
static PyObject *__pyx_n_s_charge;
static PyObject *__pyx_n_s_chosen_for_msms;
static PyObject *__pyx_n_s_class;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_clone;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_kp_s_contiguous_and_direct;
static PyObject *__pyx_kp_s_contiguous_and_indirect;
static PyObject *__pyx_n_s_convert;
static PyObject *__pyx_n_s_converted;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_current_envelope;
static PyObject *__pyx_kp_s_d_d;
static PyObject *__pyx_n_s_decode_envelopes;
static PyObject *__pyx_n_s_defaults;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_double;
static PyObject *__pyx_n_s_drift_time;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_dtype_is_object;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_end;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_envelope;
static PyObject *__pyx_n_s_envelope_list;
static PyObject *__pyx_n_s_envelope_obj;
static PyObject *__pyx_n_s_eq;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_error_tolerance;
static PyObject *__pyx_n_s_fit;
static PyObject *__pyx_n_s_flags;
static PyObject *__pyx_n_s_float;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_fortran;
static PyObject *__pyx_n_u_fortran;
static PyObject *__pyx_n_s_from_peak;
static PyObject *__pyx_n_s_full_width_at_half_max;
static PyObject *__pyx_n_s_genexpr;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_getitem___locals_genexpr;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_kp_s_got_differing_extents_in_dimensi;
static PyObject *__pyx_n_s_has_peak;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_id;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_indexed;
static PyObject *__pyx_n_s_inf;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_init___locals_lambda;
static PyObject *__pyx_n_s_intensity;
static PyObject *__pyx_n_s_itemsize;
static PyObject *__pyx_kp_s_itemsize_0_for_cython_array;
static PyObject *__pyx_n_s_iter;
static PyObject *__pyx_n_s_join;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_kind;
static PyObject *__pyx_n_s_kwargs;
static PyObject *__pyx_n_s_m1;
static PyObject *__pyx_n_s_m2;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_map;
static PyObject *__pyx_n_s_memview;
static PyObject *__pyx_n_s_mode;
static PyObject *__pyx_n_s_most_abundant_mass;
static PyObject *__pyx_n_s_ms_deisotope__c_peak_set;
static PyObject *__pyx_kp_s_ms_deisotope__c_peak_set_pyx;
static PyObject *__pyx_n_s_mz;
static PyObject *__pyx_n_s_n;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_ndim;
static PyObject *__pyx_n_s_neutral_mass;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_s_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_s_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_obj;
static PyObject *__pyx_n_s_operator;
static PyObject *__pyx_n_s_pack;
static PyObject *__pyx_n_s_pairs;
static PyObject *__pyx_n_s_peak;
static PyObject *__pyx_n_s_peaks;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_capi;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_getbuffer;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_Enum;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_reindex;
static PyObject *__pyx_n_s_release_buffers;
static PyObject *__pyx_n_s_repr___locals_genexpr;
static PyObject *__pyx_kp_s_s;
static PyObject *__pyx_n_s_score;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_kp_s_self___class_____name___a_to_a2;
static PyObject *__pyx_kp_s_self___class_____name___a_to_a2_2;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_signal_to_noise;
static PyObject *__pyx_n_s_signatures;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_solution;
static PyObject *__pyx_n_s_sorted;
static PyObject *__pyx_n_s_split;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_step;
static PyObject *__pyx_n_s_stop;
static PyObject *__pyx_kp_s_strided_and_direct;
static PyObject *__pyx_kp_s_strided_and_direct_or_indirect;
static PyObject *__pyx_kp_s_strided_and_indirect;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_strip;
static PyObject *__pyx_n_s_struct;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_tolerance;
static PyObject *__pyx_kp_s_unable_to_allocate_array_data;
static PyObject *__pyx_kp_s_unable_to_allocate_shape_and_str;
static PyObject *__pyx_n_s_unpack;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_use_mz;
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set__Index_reconstructor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_mz); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_2_Index_to_bytes(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_index); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_4_Index_from_bytes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_view); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_mz); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_4clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_6__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2mz___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2mz_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_mz, PyObject *__pyx_v_intensity); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2__getitem__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_i); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_4__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_7__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_11__richcmp__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_13__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6Envelope_to_bytes(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_from_bytes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_view); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_10Envelope_reconstructor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_pairs); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_2__getitem__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_i); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_4__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___genexpr(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_6__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_8__len__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_10_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_12__richcmp__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_14clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_16__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_intensity, PyObject *__pyx_v_charge, PyObject *__pyx_v_signal_to_noise, PyObject *__pyx_v_index, PyObject *__pyx_v_full_width_at_half_max, PyObject *__pyx_v_a_to_a2_ratio, PyObject *__pyx_v_most_abundant_mass, PyObject *__pyx_v_average_mass, PyObject *__pyx_v_score, PyObject *__pyx_v_envelope, PyObject *__pyx_v_mz, PyObject *__pyx_v_fit, PyObject *__pyx_v_chosen_for_msms, PyObject *__pyx_v_area); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static Py_hash_t __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_2__hash__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_4clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_10__richcmp__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_intensity, PyObject *__pyx_v_charge, PyObject *__pyx_v_signal_to_noise, PyObject *__pyx_v_index, PyObject *__pyx_v_full_width_at_half_max, PyObject *__pyx_v_a_to_a2_ratio, PyObject *__pyx_v_most_abundant_mass, PyObject *__pyx_v_average_mass, PyObject *__pyx_v_score, PyObject *__pyx_v_envelope, PyObject *__pyx_v_mz, PyObject *__pyx_v_fit, PyObject *__pyx_v_chosen_for_msms, PyObject *__pyx_v_area, PyObject *__pyx_v_drift_time); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_2clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_4__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_6_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_8__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self, PyObject *__pyx_v_solution, PyObject *__pyx_v_fit, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_2clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_4__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_6__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_9from_peak(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_peak, PyObject *__pyx_v_solution); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_peaks); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_2reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_4__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_6__len__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_8__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___genexpr(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_10__getitem__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_item); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_12__eq__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_14__ne__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_16__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_18clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_20copy(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_22__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_24has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_error_tolerance, int __pyx_v_use_mz); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_26all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_28get_nearest_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_30between(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_m1, PyObject *__pyx_v_m2, CYTHON_UNUSED PyObject *__pyx_v_tolerance, PyObject *__pyx_v_use_mz); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12convert(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_14decode_envelopes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16decode_envelopes(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_array); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_18decode_envelopes(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_array); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, PyObject *__pyx_v_peaks); /* proto */
static void __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_2__dealloc__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_4__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_6__getstate__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_8__setstate__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, PyObject *__pyx_v_d); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_10_release_buffers(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_12set_interval_index_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, PyObject *__pyx_v_index_size); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_14reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_16_test_interval(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_18all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_20test_interval(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_22find_interval_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_24check_interval(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, size_t __pyx_v_i); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, PyObject *__pyx_v_format, PyObject *__pyx_v_mode, int __pyx_v_allocate_buffer); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(struct __pyx_array_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_5array_7memview___get__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__len__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getattr__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_attr); /* proto */
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__getitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_12__setitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf___pyx_array___reduce_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_array_2__setstate_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(struct __pyx_MemviewEnum_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_MemviewEnum___reduce_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_MemviewEnum_2__setstate_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_v_flags, int __pyx_v_dtype_is_object); /* proto */
static void __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(struct __pyx_memoryview_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryview___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryview_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static void __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_16_memoryviewslice_4base___get__(struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryviewslice___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryviewslice_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView___pyx_unpickle_Enum(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set__Index(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_EnvelopePair(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_Envelope(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_array(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_Enum(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_memoryview(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new__memoryviewslice(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_1eneg_5;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_184977713;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_slice__24;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_codeobj__31;
static PyObject *__pyx_codeobj__33;
static PyObject *__pyx_codeobj__40;
/* Late includes */

/* "ms_deisotope/_c/peak_set.pyx":31
 * 
 * @cython.cdivision
 * cdef double ppm_error(double x, double y):             # <<<<<<<<<<<<<<
 *     return (x - y) / y
 * 
 */

static double __pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(double __pyx_v_x, double __pyx_v_y) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ppm_error", 0);

  /* "ms_deisotope/_c/peak_set.pyx":32
 * @cython.cdivision
 * cdef double ppm_error(double x, double y):
 *     return (x - y) / y             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((__pyx_v_x - __pyx_v_y) / __pyx_v_y);
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":31
 * 
 * @cython.cdivision
 * cdef double ppm_error(double x, double y):             # <<<<<<<<<<<<<<
 *     return (x - y) / y
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":45
 * 
 * 
 * cpdef _Index_reconstructor(object neutral_mass, object mz=None):             # <<<<<<<<<<<<<<
 *     if isinstance(neutral_mass, int):
 *         return _Index._create(neutral_mass, mz)
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_1_Index_reconstructor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set__Index_reconstructor(PyObject *__pyx_v_neutral_mass, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set__Index_reconstructor *__pyx_optional_args) {
  PyObject *__pyx_v_mz = ((PyObject *)Py_None);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_Index_reconstructor", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_mz = __pyx_optional_args->mz;
    }
  }

  /* "ms_deisotope/_c/peak_set.pyx":46
 * 
 * cpdef _Index_reconstructor(object neutral_mass, object mz=None):
 *     if isinstance(neutral_mass, int):             # <<<<<<<<<<<<<<
 *         return _Index._create(neutral_mass, mz)
 *     elif isinstance(neutral_mass, bytearray):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_neutral_mass); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":47
 * cpdef _Index_reconstructor(object neutral_mass, object mz=None):
 *     if isinstance(neutral_mass, int):
 *         return _Index._create(neutral_mass, mz)             # <<<<<<<<<<<<<<
 *     elif isinstance(neutral_mass, bytearray):
 *         return _Index_from_bytes(neutral_mass)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_v_neutral_mass); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 47, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_v_mz); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 47, __pyx_L1_error)
    __pyx_t_5 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(__pyx_t_3, __pyx_t_4)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 47, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":46
 * 
 * cpdef _Index_reconstructor(object neutral_mass, object mz=None):
 *     if isinstance(neutral_mass, int):             # <<<<<<<<<<<<<<
 *         return _Index._create(neutral_mass, mz)
 *     elif isinstance(neutral_mass, bytearray):
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":48
 *     if isinstance(neutral_mass, int):
 *         return _Index._create(neutral_mass, mz)
 *     elif isinstance(neutral_mass, bytearray):             # <<<<<<<<<<<<<<
 *         return _Index_from_bytes(neutral_mass)
 *     else:
 */
  __pyx_t_2 = PyByteArray_Check(__pyx_v_neutral_mass); 
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (likely(__pyx_t_1)) {

    /* "ms_deisotope/_c/peak_set.pyx":49
 *         return _Index._create(neutral_mass, mz)
 *     elif isinstance(neutral_mass, bytearray):
 *         return _Index_from_bytes(neutral_mass)             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError(type(neutral_mass))
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(PyByteArray_CheckExact(__pyx_v_neutral_mass))||((__pyx_v_neutral_mass) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytearray", Py_TYPE(__pyx_v_neutral_mass)->tp_name), 0))) __PYX_ERR(0, 49, __pyx_L1_error)
    __pyx_t_5 = __pyx_f_12ms_deisotope_2_c_8peak_set__Index_from_bytes(((PyObject*)__pyx_v_neutral_mass), 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 49, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":48
 *     if isinstance(neutral_mass, int):
 *         return _Index._create(neutral_mass, mz)
 *     elif isinstance(neutral_mass, bytearray):             # <<<<<<<<<<<<<<
 *         return _Index_from_bytes(neutral_mass)
 *     else:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":51
 *         return _Index_from_bytes(neutral_mass)
 *     else:
 *         raise TypeError(type(neutral_mass))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, ((PyObject *)Py_TYPE(__pyx_v_neutral_mass))); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 51, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 51, __pyx_L1_error)
  }

  /* "ms_deisotope/_c/peak_set.pyx":45
 * 
 * 
 * cpdef _Index_reconstructor(object neutral_mass, object mz=None):             # <<<<<<<<<<<<<<
 *     if isinstance(neutral_mass, int):
 *         return _Index._create(neutral_mass, mz)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_reconstructor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_1_Index_reconstructor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set__Index_reconstructor[] = "_Index_reconstructor(neutral_mass, mz=None)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_1_Index_reconstructor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_neutral_mass = 0;
  PyObject *__pyx_v_mz = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_Index_reconstructor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_mz,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mz);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_Index_reconstructor") < 0)) __PYX_ERR(0, 45, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_neutral_mass = values[0];
    __pyx_v_mz = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_Index_reconstructor", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 45, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_reconstructor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set__Index_reconstructor(__pyx_self, __pyx_v_neutral_mass, __pyx_v_mz);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set__Index_reconstructor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_mz) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set__Index_reconstructor __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_Index_reconstructor", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.mz = __pyx_v_mz;
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set__Index_reconstructor(__pyx_v_neutral_mass, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_reconstructor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":54
 * 
 * 
 * cpdef _Index_to_bytes(_Index index):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t j, m
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_3_Index_to_bytes(PyObject *__pyx_self, PyObject *__pyx_v_index); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set__Index_to_bytes(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_index, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_j;
  size_t __pyx_v_m;
  char *__pyx_v_buff;
  __pyx_t_5numpy_uint64_t __pyx_v_n;
  PyObject *__pyx_v_out = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_Index_to_bytes", 0);

  /* "ms_deisotope/_c/peak_set.pyx":61
 *         bytearray out
 * 
 *     m = sizeof(np.uint64_t) * 2             # <<<<<<<<<<<<<<
 *     buff = <char*>malloc(sizeof(char) * m)
 *     j = 0
 */
  __pyx_v_m = ((sizeof(__pyx_t_5numpy_uint64_t)) * 2);

  /* "ms_deisotope/_c/peak_set.pyx":62
 * 
 *     m = sizeof(np.uint64_t) * 2
 *     buff = <char*>malloc(sizeof(char) * m)             # <<<<<<<<<<<<<<
 *     j = 0
 *     n = index.neutral_mass
 */
  __pyx_v_buff = ((char *)malloc(((sizeof(char)) * __pyx_v_m)));

  /* "ms_deisotope/_c/peak_set.pyx":63
 *     m = sizeof(np.uint64_t) * 2
 *     buff = <char*>malloc(sizeof(char) * m)
 *     j = 0             # <<<<<<<<<<<<<<
 *     n = index.neutral_mass
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 */
  __pyx_v_j = 0;

  /* "ms_deisotope/_c/peak_set.pyx":64
 *     buff = <char*>malloc(sizeof(char) * m)
 *     j = 0
 *     n = index.neutral_mass             # <<<<<<<<<<<<<<
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 */
  __pyx_t_1 = __pyx_v_index->neutral_mass;
  __pyx_v_n = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":65
 *     j = 0
 *     n = index.neutral_mass
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))             # <<<<<<<<<<<<<<
 *     j += sizeof(np.uint64_t)
 *     n = index.mz
 */
  (void)(memcpy((&(__pyx_v_buff[__pyx_v_j])), (&__pyx_v_n), (sizeof(__pyx_t_5numpy_uint64_t))));

  /* "ms_deisotope/_c/peak_set.pyx":66
 *     n = index.neutral_mass
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)             # <<<<<<<<<<<<<<
 *     n = index.mz
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 */
  __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_uint64_t)));

  /* "ms_deisotope/_c/peak_set.pyx":67
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 *     n = index.mz             # <<<<<<<<<<<<<<
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 */
  __pyx_t_1 = __pyx_v_index->mz;
  __pyx_v_n = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":68
 *     j += sizeof(np.uint64_t)
 *     n = index.mz
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))             # <<<<<<<<<<<<<<
 *     j += sizeof(np.uint64_t)
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 */
  (void)(memcpy((&(__pyx_v_buff[__pyx_v_j])), (&__pyx_v_n), (sizeof(__pyx_t_5numpy_uint64_t))));

  /* "ms_deisotope/_c/peak_set.pyx":69
 *     n = index.mz
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)             # <<<<<<<<<<<<<<
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 *     free(buff)
 */
  __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_uint64_t)));

  /* "ms_deisotope/_c/peak_set.pyx":70
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)             # <<<<<<<<<<<<<<
 *     free(buff)
 *     return out
 */
  __pyx_t_2 = PyByteArray_FromStringAndSize(((char *)__pyx_v_buff), __pyx_v_m); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_out = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":71
 *     j += sizeof(np.uint64_t)
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 *     free(buff)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  free(__pyx_v_buff);

  /* "ms_deisotope/_c/peak_set.pyx":72
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 *     free(buff)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_out);
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":54
 * 
 * 
 * cpdef _Index_to_bytes(_Index index):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t j, m
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_out);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_3_Index_to_bytes(PyObject *__pyx_self, PyObject *__pyx_v_index); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_2_Index_to_bytes[] = "_Index_to_bytes(_Index index)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_3_Index_to_bytes(PyObject *__pyx_self, PyObject *__pyx_v_index) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_Index_to_bytes (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_index), __pyx_ptype_12ms_deisotope_2_c_8peak_set__Index, 1, "index", 0))) __PYX_ERR(0, 54, __pyx_L1_error)
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_2_Index_to_bytes(__pyx_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_index));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_2_Index_to_bytes(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_index) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_Index_to_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set__Index_to_bytes(__pyx_v_index, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":75
 * 
 * 
 * cpdef _Index_from_bytes(bytearray view):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t j
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_5_Index_from_bytes(PyObject *__pyx_self, PyObject *__pyx_v_view); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set__Index_from_bytes(PyObject *__pyx_v_view, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_j;
  __pyx_t_5numpy_uint64_t __pyx_v_val;
  char *__pyx_v_buff;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_inst = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char *__pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_Index_from_bytes", 0);

  /* "ms_deisotope/_c/peak_set.pyx":81
 *         char* buff
 *         _Index inst
 *     inst = _Index._create(0, 0)             # <<<<<<<<<<<<<<
 *     j = 0
 *     buff = view
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(0, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_inst = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":82
 *         _Index inst
 *     inst = _Index._create(0, 0)
 *     j = 0             # <<<<<<<<<<<<<<
 *     buff = view
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 */
  __pyx_v_j = 0;

  /* "ms_deisotope/_c/peak_set.pyx":83
 *     inst = _Index._create(0, 0)
 *     j = 0
 *     buff = view             # <<<<<<<<<<<<<<
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 */
  __pyx_t_2 = __Pyx_PyObject_AsWritableString(__pyx_v_view); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 83, __pyx_L1_error)
  __pyx_v_buff = __pyx_t_2;

  /* "ms_deisotope/_c/peak_set.pyx":84
 *     j = 0
 *     buff = view
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))             # <<<<<<<<<<<<<<
 *     j += sizeof(np.uint64_t)
 *     inst.neutral_mass = val
 */
  (void)(memcpy((&__pyx_v_val), (&(__pyx_v_buff[__pyx_v_j])), (sizeof(__pyx_t_5numpy_uint64_t))));

  /* "ms_deisotope/_c/peak_set.pyx":85
 *     buff = view
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)             # <<<<<<<<<<<<<<
 *     inst.neutral_mass = val
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 */
  __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_uint64_t)));

  /* "ms_deisotope/_c/peak_set.pyx":86
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 *     inst.neutral_mass = val             # <<<<<<<<<<<<<<
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 */
  __pyx_v_inst->neutral_mass = __pyx_v_val;

  /* "ms_deisotope/_c/peak_set.pyx":87
 *     j += sizeof(np.uint64_t)
 *     inst.neutral_mass = val
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))             # <<<<<<<<<<<<<<
 *     j += sizeof(np.uint64_t)
 *     inst.mz = val
 */
  (void)(memcpy((&__pyx_v_val), (&(__pyx_v_buff[__pyx_v_j])), (sizeof(__pyx_t_5numpy_uint64_t))));

  /* "ms_deisotope/_c/peak_set.pyx":88
 *     inst.neutral_mass = val
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)             # <<<<<<<<<<<<<<
 *     inst.mz = val
 *     return inst
 */
  __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_uint64_t)));

  /* "ms_deisotope/_c/peak_set.pyx":89
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 *     inst.mz = val             # <<<<<<<<<<<<<<
 *     return inst
 * 
 */
  __pyx_v_inst->mz = __pyx_v_val;

  /* "ms_deisotope/_c/peak_set.pyx":90
 *     j += sizeof(np.uint64_t)
 *     inst.mz = val
 *     return inst             # <<<<<<<<<<<<<<
 * 
 * @cython.freelist(100000)
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));
  __pyx_r = ((PyObject *)__pyx_v_inst);
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":75
 * 
 * 
 * cpdef _Index_from_bytes(bytearray view):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t j
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_from_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_5_Index_from_bytes(PyObject *__pyx_self, PyObject *__pyx_v_view); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_4_Index_from_bytes[] = "_Index_from_bytes(bytearray view)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_5_Index_from_bytes(PyObject *__pyx_self, PyObject *__pyx_v_view) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_Index_from_bytes (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_view), (&PyByteArray_Type), 1, "view", 1))) __PYX_ERR(0, 75, __pyx_L1_error)
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_4_Index_from_bytes(__pyx_self, ((PyObject*)__pyx_v_view));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_4_Index_from_bytes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_view) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_Index_from_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set__Index_from_bytes(__pyx_v_view, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_from_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":105
 *         Index of this object (or its parent), ordered by neutral mass
 *     """
 *     def __init__(self, neutral_mass=0, mz=0):             # <<<<<<<<<<<<<<
 *         self.neutral_mass = neutral_mass
 *         self.mz = mz
 */

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_neutral_mass = 0;
  PyObject *__pyx_v_mz = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_mz,0};
    PyObject* values[2] = {0,0};
    values[0] = ((PyObject *)__pyx_int_0);
    values[1] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mz);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 105, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_neutral_mass = values[0];
    __pyx_v_mz = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 105, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_mz);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_mz) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":106
 *     """
 *     def __init__(self, neutral_mass=0, mz=0):
 *         self.neutral_mass = neutral_mass             # <<<<<<<<<<<<<<
 *         self.mz = mz
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_neutral_mass); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 106, __pyx_L1_error)
  __pyx_v_self->neutral_mass = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":107
 *     def __init__(self, neutral_mass=0, mz=0):
 *         self.neutral_mass = neutral_mass
 *         self.mz = mz             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_mz); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 107, __pyx_L1_error)
  __pyx_v_self->mz = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":105
 *         Index of this object (or its parent), ordered by neutral mass
 *     """
 *     def __init__(self, neutral_mass=0, mz=0):             # <<<<<<<<<<<<<<
 *         self.neutral_mass = neutral_mass
 *         self.mz = mz
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":109
 *         self.mz = mz
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "%d|%d" % (self.neutral_mass, self.mz)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_3__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2__repr__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":110
 * 
 *     def __repr__(self):
 *         return "%d|%d" % (self.neutral_mass, self.mz)             # <<<<<<<<<<<<<<
 * 
 *     def clone(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->mz); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_d_d, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":109
 *         self.mz = mz
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "%d|%d" % (self.neutral_mass, self.mz)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":112
 *         return "%d|%d" % (self.neutral_mass, self.mz)
 * 
 *     def clone(self):             # <<<<<<<<<<<<<<
 *         return _Index._create(self.neutral_mass, self.mz)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_5clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_6_Index_4clone[] = "_Index.clone(self)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_5clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_4clone(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_4clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);

  /* "ms_deisotope/_c/peak_set.pyx":113
 * 
 *     def clone(self):
 *         return _Index._create(self.neutral_mass, self.mz)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(__pyx_v_self->neutral_mass, __pyx_v_self->mz)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":112
 *         return "%d|%d" % (self.neutral_mass, self.mz)
 * 
 *     def clone(self):             # <<<<<<<<<<<<<<
 *         return _Index._create(self.neutral_mass, self.mz)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":115
 *         return _Index._create(self.neutral_mass, self.mz)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return _Index_reconstructor, (self.neutral_mass, self.mz)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_7__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_6_Index_6__reduce__[] = "_Index.__reduce__(self)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_7__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_6__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_6__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":116
 * 
 *     def __reduce__(self):
 *         return _Index_reconstructor, (self.neutral_mass, self.mz)             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Index_reconstructor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->neutral_mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->mz); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":115
 *         return _Index._create(self.neutral_mass, self.mz)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return _Index_reconstructor, (self.neutral_mass, self.mz)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":119
 * 
 *     @staticmethod
 *     cdef _Index _create(size_t neutral_mass, size_t mz):             # <<<<<<<<<<<<<<
 *         cdef:
 *             _Index inst
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(size_t __pyx_v_neutral_mass, size_t __pyx_v_mz) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_inst = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create", 0);

  /* "ms_deisotope/_c/peak_set.pyx":122
 *         cdef:
 *             _Index inst
 *         inst = _Index.__new__(_Index)             # <<<<<<<<<<<<<<
 *         inst.neutral_mass = neutral_mass
 *         inst.mz = mz
 */
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set__Index(((PyTypeObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set__Index), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));
  __pyx_v_inst = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":123
 *             _Index inst
 *         inst = _Index.__new__(_Index)
 *         inst.neutral_mass = neutral_mass             # <<<<<<<<<<<<<<
 *         inst.mz = mz
 *         return inst
 */
  __pyx_v_inst->neutral_mass = __pyx_v_neutral_mass;

  /* "ms_deisotope/_c/peak_set.pyx":124
 *         inst = _Index.__new__(_Index)
 *         inst.neutral_mass = neutral_mass
 *         inst.mz = mz             # <<<<<<<<<<<<<<
 *         return inst
 * 
 */
  __pyx_v_inst->mz = __pyx_v_mz;

  /* "ms_deisotope/_c/peak_set.pyx":125
 *         inst.neutral_mass = neutral_mass
 *         inst.mz = mz
 *         return inst             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));
  __pyx_r = __pyx_v_inst;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":119
 * 
 *     @staticmethod
 *     cdef _Index _create(size_t neutral_mass, size_t mz):             # <<<<<<<<<<<<<<
 *         cdef:
 *             _Index inst
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index._create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":7
 * cdef class _Index:
 *     cdef:
 *         public size_t neutral_mass             # <<<<<<<<<<<<<<
 *         public size_t mz
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.neutral_mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 7, __pyx_L1_error)
  __pyx_v_self->neutral_mass = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.neutral_mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":8
 *     cdef:
 *         public size_t neutral_mass
 *         public size_t mz             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_2mz_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_2mz_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2mz___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2mz___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.mz.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_2mz_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_2mz_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2mz_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2mz_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 8, __pyx_L1_error)
  __pyx_v_self->mz = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.mz.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":131
 * cdef class EnvelopePair(object):
 * 
 *     def __init__(self, mz, intensity):             # <<<<<<<<<<<<<<
 *         self.mz = mz
 *         self.intensity = intensity
 */

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_mz = 0;
  PyObject *__pyx_v_intensity = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_mz,&__pyx_n_s_intensity,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mz)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_intensity)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 131, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 131, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_mz = values[0];
    __pyx_v_intensity = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 131, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self), __pyx_v_mz, __pyx_v_intensity);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_mz, PyObject *__pyx_v_intensity) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":132
 * 
 *     def __init__(self, mz, intensity):
 *         self.mz = mz             # <<<<<<<<<<<<<<
 *         self.intensity = intensity
 * 
 */
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_mz); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 132, __pyx_L1_error)
  __pyx_v_self->mz = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":133
 *     def __init__(self, mz, intensity):
 *         self.mz = mz
 *         self.intensity = intensity             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, i):
 */
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_intensity); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 133, __pyx_L1_error)
  __pyx_v_self->intensity = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":131
 * cdef class EnvelopePair(object):
 * 
 *     def __init__(self, mz, intensity):             # <<<<<<<<<<<<<<
 *         self.mz = mz
 *         self.intensity = intensity
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":135
 *         self.intensity = intensity
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *             return self.mz
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2__getitem__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self), ((PyObject *)__pyx_v_i));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2__getitem__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_i) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":136
 * 
 *     def __getitem__(self, i):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             return self.mz
 *         elif i == 1:
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":137
 *     def __getitem__(self, i):
 *         if i == 0:
 *             return self.mz             # <<<<<<<<<<<<<<
 *         elif i == 1:
 *             return self.intensity
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":136
 * 
 *     def __getitem__(self, i):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             return self.mz
 *         elif i == 1:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":138
 *         if i == 0:
 *             return self.mz
 *         elif i == 1:             # <<<<<<<<<<<<<<
 *             return self.intensity
 *         else:
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_2)) {

    /* "ms_deisotope/_c/peak_set.pyx":139
 *             return self.mz
 *         elif i == 1:
 *             return self.intensity             # <<<<<<<<<<<<<<
 *         else:
 *             raise IndexError(i)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->intensity); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":138
 *         if i == 0:
 *             return self.mz
 *         elif i == 1:             # <<<<<<<<<<<<<<
 *             return self.intensity
 *         else:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":141
 *             return self.intensity
 *         else:
 *             raise IndexError(i)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 141, __pyx_L1_error)
  }

  /* "ms_deisotope/_c/peak_set.pyx":135
 *         self.intensity = intensity
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *             return self.mz
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_12EnvelopePair_6generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "ms_deisotope/_c/peak_set.pyx":143
 *             raise IndexError(i)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.mz
 *         yield self.intensity
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_5__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_5__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_4__iter__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_4__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__(__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 143, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_12ms_deisotope_2_c_8peak_set_12EnvelopePair_6generator, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_iter, __pyx_n_s_EnvelopePair___iter, __pyx_n_s_ms_deisotope__c_peak_set); if (unlikely(!gen)) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_12EnvelopePair_6generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *__pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L4_resume_from_yield;
    case 2: goto __pyx_L5_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 143, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":144
 * 
 *     def __iter__(self):
 *         yield self.mz             # <<<<<<<<<<<<<<
 *         yield self.intensity
 * 
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  /* return from generator, yielding value */
  __pyx_generator->resume_label = 1;
  return __pyx_r;
  __pyx_L4_resume_from_yield:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 144, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":145
 *     def __iter__(self):
 *         yield self.mz
 *         yield self.intensity             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->intensity); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  /* return from generator, yielding value */
  __pyx_generator->resume_label = 2;
  return __pyx_r;
  __pyx_L5_resume_from_yield:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 145, __pyx_L1_error)
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "ms_deisotope/_c/peak_set.pyx":143
 *             raise IndexError(i)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.mz
 *         yield self.intensity
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":147
 *         yield self.intensity
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return EnvelopePair, (self.mz, self.intensity,)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_8__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_12EnvelopePair_7__reduce__[] = "EnvelopePair.__reduce__(self)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_8__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_7__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_7__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":148
 * 
 *     def __reduce__(self):
 *         return EnvelopePair, (self.mz, self.intensity,)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _eq(self, EnvelopePair other):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair));
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":147
 *         yield self.intensity
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return EnvelopePair, (self.mz, self.intensity,)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":150
 *         return EnvelopePair, (self.mz, self.intensity,)
 * 
 *     cpdef bint _eq(self, EnvelopePair other):             # <<<<<<<<<<<<<<
 *         return (abs(self.mz - other.mz) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon)
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_10_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_other, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_eq", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_eq); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_10_eq)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_other)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 150, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 150, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":151
 * 
 *     cpdef bint _eq(self, EnvelopePair other):
 *         return (abs(self.mz - other.mz) < epsilon) and (             # <<<<<<<<<<<<<<
 *             abs(self.intensity - other.intensity) < epsilon)
 * 
 */
  __pyx_t_6 = ((fabs((__pyx_v_self->mz - __pyx_v_other->mz)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }

  /* "ms_deisotope/_c/peak_set.pyx":152
 *     cpdef bint _eq(self, EnvelopePair other):
 *         return (abs(self.mz - other.mz) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon)             # <<<<<<<<<<<<<<
 * 
 *     def __richcmp__(self, other, int code):
 */
  __pyx_t_6 = ((fabs((__pyx_v_self->intensity - __pyx_v_other->intensity)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":150
 *         return EnvelopePair, (self.mz, self.intensity,)
 * 
 *     cpdef bint _eq(self, EnvelopePair other):             # <<<<<<<<<<<<<<
 *         return (abs(self.mz - other.mz) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.EnvelopePair._eq", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_10_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9_eq[] = "EnvelopePair._eq(self, EnvelopePair other) -> bool";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_10_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_eq (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair, 1, "other", 0))) __PYX_ERR(0, 150, __pyx_L1_error)
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9_eq(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self), ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_other));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_eq", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__eq(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair._eq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":154
 *             abs(self.intensity - other.intensity) < epsilon)
 * 
 *     def __richcmp__(self, other, int code):             # <<<<<<<<<<<<<<
 *         if code == 2:
 *             return self._eq(other)
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_12__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_12__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_11__richcmp__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self), ((PyObject *)__pyx_v_other), ((int)__pyx_v_code));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_11__richcmp__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__richcmp__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":155
 * 
 *     def __richcmp__(self, other, int code):
 *         if code == 2:             # <<<<<<<<<<<<<<
 *             return self._eq(other)
 *         elif code == 3:
 */
  switch (__pyx_v_code) {
    case 2:

    /* "ms_deisotope/_c/peak_set.pyx":156
 *     def __richcmp__(self, other, int code):
 *         if code == 2:
 *             return self._eq(other)             # <<<<<<<<<<<<<<
 *         elif code == 3:
 *             return not (self._eq(other))
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair))))) __PYX_ERR(0, 156, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyBool_FromLong(((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self->__pyx_vtab)->_eq(__pyx_v_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_other), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 156, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":155
 * 
 *     def __richcmp__(self, other, int code):
 *         if code == 2:             # <<<<<<<<<<<<<<
 *             return self._eq(other)
 *         elif code == 3:
 */
    break;
    case 3:

    /* "ms_deisotope/_c/peak_set.pyx":158
 *             return self._eq(other)
 *         elif code == 3:
 *             return not (self._eq(other))             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair))))) __PYX_ERR(0, 158, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyBool_FromLong((!(((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self->__pyx_vtab)->_eq(__pyx_v_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_other), 0) != 0))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":157
 *         if code == 2:
 *             return self._eq(other)
 *         elif code == 3:             # <<<<<<<<<<<<<<
 *             return not (self._eq(other))
 * 
 */
    break;
    default: break;
  }

  /* "ms_deisotope/_c/peak_set.pyx":154
 *             abs(self.intensity - other.intensity) < epsilon)
 * 
 *     def __richcmp__(self, other, int code):             # <<<<<<<<<<<<<<
 *         if code == 2:
 *             return self._eq(other)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":162
 * 
 *     @staticmethod
 *     cdef EnvelopePair _create(double mz, double intensity):             # <<<<<<<<<<<<<<
 *         cdef:
 *             EnvelopePair inst
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create(double __pyx_v_mz, double __pyx_v_intensity) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_inst = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create", 0);

  /* "ms_deisotope/_c/peak_set.pyx":165
 *         cdef:
 *             EnvelopePair inst
 *         inst = EnvelopePair.__new__(EnvelopePair)             # <<<<<<<<<<<<<<
 *         inst.mz = mz
 *         inst.intensity = intensity
 */
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set_EnvelopePair(((PyTypeObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 165, __pyx_L1_error)
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));
  __pyx_v_inst = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":166
 *             EnvelopePair inst
 *         inst = EnvelopePair.__new__(EnvelopePair)
 *         inst.mz = mz             # <<<<<<<<<<<<<<
 *         inst.intensity = intensity
 *         return inst
 */
  __pyx_v_inst->mz = __pyx_v_mz;

  /* "ms_deisotope/_c/peak_set.pyx":167
 *         inst = EnvelopePair.__new__(EnvelopePair)
 *         inst.mz = mz
 *         inst.intensity = intensity             # <<<<<<<<<<<<<<
 *         return inst
 * 
 */
  __pyx_v_inst->intensity = __pyx_v_intensity;

  /* "ms_deisotope/_c/peak_set.pyx":168
 *         inst.mz = mz
 *         inst.intensity = intensity
 *         return inst             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));
  __pyx_r = __pyx_v_inst;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":162
 * 
 *     @staticmethod
 *     cdef EnvelopePair _create(double mz, double intensity):             # <<<<<<<<<<<<<<
 *         cdef:
 *             EnvelopePair inst
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair._create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":170
 *         return inst
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "(%0.4f, %0.2f)" % (self.mz, self.intensity)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_14__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_14__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_13__repr__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_13__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":171
 * 
 *     def __repr__(self):
 *         return "(%0.4f, %0.2f)" % (self.mz, self.intensity)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_0_4f_0_2f, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":170
 *         return inst
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "(%0.4f, %0.2f)" % (self.mz, self.intensity)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":16
 * cdef class EnvelopePair:
 *     cdef:
 *         public double mz             # <<<<<<<<<<<<<<
 *         public double intensity
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.mz.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 16, __pyx_L1_error)
  __pyx_v_self->mz = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.mz.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":17
 *     cdef:
 *         public double mz
 *         public double intensity             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _eq(self, EnvelopePair other)
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->intensity); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.intensity.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_v_self->intensity = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.intensity.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":175
 * 
 * @cython.boundscheck(False)
 * cpdef bytearray Envelope_to_bytes(Envelope self):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, m
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_7Envelope_to_bytes(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_to_bytes(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_i;
  size_t __pyx_v_j;
  size_t __pyx_v_m;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_pair = 0;
  char *__pyx_v_buff;
  __pyx_t_5numpy_uint32_t __pyx_v_n;
  __pyx_t_5numpy_float64_t __pyx_v_val;
  PyObject *__pyx_v_out = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_t_5numpy_uint32_t __pyx_t_1;
  __pyx_t_5numpy_uint32_t __pyx_t_2;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("Envelope_to_bytes", 0);

  /* "ms_deisotope/_c/peak_set.pyx":184
 *         bytearray out
 * 
 *     n = self.get_size()             # <<<<<<<<<<<<<<
 *     m = sizeof(np.uint32_t) + sizeof(np.float64_t) * n * 2
 *     buff = <char*>malloc(sizeof(char) * m)
 */
  __pyx_v_n = __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(__pyx_v_self);

  /* "ms_deisotope/_c/peak_set.pyx":185
 * 
 *     n = self.get_size()
 *     m = sizeof(np.uint32_t) + sizeof(np.float64_t) * n * 2             # <<<<<<<<<<<<<<
 *     buff = <char*>malloc(sizeof(char) * m)
 *     j = 0
 */
  __pyx_v_m = ((sizeof(__pyx_t_5numpy_uint32_t)) + (((sizeof(__pyx_t_5numpy_float64_t)) * __pyx_v_n) * 2));

  /* "ms_deisotope/_c/peak_set.pyx":186
 *     n = self.get_size()
 *     m = sizeof(np.uint32_t) + sizeof(np.float64_t) * n * 2
 *     buff = <char*>malloc(sizeof(char) * m)             # <<<<<<<<<<<<<<
 *     j = 0
 *     memcpy(&buff[j], &n, sizeof(np.uint32_t))
 */
  __pyx_v_buff = ((char *)malloc(((sizeof(char)) * __pyx_v_m)));

  /* "ms_deisotope/_c/peak_set.pyx":187
 *     m = sizeof(np.uint32_t) + sizeof(np.float64_t) * n * 2
 *     buff = <char*>malloc(sizeof(char) * m)
 *     j = 0             # <<<<<<<<<<<<<<
 *     memcpy(&buff[j], &n, sizeof(np.uint32_t))
 *     j += sizeof(np.uint32_t)
 */
  __pyx_v_j = 0;

  /* "ms_deisotope/_c/peak_set.pyx":188
 *     buff = <char*>malloc(sizeof(char) * m)
 *     j = 0
 *     memcpy(&buff[j], &n, sizeof(np.uint32_t))             # <<<<<<<<<<<<<<
 *     j += sizeof(np.uint32_t)
 *     for i in range(n):
 */
  (void)(memcpy((&(__pyx_v_buff[__pyx_v_j])), (&__pyx_v_n), (sizeof(__pyx_t_5numpy_uint32_t))));

  /* "ms_deisotope/_c/peak_set.pyx":189
 *     j = 0
 *     memcpy(&buff[j], &n, sizeof(np.uint32_t))
 *     j += sizeof(np.uint32_t)             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         pair = self.getitem(i)
 */
  __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_uint32_t)));

  /* "ms_deisotope/_c/peak_set.pyx":190
 *     memcpy(&buff[j], &n, sizeof(np.uint32_t))
 *     j += sizeof(np.uint32_t)
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         pair = self.getitem(i)
 *         val = pair.mz
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "ms_deisotope/_c/peak_set.pyx":191
 *     j += sizeof(np.uint32_t)
 *     for i in range(n):
 *         pair = self.getitem(i)             # <<<<<<<<<<<<<<
 *         val = pair.mz
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 */
    __pyx_t_4 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(__pyx_v_self, __pyx_v_i)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_XDECREF_SET(__pyx_v_pair, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":192
 *     for i in range(n):
 *         pair = self.getitem(i)
 *         val = pair.mz             # <<<<<<<<<<<<<<
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 */
    __pyx_t_5 = __pyx_v_pair->mz;
    __pyx_v_val = __pyx_t_5;

    /* "ms_deisotope/_c/peak_set.pyx":193
 *         pair = self.getitem(i)
 *         val = pair.mz
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))             # <<<<<<<<<<<<<<
 *         j += sizeof(np.float64_t)
 *         val = pair.intensity
 */
    (void)(memcpy((&(__pyx_v_buff[__pyx_v_j])), (&__pyx_v_val), (sizeof(__pyx_t_5numpy_float64_t))));

    /* "ms_deisotope/_c/peak_set.pyx":194
 *         val = pair.mz
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)             # <<<<<<<<<<<<<<
 *         val = pair.intensity
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 */
    __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_float64_t)));

    /* "ms_deisotope/_c/peak_set.pyx":195
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 *         val = pair.intensity             # <<<<<<<<<<<<<<
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 */
    __pyx_t_5 = __pyx_v_pair->intensity;
    __pyx_v_val = __pyx_t_5;

    /* "ms_deisotope/_c/peak_set.pyx":196
 *         j += sizeof(np.float64_t)
 *         val = pair.intensity
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))             # <<<<<<<<<<<<<<
 *         j += sizeof(np.float64_t)
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 */
    (void)(memcpy((&(__pyx_v_buff[__pyx_v_j])), (&__pyx_v_val), (sizeof(__pyx_t_5numpy_float64_t))));

    /* "ms_deisotope/_c/peak_set.pyx":197
 *         val = pair.intensity
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)             # <<<<<<<<<<<<<<
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 *     free(buff)
 */
    __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_float64_t)));
  }

  /* "ms_deisotope/_c/peak_set.pyx":198
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)             # <<<<<<<<<<<<<<
 *     free(buff)
 *     return out
 */
  __pyx_t_4 = PyByteArray_FromStringAndSize(((char *)__pyx_v_buff), __pyx_v_m); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_out = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":199
 *         j += sizeof(np.float64_t)
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 *     free(buff)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  free(__pyx_v_buff);

  /* "ms_deisotope/_c/peak_set.pyx":200
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 *     free(buff)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_out);
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":175
 * 
 * @cython.boundscheck(False)
 * cpdef bytearray Envelope_to_bytes(Envelope self):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, m
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope_to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_pair);
  __Pyx_XDECREF(__pyx_v_out);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_7Envelope_to_bytes(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_6Envelope_to_bytes[] = "Envelope_to_bytes(Envelope self) -> bytearray";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_7Envelope_to_bytes(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("Envelope_to_bytes (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_self), __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope, 1, "self", 0))) __PYX_ERR(0, 175, __pyx_L1_error)
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_6Envelope_to_bytes(__pyx_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6Envelope_to_bytes(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("Envelope_to_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_to_bytes(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope_to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":203
 * 
 * 
 * cpdef Envelope_from_bytes(bytearray view):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, m
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_9Envelope_from_bytes(PyObject *__pyx_self, PyObject *__pyx_v_view); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_from_bytes(PyObject *__pyx_v_view, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_i;
  size_t __pyx_v_j;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_pair = 0;
  __pyx_t_5numpy_uint32_t __pyx_v_n;
  char *__pyx_v_buff;
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __pyx_t_5numpy_uint32_t __pyx_t_3;
  __pyx_t_5numpy_uint32_t __pyx_t_4;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("Envelope_from_bytes", 0);

  /* "ms_deisotope/_c/peak_set.pyx":211
 *         char* buff
 *         tuple result
 *     j = 0             # <<<<<<<<<<<<<<
 *     buff = view
 *     memcpy(&n, &buff[j], sizeof(np.uint32_t))
 */
  __pyx_v_j = 0;

  /* "ms_deisotope/_c/peak_set.pyx":212
 *         tuple result
 *     j = 0
 *     buff = view             # <<<<<<<<<<<<<<
 *     memcpy(&n, &buff[j], sizeof(np.uint32_t))
 *     j += sizeof(np.uint32_t)
 */
  __pyx_t_1 = __Pyx_PyObject_AsWritableString(__pyx_v_view); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) __PYX_ERR(0, 212, __pyx_L1_error)
  __pyx_v_buff = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":213
 *     j = 0
 *     buff = view
 *     memcpy(&n, &buff[j], sizeof(np.uint32_t))             # <<<<<<<<<<<<<<
 *     j += sizeof(np.uint32_t)
 *     result = PyTuple_New(n)
 */
  (void)(memcpy((&__pyx_v_n), (&(__pyx_v_buff[__pyx_v_j])), (sizeof(__pyx_t_5numpy_uint32_t))));

  /* "ms_deisotope/_c/peak_set.pyx":214
 *     buff = view
 *     memcpy(&n, &buff[j], sizeof(np.uint32_t))
 *     j += sizeof(np.uint32_t)             # <<<<<<<<<<<<<<
 *     result = PyTuple_New(n)
 *     for i in range(n):
 */
  __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_uint32_t)));

  /* "ms_deisotope/_c/peak_set.pyx":215
 *     memcpy(&n, &buff[j], sizeof(np.uint32_t))
 *     j += sizeof(np.uint32_t)
 *     result = PyTuple_New(n)             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         pair = EnvelopePair._create(0, 0)
 */
  __pyx_t_2 = PyTuple_New(__pyx_v_n); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_result = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":216
 *     j += sizeof(np.uint32_t)
 *     result = PyTuple_New(n)
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         pair = EnvelopePair._create(0, 0)
 *         memcpy(&pair.mz, &buff[j], sizeof(np.float64_t))
 */
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_i = __pyx_t_5;

    /* "ms_deisotope/_c/peak_set.pyx":217
 *     result = PyTuple_New(n)
 *     for i in range(n):
 *         pair = EnvelopePair._create(0, 0)             # <<<<<<<<<<<<<<
 *         memcpy(&pair.mz, &buff[j], sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 */
    __pyx_t_2 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create(0.0, 0.0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_pair, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":218
 *     for i in range(n):
 *         pair = EnvelopePair._create(0, 0)
 *         memcpy(&pair.mz, &buff[j], sizeof(np.float64_t))             # <<<<<<<<<<<<<<
 *         j += sizeof(np.float64_t)
 *         memcpy(&pair.intensity, &buff[j], sizeof(np.float64_t))
 */
    (void)(memcpy((&__pyx_v_pair->mz), (&(__pyx_v_buff[__pyx_v_j])), (sizeof(__pyx_t_5numpy_float64_t))));

    /* "ms_deisotope/_c/peak_set.pyx":219
 *         pair = EnvelopePair._create(0, 0)
 *         memcpy(&pair.mz, &buff[j], sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)             # <<<<<<<<<<<<<<
 *         memcpy(&pair.intensity, &buff[j], sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 */
    __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_float64_t)));

    /* "ms_deisotope/_c/peak_set.pyx":220
 *         memcpy(&pair.mz, &buff[j], sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 *         memcpy(&pair.intensity, &buff[j], sizeof(np.float64_t))             # <<<<<<<<<<<<<<
 *         j += sizeof(np.float64_t)
 *         Py_INCREF(pair)
 */
    (void)(memcpy((&__pyx_v_pair->intensity), (&(__pyx_v_buff[__pyx_v_j])), (sizeof(__pyx_t_5numpy_float64_t))));

    /* "ms_deisotope/_c/peak_set.pyx":221
 *         j += sizeof(np.float64_t)
 *         memcpy(&pair.intensity, &buff[j], sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)             # <<<<<<<<<<<<<<
 *         Py_INCREF(pair)
 *         PyTuple_SetItem(result, i, pair)
 */
    __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_float64_t)));

    /* "ms_deisotope/_c/peak_set.pyx":222
 *         memcpy(&pair.intensity, &buff[j], sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 *         Py_INCREF(pair)             # <<<<<<<<<<<<<<
 *         PyTuple_SetItem(result, i, pair)
 *     return Envelope._create(result)
 */
    Py_INCREF(((PyObject *)__pyx_v_pair));

    /* "ms_deisotope/_c/peak_set.pyx":223
 *         j += sizeof(np.float64_t)
 *         Py_INCREF(pair)
 *         PyTuple_SetItem(result, i, pair)             # <<<<<<<<<<<<<<
 *     return Envelope._create(result)
 * 
 */
    __pyx_t_6 = PyTuple_SetItem(__pyx_v_result, __pyx_v_i, ((PyObject *)__pyx_v_pair)); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 223, __pyx_L1_error)
  }

  /* "ms_deisotope/_c/peak_set.pyx":224
 *         Py_INCREF(pair)
 *         PyTuple_SetItem(result, i, pair)
 *     return Envelope._create(result)             # <<<<<<<<<<<<<<
 * 
 * @cython.boundscheck(False)
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(__pyx_v_result)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":203
 * 
 * 
 * cpdef Envelope_from_bytes(bytearray view):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, m
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope_from_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_pair);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_9Envelope_from_bytes(PyObject *__pyx_self, PyObject *__pyx_v_view); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_from_bytes[] = "Envelope_from_bytes(bytearray view)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_9Envelope_from_bytes(PyObject *__pyx_self, PyObject *__pyx_v_view) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("Envelope_from_bytes (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_view), (&PyByteArray_Type), 1, "view", 1))) __PYX_ERR(0, 203, __pyx_L1_error)
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_from_bytes(__pyx_self, ((PyObject*)__pyx_v_view));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_from_bytes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_view) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("Envelope_from_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_from_bytes(__pyx_v_view, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope_from_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":227
 * 
 * @cython.boundscheck(False)
 * cpdef Envelope_reconstructor(object value):             # <<<<<<<<<<<<<<
 *     if isinstance(value, tuple):
 *         return Envelope._create(value)
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_11Envelope_reconstructor(PyObject *__pyx_self, PyObject *__pyx_v_value); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_reconstructor(PyObject *__pyx_v_value, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("Envelope_reconstructor", 0);

  /* "ms_deisotope/_c/peak_set.pyx":228
 * @cython.boundscheck(False)
 * cpdef Envelope_reconstructor(object value):
 *     if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *         return Envelope._create(value)
 *     elif isinstance(value, bytearray):
 */
  __pyx_t_1 = PyTuple_Check(__pyx_v_value); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":229
 * cpdef Envelope_reconstructor(object value):
 *     if isinstance(value, tuple):
 *         return Envelope._create(value)             # <<<<<<<<<<<<<<
 *     elif isinstance(value, bytearray):
 *         return Envelope_from_bytes(value)
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(PyTuple_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(0, 229, __pyx_L1_error)
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(((PyObject*)__pyx_v_value))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":228
 * @cython.boundscheck(False)
 * cpdef Envelope_reconstructor(object value):
 *     if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *         return Envelope._create(value)
 *     elif isinstance(value, bytearray):
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":230
 *     if isinstance(value, tuple):
 *         return Envelope._create(value)
 *     elif isinstance(value, bytearray):             # <<<<<<<<<<<<<<
 *         return Envelope_from_bytes(value)
 *     else:
 */
  __pyx_t_2 = PyByteArray_Check(__pyx_v_value); 
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (likely(__pyx_t_1)) {

    /* "ms_deisotope/_c/peak_set.pyx":231
 *         return Envelope._create(value)
 *     elif isinstance(value, bytearray):
 *         return Envelope_from_bytes(value)             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError(type(value))
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(PyByteArray_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytearray", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(0, 231, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_from_bytes(((PyObject*)__pyx_v_value), 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":230
 *     if isinstance(value, tuple):
 *         return Envelope._create(value)
 *     elif isinstance(value, bytearray):             # <<<<<<<<<<<<<<
 *         return Envelope_from_bytes(value)
 *     else:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":233
 *         return Envelope_from_bytes(value)
 *     else:
 *         raise TypeError(type(value))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, ((PyObject *)Py_TYPE(__pyx_v_value))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 233, __pyx_L1_error)
  }

  /* "ms_deisotope/_c/peak_set.pyx":227
 * 
 * @cython.boundscheck(False)
 * cpdef Envelope_reconstructor(object value):             # <<<<<<<<<<<<<<
 *     if isinstance(value, tuple):
 *         return Envelope._create(value)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope_reconstructor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_11Envelope_reconstructor(PyObject *__pyx_self, PyObject *__pyx_v_value); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_10Envelope_reconstructor[] = "Envelope_reconstructor(value)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_11Envelope_reconstructor(PyObject *__pyx_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("Envelope_reconstructor (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_10Envelope_reconstructor(__pyx_self, ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_10Envelope_reconstructor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("Envelope_reconstructor", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_reconstructor(__pyx_v_value, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope_reconstructor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":250
 *         list of (mz, intensity) pairs
 *     """
 *     def __init__(self, pairs):             # <<<<<<<<<<<<<<
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))
 * 
 */

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pairs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pairs,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pairs)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 250, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_pairs = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 250, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self), __pyx_v_pairs);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":251
 *     """
 *     def __init__(self, pairs):
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, i):
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_8__init___lambda(PyObject *__pyx_self, PyObject *__pyx_v_x); /*proto*/
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_8peak_set_8Envelope_8__init___lambda = {"lambda", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_8__init___lambda, METH_O, 0};
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_8__init___lambda(PyObject *__pyx_self, PyObject *__pyx_v_x) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda (wrapper)", 0);
  __pyx_r = __pyx_lambda_funcdef_lambda(__pyx_self, ((PyObject *)__pyx_v_x));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lambda", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PySequence_Tuple(__pyx_v_x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__init__.lambda", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":250
 *         list of (mz, intensity) pairs
 *     """
 *     def __init__(self, pairs):             # <<<<<<<<<<<<<<
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))
 * 
 */

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_pairs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":251
 *     """
 *     def __init__(self, pairs):
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, i):
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_12ms_deisotope_2_c_8peak_set_8Envelope_8__init___lambda, 0, __pyx_n_s_init___locals_lambda, NULL, __pyx_n_s_ms_deisotope__c_peak_set, __pyx_d, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_pairs);
  __Pyx_GIVEREF(__pyx_v_pairs);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_pairs);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_map, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PySequence_Tuple(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->pairs);
  __Pyx_DECREF(__pyx_v_self->pairs);
  __pyx_v_self->pairs = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":250
 *         list of (mz, intensity) pairs
 *     """
 *     def __init__(self, pairs):             # <<<<<<<<<<<<<<
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":253
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         return self.pairs[i]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_2__getitem__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self), ((PyObject *)__pyx_v_i));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_2__getitem__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_i) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":254
 * 
 *     def __getitem__(self, i):
 *         return self.pairs[i]             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->pairs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 254, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_self->pairs, __pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":253
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         return self.pairs[i]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":256
 *         return self.pairs[i]
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.pairs)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_4__iter__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_4__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":257
 * 
 *     def __iter__(self):
 *         return iter(self.pairs)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_self->pairs;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":256
 *         return self.pairs[i]
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.pairs)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":259
 *         return iter(self.pairs)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_7__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_7__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_6__repr__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___2generator2(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "ms_deisotope/_c/peak_set.pyx":260
 * 
 *     def __repr__(self):
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr(__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 260, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *) __pyx_self;
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_outer_scope));
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_outer_scope);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___2generator2, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_repr___locals_genexpr, __pyx_n_s_ms_deisotope__c_peak_set); if (unlikely(!gen)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__repr__.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___2generator2(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *__pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 260, __pyx_L1_error)
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 260, __pyx_L1_error) }
  if (likely(PyList_CheckExact(((PyObject *)__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self))) || PyTuple_CheckExact(((PyObject *)__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self))) {
    __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self); __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(((PyObject *)__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 260, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 260, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 260, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 260, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 260, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 260, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_t);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_t, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PySequence_Tuple(__pyx_cur_scope->__pyx_v_t); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_0_4f_0_2f, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    __Pyx_XGIVEREF(__pyx_t_1);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 260, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":259
 *         return iter(self.pairs)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))
 * 
 */

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_6__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *__pyx_cur_scope;
  PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___2generator2 = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__(__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 259, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

  /* "ms_deisotope/_c/peak_set.pyx":260
 * 
 *     def __repr__(self):
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyString_Join(__pyx_kp_s_, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_s, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":259
 *         return iter(self.pairs)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_gb_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___2generator2);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":262
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return len(self.pairs)
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_9__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_9__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_8__len__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_8__len__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":263
 * 
 *     def __len__(self):
 *         return len(self.pairs)             # <<<<<<<<<<<<<<
 * 
 *     cdef inline size_t get_size(self):
 */
  __pyx_t_1 = __pyx_v_self->pairs;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 263, __pyx_L1_error)
  }
  __pyx_t_2 = PyTuple_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":262
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return len(self.pairs)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__len__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":265
 *         return len(self.pairs)
 * 
 *     cdef inline size_t get_size(self):             # <<<<<<<<<<<<<<
 *         return PyTuple_GET_SIZE(self.pairs)
 * 
 */

static CYTHON_INLINE size_t __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_size", 0);

  /* "ms_deisotope/_c/peak_set.pyx":266
 * 
 *     cdef inline size_t get_size(self):
 *         return PyTuple_GET_SIZE(self.pairs)             # <<<<<<<<<<<<<<
 * 
 *     cdef inline EnvelopePair getitem(self, size_t i):
 */
  __pyx_t_1 = __pyx_v_self->pairs;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_r = PyTuple_GET_SIZE(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":265
 *         return len(self.pairs)
 * 
 *     cdef inline size_t get_size(self):             # <<<<<<<<<<<<<<
 *         return PyTuple_GET_SIZE(self.pairs)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":268
 *         return PyTuple_GET_SIZE(self.pairs)
 * 
 *     cdef inline EnvelopePair getitem(self, size_t i):             # <<<<<<<<<<<<<<
 *         return <EnvelopePair>PyTuple_GetItem(self.pairs, i)
 * 
 */

static CYTHON_INLINE struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, size_t __pyx_v_i) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getitem", 0);

  /* "ms_deisotope/_c/peak_set.pyx":269
 * 
 *     cdef inline EnvelopePair getitem(self, size_t i):
 *         return <EnvelopePair>PyTuple_GetItem(self.pairs, i)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _eq(self, Envelope other):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = __pyx_v_self->pairs;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_GetItem(__pyx_t_1, __pyx_v_i); if (unlikely(__pyx_t_2 == ((PyObject *)NULL))) __PYX_ERR(0, 269, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_INCREF(((PyObject *)((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_2)));
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_2);
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":268
 *         return PyTuple_GET_SIZE(self.pairs)
 * 
 *     cdef inline EnvelopePair getitem(self, size_t i):             # <<<<<<<<<<<<<<
 *         return <EnvelopePair>PyTuple_GetItem(self.pairs, i)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.getitem", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":271
 *         return <EnvelopePair>PyTuple_GetItem(self.pairs, i)
 * 
 *     cpdef bint _eq(self, Envelope other):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_11_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_other, int __pyx_skip_dispatch) {
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_a = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_b = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_eq", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_eq); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 271, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_11_eq)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_other)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 271, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 271, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":276
 *             EnvelopePair a, b
 * 
 *         n = self.get_size()             # <<<<<<<<<<<<<<
 *         if n != other.get_size():
 *             return False
 */
  __pyx_v_n = __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(__pyx_v_self);

  /* "ms_deisotope/_c/peak_set.pyx":277
 * 
 *         n = self.get_size()
 *         if n != other.get_size():             # <<<<<<<<<<<<<<
 *             return False
 * 
 */
  __pyx_t_5 = ((__pyx_v_n != __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(__pyx_v_other)) != 0);
  if (__pyx_t_5) {

    /* "ms_deisotope/_c/peak_set.pyx":278
 *         n = self.get_size()
 *         if n != other.get_size():
 *             return False             # <<<<<<<<<<<<<<
 * 
 *         i = 0
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":277
 * 
 *         n = self.get_size()
 *         if n != other.get_size():             # <<<<<<<<<<<<<<
 *             return False
 * 
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":280
 *             return False
 * 
 *         i = 0             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             a = self.getitem(i)
 */
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":281
 * 
 *         i = 0
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             a = self.getitem(i)
 *             b = other.getitem(i)
 */
  __pyx_t_6 = __pyx_v_n;
  __pyx_t_7 = __pyx_t_6;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "ms_deisotope/_c/peak_set.pyx":282
 *         i = 0
 *         for i in range(n):
 *             a = self.getitem(i)             # <<<<<<<<<<<<<<
 *             b = other.getitem(i)
 *             if a != b:
 */
    __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(__pyx_v_self, __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_a, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":283
 *         for i in range(n):
 *             a = self.getitem(i)
 *             b = other.getitem(i)             # <<<<<<<<<<<<<<
 *             if a != b:
 *                 return False
 */
    __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(__pyx_v_other, __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 283, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_b, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":284
 *             a = self.getitem(i)
 *             b = other.getitem(i)
 *             if a != b:             # <<<<<<<<<<<<<<
 *                 return False
 *         return True
 */
    __pyx_t_1 = PyObject_RichCompare(((PyObject *)__pyx_v_a), ((PyObject *)__pyx_v_b), Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 284, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_5) {

      /* "ms_deisotope/_c/peak_set.pyx":285
 *             b = other.getitem(i)
 *             if a != b:
 *                 return False             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
      __pyx_r = 0;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":284
 *             a = self.getitem(i)
 *             b = other.getitem(i)
 *             if a != b:             # <<<<<<<<<<<<<<
 *                 return False
 *         return True
 */
    }
  }

  /* "ms_deisotope/_c/peak_set.pyx":286
 *             if a != b:
 *                 return False
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     def __richcmp__(self, other, int code):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":271
 *         return <EnvelopePair>PyTuple_GetItem(self.pairs, i)
 * 
 *     cpdef bint _eq(self, Envelope other):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.Envelope._eq", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_a);
  __Pyx_XDECREF((PyObject *)__pyx_v_b);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_11_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_10_eq[] = "Envelope._eq(self, Envelope other) -> bool";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_11_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_eq (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope, 1, "other", 0))) __PYX_ERR(0, 271, __pyx_L1_error)
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_10_eq(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self), ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_other));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_10_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_eq", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__eq(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope._eq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":288
 *         return True
 * 
 *     def __richcmp__(self, other, int code):             # <<<<<<<<<<<<<<
 *         if code == 2:
 *             return self._eq(other)
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_13__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_13__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_12__richcmp__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self), ((PyObject *)__pyx_v_other), ((int)__pyx_v_code));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_12__richcmp__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__richcmp__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":289
 * 
 *     def __richcmp__(self, other, int code):
 *         if code == 2:             # <<<<<<<<<<<<<<
 *             return self._eq(other)
 *         elif code == 3:
 */
  switch (__pyx_v_code) {
    case 2:

    /* "ms_deisotope/_c/peak_set.pyx":290
 *     def __richcmp__(self, other, int code):
 *         if code == 2:
 *             return self._eq(other)             # <<<<<<<<<<<<<<
 *         elif code == 3:
 *             return not (self._eq(other))
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope))))) __PYX_ERR(0, 290, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyBool_FromLong(((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self->__pyx_vtab)->_eq(__pyx_v_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_other), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":289
 * 
 *     def __richcmp__(self, other, int code):
 *         if code == 2:             # <<<<<<<<<<<<<<
 *             return self._eq(other)
 *         elif code == 3:
 */
    break;
    case 3:

    /* "ms_deisotope/_c/peak_set.pyx":292
 *             return self._eq(other)
 *         elif code == 3:
 *             return not (self._eq(other))             # <<<<<<<<<<<<<<
 * 
 *     cpdef Envelope clone(self):
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope))))) __PYX_ERR(0, 292, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyBool_FromLong((!(((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self->__pyx_vtab)->_eq(__pyx_v_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_other), 0) != 0))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":291
 *         if code == 2:
 *             return self._eq(other)
 *         elif code == 3:             # <<<<<<<<<<<<<<
 *             return not (self._eq(other))
 * 
 */
    break;
    default: break;
  }

  /* "ms_deisotope/_c/peak_set.pyx":288
 *         return True
 * 
 *     def __richcmp__(self, other, int code):             # <<<<<<<<<<<<<<
 *         if code == 2:
 *             return self._eq(other)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":294
 *             return not (self._eq(other))
 * 
 *     cpdef Envelope clone(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             tuple duplicate
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_15clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_duplicate = 0;
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_pair = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_dup_pair = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_15clone)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 294, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope))))) __PYX_ERR(0, 294, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":299
 *             size_t i, n
 *             EnvelopePair pair, dup_pair
 *         n = self.get_size()             # <<<<<<<<<<<<<<
 *         duplicate = PyTuple_New(n)
 *         for i in range(n):
 */
  __pyx_v_n = __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(__pyx_v_self);

  /* "ms_deisotope/_c/peak_set.pyx":300
 *             EnvelopePair pair, dup_pair
 *         n = self.get_size()
 *         duplicate = PyTuple_New(n)             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             pair = self.getitem(i)
 */
  __pyx_t_1 = PyTuple_New(__pyx_v_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_duplicate = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":301
 *         n = self.get_size()
 *         duplicate = PyTuple_New(n)
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             pair = self.getitem(i)
 *             dup_pair = EnvelopePair._create(pair.mz, pair.intensity)
 */
  __pyx_t_5 = __pyx_v_n;
  __pyx_t_6 = __pyx_t_5;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "ms_deisotope/_c/peak_set.pyx":302
 *         duplicate = PyTuple_New(n)
 *         for i in range(n):
 *             pair = self.getitem(i)             # <<<<<<<<<<<<<<
 *             dup_pair = EnvelopePair._create(pair.mz, pair.intensity)
 *             Py_INCREF(dup_pair)
 */
    __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(__pyx_v_self, __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_pair, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":303
 *         for i in range(n):
 *             pair = self.getitem(i)
 *             dup_pair = EnvelopePair._create(pair.mz, pair.intensity)             # <<<<<<<<<<<<<<
 *             Py_INCREF(dup_pair)
 *             PyTuple_SetItem(duplicate, i, dup_pair)
 */
    __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create(__pyx_v_pair->mz, __pyx_v_pair->intensity)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 303, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_dup_pair, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":304
 *             pair = self.getitem(i)
 *             dup_pair = EnvelopePair._create(pair.mz, pair.intensity)
 *             Py_INCREF(dup_pair)             # <<<<<<<<<<<<<<
 *             PyTuple_SetItem(duplicate, i, dup_pair)
 *         return Envelope._create(duplicate)
 */
    Py_INCREF(((PyObject *)__pyx_v_dup_pair));

    /* "ms_deisotope/_c/peak_set.pyx":305
 *             dup_pair = EnvelopePair._create(pair.mz, pair.intensity)
 *             Py_INCREF(dup_pair)
 *             PyTuple_SetItem(duplicate, i, dup_pair)             # <<<<<<<<<<<<<<
 *         return Envelope._create(duplicate)
 * 
 */
    __pyx_t_8 = PyTuple_SetItem(__pyx_v_duplicate, __pyx_v_i, ((PyObject *)__pyx_v_dup_pair)); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 305, __pyx_L1_error)
  }

  /* "ms_deisotope/_c/peak_set.pyx":306
 *             Py_INCREF(dup_pair)
 *             PyTuple_SetItem(duplicate, i, dup_pair)
 *         return Envelope._create(duplicate)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(__pyx_v_duplicate)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":294
 *             return not (self._eq(other))
 * 
 *     cpdef Envelope clone(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             tuple duplicate
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_duplicate);
  __Pyx_XDECREF((PyObject *)__pyx_v_pair);
  __Pyx_XDECREF((PyObject *)__pyx_v_dup_pair);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_15clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_14clone[] = "Envelope.clone(self) -> Envelope";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_15clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_14clone(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_14clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_clone(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":308
 *         return Envelope._create(duplicate)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (
 *             Envelope_reconstructor,
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_17__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_16__reduce__[] = "Envelope.__reduce__(self)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_17__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_16__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_16__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":309
 * 
 *     def __reduce__(self):
 *         return (             # <<<<<<<<<<<<<<
 *             Envelope_reconstructor,
 *             (Envelope_to_bytes(self),))
 */
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/peak_set.pyx":310
 *     def __reduce__(self):
 *         return (
 *             Envelope_reconstructor,             # <<<<<<<<<<<<<<
 *             (Envelope_to_bytes(self),))
 *             # (self.pairs, ))
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Envelope_reconstructor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "ms_deisotope/_c/peak_set.pyx":311
 *         return (
 *             Envelope_reconstructor,
 *             (Envelope_to_bytes(self),))             # <<<<<<<<<<<<<<
 *             # (self.pairs, ))
 * 
 */
  __pyx_t_2 = __pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_to_bytes(__pyx_v_self, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":310
 *     def __reduce__(self):
 *         return (
 *             Envelope_reconstructor,             # <<<<<<<<<<<<<<
 *             (Envelope_to_bytes(self),))
 *             # (self.pairs, ))
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":308
 *         return Envelope._create(duplicate)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (
 *             Envelope_reconstructor,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":315
 * 
 *     @staticmethod
 *     cdef Envelope _create(tuple pairs):             # <<<<<<<<<<<<<<
 *         cdef:
 *             Envelope inst
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(PyObject *__pyx_v_pairs) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_inst = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create", 0);

  /* "ms_deisotope/_c/peak_set.pyx":318
 *         cdef:
 *             Envelope inst
 *         inst = Envelope.__new__(Envelope)             # <<<<<<<<<<<<<<
 *         inst.pairs = pairs
 *         return inst
 */
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set_Envelope(((PyTypeObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));
  __pyx_v_inst = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":319
 *             Envelope inst
 *         inst = Envelope.__new__(Envelope)
 *         inst.pairs = pairs             # <<<<<<<<<<<<<<
 *         return inst
 * 
 */
  __Pyx_INCREF(__pyx_v_pairs);
  __Pyx_GIVEREF(__pyx_v_pairs);
  __Pyx_GOTREF(__pyx_v_inst->pairs);
  __Pyx_DECREF(__pyx_v_inst->pairs);
  __pyx_v_inst->pairs = __pyx_v_pairs;

  /* "ms_deisotope/_c/peak_set.pyx":320
 *         inst = Envelope.__new__(Envelope)
 *         inst.pairs = pairs
 *         return inst             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));
  __pyx_r = __pyx_v_inst;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":315
 * 
 *     @staticmethod
 *     cdef Envelope _create(tuple pairs):             # <<<<<<<<<<<<<<
 *         cdef:
 *             Envelope inst
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope._create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":26
 * 
 * cdef class Envelope:
 *     cdef public tuple pairs             # <<<<<<<<<<<<<<
 * 
 *     cdef inline size_t get_size(self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->pairs);
  __pyx_r = __pyx_v_self->pairs;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyTuple_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 26, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->pairs);
  __Pyx_DECREF(__pyx_v_self->pairs);
  __pyx_v_self->pairs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.pairs.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->pairs);
  __Pyx_DECREF(__pyx_v_self->pairs);
  __pyx_v_self->pairs = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":357
 *         The average signal-to-noise ratio of the peaks in the isotopic pattern this peak captures
 *     """
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,             # <<<<<<<<<<<<<<
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):
 */

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_neutral_mass = 0;
  PyObject *__pyx_v_intensity = 0;
  PyObject *__pyx_v_charge = 0;
  PyObject *__pyx_v_signal_to_noise = 0;
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_full_width_at_half_max = 0;
  PyObject *__pyx_v_a_to_a2_ratio = 0;
  PyObject *__pyx_v_most_abundant_mass = 0;
  PyObject *__pyx_v_average_mass = 0;
  PyObject *__pyx_v_score = 0;
  PyObject *__pyx_v_envelope = 0;
  PyObject *__pyx_v_mz = 0;
  PyObject *__pyx_v_fit = 0;
  PyObject *__pyx_v_chosen_for_msms = 0;
  PyObject *__pyx_v_area = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_intensity,&__pyx_n_s_charge,&__pyx_n_s_signal_to_noise,&__pyx_n_s_index,&__pyx_n_s_full_width_at_half_max,&__pyx_n_s_a_to_a2_ratio,&__pyx_n_s_most_abundant_mass,&__pyx_n_s_average_mass,&__pyx_n_s_score,&__pyx_n_s_envelope,&__pyx_n_s_mz,&__pyx_n_s_fit,&__pyx_n_s_chosen_for_msms,&__pyx_n_s_area,0};
    PyObject* values[15] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    values[6] = ((PyObject *)__pyx_int_0);
    values[7] = ((PyObject *)__pyx_int_0);
    values[8] = ((PyObject *)__pyx_int_0);
    values[9] = ((PyObject *)__pyx_int_0);

    /* "ms_deisotope/_c/peak_set.pyx":359
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):             # <<<<<<<<<<<<<<
 *         if index is None:
 *             index = _Index._create(0, 0)
 */
    values[10] = ((PyObject *)__pyx_empty_tuple);
    values[11] = ((PyObject *)__pyx_int_0);
    values[12] = ((PyObject *)Py_None);
    values[13] = ((PyObject *)Py_False);
    values[14] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 15: values[14] = PyTuple_GET_ITEM(__pyx_args, 14);
        CYTHON_FALLTHROUGH;
        case 14: values[13] = PyTuple_GET_ITEM(__pyx_args, 13);
        CYTHON_FALLTHROUGH;
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_intensity)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 15, 1); __PYX_ERR(0, 357, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_charge)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 15, 2); __PYX_ERR(0, 357, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_signal_to_noise)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 15, 3); __PYX_ERR(0, 357, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 15, 4); __PYX_ERR(0, 357, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_full_width_at_half_max)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 15, 5); __PYX_ERR(0, 357, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_a_to_a2_ratio);
          if (value) { values[6] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_most_abundant_mass);
          if (value) { values[7] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_average_mass);
          if (value) { values[8] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_score);
          if (value) { values[9] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_envelope);
          if (value) { values[10] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 11:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mz);
          if (value) { values[11] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 12:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fit);
          if (value) { values[12] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 13:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_chosen_for_msms);
          if (value) { values[13] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 14:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_area);
          if (value) { values[14] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 357, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case 15: values[14] = PyTuple_GET_ITEM(__pyx_args, 14);
        CYTHON_FALLTHROUGH;
        case 14: values[13] = PyTuple_GET_ITEM(__pyx_args, 13);
        CYTHON_FALLTHROUGH;
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_neutral_mass = values[0];
    __pyx_v_intensity = values[1];
    __pyx_v_charge = values[2];
    __pyx_v_signal_to_noise = values[3];
    __pyx_v_index = values[4];
    __pyx_v_full_width_at_half_max = values[5];
    __pyx_v_a_to_a2_ratio = values[6];
    __pyx_v_most_abundant_mass = values[7];
    __pyx_v_average_mass = values[8];
    __pyx_v_score = values[9];
    __pyx_v_envelope = values[10];
    __pyx_v_mz = values[11];
    __pyx_v_fit = values[12];
    __pyx_v_chosen_for_msms = values[13];
    __pyx_v_area = values[14];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 15, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 357, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_intensity, __pyx_v_charge, __pyx_v_signal_to_noise, __pyx_v_index, __pyx_v_full_width_at_half_max, __pyx_v_a_to_a2_ratio, __pyx_v_most_abundant_mass, __pyx_v_average_mass, __pyx_v_score, __pyx_v_envelope, __pyx_v_mz, __pyx_v_fit, __pyx_v_chosen_for_msms, __pyx_v_area);

  /* "ms_deisotope/_c/peak_set.pyx":357
 *         The average signal-to-noise ratio of the peaks in the isotopic pattern this peak captures
 *     """
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,             # <<<<<<<<<<<<<<
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_intensity, PyObject *__pyx_v_charge, PyObject *__pyx_v_signal_to_noise, PyObject *__pyx_v_index, PyObject *__pyx_v_full_width_at_half_max, PyObject *__pyx_v_a_to_a2_ratio, PyObject *__pyx_v_most_abundant_mass, PyObject *__pyx_v_average_mass, PyObject *__pyx_v_score, PyObject *__pyx_v_envelope, PyObject *__pyx_v_mz, PyObject *__pyx_v_fit, PyObject *__pyx_v_chosen_for_msms, PyObject *__pyx_v_area) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  double __pyx_t_4;
  int __pyx_t_5;
  double __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_INCREF(__pyx_v_index);
  __Pyx_INCREF(__pyx_v_envelope);

  /* "ms_deisotope/_c/peak_set.pyx":360
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):
 *         if index is None:             # <<<<<<<<<<<<<<
 *             index = _Index._create(0, 0)
 *         elif index == -1:
 */
  __pyx_t_1 = (__pyx_v_index == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":361
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):
 *         if index is None:
 *             index = _Index._create(0, 0)             # <<<<<<<<<<<<<<
 *         elif index == -1:
 *             index = _Index._create(0, 0)
 */
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(0, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 361, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_index, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":360
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):
 *         if index is None:             # <<<<<<<<<<<<<<
 *             index = _Index._create(0, 0)
 *         elif index == -1:
 */
    goto __pyx_L3;
  }

  /* "ms_deisotope/_c/peak_set.pyx":362
 *         if index is None:
 *             index = _Index._create(0, 0)
 *         elif index == -1:             # <<<<<<<<<<<<<<
 *             index = _Index._create(0, 0)
 *         if envelope is None:
 */
  __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_v_index, __pyx_int_neg_1, -1L, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 362, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":363
 *             index = _Index._create(0, 0)
 *         elif index == -1:
 *             index = _Index._create(0, 0)             # <<<<<<<<<<<<<<
 *         if envelope is None:
 *             envelope = ()
 */
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(0, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_index, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":362
 *         if index is None:
 *             index = _Index._create(0, 0)
 *         elif index == -1:             # <<<<<<<<<<<<<<
 *             index = _Index._create(0, 0)
 *         if envelope is None:
 */
  }
  __pyx_L3:;

  /* "ms_deisotope/_c/peak_set.pyx":364
 *         elif index == -1:
 *             index = _Index._create(0, 0)
 *         if envelope is None:             # <<<<<<<<<<<<<<
 *             envelope = ()
 *         self.neutral_mass = neutral_mass
 */
  __pyx_t_2 = (__pyx_v_envelope == Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":365
 *             index = _Index._create(0, 0)
 *         if envelope is None:
 *             envelope = ()             # <<<<<<<<<<<<<<
 *         self.neutral_mass = neutral_mass
 *         self.intensity = intensity
 */
    __Pyx_INCREF(__pyx_empty_tuple);
    __Pyx_DECREF_SET(__pyx_v_envelope, __pyx_empty_tuple);

    /* "ms_deisotope/_c/peak_set.pyx":364
 *         elif index == -1:
 *             index = _Index._create(0, 0)
 *         if envelope is None:             # <<<<<<<<<<<<<<
 *             envelope = ()
 *         self.neutral_mass = neutral_mass
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":366
 *         if envelope is None:
 *             envelope = ()
 *         self.neutral_mass = neutral_mass             # <<<<<<<<<<<<<<
 *         self.intensity = intensity
 *         self.signal_to_noise = signal_to_noise
 */
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_neutral_mass); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 366, __pyx_L1_error)
  __pyx_v_self->neutral_mass = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":367
 *             envelope = ()
 *         self.neutral_mass = neutral_mass
 *         self.intensity = intensity             # <<<<<<<<<<<<<<
 *         self.signal_to_noise = signal_to_noise
 *         self.index = index
 */
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_intensity); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 367, __pyx_L1_error)
  __pyx_v_self->__pyx_base.intensity = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":368
 *         self.neutral_mass = neutral_mass
 *         self.intensity = intensity
 *         self.signal_to_noise = signal_to_noise             # <<<<<<<<<<<<<<
 *         self.index = index
 *         self.full_width_at_half_max = full_width_at_half_max
 */
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_signal_to_noise); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 368, __pyx_L1_error)
  __pyx_v_self->signal_to_noise = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":369
 *         self.intensity = intensity
 *         self.signal_to_noise = signal_to_noise
 *         self.index = index             # <<<<<<<<<<<<<<
 *         self.full_width_at_half_max = full_width_at_half_max
 *         self.charge = charge
 */
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index, __pyx_v_index) < 0) __PYX_ERR(0, 369, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":370
 *         self.signal_to_noise = signal_to_noise
 *         self.index = index
 *         self.full_width_at_half_max = full_width_at_half_max             # <<<<<<<<<<<<<<
 *         self.charge = charge
 *         self.a_to_a2_ratio = a_to_a2_ratio
 */
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_full_width_at_half_max); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 370, __pyx_L1_error)
  __pyx_v_self->full_width_at_half_max = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":371
 *         self.index = index
 *         self.full_width_at_half_max = full_width_at_half_max
 *         self.charge = charge             # <<<<<<<<<<<<<<
 *         self.a_to_a2_ratio = a_to_a2_ratio
 *         self.most_abundant_mass = most_abundant_mass
 */
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_v_charge); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 371, __pyx_L1_error)
  __pyx_v_self->charge = __pyx_t_5;

  /* "ms_deisotope/_c/peak_set.pyx":372
 *         self.full_width_at_half_max = full_width_at_half_max
 *         self.charge = charge
 *         self.a_to_a2_ratio = a_to_a2_ratio             # <<<<<<<<<<<<<<
 *         self.most_abundant_mass = most_abundant_mass
 *         self.average_mass = average_mass
 */
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_a_to_a2_ratio); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 372, __pyx_L1_error)
  __pyx_v_self->a_to_a2_ratio = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":373
 *         self.charge = charge
 *         self.a_to_a2_ratio = a_to_a2_ratio
 *         self.most_abundant_mass = most_abundant_mass             # <<<<<<<<<<<<<<
 *         self.average_mass = average_mass
 *         self.score = score
 */
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_most_abundant_mass); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 373, __pyx_L1_error)
  __pyx_v_self->most_abundant_mass = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":374
 *         self.a_to_a2_ratio = a_to_a2_ratio
 *         self.most_abundant_mass = most_abundant_mass
 *         self.average_mass = average_mass             # <<<<<<<<<<<<<<
 *         self.score = score
 *         if not isinstance(envelope, Envelope):
 */
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_average_mass); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 374, __pyx_L1_error)
  __pyx_v_self->average_mass = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":375
 *         self.most_abundant_mass = most_abundant_mass
 *         self.average_mass = average_mass
 *         self.score = score             # <<<<<<<<<<<<<<
 *         if not isinstance(envelope, Envelope):
 *             self.envelope = Envelope(envelope)
 */
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_score); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 375, __pyx_L1_error)
  __pyx_v_self->score = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":376
 *         self.average_mass = average_mass
 *         self.score = score
 *         if not isinstance(envelope, Envelope):             # <<<<<<<<<<<<<<
 *             self.envelope = Envelope(envelope)
 *         else:
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_envelope, __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope); 
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":377
 *         self.score = score
 *         if not isinstance(envelope, Envelope):
 *             self.envelope = Envelope(envelope)             # <<<<<<<<<<<<<<
 *         else:
 *             self.envelope = envelope
 */
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope), __pyx_v_envelope); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 377, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->envelope);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->envelope));
    __pyx_v_self->envelope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":376
 *         self.average_mass = average_mass
 *         self.score = score
 *         if not isinstance(envelope, Envelope):             # <<<<<<<<<<<<<<
 *             self.envelope = Envelope(envelope)
 *         else:
 */
    goto __pyx_L5;
  }

  /* "ms_deisotope/_c/peak_set.pyx":379
 *             self.envelope = Envelope(envelope)
 *         else:
 *             self.envelope = envelope             # <<<<<<<<<<<<<<
 *         self.mz = mz or mass_charge_ratio(self.neutral_mass, self.charge)
 *         self.fit = fit
 */
  /*else*/ {
    if (!(likely(((__pyx_v_envelope) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_envelope, __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope))))) __PYX_ERR(0, 379, __pyx_L1_error)
    __pyx_t_3 = __pyx_v_envelope;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->envelope);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->envelope));
    __pyx_v_self->envelope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L5:;

  /* "ms_deisotope/_c/peak_set.pyx":380
 *         else:
 *             self.envelope = envelope
 *         self.mz = mz or mass_charge_ratio(self.neutral_mass, self.charge)             # <<<<<<<<<<<<<<
 *         self.fit = fit
 *         self.chosen_for_msms = chosen_for_msms
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_mz); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 380, __pyx_L1_error)
  if (!__pyx_t_2) {
  } else {
    __pyx_t_6 = __pyx_PyFloat_AsDouble(__pyx_v_mz); if (unlikely((__pyx_t_6 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 380, __pyx_L1_error)
    __pyx_t_4 = __pyx_t_6;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_6 = __pyx_f_12ms_deisotope_2_c_9averagine_mass_charge_ratio(__pyx_v_self->neutral_mass, __pyx_v_self->charge, NULL);
  __pyx_t_4 = __pyx_t_6;
  __pyx_L6_bool_binop_done:;
  __pyx_v_self->__pyx_base.mz = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":381
 *             self.envelope = envelope
 *         self.mz = mz or mass_charge_ratio(self.neutral_mass, self.charge)
 *         self.fit = fit             # <<<<<<<<<<<<<<
 *         self.chosen_for_msms = chosen_for_msms
 *         self.area = area
 */
  if (!(likely(((__pyx_v_fit) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_fit, __pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord))))) __PYX_ERR(0, 381, __pyx_L1_error)
  __pyx_t_3 = __pyx_v_fit;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->fit);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->fit));
  __pyx_v_self->fit = ((struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":382
 *         self.mz = mz or mass_charge_ratio(self.neutral_mass, self.charge)
 *         self.fit = fit
 *         self.chosen_for_msms = chosen_for_msms             # <<<<<<<<<<<<<<
 *         self.area = area
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_chosen_for_msms); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 382, __pyx_L1_error)
  __pyx_v_self->chosen_for_msms = __pyx_t_2;

  /* "ms_deisotope/_c/peak_set.pyx":383
 *         self.fit = fit
 *         self.chosen_for_msms = chosen_for_msms
 *         self.area = area             # <<<<<<<<<<<<<<
 * 
 *     property index:
 */
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_area); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 383, __pyx_L1_error)
  __pyx_v_self->__pyx_base.area = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":357
 *         The average signal-to-noise ratio of the peaks in the isotopic pattern this peak captures
 *     """
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,             # <<<<<<<<<<<<<<
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XDECREF(__pyx_v_envelope);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":386
 * 
 *     property index:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._index
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":387
 *     property index:
 *         def __get__(self):
 *             return self._index             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_index));
  __pyx_r = ((PyObject *)__pyx_v_self->_index);
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":386
 * 
 *     property index:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._index
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":389
 *             return self._index
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if isinstance(value, _Index):
 *                 self._index = value
 */

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":390
 * 
 *         def __set__(self, value):
 *             if isinstance(value, _Index):             # <<<<<<<<<<<<<<
 *                 self._index = value
 *             elif value >= 0:
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_value, __pyx_ptype_12ms_deisotope_2_c_8peak_set__Index); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":391
 *         def __set__(self, value):
 *             if isinstance(value, _Index):
 *                 self._index = value             # <<<<<<<<<<<<<<
 *             elif value >= 0:
 *                 self._index = _Index._create(value, value)
 */
    if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_12ms_deisotope_2_c_8peak_set__Index))))) __PYX_ERR(0, 391, __pyx_L1_error)
    __pyx_t_3 = __pyx_v_value;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->_index);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->_index));
    __pyx_v_self->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":390
 * 
 *         def __set__(self, value):
 *             if isinstance(value, _Index):             # <<<<<<<<<<<<<<
 *                 self._index = value
 *             elif value >= 0:
 */
    goto __pyx_L3;
  }

  /* "ms_deisotope/_c/peak_set.pyx":392
 *             if isinstance(value, _Index):
 *                 self._index = value
 *             elif value >= 0:             # <<<<<<<<<<<<<<
 *                 self._index = _Index._create(value, value)
 *             else:
 */
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_value, __pyx_int_0, Py_GE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 392, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 392, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":393
 *                 self._index = value
 *             elif value >= 0:
 *                 self._index = _Index._create(value, value)             # <<<<<<<<<<<<<<
 *             else:
 *                 self._index = _Index._create(0, 0)
 */
    __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 393, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 393, __pyx_L1_error)
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(__pyx_t_4, __pyx_t_5)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->_index);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->_index));
    __pyx_v_self->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":392
 *             if isinstance(value, _Index):
 *                 self._index = value
 *             elif value >= 0:             # <<<<<<<<<<<<<<
 *                 self._index = _Index._create(value, value)
 *             else:
 */
    goto __pyx_L3;
  }

  /* "ms_deisotope/_c/peak_set.pyx":395
 *                 self._index = _Index._create(value, value)
 *             else:
 *                 self._index = _Index._create(0, 0)             # <<<<<<<<<<<<<<
 * 
 *     def __hash__(self):
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(0, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 395, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->_index);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->_index));
    __pyx_v_self->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "ms_deisotope/_c/peak_set.pyx":389
 *             return self._index
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if isinstance(value, _Index):
 *                 self._index = value
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":397
 *                 self._index = _Index._create(0, 0)
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return hash((self.mz, self.intensity, self.charge))
 * 
 */

/* Python wrapper */
static Py_hash_t __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3__hash__(PyObject *__pyx_v_self); /*proto*/
static Py_hash_t __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3__hash__(PyObject *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_2__hash__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_hash_t __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_2__hash__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_hash_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__hash__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":398
 * 
 *     def __hash__(self):
 *         return hash((self.mz, self.intensity, self.charge))             # <<<<<<<<<<<<<<
 * 
 *     cpdef PeakBase clone(self):
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_5 = PyObject_Hash(__pyx_t_4); if (unlikely(__pyx_t_5 == ((Py_hash_t)-1))) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":397
 *                 self._index = _Index._create(0, 0)
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return hash((self.mz, self.intensity, self.charge))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.__hash__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  if (unlikely(__pyx_r == -1) && !PyErr_Occurred()) __pyx_r = -2;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":400
 *         return hash((self.mz, self.intensity, self.charge))
 * 
 *     cpdef PeakBase clone(self):             # <<<<<<<<<<<<<<
 *         return DeconvolutedPeak(self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 400, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5clone)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 400, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase))))) __PYX_ERR(0, 400, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":401
 * 
 *     cpdef PeakBase clone(self):
 *         return DeconvolutedPeak(self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                 self.most_abundant_mass, self.average_mass, self.score,
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->signal_to_noise); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "ms_deisotope/_c/peak_set.pyx":402
 *     cpdef PeakBase clone(self):
 *         return DeconvolutedPeak(self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,             # <<<<<<<<<<<<<<
 *                                 self.most_abundant_mass, self.average_mass, self.score,
 *                                 self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->full_width_at_half_max); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->a_to_a2_ratio); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "ms_deisotope/_c/peak_set.pyx":403
 *         return DeconvolutedPeak(self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                 self.most_abundant_mass, self.average_mass, self.score,             # <<<<<<<<<<<<<<
 *                                 self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 * 
 */
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->most_abundant_mass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_self->average_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_self->score); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);

  /* "ms_deisotope/_c/peak_set.pyx":404
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                 self.most_abundant_mass, self.average_mass, self.score,
 *                                 self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.mz); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = __Pyx_PyBool_FromLong(__pyx_v_self->chosen_for_msms); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.area); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);

  /* "ms_deisotope/_c/peak_set.pyx":401
 * 
 *     cpdef PeakBase clone(self):
 *         return DeconvolutedPeak(self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                 self.most_abundant_mass, self.average_mass, self.score,
 */
  __pyx_t_14 = PyTuple_New(15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_14, 3, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_14, 4, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_14, 5, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_14, 6, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_14, 7, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_14, 8, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_14, 9, __pyx_t_10);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->envelope));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->envelope));
  PyTuple_SET_ITEM(__pyx_t_14, 10, ((PyObject *)__pyx_v_self->envelope));
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_14, 11, __pyx_t_11);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->fit));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->fit));
  PyTuple_SET_ITEM(__pyx_t_14, 12, ((PyObject *)__pyx_v_self->fit));
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_14, 13, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_14, 14, __pyx_t_13);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak), __pyx_t_14, NULL); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_r = ((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_t_13);
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":400
 *         return hash((self.mz, self.intensity, self.charge))
 * 
 *     cpdef PeakBase clone(self):             # <<<<<<<<<<<<<<
 *         return DeconvolutedPeak(self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_4clone[] = "DeconvolutedPeak.clone(self) -> PeakBase";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_4clone(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_4clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_clone(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 400, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":406
 *                                 self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return DeconvolutedPeak, (self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_7__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6__reduce__[] = "DeconvolutedPeak.__reduce__(self)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_7__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":407
 * 
 *     def __reduce__(self):
 *         return DeconvolutedPeak, (self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                   self.most_abundant_mass, self.average_mass, self.score,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->signal_to_noise); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "ms_deisotope/_c/peak_set.pyx":408
 *     def __reduce__(self):
 *         return DeconvolutedPeak, (self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,             # <<<<<<<<<<<<<<
 *                                   self.most_abundant_mass, self.average_mass, self.score,
 *                                   self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->full_width_at_half_max); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->a_to_a2_ratio); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "ms_deisotope/_c/peak_set.pyx":409
 *         return DeconvolutedPeak, (self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                   self.most_abundant_mass, self.average_mass, self.score,             # <<<<<<<<<<<<<<
 *                                   self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 * 
 */
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->most_abundant_mass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_self->average_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_self->score); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);

  /* "ms_deisotope/_c/peak_set.pyx":410
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                   self.most_abundant_mass, self.average_mass, self.score,
 *                                   self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):
 */
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.mz); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 410, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = __Pyx_PyBool_FromLong(__pyx_v_self->chosen_for_msms); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 410, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.area); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 410, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);

  /* "ms_deisotope/_c/peak_set.pyx":407
 * 
 *     def __reduce__(self):
 *         return DeconvolutedPeak, (self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                   self.most_abundant_mass, self.average_mass, self.score,
 */
  __pyx_t_14 = PyTuple_New(15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_14, 3, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_14, 4, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_14, 5, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_14, 6, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_14, 7, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_14, 8, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_14, 9, __pyx_t_10);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->envelope));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->envelope));
  PyTuple_SET_ITEM(__pyx_t_14, 10, ((PyObject *)__pyx_v_self->envelope));
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_14, 11, __pyx_t_11);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->fit));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->fit));
  PyTuple_SET_ITEM(__pyx_t_14, 12, ((PyObject *)__pyx_v_self->fit));
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_14, 13, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_14, 14, __pyx_t_13);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_13 = PyTuple_New(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak));
  PyTuple_SET_ITEM(__pyx_t_13, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak));
  __Pyx_GIVEREF(__pyx_t_14);
  PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_t_14);
  __pyx_t_14 = 0;
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":406
 *                                 self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return DeconvolutedPeak, (self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":412
 *                                   self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):             # <<<<<<<<<<<<<<
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_9_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_eq", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_eq); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_9_eq)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_other)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 412, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 412, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":413
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (             # <<<<<<<<<<<<<<
 *             abs(self.intensity - other.intensity) < epsilon) and (
 *             self.charge == other.charge) and (
 */
  __pyx_t_6 = ((fabs((__pyx_v_self->neutral_mass - __pyx_v_other->neutral_mass)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }

  /* "ms_deisotope/_c/peak_set.pyx":414
 *     cpdef bint _eq(self, DeconvolutedPeak other):
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (             # <<<<<<<<<<<<<<
 *             self.charge == other.charge) and (
 *             abs(self.score - other.score) < epsilon)
 */
  __pyx_t_6 = ((fabs((__pyx_v_self->__pyx_base.intensity - __pyx_v_other->__pyx_base.intensity)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }

  /* "ms_deisotope/_c/peak_set.pyx":415
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (
 *             self.charge == other.charge) and (             # <<<<<<<<<<<<<<
 *             abs(self.score - other.score) < epsilon)
 * 
 */
  __pyx_t_6 = ((__pyx_v_self->charge == __pyx_v_other->charge) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }

  /* "ms_deisotope/_c/peak_set.pyx":416
 *             abs(self.intensity - other.intensity) < epsilon) and (
 *             self.charge == other.charge) and (
 *             abs(self.score - other.score) < epsilon)             # <<<<<<<<<<<<<<
 * 
 *     def __richcmp__(self, other, int code):
 */
  __pyx_t_6 = ((fabs((__pyx_v_self->score - __pyx_v_other->score)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":412
 *                                   self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):             # <<<<<<<<<<<<<<
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.DeconvolutedPeak._eq", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_9_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8_eq[] = "DeconvolutedPeak._eq(self, DeconvolutedPeak other) -> bool";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_9_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_eq (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, 1, "other", 0))) __PYX_ERR(0, 412, __pyx_L1_error)
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8_eq(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_other));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_eq", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__eq(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak._eq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":418
 *             abs(self.score - other.score) < epsilon)
 * 
 *     def __richcmp__(self, other, int code):             # <<<<<<<<<<<<<<
 *         if code == 2:
 *             return self._eq(other)
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_11__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_11__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_10__richcmp__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_other), ((int)__pyx_v_code));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_10__richcmp__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__richcmp__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":419
 * 
 *     def __richcmp__(self, other, int code):
 *         if code == 2:             # <<<<<<<<<<<<<<
 *             return self._eq(other)
 *         elif code == 3:
 */
  switch (__pyx_v_code) {
    case 2:

    /* "ms_deisotope/_c/peak_set.pyx":420
 *     def __richcmp__(self, other, int code):
 *         if code == 2:
 *             return self._eq(other)             # <<<<<<<<<<<<<<
 *         elif code == 3:
 *             return not (self._eq(other))
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 420, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyBool_FromLong(((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self->__pyx_base.__pyx_vtab)->_eq(__pyx_v_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_other), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 420, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":419
 * 
 *     def __richcmp__(self, other, int code):
 *         if code == 2:             # <<<<<<<<<<<<<<
 *             return self._eq(other)
 *         elif code == 3:
 */
    break;
    case 3:

    /* "ms_deisotope/_c/peak_set.pyx":422
 *             return self._eq(other)
 *         elif code == 3:
 *             return not (self._eq(other))             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 422, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyBool_FromLong((!(((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self->__pyx_base.__pyx_vtab)->_eq(__pyx_v_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_other), 0) != 0))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 422, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":421
 *         if code == 2:
 *             return self._eq(other)
 *         elif code == 3:             # <<<<<<<<<<<<<<
 *             return not (self._eq(other))
 * 
 */
    break;
    default: break;
  }

  /* "ms_deisotope/_c/peak_set.pyx":418
 *             abs(self.score - other.score) < epsilon)
 * 
 *     def __richcmp__(self, other, int code):             # <<<<<<<<<<<<<<
 *         if code == 2:
 *             return self._eq(other)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":424
 *             return not (self._eq(other))
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return ("{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, "
 *             "average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, "
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12__repr__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":425
 * 
 *     def __repr__(self):
 *         return ("{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, "             # <<<<<<<<<<<<<<
 *             "average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, "
 *             "envelope={self.envelope}, full_width_at_half_max={self.full_width_at_half_max}, index={self.index}, "
 */
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/peak_set.pyx":429
 *             "envelope={self.envelope}, full_width_at_half_max={self.full_width_at_half_max}, index={self.index}, "
 *             "intensity={self.intensity}, most_abundant_mass={self.most_abundant_mass}, mz={self.mz}, "
 *             "neutral_mass={self.neutral_mass}, score={self.score}, signal_to_noise={self.signal_to_noise})").format(self=self)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_self___class_____name___a_to_a2, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_self, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 429, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":424
 *             return not (self._eq(other))
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return ("{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, "
 *             "average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, "
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":433
 * 
 *     @staticmethod
 *     cdef DeconvolutedPeak _create_simple(double neutral_mass, double intensity, int charge,             # <<<<<<<<<<<<<<
 *                                          double score, double mz, Envelope envelope):
 *         cdef:
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__create_simple(double __pyx_v_neutral_mass, double __pyx_v_intensity, int __pyx_v_charge, double __pyx_v_score, double __pyx_v_mz, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_envelope) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_inst = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create_simple", 0);

  /* "ms_deisotope/_c/peak_set.pyx":437
 *         cdef:
 *             DeconvolutedPeak inst
 *         inst = DeconvolutedPeak.__new__(DeconvolutedPeak)             # <<<<<<<<<<<<<<
 *         inst.neutral_mass = neutral_mass
 *         inst.intensity = intensity
 */
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(((PyTypeObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));
  __pyx_v_inst = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":438
 *             DeconvolutedPeak inst
 *         inst = DeconvolutedPeak.__new__(DeconvolutedPeak)
 *         inst.neutral_mass = neutral_mass             # <<<<<<<<<<<<<<
 *         inst.intensity = intensity
 *         inst.charge = charge
 */
  __pyx_v_inst->neutral_mass = __pyx_v_neutral_mass;

  /* "ms_deisotope/_c/peak_set.pyx":439
 *         inst = DeconvolutedPeak.__new__(DeconvolutedPeak)
 *         inst.neutral_mass = neutral_mass
 *         inst.intensity = intensity             # <<<<<<<<<<<<<<
 *         inst.charge = charge
 *         inst.score = score
 */
  __pyx_v_inst->__pyx_base.intensity = __pyx_v_intensity;

  /* "ms_deisotope/_c/peak_set.pyx":440
 *         inst.neutral_mass = neutral_mass
 *         inst.intensity = intensity
 *         inst.charge = charge             # <<<<<<<<<<<<<<
 *         inst.score = score
 *         inst.signal_to_noise = score
 */
  __pyx_v_inst->charge = __pyx_v_charge;

  /* "ms_deisotope/_c/peak_set.pyx":441
 *         inst.intensity = intensity
 *         inst.charge = charge
 *         inst.score = score             # <<<<<<<<<<<<<<
 *         inst.signal_to_noise = score
 *         inst.mz = mz
 */
  __pyx_v_inst->score = __pyx_v_score;

  /* "ms_deisotope/_c/peak_set.pyx":442
 *         inst.charge = charge
 *         inst.score = score
 *         inst.signal_to_noise = score             # <<<<<<<<<<<<<<
 *         inst.mz = mz
 *         inst.envelope = envelope
 */
  __pyx_v_inst->signal_to_noise = __pyx_v_score;

  /* "ms_deisotope/_c/peak_set.pyx":443
 *         inst.score = score
 *         inst.signal_to_noise = score
 *         inst.mz = mz             # <<<<<<<<<<<<<<
 *         inst.envelope = envelope
 *         inst.index = _Index._create(0, 0)
 */
  __pyx_v_inst->__pyx_base.mz = __pyx_v_mz;

  /* "ms_deisotope/_c/peak_set.pyx":444
 *         inst.signal_to_noise = score
 *         inst.mz = mz
 *         inst.envelope = envelope             # <<<<<<<<<<<<<<
 *         inst.index = _Index._create(0, 0)
 *         inst.full_width_at_half_max = 0
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_envelope));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_envelope));
  __Pyx_GOTREF(__pyx_v_inst->envelope);
  __Pyx_DECREF(((PyObject *)__pyx_v_inst->envelope));
  __pyx_v_inst->envelope = __pyx_v_envelope;

  /* "ms_deisotope/_c/peak_set.pyx":445
 *         inst.mz = mz
 *         inst.envelope = envelope
 *         inst.index = _Index._create(0, 0)             # <<<<<<<<<<<<<<
 *         inst.full_width_at_half_max = 0
 *         inst.a_to_a2_ratio = 0
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(0, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_inst), __pyx_n_s_index, __pyx_t_1) < 0) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":446
 *         inst.envelope = envelope
 *         inst.index = _Index._create(0, 0)
 *         inst.full_width_at_half_max = 0             # <<<<<<<<<<<<<<
 *         inst.a_to_a2_ratio = 0
 *         inst.most_abundant_mass = 0
 */
  __pyx_v_inst->full_width_at_half_max = 0.0;

  /* "ms_deisotope/_c/peak_set.pyx":447
 *         inst.index = _Index._create(0, 0)
 *         inst.full_width_at_half_max = 0
 *         inst.a_to_a2_ratio = 0             # <<<<<<<<<<<<<<
 *         inst.most_abundant_mass = 0
 *         inst.average_mass = 0
 */
  __pyx_v_inst->a_to_a2_ratio = 0.0;

  /* "ms_deisotope/_c/peak_set.pyx":448
 *         inst.full_width_at_half_max = 0
 *         inst.a_to_a2_ratio = 0
 *         inst.most_abundant_mass = 0             # <<<<<<<<<<<<<<
 *         inst.average_mass = 0
 *         inst.area = 0
 */
  __pyx_v_inst->most_abundant_mass = 0.0;

  /* "ms_deisotope/_c/peak_set.pyx":449
 *         inst.a_to_a2_ratio = 0
 *         inst.most_abundant_mass = 0
 *         inst.average_mass = 0             # <<<<<<<<<<<<<<
 *         inst.area = 0
 * 
 */
  __pyx_v_inst->average_mass = 0.0;

  /* "ms_deisotope/_c/peak_set.pyx":450
 *         inst.most_abundant_mass = 0
 *         inst.average_mass = 0
 *         inst.area = 0             # <<<<<<<<<<<<<<
 * 
 *         return inst
 */
  __pyx_v_inst->__pyx_base.area = 0.0;

  /* "ms_deisotope/_c/peak_set.pyx":452
 *         inst.area = 0
 * 
 *         return inst             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));
  __pyx_r = __pyx_v_inst;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":433
 * 
 *     @staticmethod
 *     cdef DeconvolutedPeak _create_simple(double neutral_mass, double intensity, int charge,             # <<<<<<<<<<<<<<
 *                                          double score, double mz, Envelope envelope):
 *         cdef:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak._create_simple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":42
 * cdef class DeconvolutedPeak(PeakBase):
 *     cdef:
 *         public double neutral_mass             # <<<<<<<<<<<<<<
 *         # public double intensity
 *         public double signal_to_noise
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.neutral_mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 42, __pyx_L1_error)
  __pyx_v_self->neutral_mass = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.neutral_mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":44
 *         public double neutral_mass
 *         # public double intensity
 *         public double signal_to_noise             # <<<<<<<<<<<<<<
 *         public _Index _index
 *         public double full_width_at_half_max
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->signal_to_noise); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.signal_to_noise.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 44, __pyx_L1_error)
  __pyx_v_self->signal_to_noise = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.signal_to_noise.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":45
 *         # public double intensity
 *         public double signal_to_noise
 *         public _Index _index             # <<<<<<<<<<<<<<
 *         public double full_width_at_half_max
 *         public int charge
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_index));
  __pyx_r = ((PyObject *)__pyx_v_self->_index);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_12ms_deisotope_2_c_8peak_set__Index))))) __PYX_ERR(1, 45, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_index);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_index));
  __pyx_v_self->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak._index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_index);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_index));
  __pyx_v_self->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":46
 *         public double signal_to_noise
 *         public _Index _index
 *         public double full_width_at_half_max             # <<<<<<<<<<<<<<
 *         public int charge
 *         public double a_to_a2_ratio
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->full_width_at_half_max); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.full_width_at_half_max.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 46, __pyx_L1_error)
  __pyx_v_self->full_width_at_half_max = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.full_width_at_half_max.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":47
 *         public _Index _index
 *         public double full_width_at_half_max
 *         public int charge             # <<<<<<<<<<<<<<
 *         public double a_to_a2_ratio
 *         public double most_abundant_mass
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->charge); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.charge.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 47, __pyx_L1_error)
  __pyx_v_self->charge = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.charge.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":48
 *         public double full_width_at_half_max
 *         public int charge
 *         public double a_to_a2_ratio             # <<<<<<<<<<<<<<
 *         public double most_abundant_mass
 *         public double average_mass
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->a_to_a2_ratio); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.a_to_a2_ratio.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 48, __pyx_L1_error)
  __pyx_v_self->a_to_a2_ratio = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.a_to_a2_ratio.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":49
 *         public int charge
 *         public double a_to_a2_ratio
 *         public double most_abundant_mass             # <<<<<<<<<<<<<<
 *         public double average_mass
 *         public double score
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->most_abundant_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.most_abundant_mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 49, __pyx_L1_error)
  __pyx_v_self->most_abundant_mass = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.most_abundant_mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":50
 *         public double a_to_a2_ratio
 *         public double most_abundant_mass
 *         public double average_mass             # <<<<<<<<<<<<<<
 *         public double score
 *         # public double area
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->average_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.average_mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 50, __pyx_L1_error)
  __pyx_v_self->average_mass = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.average_mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":51
 *         public double most_abundant_mass
 *         public double average_mass
 *         public double score             # <<<<<<<<<<<<<<
 *         # public double area
 *         public Envelope envelope
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->score); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.score.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 51, __pyx_L1_error)
  __pyx_v_self->score = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.score.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":53
 *         public double score
 *         # public double area
 *         public Envelope envelope             # <<<<<<<<<<<<<<
 *         # public double mz
 *         public IsotopicFitRecord fit
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->envelope));
  __pyx_r = ((PyObject *)__pyx_v_self->envelope);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope))))) __PYX_ERR(1, 53, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->envelope);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->envelope));
  __pyx_v_self->envelope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.envelope.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->envelope);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->envelope));
  __pyx_v_self->envelope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":55
 *         public Envelope envelope
 *         # public double mz
 *         public IsotopicFitRecord fit             # <<<<<<<<<<<<<<
 *         public bint chosen_for_msms
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->fit));
  __pyx_r = ((PyObject *)__pyx_v_self->fit);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord))))) __PYX_ERR(1, 55, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->fit);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->fit));
  __pyx_v_self->fit = ((struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.fit.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->fit);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->fit));
  __pyx_v_self->fit = ((struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":56
 *         # public double mz
 *         public IsotopicFitRecord fit
 *         public bint chosen_for_msms             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other)
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->chosen_for_msms); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.chosen_for_msms.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 56, __pyx_L1_error)
  __pyx_v_self->chosen_for_msms = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.chosen_for_msms.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":456
 * 
 * cdef class IonMobilityDeconvolutedPeak(DeconvolutedPeak):
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,             # <<<<<<<<<<<<<<
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0, drift_time=0):
 */

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_neutral_mass = 0;
  PyObject *__pyx_v_intensity = 0;
  PyObject *__pyx_v_charge = 0;
  PyObject *__pyx_v_signal_to_noise = 0;
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_full_width_at_half_max = 0;
  PyObject *__pyx_v_a_to_a2_ratio = 0;
  PyObject *__pyx_v_most_abundant_mass = 0;
  PyObject *__pyx_v_average_mass = 0;
  PyObject *__pyx_v_score = 0;
  PyObject *__pyx_v_envelope = 0;
  PyObject *__pyx_v_mz = 0;
  PyObject *__pyx_v_fit = 0;
  PyObject *__pyx_v_chosen_for_msms = 0;
  PyObject *__pyx_v_area = 0;
  PyObject *__pyx_v_drift_time = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_intensity,&__pyx_n_s_charge,&__pyx_n_s_signal_to_noise,&__pyx_n_s_index,&__pyx_n_s_full_width_at_half_max,&__pyx_n_s_a_to_a2_ratio,&__pyx_n_s_most_abundant_mass,&__pyx_n_s_average_mass,&__pyx_n_s_score,&__pyx_n_s_envelope,&__pyx_n_s_mz,&__pyx_n_s_fit,&__pyx_n_s_chosen_for_msms,&__pyx_n_s_area,&__pyx_n_s_drift_time,0};
    PyObject* values[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    values[6] = ((PyObject *)__pyx_int_0);
    values[7] = ((PyObject *)__pyx_int_0);
    values[8] = ((PyObject *)__pyx_int_0);
    values[9] = ((PyObject *)__pyx_int_0);

    /* "ms_deisotope/_c/peak_set.pyx":458
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0, drift_time=0):             # <<<<<<<<<<<<<<
 *         super(IonMobilityDeconvolutedPeak, self).__init__(
 *             neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,
 */
    values[10] = ((PyObject *)__pyx_empty_tuple);
    values[11] = ((PyObject *)__pyx_int_0);
    values[12] = ((PyObject *)Py_None);
    values[13] = ((PyObject *)Py_False);
    values[14] = ((PyObject *)__pyx_int_0);
    values[15] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 16: values[15] = PyTuple_GET_ITEM(__pyx_args, 15);
        CYTHON_FALLTHROUGH;
        case 15: values[14] = PyTuple_GET_ITEM(__pyx_args, 14);
        CYTHON_FALLTHROUGH;
        case 14: values[13] = PyTuple_GET_ITEM(__pyx_args, 13);
        CYTHON_FALLTHROUGH;
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_intensity)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 16, 1); __PYX_ERR(0, 456, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_charge)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 16, 2); __PYX_ERR(0, 456, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_signal_to_noise)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 16, 3); __PYX_ERR(0, 456, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 16, 4); __PYX_ERR(0, 456, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_full_width_at_half_max)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 16, 5); __PYX_ERR(0, 456, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_a_to_a2_ratio);
          if (value) { values[6] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_most_abundant_mass);
          if (value) { values[7] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_average_mass);
          if (value) { values[8] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_score);
          if (value) { values[9] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_envelope);
          if (value) { values[10] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 11:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mz);
          if (value) { values[11] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 12:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fit);
          if (value) { values[12] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 13:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_chosen_for_msms);
          if (value) { values[13] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 14:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_area);
          if (value) { values[14] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 15:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_drift_time);
          if (value) { values[15] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 456, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case 16: values[15] = PyTuple_GET_ITEM(__pyx_args, 15);
        CYTHON_FALLTHROUGH;
        case 15: values[14] = PyTuple_GET_ITEM(__pyx_args, 14);
        CYTHON_FALLTHROUGH;
        case 14: values[13] = PyTuple_GET_ITEM(__pyx_args, 13);
        CYTHON_FALLTHROUGH;
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_neutral_mass = values[0];
    __pyx_v_intensity = values[1];
    __pyx_v_charge = values[2];
    __pyx_v_signal_to_noise = values[3];
    __pyx_v_index = values[4];
    __pyx_v_full_width_at_half_max = values[5];
    __pyx_v_a_to_a2_ratio = values[6];
    __pyx_v_most_abundant_mass = values[7];
    __pyx_v_average_mass = values[8];
    __pyx_v_score = values[9];
    __pyx_v_envelope = values[10];
    __pyx_v_mz = values[11];
    __pyx_v_fit = values[12];
    __pyx_v_chosen_for_msms = values[13];
    __pyx_v_area = values[14];
    __pyx_v_drift_time = values[15];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 16, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 456, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_intensity, __pyx_v_charge, __pyx_v_signal_to_noise, __pyx_v_index, __pyx_v_full_width_at_half_max, __pyx_v_a_to_a2_ratio, __pyx_v_most_abundant_mass, __pyx_v_average_mass, __pyx_v_score, __pyx_v_envelope, __pyx_v_mz, __pyx_v_fit, __pyx_v_chosen_for_msms, __pyx_v_area, __pyx_v_drift_time);

  /* "ms_deisotope/_c/peak_set.pyx":456
 * 
 * cdef class IonMobilityDeconvolutedPeak(DeconvolutedPeak):
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,             # <<<<<<<<<<<<<<
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0, drift_time=0):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_intensity, PyObject *__pyx_v_charge, PyObject *__pyx_v_signal_to_noise, PyObject *__pyx_v_index, PyObject *__pyx_v_full_width_at_half_max, PyObject *__pyx_v_a_to_a2_ratio, PyObject *__pyx_v_most_abundant_mass, PyObject *__pyx_v_average_mass, PyObject *__pyx_v_score, PyObject *__pyx_v_envelope, PyObject *__pyx_v_mz, PyObject *__pyx_v_fit, PyObject *__pyx_v_chosen_for_msms, PyObject *__pyx_v_area, PyObject *__pyx_v_drift_time) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  double __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":459
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0, drift_time=0):
 *         super(IonMobilityDeconvolutedPeak, self).__init__(             # <<<<<<<<<<<<<<
 *             neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,
 *             a_to_a2_ratio, most_abundant_mass, average_mass, score,
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":462
 *             neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,
 *             a_to_a2_ratio, most_abundant_mass, average_mass, score,
 *             envelope, mz, fit, chosen_for_msms, area)             # <<<<<<<<<<<<<<
 *         self.drift_time = drift_time
 * 
 */
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[16] = {__pyx_t_3, __pyx_v_neutral_mass, __pyx_v_intensity, __pyx_v_charge, __pyx_v_signal_to_noise, __pyx_v_index, __pyx_v_full_width_at_half_max, __pyx_v_a_to_a2_ratio, __pyx_v_most_abundant_mass, __pyx_v_average_mass, __pyx_v_score, __pyx_v_envelope, __pyx_v_mz, __pyx_v_fit, __pyx_v_chosen_for_msms, __pyx_v_area};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 15+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[16] = {__pyx_t_3, __pyx_v_neutral_mass, __pyx_v_intensity, __pyx_v_charge, __pyx_v_signal_to_noise, __pyx_v_index, __pyx_v_full_width_at_half_max, __pyx_v_a_to_a2_ratio, __pyx_v_most_abundant_mass, __pyx_v_average_mass, __pyx_v_score, __pyx_v_envelope, __pyx_v_mz, __pyx_v_fit, __pyx_v_chosen_for_msms, __pyx_v_area};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 15+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(15+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_neutral_mass);
    __Pyx_GIVEREF(__pyx_v_neutral_mass);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_neutral_mass);
    __Pyx_INCREF(__pyx_v_intensity);
    __Pyx_GIVEREF(__pyx_v_intensity);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_intensity);
    __Pyx_INCREF(__pyx_v_charge);
    __Pyx_GIVEREF(__pyx_v_charge);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, __pyx_v_charge);
    __Pyx_INCREF(__pyx_v_signal_to_noise);
    __Pyx_GIVEREF(__pyx_v_signal_to_noise);
    PyTuple_SET_ITEM(__pyx_t_5, 3+__pyx_t_4, __pyx_v_signal_to_noise);
    __Pyx_INCREF(__pyx_v_index);
    __Pyx_GIVEREF(__pyx_v_index);
    PyTuple_SET_ITEM(__pyx_t_5, 4+__pyx_t_4, __pyx_v_index);
    __Pyx_INCREF(__pyx_v_full_width_at_half_max);
    __Pyx_GIVEREF(__pyx_v_full_width_at_half_max);
    PyTuple_SET_ITEM(__pyx_t_5, 5+__pyx_t_4, __pyx_v_full_width_at_half_max);
    __Pyx_INCREF(__pyx_v_a_to_a2_ratio);
    __Pyx_GIVEREF(__pyx_v_a_to_a2_ratio);
    PyTuple_SET_ITEM(__pyx_t_5, 6+__pyx_t_4, __pyx_v_a_to_a2_ratio);
    __Pyx_INCREF(__pyx_v_most_abundant_mass);
    __Pyx_GIVEREF(__pyx_v_most_abundant_mass);
    PyTuple_SET_ITEM(__pyx_t_5, 7+__pyx_t_4, __pyx_v_most_abundant_mass);
    __Pyx_INCREF(__pyx_v_average_mass);
    __Pyx_GIVEREF(__pyx_v_average_mass);
    PyTuple_SET_ITEM(__pyx_t_5, 8+__pyx_t_4, __pyx_v_average_mass);
    __Pyx_INCREF(__pyx_v_score);
    __Pyx_GIVEREF(__pyx_v_score);
    PyTuple_SET_ITEM(__pyx_t_5, 9+__pyx_t_4, __pyx_v_score);
    __Pyx_INCREF(__pyx_v_envelope);
    __Pyx_GIVEREF(__pyx_v_envelope);
    PyTuple_SET_ITEM(__pyx_t_5, 10+__pyx_t_4, __pyx_v_envelope);
    __Pyx_INCREF(__pyx_v_mz);
    __Pyx_GIVEREF(__pyx_v_mz);
    PyTuple_SET_ITEM(__pyx_t_5, 11+__pyx_t_4, __pyx_v_mz);
    __Pyx_INCREF(__pyx_v_fit);
    __Pyx_GIVEREF(__pyx_v_fit);
    PyTuple_SET_ITEM(__pyx_t_5, 12+__pyx_t_4, __pyx_v_fit);
    __Pyx_INCREF(__pyx_v_chosen_for_msms);
    __Pyx_GIVEREF(__pyx_v_chosen_for_msms);
    PyTuple_SET_ITEM(__pyx_t_5, 13+__pyx_t_4, __pyx_v_chosen_for_msms);
    __Pyx_INCREF(__pyx_v_area);
    __Pyx_GIVEREF(__pyx_v_area);
    PyTuple_SET_ITEM(__pyx_t_5, 14+__pyx_t_4, __pyx_v_area);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":463
 *             a_to_a2_ratio, most_abundant_mass, average_mass, score,
 *             envelope, mz, fit, chosen_for_msms, area)
 *         self.drift_time = drift_time             # <<<<<<<<<<<<<<
 * 
 *     cpdef PeakBase clone(self):
 */
  __pyx_t_6 = __pyx_PyFloat_AsDouble(__pyx_v_drift_time); if (unlikely((__pyx_t_6 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 463, __pyx_L1_error)
  __pyx_v_self->drift_time = __pyx_t_6;

  /* "ms_deisotope/_c/peak_set.pyx":456
 * 
 * cdef class IonMobilityDeconvolutedPeak(DeconvolutedPeak):
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,             # <<<<<<<<<<<<<<
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0, drift_time=0):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":465
 *         self.drift_time = drift_time
 * 
 *     cpdef PeakBase clone(self):             # <<<<<<<<<<<<<<
 *         return IonMobilityDeconvolutedPeak(
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_3clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_3clone)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 465, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase))))) __PYX_ERR(0, 465, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":466
 * 
 *     cpdef PeakBase clone(self):
 *         return IonMobilityDeconvolutedPeak(             # <<<<<<<<<<<<<<
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));

  /* "ms_deisotope/_c/peak_set.pyx":467
 *     cpdef PeakBase clone(self):
 *         return IonMobilityDeconvolutedPeak(
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.signal_to_noise); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "ms_deisotope/_c/peak_set.pyx":468
 *         return IonMobilityDeconvolutedPeak(
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,             # <<<<<<<<<<<<<<
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.full_width_at_half_max); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.a_to_a2_ratio); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "ms_deisotope/_c/peak_set.pyx":469
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,             # <<<<<<<<<<<<<<
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,
 *             self.drift_time)
 */
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.most_abundant_mass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.average_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.score); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);

  /* "ms_deisotope/_c/peak_set.pyx":470
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,             # <<<<<<<<<<<<<<
 *             self.drift_time)
 * 
 */
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.mz); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.chosen_for_msms); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.area); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);

  /* "ms_deisotope/_c/peak_set.pyx":471
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,
 *             self.drift_time)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */
  __pyx_t_14 = PyFloat_FromDouble(__pyx_v_self->drift_time); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);

  /* "ms_deisotope/_c/peak_set.pyx":466
 * 
 *     cpdef PeakBase clone(self):
 *         return IonMobilityDeconvolutedPeak(             # <<<<<<<<<<<<<<
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */
  __pyx_t_15 = PyTuple_New(16); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_15, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_15, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_15, 3, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_15, 4, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_15, 5, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_15, 6, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_15, 7, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_15, 8, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_15, 9, __pyx_t_10);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));
  PyTuple_SET_ITEM(__pyx_t_15, 10, ((PyObject *)__pyx_v_self->__pyx_base.envelope));
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_15, 11, __pyx_t_11);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.fit));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.fit));
  PyTuple_SET_ITEM(__pyx_t_15, 12, ((PyObject *)__pyx_v_self->__pyx_base.fit));
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_15, 13, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_15, 14, __pyx_t_13);
  __Pyx_GIVEREF(__pyx_t_14);
  PyTuple_SET_ITEM(__pyx_t_15, 15, __pyx_t_14);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_14 = 0;
  __pyx_t_14 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak), __pyx_t_15, NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  __pyx_r = ((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_t_14);
  __pyx_t_14 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":465
 *         self.drift_time = drift_time
 * 
 *     cpdef PeakBase clone(self):             # <<<<<<<<<<<<<<
 *         return IonMobilityDeconvolutedPeak(
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_3clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_2clone[] = "IonMobilityDeconvolutedPeak.clone(self) -> PeakBase";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_3clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_2clone(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_2clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_clone(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":473
 *             self.drift_time)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return IonMobilityDeconvolutedPeak, (
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_4__reduce__[] = "IonMobilityDeconvolutedPeak.__reduce__(self)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_4__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_4__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":474
 * 
 *     def __reduce__(self):
 *         return IonMobilityDeconvolutedPeak, (             # <<<<<<<<<<<<<<
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/peak_set.pyx":475
 *     def __reduce__(self):
 *         return IonMobilityDeconvolutedPeak, (
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.signal_to_noise); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "ms_deisotope/_c/peak_set.pyx":476
 *         return IonMobilityDeconvolutedPeak, (
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,             # <<<<<<<<<<<<<<
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.full_width_at_half_max); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.a_to_a2_ratio); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "ms_deisotope/_c/peak_set.pyx":477
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,             # <<<<<<<<<<<<<<
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,
 *             self.drift_time)
 */
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.most_abundant_mass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.average_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.score); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);

  /* "ms_deisotope/_c/peak_set.pyx":478
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,             # <<<<<<<<<<<<<<
 *             self.drift_time)
 * 
 */
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.mz); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 478, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.chosen_for_msms); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 478, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.area); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 478, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);

  /* "ms_deisotope/_c/peak_set.pyx":479
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,
 *             self.drift_time)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):
 */
  __pyx_t_14 = PyFloat_FromDouble(__pyx_v_self->drift_time); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 479, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);

  /* "ms_deisotope/_c/peak_set.pyx":475
 *     def __reduce__(self):
 *         return IonMobilityDeconvolutedPeak, (
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 */
  __pyx_t_15 = PyTuple_New(16); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_15, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_15, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_15, 3, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_15, 4, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_15, 5, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_15, 6, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_15, 7, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_15, 8, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_15, 9, __pyx_t_10);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));
  PyTuple_SET_ITEM(__pyx_t_15, 10, ((PyObject *)__pyx_v_self->__pyx_base.envelope));
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_15, 11, __pyx_t_11);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.fit));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.fit));
  PyTuple_SET_ITEM(__pyx_t_15, 12, ((PyObject *)__pyx_v_self->__pyx_base.fit));
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_15, 13, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_15, 14, __pyx_t_13);
  __Pyx_GIVEREF(__pyx_t_14);
  PyTuple_SET_ITEM(__pyx_t_15, 15, __pyx_t_14);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_14 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":474
 * 
 *     def __reduce__(self):
 *         return IonMobilityDeconvolutedPeak, (             # <<<<<<<<<<<<<<
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */
  __pyx_t_14 = PyTuple_New(2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak));
  PyTuple_SET_ITEM(__pyx_t_14, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak));
  __Pyx_GIVEREF(__pyx_t_15);
  PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_15);
  __pyx_t_15 = 0;
  __pyx_r = __pyx_t_14;
  __pyx_t_14 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":473
 *             self.drift_time)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return IonMobilityDeconvolutedPeak, (
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":481
 *             self.drift_time)
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):             # <<<<<<<<<<<<<<
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_7_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_eq", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_eq); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_7_eq)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_other)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 481, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 481, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":482
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (             # <<<<<<<<<<<<<<
 *             abs(self.intensity - other.intensity) < epsilon) and (
 *             self.charge == other.charge) and (
 */
  __pyx_t_6 = ((fabs((__pyx_v_self->__pyx_base.neutral_mass - __pyx_v_other->neutral_mass)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }

  /* "ms_deisotope/_c/peak_set.pyx":483
 *     cpdef bint _eq(self, DeconvolutedPeak other):
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (             # <<<<<<<<<<<<<<
 *             self.charge == other.charge) and (
 *             abs(self.score - other.score) < epsilon) and (
 */
  __pyx_t_6 = ((fabs((__pyx_v_self->__pyx_base.__pyx_base.intensity - __pyx_v_other->__pyx_base.intensity)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }

  /* "ms_deisotope/_c/peak_set.pyx":484
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (
 *             self.charge == other.charge) and (             # <<<<<<<<<<<<<<
 *             abs(self.score - other.score) < epsilon) and (
 *             abs(self.drift_time - other.drift_time) < epsilon)
 */
  __pyx_t_6 = ((__pyx_v_self->__pyx_base.charge == __pyx_v_other->charge) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }

  /* "ms_deisotope/_c/peak_set.pyx":485
 *             abs(self.intensity - other.intensity) < epsilon) and (
 *             self.charge == other.charge) and (
 *             abs(self.score - other.score) < epsilon) and (             # <<<<<<<<<<<<<<
 *             abs(self.drift_time - other.drift_time) < epsilon)
 * 
 */
  __pyx_t_6 = ((fabs((__pyx_v_self->__pyx_base.score - __pyx_v_other->score)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }

  /* "ms_deisotope/_c/peak_set.pyx":486
 *             self.charge == other.charge) and (
 *             abs(self.score - other.score) < epsilon) and (
 *             abs(self.drift_time - other.drift_time) < epsilon)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->drift_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_other), __pyx_n_s_drift_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyNumber_Absolute(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_12ms_deisotope_2_c_8peak_set_epsilon); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_t_3, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __pyx_t_6;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":481
 *             self.drift_time)
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):             # <<<<<<<<<<<<<<
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak._eq", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_7_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_6_eq[] = "IonMobilityDeconvolutedPeak._eq(self, DeconvolutedPeak other) -> bool";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_7_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_eq (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, 1, "other", 0))) __PYX_ERR(0, 481, __pyx_L1_error)
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_6_eq(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self), ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_other));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_6_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_eq", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak__eq(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak._eq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":488
 *             abs(self.drift_time - other.drift_time) < epsilon)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return ("{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, "
 *             "average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, "
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_9__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_9__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_8__repr__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_8__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":489
 * 
 *     def __repr__(self):
 *         return ("{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, "             # <<<<<<<<<<<<<<
 *             "average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, "
 *             "envelope={self.envelope}, full_width_at_half_max={self.full_width_at_half_max}, index={self.index}, "
 */
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/peak_set.pyx":494
 *             "intensity={self.intensity}, most_abundant_mass={self.most_abundant_mass}, mz={self.mz}, "
 *             "neutral_mass={self.neutral_mass}, score={self.score}, signal_to_noise={self.signal_to_noise},"
 *             "drift_time={self.drift_time})").format(self=self)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_self___class_____name___a_to_a2_2, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_self, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 494, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":488
 *             abs(self.drift_time - other.drift_time) < epsilon)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return ("{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, "
 *             "average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, "
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":68
 * cdef class IonMobilityDeconvolutedPeak(DeconvolutedPeak):
 *     cdef:
 *         public double drift_time             # <<<<<<<<<<<<<<
 * 
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->drift_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.drift_time.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 68, __pyx_L1_error)
  __pyx_v_self->drift_time = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.drift_time.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":510
 *         Representation of the "source" of the isotopic fit
 *     """
 *     def __init__(self, solution, fit, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         self.solution = solution
 *         super(DeconvolutedPeakSolution, self).__init__(*args, **kwargs)
 */

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_solution = 0;
  PyObject *__pyx_v_fit = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  if (PyTuple_GET_SIZE(__pyx_args) > 2) {
    __pyx_v_args = PyTuple_GetSlice(__pyx_args, 2, PyTuple_GET_SIZE(__pyx_args));
    if (unlikely(!__pyx_v_args)) {
      __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
      __Pyx_RefNannyFinishContext();
      return -1;
    }
    __Pyx_GOTREF(__pyx_v_args);
  } else {
    __pyx_v_args = __pyx_empty_tuple; __Pyx_INCREF(__pyx_empty_tuple);
  }
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_solution,&__pyx_n_s_fit,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        default:
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_solution)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fit)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 2, 1); __PYX_ERR(0, 510, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t used_pos_args = (pos_args < 2) ? pos_args : 2;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, used_pos_args, "__init__") < 0)) __PYX_ERR(0, 510, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) < 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_solution = values[0];
    __pyx_v_fit = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 510, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_args); __pyx_v_args = 0;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self), __pyx_v_solution, __pyx_v_fit, __pyx_v_args, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self, PyObject *__pyx_v_solution, PyObject *__pyx_v_fit, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":511
 *     """
 *     def __init__(self, solution, fit, *args, **kwargs):
 *         self.solution = solution             # <<<<<<<<<<<<<<
 *         super(DeconvolutedPeakSolution, self).__init__(*args, **kwargs)
 *         self.fit = fit
 */
  __Pyx_INCREF(__pyx_v_solution);
  __Pyx_GIVEREF(__pyx_v_solution);
  __Pyx_GOTREF(__pyx_v_self->solution);
  __Pyx_DECREF(__pyx_v_self->solution);
  __pyx_v_self->solution = __pyx_v_solution;

  /* "ms_deisotope/_c/peak_set.pyx":512
 *     def __init__(self, solution, fit, *args, **kwargs):
 *         self.solution = solution
 *         super(DeconvolutedPeakSolution, self).__init__(*args, **kwargs)             # <<<<<<<<<<<<<<
 *         self.fit = fit
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_v_args, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":513
 *         self.solution = solution
 *         super(DeconvolutedPeakSolution, self).__init__(*args, **kwargs)
 *         self.fit = fit             # <<<<<<<<<<<<<<
 * 
 *     cpdef PeakBase clone(self):
 */
  if (!(likely(((__pyx_v_fit) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_fit, __pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord))))) __PYX_ERR(0, 513, __pyx_L1_error)
  __pyx_t_3 = __pyx_v_fit;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.fit);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.fit));
  __pyx_v_self->__pyx_base.fit = ((struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":510
 *         Representation of the "source" of the isotopic fit
 *     """
 *     def __init__(self, solution, fit, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         self.solution = solution
 *         super(DeconvolutedPeakSolution, self).__init__(*args, **kwargs)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":515
 *         self.fit = fit
 * 
 *     cpdef PeakBase clone(self):             # <<<<<<<<<<<<<<
 *         return DeconvolutedPeakSolution(
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_3clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_f_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 515, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_3clone)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 515, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase))))) __PYX_ERR(0, 515, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":516
 * 
 *     cpdef PeakBase clone(self):
 *         return DeconvolutedPeakSolution(             # <<<<<<<<<<<<<<
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));

  /* "ms_deisotope/_c/peak_set.pyx":517
 *     cpdef PeakBase clone(self):
 *         return DeconvolutedPeakSolution(
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.signal_to_noise); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "ms_deisotope/_c/peak_set.pyx":518
 *         return DeconvolutedPeakSolution(
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,             # <<<<<<<<<<<<<<
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.full_width_at_half_max); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.a_to_a2_ratio); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "ms_deisotope/_c/peak_set.pyx":519
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,             # <<<<<<<<<<<<<<
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 */
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.most_abundant_mass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.average_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.score); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);

  /* "ms_deisotope/_c/peak_set.pyx":520
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.mz); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.chosen_for_msms); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.area); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);

  /* "ms_deisotope/_c/peak_set.pyx":516
 * 
 *     cpdef PeakBase clone(self):
 *         return DeconvolutedPeakSolution(             # <<<<<<<<<<<<<<
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */
  __pyx_t_14 = PyTuple_New(16); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_INCREF(__pyx_v_self->solution);
  __Pyx_GIVEREF(__pyx_v_self->solution);
  PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_v_self->solution);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.fit));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.fit));
  PyTuple_SET_ITEM(__pyx_t_14, 1, ((PyObject *)__pyx_v_self->__pyx_base.fit));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_14, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_14, 4, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_14, 5, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_14, 6, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_14, 7, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_14, 8, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_14, 9, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_14, 10, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_14, 11, __pyx_t_10);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));
  PyTuple_SET_ITEM(__pyx_t_14, 12, ((PyObject *)__pyx_v_self->__pyx_base.envelope));
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_14, 13, __pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_14, 14, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_14, 15, __pyx_t_13);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution), __pyx_t_14, NULL); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_r = ((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_t_13);
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":515
 *         self.fit = fit
 * 
 *     cpdef PeakBase clone(self):             # <<<<<<<<<<<<<<
 *         return DeconvolutedPeakSolution(
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_3clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_2clone[] = "DeconvolutedPeakSolution.clone(self) -> PeakBase";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_3clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_2clone(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_2clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_clone(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":522
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return DeconvolutedPeakSolution, (
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_4__reduce__[] = "DeconvolutedPeakSolution.__reduce__(self)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_4__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_4__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":523
 * 
 *     def __reduce__(self):
 *         return DeconvolutedPeakSolution, (             # <<<<<<<<<<<<<<
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/peak_set.pyx":524
 *     def __reduce__(self):
 *         return DeconvolutedPeakSolution, (
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.signal_to_noise); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "ms_deisotope/_c/peak_set.pyx":525
 *         return DeconvolutedPeakSolution, (
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,             # <<<<<<<<<<<<<<
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 525, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.full_width_at_half_max); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 525, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.a_to_a2_ratio); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 525, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "ms_deisotope/_c/peak_set.pyx":526
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,             # <<<<<<<<<<<<<<
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 */
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.most_abundant_mass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.average_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.score); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);

  /* "ms_deisotope/_c/peak_set.pyx":527
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.mz); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.chosen_for_msms); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.area); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);

  /* "ms_deisotope/_c/peak_set.pyx":524
 *     def __reduce__(self):
 *         return DeconvolutedPeakSolution, (
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 */
  __pyx_t_14 = PyTuple_New(16); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_INCREF(__pyx_v_self->solution);
  __Pyx_GIVEREF(__pyx_v_self->solution);
  PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_v_self->solution);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.fit));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.fit));
  PyTuple_SET_ITEM(__pyx_t_14, 1, ((PyObject *)__pyx_v_self->__pyx_base.fit));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_14, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_14, 4, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_14, 5, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_14, 6, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_14, 7, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_14, 8, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_14, 9, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_14, 10, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_14, 11, __pyx_t_10);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));
  PyTuple_SET_ITEM(__pyx_t_14, 12, ((PyObject *)__pyx_v_self->__pyx_base.envelope));
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_14, 13, __pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_14, 14, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_14, 15, __pyx_t_13);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":523
 * 
 *     def __reduce__(self):
 *         return DeconvolutedPeakSolution, (             # <<<<<<<<<<<<<<
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */
  __pyx_t_13 = PyTuple_New(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution));
  PyTuple_SET_ITEM(__pyx_t_13, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution));
  __Pyx_GIVEREF(__pyx_t_14);
  PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_t_14);
  __pyx_t_14 = 0;
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":522
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return DeconvolutedPeakSolution, (
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "ms_deisotope/_c/peak_set.pyx":529
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.solution
 *         yield self
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_7__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_7__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_6__iter__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_6__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__(__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 529, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8generator1, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_iter, __pyx_n_s_DeconvolutedPeakSolution___iter, __pyx_n_s_ms_deisotope__c_peak_set); if (unlikely(!gen)) __PYX_ERR(0, 529, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *__pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L4_resume_from_yield;
    case 2: goto __pyx_L5_resume_from_yield;
    case 3: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 529, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":530
 * 
 *     def __iter__(self):
 *         yield self.solution             # <<<<<<<<<<<<<<
 *         yield self
 *         yield self.fit
 */
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->solution);
  __pyx_r = __pyx_cur_scope->__pyx_v_self->solution;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  /* return from generator, yielding value */
  __pyx_generator->resume_label = 1;
  return __pyx_r;
  __pyx_L4_resume_from_yield:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 530, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":531
 *     def __iter__(self):
 *         yield self.solution
 *         yield self             # <<<<<<<<<<<<<<
 *         yield self.fit
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  /* return from generator, yielding value */
  __pyx_generator->resume_label = 2;
  return __pyx_r;
  __pyx_L5_resume_from_yield:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 531, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":532
 *         yield self.solution
 *         yield self
 *         yield self.fit             # <<<<<<<<<<<<<<
 * 
 *     @classmethod
 */
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_v_self->__pyx_base.fit));
  __pyx_r = ((PyObject *)__pyx_cur_scope->__pyx_v_self->__pyx_base.fit);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  /* return from generator, yielding value */
  __pyx_generator->resume_label = 3;
  return __pyx_r;
  __pyx_L6_resume_from_yield:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 532, __pyx_L1_error)
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "ms_deisotope/_c/peak_set.pyx":529
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.solution
 *         yield self
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":535
 * 
 *     @classmethod
 *     def from_peak(cls, peak, solution):             # <<<<<<<<<<<<<<
 *         inst = cls(
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_10from_peak(PyObject *__pyx_v_cls, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_9from_peak[] = "DeconvolutedPeakSolution.from_peak(type cls, peak, solution)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_10from_peak(PyObject *__pyx_v_cls, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_peak = 0;
  PyObject *__pyx_v_solution = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("from_peak (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_peak,&__pyx_n_s_solution,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_peak)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_solution)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("from_peak", 1, 2, 2, 1); __PYX_ERR(0, 535, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "from_peak") < 0)) __PYX_ERR(0, 535, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_peak = values[0];
    __pyx_v_solution = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("from_peak", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 535, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.from_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_9from_peak(((PyTypeObject*)__pyx_v_cls), __pyx_v_peak, __pyx_v_solution);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_9from_peak(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_peak, PyObject *__pyx_v_solution) {
  PyObject *__pyx_v_inst = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("from_peak", 0);

  /* "ms_deisotope/_c/peak_set.pyx":537
 *     def from_peak(cls, peak, solution):
 *         inst = cls(
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,             # <<<<<<<<<<<<<<
 *             peak.charge, peak.signal_to_noise, peak.index, peak.full_width_at_half_max,
 *             peak.a_to_a2_ratio, peak.most_abundant_mass, peak.average_mass,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_fit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_neutral_mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_intensity); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ms_deisotope/_c/peak_set.pyx":538
 *         inst = cls(
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,
 *             peak.charge, peak.signal_to_noise, peak.index, peak.full_width_at_half_max,             # <<<<<<<<<<<<<<
 *             peak.a_to_a2_ratio, peak.most_abundant_mass, peak.average_mass,
 *             peak.score, peak.envelope, peak.mz, peak.chosen_for_msms, peak.area)
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_charge); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_signal_to_noise); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_index); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_full_width_at_half_max); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "ms_deisotope/_c/peak_set.pyx":539
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,
 *             peak.charge, peak.signal_to_noise, peak.index, peak.full_width_at_half_max,
 *             peak.a_to_a2_ratio, peak.most_abundant_mass, peak.average_mass,             # <<<<<<<<<<<<<<
 *             peak.score, peak.envelope, peak.mz, peak.chosen_for_msms, peak.area)
 *         return inst
 */
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_a_to_a2_ratio); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_most_abundant_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_average_mass); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);

  /* "ms_deisotope/_c/peak_set.pyx":540
 *             peak.charge, peak.signal_to_noise, peak.index, peak.full_width_at_half_max,
 *             peak.a_to_a2_ratio, peak.most_abundant_mass, peak.average_mass,
 *             peak.score, peak.envelope, peak.mz, peak.chosen_for_msms, peak.area)             # <<<<<<<<<<<<<<
 *         return inst
 * 
 */
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_score); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_envelope); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_mz); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_chosen_for_msms); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_area); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);

  /* "ms_deisotope/_c/peak_set.pyx":536
 *     @classmethod
 *     def from_peak(cls, peak, solution):
 *         inst = cls(             # <<<<<<<<<<<<<<
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,
 *             peak.charge, peak.signal_to_noise, peak.index, peak.full_width_at_half_max,
 */
  __pyx_t_16 = PyTuple_New(16); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_INCREF(__pyx_v_solution);
  __Pyx_GIVEREF(__pyx_v_solution);
  PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_v_solution);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_16, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_16, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_16, 3, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_16, 4, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_16, 5, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_16, 6, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_16, 7, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_16, 8, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_16, 9, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_16, 10, __pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_16, 11, __pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_16, 12, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_16, 13, __pyx_t_13);
  __Pyx_GIVEREF(__pyx_t_14);
  PyTuple_SET_ITEM(__pyx_t_16, 14, __pyx_t_14);
  __Pyx_GIVEREF(__pyx_t_15);
  PyTuple_SET_ITEM(__pyx_t_16, 15, __pyx_t_15);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_14 = 0;
  __pyx_t_15 = 0;
  __pyx_t_15 = __Pyx_PyObject_Call(((PyObject *)__pyx_v_cls), __pyx_t_16, NULL); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __pyx_v_inst = __pyx_t_15;
  __pyx_t_15 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":541
 *             peak.a_to_a2_ratio, peak.most_abundant_mass, peak.average_mass,
 *             peak.score, peak.envelope, peak.mz, peak.chosen_for_msms, peak.area)
 *         return inst             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_inst);
  __pyx_r = __pyx_v_inst;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":535
 * 
 *     @classmethod
 *     def from_peak(cls, peak, solution):             # <<<<<<<<<<<<<<
 *         inst = cls(
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.from_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_inst);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":73
 * cdef class DeconvolutedPeakSolution(DeconvolutedPeak):
 *     cdef:
 *         public object solution             # <<<<<<<<<<<<<<
 * 
 * cdef class DeconvolutedPeakSet(object):
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->solution);
  __pyx_r = __pyx_v_self->solution;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->solution);
  __Pyx_DECREF(__pyx_v_self->solution);
  __pyx_v_self->solution = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->solution);
  __Pyx_DECREF(__pyx_v_self->solution);
  __pyx_v_self->solution = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":555
 *         Collection of peaks ordered by `mz`
 *     """
 *     def __init__(self, peaks):             # <<<<<<<<<<<<<<
 *         self.peaks = tuple(peaks)
 *         self._mz_ordered = None
 */

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_peaks = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_peaks,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_peaks)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 555, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_peaks = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 555, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_peaks);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_peaks) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":556
 *     """
 *     def __init__(self, peaks):
 *         self.peaks = tuple(peaks)             # <<<<<<<<<<<<<<
 *         self._mz_ordered = None
 *         self.indexed = False
 */
  __pyx_t_1 = __Pyx_PySequence_Tuple(__pyx_v_peaks); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->peaks);
  __Pyx_DECREF(__pyx_v_self->peaks);
  __pyx_v_self->peaks = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":557
 *     def __init__(self, peaks):
 *         self.peaks = tuple(peaks)
 *         self._mz_ordered = None             # <<<<<<<<<<<<<<
 *         self.indexed = False
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_mz_ordered);
  __Pyx_DECREF(__pyx_v_self->_mz_ordered);
  __pyx_v_self->_mz_ordered = ((PyObject*)Py_None);

  /* "ms_deisotope/_c/peak_set.pyx":558
 *         self.peaks = tuple(peaks)
 *         self._mz_ordered = None
 *         self.indexed = False             # <<<<<<<<<<<<<<
 * 
 *     cpdef reindex(self):
 */
  __pyx_v_self->indexed = 0;

  /* "ms_deisotope/_c/peak_set.pyx":555
 *         Collection of peaks ordered by `mz`
 *     """
 *     def __init__(self, peaks):             # <<<<<<<<<<<<<<
 *         self.peaks = tuple(peaks)
 *         self._mz_ordered = None
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":560
 *         self.indexed = False
 * 
 *     cpdef reindex(self):             # <<<<<<<<<<<<<<
 *         """
 *         Updates the :attr:`index` of each peak in `self` and updates the
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_3reindex(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, int __pyx_skip_dispatch) {
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  PyObject *__pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("reindex", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_reindex); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 560, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_3reindex)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 560, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":572
 *             size_t i, n
 *             DeconvolutedPeak peak
 *         self.peaks = tuple(sorted(self.peaks, key=neutral_mass_getter))             # <<<<<<<<<<<<<<
 *         self._mz_ordered = tuple(sorted(self.peaks, key=mz_getter))
 *         n = PyTuple_GET_SIZE(self.peaks)
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->peaks);
  __Pyx_GIVEREF(__pyx_v_self->peaks);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->peaks);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_key, __pyx_v_12ms_deisotope_2_c_8peak_set_neutral_mass_getter) < 0) __PYX_ERR(0, 572, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_sorted, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PySequence_Tuple(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->peaks);
  __Pyx_DECREF(__pyx_v_self->peaks);
  __pyx_v_self->peaks = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":573
 *             DeconvolutedPeak peak
 *         self.peaks = tuple(sorted(self.peaks, key=neutral_mass_getter))
 *         self._mz_ordered = tuple(sorted(self.peaks, key=mz_getter))             # <<<<<<<<<<<<<<
 *         n = PyTuple_GET_SIZE(self.peaks)
 *         i = 0
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->peaks);
  __Pyx_GIVEREF(__pyx_v_self->peaks);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->peaks);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_key, __pyx_v_12ms_deisotope_2_c_8peak_set_mz_getter) < 0) __PYX_ERR(0, 573, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_sorted, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PySequence_Tuple(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_mz_ordered);
  __Pyx_DECREF(__pyx_v_self->_mz_ordered);
  __pyx_v_self->_mz_ordered = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":574
 *         self.peaks = tuple(sorted(self.peaks, key=neutral_mass_getter))
 *         self._mz_ordered = tuple(sorted(self.peaks, key=mz_getter))
 *         n = PyTuple_GET_SIZE(self.peaks)             # <<<<<<<<<<<<<<
 *         i = 0
 *         for i in range(n):
 */
  __pyx_t_3 = __pyx_v_self->peaks;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_n = PyTuple_GET_SIZE(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":575
 *         self._mz_ordered = tuple(sorted(self.peaks, key=mz_getter))
 *         n = PyTuple_GET_SIZE(self.peaks)
 *         i = 0             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             peak = self.getitem(i)
 */
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":576
 *         n = PyTuple_GET_SIZE(self.peaks)
 *         i = 0
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             peak = self.getitem(i)
 *             peak._index = _Index._create(0, 0)
 */
  __pyx_t_5 = __pyx_v_n;
  __pyx_t_6 = __pyx_t_5;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "ms_deisotope/_c/peak_set.pyx":577
 *         i = 0
 *         for i in range(n):
 *             peak = self.getitem(i)             # <<<<<<<<<<<<<<
 *             peak._index = _Index._create(0, 0)
 *             peak._index.neutral_mass = i
 */
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->getitem(__pyx_v_self, __pyx_v_i)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 577, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":578
 *         for i in range(n):
 *             peak = self.getitem(i)
 *             peak._index = _Index._create(0, 0)             # <<<<<<<<<<<<<<
 *             peak._index.neutral_mass = i
 *         i = 0
 */
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(0, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 578, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_peak->_index);
    __Pyx_DECREF(((PyObject *)__pyx_v_peak->_index));
    __pyx_v_peak->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":579
 *             peak = self.getitem(i)
 *             peak._index = _Index._create(0, 0)
 *             peak._index.neutral_mass = i             # <<<<<<<<<<<<<<
 *         i = 0
 *         for i in range(n):
 */
    __pyx_v_peak->_index->neutral_mass = __pyx_v_i;
  }

  /* "ms_deisotope/_c/peak_set.pyx":580
 *             peak._index = _Index._create(0, 0)
 *             peak._index.neutral_mass = i
 *         i = 0             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             peak = <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 */
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":581
 *             peak._index.neutral_mass = i
 *         i = 0
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             peak = <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 *             peak._index.mz = i
 */
  __pyx_t_5 = __pyx_v_n;
  __pyx_t_6 = __pyx_t_5;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "ms_deisotope/_c/peak_set.pyx":582
 *         i = 0
 *         for i in range(n):
 *             peak = <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)             # <<<<<<<<<<<<<<
 *             peak._index.mz = i
 *         self.indexed = True
 */
    __pyx_t_3 = __pyx_v_self->_mz_ordered;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_v_i);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)__pyx_t_8);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":583
 *         for i in range(n):
 *             peak = <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 *             peak._index.mz = i             # <<<<<<<<<<<<<<
 *         self.indexed = True
 *         return self
 */
    __pyx_v_peak->_index->mz = __pyx_v_i;
  }

  /* "ms_deisotope/_c/peak_set.pyx":584
 *             peak = <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 *             peak._index.mz = i
 *         self.indexed = True             # <<<<<<<<<<<<<<
 *         return self
 * 
 */
  __pyx_v_self->indexed = 1;

  /* "ms_deisotope/_c/peak_set.pyx":585
 *             peak._index.mz = i
 *         self.indexed = True
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":560
 *         self.indexed = False
 * 
 *     cpdef reindex(self):             # <<<<<<<<<<<<<<
 *         """
 *         Updates the :attr:`index` of each peak in `self` and updates the
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.reindex", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_3reindex(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_2reindex[] = "DeconvolutedPeakSet.reindex(self)\n\n        Updates the :attr:`index` of each peak in `self` and updates the\n        sorted order.\n\n        Returns\n        -------\n        self: DeconvolutedPeakSet\n        ";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_3reindex(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("reindex (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_2reindex(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_2reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("reindex", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_reindex(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.reindex", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":587
 *         return self
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.peaks)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_4__iter__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_4__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":588
 * 
 *     def __iter__(self):
 *         return iter(self.peaks)             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_self->peaks;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":587
 *         return self
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.peaks)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":590
 *         return iter(self.peaks)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self.get_size()
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_6__len__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_6__len__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":591
 * 
 *     def __len__(self):
 *         return self.get_size()             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t get_size(self):
 */
  __pyx_r = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->get_size(__pyx_v_self);
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":590
 *         return iter(self.peaks)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self.get_size()
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":593
 *         return self.get_size()
 * 
 *     cdef size_t get_size(self):             # <<<<<<<<<<<<<<
 *         return PyTuple_GET_SIZE(self.peaks)
 * 
 */

static size_t __pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_get_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_size", 0);

  /* "ms_deisotope/_c/peak_set.pyx":594
 * 
 *     cdef size_t get_size(self):
 *         return PyTuple_GET_SIZE(self.peaks)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_t_1 = __pyx_v_self->peaks;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_r = PyTuple_GET_SIZE(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":593
 *         return self.get_size()
 * 
 *     cdef size_t get_size(self):             # <<<<<<<<<<<<<<
 *         return PyTuple_GET_SIZE(self.peaks)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":596
 *         return PyTuple_GET_SIZE(self.peaks)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_9__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_9__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_8__repr__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_8__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":597
 * 
 *     def __repr__(self):
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, item):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_Length(((PyObject *)__pyx_v_self)); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 597, __pyx_L1_error)
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_DeconvolutedPeakSet_d_Peaks, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":596
 *         return PyTuple_GET_SIZE(self.peaks)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":599
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_10__getitem__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((PyObject *)__pyx_v_item));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___2generator3(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "ms_deisotope/_c/peak_set.pyx":601
 *     def __getitem__(self, item):
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))             # <<<<<<<<<<<<<<
 *         return self.peaks[item]
 * 
 */

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr(__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 601, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *) __pyx_self;
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_outer_scope));
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_outer_scope);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___2generator3, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_getitem___locals_genexpr, __pyx_n_s_ms_deisotope__c_peak_set); if (unlikely(!gen)) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__getitem__.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___2generator3(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *__pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 601, __pyx_L1_error)
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 601, __pyx_L1_error) }
  if (unlikely(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self->peaks == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 601, __pyx_L1_error)
  }
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_item)) { __Pyx_RaiseClosureNameError("item"); __PYX_ERR(0, 601, __pyx_L1_error) }
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self->peaks, __pyx_cur_scope->__pyx_outer_scope->__pyx_v_item); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 601, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 601, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 601, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 601, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_p);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_p, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_p, __pyx_n_s_clone); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    __Pyx_XGIVEREF(__pyx_t_2);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_3;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_4;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_2);
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_4 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 601, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":599
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 */

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_10__getitem__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_item) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *__pyx_cur_scope;
  PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___2generator3 = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__(__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 599, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_item = __pyx_v_item;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_item);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_item);

  /* "ms_deisotope/_c/peak_set.pyx":600
 * 
 *     def __getitem__(self, item):
 *         if isinstance(item, slice):             # <<<<<<<<<<<<<<
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 *         return self.peaks[item]
 */
  __pyx_t_1 = __pyx_cur_scope->__pyx_v_item;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PySlice_Check(__pyx_t_1); 
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "ms_deisotope/_c/peak_set.pyx":601
 *     def __getitem__(self, item):
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))             # <<<<<<<<<<<<<<
 *         return self.peaks[item]
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PySequence_Tuple(__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":600
 * 
 *     def __getitem__(self, item):
 *         if isinstance(item, slice):             # <<<<<<<<<<<<<<
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 *         return self.peaks[item]
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":602
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 *         return self.peaks[item]             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, other):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_cur_scope->__pyx_v_self->peaks == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 602, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_self->peaks, __pyx_cur_scope->__pyx_v_item); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":599
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_gb_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___2generator3);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":604
 *         return self.peaks[item]
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         try:
 *             return self.peaks == other.peaks
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_13__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_13__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_12__eq__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_12__eq__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":605
 * 
 *     def __eq__(self, other):
 *         try:             # <<<<<<<<<<<<<<
 *             return self.peaks == other.peaks
 *         except AttributeError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "ms_deisotope/_c/peak_set.pyx":606
 *     def __eq__(self, other):
 *         try:
 *             return self.peaks == other.peaks             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             return tuple(self) == tuple(other)
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_peaks); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 606, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_self->peaks, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 606, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_r = __pyx_t_5;
      __pyx_t_5 = 0;
      goto __pyx_L7_try_return;

      /* "ms_deisotope/_c/peak_set.pyx":605
 * 
 *     def __eq__(self, other):
 *         try:             # <<<<<<<<<<<<<<
 *             return self.peaks == other.peaks
 *         except AttributeError:
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":607
 *         try:
 *             return self.peaks == other.peaks
 *         except AttributeError:             # <<<<<<<<<<<<<<
 *             return tuple(self) == tuple(other)
 * 
 */
    __pyx_t_6 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);
    if (__pyx_t_6) {
      __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_4, &__pyx_t_7) < 0) __PYX_ERR(0, 607, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_7);

      /* "ms_deisotope/_c/peak_set.pyx":608
 *             return self.peaks == other.peaks
 *         except AttributeError:
 *             return tuple(self) == tuple(other)             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, other):
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_8 = PySequence_Tuple(((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 608, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = __Pyx_PySequence_Tuple(__pyx_v_other); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 608, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = PyObject_RichCompare(__pyx_t_8, __pyx_t_9, Py_EQ); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 608, __pyx_L5_except_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_r = __pyx_t_10;
      __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_except_return;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "ms_deisotope/_c/peak_set.pyx":605
 * 
 *     def __eq__(self, other):
 *         try:             # <<<<<<<<<<<<<<
 *             return self.peaks == other.peaks
 *         except AttributeError:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "ms_deisotope/_c/peak_set.pyx":604
 *         return self.peaks[item]
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         try:
 *             return self.peaks == other.peaks
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":610
 *             return tuple(self) == tuple(other)
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         try:
 *             return self.peaks != other.peaks
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_15__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_15__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_14__ne__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_14__ne__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":611
 * 
 *     def __ne__(self, other):
 *         try:             # <<<<<<<<<<<<<<
 *             return self.peaks != other.peaks
 *         except AttributeError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "ms_deisotope/_c/peak_set.pyx":612
 *     def __ne__(self, other):
 *         try:
 *             return self.peaks != other.peaks             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             return tuple(self) != tuple(other)
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_peaks); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 612, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_self->peaks, __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 612, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_r = __pyx_t_5;
      __pyx_t_5 = 0;
      goto __pyx_L7_try_return;

      /* "ms_deisotope/_c/peak_set.pyx":611
 * 
 *     def __ne__(self, other):
 *         try:             # <<<<<<<<<<<<<<
 *             return self.peaks != other.peaks
 *         except AttributeError:
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":613
 *         try:
 *             return self.peaks != other.peaks
 *         except AttributeError:             # <<<<<<<<<<<<<<
 *             return tuple(self) != tuple(other)
 * 
 */
    __pyx_t_6 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);
    if (__pyx_t_6) {
      __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_4, &__pyx_t_7) < 0) __PYX_ERR(0, 613, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_7);

      /* "ms_deisotope/_c/peak_set.pyx":614
 *             return self.peaks != other.peaks
 *         except AttributeError:
 *             return tuple(self) != tuple(other)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_8 = PySequence_Tuple(((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 614, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = __Pyx_PySequence_Tuple(__pyx_v_other); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 614, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = PyObject_RichCompare(__pyx_t_8, __pyx_t_9, Py_NE); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 614, __pyx_L5_except_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_r = __pyx_t_10;
      __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_except_return;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "ms_deisotope/_c/peak_set.pyx":611
 * 
 *     def __ne__(self, other):
 *         try:             # <<<<<<<<<<<<<<
 *             return self.peaks != other.peaks
 *         except AttributeError:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "ms_deisotope/_c/peak_set.pyx":610
 *             return tuple(self) == tuple(other)
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         try:
 *             return self.peaks != other.peaks
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":616
 *             return tuple(self) != tuple(other)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.peaks)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_17__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_17__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_16__iter__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_16__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":617
 * 
 *     def __iter__(self):
 *         return iter(self.peaks)             # <<<<<<<<<<<<<<
 * 
 *     cpdef DeconvolutedPeakSet clone(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_self->peaks;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":616
 *             return tuple(self) != tuple(other)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.peaks)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":619
 *         return iter(self.peaks)
 * 
 *     cpdef DeconvolutedPeakSet clone(self):             # <<<<<<<<<<<<<<
 *         return self.copy()
 * 
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_19clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 619, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_19clone)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 619, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet))))) __PYX_ERR(0, 619, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":620
 * 
 *     cpdef DeconvolutedPeakSet clone(self):
 *         return self.copy()             # <<<<<<<<<<<<<<
 * 
 *     cpdef DeconvolutedPeakSet copy(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->copy(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 620, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":619
 *         return iter(self.peaks)
 * 
 *     cpdef DeconvolutedPeakSet clone(self):             # <<<<<<<<<<<<<<
 *         return self.copy()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_19clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_18clone[] = "DeconvolutedPeakSet.clone(self) -> DeconvolutedPeakSet";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_19clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_18clone(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_18clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_clone(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 619, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":622
 *         return self.copy()
 * 
 *     cpdef DeconvolutedPeakSet copy(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             tuple duplicate_peaks
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_21copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_copy(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_duplicate_peaks = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0;
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_copy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 622, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_21copy)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 622, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet))))) __PYX_ERR(0, 622, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":627
 *             DeconvolutedPeak peak
 *             size_t i, n
 *         n = self.get_size()             # <<<<<<<<<<<<<<
 *         duplicate_peaks = PyTuple_New(n)
 *         for i in range(n):
 */
  __pyx_v_n = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->get_size(__pyx_v_self);

  /* "ms_deisotope/_c/peak_set.pyx":628
 *             size_t i, n
 *         n = self.get_size()
 *         duplicate_peaks = PyTuple_New(n)             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             peak = self.getitem(i)
 */
  __pyx_t_1 = PyTuple_New(__pyx_v_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 628, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_duplicate_peaks = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":629
 *         n = self.get_size()
 *         duplicate_peaks = PyTuple_New(n)
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             peak = self.getitem(i)
 *             peak = peak.clone()
 */
  __pyx_t_5 = __pyx_v_n;
  __pyx_t_6 = __pyx_t_5;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "ms_deisotope/_c/peak_set.pyx":630
 *         duplicate_peaks = PyTuple_New(n)
 *         for i in range(n):
 *             peak = self.getitem(i)             # <<<<<<<<<<<<<<
 *             peak = peak.clone()
 *             Py_INCREF(peak)
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->getitem(__pyx_v_self, __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 630, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":631
 *         for i in range(n):
 *             peak = self.getitem(i)
 *             peak = peak.clone()             # <<<<<<<<<<<<<<
 *             Py_INCREF(peak)
 *             PyTuple_SetItem(duplicate_peaks, i, peak)
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_peak->__pyx_base.__pyx_vtab)->__pyx_base.clone(((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_v_peak), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 631, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 631, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":632
 *             peak = self.getitem(i)
 *             peak = peak.clone()
 *             Py_INCREF(peak)             # <<<<<<<<<<<<<<
 *             PyTuple_SetItem(duplicate_peaks, i, peak)
 *         return self.__class__(duplicate_peaks)
 */
    Py_INCREF(((PyObject *)__pyx_v_peak));

    /* "ms_deisotope/_c/peak_set.pyx":633
 *             peak = peak.clone()
 *             Py_INCREF(peak)
 *             PyTuple_SetItem(duplicate_peaks, i, peak)             # <<<<<<<<<<<<<<
 *         return self.__class__(duplicate_peaks)
 * 
 */
    __pyx_t_8 = PyTuple_SetItem(__pyx_v_duplicate_peaks, __pyx_v_i, ((PyObject *)__pyx_v_peak)); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 633, __pyx_L1_error)
  }

  /* "ms_deisotope/_c/peak_set.pyx":634
 *             Py_INCREF(peak)
 *             PyTuple_SetItem(duplicate_peaks, i, peak)
 *         return self.__class__(duplicate_peaks)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_duplicate_peaks) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_duplicate_peaks);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet))))) __PYX_ERR(0, 634, __pyx_L1_error)
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":622
 *         return self.copy()
 * 
 *     cpdef DeconvolutedPeakSet copy(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             tuple duplicate_peaks
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_duplicate_peaks);
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_21copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_20copy[] = "DeconvolutedPeakSet.copy(self) -> DeconvolutedPeakSet";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_21copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_20copy(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_20copy(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_copy(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 622, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":636
 *         return self.__class__(duplicate_peaks)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, (self.peaks, )
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_23__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_22__reduce__[] = "DeconvolutedPeakSet.__reduce__(self)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_23__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_22__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_22__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":637
 * 
 *     def __reduce__(self):
 *         return self.__class__, (self.peaks, )             # <<<<<<<<<<<<<<
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->peaks);
  __Pyx_GIVEREF(__pyx_v_self->peaks);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->peaks);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":636
 *         return self.__class__(duplicate_peaks)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, (self.peaks, )
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":639
 *         return self.__class__, (self.peaks, )
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak *__pyx_optional_args) {
  double __pyx_v_error_tolerance = ((double)1e-5);
  int __pyx_v_use_mz = ((int)0);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_has_peak", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_error_tolerance = __pyx_optional_args->error_tolerance;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_use_mz = __pyx_optional_args->use_mz;
      }
    }
  }

  /* "ms_deisotope/_c/peak_set.pyx":658
 *             The found peak, or None if no peak is found
 *         '''
 *         if use_mz:             # <<<<<<<<<<<<<<
 *             return binary_search_mz(self._mz_ordered, neutral_mass, error_tolerance)
 *         else:
 */
  __pyx_t_1 = (__pyx_v_use_mz != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":659
 *         '''
 *         if use_mz:
 *             return binary_search_mz(self._mz_ordered, neutral_mass, error_tolerance)             # <<<<<<<<<<<<<<
 *         else:
 *             return binary_search_neutral_mass(self.peaks, neutral_mass, error_tolerance)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_2 = __pyx_v_self->_mz_ordered;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_mz(((PyObject*)__pyx_t_2), __pyx_v_neutral_mass, __pyx_v_error_tolerance)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 659, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":658
 *             The found peak, or None if no peak is found
 *         '''
 *         if use_mz:             # <<<<<<<<<<<<<<
 *             return binary_search_mz(self._mz_ordered, neutral_mass, error_tolerance)
 *         else:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":661
 *             return binary_search_mz(self._mz_ordered, neutral_mass, error_tolerance)
 *         else:
 *             return binary_search_neutral_mass(self.peaks, neutral_mass, error_tolerance)             # <<<<<<<<<<<<<<
 * 
 *     cpdef DeconvolutedPeak has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):
 */
  /*else*/ {
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = __pyx_v_self->peaks;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_neutral_mass(((PyObject*)__pyx_t_3), __pyx_v_neutral_mass, __pyx_v_error_tolerance)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 661, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "ms_deisotope/_c/peak_set.pyx":639
 *         return self.__class__, (self.peaks, )
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet._has_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":663
 *             return binary_search_neutral_mass(self.peaks, neutral_mass, error_tolerance)
 * 
 *     cpdef DeconvolutedPeak has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_25has_peak(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak *__pyx_optional_args) {
  double __pyx_v_error_tolerance = ((double)1e-5);
  int __pyx_v_use_mz = ((int)0);
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("has_peak", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_error_tolerance = __pyx_optional_args->error_tolerance;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_use_mz = __pyx_optional_args->use_mz;
      }
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_has_peak); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 663, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_25has_peak)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __pyx_t_3 = PyFloat_FromDouble(__pyx_v_neutral_mass); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 663, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyFloat_FromDouble(__pyx_v_error_tolerance); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 663, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_use_mz); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 663, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_6 = __pyx_t_1; __pyx_t_7 = NULL;
        __pyx_t_8 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
            __pyx_t_8 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 663, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 663, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        {
          __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 663, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_5);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_5 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 663, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 663, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":684
 *             The found peak, or None if no peak is found
 *         '''
 *         if not self.indexed:             # <<<<<<<<<<<<<<
 *             self.reindex()
 *         return self._has_peak(neutral_mass, error_tolerance, use_mz)
 */
  __pyx_t_10 = ((!(__pyx_v_self->indexed != 0)) != 0);
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":685
 *         '''
 *         if not self.indexed:
 *             self.reindex()             # <<<<<<<<<<<<<<
 *         return self._has_peak(neutral_mass, error_tolerance, use_mz)
 * 
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->reindex(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":684
 *             The found peak, or None if no peak is found
 *         '''
 *         if not self.indexed:             # <<<<<<<<<<<<<<
 *             self.reindex()
 *         return self._has_peak(neutral_mass, error_tolerance, use_mz)
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":686
 *         if not self.indexed:
 *             self.reindex()
 *         return self._has_peak(neutral_mass, error_tolerance, use_mz)             # <<<<<<<<<<<<<<
 * 
 *     cdef DeconvolutedPeak getitem(self, size_t i):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_11.__pyx_n = 2;
  __pyx_t_11.error_tolerance = __pyx_v_error_tolerance;
  __pyx_t_11.use_mz = __pyx_v_use_mz;
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->_has_peak(__pyx_v_self, __pyx_v_neutral_mass, &__pyx_t_11)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 686, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":663
 *             return binary_search_neutral_mass(self.peaks, neutral_mass, error_tolerance)
 * 
 *     cpdef DeconvolutedPeak has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.has_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_25has_peak(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_24has_peak[] = "DeconvolutedPeakSet.has_peak(self, double neutral_mass, double error_tolerance=1e-5, bool use_mz=False) -> DeconvolutedPeak\nFind the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.\n\n        If ``use_mz`` is True, instead of matching neutral masses, match peaks using m/z instead.\n\n        If :attr:`indexed` is not :const:`True`, then :meth:`reindex` will be called.\n\n        Parameters\n        ----------\n        neutral_mass: double\n            The mass to search for\n        error_tolerance: double\n            The PPM error tolerance to apply\n        use_mz: bool\n            Whether to search using m/z instead of neutral mass\n\n        Returns\n        -------\n        DeconvolutedPeak\n            The found peak, or None if no peak is found\n        ";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_25has_peak(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_neutral_mass;
  double __pyx_v_error_tolerance;
  int __pyx_v_use_mz;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("has_peak (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_error_tolerance,&__pyx_n_s_use_mz,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_error_tolerance);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_mz);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "has_peak") < 0)) __PYX_ERR(0, 663, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_neutral_mass = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_neutral_mass == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 663, __pyx_L3_error)
    if (values[1]) {
      __pyx_v_error_tolerance = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_error_tolerance == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 663, __pyx_L3_error)
    } else {
      __pyx_v_error_tolerance = ((double)1e-5);
    }
    if (values[2]) {
      __pyx_v_use_mz = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_use_mz == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 663, __pyx_L3_error)
    } else {
      __pyx_v_use_mz = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("has_peak", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 663, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.has_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_24has_peak(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_error_tolerance, __pyx_v_use_mz);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_24has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_error_tolerance, int __pyx_v_use_mz) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("has_peak", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.error_tolerance = __pyx_v_error_tolerance;
  __pyx_t_2.use_mz = __pyx_v_use_mz;
  __pyx_t_1 = ((PyObject *)__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet->has_peak(__pyx_v_self, __pyx_v_neutral_mass, 1, &__pyx_t_2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.has_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":688
 *         return self._has_peak(neutral_mass, error_tolerance, use_mz)
 * 
 *     cdef DeconvolutedPeak getitem(self, size_t i):             # <<<<<<<<<<<<<<
 *         return <DeconvolutedPeak>PyTuple_GET_ITEM(self.peaks, i)
 * 
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_getitem(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, size_t __pyx_v_i) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2;
  __Pyx_RefNannySetupContext("getitem", 0);

  /* "ms_deisotope/_c/peak_set.pyx":689
 * 
 *     cdef DeconvolutedPeak getitem(self, size_t i):
 *         return <DeconvolutedPeak>PyTuple_GET_ITEM(self.peaks, i)             # <<<<<<<<<<<<<<
 * 
 *     cdef tuple getslice(self, size_t start, size_t end):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = __pyx_v_self->peaks;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_v_i);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_INCREF(((PyObject *)((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2)));
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2);
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":688
 *         return self._has_peak(neutral_mass, error_tolerance, use_mz)
 * 
 *     cdef DeconvolutedPeak getitem(self, size_t i):             # <<<<<<<<<<<<<<
 *         return <DeconvolutedPeak>PyTuple_GET_ITEM(self.peaks, i)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":691
 *         return <DeconvolutedPeak>PyTuple_GET_ITEM(self.peaks, i)
 * 
 *     cdef tuple getslice(self, size_t start, size_t end):             # <<<<<<<<<<<<<<
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 */

static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_getslice(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, size_t __pyx_v_start, size_t __pyx_v_end) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getslice", 0);

  /* "ms_deisotope/_c/peak_set.pyx":692
 * 
 *     cdef tuple getslice(self, size_t start, size_t end):
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)             # <<<<<<<<<<<<<<
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_self->peaks;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_GetSlice(__pyx_t_1, __pyx_v_start, __pyx_v_end); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_INCREF(((PyObject*)__pyx_t_2));
  __pyx_r = ((PyObject*)__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":691
 *         return <DeconvolutedPeak>PyTuple_GET_ITEM(self.peaks, i)
 * 
 *     cdef tuple getslice(self, size_t start, size_t end):             # <<<<<<<<<<<<<<
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.getslice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":694
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         '''Find all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_27all_peaks_for(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for *__pyx_optional_args) {
  double __pyx_v_tolerance = ((double)1e-5);
  double __pyx_v_lo;
  double __pyx_v_hi;
  double __pyx_v_lo_err;
  double __pyx_v_hi_err;
  int __pyx_v_lo_ix;
  int __pyx_v_hi_ix;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_lo_peak = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_hi_peak = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("all_peaks_for", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_tolerance = __pyx_optional_args->tolerance;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_all_peaks_for); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 694, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_27all_peaks_for)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = PyFloat_FromDouble(__pyx_v_neutral_mass); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 694, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyFloat_FromDouble(__pyx_v_tolerance); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 694, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 694, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 694, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 694, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 694, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (!(likely(PyTuple_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 694, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":714
 *             DeconvolutedPeak lo_peak
 *             DeconvolutedPeak hi_peak
 *         lo = neutral_mass - neutral_mass * tolerance             # <<<<<<<<<<<<<<
 *         hi = neutral_mass + neutral_mass * tolerance
 *         lo_peak = binary_search_nearest_neutral_mass(self.peaks, lo, &lo_err)
 */
  __pyx_v_lo = (__pyx_v_neutral_mass - (__pyx_v_neutral_mass * __pyx_v_tolerance));

  /* "ms_deisotope/_c/peak_set.pyx":715
 *             DeconvolutedPeak hi_peak
 *         lo = neutral_mass - neutral_mass * tolerance
 *         hi = neutral_mass + neutral_mass * tolerance             # <<<<<<<<<<<<<<
 *         lo_peak = binary_search_nearest_neutral_mass(self.peaks, lo, &lo_err)
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)
 */
  __pyx_v_hi = (__pyx_v_neutral_mass + (__pyx_v_neutral_mass * __pyx_v_tolerance));

  /* "ms_deisotope/_c/peak_set.pyx":716
 *         lo = neutral_mass - neutral_mass * tolerance
 *         hi = neutral_mass + neutral_mass * tolerance
 *         lo_peak = binary_search_nearest_neutral_mass(self.peaks, lo, &lo_err)             # <<<<<<<<<<<<<<
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)
 *         lo_ix = lo_peak._index.neutral_mass
 */
  __pyx_t_1 = __pyx_v_self->peaks;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_nearest_neutral_mass(((PyObject*)__pyx_t_1), __pyx_v_lo, (&__pyx_v_lo_err))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_lo_peak = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":717
 *         hi = neutral_mass + neutral_mass * tolerance
 *         lo_peak = binary_search_nearest_neutral_mass(self.peaks, lo, &lo_err)
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)             # <<<<<<<<<<<<<<
 *         lo_ix = lo_peak._index.neutral_mass
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(
 */
  __pyx_t_2 = __pyx_v_self->peaks;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_nearest_neutral_mass(((PyObject*)__pyx_t_2), __pyx_v_hi, (&__pyx_v_hi_err))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_hi_peak = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":718
 *         lo_peak = binary_search_nearest_neutral_mass(self.peaks, lo, &lo_err)
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)
 *         lo_ix = lo_peak._index.neutral_mass             # <<<<<<<<<<<<<<
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:
 */
  __pyx_t_9 = __pyx_v_lo_peak->_index->neutral_mass;
  __pyx_v_lo_ix = __pyx_t_9;

  /* "ms_deisotope/_c/peak_set.pyx":719
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)
 *         lo_ix = lo_peak._index.neutral_mass
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(             # <<<<<<<<<<<<<<
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:
 *             lo_ix += 1
 */
  __pyx_t_1 = __pyx_v_self->peaks;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_11 = ((__pyx_v_lo_ix < PyTuple_GET_SIZE(__pyx_t_1)) != 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_11) {
  } else {
    __pyx_t_10 = __pyx_t_11;
    goto __pyx_L4_bool_binop_done;
  }

  /* "ms_deisotope/_c/peak_set.pyx":720
 *         lo_ix = lo_peak._index.neutral_mass
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:             # <<<<<<<<<<<<<<
 *             lo_ix += 1
 *         hi_ix = hi_peak._index.neutral_mass + 1
 */
  __pyx_t_11 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_lo_peak->neutral_mass, __pyx_v_neutral_mass)) > __pyx_v_tolerance) != 0);
  __pyx_t_10 = __pyx_t_11;
  __pyx_L4_bool_binop_done:;

  /* "ms_deisotope/_c/peak_set.pyx":719
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)
 *         lo_ix = lo_peak._index.neutral_mass
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(             # <<<<<<<<<<<<<<
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:
 *             lo_ix += 1
 */
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":721
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:
 *             lo_ix += 1             # <<<<<<<<<<<<<<
 *         hi_ix = hi_peak._index.neutral_mass + 1
 *         if hi_ix != 0 and abs(
 */
    __pyx_v_lo_ix = (__pyx_v_lo_ix + 1);

    /* "ms_deisotope/_c/peak_set.pyx":719
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)
 *         lo_ix = lo_peak._index.neutral_mass
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(             # <<<<<<<<<<<<<<
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:
 *             lo_ix += 1
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":722
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:
 *             lo_ix += 1
 *         hi_ix = hi_peak._index.neutral_mass + 1             # <<<<<<<<<<<<<<
 *         if hi_ix != 0 and abs(
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:
 */
  __pyx_v_hi_ix = (__pyx_v_hi_peak->_index->neutral_mass + 1);

  /* "ms_deisotope/_c/peak_set.pyx":723
 *             lo_ix += 1
 *         hi_ix = hi_peak._index.neutral_mass + 1
 *         if hi_ix != 0 and abs(             # <<<<<<<<<<<<<<
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:
 *             hi_ix -= 1
 */
  __pyx_t_11 = ((__pyx_v_hi_ix != 0) != 0);
  if (__pyx_t_11) {
  } else {
    __pyx_t_10 = __pyx_t_11;
    goto __pyx_L7_bool_binop_done;
  }

  /* "ms_deisotope/_c/peak_set.pyx":724
 *         hi_ix = hi_peak._index.neutral_mass + 1
 *         if hi_ix != 0 and abs(
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:             # <<<<<<<<<<<<<<
 *             hi_ix -= 1
 *         return <tuple>PyTuple_GetSlice(self.peaks, lo_ix, hi_ix)
 */
  __pyx_t_11 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_hi_peak->neutral_mass, __pyx_v_neutral_mass)) > __pyx_v_tolerance) != 0);
  __pyx_t_10 = __pyx_t_11;
  __pyx_L7_bool_binop_done:;

  /* "ms_deisotope/_c/peak_set.pyx":723
 *             lo_ix += 1
 *         hi_ix = hi_peak._index.neutral_mass + 1
 *         if hi_ix != 0 and abs(             # <<<<<<<<<<<<<<
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:
 *             hi_ix -= 1
 */
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":725
 *         if hi_ix != 0 and abs(
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:
 *             hi_ix -= 1             # <<<<<<<<<<<<<<
 *         return <tuple>PyTuple_GetSlice(self.peaks, lo_ix, hi_ix)
 * 
 */
    __pyx_v_hi_ix = (__pyx_v_hi_ix - 1);

    /* "ms_deisotope/_c/peak_set.pyx":723
 *             lo_ix += 1
 *         hi_ix = hi_peak._index.neutral_mass + 1
 *         if hi_ix != 0 and abs(             # <<<<<<<<<<<<<<
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:
 *             hi_ix -= 1
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":726
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:
 *             hi_ix -= 1
 *         return <tuple>PyTuple_GetSlice(self.peaks, lo_ix, hi_ix)             # <<<<<<<<<<<<<<
 * 
 *     cdef DeconvolutedPeak _get_nearest_peak(self, double neutral_mass, double* errout):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_self->peaks;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_GetSlice(__pyx_t_1, __pyx_v_lo_ix, __pyx_v_hi_ix); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 726, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_INCREF(((PyObject*)__pyx_t_2));
  __pyx_r = ((PyObject*)__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":694
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         '''Find all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.all_peaks_for", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_lo_peak);
  __Pyx_XDECREF((PyObject *)__pyx_v_hi_peak);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_27all_peaks_for(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_26all_peaks_for[] = "DeconvolutedPeakSet.all_peaks_for(self, double neutral_mass, double tolerance=1e-5) -> tuple\nFind all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.\n\n        Parameters\n        ----------\n        neutral_mass: double\n            The mass to search for\n        error_tolerance: double\n            The PPM error tolerance to apply\n\n        Returns\n        -------\n        tuple of DeconvolutedPeak\n            The found peaks\n        ";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_27all_peaks_for(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_neutral_mass;
  double __pyx_v_tolerance;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("all_peaks_for (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_tolerance,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tolerance);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "all_peaks_for") < 0)) __PYX_ERR(0, 694, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_neutral_mass = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_neutral_mass == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 694, __pyx_L3_error)
    if (values[1]) {
      __pyx_v_tolerance = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_tolerance == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 694, __pyx_L3_error)
    } else {
      __pyx_v_tolerance = ((double)1e-5);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("all_peaks_for", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 694, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.all_peaks_for", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_26all_peaks_for(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_tolerance);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_26all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("all_peaks_for", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.tolerance = __pyx_v_tolerance;
  __pyx_t_1 = __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet->all_peaks_for(__pyx_v_self, __pyx_v_neutral_mass, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.all_peaks_for", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":728
 *         return <tuple>PyTuple_GetSlice(self.peaks, lo_ix, hi_ix)
 * 
 *     cdef DeconvolutedPeak _get_nearest_peak(self, double neutral_mass, double* errout):             # <<<<<<<<<<<<<<
 *         '''Find the peak nearest to ``neutral_mass``, regardless of error.
 * 
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__get_nearest_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, double *__pyx_v_errout) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_nearest_peak", 0);

  /* "ms_deisotope/_c/peak_set.pyx":744
 *         '''
 *         cdef DeconvolutedPeak peak
 *         peak = binary_search_nearest_neutral_mass(self.peaks, neutral_mass, errout)             # <<<<<<<<<<<<<<
 *         return peak
 * 
 */
  __pyx_t_1 = __pyx_v_self->peaks;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_nearest_neutral_mass(((PyObject*)__pyx_t_1), __pyx_v_neutral_mass, __pyx_v_errout)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_peak = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":745
 *         cdef DeconvolutedPeak peak
 *         peak = binary_search_nearest_neutral_mass(self.peaks, neutral_mass, errout)
 *         return peak             # <<<<<<<<<<<<<<
 * 
 *     def get_nearest_peak(self, double neutral_mass):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_peak));
  __pyx_r = __pyx_v_peak;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":728
 *         return <tuple>PyTuple_GetSlice(self.peaks, lo_ix, hi_ix)
 * 
 *     cdef DeconvolutedPeak _get_nearest_peak(self, double neutral_mass, double* errout):             # <<<<<<<<<<<<<<
 *         '''Find the peak nearest to ``neutral_mass``, regardless of error.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet._get_nearest_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":747
 *         return peak
 * 
 *     def get_nearest_peak(self, double neutral_mass):             # <<<<<<<<<<<<<<
 *         '''Find the peak nearest to ``neutral_mass``, regardless of error.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_29get_nearest_peak(PyObject *__pyx_v_self, PyObject *__pyx_arg_neutral_mass); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_28get_nearest_peak[] = "DeconvolutedPeakSet.get_nearest_peak(self, double neutral_mass)\nFind the peak nearest to ``neutral_mass``, regardless of error.\n\n        Parameters\n        ----------\n        neutral_mass: double\n            The mass to search for\n\n        Returns\n        -------\n        DeconvolutedPeak\n            The nearest peak\n        double\n            The error between ``neutral_mass`` and the found peak\n        ";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_29get_nearest_peak(PyObject *__pyx_v_self, PyObject *__pyx_arg_neutral_mass) {
  double __pyx_v_neutral_mass;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_nearest_peak (wrapper)", 0);
  assert(__pyx_arg_neutral_mass); {
    __pyx_v_neutral_mass = __pyx_PyFloat_AsDouble(__pyx_arg_neutral_mass); if (unlikely((__pyx_v_neutral_mass == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 747, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.get_nearest_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_28get_nearest_peak(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((double)__pyx_v_neutral_mass));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_28get_nearest_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0;
  double __pyx_v_errout;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_nearest_peak", 0);

  /* "ms_deisotope/_c/peak_set.pyx":765
 *             DeconvolutedPeak peak
 *             double errout
 *         peak = self._get_nearest_peak(neutral_mass, &errout)             # <<<<<<<<<<<<<<
 *         return peak, errout
 * 
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->_get_nearest_peak(__pyx_v_self, __pyx_v_neutral_mass, (&__pyx_v_errout))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 765, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_peak = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":766
 *             double errout
 *         peak = self._get_nearest_peak(neutral_mass, &errout)
 *         return peak, errout             # <<<<<<<<<<<<<<
 * 
 *     def between(self, m1, m2, tolerance=1e-5, use_mz=False):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_errout); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 766, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 766, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_peak));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_peak));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_peak));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":747
 *         return peak
 * 
 *     def get_nearest_peak(self, double neutral_mass):             # <<<<<<<<<<<<<<
 *         '''Find the peak nearest to ``neutral_mass``, regardless of error.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.get_nearest_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":768
 *         return peak, errout
 * 
 *     def between(self, m1, m2, tolerance=1e-5, use_mz=False):             # <<<<<<<<<<<<<<
 *         """Retrieve a :class:`DeconvolutedPeakSet` containing all the peaks
 *         whose mass is between ``m1`` and ``m2``.
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_31between(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_30between[] = "DeconvolutedPeakSet.between(self, m1, m2, tolerance=1e-5, use_mz=False)\nRetrieve a :class:`DeconvolutedPeakSet` containing all the peaks\n        whose mass is between ``m1`` and ``m2``.\n\n        If ``use_mz`` is :const:`True` then search by m/z instead of mass\n\n        Parameters\n        ----------\n        m1 : float\n            The lower mass limit\n        m2 : float\n            The upper mass limit\n        use_mz: bool\n            Whether to search for m/z instead of neutral mass\n\n        Returns\n        -------\n        DeconvolutedPeakSet\n        ";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_31between(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_m1 = 0;
  PyObject *__pyx_v_m2 = 0;
  CYTHON_UNUSED PyObject *__pyx_v_tolerance = 0;
  PyObject *__pyx_v_use_mz = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("between (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_m1,&__pyx_n_s_m2,&__pyx_n_s_tolerance,&__pyx_n_s_use_mz,0};
    PyObject* values[4] = {0,0,0,0};
    values[2] = ((PyObject *)__pyx_float_1eneg_5);
    values[3] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_m1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_m2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("between", 0, 2, 4, 1); __PYX_ERR(0, 768, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tolerance);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_mz);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "between") < 0)) __PYX_ERR(0, 768, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_m1 = values[0];
    __pyx_v_m2 = values[1];
    __pyx_v_tolerance = values[2];
    __pyx_v_use_mz = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("between", 0, 2, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 768, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.between", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_30between(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_m1, __pyx_v_m2, __pyx_v_tolerance, __pyx_v_use_mz);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_30between(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_m1, PyObject *__pyx_v_m2, CYTHON_UNUSED PyObject *__pyx_v_tolerance, PyObject *__pyx_v_use_mz) {
  PyObject *__pyx_v_acc = NULL;
  int __pyx_v_collecting;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_v_getter = NULL;
  PyObject *__pyx_v_iterator = NULL;
  PyObject *__pyx_v_peak = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("between", 0);

  /* "ms_deisotope/_c/peak_set.pyx":787
 *         DeconvolutedPeakSet
 *         """
 *         acc = []             # <<<<<<<<<<<<<<
 *         collecting = False
 *         value = None
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_acc = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":788
 *         """
 *         acc = []
 *         collecting = False             # <<<<<<<<<<<<<<
 *         value = None
 *         if not use_mz:
 */
  __pyx_v_collecting = 0;

  /* "ms_deisotope/_c/peak_set.pyx":789
 *         acc = []
 *         collecting = False
 *         value = None             # <<<<<<<<<<<<<<
 *         if not use_mz:
 *             getter = operator.attrgetter("neutral_mass")
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_value = Py_None;

  /* "ms_deisotope/_c/peak_set.pyx":790
 *         collecting = False
 *         value = None
 *         if not use_mz:             # <<<<<<<<<<<<<<
 *             getter = operator.attrgetter("neutral_mass")
 *             iterator = self.peaks
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_use_mz); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 790, __pyx_L1_error)
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (__pyx_t_3) {

    /* "ms_deisotope/_c/peak_set.pyx":791
 *         value = None
 *         if not use_mz:
 *             getter = operator.attrgetter("neutral_mass")             # <<<<<<<<<<<<<<
 *             iterator = self.peaks
 *         else:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_operator); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 791, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_attrgetter); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 791, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_n_s_neutral_mass) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_s_neutral_mass);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 791, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_getter = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":792
 *         if not use_mz:
 *             getter = operator.attrgetter("neutral_mass")
 *             iterator = self.peaks             # <<<<<<<<<<<<<<
 *         else:
 *             getter = operator.attrgetter("mz")
 */
    __pyx_t_1 = __pyx_v_self->peaks;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_iterator = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":790
 *         collecting = False
 *         value = None
 *         if not use_mz:             # <<<<<<<<<<<<<<
 *             getter = operator.attrgetter("neutral_mass")
 *             iterator = self.peaks
 */
    goto __pyx_L3;
  }

  /* "ms_deisotope/_c/peak_set.pyx":794
 *             iterator = self.peaks
 *         else:
 *             getter = operator.attrgetter("mz")             # <<<<<<<<<<<<<<
 *             iterator = self._mz_ordered
 *         for peak in iterator:
 */
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_operator); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 794, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_attrgetter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 794, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_n_s_mz) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_n_s_mz);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 794, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_getter = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":795
 *         else:
 *             getter = operator.attrgetter("mz")
 *             iterator = self._mz_ordered             # <<<<<<<<<<<<<<
 *         for peak in iterator:
 *             value = getter(peak)
 */
    __pyx_t_1 = __pyx_v_self->_mz_ordered;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_iterator = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "ms_deisotope/_c/peak_set.pyx":796
 *             getter = operator.attrgetter("mz")
 *             iterator = self._mz_ordered
 *         for peak in iterator:             # <<<<<<<<<<<<<<
 *             value = getter(peak)
 *             if not collecting and value >= m1 and value < m2:
 */
  if (unlikely(__pyx_v_iterator == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 796, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_iterator; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
  for (;;) {
    if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 796, __pyx_L1_error)
    #else
    __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 796, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_peak, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":797
 *             iterator = self._mz_ordered
 *         for peak in iterator:
 *             value = getter(peak)             # <<<<<<<<<<<<<<
 *             if not collecting and value >= m1 and value < m2:
 *                 collecting = True
 */
    __Pyx_INCREF(__pyx_v_getter);
    __pyx_t_5 = __pyx_v_getter; __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_7, __pyx_v_peak) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_peak);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 797, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":798
 *         for peak in iterator:
 *             value = getter(peak)
 *             if not collecting and value >= m1 and value < m2:             # <<<<<<<<<<<<<<
 *                 collecting = True
 *             elif collecting and value > m2:
 */
    __pyx_t_2 = ((!(__pyx_v_collecting != 0)) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_value, __pyx_v_m1, Py_GE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 798, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 798, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_value, __pyx_v_m2, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 798, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 798, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_3 = __pyx_t_2;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_3) {

      /* "ms_deisotope/_c/peak_set.pyx":799
 *             value = getter(peak)
 *             if not collecting and value >= m1 and value < m2:
 *                 collecting = True             # <<<<<<<<<<<<<<
 *             elif collecting and value > m2:
 *                 break
 */
      __pyx_v_collecting = 1;

      /* "ms_deisotope/_c/peak_set.pyx":798
 *         for peak in iterator:
 *             value = getter(peak)
 *             if not collecting and value >= m1 and value < m2:             # <<<<<<<<<<<<<<
 *                 collecting = True
 *             elif collecting and value > m2:
 */
      goto __pyx_L6;
    }

    /* "ms_deisotope/_c/peak_set.pyx":800
 *             if not collecting and value >= m1 and value < m2:
 *                 collecting = True
 *             elif collecting and value > m2:             # <<<<<<<<<<<<<<
 *                 break
 * 
 */
    __pyx_t_2 = (__pyx_v_collecting != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L10_bool_binop_done;
    }
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_value, __pyx_v_m2, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 800, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 800, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_3 = __pyx_t_2;
    __pyx_L10_bool_binop_done:;
    if (__pyx_t_3) {

      /* "ms_deisotope/_c/peak_set.pyx":801
 *                 collecting = True
 *             elif collecting and value > m2:
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             if collecting:
 */
      goto __pyx_L5_break;

      /* "ms_deisotope/_c/peak_set.pyx":800
 *             if not collecting and value >= m1 and value < m2:
 *                 collecting = True
 *             elif collecting and value > m2:             # <<<<<<<<<<<<<<
 *                 break
 * 
 */
    }
    __pyx_L6:;

    /* "ms_deisotope/_c/peak_set.pyx":803
 *                 break
 * 
 *             if collecting:             # <<<<<<<<<<<<<<
 *                 acc.append(peak.clone())
 * 
 */
    __pyx_t_3 = (__pyx_v_collecting != 0);
    if (__pyx_t_3) {

      /* "ms_deisotope/_c/peak_set.pyx":804
 * 
 *             if collecting:
 *                 acc.append(peak.clone())             # <<<<<<<<<<<<<<
 * 
 *         return self.__class__(acc).reindex()
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_clone); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 804, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 804, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_acc, __pyx_t_4); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 804, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "ms_deisotope/_c/peak_set.pyx":803
 *                 break
 * 
 *             if collecting:             # <<<<<<<<<<<<<<
 *                 acc.append(peak.clone())
 * 
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":796
 *             getter = operator.attrgetter("mz")
 *             iterator = self._mz_ordered
 *         for peak in iterator:             # <<<<<<<<<<<<<<
 *             value = getter(peak)
 *             if not collecting and value >= m1 and value < m2:
 */
  }
  __pyx_L5_break:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":806
 *                 acc.append(peak.clone())
 * 
 *         return self.__class__(acc).reindex()             # <<<<<<<<<<<<<<
 * 
 * cdef double INF
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_7, __pyx_v_acc) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_acc);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_reindex); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":768
 *         return peak, errout
 * 
 *     def between(self, m1, m2, tolerance=1e-5, use_mz=False):             # <<<<<<<<<<<<<<
 *         """Retrieve a :class:`DeconvolutedPeakSet` containing all the peaks
 *         whose mass is between ``m1`` and ``m2``.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.between", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_acc);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_getter);
  __Pyx_XDECREF(__pyx_v_iterator);
  __Pyx_XDECREF(__pyx_v_peak);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":77
 * cdef class DeconvolutedPeakSet(object):
 *     cdef:
 *         public tuple peaks             # <<<<<<<<<<<<<<
 *         public tuple _mz_ordered
 *         public bint indexed
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->peaks);
  __pyx_r = __pyx_v_self->peaks;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyTuple_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 77, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->peaks);
  __Pyx_DECREF(__pyx_v_self->peaks);
  __pyx_v_self->peaks = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.peaks.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->peaks);
  __Pyx_DECREF(__pyx_v_self->peaks);
  __pyx_v_self->peaks = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":78
 *     cdef:
 *         public tuple peaks
 *         public tuple _mz_ordered             # <<<<<<<<<<<<<<
 *         public bint indexed
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_mz_ordered);
  __pyx_r = __pyx_v_self->_mz_ordered;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyTuple_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 78, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_mz_ordered);
  __Pyx_DECREF(__pyx_v_self->_mz_ordered);
  __pyx_v_self->_mz_ordered = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet._mz_ordered.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_mz_ordered);
  __Pyx_DECREF(__pyx_v_self->_mz_ordered);
  __pyx_v_self->_mz_ordered = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pxd":79
 *         public tuple peaks
 *         public tuple _mz_ordered
 *         public bint indexed             # <<<<<<<<<<<<<<
 * 
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->indexed); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.indexed.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 79, __pyx_L1_error)
  __pyx_v_self->indexed = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.indexed.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":812
 * 
 * 
 * cdef DeconvolutedPeak _sweep_solution_neutral_mass(tuple array, double value, size_t mid, double tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t best_index, i, n
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_solution_neutral_mass(PyObject *__pyx_v_array, double __pyx_v_value, size_t __pyx_v_mid, double __pyx_v_tolerance) {
  size_t __pyx_v_best_index;
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  double __pyx_v_best_error;
  double __pyx_v_abs_error;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_target = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_sweep_solution_neutral_mass", 0);

  /* "ms_deisotope/_c/peak_set.pyx":818
 *         DeconvolutedPeak target
 * 
 *     best_index = mid             # <<<<<<<<<<<<<<
 *     best_error = INF
 *     n = PyTuple_GET_SIZE(array)
 */
  __pyx_v_best_index = __pyx_v_mid;

  /* "ms_deisotope/_c/peak_set.pyx":819
 * 
 *     best_index = mid
 *     best_error = INF             # <<<<<<<<<<<<<<
 *     n = PyTuple_GET_SIZE(array)
 * 
 */
  __pyx_v_best_error = __pyx_v_12ms_deisotope_2_c_8peak_set_INF;

  /* "ms_deisotope/_c/peak_set.pyx":820
 *     best_index = mid
 *     best_error = INF
 *     n = PyTuple_GET_SIZE(array)             # <<<<<<<<<<<<<<
 * 
 *     i = 0
 */
  __pyx_v_n = PyTuple_GET_SIZE(__pyx_v_array);

  /* "ms_deisotope/_c/peak_set.pyx":822
 *     n = PyTuple_GET_SIZE(array)
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     while mid - i >= 0 and i <= mid:
 *         target = <DeconvolutedPeak>array[mid - i]
 */
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":823
 * 
 *     i = 0
 *     while mid - i >= 0 and i <= mid:             # <<<<<<<<<<<<<<
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 */
  while (1) {
    __pyx_t_2 = (((__pyx_v_mid - __pyx_v_i) >= 0) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_i <= __pyx_v_mid) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L5_bool_binop_done:;
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":824
 *     i = 0
 *     while mid - i >= 0 and i <= mid:
 *         target = <DeconvolutedPeak>array[mid - i]             # <<<<<<<<<<<<<<
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:
 */
    if (unlikely(__pyx_v_array == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 824, __pyx_L1_error)
    }
    __pyx_t_3 = (__pyx_v_mid - __pyx_v_i);
    __pyx_t_4 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_t_3, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 824, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __pyx_t_4;
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_target, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":825
 *     while mid - i >= 0 and i <= mid:
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))             # <<<<<<<<<<<<<<
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 */
    __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_value, __pyx_v_target->neutral_mass));

    /* "ms_deisotope/_c/peak_set.pyx":826
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid - i
 */
    __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_tolerance) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":827
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid - i
 *                 best_error = abs_error
 */
      __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":828
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 *                 best_index = mid - i             # <<<<<<<<<<<<<<
 *                 best_error = abs_error
 *         else:
 */
        __pyx_v_best_index = (__pyx_v_mid - __pyx_v_i);

        /* "ms_deisotope/_c/peak_set.pyx":829
 *             if abs_error < best_error:
 *                 best_index = mid - i
 *                 best_error = abs_error             # <<<<<<<<<<<<<<
 *         else:
 *             break
 */
        __pyx_v_best_error = __pyx_v_abs_error;

        /* "ms_deisotope/_c/peak_set.pyx":827
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid - i
 *                 best_error = abs_error
 */
      }

      /* "ms_deisotope/_c/peak_set.pyx":826
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid - i
 */
      goto __pyx_L7;
    }

    /* "ms_deisotope/_c/peak_set.pyx":831
 *                 best_error = abs_error
 *         else:
 *             break             # <<<<<<<<<<<<<<
 *         i += 1
 *     i = 1
 */
    /*else*/ {
      goto __pyx_L4_break;
    }
    __pyx_L7:;

    /* "ms_deisotope/_c/peak_set.pyx":832
 *         else:
 *             break
 *         i += 1             # <<<<<<<<<<<<<<
 *     i = 1
 *     while (mid + i) < (n - 1):
 */
    __pyx_v_i = (__pyx_v_i + 1);
  }
  __pyx_L4_break:;

  /* "ms_deisotope/_c/peak_set.pyx":833
 *             break
 *         i += 1
 *     i = 1             # <<<<<<<<<<<<<<
 *     while (mid + i) < (n - 1):
 *         target = <DeconvolutedPeak>array[mid + i]
 */
  __pyx_v_i = 1;

  /* "ms_deisotope/_c/peak_set.pyx":834
 *         i += 1
 *     i = 1
 *     while (mid + i) < (n - 1):             # <<<<<<<<<<<<<<
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 */
  while (1) {
    __pyx_t_1 = (((__pyx_v_mid + __pyx_v_i) < (__pyx_v_n - 1)) != 0);
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":835
 *     i = 1
 *     while (mid + i) < (n - 1):
 *         target = <DeconvolutedPeak>array[mid + i]             # <<<<<<<<<<<<<<
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:
 */
    if (unlikely(__pyx_v_array == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 835, __pyx_L1_error)
    }
    __pyx_t_3 = (__pyx_v_mid + __pyx_v_i);
    __pyx_t_5 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_t_3, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 835, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_target, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":836
 *     while (mid + i) < (n - 1):
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))             # <<<<<<<<<<<<<<
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 */
    __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_value, __pyx_v_target->neutral_mass));

    /* "ms_deisotope/_c/peak_set.pyx":837
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid + i
 */
    __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_tolerance) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":838
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid + i
 *                 best_error = abs_error
 */
      __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":839
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 *                 best_index = mid + i             # <<<<<<<<<<<<<<
 *                 best_error = abs_error
 *         else:
 */
        __pyx_v_best_index = (__pyx_v_mid + __pyx_v_i);

        /* "ms_deisotope/_c/peak_set.pyx":840
 *             if abs_error < best_error:
 *                 best_index = mid + i
 *                 best_error = abs_error             # <<<<<<<<<<<<<<
 *         else:
 *             break
 */
        __pyx_v_best_error = __pyx_v_abs_error;

        /* "ms_deisotope/_c/peak_set.pyx":838
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid + i
 *                 best_error = abs_error
 */
      }

      /* "ms_deisotope/_c/peak_set.pyx":837
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid + i
 */
      goto __pyx_L11;
    }

    /* "ms_deisotope/_c/peak_set.pyx":842
 *                 best_error = abs_error
 *         else:
 *             break             # <<<<<<<<<<<<<<
 *         i += 1
 *     if best_error == INF:
 */
    /*else*/ {
      goto __pyx_L10_break;
    }
    __pyx_L11:;

    /* "ms_deisotope/_c/peak_set.pyx":843
 *         else:
 *             break
 *         i += 1             # <<<<<<<<<<<<<<
 *     if best_error == INF:
 *         return None
 */
    __pyx_v_i = (__pyx_v_i + 1);
  }
  __pyx_L10_break:;

  /* "ms_deisotope/_c/peak_set.pyx":844
 *             break
 *         i += 1
 *     if best_error == INF:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_best_error == __pyx_v_12ms_deisotope_2_c_8peak_set_INF) != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":845
 *         i += 1
 *     if best_error == INF:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return array[best_index]
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":844
 *             break
 *         i += 1
 *     if best_error == INF:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":847
 *         return None
 *     else:
 *         return array[best_index]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    if (unlikely(__pyx_v_array == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 847, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_v_best_index, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 847, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 847, __pyx_L1_error)
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4);
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "ms_deisotope/_c/peak_set.pyx":812
 * 
 * 
 * cdef DeconvolutedPeak _sweep_solution_neutral_mass(tuple array, double value, size_t mid, double tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t best_index, i, n
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._sweep_solution_neutral_mass", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_target);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":851
 * 
 * @cython.cdivision(True)
 * cdef double _ppm_error(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     return (x - y) / y
 * 
 */

static double __pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(double __pyx_v_x, double __pyx_v_y) {
  double __pyx_r;

  /* "ms_deisotope/_c/peak_set.pyx":852
 * @cython.cdivision(True)
 * cdef double _ppm_error(double x, double y) nogil:
 *     return (x - y) / y             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((__pyx_v_x - __pyx_v_y) / __pyx_v_y);
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":851
 * 
 * @cython.cdivision(True)
 * cdef double _ppm_error(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     return (x - y) / y
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":855
 * 
 * 
 * cdef int _binary_search(double* array, double target, double error_tolerance, size_t n, size_t* out) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search(double *__pyx_v_array, double __pyx_v_target, double __pyx_v_error_tolerance, size_t __pyx_v_n, size_t *__pyx_v_out) {
  size_t __pyx_v_lo;
  size_t __pyx_v_hi;
  size_t __pyx_v_mid;
  size_t __pyx_v_i;
  size_t __pyx_v_best_index;
  double __pyx_v_found_mass;
  double __pyx_v_best_error;
  double __pyx_v_abs_error;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "ms_deisotope/_c/peak_set.pyx":862
 *         double best_error, abs_error
 * 
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = n
 * 
 */
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/peak_set.pyx":863
 * 
 *     lo = 0
 *     hi = n             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */
  __pyx_v_hi = __pyx_v_n;

  /* "ms_deisotope/_c/peak_set.pyx":865
 *     hi = n
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":866
 * 
 *     while hi != lo:
 *         mid = (hi + lo) / 2             # <<<<<<<<<<<<<<
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 */
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":867
 *     while hi != lo:
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]             # <<<<<<<<<<<<<<
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 */
    __pyx_v_found_mass = (__pyx_v_array[__pyx_v_mid]);

    /* "ms_deisotope/_c/peak_set.pyx":868
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */
    __pyx_t_1 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_found_mass, __pyx_v_target)) < __pyx_v_error_tolerance) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":869
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid             # <<<<<<<<<<<<<<
 *             best_error = INF
 *             i = 0
 */
      __pyx_v_best_index = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":870
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 *             best_error = INF             # <<<<<<<<<<<<<<
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 */
      __pyx_v_best_error = __pyx_v_12ms_deisotope_2_c_8peak_set_INF;

      /* "ms_deisotope/_c/peak_set.pyx":871
 *             best_index = mid
 *             best_error = INF
 *             i = 0             # <<<<<<<<<<<<<<
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 */
      __pyx_v_i = 0;

      /* "ms_deisotope/_c/peak_set.pyx":872
 *             best_error = INF
 *             i = 0
 *             while mid - i >= 0 and i <= mid:             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */
      while (1) {
        __pyx_t_2 = (((__pyx_v_mid - __pyx_v_i) >= 0) != 0);
        if (__pyx_t_2) {
        } else {
          __pyx_t_1 = __pyx_t_2;
          goto __pyx_L8_bool_binop_done;
        }
        __pyx_t_2 = ((__pyx_v_i <= __pyx_v_mid) != 0);
        __pyx_t_1 = __pyx_t_2;
        __pyx_L8_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":873
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid - __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":874
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":875
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":876
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":877
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid - i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */
            __pyx_v_best_index = (__pyx_v_mid - __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":878
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":876
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */
          }

          /* "ms_deisotope/_c/peak_set.pyx":875
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */
          goto __pyx_L10;
        }

        /* "ms_deisotope/_c/peak_set.pyx":880
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             i = 1
 */
        /*else*/ {
          goto __pyx_L7_break;
        }
        __pyx_L10:;

        /* "ms_deisotope/_c/peak_set.pyx":881
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             i = 1
 *             while (mid + i) < (n - 1):
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L7_break:;

      /* "ms_deisotope/_c/peak_set.pyx":882
 *                     break
 *                 i += 1
 *             i = 1             # <<<<<<<<<<<<<<
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 */
      __pyx_v_i = 1;

      /* "ms_deisotope/_c/peak_set.pyx":883
 *                 i += 1
 *             i = 1
 *             while (mid + i) < (n - 1):             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */
      while (1) {
        __pyx_t_1 = (((__pyx_v_mid + __pyx_v_i) < (__pyx_v_n - 1)) != 0);
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":884
 *             i = 1
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid + __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":885
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":886
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":887
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":888
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid + i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */
            __pyx_v_best_index = (__pyx_v_mid + __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":889
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":887
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */
          }

          /* "ms_deisotope/_c/peak_set.pyx":886
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */
          goto __pyx_L14;
        }

        /* "ms_deisotope/_c/peak_set.pyx":891
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             out[0] = best_index
 */
        /*else*/ {
          goto __pyx_L13_break;
        }
        __pyx_L14:;

        /* "ms_deisotope/_c/peak_set.pyx":892
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             out[0] = best_index
 *             if best_error == INF:
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L13_break:;

      /* "ms_deisotope/_c/peak_set.pyx":893
 *                     break
 *                 i += 1
 *             out[0] = best_index             # <<<<<<<<<<<<<<
 *             if best_error == INF:
 *                 return 3
 */
      (__pyx_v_out[0]) = __pyx_v_best_index;

      /* "ms_deisotope/_c/peak_set.pyx":894
 *                 i += 1
 *             out[0] = best_index
 *             if best_error == INF:             # <<<<<<<<<<<<<<
 *                 return 3
 *             return 0
 */
      __pyx_t_1 = ((__pyx_v_best_error == __pyx_v_12ms_deisotope_2_c_8peak_set_INF) != 0);
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":895
 *             out[0] = best_index
 *             if best_error == INF:
 *                 return 3             # <<<<<<<<<<<<<<
 *             return 0
 *         elif hi - lo == 1:
 */
        __pyx_r = 3;
        goto __pyx_L0;

        /* "ms_deisotope/_c/peak_set.pyx":894
 *                 i += 1
 *             out[0] = best_index
 *             if best_error == INF:             # <<<<<<<<<<<<<<
 *                 return 3
 *             return 0
 */
      }

      /* "ms_deisotope/_c/peak_set.pyx":896
 *             if best_error == INF:
 *                 return 3
 *             return 0             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             out[0] = 0
 */
      __pyx_r = 0;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":868
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":897
 *                 return 3
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             out[0] = 0
 *             return 1
 */
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":898
 *             return 0
 *         elif hi - lo == 1:
 *             out[0] = 0             # <<<<<<<<<<<<<<
 *             return 1
 *         elif found_mass > target:
 */
      (__pyx_v_out[0]) = 0;

      /* "ms_deisotope/_c/peak_set.pyx":899
 *         elif hi - lo == 1:
 *             out[0] = 0
 *             return 1             # <<<<<<<<<<<<<<
 *         elif found_mass > target:
 *             hi = mid
 */
      __pyx_r = 1;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":897
 *                 return 3
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             out[0] = 0
 *             return 1
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":900
 *             out[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_found_mass > __pyx_v_target) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":901
 *             return 1
 *         elif found_mass > target:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":900
 *             out[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
      goto __pyx_L5;
    }

    /* "ms_deisotope/_c/peak_set.pyx":903
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 *     out[0] = 0
 *     return 2
 */
    /*else*/ {
      __pyx_v_lo = __pyx_v_mid;
    }
    __pyx_L5:;
  }

  /* "ms_deisotope/_c/peak_set.pyx":904
 *         else:
 *             lo = mid
 *     out[0] = 0             # <<<<<<<<<<<<<<
 *     return 2
 * 
 */
  (__pyx_v_out[0]) = 0;

  /* "ms_deisotope/_c/peak_set.pyx":905
 *             lo = mid
 *     out[0] = 0
 *     return 2             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 2;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":855
 * 
 * 
 * cdef int _binary_search(double* array, double target, double error_tolerance, size_t n, size_t* out) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":908
 * 
 * 
 * cdef int _binary_search_interval(double* array, double target, double error_tolerance, size_t n, size_t* start, size_t* end) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval(double *__pyx_v_array, double __pyx_v_target, double __pyx_v_error_tolerance, size_t __pyx_v_n, size_t *__pyx_v_start, size_t *__pyx_v_end) {
  size_t __pyx_v_lo;
  size_t __pyx_v_hi;
  size_t __pyx_v_mid;
  size_t __pyx_v_i;
  CYTHON_UNUSED size_t __pyx_v_best_index;
  double __pyx_v_found_mass;
  double __pyx_v_best_error;
  double __pyx_v_abs_error;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "ms_deisotope/_c/peak_set.pyx":915
 *         double best_error, abs_error
 * 
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = n
 * 
 */
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/peak_set.pyx":916
 * 
 *     lo = 0
 *     hi = n             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */
  __pyx_v_hi = __pyx_v_n;

  /* "ms_deisotope/_c/peak_set.pyx":918
 *     hi = n
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":919
 * 
 *     while hi != lo:
 *         mid = (hi + lo) / 2             # <<<<<<<<<<<<<<
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 */
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":920
 *     while hi != lo:
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]             # <<<<<<<<<<<<<<
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 */
    __pyx_v_found_mass = (__pyx_v_array[__pyx_v_mid]);

    /* "ms_deisotope/_c/peak_set.pyx":921
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */
    __pyx_t_1 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_found_mass, __pyx_v_target)) < __pyx_v_error_tolerance) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":922
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid             # <<<<<<<<<<<<<<
 *             best_error = INF
 *             i = 0
 */
      __pyx_v_best_index = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":923
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 *             best_error = INF             # <<<<<<<<<<<<<<
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 */
      __pyx_v_best_error = __pyx_v_12ms_deisotope_2_c_8peak_set_INF;

      /* "ms_deisotope/_c/peak_set.pyx":924
 *             best_index = mid
 *             best_error = INF
 *             i = 0             # <<<<<<<<<<<<<<
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 */
      __pyx_v_i = 0;

      /* "ms_deisotope/_c/peak_set.pyx":925
 *             best_error = INF
 *             i = 0
 *             while mid - i >= 0 and i <= mid:             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */
      while (1) {
        __pyx_t_2 = (((__pyx_v_mid - __pyx_v_i) >= 0) != 0);
        if (__pyx_t_2) {
        } else {
          __pyx_t_1 = __pyx_t_2;
          goto __pyx_L8_bool_binop_done;
        }
        __pyx_t_2 = ((__pyx_v_i <= __pyx_v_mid) != 0);
        __pyx_t_1 = __pyx_t_2;
        __pyx_L8_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":926
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid - __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":927
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":928
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":929
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":930
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid - i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */
            __pyx_v_best_index = (__pyx_v_mid - __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":931
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":929
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */
          }

          /* "ms_deisotope/_c/peak_set.pyx":928
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */
          goto __pyx_L10;
        }

        /* "ms_deisotope/_c/peak_set.pyx":933
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             start[0] = mid - i + 1
 */
        /*else*/ {
          goto __pyx_L7_break;
        }
        __pyx_L10:;

        /* "ms_deisotope/_c/peak_set.pyx":934
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             start[0] = mid - i + 1
 *             i = 1
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L7_break:;

      /* "ms_deisotope/_c/peak_set.pyx":935
 *                     break
 *                 i += 1
 *             start[0] = mid - i + 1             # <<<<<<<<<<<<<<
 *             i = 1
 *             while (mid + i) < (n - 1):
 */
      (__pyx_v_start[0]) = ((__pyx_v_mid - __pyx_v_i) + 1);

      /* "ms_deisotope/_c/peak_set.pyx":936
 *                 i += 1
 *             start[0] = mid - i + 1
 *             i = 1             # <<<<<<<<<<<<<<
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 */
      __pyx_v_i = 1;

      /* "ms_deisotope/_c/peak_set.pyx":937
 *             start[0] = mid - i + 1
 *             i = 1
 *             while (mid + i) < (n - 1):             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */
      while (1) {
        __pyx_t_1 = (((__pyx_v_mid + __pyx_v_i) < (__pyx_v_n - 1)) != 0);
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":938
 *             i = 1
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid + __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":939
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":940
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":941
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":942
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid + i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */
            __pyx_v_best_index = (__pyx_v_mid + __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":943
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":941
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */
          }

          /* "ms_deisotope/_c/peak_set.pyx":940
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */
          goto __pyx_L14;
        }

        /* "ms_deisotope/_c/peak_set.pyx":945
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             end[0] = mid + i
 */
        /*else*/ {
          goto __pyx_L13_break;
        }
        __pyx_L14:;

        /* "ms_deisotope/_c/peak_set.pyx":946
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             end[0] = mid + i
 *             return 0
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L13_break:;

      /* "ms_deisotope/_c/peak_set.pyx":947
 *                     break
 *                 i += 1
 *             end[0] = mid + i             # <<<<<<<<<<<<<<
 *             return 0
 *         elif hi - lo == 1:
 */
      (__pyx_v_end[0]) = (__pyx_v_mid + __pyx_v_i);

      /* "ms_deisotope/_c/peak_set.pyx":948
 *                 i += 1
 *             end[0] = mid + i
 *             return 0             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             start[0] = 0
 */
      __pyx_r = 0;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":921
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":949
 *             end[0] = mid + i
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             start[0] = 0
 *             end[0] = 0
 */
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":950
 *             return 0
 *         elif hi - lo == 1:
 *             start[0] = 0             # <<<<<<<<<<<<<<
 *             end[0] = 0
 *             return 1
 */
      (__pyx_v_start[0]) = 0;

      /* "ms_deisotope/_c/peak_set.pyx":951
 *         elif hi - lo == 1:
 *             start[0] = 0
 *             end[0] = 0             # <<<<<<<<<<<<<<
 *             return 1
 *         elif found_mass > target:
 */
      (__pyx_v_end[0]) = 0;

      /* "ms_deisotope/_c/peak_set.pyx":952
 *             start[0] = 0
 *             end[0] = 0
 *             return 1             # <<<<<<<<<<<<<<
 *         elif found_mass > target:
 *             hi = mid
 */
      __pyx_r = 1;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":949
 *             end[0] = mid + i
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             start[0] = 0
 *             end[0] = 0
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":953
 *             end[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_found_mass > __pyx_v_target) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":954
 *             return 1
 *         elif found_mass > target:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":953
 *             end[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
      goto __pyx_L5;
    }

    /* "ms_deisotope/_c/peak_set.pyx":956
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 *     start[0] = 0
 *     end[0] = 0
 */
    /*else*/ {
      __pyx_v_lo = __pyx_v_mid;
    }
    __pyx_L5:;
  }

  /* "ms_deisotope/_c/peak_set.pyx":957
 *         else:
 *             lo = mid
 *     start[0] = 0             # <<<<<<<<<<<<<<
 *     end[0] = 0
 *     return 2
 */
  (__pyx_v_start[0]) = 0;

  /* "ms_deisotope/_c/peak_set.pyx":958
 *             lo = mid
 *     start[0] = 0
 *     end[0] = 0             # <<<<<<<<<<<<<<
 *     return 2
 * 
 */
  (__pyx_v_end[0]) = 0;

  /* "ms_deisotope/_c/peak_set.pyx":959
 *     start[0] = 0
 *     end[0] = 0
 *     return 2             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 2;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":908
 * 
 * 
 * cdef int _binary_search_interval(double* array, double target, double error_tolerance, size_t n, size_t* start, size_t* end) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":962
 * 
 * 
 * cdef DeconvolutedPeak binary_search_neutral_mass(tuple peak_set, double neutral_mass, double error_tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_neutral_mass(PyObject *__pyx_v_peak_set, double __pyx_v_neutral_mass, double __pyx_v_error_tolerance) {
  size_t __pyx_v_lo;
  size_t __pyx_v_hi;
  size_t __pyx_v_mid;
  double __pyx_v_found_mass;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_found_peak = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  double __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("binary_search_neutral_mass", 0);

  /* "ms_deisotope/_c/peak_set.pyx":968
 *         DeconvolutedPeak found_peak
 * 
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = PyTuple_GET_SIZE(peak_set)
 * 
 */
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/peak_set.pyx":969
 * 
 *     lo = 0
 *     hi = PyTuple_GET_SIZE(peak_set)             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */
  __pyx_v_hi = PyTuple_GET_SIZE(__pyx_v_peak_set);

  /* "ms_deisotope/_c/peak_set.pyx":971
 *     hi = PyTuple_GET_SIZE(peak_set)
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) / 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":972
 * 
 *     while hi != lo:
 *         mid = (hi + lo) / 2             # <<<<<<<<<<<<<<
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 *         found_mass = found_peak.neutral_mass
 */
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":973
 *     while hi != lo:
 *         mid = (hi + lo) / 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]             # <<<<<<<<<<<<<<
 *         found_mass = found_peak.neutral_mass
 * 
 */
    if (unlikely(__pyx_v_peak_set == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 973, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_peak_set, __pyx_v_mid, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 973, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF_SET(__pyx_v_found_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":974
 *         mid = (hi + lo) / 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 *         found_mass = found_peak.neutral_mass             # <<<<<<<<<<<<<<
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < error_tolerance:
 */
    __pyx_t_4 = __pyx_v_found_peak->neutral_mass;
    __pyx_v_found_mass = __pyx_t_4;

    /* "ms_deisotope/_c/peak_set.pyx":976
 *         found_mass = found_peak.neutral_mass
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             found_peak = _sweep_solution_neutral_mass(peak_set, neutral_mass, mid, error_tolerance)
 *             return found_peak
 */
    __pyx_t_1 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_found_mass, __pyx_v_neutral_mass)) < __pyx_v_error_tolerance) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":977
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < error_tolerance:
 *             found_peak = _sweep_solution_neutral_mass(peak_set, neutral_mass, mid, error_tolerance)             # <<<<<<<<<<<<<<
 *             return found_peak
 *         elif hi - lo == 1:
 */
      __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_solution_neutral_mass(__pyx_v_peak_set, __pyx_v_neutral_mass, __pyx_v_mid, __pyx_v_error_tolerance)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 977, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF_SET(__pyx_v_found_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "ms_deisotope/_c/peak_set.pyx":978
 *         if abs(ppm_error(found_mass, neutral_mass)) < error_tolerance:
 *             found_peak = _sweep_solution_neutral_mass(peak_set, neutral_mass, mid, error_tolerance)
 *             return found_peak             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             return None
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(((PyObject *)__pyx_v_found_peak));
      __pyx_r = __pyx_v_found_peak;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":976
 *         found_mass = found_peak.neutral_mass
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             found_peak = _sweep_solution_neutral_mass(peak_set, neutral_mass, mid, error_tolerance)
 *             return found_peak
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":979
 *             found_peak = _sweep_solution_neutral_mass(peak_set, neutral_mass, mid, error_tolerance)
 *             return found_peak
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return None
 *         elif found_mass > neutral_mass:
 */
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":980
 *             return found_peak
 *         elif hi - lo == 1:
 *             return None             # <<<<<<<<<<<<<<
 *         elif found_mass > neutral_mass:
 *             hi = mid
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":979
 *             found_peak = _sweep_solution_neutral_mass(peak_set, neutral_mass, mid, error_tolerance)
 *             return found_peak
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return None
 *         elif found_mass > neutral_mass:
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":981
 *         elif hi - lo == 1:
 *             return None
 *         elif found_mass > neutral_mass:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_found_mass > __pyx_v_neutral_mass) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":982
 *             return None
 *         elif found_mass > neutral_mass:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":981
 *         elif hi - lo == 1:
 *             return None
 *         elif found_mass > neutral_mass:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
      goto __pyx_L5;
    }

    /* "ms_deisotope/_c/peak_set.pyx":984
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 * 
 * 
 */
    /*else*/ {
      __pyx_v_lo = __pyx_v_mid;
    }
    __pyx_L5:;
  }

  /* "ms_deisotope/_c/peak_set.pyx":962
 * 
 * 
 * cdef DeconvolutedPeak binary_search_neutral_mass(tuple peak_set, double neutral_mass, double error_tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

  /* function exit code */
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.binary_search_neutral_mass", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_found_peak);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":987
 * 
 * 
 * cdef DeconvolutedPeak _sweep_solution_mz(tuple array, double value, size_t mid, double tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t best_index, i, n
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_solution_mz(PyObject *__pyx_v_array, double __pyx_v_value, size_t __pyx_v_mid, double __pyx_v_tolerance) {
  size_t __pyx_v_best_index;
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  double __pyx_v_best_error;
  double __pyx_v_abs_error;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_target = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_sweep_solution_mz", 0);

  /* "ms_deisotope/_c/peak_set.pyx":993
 *         DeconvolutedPeak target
 * 
 *     best_index = mid             # <<<<<<<<<<<<<<
 *     best_error = INF
 *     n = PyTuple_GET_SIZE(array)
 */
  __pyx_v_best_index = __pyx_v_mid;

  /* "ms_deisotope/_c/peak_set.pyx":994
 * 
 *     best_index = mid
 *     best_error = INF             # <<<<<<<<<<<<<<
 *     n = PyTuple_GET_SIZE(array)
 * 
 */
  __pyx_v_best_error = __pyx_v_12ms_deisotope_2_c_8peak_set_INF;

  /* "ms_deisotope/_c/peak_set.pyx":995
 *     best_index = mid
 *     best_error = INF
 *     n = PyTuple_GET_SIZE(array)             # <<<<<<<<<<<<<<
 * 
 *     i = 0
 */
  __pyx_v_n = PyTuple_GET_SIZE(__pyx_v_array);

  /* "ms_deisotope/_c/peak_set.pyx":997
 *     n = PyTuple_GET_SIZE(array)
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     while mid - i >= 0 and i <= mid:
 *         target = <DeconvolutedPeak>array[mid - i]
 */
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":998
 * 
 *     i = 0
 *     while mid - i >= 0 and i <= mid:             # <<<<<<<<<<<<<<
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.mz))
 */
  while (1) {
    __pyx_t_2 = (((__pyx_v_mid - __pyx_v_i) >= 0) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_i <= __pyx_v_mid) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L5_bool_binop_done:;
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":999
 *     i = 0
 *     while mid - i >= 0 and i <= mid:
 *         target = <DeconvolutedPeak>array[mid - i]             # <<<<<<<<<<<<<<
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:
 */
    if (unlikely(__pyx_v_array == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 999, __pyx_L1_error)
    }
    __pyx_t_3 = (__pyx_v_mid - __pyx_v_i);
    __pyx_t_4 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_t_3, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 999, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __pyx_t_4;
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_target, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1000
 *     while mid - i >= 0 and i <= mid:
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.mz))             # <<<<<<<<<<<<<<
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 */
    __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_value, __pyx_v_target->__pyx_base.mz));

    /* "ms_deisotope/_c/peak_set.pyx":1001
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid - i
 */
    __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_tolerance) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1002
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid - i
 *                 best_error = abs_error
 */
      __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":1003
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 *                 best_index = mid - i             # <<<<<<<<<<<<<<
 *                 best_error = abs_error
 *         else:
 */
        __pyx_v_best_index = (__pyx_v_mid - __pyx_v_i);

        /* "ms_deisotope/_c/peak_set.pyx":1004
 *             if abs_error < best_error:
 *                 best_index = mid - i
 *                 best_error = abs_error             # <<<<<<<<<<<<<<
 *         else:
 *             break
 */
        __pyx_v_best_error = __pyx_v_abs_error;

        /* "ms_deisotope/_c/peak_set.pyx":1002
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid - i
 *                 best_error = abs_error
 */
      }

      /* "ms_deisotope/_c/peak_set.pyx":1001
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid - i
 */
      goto __pyx_L7;
    }

    /* "ms_deisotope/_c/peak_set.pyx":1006
 *                 best_error = abs_error
 *         else:
 *             break             # <<<<<<<<<<<<<<
 *         i += 1
 *     i = 1
 */
    /*else*/ {
      goto __pyx_L4_break;
    }
    __pyx_L7:;

    /* "ms_deisotope/_c/peak_set.pyx":1007
 *         else:
 *             break
 *         i += 1             # <<<<<<<<<<<<<<
 *     i = 1
 *     while (mid + i) < (n - 1):
 */
    __pyx_v_i = (__pyx_v_i + 1);
  }
  __pyx_L4_break:;

  /* "ms_deisotope/_c/peak_set.pyx":1008
 *             break
 *         i += 1
 *     i = 1             # <<<<<<<<<<<<<<
 *     while (mid + i) < (n - 1):
 *         target = <DeconvolutedPeak>array[mid + i]
 */
  __pyx_v_i = 1;

  /* "ms_deisotope/_c/peak_set.pyx":1009
 *         i += 1
 *     i = 1
 *     while (mid + i) < (n - 1):             # <<<<<<<<<<<<<<
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.mz))
 */
  while (1) {
    __pyx_t_1 = (((__pyx_v_mid + __pyx_v_i) < (__pyx_v_n - 1)) != 0);
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":1010
 *     i = 1
 *     while (mid + i) < (n - 1):
 *         target = <DeconvolutedPeak>array[mid + i]             # <<<<<<<<<<<<<<
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:
 */
    if (unlikely(__pyx_v_array == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 1010, __pyx_L1_error)
    }
    __pyx_t_3 = (__pyx_v_mid + __pyx_v_i);
    __pyx_t_5 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_t_3, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1010, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_target, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1011
 *     while (mid + i) < (n - 1):
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.mz))             # <<<<<<<<<<<<<<
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 */
    __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_value, __pyx_v_target->__pyx_base.mz));

    /* "ms_deisotope/_c/peak_set.pyx":1012
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid + i
 */
    __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_tolerance) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1013
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid + i
 *                 best_error = abs_error
 */
      __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":1014
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 *                 best_index = mid + i             # <<<<<<<<<<<<<<
 *                 best_error = abs_error
 *         else:
 */
        __pyx_v_best_index = (__pyx_v_mid + __pyx_v_i);

        /* "ms_deisotope/_c/peak_set.pyx":1015
 *             if abs_error < best_error:
 *                 best_index = mid + i
 *                 best_error = abs_error             # <<<<<<<<<<<<<<
 *         else:
 *             break
 */
        __pyx_v_best_error = __pyx_v_abs_error;

        /* "ms_deisotope/_c/peak_set.pyx":1013
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid + i
 *                 best_error = abs_error
 */
      }

      /* "ms_deisotope/_c/peak_set.pyx":1012
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid + i
 */
      goto __pyx_L11;
    }

    /* "ms_deisotope/_c/peak_set.pyx":1017
 *                 best_error = abs_error
 *         else:
 *             break             # <<<<<<<<<<<<<<
 *         i += 1
 *     if best_error == INF:
 */
    /*else*/ {
      goto __pyx_L10_break;
    }
    __pyx_L11:;

    /* "ms_deisotope/_c/peak_set.pyx":1018
 *         else:
 *             break
 *         i += 1             # <<<<<<<<<<<<<<
 *     if best_error == INF:
 *         return None
 */
    __pyx_v_i = (__pyx_v_i + 1);
  }
  __pyx_L10_break:;

  /* "ms_deisotope/_c/peak_set.pyx":1019
 *             break
 *         i += 1
 *     if best_error == INF:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_best_error == __pyx_v_12ms_deisotope_2_c_8peak_set_INF) != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1020
 *         i += 1
 *     if best_error == INF:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return array[best_index]
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1019
 *             break
 *         i += 1
 *     if best_error == INF:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1022
 *         return None
 *     else:
 *         return array[best_index]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    if (unlikely(__pyx_v_array == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 1022, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_v_best_index, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1022, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 1022, __pyx_L1_error)
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4);
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "ms_deisotope/_c/peak_set.pyx":987
 * 
 * 
 * cdef DeconvolutedPeak _sweep_solution_mz(tuple array, double value, size_t mid, double tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t best_index, i, n
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._sweep_solution_mz", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_target);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1025
 * 
 * 
 * cdef DeconvolutedPeak binary_search_mz(tuple peak_set, double mz, double error_tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_mz(PyObject *__pyx_v_peak_set, double __pyx_v_mz, double __pyx_v_error_tolerance) {
  size_t __pyx_v_lo;
  size_t __pyx_v_hi;
  size_t __pyx_v_mid;
  double __pyx_v_found_mass;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_found_peak = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("binary_search_mz", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1031
 *         DeconvolutedPeak found_peak
 * 
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = len(peak_set)
 * 
 */
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1032
 * 
 *     lo = 0
 *     hi = len(peak_set)             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */
  if (unlikely(__pyx_v_peak_set == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 1032, __pyx_L1_error)
  }
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_peak_set); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1032, __pyx_L1_error)
  __pyx_v_hi = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1034
 *     hi = len(peak_set)
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) / 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_hi != __pyx_v_lo) != 0);
    if (!__pyx_t_2) break;

    /* "ms_deisotope/_c/peak_set.pyx":1035
 * 
 *     while hi != lo:
 *         mid = (hi + lo) / 2             # <<<<<<<<<<<<<<
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 *         found_mass = found_peak.mz
 */
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":1036
 *     while hi != lo:
 *         mid = (hi + lo) / 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]             # <<<<<<<<<<<<<<
 *         found_mass = found_peak.mz
 * 
 */
    if (unlikely(__pyx_v_peak_set == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 1036, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_peak_set, __pyx_v_mid, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1036, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __pyx_t_3;
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_found_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1037
 *         mid = (hi + lo) / 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 *         found_mass = found_peak.mz             # <<<<<<<<<<<<<<
 * 
 *         if abs(ppm_error(found_mass, mz)) < error_tolerance:
 */
    __pyx_t_5 = __pyx_v_found_peak->__pyx_base.mz;
    __pyx_v_found_mass = __pyx_t_5;

    /* "ms_deisotope/_c/peak_set.pyx":1039
 *         found_mass = found_peak.mz
 * 
 *         if abs(ppm_error(found_mass, mz)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             found_peak = _sweep_solution_mz(peak_set, mz, mid, error_tolerance)
 *             return found_peak
 */
    __pyx_t_2 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_found_mass, __pyx_v_mz)) < __pyx_v_error_tolerance) != 0);
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1040
 * 
 *         if abs(ppm_error(found_mass, mz)) < error_tolerance:
 *             found_peak = _sweep_solution_mz(peak_set, mz, mid, error_tolerance)             # <<<<<<<<<<<<<<
 *             return found_peak
 *         elif hi - lo == 1:
 */
      __pyx_t_4 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_solution_mz(__pyx_v_peak_set, __pyx_v_mz, __pyx_v_mid, __pyx_v_error_tolerance)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1040, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF_SET(__pyx_v_found_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4));
      __pyx_t_4 = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1041
 *         if abs(ppm_error(found_mass, mz)) < error_tolerance:
 *             found_peak = _sweep_solution_mz(peak_set, mz, mid, error_tolerance)
 *             return found_peak             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             return None
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(((PyObject *)__pyx_v_found_peak));
      __pyx_r = __pyx_v_found_peak;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1039
 *         found_mass = found_peak.mz
 * 
 *         if abs(ppm_error(found_mass, mz)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             found_peak = _sweep_solution_mz(peak_set, mz, mid, error_tolerance)
 *             return found_peak
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":1042
 *             found_peak = _sweep_solution_mz(peak_set, mz, mid, error_tolerance)
 *             return found_peak
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return None
 *         elif found_mass > mz:
 */
    __pyx_t_2 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1043
 *             return found_peak
 *         elif hi - lo == 1:
 *             return None             # <<<<<<<<<<<<<<
 *         elif found_mass > mz:
 *             hi = mid
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1042
 *             found_peak = _sweep_solution_mz(peak_set, mz, mid, error_tolerance)
 *             return found_peak
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return None
 *         elif found_mass > mz:
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":1044
 *         elif hi - lo == 1:
 *             return None
 *         elif found_mass > mz:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
    __pyx_t_2 = ((__pyx_v_found_mass > __pyx_v_mz) != 0);
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1045
 *             return None
 *         elif found_mass > mz:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":1044
 *         elif hi - lo == 1:
 *             return None
 *         elif found_mass > mz:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
      goto __pyx_L5;
    }

    /* "ms_deisotope/_c/peak_set.pyx":1047
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 * 
 * 
 */
    /*else*/ {
      __pyx_v_lo = __pyx_v_mid;
    }
    __pyx_L5:;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1025
 * 
 * 
 * cdef DeconvolutedPeak binary_search_mz(tuple peak_set, double mz, double error_tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

  /* function exit code */
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.binary_search_mz", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_found_peak);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1050
 * 
 * 
 * cdef DeconvolutedPeak _sweep_nearest_match_neutral_mass(tuple array, double value, size_t lo, size_t hi, double* errout):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_nearest_match_neutral_mass(PyObject *__pyx_v_array, double __pyx_v_value, size_t __pyx_v_lo, size_t __pyx_v_hi, double *__pyx_v_errout) {
  size_t __pyx_v_i;
  size_t __pyx_v_best_index;
  double __pyx_v_best_error;
  double __pyx_v_err;
  double __pyx_v_v;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_sweep_nearest_match_neutral_mass", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1057
 *         double v
 * 
 *     best_error = float('inf')             # <<<<<<<<<<<<<<
 *     best_index = -1
 *     for i in range(hi - lo):
 */
  __pyx_t_1 = __Pyx_PyObject_AsDouble(__pyx_n_s_inf); if (unlikely(__pyx_t_1 == ((double)((double)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1057, __pyx_L1_error)
  __pyx_v_best_error = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1058
 * 
 *     best_error = float('inf')
 *     best_index = -1             # <<<<<<<<<<<<<<
 *     for i in range(hi - lo):
 *         i += lo
 */
  __pyx_v_best_index = -1L;

  /* "ms_deisotope/_c/peak_set.pyx":1059
 *     best_error = float('inf')
 *     best_index = -1
 *     for i in range(hi - lo):             # <<<<<<<<<<<<<<
 *         i += lo
 *         v = array[i].neutral_mass
 */
  __pyx_t_2 = (__pyx_v_hi - __pyx_v_lo);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "ms_deisotope/_c/peak_set.pyx":1060
 *     best_index = -1
 *     for i in range(hi - lo):
 *         i += lo             # <<<<<<<<<<<<<<
 *         v = array[i].neutral_mass
 *         err = abs(v - value)
 */
    __pyx_v_i = (__pyx_v_i + __pyx_v_lo);

    /* "ms_deisotope/_c/peak_set.pyx":1061
 *     for i in range(hi - lo):
 *         i += lo
 *         v = array[i].neutral_mass             # <<<<<<<<<<<<<<
 *         err = abs(v - value)
 *         if err < best_error:
 */
    if (unlikely(__pyx_v_array == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 1061, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_v_i, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1061, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_neutral_mass); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1061, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1061, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_v = __pyx_t_1;

    /* "ms_deisotope/_c/peak_set.pyx":1062
 *         i += lo
 *         v = array[i].neutral_mass
 *         err = abs(v - value)             # <<<<<<<<<<<<<<
 *         if err < best_error:
 *             best_error = err
 */
    __pyx_v_err = fabs((__pyx_v_v - __pyx_v_value));

    /* "ms_deisotope/_c/peak_set.pyx":1063
 *         v = array[i].neutral_mass
 *         err = abs(v - value)
 *         if err < best_error:             # <<<<<<<<<<<<<<
 *             best_error = err
 *             best_index = i
 */
    __pyx_t_7 = ((__pyx_v_err < __pyx_v_best_error) != 0);
    if (__pyx_t_7) {

      /* "ms_deisotope/_c/peak_set.pyx":1064
 *         err = abs(v - value)
 *         if err < best_error:
 *             best_error = err             # <<<<<<<<<<<<<<
 *             best_index = i
 *     errout[0] = best_error
 */
      __pyx_v_best_error = __pyx_v_err;

      /* "ms_deisotope/_c/peak_set.pyx":1065
 *         if err < best_error:
 *             best_error = err
 *             best_index = i             # <<<<<<<<<<<<<<
 *     errout[0] = best_error
 *     return array[best_index]
 */
      __pyx_v_best_index = __pyx_v_i;

      /* "ms_deisotope/_c/peak_set.pyx":1063
 *         v = array[i].neutral_mass
 *         err = abs(v - value)
 *         if err < best_error:             # <<<<<<<<<<<<<<
 *             best_error = err
 *             best_index = i
 */
    }
  }

  /* "ms_deisotope/_c/peak_set.pyx":1066
 *             best_error = err
 *             best_index = i
 *     errout[0] = best_error             # <<<<<<<<<<<<<<
 *     return array[best_index]
 * 
 */
  (__pyx_v_errout[0]) = __pyx_v_best_error;

  /* "ms_deisotope/_c/peak_set.pyx":1067
 *             best_index = i
 *     errout[0] = best_error
 *     return array[best_index]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  if (unlikely(__pyx_v_array == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 1067, __pyx_L1_error)
  }
  __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_v_best_index, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1067, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 1067, __pyx_L1_error)
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_6);
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1050
 * 
 * 
 * cdef DeconvolutedPeak _sweep_nearest_match_neutral_mass(tuple array, double value, size_t lo, size_t hi, double* errout):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._sweep_nearest_match_neutral_mass", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1070
 * 
 * 
 * cdef DeconvolutedPeak binary_search_nearest_neutral_mass(tuple peak_set, double neutral_mass, double* errout):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_nearest_neutral_mass(PyObject *__pyx_v_peak_set, double __pyx_v_neutral_mass, double *__pyx_v_errout) {
  size_t __pyx_v_lo;
  size_t __pyx_v_hi;
  size_t __pyx_v_mid;
  double __pyx_v_found_mass;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_found_peak = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("binary_search_nearest_neutral_mass", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1076
 *         DeconvolutedPeak found_peak
 * 
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = len(peak_set)
 *     while hi != lo:
 */
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1077
 * 
 *     lo = 0
 *     hi = len(peak_set)             # <<<<<<<<<<<<<<
 *     while hi != lo:
 *         mid = (hi + lo) / 2
 */
  if (unlikely(__pyx_v_peak_set == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 1077, __pyx_L1_error)
  }
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_peak_set); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1077, __pyx_L1_error)
  __pyx_v_hi = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1078
 *     lo = 0
 *     hi = len(peak_set)
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) / 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_hi != __pyx_v_lo) != 0);
    if (!__pyx_t_2) break;

    /* "ms_deisotope/_c/peak_set.pyx":1079
 *     hi = len(peak_set)
 *     while hi != lo:
 *         mid = (hi + lo) / 2             # <<<<<<<<<<<<<<
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 *         found_mass = found_peak.neutral_mass
 */
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":1080
 *     while hi != lo:
 *         mid = (hi + lo) / 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]             # <<<<<<<<<<<<<<
 *         found_mass = found_peak.neutral_mass
 * 
 */
    if (unlikely(__pyx_v_peak_set == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 1080, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_peak_set, __pyx_v_mid, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1080, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __pyx_t_3;
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_found_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1081
 *         mid = (hi + lo) / 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 *         found_mass = found_peak.neutral_mass             # <<<<<<<<<<<<<<
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < 1:
 */
    __pyx_t_5 = __pyx_v_found_peak->neutral_mass;
    __pyx_v_found_mass = __pyx_t_5;

    /* "ms_deisotope/_c/peak_set.pyx":1083
 *         found_mass = found_peak.neutral_mass
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < 1:             # <<<<<<<<<<<<<<
 *             found_peak = _sweep_nearest_match_neutral_mass(peak_set, neutral_mass, lo, hi, errout)
 *             return found_peak
 */
    __pyx_t_2 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_found_mass, __pyx_v_neutral_mass)) < 1.0) != 0);
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1084
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < 1:
 *             found_peak = _sweep_nearest_match_neutral_mass(peak_set, neutral_mass, lo, hi, errout)             # <<<<<<<<<<<<<<
 *             return found_peak
 *         elif hi - lo == 1:
 */
      __pyx_t_4 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_nearest_match_neutral_mass(__pyx_v_peak_set, __pyx_v_neutral_mass, __pyx_v_lo, __pyx_v_hi, __pyx_v_errout)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1084, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF_SET(__pyx_v_found_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4));
      __pyx_t_4 = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1085
 *         if abs(ppm_error(found_mass, neutral_mass)) < 1:
 *             found_peak = _sweep_nearest_match_neutral_mass(peak_set, neutral_mass, lo, hi, errout)
 *             return found_peak             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             errout[0] = found_mass - neutral_mass
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(((PyObject *)__pyx_v_found_peak));
      __pyx_r = __pyx_v_found_peak;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1083
 *         found_mass = found_peak.neutral_mass
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < 1:             # <<<<<<<<<<<<<<
 *             found_peak = _sweep_nearest_match_neutral_mass(peak_set, neutral_mass, lo, hi, errout)
 *             return found_peak
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":1086
 *             found_peak = _sweep_nearest_match_neutral_mass(peak_set, neutral_mass, lo, hi, errout)
 *             return found_peak
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             errout[0] = found_mass - neutral_mass
 *             return found_peak
 */
    __pyx_t_2 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1087
 *             return found_peak
 *         elif hi - lo == 1:
 *             errout[0] = found_mass - neutral_mass             # <<<<<<<<<<<<<<
 *             return found_peak
 *         elif found_mass > neutral_mass:
 */
      (__pyx_v_errout[0]) = (__pyx_v_found_mass - __pyx_v_neutral_mass);

      /* "ms_deisotope/_c/peak_set.pyx":1088
 *         elif hi - lo == 1:
 *             errout[0] = found_mass - neutral_mass
 *             return found_peak             # <<<<<<<<<<<<<<
 *         elif found_mass > neutral_mass:
 *             hi = mid
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(((PyObject *)__pyx_v_found_peak));
      __pyx_r = __pyx_v_found_peak;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1086
 *             found_peak = _sweep_nearest_match_neutral_mass(peak_set, neutral_mass, lo, hi, errout)
 *             return found_peak
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             errout[0] = found_mass - neutral_mass
 *             return found_peak
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":1089
 *             errout[0] = found_mass - neutral_mass
 *             return found_peak
 *         elif found_mass > neutral_mass:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
    __pyx_t_2 = ((__pyx_v_found_mass > __pyx_v_neutral_mass) != 0);
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1090
 *             return found_peak
 *         elif found_mass > neutral_mass:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":1089
 *             errout[0] = found_mass - neutral_mass
 *             return found_peak
 *         elif found_mass > neutral_mass:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
      goto __pyx_L5;
    }

    /* "ms_deisotope/_c/peak_set.pyx":1092
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 * 
 * 
 */
    /*else*/ {
      __pyx_v_lo = __pyx_v_mid;
    }
    __pyx_L5:;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1070
 * 
 * 
 * cdef DeconvolutedPeak binary_search_nearest_neutral_mass(tuple peak_set, double neutral_mass, double* errout):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

  /* function exit code */
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.binary_search_nearest_neutral_mass", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_found_peak);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1095
 * 
 * 
 * def convert(self):             # <<<<<<<<<<<<<<
 *     return DeconvolutedPeak(
 *         self.neutral_mass, self.intensity, self.charge,
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_13convert(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_12convert[] = "convert(self)";
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_8peak_set_13convert = {"convert", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_13convert, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_12convert};
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_13convert(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("convert (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_12convert(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12convert(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("convert", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1096
 * 
 * def convert(self):
 *     return DeconvolutedPeak(             # <<<<<<<<<<<<<<
 *         self.neutral_mass, self.intensity, self.charge,
 *         self.signal_to_noise, -1, self.full_width_at_half_max,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/peak_set.pyx":1097
 * def convert(self):
 *     return DeconvolutedPeak(
 *         self.neutral_mass, self.intensity, self.charge,             # <<<<<<<<<<<<<<
 *         self.signal_to_noise, -1, self.full_width_at_half_max,
 *         self.a_to_a2_ratio, self.most_abundant_mass, self.average_mass,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1097, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1097, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1097, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ms_deisotope/_c/peak_set.pyx":1098
 *     return DeconvolutedPeak(
 *         self.neutral_mass, self.intensity, self.charge,
 *         self.signal_to_noise, -1, self.full_width_at_half_max,             # <<<<<<<<<<<<<<
 *         self.a_to_a2_ratio, self.most_abundant_mass, self.average_mass,
 *         self.score, Envelope(self.envelope), self.mz, None, self.chosen_for_msms,
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_signal_to_noise); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1098, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_full_width_at_half_max); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1098, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "ms_deisotope/_c/peak_set.pyx":1099
 *         self.neutral_mass, self.intensity, self.charge,
 *         self.signal_to_noise, -1, self.full_width_at_half_max,
 *         self.a_to_a2_ratio, self.most_abundant_mass, self.average_mass,             # <<<<<<<<<<<<<<
 *         self.score, Envelope(self.envelope), self.mz, None, self.chosen_for_msms,
 *         self.area)
 */
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_a_to_a2_ratio); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1099, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_most_abundant_mass); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1099, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_average_mass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1099, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);

  /* "ms_deisotope/_c/peak_set.pyx":1100
 *         self.signal_to_noise, -1, self.full_width_at_half_max,
 *         self.a_to_a2_ratio, self.most_abundant_mass, self.average_mass,
 *         self.score, Envelope(self.envelope), self.mz, None, self.chosen_for_msms,             # <<<<<<<<<<<<<<
 *         self.area)
 * 
 */
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_score); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_envelope); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope), __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_mz); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_chosen_for_msms); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);

  /* "ms_deisotope/_c/peak_set.pyx":1101
 *         self.a_to_a2_ratio, self.most_abundant_mass, self.average_mass,
 *         self.score, Envelope(self.envelope), self.mz, None, self.chosen_for_msms,
 *         self.area)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_area); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);

  /* "ms_deisotope/_c/peak_set.pyx":1096
 * 
 * def convert(self):
 *     return DeconvolutedPeak(             # <<<<<<<<<<<<<<
 *         self.neutral_mass, self.intensity, self.charge,
 *         self.signal_to_noise, -1, self.full_width_at_half_max,
 */
  __pyx_t_14 = PyTuple_New(15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1096, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_14, 3, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_neg_1);
  __Pyx_GIVEREF(__pyx_int_neg_1);
  PyTuple_SET_ITEM(__pyx_t_14, 4, __pyx_int_neg_1);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_14, 5, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_14, 6, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_14, 7, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_14, 8, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_14, 9, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_14, 10, __pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_14, 11, __pyx_t_10);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_14, 12, Py_None);
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_14, 13, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_14, 14, __pyx_t_13);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_11 = 0;
  __pyx_t_10 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak), __pyx_t_14, NULL); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1096, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1095
 * 
 * 
 * def convert(self):             # <<<<<<<<<<<<<<
 *     return DeconvolutedPeak(
 *         self.neutral_mass, self.intensity, self.charge,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.convert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1106
 * @cython.boundscheck(False)
 * @cython.wraparound(False)
 * def decode_envelopes(np.ndarray[cython.floating, ndim=1, mode='c'] array):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list envelope_list, current_envelope
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_15decode_envelopes(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_14decode_envelopes[] = "decode_envelopes(signatures, args, kwargs, defaults)";
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_8peak_set_15decode_envelopes = {"decode_envelopes", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_15decode_envelopes, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_14decode_envelopes};
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_15decode_envelopes(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); __PYX_ERR(0, 1106, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); __PYX_ERR(0, 1106, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); __PYX_ERR(0, 1106, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) __PYX_ERR(0, 1106, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1106, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_14decode_envelopes(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_14decode_envelopes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  Py_ssize_t __pyx_v_i;
  PyTypeObject *__pyx_v_ndarray = 0;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_sig = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  long __pyx_t_7;
  __Pyx_memviewslice __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_t_18;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("decode_envelopes", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_v_kwargs != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_kwargs); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __pyx_t_3 = ((!__pyx_t_4) != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {
    __Pyx_INCREF(Py_None);
    __Pyx_DECREF_SET(__pyx_v_kwargs, Py_None);
  }
  __pyx_t_1 = ((PyObject *)__Pyx_ImportNumPyArrayTypeIfAvailable()); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 1106, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1106, __pyx_L1_error)
  __pyx_t_2 = ((0 < __pyx_t_5) != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 1106, __pyx_L1_error)
    }
    __pyx_t_1 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_arg = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L6;
  }
  __pyx_t_3 = (__pyx_v_kwargs != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L7_bool_binop_done;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 1106, __pyx_L1_error)
  }
  __pyx_t_4 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_array, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __pyx_t_3 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 1106, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_array); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_arg = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L6;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 1106, __pyx_L1_error)
    }
    __pyx_t_5 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1106, __pyx_L1_error)
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_int_1);
    __Pyx_GIVEREF(__pyx_int_1);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_int_1);
    __Pyx_INCREF(__pyx_kp_s__2);
    __Pyx_GIVEREF(__pyx_kp_s__2);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_kp_s__2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_argument_s_g, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 1106, __pyx_L1_error)
  }
  __pyx_L6:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_v_dtype = __pyx_t_6;
        __pyx_t_6 = 0;
        goto __pyx_L12;
      }
      __pyx_t_2 = __pyx_memoryview_check(__pyx_v_arg); 
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_v_arg_base = __pyx_t_6;
        __pyx_t_6 = 0;
        __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_2 = (__pyx_t_3 != 0);
        if (__pyx_t_2) {
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_v_dtype = __pyx_t_6;
          __pyx_t_6 = 0;
          goto __pyx_L13;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L13:;
        goto __pyx_L12;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L12:;
      __pyx_v_itemsize = -1L;
      __pyx_t_2 = (__pyx_v_dtype != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1106, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_v_itemsize = __pyx_t_5;
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_Ord(__pyx_t_6); if (unlikely(__pyx_t_7 == ((long)(long)(Py_UCS4)-1))) __PYX_ERR(0, 1106, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_v_kind = __pyx_t_7;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_2 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_2) {
          } else {
            __pyx_t_3 = __pyx_t_2;
            goto __pyx_L16_bool_binop_done;
          }
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1106, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __pyx_t_2 = ((((Py_ssize_t)__pyx_t_5) == 1) != 0);
          __pyx_t_3 = __pyx_t_2;
          __pyx_L16_bool_binop_done:;
          if (__pyx_t_3) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)
            goto __pyx_L10_break;
          }
          __pyx_t_2 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_2) {
          } else {
            __pyx_t_3 = __pyx_t_2;
            goto __pyx_L19_bool_binop_done;
          }
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1106, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __pyx_t_2 = ((((Py_ssize_t)__pyx_t_5) == 1) != 0);
          __pyx_t_3 = __pyx_t_2;
          __pyx_L19_bool_binop_done:;
          if (__pyx_t_3) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)
            goto __pyx_L10_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_2 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L22_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_3 = __pyx_t_2;
    __pyx_L22_bool_binop_done:;
    if (__pyx_t_3) {
      __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_v_arg, 0); 
      __pyx_v_memslice = __pyx_t_8;
      __pyx_t_3 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_3) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1); 
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)
        goto __pyx_L10_break;
      }
      /*else*/ {
        PyErr_Clear(); 
      }
    }
    __pyx_t_2 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L26_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_3 = __pyx_t_2;
    __pyx_L26_bool_binop_done:;
    if (__pyx_t_3) {
      __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_v_arg, 0); 
      __pyx_v_memslice = __pyx_t_8;
      __pyx_t_3 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_3) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1); 
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)
        goto __pyx_L10_break;
      }
      /*else*/ {
        PyErr_Clear(); 
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)
    goto __pyx_L10_break;
  }
  __pyx_L10_break:;
  __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v_candidates = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_5 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 1106, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_9), (&__pyx_t_10)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __pyx_t_6 = __pyx_t_1;
  __pyx_t_1 = 0;
  while (1) {
    __pyx_t_11 = __Pyx_dict_iter_next(__pyx_t_6, __pyx_t_9, &__pyx_t_5, &__pyx_t_1, NULL, NULL, __pyx_t_10);
    if (unlikely(__pyx_t_11 == 0)) break;
    if (unlikely(__pyx_t_11 == -1)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __pyx_t_14 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
      __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_13);
      if (likely(__pyx_t_14)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
        __Pyx_INCREF(__pyx_t_14);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_13, function);
      }
    }
    __pyx_t_12 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_14, __pyx_kp_s__3) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_kp_s__3);
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_split); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_12 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
      __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_13);
      if (likely(__pyx_t_12)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
        __Pyx_INCREF(__pyx_t_12);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_13, function);
      }
    }
    __pyx_t_1 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_12, __pyx_kp_s__4) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_kp_s__4);
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_XDECREF_SET(__pyx_v_src_sig, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_15 = PyList_GET_SIZE(__pyx_v_dest_sig); if (unlikely(__pyx_t_15 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1106, __pyx_L1_error)
    __pyx_t_16 = __pyx_t_15;
    for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
      __pyx_v_i = __pyx_t_17;
      __pyx_t_1 = PyList_GET_ITEM(__pyx_v_dest_sig, __pyx_v_i);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_3 = (__pyx_v_dst_type != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_src_sig, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_13 = PyObject_RichCompare(__pyx_t_1, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_13); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1106, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_13); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (__pyx_t_2) {
          __pyx_v_match_found = 1;
          goto __pyx_L34;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L32_break;
        }
        __pyx_L34:;
      }
    }
    __pyx_L32_break:;
    __pyx_t_2 = (__pyx_v_match_found != 0);
    if (__pyx_t_2) {
      __pyx_t_18 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_18 == ((int)-1))) __PYX_ERR(0, 1106, __pyx_L1_error)
    }
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_2 = (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (__pyx_t_3) {
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 1106, __pyx_L1_error)
  }
  __pyx_t_9 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1106, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_9 > 1) != 0);
  if (__pyx_t_3) {
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 1106, __pyx_L1_error)
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 1106, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_sig);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_12ms_deisotope_2_c_8peak_set_17decode_envelopes(PyObject *__pyx_self, PyObject *__pyx_v_array); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_12ms_deisotope_2_c_8peak_set_17decode_envelopes = {"__pyx_fuse_0decode_envelopes", (PyCFunction)__pyx_fuse_0__pyx_pw_12ms_deisotope_2_c_8peak_set_17decode_envelopes, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_14decode_envelopes};
static PyObject *__pyx_fuse_0__pyx_pw_12ms_deisotope_2_c_8peak_set_17decode_envelopes(PyObject *__pyx_self, PyObject *__pyx_v_array) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("decode_envelopes (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_array), __pyx_ptype_5numpy_ndarray, 1, "array", 0))) __PYX_ERR(0, 1106, __pyx_L1_error)
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_16decode_envelopes(__pyx_self, ((PyArrayObject *)__pyx_v_array));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16decode_envelopes(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_array) {
  PyObject *__pyx_v_envelope_list = 0;
  PyObject *__pyx_v_current_envelope = 0;
  PyObject *__pyx_v_converted = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_envelope_obj = 0;
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  float __pyx_v_a;
  float __pyx_v_b;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_array;
  __Pyx_Buffer __pyx_pybuffer_array;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  size_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0decode_envelopes", 0);
  __pyx_pybuffer_array.pybuffer.buf = NULL;
  __pyx_pybuffer_array.refcount = 0;
  __pyx_pybuffernd_array.data = NULL;
  __pyx_pybuffernd_array.rcbuffer = &__pyx_pybuffer_array;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_array.rcbuffer->pybuffer, (PyObject*)__pyx_v_array, &__Pyx_TypeInfo_float, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 1106, __pyx_L1_error)
  }
  __pyx_pybuffernd_array.diminfo[0].strides = __pyx_pybuffernd_array.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_array.diminfo[0].shape = __pyx_pybuffernd_array.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/peak_set.pyx":1113
 *         size_t i, n
 *         cython.floating a, b
 *     envelope_list = []             # <<<<<<<<<<<<<<
 *     current_envelope = []
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_envelope_list = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1114
 *         cython.floating a, b
 *     envelope_list = []
 *     current_envelope = []             # <<<<<<<<<<<<<<
 * 
 *     i = 0
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_current_envelope = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1116
 *     current_envelope = []
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     n = array.shape[0]
 *     while i < n:
 */
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1117
 * 
 *     i = 0
 *     n = array.shape[0]             # <<<<<<<<<<<<<<
 *     while i < n:
 *         a = array[i]
 */
  __pyx_v_n = (__pyx_v_array->dimensions[0]);

  /* "ms_deisotope/_c/peak_set.pyx":1118
 *     i = 0
 *     n = array.shape[0]
 *     while i < n:             # <<<<<<<<<<<<<<
 *         a = array[i]
 *         b = array[i + 1]
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_i < __pyx_v_n) != 0);
    if (!__pyx_t_2) break;

    /* "ms_deisotope/_c/peak_set.pyx":1119
 *     n = array.shape[0]
 *     while i < n:
 *         a = array[i]             # <<<<<<<<<<<<<<
 *         b = array[i + 1]
 *         i += 2
 */
    __pyx_t_3 = __pyx_v_i;
    __pyx_v_a = (*__Pyx_BufPtrCContig1d(float *, __pyx_pybuffernd_array.rcbuffer->pybuffer.buf, __pyx_t_3, __pyx_pybuffernd_array.diminfo[0].strides));

    /* "ms_deisotope/_c/peak_set.pyx":1120
 *     while i < n:
 *         a = array[i]
 *         b = array[i + 1]             # <<<<<<<<<<<<<<
 *         i += 2
 *         if a == 0 and b == 0:
 */
    __pyx_t_3 = (__pyx_v_i + 1);
    __pyx_v_b = (*__Pyx_BufPtrCContig1d(float *, __pyx_pybuffernd_array.rcbuffer->pybuffer.buf, __pyx_t_3, __pyx_pybuffernd_array.diminfo[0].strides));

    /* "ms_deisotope/_c/peak_set.pyx":1121
 *         a = array[i]
 *         b = array[i + 1]
 *         i += 2             # <<<<<<<<<<<<<<
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:
 */
    __pyx_v_i = (__pyx_v_i + 2);

    /* "ms_deisotope/_c/peak_set.pyx":1122
 *         b = array[i + 1]
 *         i += 2
 *         if a == 0 and b == 0:             # <<<<<<<<<<<<<<
 *             if current_envelope is not None:
 *                 if current_envelope:
 */
    __pyx_t_4 = ((__pyx_v_a == 0.0) != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_2 = __pyx_t_4;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_4 = ((__pyx_v_b == 0.0) != 0);
    __pyx_t_2 = __pyx_t_4;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1123
 *         i += 2
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:             # <<<<<<<<<<<<<<
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)
 */
      __pyx_t_2 = (__pyx_v_current_envelope != ((PyObject*)Py_None));
      __pyx_t_4 = (__pyx_t_2 != 0);
      if (__pyx_t_4) {

        /* "ms_deisotope/_c/peak_set.pyx":1124
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:
 *                 if current_envelope:             # <<<<<<<<<<<<<<
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)
 */
        __pyx_t_4 = (PyList_GET_SIZE(__pyx_v_current_envelope) != 0);
        if (__pyx_t_4) {

          /* "ms_deisotope/_c/peak_set.pyx":1125
 *             if current_envelope is not None:
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)             # <<<<<<<<<<<<<<
 *                     envelope_obj = Envelope._create(converted)
 *                     envelope_list.append(envelope_obj)
 */
          __pyx_t_1 = PyList_AsTuple(__pyx_v_current_envelope); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1125, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_XDECREF_SET(__pyx_v_converted, ((PyObject*)__pyx_t_1));
          __pyx_t_1 = 0;

          /* "ms_deisotope/_c/peak_set.pyx":1126
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)             # <<<<<<<<<<<<<<
 *                     envelope_list.append(envelope_obj)
 *                 current_envelope = []
 */
          __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(__pyx_v_converted)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1126, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_XDECREF_SET(__pyx_v_envelope_obj, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1));
          __pyx_t_1 = 0;

          /* "ms_deisotope/_c/peak_set.pyx":1127
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)
 *                     envelope_list.append(envelope_obj)             # <<<<<<<<<<<<<<
 *                 current_envelope = []
 *         else:
 */
          __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_envelope_list, ((PyObject *)__pyx_v_envelope_obj)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1127, __pyx_L1_error)

          /* "ms_deisotope/_c/peak_set.pyx":1124
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:
 *                 if current_envelope:             # <<<<<<<<<<<<<<
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)
 */
        }

        /* "ms_deisotope/_c/peak_set.pyx":1128
 *                     envelope_obj = Envelope._create(converted)
 *                     envelope_list.append(envelope_obj)
 *                 current_envelope = []             # <<<<<<<<<<<<<<
 *         else:
 *             current_envelope.append(EnvelopePair._create(a, b))
 */
        __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1128, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF_SET(__pyx_v_current_envelope, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "ms_deisotope/_c/peak_set.pyx":1123
 *         i += 2
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:             # <<<<<<<<<<<<<<
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)
 */
      }

      /* "ms_deisotope/_c/peak_set.pyx":1122
 *         b = array[i + 1]
 *         i += 2
 *         if a == 0 and b == 0:             # <<<<<<<<<<<<<<
 *             if current_envelope is not None:
 *                 if current_envelope:
 */
      goto __pyx_L5;
    }

    /* "ms_deisotope/_c/peak_set.pyx":1130
 *                 current_envelope = []
 *         else:
 *             current_envelope.append(EnvelopePair._create(a, b))             # <<<<<<<<<<<<<<
 *     converted = tuple(current_envelope)
 *     envelope_obj = Envelope._create(converted)
 */
    /*else*/ {
      __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create(__pyx_v_a, __pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1130, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_current_envelope, __pyx_t_1); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1130, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L5:;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1131
 *         else:
 *             current_envelope.append(EnvelopePair._create(a, b))
 *     converted = tuple(current_envelope)             # <<<<<<<<<<<<<<
 *     envelope_obj = Envelope._create(converted)
 *     envelope_list.append(envelope_obj)
 */
  __pyx_t_1 = PyList_AsTuple(__pyx_v_current_envelope); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XDECREF_SET(__pyx_v_converted, ((PyObject*)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1132
 *             current_envelope.append(EnvelopePair._create(a, b))
 *     converted = tuple(current_envelope)
 *     envelope_obj = Envelope._create(converted)             # <<<<<<<<<<<<<<
 *     envelope_list.append(envelope_obj)
 *     return envelope_list
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(__pyx_v_converted)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XDECREF_SET(__pyx_v_envelope_obj, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1133
 *     converted = tuple(current_envelope)
 *     envelope_obj = Envelope._create(converted)
 *     envelope_list.append(envelope_obj)             # <<<<<<<<<<<<<<
 *     return envelope_list
 * 
 */
  __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_envelope_list, ((PyObject *)__pyx_v_envelope_obj)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1133, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":1134
 *     envelope_obj = Envelope._create(converted)
 *     envelope_list.append(envelope_obj)
 *     return envelope_list             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_envelope_list);
  __pyx_r = __pyx_v_envelope_list;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1106
 * @cython.boundscheck(False)
 * @cython.wraparound(False)
 * def decode_envelopes(np.ndarray[cython.floating, ndim=1, mode='c'] array):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list envelope_list, current_envelope
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_array.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.decode_envelopes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_array.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF(__pyx_v_envelope_list);
  __Pyx_XDECREF(__pyx_v_current_envelope);
  __Pyx_XDECREF(__pyx_v_converted);
  __Pyx_XDECREF((PyObject *)__pyx_v_envelope_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_12ms_deisotope_2_c_8peak_set_19decode_envelopes(PyObject *__pyx_self, PyObject *__pyx_v_array); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_12ms_deisotope_2_c_8peak_set_19decode_envelopes = {"__pyx_fuse_1decode_envelopes", (PyCFunction)__pyx_fuse_1__pyx_pw_12ms_deisotope_2_c_8peak_set_19decode_envelopes, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_14decode_envelopes};
static PyObject *__pyx_fuse_1__pyx_pw_12ms_deisotope_2_c_8peak_set_19decode_envelopes(PyObject *__pyx_self, PyObject *__pyx_v_array) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("decode_envelopes (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_array), __pyx_ptype_5numpy_ndarray, 1, "array", 0))) __PYX_ERR(0, 1106, __pyx_L1_error)
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_18decode_envelopes(__pyx_self, ((PyArrayObject *)__pyx_v_array));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_18decode_envelopes(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_array) {
  PyObject *__pyx_v_envelope_list = 0;
  PyObject *__pyx_v_current_envelope = 0;
  PyObject *__pyx_v_converted = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_envelope_obj = 0;
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  double __pyx_v_a;
  double __pyx_v_b;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_array;
  __Pyx_Buffer __pyx_pybuffer_array;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  size_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1decode_envelopes", 0);
  __pyx_pybuffer_array.pybuffer.buf = NULL;
  __pyx_pybuffer_array.refcount = 0;
  __pyx_pybuffernd_array.data = NULL;
  __pyx_pybuffernd_array.rcbuffer = &__pyx_pybuffer_array;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_array.rcbuffer->pybuffer, (PyObject*)__pyx_v_array, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 1106, __pyx_L1_error)
  }
  __pyx_pybuffernd_array.diminfo[0].strides = __pyx_pybuffernd_array.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_array.diminfo[0].shape = __pyx_pybuffernd_array.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/peak_set.pyx":1113
 *         size_t i, n
 *         cython.floating a, b
 *     envelope_list = []             # <<<<<<<<<<<<<<
 *     current_envelope = []
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_envelope_list = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1114
 *         cython.floating a, b
 *     envelope_list = []
 *     current_envelope = []             # <<<<<<<<<<<<<<
 * 
 *     i = 0
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_current_envelope = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1116
 *     current_envelope = []
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     n = array.shape[0]
 *     while i < n:
 */
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1117
 * 
 *     i = 0
 *     n = array.shape[0]             # <<<<<<<<<<<<<<
 *     while i < n:
 *         a = array[i]
 */
  __pyx_v_n = (__pyx_v_array->dimensions[0]);

  /* "ms_deisotope/_c/peak_set.pyx":1118
 *     i = 0
 *     n = array.shape[0]
 *     while i < n:             # <<<<<<<<<<<<<<
 *         a = array[i]
 *         b = array[i + 1]
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_i < __pyx_v_n) != 0);
    if (!__pyx_t_2) break;

    /* "ms_deisotope/_c/peak_set.pyx":1119
 *     n = array.shape[0]
 *     while i < n:
 *         a = array[i]             # <<<<<<<<<<<<<<
 *         b = array[i + 1]
 *         i += 2
 */
    __pyx_t_3 = __pyx_v_i;
    __pyx_v_a = (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_array.rcbuffer->pybuffer.buf, __pyx_t_3, __pyx_pybuffernd_array.diminfo[0].strides));

    /* "ms_deisotope/_c/peak_set.pyx":1120
 *     while i < n:
 *         a = array[i]
 *         b = array[i + 1]             # <<<<<<<<<<<<<<
 *         i += 2
 *         if a == 0 and b == 0:
 */
    __pyx_t_3 = (__pyx_v_i + 1);
    __pyx_v_b = (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_array.rcbuffer->pybuffer.buf, __pyx_t_3, __pyx_pybuffernd_array.diminfo[0].strides));

    /* "ms_deisotope/_c/peak_set.pyx":1121
 *         a = array[i]
 *         b = array[i + 1]
 *         i += 2             # <<<<<<<<<<<<<<
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:
 */
    __pyx_v_i = (__pyx_v_i + 2);

    /* "ms_deisotope/_c/peak_set.pyx":1122
 *         b = array[i + 1]
 *         i += 2
 *         if a == 0 and b == 0:             # <<<<<<<<<<<<<<
 *             if current_envelope is not None:
 *                 if current_envelope:
 */
    __pyx_t_4 = ((__pyx_v_a == 0.0) != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_2 = __pyx_t_4;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_4 = ((__pyx_v_b == 0.0) != 0);
    __pyx_t_2 = __pyx_t_4;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1123
 *         i += 2
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:             # <<<<<<<<<<<<<<
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)
 */
      __pyx_t_2 = (__pyx_v_current_envelope != ((PyObject*)Py_None));
      __pyx_t_4 = (__pyx_t_2 != 0);
      if (__pyx_t_4) {

        /* "ms_deisotope/_c/peak_set.pyx":1124
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:
 *                 if current_envelope:             # <<<<<<<<<<<<<<
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)
 */
        __pyx_t_4 = (PyList_GET_SIZE(__pyx_v_current_envelope) != 0);
        if (__pyx_t_4) {

          /* "ms_deisotope/_c/peak_set.pyx":1125
 *             if current_envelope is not None:
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)             # <<<<<<<<<<<<<<
 *                     envelope_obj = Envelope._create(converted)
 *                     envelope_list.append(envelope_obj)
 */
          __pyx_t_1 = PyList_AsTuple(__pyx_v_current_envelope); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1125, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_XDECREF_SET(__pyx_v_converted, ((PyObject*)__pyx_t_1));
          __pyx_t_1 = 0;

          /* "ms_deisotope/_c/peak_set.pyx":1126
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)             # <<<<<<<<<<<<<<
 *                     envelope_list.append(envelope_obj)
 *                 current_envelope = []
 */
          __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(__pyx_v_converted)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1126, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_XDECREF_SET(__pyx_v_envelope_obj, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1));
          __pyx_t_1 = 0;

          /* "ms_deisotope/_c/peak_set.pyx":1127
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)
 *                     envelope_list.append(envelope_obj)             # <<<<<<<<<<<<<<
 *                 current_envelope = []
 *         else:
 */
          __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_envelope_list, ((PyObject *)__pyx_v_envelope_obj)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1127, __pyx_L1_error)

          /* "ms_deisotope/_c/peak_set.pyx":1124
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:
 *                 if current_envelope:             # <<<<<<<<<<<<<<
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)
 */
        }

        /* "ms_deisotope/_c/peak_set.pyx":1128
 *                     envelope_obj = Envelope._create(converted)
 *                     envelope_list.append(envelope_obj)
 *                 current_envelope = []             # <<<<<<<<<<<<<<
 *         else:
 *             current_envelope.append(EnvelopePair._create(a, b))
 */
        __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1128, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF_SET(__pyx_v_current_envelope, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "ms_deisotope/_c/peak_set.pyx":1123
 *         i += 2
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:             # <<<<<<<<<<<<<<
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)
 */
      }

      /* "ms_deisotope/_c/peak_set.pyx":1122
 *         b = array[i + 1]
 *         i += 2
 *         if a == 0 and b == 0:             # <<<<<<<<<<<<<<
 *             if current_envelope is not None:
 *                 if current_envelope:
 */
      goto __pyx_L5;
    }

    /* "ms_deisotope/_c/peak_set.pyx":1130
 *                 current_envelope = []
 *         else:
 *             current_envelope.append(EnvelopePair._create(a, b))             # <<<<<<<<<<<<<<
 *     converted = tuple(current_envelope)
 *     envelope_obj = Envelope._create(converted)
 */
    /*else*/ {
      __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create(__pyx_v_a, __pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1130, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_current_envelope, __pyx_t_1); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1130, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L5:;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1131
 *         else:
 *             current_envelope.append(EnvelopePair._create(a, b))
 *     converted = tuple(current_envelope)             # <<<<<<<<<<<<<<
 *     envelope_obj = Envelope._create(converted)
 *     envelope_list.append(envelope_obj)
 */
  __pyx_t_1 = PyList_AsTuple(__pyx_v_current_envelope); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XDECREF_SET(__pyx_v_converted, ((PyObject*)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1132
 *             current_envelope.append(EnvelopePair._create(a, b))
 *     converted = tuple(current_envelope)
 *     envelope_obj = Envelope._create(converted)             # <<<<<<<<<<<<<<
 *     envelope_list.append(envelope_obj)
 *     return envelope_list
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(__pyx_v_converted)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XDECREF_SET(__pyx_v_envelope_obj, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1133
 *     converted = tuple(current_envelope)
 *     envelope_obj = Envelope._create(converted)
 *     envelope_list.append(envelope_obj)             # <<<<<<<<<<<<<<
 *     return envelope_list
 * 
 */
  __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_envelope_list, ((PyObject *)__pyx_v_envelope_obj)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1133, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":1134
 *     envelope_obj = Envelope._create(converted)
 *     envelope_list.append(envelope_obj)
 *     return envelope_list             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_envelope_list);
  __pyx_r = __pyx_v_envelope_list;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1106
 * @cython.boundscheck(False)
 * @cython.wraparound(False)
 * def decode_envelopes(np.ndarray[cython.floating, ndim=1, mode='c'] array):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list envelope_list, current_envelope
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_array.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.decode_envelopes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_array.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF(__pyx_v_envelope_list);
  __Pyx_XDECREF(__pyx_v_current_envelope);
  __Pyx_XDECREF(__pyx_v_converted);
  __Pyx_XDECREF((PyObject *)__pyx_v_envelope_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1140
 * 
 * cdef class DeconvolutedPeakSetIndexed(DeconvolutedPeakSet):
 *     def __init__(self, peaks):             # <<<<<<<<<<<<<<
 *         self.neutral_mass_array = NULL
 *         self.mz_array = NULL
 */

/* Python wrapper */
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_peaks = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_peaks,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_peaks)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 1140, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_peaks = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1140, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), __pyx_v_peaks);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, PyObject *__pyx_v_peaks) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1141
 * cdef class DeconvolutedPeakSetIndexed(DeconvolutedPeakSet):
 *     def __init__(self, peaks):
 *         self.neutral_mass_array = NULL             # <<<<<<<<<<<<<<
 *         self.mz_array = NULL
 *         self.interval_index = NULL
 */
  __pyx_v_self->neutral_mass_array = NULL;

  /* "ms_deisotope/_c/peak_set.pyx":1142
 *     def __init__(self, peaks):
 *         self.neutral_mass_array = NULL
 *         self.mz_array = NULL             # <<<<<<<<<<<<<<
 *         self.interval_index = NULL
 *         super(DeconvolutedPeakSetIndexed, self).__init__(peaks)
 */
  __pyx_v_self->mz_array = NULL;

  /* "ms_deisotope/_c/peak_set.pyx":1143
 *         self.neutral_mass_array = NULL
 *         self.mz_array = NULL
 *         self.interval_index = NULL             # <<<<<<<<<<<<<<
 *         super(DeconvolutedPeakSetIndexed, self).__init__(peaks)
 * 
 */
  __pyx_v_self->interval_index = NULL;

  /* "ms_deisotope/_c/peak_set.pyx":1144
 *         self.mz_array = NULL
 *         self.interval_index = NULL
 *         super(DeconvolutedPeakSetIndexed, self).__init__(peaks)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_peaks) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_peaks);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1140
 * 
 * cdef class DeconvolutedPeakSetIndexed(DeconvolutedPeakSet):
 *     def __init__(self, peaks):             # <<<<<<<<<<<<<<
 *         self.neutral_mass_array = NULL
 *         self.mz_array = NULL
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1146
 *         super(DeconvolutedPeakSetIndexed, self).__init__(peaks)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self._release_buffers()
 * 
 */

/* Python wrapper */
static void __pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_2__dealloc__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_2__dealloc__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1147
 * 
 *     def __dealloc__(self):
 *         self._release_buffers()             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_release_buffers); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1146
 *         super(DeconvolutedPeakSetIndexed, self).__init__(peaks)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self._release_buffers()
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "ms_deisotope/_c/peak_set.pyx":1149
 *         self._release_buffers()
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, (self.peaks, ), self.__getstate__()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_4__reduce__[] = "DeconvolutedPeakSetIndexed.__reduce__(self)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_4__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_4__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1150
 * 
 *     def __reduce__(self):
 *         return self.__class__, (self.peaks, ), self.__getstate__()             # <<<<<<<<<<<<<<
 * 
 *     def __getstate__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.peaks);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.peaks);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->__pyx_base.peaks);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getstate); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1149
 *         self._release_buffers()
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, (self.peaks, ), self.__getstate__()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1152
 *         return self.__class__, (self.peaks, ), self.__getstate__()
 * 
 *     def __getstate__(self):             # <<<<<<<<<<<<<<
 *         d = {"indexed": self.indexed}
 *         return d
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_7__getstate__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_6__getstate__[] = "DeconvolutedPeakSetIndexed.__getstate__(self)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_7__getstate__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getstate__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_6__getstate__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_6__getstate__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self) {
  PyObject *__pyx_v_d = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getstate__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1153
 * 
 *     def __getstate__(self):
 *         d = {"indexed": self.indexed}             # <<<<<<<<<<<<<<
 *         return d
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.indexed); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_indexed, __pyx_t_2) < 0) __PYX_ERR(0, 1153, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_d = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1154
 *     def __getstate__(self):
 *         d = {"indexed": self.indexed}
 *         return d             # <<<<<<<<<<<<<<
 * 
 *     def __setstate__(self, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_d);
  __pyx_r = __pyx_v_d;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1152
 *         return self.__class__, (self.peaks, ), self.__getstate__()
 * 
 *     def __getstate__(self):             # <<<<<<<<<<<<<<
 *         d = {"indexed": self.indexed}
 *         return d
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.__getstate__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_d);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1156
 *         return d
 * 
 *     def __setstate__(self, d):             # <<<<<<<<<<<<<<
 *         if d.get("indexed", False):
 *             self.reindex()
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_9__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_d); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_8__setstate__[] = "DeconvolutedPeakSetIndexed.__setstate__(self, d)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_9__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate__ (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_8__setstate__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), ((PyObject *)__pyx_v_d));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_8__setstate__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1157
 * 
 *     def __setstate__(self, d):
 *         if d.get("indexed", False):             # <<<<<<<<<<<<<<
 *             self.reindex()
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_d, __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 1157, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "ms_deisotope/_c/peak_set.pyx":1158
 *     def __setstate__(self, d):
 *         if d.get("indexed", False):
 *             self.reindex()             # <<<<<<<<<<<<<<
 * 
 *     def _release_buffers(self):
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.reindex(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1157
 * 
 *     def __setstate__(self, d):
 *         if d.get("indexed", False):             # <<<<<<<<<<<<<<
 *             self.reindex()
 * 
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1156
 *         return d
 * 
 *     def __setstate__(self, d):             # <<<<<<<<<<<<<<
 *         if d.get("indexed", False):
 *             self.reindex()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.__setstate__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1160
 *             self.reindex()
 * 
 *     def _release_buffers(self):             # <<<<<<<<<<<<<<
 *         if self.neutral_mass_array != NULL:
 *             free(self.neutral_mass_array)
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_11_release_buffers(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_10_release_buffers[] = "DeconvolutedPeakSetIndexed._release_buffers(self)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_11_release_buffers(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_release_buffers (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_10_release_buffers(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_10_release_buffers(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_release_buffers", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1161
 * 
 *     def _release_buffers(self):
 *         if self.neutral_mass_array != NULL:             # <<<<<<<<<<<<<<
 *             free(self.neutral_mass_array)
 *             self.neutral_mass_array = NULL
 */
  __pyx_t_1 = ((__pyx_v_self->neutral_mass_array != NULL) != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1162
 *     def _release_buffers(self):
 *         if self.neutral_mass_array != NULL:
 *             free(self.neutral_mass_array)             # <<<<<<<<<<<<<<
 *             self.neutral_mass_array = NULL
 *         if self.mz_array != NULL:
 */
    free(__pyx_v_self->neutral_mass_array);

    /* "ms_deisotope/_c/peak_set.pyx":1163
 *         if self.neutral_mass_array != NULL:
 *             free(self.neutral_mass_array)
 *             self.neutral_mass_array = NULL             # <<<<<<<<<<<<<<
 *         if self.mz_array != NULL:
 *             free(self.mz_array)
 */
    __pyx_v_self->neutral_mass_array = NULL;

    /* "ms_deisotope/_c/peak_set.pyx":1161
 * 
 *     def _release_buffers(self):
 *         if self.neutral_mass_array != NULL:             # <<<<<<<<<<<<<<
 *             free(self.neutral_mass_array)
 *             self.neutral_mass_array = NULL
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1164
 *             free(self.neutral_mass_array)
 *             self.neutral_mass_array = NULL
 *         if self.mz_array != NULL:             # <<<<<<<<<<<<<<
 *             free(self.mz_array)
 *             self.mz_array = NULL
 */
  __pyx_t_1 = ((__pyx_v_self->mz_array != NULL) != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1165
 *             self.neutral_mass_array = NULL
 *         if self.mz_array != NULL:
 *             free(self.mz_array)             # <<<<<<<<<<<<<<
 *             self.mz_array = NULL
 *         if self.interval_index != NULL:
 */
    free(__pyx_v_self->mz_array);

    /* "ms_deisotope/_c/peak_set.pyx":1166
 *         if self.mz_array != NULL:
 *             free(self.mz_array)
 *             self.mz_array = NULL             # <<<<<<<<<<<<<<
 *         if self.interval_index != NULL:
 *             free_index_list(self.interval_index)
 */
    __pyx_v_self->mz_array = NULL;

    /* "ms_deisotope/_c/peak_set.pyx":1164
 *             free(self.neutral_mass_array)
 *             self.neutral_mass_array = NULL
 *         if self.mz_array != NULL:             # <<<<<<<<<<<<<<
 *             free(self.mz_array)
 *             self.mz_array = NULL
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1167
 *             free(self.mz_array)
 *             self.mz_array = NULL
 *         if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *             free_index_list(self.interval_index)
 *             self.interval_index = NULL
 */
  __pyx_t_1 = ((__pyx_v_self->interval_index != NULL) != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1168
 *             self.mz_array = NULL
 *         if self.interval_index != NULL:
 *             free_index_list(self.interval_index)             # <<<<<<<<<<<<<<
 *             self.interval_index = NULL
 * 
 */
    __pyx_f_12ms_deisotope_2_c_8peak_set_free_index_list(__pyx_v_self->interval_index);

    /* "ms_deisotope/_c/peak_set.pyx":1169
 *         if self.interval_index != NULL:
 *             free_index_list(self.interval_index)
 *             self.interval_index = NULL             # <<<<<<<<<<<<<<
 * 
 *     def set_interval_index_size(self, index_size):
 */
    __pyx_v_self->interval_index = NULL;

    /* "ms_deisotope/_c/peak_set.pyx":1167
 *             free(self.mz_array)
 *             self.mz_array = NULL
 *         if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *             free_index_list(self.interval_index)
 *             self.interval_index = NULL
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1160
 *             self.reindex()
 * 
 *     def _release_buffers(self):             # <<<<<<<<<<<<<<
 *         if self.neutral_mass_array != NULL:
 *             free(self.neutral_mass_array)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1171
 *             self.interval_index = NULL
 * 
 *     def set_interval_index_size(self, index_size):             # <<<<<<<<<<<<<<
 *         global INTERVAL_INDEX_SIZE
 *         INTERVAL_INDEX_SIZE = index_size
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_13set_interval_index_size(PyObject *__pyx_v_self, PyObject *__pyx_v_index_size); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_12set_interval_index_size[] = "DeconvolutedPeakSetIndexed.set_interval_index_size(self, index_size)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_13set_interval_index_size(PyObject *__pyx_v_self, PyObject *__pyx_v_index_size) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_interval_index_size (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_12set_interval_index_size(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), ((PyObject *)__pyx_v_index_size));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_12set_interval_index_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, PyObject *__pyx_v_index_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_interval_index_size", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1173
 *     def set_interval_index_size(self, index_size):
 *         global INTERVAL_INDEX_SIZE
 *         INTERVAL_INDEX_SIZE = index_size             # <<<<<<<<<<<<<<
 *         self.reindex()
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_index_size); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1173, __pyx_L1_error)
  __pyx_v_12ms_deisotope_2_c_8peak_set_INTERVAL_INDEX_SIZE = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1174
 *         global INTERVAL_INDEX_SIZE
 *         INTERVAL_INDEX_SIZE = index_size
 *         self.reindex()             # <<<<<<<<<<<<<<
 * 
 *     cdef void _build_index_arrays(self):
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.reindex(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1171
 *             self.interval_index = NULL
 * 
 *     def set_interval_index_size(self, index_size):             # <<<<<<<<<<<<<<
 *         global INTERVAL_INDEX_SIZE
 *         INTERVAL_INDEX_SIZE = index_size
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.set_interval_index_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1176
 *         self.reindex()
 * 
 *     cdef void _build_index_arrays(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
 */

static void __pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__build_index_arrays(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self) {
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0;
  struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *__pyx_v_interval_index;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  size_t __pyx_t_5;
  size_t __pyx_t_6;
  double __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_build_index_arrays", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1180
 *             size_t i, n
 *             DeconvolutedPeak peak
 *         n = PyTuple_GET_SIZE(self.peaks)             # <<<<<<<<<<<<<<
 *         self._size = n
 *         self._release_buffers()
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.peaks;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_n = PyTuple_GET_SIZE(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1181
 *             DeconvolutedPeak peak
 *         n = PyTuple_GET_SIZE(self.peaks)
 *         self._size = n             # <<<<<<<<<<<<<<
 *         self._release_buffers()
 *         self.neutral_mass_array = <double*>malloc(sizeof(double) * n)
 */
  __pyx_v_self->_size = __pyx_v_n;

  /* "ms_deisotope/_c/peak_set.pyx":1182
 *         n = PyTuple_GET_SIZE(self.peaks)
 *         self._size = n
 *         self._release_buffers()             # <<<<<<<<<<<<<<
 *         self.neutral_mass_array = <double*>malloc(sizeof(double) * n)
 *         self.mz_array = <double*>malloc(sizeof(double) * n)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_release_buffers); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1183
 *         self._size = n
 *         self._release_buffers()
 *         self.neutral_mass_array = <double*>malloc(sizeof(double) * n)             # <<<<<<<<<<<<<<
 *         self.mz_array = <double*>malloc(sizeof(double) * n)
 * 
 */
  __pyx_v_self->neutral_mass_array = ((double *)malloc(((sizeof(double)) * __pyx_v_n)));

  /* "ms_deisotope/_c/peak_set.pyx":1184
 *         self._release_buffers()
 *         self.neutral_mass_array = <double*>malloc(sizeof(double) * n)
 *         self.mz_array = <double*>malloc(sizeof(double) * n)             # <<<<<<<<<<<<<<
 * 
 *         for i in range(n):
 */
  __pyx_v_self->mz_array = ((double *)malloc(((sizeof(double)) * __pyx_v_n)));

  /* "ms_deisotope/_c/peak_set.pyx":1186
 *         self.mz_array = <double*>malloc(sizeof(double) * n)
 * 
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             peak = self.getitem(i)
 *             self.neutral_mass_array[i] = peak.neutral_mass
 */
  __pyx_t_4 = __pyx_v_n;
  __pyx_t_5 = __pyx_t_4;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
    __pyx_v_i = __pyx_t_6;

    /* "ms_deisotope/_c/peak_set.pyx":1187
 * 
 *         for i in range(n):
 *             peak = self.getitem(i)             # <<<<<<<<<<<<<<
 *             self.neutral_mass_array[i] = peak.neutral_mass
 *             self.mz_array[peak._index.mz] = peak.mz
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.getitem(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1188
 *         for i in range(n):
 *             peak = self.getitem(i)
 *             self.neutral_mass_array[i] = peak.neutral_mass             # <<<<<<<<<<<<<<
 *             self.mz_array[peak._index.mz] = peak.mz
 * 
 */
    __pyx_t_7 = __pyx_v_peak->neutral_mass;
    (__pyx_v_self->neutral_mass_array[__pyx_v_i]) = __pyx_t_7;

    /* "ms_deisotope/_c/peak_set.pyx":1189
 *             peak = self.getitem(i)
 *             self.neutral_mass_array[i] = peak.neutral_mass
 *             self.mz_array[peak._index.mz] = peak.mz             # <<<<<<<<<<<<<<
 * 
 *         # The interpolating interval index is just a bit slower than
 */
    __pyx_t_7 = __pyx_v_peak->__pyx_base.mz;
    (__pyx_v_self->mz_array[__pyx_v_peak->_index->mz]) = __pyx_t_7;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1193
 *         # The interpolating interval index is just a bit slower than
 *         # the normal binary search, likely due to cache friendliness.
 *         if n > 2 and INTERVAL_INDEX_SIZE > 0:             # <<<<<<<<<<<<<<
 *             if self.interval_index != NULL:
 *                 free_index_list(self.interval_index)
 */
  __pyx_t_9 = ((__pyx_v_n > 2) != 0);
  if (__pyx_t_9) {
  } else {
    __pyx_t_8 = __pyx_t_9;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_9 = ((__pyx_v_12ms_deisotope_2_c_8peak_set_INTERVAL_INDEX_SIZE > 0) != 0);
  __pyx_t_8 = __pyx_t_9;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_8) {

    /* "ms_deisotope/_c/peak_set.pyx":1194
 *         # the normal binary search, likely due to cache friendliness.
 *         if n > 2 and INTERVAL_INDEX_SIZE > 0:
 *             if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *                 free_index_list(self.interval_index)
 *                 self.interval_index = NULL
 */
    __pyx_t_8 = ((__pyx_v_self->interval_index != NULL) != 0);
    if (__pyx_t_8) {

      /* "ms_deisotope/_c/peak_set.pyx":1195
 *         if n > 2 and INTERVAL_INDEX_SIZE > 0:
 *             if self.interval_index != NULL:
 *                 free_index_list(self.interval_index)             # <<<<<<<<<<<<<<
 *                 self.interval_index = NULL
 *             interval_index = <index_list*>malloc(sizeof(index_list))
 */
      __pyx_f_12ms_deisotope_2_c_8peak_set_free_index_list(__pyx_v_self->interval_index);

      /* "ms_deisotope/_c/peak_set.pyx":1196
 *             if self.interval_index != NULL:
 *                 free_index_list(self.interval_index)
 *                 self.interval_index = NULL             # <<<<<<<<<<<<<<
 *             interval_index = <index_list*>malloc(sizeof(index_list))
 *             build_interval_index(self, interval_index, INTERVAL_INDEX_SIZE)
 */
      __pyx_v_self->interval_index = NULL;

      /* "ms_deisotope/_c/peak_set.pyx":1194
 *         # the normal binary search, likely due to cache friendliness.
 *         if n > 2 and INTERVAL_INDEX_SIZE > 0:
 *             if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *                 free_index_list(self.interval_index)
 *                 self.interval_index = NULL
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":1197
 *                 free_index_list(self.interval_index)
 *                 self.interval_index = NULL
 *             interval_index = <index_list*>malloc(sizeof(index_list))             # <<<<<<<<<<<<<<
 *             build_interval_index(self, interval_index, INTERVAL_INDEX_SIZE)
 *             if check_index(interval_index) != 0:
 */
    __pyx_v_interval_index = ((struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *)malloc((sizeof(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list))));

    /* "ms_deisotope/_c/peak_set.pyx":1198
 *                 self.interval_index = NULL
 *             interval_index = <index_list*>malloc(sizeof(index_list))
 *             build_interval_index(self, interval_index, INTERVAL_INDEX_SIZE)             # <<<<<<<<<<<<<<
 *             if check_index(interval_index) != 0:
 *                 free_index_list(interval_index)
 */
    (void)(__pyx_f_12ms_deisotope_2_c_8peak_set_build_interval_index(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_interval_index, __pyx_v_12ms_deisotope_2_c_8peak_set_INTERVAL_INDEX_SIZE));

    /* "ms_deisotope/_c/peak_set.pyx":1199
 *             interval_index = <index_list*>malloc(sizeof(index_list))
 *             build_interval_index(self, interval_index, INTERVAL_INDEX_SIZE)
 *             if check_index(interval_index) != 0:             # <<<<<<<<<<<<<<
 *                 free_index_list(interval_index)
 *             else:
 */
    __pyx_t_8 = ((__pyx_f_12ms_deisotope_2_c_8peak_set_check_index(__pyx_v_interval_index) != 0) != 0);
    if (__pyx_t_8) {

      /* "ms_deisotope/_c/peak_set.pyx":1200
 *             build_interval_index(self, interval_index, INTERVAL_INDEX_SIZE)
 *             if check_index(interval_index) != 0:
 *                 free_index_list(interval_index)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.interval_index = interval_index
 */
      __pyx_f_12ms_deisotope_2_c_8peak_set_free_index_list(__pyx_v_interval_index);

      /* "ms_deisotope/_c/peak_set.pyx":1199
 *             interval_index = <index_list*>malloc(sizeof(index_list))
 *             build_interval_index(self, interval_index, INTERVAL_INDEX_SIZE)
 *             if check_index(interval_index) != 0:             # <<<<<<<<<<<<<<
 *                 free_index_list(interval_index)
 *             else:
 */
      goto __pyx_L9;
    }

    /* "ms_deisotope/_c/peak_set.pyx":1202
 *                 free_index_list(interval_index)
 *             else:
 *                 self.interval_index = interval_index             # <<<<<<<<<<<<<<
 * 
 *     cpdef reindex(self):
 */
    /*else*/ {
      __pyx_v_self->interval_index = __pyx_v_interval_index;
    }
    __pyx_L9:;

    /* "ms_deisotope/_c/peak_set.pyx":1193
 *         # The interpolating interval index is just a bit slower than
 *         # the normal binary search, likely due to cache friendliness.
 *         if n > 2 and INTERVAL_INDEX_SIZE > 0:             # <<<<<<<<<<<<<<
 *             if self.interval_index != NULL:
 *                 free_index_list(self.interval_index)
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1176
 *         self.reindex()
 * 
 *     cdef void _build_index_arrays(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed._build_index_arrays", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);
  __Pyx_RefNannyFinishContext();
}

/* "ms_deisotope/_c/peak_set.pyx":1204
 *                 self.interval_index = interval_index
 * 
 *     cpdef reindex(self):             # <<<<<<<<<<<<<<
 *         super(DeconvolutedPeakSetIndexed, self).reindex()
 *         self._build_index_arrays()
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_15reindex(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("reindex", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_reindex); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1204, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_15reindex)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1204, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":1205
 * 
 *     cpdef reindex(self):
 *         super(DeconvolutedPeakSetIndexed, self).reindex()             # <<<<<<<<<<<<<<
 *         self._build_index_arrays()
 *         return self
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_reindex); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1206
 *     cpdef reindex(self):
 *         super(DeconvolutedPeakSetIndexed, self).reindex()
 *         self._build_index_arrays()             # <<<<<<<<<<<<<<
 *         return self
 * 
 */
  ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self->__pyx_base.__pyx_vtab)->_build_index_arrays(__pyx_v_self);

  /* "ms_deisotope/_c/peak_set.pyx":1207
 *         super(DeconvolutedPeakSetIndexed, self).reindex()
 *         self._build_index_arrays()
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1204
 *                 self.interval_index = interval_index
 * 
 *     cpdef reindex(self):             # <<<<<<<<<<<<<<
 *         super(DeconvolutedPeakSetIndexed, self).reindex()
 *         self._build_index_arrays()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.reindex", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_15reindex(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_14reindex[] = "DeconvolutedPeakSetIndexed.reindex(self)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_15reindex(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("reindex (wrapper)", 0);
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_14reindex(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_14reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("reindex", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_reindex(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.reindex", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1209
 *         return self
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak *__pyx_optional_args) {
  double __pyx_v_error_tolerance = ((double)1e-5);
  int __pyx_v_use_mz = ((int)0);
  int __pyx_v_status;
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  size_t __pyx_v_s;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_has_peak", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_error_tolerance = __pyx_optional_args->error_tolerance;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_use_mz = __pyx_optional_args->use_mz;
      }
    }
  }

  /* "ms_deisotope/_c/peak_set.pyx":1233
 *             DeconvolutedPeak peak
 * 
 *         n = self._size             # <<<<<<<<<<<<<<
 *         if use_mz:
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)
 */
  __pyx_t_1 = __pyx_v_self->_size;
  __pyx_v_n = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1234
 * 
 *         n = self._size
 *         if use_mz:             # <<<<<<<<<<<<<<
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)
 *             if status != 0:
 */
  __pyx_t_2 = (__pyx_v_use_mz != 0);
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":1235
 *         n = self._size
 *         if use_mz:
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)             # <<<<<<<<<<<<<<
 *             if status != 0:
 *                 return None
 */
    __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search(__pyx_v_self->mz_array, __pyx_v_neutral_mass, __pyx_v_error_tolerance, __pyx_v_n, (&__pyx_v_i));

    /* "ms_deisotope/_c/peak_set.pyx":1236
 *         if use_mz:
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)
 *             if status != 0:             # <<<<<<<<<<<<<<
 *                 return None
 *             else:
 */
    __pyx_t_2 = ((__pyx_v_status != 0) != 0);
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1237
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)
 *             if status != 0:
 *                 return None             # <<<<<<<<<<<<<<
 *             else:
 *                 return <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1236
 *         if use_mz:
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)
 *             if status != 0:             # <<<<<<<<<<<<<<
 *                 return None
 *             else:
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":1239
 *                 return None
 *             else:
 *                 return <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)             # <<<<<<<<<<<<<<
 *         if n == 0:
 *             return None
 */
    /*else*/ {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __pyx_t_3 = __pyx_v_self->__pyx_base._mz_ordered;
      __Pyx_INCREF(__pyx_t_3);
      __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_v_i);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_INCREF(((PyObject *)((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4)));
      __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4);
      goto __pyx_L0;
    }

    /* "ms_deisotope/_c/peak_set.pyx":1234
 * 
 *         n = self._size
 *         if use_mz:             # <<<<<<<<<<<<<<
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)
 *             if status != 0:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1240
 *             else:
 *                 return <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return None
 *         if self.interval_index != NULL:
 */
  __pyx_t_2 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":1241
 *                 return <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 *         if n == 0:
 *             return None             # <<<<<<<<<<<<<<
 *         if self.interval_index != NULL:
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1240
 *             else:
 *                 return <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return None
 *         if self.interval_index != NULL:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1242
 *         if n == 0:
 *             return None
 *         if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 *             status = _binary_search_with_hint(self.neutral_mass_array, neutral_mass, error_tolerance, n, s, &i)
 */
  __pyx_t_2 = ((__pyx_v_self->interval_index != NULL) != 0);
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":1243
 *             return None
 *         if self.interval_index != NULL:
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)             # <<<<<<<<<<<<<<
 *             status = _binary_search_with_hint(self.neutral_mass_array, neutral_mass, error_tolerance, n, s, &i)
 *         else:
 */
    (void)(__pyx_f_12ms_deisotope_2_c_8peak_set_find_search_interval(__pyx_v_self->interval_index, __pyx_v_neutral_mass, (&__pyx_v_s), (&__pyx_v_n)));

    /* "ms_deisotope/_c/peak_set.pyx":1244
 *         if self.interval_index != NULL:
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 *             status = _binary_search_with_hint(self.neutral_mass_array, neutral_mass, error_tolerance, n, s, &i)             # <<<<<<<<<<<<<<
 *         else:
 *             status = _binary_search(self.neutral_mass_array, neutral_mass, error_tolerance, n, &i)
 */
    __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_with_hint(__pyx_v_self->neutral_mass_array, __pyx_v_neutral_mass, __pyx_v_error_tolerance, __pyx_v_n, __pyx_v_s, (&__pyx_v_i));

    /* "ms_deisotope/_c/peak_set.pyx":1242
 *         if n == 0:
 *             return None
 *         if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 *             status = _binary_search_with_hint(self.neutral_mass_array, neutral_mass, error_tolerance, n, s, &i)
 */
    goto __pyx_L6;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1246
 *             status = _binary_search_with_hint(self.neutral_mass_array, neutral_mass, error_tolerance, n, s, &i)
 *         else:
 *             status = _binary_search(self.neutral_mass_array, neutral_mass, error_tolerance, n, &i)             # <<<<<<<<<<<<<<
 *         if status != 0:
 *             return None
 */
  /*else*/ {
    __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search(__pyx_v_self->neutral_mass_array, __pyx_v_neutral_mass, __pyx_v_error_tolerance, __pyx_v_n, (&__pyx_v_i));
  }
  __pyx_L6:;

  /* "ms_deisotope/_c/peak_set.pyx":1247
 *         else:
 *             status = _binary_search(self.neutral_mass_array, neutral_mass, error_tolerance, n, &i)
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             return None
 *         peak = self.getitem(i)
 */
  __pyx_t_2 = ((__pyx_v_status != 0) != 0);
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":1248
 *             status = _binary_search(self.neutral_mass_array, neutral_mass, error_tolerance, n, &i)
 *         if status != 0:
 *             return None             # <<<<<<<<<<<<<<
 *         peak = self.getitem(i)
 *         if abs((peak.neutral_mass - neutral_mass) / neutral_mass) < error_tolerance:
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1247
 *         else:
 *             status = _binary_search(self.neutral_mass_array, neutral_mass, error_tolerance, n, &i)
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             return None
 *         peak = self.getitem(i)
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1249
 *         if status != 0:
 *             return None
 *         peak = self.getitem(i)             # <<<<<<<<<<<<<<
 *         if abs((peak.neutral_mass - neutral_mass) / neutral_mass) < error_tolerance:
 *             return peak
 */
  __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.getitem(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_i)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_peak = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1250
 *             return None
 *         peak = self.getitem(i)
 *         if abs((peak.neutral_mass - neutral_mass) / neutral_mass) < error_tolerance:             # <<<<<<<<<<<<<<
 *             return peak
 *         else:
 */
  __pyx_t_5 = (__pyx_v_peak->neutral_mass - __pyx_v_neutral_mass);
  if (unlikely(__pyx_v_neutral_mass == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 1250, __pyx_L1_error)
  }
  __pyx_t_2 = ((fabs((__pyx_t_5 / __pyx_v_neutral_mass)) < __pyx_v_error_tolerance) != 0);
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":1251
 *         peak = self.getitem(i)
 *         if abs((peak.neutral_mass - neutral_mass) / neutral_mass) < error_tolerance:
 *             return peak             # <<<<<<<<<<<<<<
 *         else:
 *             return None
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __Pyx_INCREF(((PyObject *)__pyx_v_peak));
    __pyx_r = __pyx_v_peak;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1250
 *             return None
 *         peak = self.getitem(i)
 *         if abs((peak.neutral_mass - neutral_mass) / neutral_mass) < error_tolerance:             # <<<<<<<<<<<<<<
 *             return peak
 *         else:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1253
 *             return peak
 *         else:
 *             return None             # <<<<<<<<<<<<<<
 * 
 *     def _test_interval(self, double neutral_mass, double tolerance=1e-5):
 */
  /*else*/ {
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);
    goto __pyx_L0;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1209
 *         return self
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed._has_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1255
 *             return None
 * 
 *     def _test_interval(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int status
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_17_test_interval(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_16_test_interval[] = "DeconvolutedPeakSetIndexed._test_interval(self, double neutral_mass, double tolerance=1e-5)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_17_test_interval(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_neutral_mass;
  double __pyx_v_tolerance;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_interval (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_tolerance,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tolerance);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_test_interval") < 0)) __PYX_ERR(0, 1255, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_neutral_mass = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_neutral_mass == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1255, __pyx_L3_error)
    if (values[1]) {
      __pyx_v_tolerance = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_tolerance == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1255, __pyx_L3_error)
    } else {
      __pyx_v_tolerance = ((double)1e-5);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_test_interval", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1255, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed._test_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_16_test_interval(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_tolerance);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_16_test_interval(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance) {
  int __pyx_v_status;
  size_t __pyx_v_n;
  size_t __pyx_v_start;
  size_t __pyx_v_end;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_interval", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1259
 *             int status
 *             size_t n, start, end
 *         n = self._size             # <<<<<<<<<<<<<<
 *         status = _binary_search_interval(
 *             self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 */
  __pyx_t_1 = __pyx_v_self->_size;
  __pyx_v_n = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1260
 *             size_t n, start, end
 *         n = self._size
 *         status = _binary_search_interval(             # <<<<<<<<<<<<<<
 *             self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 *         return status, start, end
 */
  __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval(__pyx_v_self->neutral_mass_array, __pyx_v_neutral_mass, __pyx_v_tolerance, __pyx_v_n, (&__pyx_v_start), (&__pyx_v_end));

  /* "ms_deisotope/_c/peak_set.pyx":1262
 *         status = _binary_search_interval(
 *             self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 *         return status, start, end             # <<<<<<<<<<<<<<
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_start); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_4);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1255
 *             return None
 * 
 *     def _test_interval(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int status
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed._test_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1264
 *         return status, start, end
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         '''Find all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_19all_peaks_for(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for *__pyx_optional_args) {
  double __pyx_v_tolerance = ((double)1e-5);
  int __pyx_v_status;
  size_t __pyx_v_n;
  size_t __pyx_v_s;
  size_t __pyx_v_start;
  size_t __pyx_v_end;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("all_peaks_for", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_tolerance = __pyx_optional_args->tolerance;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_all_peaks_for); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1264, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_19all_peaks_for)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = PyFloat_FromDouble(__pyx_v_neutral_mass); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1264, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyFloat_FromDouble(__pyx_v_tolerance); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1264, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1264, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1264, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1264, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1264, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (!(likely(PyTuple_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 1264, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ms_deisotope/_c/peak_set.pyx":1282
 *             int status
 *             size_t n, s, start, end
 *         n = self._size             # <<<<<<<<<<<<<<
 *         if not self.indexed:
 *             self.reindex()
 */
  __pyx_t_9 = __pyx_v_self->_size;
  __pyx_v_n = __pyx_t_9;

  /* "ms_deisotope/_c/peak_set.pyx":1283
 *             size_t n, s, start, end
 *         n = self._size
 *         if not self.indexed:             # <<<<<<<<<<<<<<
 *             self.reindex()
 *         if n == 0:
 */
  __pyx_t_10 = ((!(__pyx_v_self->__pyx_base.indexed != 0)) != 0);
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":1284
 *         n = self._size
 *         if not self.indexed:
 *             self.reindex()             # <<<<<<<<<<<<<<
 *         if n == 0:
 *             return ()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.reindex(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1284, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1283
 *             size_t n, s, start, end
 *         n = self._size
 *         if not self.indexed:             # <<<<<<<<<<<<<<
 *             self.reindex()
 *         if n == 0:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1285
 *         if not self.indexed:
 *             self.reindex()
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return ()
 *         if self.interval_index != NULL:
 */
  __pyx_t_10 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":1286
 *             self.reindex()
 *         if n == 0:
 *             return ()             # <<<<<<<<<<<<<<
 *         if self.interval_index != NULL:
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1285
 *         if not self.indexed:
 *             self.reindex()
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return ()
 *         if self.interval_index != NULL:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1287
 *         if n == 0:
 *             return ()
 *         if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 *             status = _binary_search_interval_with_hint(
 */
  __pyx_t_10 = ((__pyx_v_self->interval_index != NULL) != 0);
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":1288
 *             return ()
 *         if self.interval_index != NULL:
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)             # <<<<<<<<<<<<<<
 *             status = _binary_search_interval_with_hint(
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, s, &start, &end)
 */
    (void)(__pyx_f_12ms_deisotope_2_c_8peak_set_find_search_interval(__pyx_v_self->interval_index, __pyx_v_neutral_mass, (&__pyx_v_s), (&__pyx_v_n)));

    /* "ms_deisotope/_c/peak_set.pyx":1289
 *         if self.interval_index != NULL:
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 *             status = _binary_search_interval_with_hint(             # <<<<<<<<<<<<<<
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, s, &start, &end)
 *         else:
 */
    __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval_with_hint(__pyx_v_self->neutral_mass_array, __pyx_v_neutral_mass, __pyx_v_tolerance, __pyx_v_n, __pyx_v_s, (&__pyx_v_start), (&__pyx_v_end));

    /* "ms_deisotope/_c/peak_set.pyx":1287
 *         if n == 0:
 *             return ()
 *         if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 *             status = _binary_search_interval_with_hint(
 */
    goto __pyx_L5;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1292
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, s, &start, &end)
 *         else:
 *             status = _binary_search_interval(             # <<<<<<<<<<<<<<
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 *         if status != 0:
 */
  /*else*/ {

    /* "ms_deisotope/_c/peak_set.pyx":1293
 *         else:
 *             status = _binary_search_interval(
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)             # <<<<<<<<<<<<<<
 *         if status != 0:
 *             return ()
 */
    __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval(__pyx_v_self->neutral_mass_array, __pyx_v_neutral_mass, __pyx_v_tolerance, __pyx_v_n, (&__pyx_v_start), (&__pyx_v_end));
  }
  __pyx_L5:;

  /* "ms_deisotope/_c/peak_set.pyx":1294
 *             status = _binary_search_interval(
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             return ()
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 */
  __pyx_t_10 = ((__pyx_v_status != 0) != 0);
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":1295
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 *         if status != 0:
 *             return ()             # <<<<<<<<<<<<<<
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1294
 *             status = _binary_search_interval(
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             return ()
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1296
 *         if status != 0:
 *             return ()
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)             # <<<<<<<<<<<<<<
 * 
 *     cdef int _interval_for(self, double neutral_mass, double tolerance, size_t* start, size_t* end) nogil:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_self->__pyx_base.peaks;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_GetSlice(__pyx_t_1, __pyx_v_start, __pyx_v_end); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_INCREF(((PyObject*)__pyx_t_2));
  __pyx_r = ((PyObject*)__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1264
 *         return status, start, end
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         '''Find all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.all_peaks_for", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_19all_peaks_for(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_18all_peaks_for[] = "DeconvolutedPeakSetIndexed.all_peaks_for(self, double neutral_mass, double tolerance=1e-5) -> tuple\nFind all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.\n\n        Parameters\n        ----------\n        neutral_mass: double\n            The mass to search for\n        error_tolerance: double\n            The PPM error tolerance to apply\n\n        Returns\n        -------\n        tuple of DeconvolutedPeak\n            The found peaks\n        ";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_19all_peaks_for(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_neutral_mass;
  double __pyx_v_tolerance;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("all_peaks_for (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_tolerance,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tolerance);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "all_peaks_for") < 0)) __PYX_ERR(0, 1264, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_neutral_mass = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_neutral_mass == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1264, __pyx_L3_error)
    if (values[1]) {
      __pyx_v_tolerance = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_tolerance == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1264, __pyx_L3_error)
    } else {
      __pyx_v_tolerance = ((double)1e-5);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("all_peaks_for", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1264, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.all_peaks_for", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_18all_peaks_for(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_tolerance);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_18all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("all_peaks_for", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.tolerance = __pyx_v_tolerance;
  __pyx_t_1 = __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed->__pyx_base.all_peaks_for(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_neutral_mass, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.all_peaks_for", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1298
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 *     cdef int _interval_for(self, double neutral_mass, double tolerance, size_t* start, size_t* end) nogil:             # <<<<<<<<<<<<<<
 *         cdef:
 *             int status
 */

static int __pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__interval_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance, size_t *__pyx_v_start, size_t *__pyx_v_end) {
  int __pyx_v_status;
  size_t __pyx_v_n;
  int __pyx_r;
  size_t __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1302
 *             int status
 *             size_t n
 *         n = self._size             # <<<<<<<<<<<<<<
 *         status = _binary_search_interval(
 *             self.neutral_mass_array, neutral_mass, tolerance, n, start, end)
 */
  __pyx_t_1 = __pyx_v_self->_size;
  __pyx_v_n = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1303
 *             size_t n
 *         n = self._size
 *         status = _binary_search_interval(             # <<<<<<<<<<<<<<
 *             self.neutral_mass_array, neutral_mass, tolerance, n, start, end)
 *         return status
 */
  __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval(__pyx_v_self->neutral_mass_array, __pyx_v_neutral_mass, __pyx_v_tolerance, __pyx_v_n, __pyx_v_start, __pyx_v_end);

  /* "ms_deisotope/_c/peak_set.pyx":1305
 *         status = _binary_search_interval(
 *             self.neutral_mass_array, neutral_mass, tolerance, n, start, end)
 *         return status             # <<<<<<<<<<<<<<
 * 
 *     def test_interval(self, double value):
 */
  __pyx_r = __pyx_v_status;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1298
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 *     cdef int _interval_for(self, double neutral_mass, double tolerance, size_t* start, size_t* end) nogil:             # <<<<<<<<<<<<<<
 *         cdef:
 *             int status
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1307
 *         return status
 * 
 *     def test_interval(self, double value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int status
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_21test_interval(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_20test_interval[] = "DeconvolutedPeakSetIndexed.test_interval(self, double value)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_21test_interval(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  double __pyx_v_value;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("test_interval (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsDouble(__pyx_arg_value); if (unlikely((__pyx_v_value == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1307, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.test_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_20test_interval(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), ((double)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_20test_interval(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_value) {
  CYTHON_UNUSED int __pyx_v_status;
  size_t __pyx_v_start;
  size_t __pyx_v_end;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("test_interval", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1313
 *             size_t i
 * 
 *         status = find_search_interval(self.interval_index, value, &start, &end)             # <<<<<<<<<<<<<<
 *         return start, end, interpolate_index(self.interval_index, value)
 * 
 */
  __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set_find_search_interval(__pyx_v_self->interval_index, __pyx_v_value, (&__pyx_v_start), (&__pyx_v_end));

  /* "ms_deisotope/_c/peak_set.pyx":1314
 * 
 *         status = find_search_interval(self.interval_index, value, &start, &end)
 *         return start, end, interpolate_index(self.interval_index, value)             # <<<<<<<<<<<<<<
 * 
 *     def find_interval_for(self, double value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_start); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_end); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_f_12ms_deisotope_2_c_8peak_set_interpolate_index(__pyx_v_self->interval_index, __pyx_v_value)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1307
 *         return status
 * 
 *     def test_interval(self, double value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int status
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.test_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1316
 *         return start, end, interpolate_index(self.interval_index, value)
 * 
 *     def find_interval_for(self, double value):             # <<<<<<<<<<<<<<
 *         return interpolate_index(self.interval_index, value)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_23find_interval_for(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_22find_interval_for[] = "DeconvolutedPeakSetIndexed.find_interval_for(self, double value)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_23find_interval_for(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  double __pyx_v_value;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("find_interval_for (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsDouble(__pyx_arg_value); if (unlikely((__pyx_v_value == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1316, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.find_interval_for", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_22find_interval_for(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), ((double)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_22find_interval_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("find_interval_for", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1317
 * 
 *     def find_interval_for(self, double value):
 *         return interpolate_index(self.interval_index, value)             # <<<<<<<<<<<<<<
 * 
 *     def check_interval(self, size_t i):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_f_12ms_deisotope_2_c_8peak_set_interpolate_index(__pyx_v_self->interval_index, __pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1316
 *         return start, end, interpolate_index(self.interval_index, value)
 * 
 *     def find_interval_for(self, double value):             # <<<<<<<<<<<<<<
 *         return interpolate_index(self.interval_index, value)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.find_interval_for", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1319
 *         return interpolate_index(self.interval_index, value)
 * 
 *     def check_interval(self, size_t i):             # <<<<<<<<<<<<<<
 *         cdef:
 *             index_cell cell
 */

/* Python wrapper */
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_25check_interval(PyObject *__pyx_v_self, PyObject *__pyx_arg_i); /*proto*/
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_24check_interval[] = "DeconvolutedPeakSetIndexed.check_interval(self, size_t i)";
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_25check_interval(PyObject *__pyx_v_self, PyObject *__pyx_arg_i) {
  size_t __pyx_v_i;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("check_interval (wrapper)", 0);
  assert(__pyx_arg_i); {
    __pyx_v_i = __Pyx_PyInt_As_size_t(__pyx_arg_i); if (unlikely((__pyx_v_i == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1319, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.check_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_24check_interval(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), ((size_t)__pyx_v_i));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_24check_interval(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, size_t __pyx_v_i) {
  struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell __pyx_v_cell;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("check_interval", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1322
 *         cdef:
 *             index_cell cell
 *         cell = self.interval_index.index[i]             # <<<<<<<<<<<<<<
 *         return cell
 * 
 */
  __pyx_v_cell = (__pyx_v_self->interval_index->index[__pyx_v_i]);

  /* "ms_deisotope/_c/peak_set.pyx":1323
 *             index_cell cell
 *         cell = self.interval_index.index[i]
 *         return cell             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_convert__to_py_struct____pyx_t_12ms_deisotope_2_c_8peak_set_index_cell(__pyx_v_cell); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1319
 *         return interpolate_index(self.interval_index, value)
 * 
 *     def check_interval(self, size_t i):             # <<<<<<<<<<<<<<
 *         cdef:
 *             index_cell cell
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.check_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1327
 * 
 * 
 * cdef int _binary_search_with_hint(double* array, double target, double error_tolerance, size_t n, size_t hint, size_t* out) nogil:             # <<<<<<<<<<<<<<
 *     '''Performs a best-matching binary search with a hint at the lower bound as well as the upper bound.
 * 
 */

static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_with_hint(double *__pyx_v_array, double __pyx_v_target, double __pyx_v_error_tolerance, size_t __pyx_v_n, size_t __pyx_v_hint, size_t *__pyx_v_out) {
  size_t __pyx_v_lo;
  size_t __pyx_v_hi;
  size_t __pyx_v_mid;
  size_t __pyx_v_i;
  size_t __pyx_v_best_index;
  double __pyx_v_found_mass;
  double __pyx_v_best_error;
  double __pyx_v_abs_error;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "ms_deisotope/_c/peak_set.pyx":1359
 *         double best_error, abs_error
 * 
 *     lo = hint             # <<<<<<<<<<<<<<
 *     hi = n
 * 
 */
  __pyx_v_lo = __pyx_v_hint;

  /* "ms_deisotope/_c/peak_set.pyx":1360
 * 
 *     lo = hint
 *     hi = n             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */
  __pyx_v_hi = __pyx_v_n;

  /* "ms_deisotope/_c/peak_set.pyx":1362
 *     hi = n
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":1363
 * 
 *     while hi != lo:
 *         mid = (hi + lo) / 2             # <<<<<<<<<<<<<<
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 */
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":1364
 *     while hi != lo:
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]             # <<<<<<<<<<<<<<
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 */
    __pyx_v_found_mass = (__pyx_v_array[__pyx_v_mid]);

    /* "ms_deisotope/_c/peak_set.pyx":1365
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */
    __pyx_t_1 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_found_mass, __pyx_v_target)) < __pyx_v_error_tolerance) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1366
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid             # <<<<<<<<<<<<<<
 *             best_error = INF
 *             i = 0
 */
      __pyx_v_best_index = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":1367
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 *             best_error = INF             # <<<<<<<<<<<<<<
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 */
      __pyx_v_best_error = __pyx_v_12ms_deisotope_2_c_8peak_set_INF;

      /* "ms_deisotope/_c/peak_set.pyx":1368
 *             best_index = mid
 *             best_error = INF
 *             i = 0             # <<<<<<<<<<<<<<
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 */
      __pyx_v_i = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1369
 *             best_error = INF
 *             i = 0
 *             while mid - i >= 0 and i <= mid:             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */
      while (1) {
        __pyx_t_2 = (((__pyx_v_mid - __pyx_v_i) >= 0) != 0);
        if (__pyx_t_2) {
        } else {
          __pyx_t_1 = __pyx_t_2;
          goto __pyx_L8_bool_binop_done;
        }
        __pyx_t_2 = ((__pyx_v_i <= __pyx_v_mid) != 0);
        __pyx_t_1 = __pyx_t_2;
        __pyx_L8_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":1370
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid - __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":1371
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":1372
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":1373
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":1374
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid - i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */
            __pyx_v_best_index = (__pyx_v_mid - __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":1375
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":1373
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */
          }

          /* "ms_deisotope/_c/peak_set.pyx":1372
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */
          goto __pyx_L10;
        }

        /* "ms_deisotope/_c/peak_set.pyx":1377
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             i = 1
 */
        /*else*/ {
          goto __pyx_L7_break;
        }
        __pyx_L10:;

        /* "ms_deisotope/_c/peak_set.pyx":1378
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             i = 1
 *             while (mid + i) < (n - 1):
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L7_break:;

      /* "ms_deisotope/_c/peak_set.pyx":1379
 *                     break
 *                 i += 1
 *             i = 1             # <<<<<<<<<<<<<<
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 */
      __pyx_v_i = 1;

      /* "ms_deisotope/_c/peak_set.pyx":1380
 *                 i += 1
 *             i = 1
 *             while (mid + i) < (n - 1):             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */
      while (1) {
        __pyx_t_1 = (((__pyx_v_mid + __pyx_v_i) < (__pyx_v_n - 1)) != 0);
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":1381
 *             i = 1
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid + __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":1382
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":1383
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":1384
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":1385
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid + i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */
            __pyx_v_best_index = (__pyx_v_mid + __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":1386
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":1384
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */
          }

          /* "ms_deisotope/_c/peak_set.pyx":1383
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */
          goto __pyx_L14;
        }

        /* "ms_deisotope/_c/peak_set.pyx":1388
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             out[0] = best_index
 */
        /*else*/ {
          goto __pyx_L13_break;
        }
        __pyx_L14:;

        /* "ms_deisotope/_c/peak_set.pyx":1389
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             out[0] = best_index
 *             if best_error == INF:
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L13_break:;

      /* "ms_deisotope/_c/peak_set.pyx":1390
 *                     break
 *                 i += 1
 *             out[0] = best_index             # <<<<<<<<<<<<<<
 *             if best_error == INF:
 *                 return 3
 */
      (__pyx_v_out[0]) = __pyx_v_best_index;

      /* "ms_deisotope/_c/peak_set.pyx":1391
 *                 i += 1
 *             out[0] = best_index
 *             if best_error == INF:             # <<<<<<<<<<<<<<
 *                 return 3
 *             return 0
 */
      __pyx_t_1 = ((__pyx_v_best_error == __pyx_v_12ms_deisotope_2_c_8peak_set_INF) != 0);
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":1392
 *             out[0] = best_index
 *             if best_error == INF:
 *                 return 3             # <<<<<<<<<<<<<<
 *             return 0
 *         elif hi - lo == 1:
 */
        __pyx_r = 3;
        goto __pyx_L0;

        /* "ms_deisotope/_c/peak_set.pyx":1391
 *                 i += 1
 *             out[0] = best_index
 *             if best_error == INF:             # <<<<<<<<<<<<<<
 *                 return 3
 *             return 0
 */
      }

      /* "ms_deisotope/_c/peak_set.pyx":1393
 *             if best_error == INF:
 *                 return 3
 *             return 0             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             out[0] = 0
 */
      __pyx_r = 0;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1365
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":1394
 *                 return 3
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             out[0] = 0
 *             return 1
 */
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1395
 *             return 0
 *         elif hi - lo == 1:
 *             out[0] = 0             # <<<<<<<<<<<<<<
 *             return 1
 *         elif found_mass > target:
 */
      (__pyx_v_out[0]) = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1396
 *         elif hi - lo == 1:
 *             out[0] = 0
 *             return 1             # <<<<<<<<<<<<<<
 *         elif found_mass > target:
 *             hi = mid
 */
      __pyx_r = 1;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1394
 *                 return 3
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             out[0] = 0
 *             return 1
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":1397
 *             out[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_found_mass > __pyx_v_target) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1398
 *             return 1
 *         elif found_mass > target:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":1397
 *             out[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
      goto __pyx_L5;
    }

    /* "ms_deisotope/_c/peak_set.pyx":1400
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 *     out[0] = 0
 *     return 2
 */
    /*else*/ {
      __pyx_v_lo = __pyx_v_mid;
    }
    __pyx_L5:;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1401
 *         else:
 *             lo = mid
 *     out[0] = 0             # <<<<<<<<<<<<<<
 *     return 2
 * 
 */
  (__pyx_v_out[0]) = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1402
 *             lo = mid
 *     out[0] = 0
 *     return 2             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 2;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1327
 * 
 * 
 * cdef int _binary_search_with_hint(double* array, double target, double error_tolerance, size_t n, size_t hint, size_t* out) nogil:             # <<<<<<<<<<<<<<
 *     '''Performs a best-matching binary search with a hint at the lower bound as well as the upper bound.
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1405
 * 
 * 
 * cdef int _binary_search_interval_with_hint(double* array, double target, double error_tolerance, size_t n, size_t hint, size_t* start, size_t* end) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval_with_hint(double *__pyx_v_array, double __pyx_v_target, double __pyx_v_error_tolerance, size_t __pyx_v_n, size_t __pyx_v_hint, size_t *__pyx_v_start, size_t *__pyx_v_end) {
  size_t __pyx_v_lo;
  size_t __pyx_v_hi;
  size_t __pyx_v_mid;
  size_t __pyx_v_i;
  CYTHON_UNUSED size_t __pyx_v_best_index;
  double __pyx_v_found_mass;
  double __pyx_v_best_error;
  double __pyx_v_abs_error;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "ms_deisotope/_c/peak_set.pyx":1412
 *         double best_error, abs_error
 * 
 *     lo = hint             # <<<<<<<<<<<<<<
 *     hi = n
 * 
 */
  __pyx_v_lo = __pyx_v_hint;

  /* "ms_deisotope/_c/peak_set.pyx":1413
 * 
 *     lo = hint
 *     hi = n             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */
  __pyx_v_hi = __pyx_v_n;

  /* "ms_deisotope/_c/peak_set.pyx":1415
 *     hi = n
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":1416
 * 
 *     while hi != lo:
 *         mid = (hi + lo) / 2             # <<<<<<<<<<<<<<
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 */
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":1417
 *     while hi != lo:
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]             # <<<<<<<<<<<<<<
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 */
    __pyx_v_found_mass = (__pyx_v_array[__pyx_v_mid]);

    /* "ms_deisotope/_c/peak_set.pyx":1418
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */
    __pyx_t_1 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_found_mass, __pyx_v_target)) < __pyx_v_error_tolerance) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1419
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid             # <<<<<<<<<<<<<<
 *             best_error = INF
 *             i = 0
 */
      __pyx_v_best_index = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":1420
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 *             best_error = INF             # <<<<<<<<<<<<<<
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 */
      __pyx_v_best_error = __pyx_v_12ms_deisotope_2_c_8peak_set_INF;

      /* "ms_deisotope/_c/peak_set.pyx":1421
 *             best_index = mid
 *             best_error = INF
 *             i = 0             # <<<<<<<<<<<<<<
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 */
      __pyx_v_i = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1422
 *             best_error = INF
 *             i = 0
 *             while mid - i >= 0 and i <= mid:             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */
      while (1) {
        __pyx_t_2 = (((__pyx_v_mid - __pyx_v_i) >= 0) != 0);
        if (__pyx_t_2) {
        } else {
          __pyx_t_1 = __pyx_t_2;
          goto __pyx_L8_bool_binop_done;
        }
        __pyx_t_2 = ((__pyx_v_i <= __pyx_v_mid) != 0);
        __pyx_t_1 = __pyx_t_2;
        __pyx_L8_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":1423
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid - __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":1424
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":1425
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":1426
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":1427
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid - i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */
            __pyx_v_best_index = (__pyx_v_mid - __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":1428
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":1426
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */
          }

          /* "ms_deisotope/_c/peak_set.pyx":1425
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */
          goto __pyx_L10;
        }

        /* "ms_deisotope/_c/peak_set.pyx":1430
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             start[0] = mid - i + 1
 */
        /*else*/ {
          goto __pyx_L7_break;
        }
        __pyx_L10:;

        /* "ms_deisotope/_c/peak_set.pyx":1431
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             start[0] = mid - i + 1
 *             i = 1
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L7_break:;

      /* "ms_deisotope/_c/peak_set.pyx":1432
 *                     break
 *                 i += 1
 *             start[0] = mid - i + 1             # <<<<<<<<<<<<<<
 *             i = 1
 *             while (mid + i) < (n - 1):
 */
      (__pyx_v_start[0]) = ((__pyx_v_mid - __pyx_v_i) + 1);

      /* "ms_deisotope/_c/peak_set.pyx":1433
 *                 i += 1
 *             start[0] = mid - i + 1
 *             i = 1             # <<<<<<<<<<<<<<
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 */
      __pyx_v_i = 1;

      /* "ms_deisotope/_c/peak_set.pyx":1434
 *             start[0] = mid - i + 1
 *             i = 1
 *             while (mid + i) < (n - 1):             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */
      while (1) {
        __pyx_t_1 = (((__pyx_v_mid + __pyx_v_i) < (__pyx_v_n - 1)) != 0);
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":1435
 *             i = 1
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid + __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":1436
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":1437
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":1438
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":1439
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid + i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */
            __pyx_v_best_index = (__pyx_v_mid + __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":1440
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":1438
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */
          }

          /* "ms_deisotope/_c/peak_set.pyx":1437
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */
          goto __pyx_L14;
        }

        /* "ms_deisotope/_c/peak_set.pyx":1442
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             end[0] = mid + i
 */
        /*else*/ {
          goto __pyx_L13_break;
        }
        __pyx_L14:;

        /* "ms_deisotope/_c/peak_set.pyx":1443
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             end[0] = mid + i
 *             return 0
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L13_break:;

      /* "ms_deisotope/_c/peak_set.pyx":1444
 *                     break
 *                 i += 1
 *             end[0] = mid + i             # <<<<<<<<<<<<<<
 *             return 0
 *         elif hi - lo == 1:
 */
      (__pyx_v_end[0]) = (__pyx_v_mid + __pyx_v_i);

      /* "ms_deisotope/_c/peak_set.pyx":1445
 *                 i += 1
 *             end[0] = mid + i
 *             return 0             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             start[0] = 0
 */
      __pyx_r = 0;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1418
 *         mid = (hi + lo) / 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":1446
 *             end[0] = mid + i
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             start[0] = 0
 *             end[0] = 0
 */
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1447
 *             return 0
 *         elif hi - lo == 1:
 *             start[0] = 0             # <<<<<<<<<<<<<<
 *             end[0] = 0
 *             return 1
 */
      (__pyx_v_start[0]) = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1448
 *         elif hi - lo == 1:
 *             start[0] = 0
 *             end[0] = 0             # <<<<<<<<<<<<<<
 *             return 1
 *         elif found_mass > target:
 */
      (__pyx_v_end[0]) = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1449
 *             start[0] = 0
 *             end[0] = 0
 *             return 1             # <<<<<<<<<<<<<<
 *         elif found_mass > target:
 *             hi = mid
 */
      __pyx_r = 1;
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1446
 *             end[0] = mid + i
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             start[0] = 0
 *             end[0] = 0
 */
    }

    /* "ms_deisotope/_c/peak_set.pyx":1450
 *             end[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_found_mass > __pyx_v_target) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1451
 *             return 1
 *         elif found_mass > target:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":1450
 *             end[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */
      goto __pyx_L5;
    }

    /* "ms_deisotope/_c/peak_set.pyx":1453
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 *     start[0] = 0
 *     end[0] = 0
 */
    /*else*/ {
      __pyx_v_lo = __pyx_v_mid;
    }
    __pyx_L5:;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1454
 *         else:
 *             lo = mid
 *     start[0] = 0             # <<<<<<<<<<<<<<
 *     end[0] = 0
 *     return 2
 */
  (__pyx_v_start[0]) = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1455
 *             lo = mid
 *     start[0] = 0
 *     end[0] = 0             # <<<<<<<<<<<<<<
 *     return 2
 * 
 */
  (__pyx_v_end[0]) = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1456
 *     start[0] = 0
 *     end[0] = 0
 *     return 2             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 2;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1405
 * 
 * 
 * cdef int _binary_search_interval_with_hint(double* array, double target, double error_tolerance, size_t n, size_t hint, size_t* start, size_t* end) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1459
 * 
 * 
 * cdef int build_interval_index(DeconvolutedPeakSet peaks, index_list* index, size_t index_size):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* linear_spacing
 */

static int __pyx_f_12ms_deisotope_2_c_8peak_set_build_interval_index(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_peaks, struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *__pyx_v_index, size_t __pyx_v_index_size) {
  double *__pyx_v_linear_spacing;
  double __pyx_v_current_value;
  double __pyx_v_err;
  double __pyx_v_next_value;
  size_t __pyx_v_i;
  size_t __pyx_v_start_i;
  size_t __pyx_v_end_i;
  size_t __pyx_v_index_i;
  size_t __pyx_v_peaks_size;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  double __pyx_t_3;
  size_t __pyx_t_4;
  size_t __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("build_interval_index", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1465
 *         size_t i, start_i, end_i, index_i, peaks_size
 *         DeconvolutedPeak peak
 *     peaks_size = peaks.get_size()             # <<<<<<<<<<<<<<
 *     if peaks_size > 0:
 *         index.low = peaks.getitem(0).neutral_mass
 */
  __pyx_v_peaks_size = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_peaks->__pyx_vtab)->get_size(__pyx_v_peaks);

  /* "ms_deisotope/_c/peak_set.pyx":1466
 *         DeconvolutedPeak peak
 *     peaks_size = peaks.get_size()
 *     if peaks_size > 0:             # <<<<<<<<<<<<<<
 *         index.low = peaks.getitem(0).neutral_mass
 *         index.high = peaks.getitem(peaks_size - 1).neutral_mass
 */
  __pyx_t_1 = ((__pyx_v_peaks_size > 0) != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1467
 *     peaks_size = peaks.get_size()
 *     if peaks_size > 0:
 *         index.low = peaks.getitem(0).neutral_mass             # <<<<<<<<<<<<<<
 *         index.high = peaks.getitem(peaks_size - 1).neutral_mass
 *     else:
 */
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_peaks->__pyx_vtab)->getitem(__pyx_v_peaks, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2)->neutral_mass;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_index->low = __pyx_t_3;

    /* "ms_deisotope/_c/peak_set.pyx":1468
 *     if peaks_size > 0:
 *         index.low = peaks.getitem(0).neutral_mass
 *         index.high = peaks.getitem(peaks_size - 1).neutral_mass             # <<<<<<<<<<<<<<
 *     else:
 *         index.low = 0
 */
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_peaks->__pyx_vtab)->getitem(__pyx_v_peaks, (__pyx_v_peaks_size - 1))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1468, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2)->neutral_mass;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_index->high = __pyx_t_3;

    /* "ms_deisotope/_c/peak_set.pyx":1466
 *         DeconvolutedPeak peak
 *     peaks_size = peaks.get_size()
 *     if peaks_size > 0:             # <<<<<<<<<<<<<<
 *         index.low = peaks.getitem(0).neutral_mass
 *         index.high = peaks.getitem(peaks_size - 1).neutral_mass
 */
    goto __pyx_L3;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1470
 *         index.high = peaks.getitem(peaks_size - 1).neutral_mass
 *     else:
 *         index.low = 0             # <<<<<<<<<<<<<<
 *         index.high = 1
 *     index.size = index_size
 */
  /*else*/ {
    __pyx_v_index->low = 0.0;

    /* "ms_deisotope/_c/peak_set.pyx":1471
 *     else:
 *         index.low = 0
 *         index.high = 1             # <<<<<<<<<<<<<<
 *     index.size = index_size
 *     linear_spacing = build_linear_spaced_array(
 */
    __pyx_v_index->high = 1.0;
  }
  __pyx_L3:;

  /* "ms_deisotope/_c/peak_set.pyx":1472
 *         index.low = 0
 *         index.high = 1
 *     index.size = index_size             # <<<<<<<<<<<<<<
 *     linear_spacing = build_linear_spaced_array(
 *         index.low,
 */
  __pyx_v_index->size = __pyx_v_index_size;

  /* "ms_deisotope/_c/peak_set.pyx":1473
 *         index.high = 1
 *     index.size = index_size
 *     linear_spacing = build_linear_spaced_array(             # <<<<<<<<<<<<<<
 *         index.low,
 *         index.high,
 */
  __pyx_v_linear_spacing = __pyx_f_12ms_deisotope_2_c_8peak_set_build_linear_spaced_array(__pyx_v_index->low, __pyx_v_index->high, __pyx_v_index_size);

  /* "ms_deisotope/_c/peak_set.pyx":1478
 *         index_size)
 * 
 *     index.index = <index_cell*>malloc(sizeof(index_cell) * index_size)             # <<<<<<<<<<<<<<
 * 
 *     for index_i in range(index_size):
 */
  __pyx_v_index->index = ((struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell *)malloc(((sizeof(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell)) * __pyx_v_index_size)));

  /* "ms_deisotope/_c/peak_set.pyx":1480
 *     index.index = <index_cell*>malloc(sizeof(index_cell) * index_size)
 * 
 *     for index_i in range(index_size):             # <<<<<<<<<<<<<<
 *         current_value = linear_spacing[index_i]
 *         peak = peaks._get_nearest_peak(current_value, &err)
 */
  __pyx_t_4 = __pyx_v_index_size;
  __pyx_t_5 = __pyx_t_4;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
    __pyx_v_index_i = __pyx_t_6;

    /* "ms_deisotope/_c/peak_set.pyx":1481
 * 
 *     for index_i in range(index_size):
 *         current_value = linear_spacing[index_i]             # <<<<<<<<<<<<<<
 *         peak = peaks._get_nearest_peak(current_value, &err)
 *         if peaks_size > 0:
 */
    __pyx_v_current_value = (__pyx_v_linear_spacing[__pyx_v_index_i]);

    /* "ms_deisotope/_c/peak_set.pyx":1482
 *     for index_i in range(index_size):
 *         current_value = linear_spacing[index_i]
 *         peak = peaks._get_nearest_peak(current_value, &err)             # <<<<<<<<<<<<<<
 *         if peaks_size > 0:
 *             start_i = peak._index.neutral_mass
 */
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_peaks->__pyx_vtab)->_get_nearest_peak(__pyx_v_peaks, __pyx_v_current_value, (&__pyx_v_err))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1482, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1483
 *         current_value = linear_spacing[index_i]
 *         peak = peaks._get_nearest_peak(current_value, &err)
 *         if peaks_size > 0:             # <<<<<<<<<<<<<<
 *             start_i = peak._index.neutral_mass
 *             if index_i > 0:
 */
    __pyx_t_1 = ((__pyx_v_peaks_size > 0) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1484
 *         peak = peaks._get_nearest_peak(current_value, &err)
 *         if peaks_size > 0:
 *             start_i = peak._index.neutral_mass             # <<<<<<<<<<<<<<
 *             if index_i > 0:
 *                 start_i = index.index[index_i - 1].end - 1
 */
      __pyx_t_7 = __pyx_v_peak->_index->neutral_mass;
      __pyx_v_start_i = __pyx_t_7;

      /* "ms_deisotope/_c/peak_set.pyx":1485
 *         if peaks_size > 0:
 *             start_i = peak._index.neutral_mass
 *             if index_i > 0:             # <<<<<<<<<<<<<<
 *                 start_i = index.index[index_i - 1].end - 1
 *             if index_i == index_size - 1:
 */
      __pyx_t_1 = ((__pyx_v_index_i > 0) != 0);
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":1486
 *             start_i = peak._index.neutral_mass
 *             if index_i > 0:
 *                 start_i = index.index[index_i - 1].end - 1             # <<<<<<<<<<<<<<
 *             if index_i == index_size - 1:
 *                 end_i = peaks_size - 1
 */
        __pyx_v_start_i = ((__pyx_v_index->index[(__pyx_v_index_i - 1)]).end - 1);

        /* "ms_deisotope/_c/peak_set.pyx":1485
 *         if peaks_size > 0:
 *             start_i = peak._index.neutral_mass
 *             if index_i > 0:             # <<<<<<<<<<<<<<
 *                 start_i = index.index[index_i - 1].end - 1
 *             if index_i == index_size - 1:
 */
      }

      /* "ms_deisotope/_c/peak_set.pyx":1487
 *             if index_i > 0:
 *                 start_i = index.index[index_i - 1].end - 1
 *             if index_i == index_size - 1:             # <<<<<<<<<<<<<<
 *                 end_i = peaks_size - 1
 *             else:
 */
      __pyx_t_1 = ((__pyx_v_index_i == (__pyx_v_index_size - 1)) != 0);
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":1488
 *                 start_i = index.index[index_i - 1].end - 1
 *             if index_i == index_size - 1:
 *                 end_i = peaks_size - 1             # <<<<<<<<<<<<<<
 *             else:
 *                 next_value = linear_spacing[index_i + 1]
 */
        __pyx_v_end_i = (__pyx_v_peaks_size - 1);

        /* "ms_deisotope/_c/peak_set.pyx":1487
 *             if index_i > 0:
 *                 start_i = index.index[index_i - 1].end - 1
 *             if index_i == index_size - 1:             # <<<<<<<<<<<<<<
 *                 end_i = peaks_size - 1
 *             else:
 */
        goto __pyx_L8;
      }

      /* "ms_deisotope/_c/peak_set.pyx":1490
 *                 end_i = peaks_size - 1
 *             else:
 *                 next_value = linear_spacing[index_i + 1]             # <<<<<<<<<<<<<<
 *                 i = peak._index.neutral_mass
 *                 while i < peaks_size:
 */
      /*else*/ {
        __pyx_v_next_value = (__pyx_v_linear_spacing[(__pyx_v_index_i + 1)]);

        /* "ms_deisotope/_c/peak_set.pyx":1491
 *             else:
 *                 next_value = linear_spacing[index_i + 1]
 *                 i = peak._index.neutral_mass             # <<<<<<<<<<<<<<
 *                 while i < peaks_size:
 *                     peak = peaks.getitem(i)
 */
        __pyx_t_7 = __pyx_v_peak->_index->neutral_mass;
        __pyx_v_i = __pyx_t_7;

        /* "ms_deisotope/_c/peak_set.pyx":1492
 *                 next_value = linear_spacing[index_i + 1]
 *                 i = peak._index.neutral_mass
 *                 while i < peaks_size:             # <<<<<<<<<<<<<<
 *                     peak = peaks.getitem(i)
 *                     if abs(current_value - peak.neutral_mass) > abs(next_value - peak.neutral_mass):
 */
        while (1) {
          __pyx_t_1 = ((__pyx_v_i < __pyx_v_peaks_size) != 0);
          if (!__pyx_t_1) break;

          /* "ms_deisotope/_c/peak_set.pyx":1493
 *                 i = peak._index.neutral_mass
 *                 while i < peaks_size:
 *                     peak = peaks.getitem(i)             # <<<<<<<<<<<<<<
 *                     if abs(current_value - peak.neutral_mass) > abs(next_value - peak.neutral_mass):
 *                         break
 */
          __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_peaks->__pyx_vtab)->getitem(__pyx_v_peaks, __pyx_v_i)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1493, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2));
          __pyx_t_2 = 0;

          /* "ms_deisotope/_c/peak_set.pyx":1494
 *                 while i < peaks_size:
 *                     peak = peaks.getitem(i)
 *                     if abs(current_value - peak.neutral_mass) > abs(next_value - peak.neutral_mass):             # <<<<<<<<<<<<<<
 *                         break
 *                     i += 1
 */
          __pyx_t_1 = ((fabs((__pyx_v_current_value - __pyx_v_peak->neutral_mass)) > fabs((__pyx_v_next_value - __pyx_v_peak->neutral_mass))) != 0);
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":1495
 *                     peak = peaks.getitem(i)
 *                     if abs(current_value - peak.neutral_mass) > abs(next_value - peak.neutral_mass):
 *                         break             # <<<<<<<<<<<<<<
 *                     i += 1
 *                 end_i = i
 */
            goto __pyx_L10_break;

            /* "ms_deisotope/_c/peak_set.pyx":1494
 *                 while i < peaks_size:
 *                     peak = peaks.getitem(i)
 *                     if abs(current_value - peak.neutral_mass) > abs(next_value - peak.neutral_mass):             # <<<<<<<<<<<<<<
 *                         break
 *                     i += 1
 */
          }

          /* "ms_deisotope/_c/peak_set.pyx":1496
 *                     if abs(current_value - peak.neutral_mass) > abs(next_value - peak.neutral_mass):
 *                         break
 *                     i += 1             # <<<<<<<<<<<<<<
 *                 end_i = i
 *         else:
 */
          __pyx_v_i = (__pyx_v_i + 1);
        }
        __pyx_L10_break:;

        /* "ms_deisotope/_c/peak_set.pyx":1497
 *                         break
 *                     i += 1
 *                 end_i = i             # <<<<<<<<<<<<<<
 *         else:
 *             start_i = 0
 */
        __pyx_v_end_i = __pyx_v_i;
      }
      __pyx_L8:;

      /* "ms_deisotope/_c/peak_set.pyx":1483
 *         current_value = linear_spacing[index_i]
 *         peak = peaks._get_nearest_peak(current_value, &err)
 *         if peaks_size > 0:             # <<<<<<<<<<<<<<
 *             start_i = peak._index.neutral_mass
 *             if index_i > 0:
 */
      goto __pyx_L6;
    }

    /* "ms_deisotope/_c/peak_set.pyx":1499
 *                 end_i = i
 *         else:
 *             start_i = 0             # <<<<<<<<<<<<<<
 *             end_i = 0
 *         index.index[index_i].center_value = current_value
 */
    /*else*/ {
      __pyx_v_start_i = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1500
 *         else:
 *             start_i = 0
 *             end_i = 0             # <<<<<<<<<<<<<<
 *         index.index[index_i].center_value = current_value
 *         index.index[index_i].start = start_i
 */
      __pyx_v_end_i = 0;
    }
    __pyx_L6:;

    /* "ms_deisotope/_c/peak_set.pyx":1501
 *             start_i = 0
 *             end_i = 0
 *         index.index[index_i].center_value = current_value             # <<<<<<<<<<<<<<
 *         index.index[index_i].start = start_i
 *         index.index[index_i].end = end_i
 */
    (__pyx_v_index->index[__pyx_v_index_i]).center_value = __pyx_v_current_value;

    /* "ms_deisotope/_c/peak_set.pyx":1502
 *             end_i = 0
 *         index.index[index_i].center_value = current_value
 *         index.index[index_i].start = start_i             # <<<<<<<<<<<<<<
 *         index.index[index_i].end = end_i
 * 
 */
    (__pyx_v_index->index[__pyx_v_index_i]).start = __pyx_v_start_i;

    /* "ms_deisotope/_c/peak_set.pyx":1503
 *         index.index[index_i].center_value = current_value
 *         index.index[index_i].start = start_i
 *         index.index[index_i].end = end_i             # <<<<<<<<<<<<<<
 * 
 *     free(linear_spacing)
 */
    (__pyx_v_index->index[__pyx_v_index_i]).end = __pyx_v_end_i;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1505
 *         index.index[index_i].end = end_i
 * 
 *     free(linear_spacing)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_linear_spacing);

  /* "ms_deisotope/_c/peak_set.pyx":1506
 * 
 *     free(linear_spacing)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1459
 * 
 * 
 * cdef int build_interval_index(DeconvolutedPeakSet peaks, index_list* index, size_t index_size):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* linear_spacing
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.build_interval_index", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1509
 * 
 * 
 * cdef double* build_linear_spaced_array(double low, double high, size_t n):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* array
 */

static double *__pyx_f_12ms_deisotope_2_c_8peak_set_build_linear_spaced_array(double __pyx_v_low, double __pyx_v_high, size_t __pyx_v_n) {
  double *__pyx_v_array;
  double __pyx_v_delta;
  size_t __pyx_v_i;
  double *__pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("build_linear_spaced_array", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1515
 *         size_t i
 * 
 *     delta = (high - low) / (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     array = <double*>malloc(sizeof(double) * n)
 */
  __pyx_t_1 = (__pyx_v_high - __pyx_v_low);
  __pyx_t_2 = (__pyx_v_n - 1);
  if (unlikely(__pyx_t_2 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 1515, __pyx_L1_error)
  }
  __pyx_v_delta = (__pyx_t_1 / __pyx_t_2);

  /* "ms_deisotope/_c/peak_set.pyx":1517
 *     delta = (high - low) / (n - 1)
 * 
 *     array = <double*>malloc(sizeof(double) * n)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(n):
 */
  __pyx_v_array = ((double *)malloc(((sizeof(double)) * __pyx_v_n)));

  /* "ms_deisotope/_c/peak_set.pyx":1519
 *     array = <double*>malloc(sizeof(double) * n)
 * 
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         array[i] = low + i * delta
 *     return array
 */
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "ms_deisotope/_c/peak_set.pyx":1520
 * 
 *     for i in range(n):
 *         array[i] = low + i * delta             # <<<<<<<<<<<<<<
 *     return array
 * 
 */
    (__pyx_v_array[__pyx_v_i]) = (__pyx_v_low + (__pyx_v_i * __pyx_v_delta));
  }

  /* "ms_deisotope/_c/peak_set.pyx":1521
 *     for i in range(n):
 *         array[i] = low + i * delta
 *     return array             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_array;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1509
 * 
 * 
 * cdef double* build_linear_spaced_array(double low, double high, size_t n):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* array
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.build_linear_spaced_array", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1524
 * 
 * 
 * cdef void free_index_list(index_list* index):             # <<<<<<<<<<<<<<
 *     free(index.index)
 *     free(index)
 */

static void __pyx_f_12ms_deisotope_2_c_8peak_set_free_index_list(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *__pyx_v_index) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("free_index_list", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1525
 * 
 * cdef void free_index_list(index_list* index):
 *     free(index.index)             # <<<<<<<<<<<<<<
 *     free(index)
 * 
 */
  free(__pyx_v_index->index);

  /* "ms_deisotope/_c/peak_set.pyx":1526
 * cdef void free_index_list(index_list* index):
 *     free(index.index)
 *     free(index)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_index);

  /* "ms_deisotope/_c/peak_set.pyx":1524
 * 
 * 
 * cdef void free_index_list(index_list* index):             # <<<<<<<<<<<<<<
 *     free(index.index)
 *     free(index)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "ms_deisotope/_c/peak_set.pyx":1529
 * 
 * 
 * cdef int check_index(index_list* index) nogil:             # <<<<<<<<<<<<<<
 *     if index.size == 0:
 *         return 1
 */

static int __pyx_f_12ms_deisotope_2_c_8peak_set_check_index(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *__pyx_v_index) {
  int __pyx_r;
  int __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1530
 * 
 * cdef int check_index(index_list* index) nogil:
 *     if index.size == 0:             # <<<<<<<<<<<<<<
 *         return 1
 *     elif (index.high - index.low) == 0:
 */
  __pyx_t_1 = ((__pyx_v_index->size == 0) != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1531
 * cdef int check_index(index_list* index) nogil:
 *     if index.size == 0:
 *         return 1             # <<<<<<<<<<<<<<
 *     elif (index.high - index.low) == 0:
 *         return 2
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1530
 * 
 * cdef int check_index(index_list* index) nogil:
 *     if index.size == 0:             # <<<<<<<<<<<<<<
 *         return 1
 *     elif (index.high - index.low) == 0:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1532
 *     if index.size == 0:
 *         return 1
 *     elif (index.high - index.low) == 0:             # <<<<<<<<<<<<<<
 *         return 2
 *     else:
 */
  __pyx_t_1 = (((__pyx_v_index->high - __pyx_v_index->low) == 0.0) != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1533
 *         return 1
 *     elif (index.high - index.low) == 0:
 *         return 2             # <<<<<<<<<<<<<<
 *     else:
 *         return 0
 */
    __pyx_r = 2;
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1532
 *     if index.size == 0:
 *         return 1
 *     elif (index.high - index.low) == 0:             # <<<<<<<<<<<<<<
 *         return 2
 *     else:
 */
  }

  /* "ms_deisotope/_c/peak_set.pyx":1535
 *         return 2
 *     else:
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1529
 * 
 * 
 * cdef int check_index(index_list* index) nogil:             # <<<<<<<<<<<<<<
 *     if index.size == 0:
 *         return 1
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1538
 * 
 * 
 * cdef size_t interpolate_index(index_list* index, double value):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double v
 */

static size_t __pyx_f_12ms_deisotope_2_c_8peak_set_interpolate_index(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *__pyx_v_index, double __pyx_v_value) {
  double __pyx_v_v;
  size_t __pyx_v_i;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  double __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("interpolate_index", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1542
 *         double v
 *         size_t i
 *     v = (((value - index.low) / (index.high - index.low)) * index.size)             # <<<<<<<<<<<<<<
 *     i = <size_t>v
 *     return i
 */
  __pyx_t_1 = (__pyx_v_value - __pyx_v_index->low);
  __pyx_t_2 = (__pyx_v_index->high - __pyx_v_index->low);
  if (unlikely(__pyx_t_2 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 1542, __pyx_L1_error)
  }
  __pyx_v_v = ((__pyx_t_1 / __pyx_t_2) * __pyx_v_index->size);

  /* "ms_deisotope/_c/peak_set.pyx":1543
 *         size_t i
 *     v = (((value - index.low) / (index.high - index.low)) * index.size)
 *     i = <size_t>v             # <<<<<<<<<<<<<<
 *     return i
 * 
 */
  __pyx_v_i = ((size_t)__pyx_v_v);

  /* "ms_deisotope/_c/peak_set.pyx":1544
 *     v = (((value - index.low) / (index.high - index.low)) * index.size)
 *     i = <size_t>v
 *     return i             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_i;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1538
 * 
 * 
 * cdef size_t interpolate_index(index_list* index, double value):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double v
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.interpolate_index", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ms_deisotope/_c/peak_set.pyx":1547
 * 
 * 
 * cdef int find_search_interval(index_list* index, double value, size_t* start, size_t* end):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

static int __pyx_f_12ms_deisotope_2_c_8peak_set_find_search_interval(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *__pyx_v_index, double __pyx_v_value, size_t *__pyx_v_start, size_t *__pyx_v_end) {
  size_t __pyx_v_i;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  size_t __pyx_t_2;
  __Pyx_RefNannySetupContext("find_search_interval", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1550
 *     cdef:
 *         size_t i
 *     if value > index.high:             # <<<<<<<<<<<<<<
 *         i = index.size - 1
 *     elif value < index.low:
 */
  __pyx_t_1 = ((__pyx_v_value > __pyx_v_index->high) != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1551
 *         size_t i
 *     if value > index.high:
 *         i = index.size - 1             # <<<<<<<<<<<<<<
 *     elif value < index.low:
 *         i = 0
 */
    __pyx_v_i = (__pyx_v_index->size - 1);

    /* "ms_deisotope/_c/peak_set.pyx":1550
 *     cdef:
 *         size_t i
 *     if value > index.high:             # <<<<<<<<<<<<<<
 *         i = index.size - 1
 *     elif value < index.low:
 */
    goto __pyx_L3;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1552
 *     if value > index.high:
 *         i = index.size - 1
 *     elif value < index.low:             # <<<<<<<<<<<<<<
 *         i = 0
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_value < __pyx_v_index->low) != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1553
 *         i = index.size - 1
 *     elif value < index.low:
 *         i = 0             # <<<<<<<<<<<<<<
 *     else:
 *         i = interpolate_index(index, value)
 */
    __pyx_v_i = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1552
 *     if value > index.high:
 *         i = index.size - 1
 *     elif value < index.low:             # <<<<<<<<<<<<<<
 *         i = 0
 *     else:
 */
    goto __pyx_L3;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1555
 *         i = 0
 *     else:
 *         i = interpolate_index(index, value)             # <<<<<<<<<<<<<<
 *     if i > 0:
 *         if i < index.size:
 */
  /*else*/ {
    __pyx_v_i = __pyx_f_12ms_deisotope_2_c_8peak_set_interpolate_index(__pyx_v_index, __pyx_v_value);
  }
  __pyx_L3:;

  /* "ms_deisotope/_c/peak_set.pyx":1556
 *     else:
 *         i = interpolate_index(index, value)
 *     if i > 0:             # <<<<<<<<<<<<<<
 *         if i < index.size:
 *             start[0] = index.index[i - 1].start
 */
  __pyx_t_1 = ((__pyx_v_i > 0) != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1557
 *         i = interpolate_index(index, value)
 *     if i > 0:
 *         if i < index.size:             # <<<<<<<<<<<<<<
 *             start[0] = index.index[i - 1].start
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_i < __pyx_v_index->size) != 0);
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1558
 *     if i > 0:
 *         if i < index.size:
 *             start[0] = index.index[i - 1].start             # <<<<<<<<<<<<<<
 *         else:
 *             # if we're at index.size or greater, act as if we're at the last
 */
      __pyx_t_2 = (__pyx_v_index->index[(__pyx_v_i - 1)]).start;
      (__pyx_v_start[0]) = __pyx_t_2;

      /* "ms_deisotope/_c/peak_set.pyx":1557
 *         i = interpolate_index(index, value)
 *     if i > 0:
 *         if i < index.size:             # <<<<<<<<<<<<<<
 *             start[0] = index.index[i - 1].start
 *         else:
 */
      goto __pyx_L5;
    }

    /* "ms_deisotope/_c/peak_set.pyx":1562
 *             # if we're at index.size or greater, act as if we're at the last
 *             # cell of the index
 *             start[0] = index.index[index.size - 2].start             # <<<<<<<<<<<<<<
 *     else:
 *         # if somehow the index were negative, this could end badly.
 */
    /*else*/ {
      __pyx_t_2 = (__pyx_v_index->index[(__pyx_v_index->size - 2)]).start;
      (__pyx_v_start[0]) = __pyx_t_2;
    }
    __pyx_L5:;

    /* "ms_deisotope/_c/peak_set.pyx":1556
 *     else:
 *         i = interpolate_index(index, value)
 *     if i > 0:             # <<<<<<<<<<<<<<
 *         if i < index.size:
 *             start[0] = index.index[i - 1].start
 */
    goto __pyx_L4;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1565
 *     else:
 *         # if somehow the index were negative, this could end badly.
 *         start[0] = index.index[i].start             # <<<<<<<<<<<<<<
 *     if i >= (index.size - 1):
 *         end[0] = index.index[index.size - 1].end + 1
 */
  /*else*/ {
    __pyx_t_2 = (__pyx_v_index->index[__pyx_v_i]).start;
    (__pyx_v_start[0]) = __pyx_t_2;
  }
  __pyx_L4:;

  /* "ms_deisotope/_c/peak_set.pyx":1566
 *         # if somehow the index were negative, this could end badly.
 *         start[0] = index.index[i].start
 *     if i >= (index.size - 1):             # <<<<<<<<<<<<<<
 *         end[0] = index.index[index.size - 1].end + 1
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_i >= (__pyx_v_index->size - 1)) != 0);
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1567
 *         start[0] = index.index[i].start
 *     if i >= (index.size - 1):
 *         end[0] = index.index[index.size - 1].end + 1             # <<<<<<<<<<<<<<
 *     else:
 *         end[0] = index.index[i + 1].end + 1
 */
    (__pyx_v_end[0]) = ((__pyx_v_index->index[(__pyx_v_index->size - 1)]).end + 1);

    /* "ms_deisotope/_c/peak_set.pyx":1566
 *         # if somehow the index were negative, this could end badly.
 *         start[0] = index.index[i].start
 *     if i >= (index.size - 1):             # <<<<<<<<<<<<<<
 *         end[0] = index.index[index.size - 1].end + 1
 *     else:
 */
    goto __pyx_L6;
  }

  /* "ms_deisotope/_c/peak_set.pyx":1569
 *         end[0] = index.index[index.size - 1].end + 1
 *     else:
 *         end[0] = index.index[i + 1].end + 1             # <<<<<<<<<<<<<<
 *     return 0
 */
  /*else*/ {
    (__pyx_v_end[0]) = ((__pyx_v_index->index[(__pyx_v_i + 1)]).end + 1);
  }
  __pyx_L6:;

  /* "ms_deisotope/_c/peak_set.pyx":1570
 *     else:
 *         end[0] = index.index[i + 1].end + 1
 *     return 0             # <<<<<<<<<<<<<<
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1547
 * 
 * 
 * cdef int find_search_interval(index_list* index, double value, size_t* start, size_t* end):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":734
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":735
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":734
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":737
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":738
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 738, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":737
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":740
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":741
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 741, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":740
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":743
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":744
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":743
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":746
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":747
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 747, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":746
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":749
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":750
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":751
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":750
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":753
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":749
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":868
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":869
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":870
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":868
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":872
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":873
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":874
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":875
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":874
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":876
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":872
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":880
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":882
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 882, __pyx_L3_error)

      /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":883
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 883, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":884
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 884, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 884, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":880
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":886
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":888
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 888, __pyx_L3_error)

      /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":889
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 889, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":890
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 890, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 890, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":886
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":892
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":894
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 894, __pyx_L3_error)

      /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":895
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 895, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":896
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef extern from *:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 896, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 896, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":892
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":122
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

/* Python wrapper */
static int __pyx_array___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_array___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shape = 0;
  Py_ssize_t __pyx_v_itemsize;
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_v_mode = 0;
  int __pyx_v_allocate_buffer;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shape,&__pyx_n_s_itemsize,&__pyx_n_s_format,&__pyx_n_s_mode,&__pyx_n_s_allocate_buffer,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[3] = ((PyObject *)__pyx_n_s_c);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_shape)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_itemsize)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, 1); __PYX_ERR(3, 122, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_format)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, 2); __PYX_ERR(3, 122, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mode);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_allocate_buffer);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(3, 122, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_shape = ((PyObject*)values[0]);
    __pyx_v_itemsize = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_itemsize == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 122, __pyx_L3_error)
    __pyx_v_format = values[2];
    __pyx_v_mode = values[3];
    if (values[4]) {
      __pyx_v_allocate_buffer = __Pyx_PyObject_IsTrue(values[4]); if (unlikely((__pyx_v_allocate_buffer == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 123, __pyx_L3_error)
    } else {

      /* "View.MemoryView":123
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,
 *                   mode="c", bint allocate_buffer=True):             # <<<<<<<<<<<<<<
 * 
 *         cdef int idx
 */
      __pyx_v_allocate_buffer = ((int)1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(3, 122, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("View.MemoryView.array.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_shape), (&PyTuple_Type), 1, "shape", 1))) __PYX_ERR(3, 122, __pyx_L1_error)
  if (unlikely(((PyObject *)__pyx_v_format) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "format"); __PYX_ERR(3, 122, __pyx_L1_error)
  }
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(((struct __pyx_array_obj *)__pyx_v_self), __pyx_v_shape, __pyx_v_itemsize, __pyx_v_format, __pyx_v_mode, __pyx_v_allocate_buffer);

  /* "View.MemoryView":122
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, PyObject *__pyx_v_format, PyObject *__pyx_v_mode, int __pyx_v_allocate_buffer) {
  int __pyx_v_idx;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_dim;
  PyObject **__pyx_v_p;
  char __pyx_v_order;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  char *__pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);
  __Pyx_INCREF(__pyx_v_format);

  /* "View.MemoryView":129
 *         cdef PyObject **p
 * 
 *         self.ndim = <int> len(shape)             # <<<<<<<<<<<<<<
 *         self.itemsize = itemsize
 * 
 */
  if (unlikely(__pyx_v_shape == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(3, 129, __pyx_L1_error)
  }
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_shape); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(3, 129, __pyx_L1_error)
  __pyx_v_self->ndim = ((int)__pyx_t_1);

  /* "View.MemoryView":130
 * 
 *         self.ndim = <int> len(shape)
 *         self.itemsize = itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not self.ndim:
 */
  __pyx_v_self->itemsize = __pyx_v_itemsize;

  /* "View.MemoryView":132
 *         self.itemsize = itemsize
 * 
 *         if not self.ndim:             # <<<<<<<<<<<<<<
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 */
  __pyx_t_2 = ((!(__pyx_v_self->ndim != 0)) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "View.MemoryView":133
 * 
 *         if not self.ndim:
 *             raise ValueError("Empty shape tuple for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if itemsize <= 0:
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 133, __pyx_L1_error)

    /* "View.MemoryView":132
 *         self.itemsize = itemsize
 * 
 *         if not self.ndim:             # <<<<<<<<<<<<<<
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 */
  }

  /* "View.MemoryView":135
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 *         if itemsize <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 */
  __pyx_t_2 = ((__pyx_v_itemsize <= 0) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "View.MemoryView":136
 * 
 *         if itemsize <= 0:
 *             raise ValueError("itemsize <= 0 for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(format, bytes):
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 136, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 136, __pyx_L1_error)

    /* "View.MemoryView":135
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 *         if itemsize <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 */
  }

  /* "View.MemoryView":138
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 *         if not isinstance(format, bytes):             # <<<<<<<<<<<<<<
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 */
  __pyx_t_2 = PyBytes_Check(__pyx_v_format); 
  __pyx_t_4 = ((!(__pyx_t_2 != 0)) != 0);
  if (__pyx_t_4) {

    /* "View.MemoryView":139
 * 
 *         if not isinstance(format, bytes):
 *             format = format.encode('ASCII')             # <<<<<<<<<<<<<<
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_format, __pyx_n_s_encode); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_n_s_ASCII) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_s_ASCII);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_format, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "View.MemoryView":138
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 *         if not isinstance(format, bytes):             # <<<<<<<<<<<<<<
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 */
  }

  /* "View.MemoryView":140
 *         if not isinstance(format, bytes):
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string             # <<<<<<<<<<<<<<
 *         self.format = self._format
 * 
 */
  if (!(likely(PyBytes_CheckExact(__pyx_v_format))||((__pyx_v_format) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_format)->tp_name), 0))) __PYX_ERR(3, 140, __pyx_L1_error)
  __pyx_t_3 = __pyx_v_format;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_format);
  __Pyx_DECREF(__pyx_v_self->_format);
  __pyx_v_self->_format = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "View.MemoryView":141
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (unlikely(__pyx_v_self->_format == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(3, 141, __pyx_L1_error)
  }
  __pyx_t_7 = __Pyx_PyBytes_AsWritableString(__pyx_v_self->_format); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) __PYX_ERR(3, 141, __pyx_L1_error)
  __pyx_v_self->format = __pyx_t_7;

  /* "View.MemoryView":144
 * 
 * 
 *         self._shape = <Py_ssize_t *> PyObject_Malloc(sizeof(Py_ssize_t)*self.ndim*2)             # <<<<<<<<<<<<<<
 *         self._strides = self._shape + self.ndim
 * 
 */
  __pyx_v_self->_shape = ((Py_ssize_t *)PyObject_Malloc((((sizeof(Py_ssize_t)) * __pyx_v_self->ndim) * 2)));

  /* "View.MemoryView":145
 * 
 *         self._shape = <Py_ssize_t *> PyObject_Malloc(sizeof(Py_ssize_t)*self.ndim*2)
 *         self._strides = self._shape + self.ndim             # <<<<<<<<<<<<<<
 * 
 *         if not self._shape:
 */
  __pyx_v_self->_strides = (__pyx_v_self->_shape + __pyx_v_self->ndim);

  /* "View.MemoryView":147
 *         self._strides = self._shape + self.ndim
 * 
 *         if not self._shape:             # <<<<<<<<<<<<<<
 *             raise MemoryError("unable to allocate shape and strides.")
 * 
 */
  __pyx_t_4 = ((!(__pyx_v_self->_shape != 0)) != 0);
  if (unlikely(__pyx_t_4)) {

    /* "View.MemoryView":148
 * 
 *         if not self._shape:
 *             raise MemoryError("unable to allocate shape and strides.")             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 148, __pyx_L1_error)

    /* "View.MemoryView":147
 *         self._strides = self._shape + self.ndim
 * 
 *         if not self._shape:             # <<<<<<<<<<<<<<
 *             raise MemoryError("unable to allocate shape and strides.")
 * 
 */
  }

  /* "View.MemoryView":151
 * 
 * 
 *         for idx, dim in enumerate(shape):             # <<<<<<<<<<<<<<
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 */
  __pyx_t_8 = 0;
  __pyx_t_3 = __pyx_v_shape; __Pyx_INCREF(__pyx_t_3); __pyx_t_1 = 0;
  for (;;) {
    if (__pyx_t_1 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_1); __Pyx_INCREF(__pyx_t_5); __pyx_t_1++; if (unlikely(0 < 0)) __PYX_ERR(3, 151, __pyx_L1_error)
    #else
    __pyx_t_5 = PySequence_ITEM(__pyx_t_3, __pyx_t_1); __pyx_t_1++; if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    #endif
    __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_t_5); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 151, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_dim = __pyx_t_9;
    __pyx_v_idx = __pyx_t_8;
    __pyx_t_8 = (__pyx_t_8 + 1);

    /* "View.MemoryView":152
 * 
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim
 */
    __pyx_t_4 = ((__pyx_v_dim <= 0) != 0);
    if (unlikely(__pyx_t_4)) {

      /* "View.MemoryView":153
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))             # <<<<<<<<<<<<<<
 *             self._shape[idx] = dim
 * 
 */
      __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_idx); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 153, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 153, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(3, 153, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_6);
      __pyx_t_5 = 0;
      __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_shape_in_axis_d_d, __pyx_t_10); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 153, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_10)) __PYX_ERR(3, 153, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_Raise(__pyx_t_10, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __PYX_ERR(3, 153, __pyx_L1_error)

      /* "View.MemoryView":152
 * 
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim
 */
    }

    /* "View.MemoryView":154
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim             # <<<<<<<<<<<<<<
 * 
 *         cdef char order
 */
    (__pyx_v_self->_shape[__pyx_v_idx]) = __pyx_v_dim;

    /* "View.MemoryView":151
 * 
 * 
 *         for idx, dim in enumerate(shape):             # <<<<<<<<<<<<<<
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":157
 * 
 *         cdef char order
 *         if mode == 'fortran':             # <<<<<<<<<<<<<<
 *             order = b'F'
 *             self.mode = u'fortran'
 */
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_mode, __pyx_n_s_fortran, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(3, 157, __pyx_L1_error)
  if (__pyx_t_4) {

    /* "View.MemoryView":158
 *         cdef char order
 *         if mode == 'fortran':
 *             order = b'F'             # <<<<<<<<<<<<<<
 *             self.mode = u'fortran'
 *         elif mode == 'c':
 */
    __pyx_v_order = 'F';

    /* "View.MemoryView":159
 *         if mode == 'fortran':
 *             order = b'F'
 *             self.mode = u'fortran'             # <<<<<<<<<<<<<<
 *         elif mode == 'c':
 *             order = b'C'
 */
    __Pyx_INCREF(__pyx_n_u_fortran);
    __Pyx_GIVEREF(__pyx_n_u_fortran);
    __Pyx_GOTREF(__pyx_v_self->mode);
    __Pyx_DECREF(__pyx_v_self->mode);
    __pyx_v_self->mode = __pyx_n_u_fortran;

    /* "View.MemoryView":157
 * 
 *         cdef char order
 *         if mode == 'fortran':             # <<<<<<<<<<<<<<
 *             order = b'F'
 *             self.mode = u'fortran'
 */
    goto __pyx_L10;
  }

  /* "View.MemoryView":160
 *             order = b'F'
 *             self.mode = u'fortran'
 *         elif mode == 'c':             # <<<<<<<<<<<<<<
 *             order = b'C'
 *             self.mode = u'c'
 */
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_mode, __pyx_n_s_c, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(3, 160, __pyx_L1_error)
  if (likely(__pyx_t_4)) {

    /* "View.MemoryView":161
 *             self.mode = u'fortran'
 *         elif mode == 'c':
 *             order = b'C'             # <<<<<<<<<<<<<<
 *             self.mode = u'c'
 *         else:
 */
    __pyx_v_order = 'C';

    /* "View.MemoryView":162
 *         elif mode == 'c':
 *             order = b'C'
 *             self.mode = u'c'             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)
 */
    __Pyx_INCREF(__pyx_n_u_c);
    __Pyx_GIVEREF(__pyx_n_u_c);
    __Pyx_GOTREF(__pyx_v_self->mode);
    __Pyx_DECREF(__pyx_v_self->mode);
    __pyx_v_self->mode = __pyx_n_u_c;

    /* "View.MemoryView":160
 *             order = b'F'
 *             self.mode = u'fortran'
 *         elif mode == 'c':             # <<<<<<<<<<<<<<
 *             order = b'C'
 *             self.mode = u'c'
 */
    goto __pyx_L10;
  }

  /* "View.MemoryView":164
 *             self.mode = u'c'
 *         else:
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)             # <<<<<<<<<<<<<<
 * 
 *         self.len = fill_contig_strides_array(self._shape, self._strides,
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyString_FormatSafe(__pyx_kp_s_Invalid_mode_expected_c_or_fortr, __pyx_v_mode); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_10 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(3, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_10, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __PYX_ERR(3, 164, __pyx_L1_error)
  }
  __pyx_L10:;

  /* "View.MemoryView":166
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)
 * 
 *         self.len = fill_contig_strides_array(self._shape, self._strides,             # <<<<<<<<<<<<<<
 *                                              itemsize, self.ndim, order)
 * 
 */
  __pyx_v_self->len = __pyx_fill_contig_strides_array(__pyx_v_self->_shape, __pyx_v_self->_strides, __pyx_v_itemsize, __pyx_v_self->ndim, __pyx_v_order);

  /* "View.MemoryView":169
 *                                              itemsize, self.ndim, order)
 * 
 *         self.free_data = allocate_buffer             # <<<<<<<<<<<<<<
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:
 */
  __pyx_v_self->free_data = __pyx_v_allocate_buffer;

  /* "View.MemoryView":170
 * 
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'             # <<<<<<<<<<<<<<
 *         if allocate_buffer:
 * 
 */
  __pyx_t_10 = PyObject_RichCompare(__pyx_v_format, __pyx_n_b_O, Py_EQ); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(3, 170, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 170, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_self->dtype_is_object = __pyx_t_4;

  /* "View.MemoryView":171
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = (__pyx_v_allocate_buffer != 0);
  if (__pyx_t_4) {

    /* "View.MemoryView":174
 * 
 * 
 *             self.data = <char *>malloc(self.len)             # <<<<<<<<<<<<<<
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")
 */
    __pyx_v_self->data = ((char *)malloc(__pyx_v_self->len));

    /* "View.MemoryView":175
 * 
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:             # <<<<<<<<<<<<<<
 *                 raise MemoryError("unable to allocate array data.")
 * 
 */
    __pyx_t_4 = ((!(__pyx_v_self->data != 0)) != 0);
    if (unlikely(__pyx_t_4)) {

      /* "View.MemoryView":176
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")             # <<<<<<<<<<<<<<
 * 
 *             if self.dtype_is_object:
 */
      __pyx_t_10 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(3, 176, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_Raise(__pyx_t_10, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __PYX_ERR(3, 176, __pyx_L1_error)

      /* "View.MemoryView":175
 * 
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:             # <<<<<<<<<<<<<<
 *                 raise MemoryError("unable to allocate array data.")
 * 
 */
    }

    /* "View.MemoryView":178
 *                 raise MemoryError("unable to allocate array data.")
 * 
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 */
    __pyx_t_4 = (__pyx_v_self->dtype_is_object != 0);
    if (__pyx_t_4) {

      /* "View.MemoryView":179
 * 
 *             if self.dtype_is_object:
 *                 p = <PyObject **> self.data             # <<<<<<<<<<<<<<
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None
 */
      __pyx_v_p = ((PyObject **)__pyx_v_self->data);

      /* "View.MemoryView":180
 *             if self.dtype_is_object:
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):             # <<<<<<<<<<<<<<
 *                     p[i] = Py_None
 *                     Py_INCREF(Py_None)
 */
      if (unlikely(__pyx_v_itemsize == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        __PYX_ERR(3, 180, __pyx_L1_error)
      }
      else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((Py_ssize_t)-1) > 0)) && unlikely(__pyx_v_itemsize == (Py_ssize_t)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_self->len))) {
        PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
        __PYX_ERR(3, 180, __pyx_L1_error)
      }
      __pyx_t_1 = __Pyx_div_Py_ssize_t(__pyx_v_self->len, __pyx_v_itemsize);
      __pyx_t_9 = __pyx_t_1;
      for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_9; __pyx_t_11+=1) {
        __pyx_v_i = __pyx_t_11;

        /* "View.MemoryView":181
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None             # <<<<<<<<<<<<<<
 *                     Py_INCREF(Py_None)
 * 
 */
        (__pyx_v_p[__pyx_v_i]) = Py_None;

        /* "View.MemoryView":182
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None
 *                     Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */
        Py_INCREF(Py_None);
      }

      /* "View.MemoryView":178
 *                 raise MemoryError("unable to allocate array data.")
 * 
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 */
    }

    /* "View.MemoryView":171
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:             # <<<<<<<<<<<<<<
 * 
 * 
 */
  }

  /* "View.MemoryView":122
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("View.MemoryView.array.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_format);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":185
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(((struct __pyx_array_obj *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(struct __pyx_array_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_bufmode;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  if (__pyx_v_info == NULL) {
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");
    return -1;
  }
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "View.MemoryView":186
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1             # <<<<<<<<<<<<<<
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 */
  __pyx_v_bufmode = -1;

  /* "View.MemoryView":187
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
 *         if self.mode == u"c":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_self->mode, __pyx_n_u_c, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(3, 187, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":188
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS             # <<<<<<<<<<<<<<
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 */
    __pyx_v_bufmode = (PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS);

    /* "View.MemoryView":187
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
 *         if self.mode == u"c":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":189
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 */
  __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_self->mode, __pyx_n_u_fortran, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(3, 189, __pyx_L1_error)
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":190
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS             # <<<<<<<<<<<<<<
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 */
    __pyx_v_bufmode = (PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS);

    /* "View.MemoryView":189
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 */
  }
  __pyx_L3:;

  /* "View.MemoryView":191
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):             # <<<<<<<<<<<<<<
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 */
  __pyx_t_1 = ((!((__pyx_v_flags & __pyx_v_bufmode) != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":192
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")             # <<<<<<<<<<<<<<
 *         info.buf = self.data
 *         info.len = self.len
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 192, __pyx_L1_error)

    /* "View.MemoryView":191
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):             # <<<<<<<<<<<<<<
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 */
  }

  /* "View.MemoryView":193
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data             # <<<<<<<<<<<<<<
 *         info.len = self.len
 *         info.ndim = self.ndim
 */
  __pyx_t_4 = __pyx_v_self->data;
  __pyx_v_info->buf = __pyx_t_4;

  /* "View.MemoryView":194
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 *         info.len = self.len             # <<<<<<<<<<<<<<
 *         info.ndim = self.ndim
 *         info.shape = self._shape
 */
  __pyx_t_5 = __pyx_v_self->len;
  __pyx_v_info->len = __pyx_t_5;

  /* "View.MemoryView":195
 *         info.buf = self.data
 *         info.len = self.len
 *         info.ndim = self.ndim             # <<<<<<<<<<<<<<
 *         info.shape = self._shape
 *         info.strides = self._strides
 */
  __pyx_t_6 = __pyx_v_self->ndim;
  __pyx_v_info->ndim = __pyx_t_6;

  /* "View.MemoryView":196
 *         info.len = self.len
 *         info.ndim = self.ndim
 *         info.shape = self._shape             # <<<<<<<<<<<<<<
 *         info.strides = self._strides
 *         info.suboffsets = NULL
 */
  __pyx_t_7 = __pyx_v_self->_shape;
  __pyx_v_info->shape = __pyx_t_7;

  /* "View.MemoryView":197
 *         info.ndim = self.ndim
 *         info.shape = self._shape
 *         info.strides = self._strides             # <<<<<<<<<<<<<<
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize
 */
  __pyx_t_7 = __pyx_v_self->_strides;
  __pyx_v_info->strides = __pyx_t_7;

  /* "View.MemoryView":198
 *         info.shape = self._shape
 *         info.strides = self._strides
 *         info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *         info.itemsize = self.itemsize
 *         info.readonly = 0
 */
  __pyx_v_info->suboffsets = NULL;

  /* "View.MemoryView":199
 *         info.strides = self._strides
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize             # <<<<<<<<<<<<<<
 *         info.readonly = 0
 * 
 */
  __pyx_t_5 = __pyx_v_self->itemsize;
  __pyx_v_info->itemsize = __pyx_t_5;

  /* "View.MemoryView":200
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize
 *         info.readonly = 0             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */
  __pyx_v_info->readonly = 0;

  /* "View.MemoryView":202
 *         info.readonly = 0
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.format
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":203
 * 
 *         if flags & PyBUF_FORMAT:
 *             info.format = self.format             # <<<<<<<<<<<<<<
 *         else:
 *             info.format = NULL
 */
    __pyx_t_4 = __pyx_v_self->format;
    __pyx_v_info->format = __pyx_t_4;

    /* "View.MemoryView":202
 *         info.readonly = 0
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.format
 *         else:
 */
    goto __pyx_L5;
  }

  /* "View.MemoryView":205
 *             info.format = self.format
 *         else:
 *             info.format = NULL             # <<<<<<<<<<<<<<
 * 
 *         info.obj = self
 */
  /*else*/ {
    __pyx_v_info->format = NULL;
  }
  __pyx_L5:;

  /* "View.MemoryView":207
 *             info.format = NULL
 * 
 *         info.obj = self             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "View.MemoryView":185
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.array.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":211
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 * 
 *     def __dealloc__(array self):             # <<<<<<<<<<<<<<
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 */

/* Python wrapper */
static void __pyx_array___dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_array___dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(((struct __pyx_array_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(struct __pyx_array_obj *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":212
 * 
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:             # <<<<<<<<<<<<<<
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 */
  __pyx_t_1 = ((__pyx_v_self->callback_free_data != NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":213
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)             # <<<<<<<<<<<<<<
 *         elif self.free_data:
 *             if self.dtype_is_object:
 */
    __pyx_v_self->callback_free_data(__pyx_v_self->data);

    /* "View.MemoryView":212
 * 
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:             # <<<<<<<<<<<<<<
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":214
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 *         elif self.free_data:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,
 */
  __pyx_t_1 = (__pyx_v_self->free_data != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":215
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 */
    __pyx_t_1 = (__pyx_v_self->dtype_is_object != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":216
 *         elif self.free_data:
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,             # <<<<<<<<<<<<<<
 *                                           self._strides, self.ndim, False)
 *             free(self.data)
 */
      __pyx_memoryview_refcount_objects_in_slice(__pyx_v_self->data, __pyx_v_self->_shape, __pyx_v_self->_strides, __pyx_v_self->ndim, 0);

      /* "View.MemoryView":215
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 */
    }

    /* "View.MemoryView":218
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 *             free(self.data)             # <<<<<<<<<<<<<<
 *         PyObject_Free(self._shape)
 * 
 */
    free(__pyx_v_self->data);

    /* "View.MemoryView":214
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 *         elif self.free_data:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,
 */
  }
  __pyx_L3:;

  /* "View.MemoryView":219
 *                                           self._strides, self.ndim, False)
 *             free(self.data)
 *         PyObject_Free(self._shape)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  PyObject_Free(__pyx_v_self->_shape);

  /* "View.MemoryView":211
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 * 
 *     def __dealloc__(array self):             # <<<<<<<<<<<<<<
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":222
 * 
 *     @property
 *     def memview(self):             # <<<<<<<<<<<<<<
 *         return self.get_memview()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_5array_7memview_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_5array_7memview_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_5array_7memview___get__(((struct __pyx_array_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_5array_7memview___get__(struct __pyx_array_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":223
 *     @property
 *     def memview(self):
 *         return self.get_memview()             # <<<<<<<<<<<<<<
 * 
 *     @cname('get_memview')
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_array *)__pyx_v_self->__pyx_vtab)->get_memview(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":222
 * 
 *     @property
 *     def memview(self):             # <<<<<<<<<<<<<<
 *         return self.get_memview()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.array.memview.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":226
 * 
 *     @cname('get_memview')
 *     cdef get_memview(self):             # <<<<<<<<<<<<<<
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 *         return  memoryview(self, flags, self.dtype_is_object)
 */

static PyObject *__pyx_array_get_memview(struct __pyx_array_obj *__pyx_v_self) {
  int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_memview", 0);

  /* "View.MemoryView":227
 *     @cname('get_memview')
 *     cdef get_memview(self):
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE             # <<<<<<<<<<<<<<
 *         return  memoryview(self, flags, self.dtype_is_object)
 * 
 */
  __pyx_v_flags = ((PyBUF_ANY_CONTIGUOUS | PyBUF_FORMAT) | PyBUF_WRITABLE);

  /* "View.MemoryView":228
 *     cdef get_memview(self):
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 *         return  memoryview(self, flags, self.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->dtype_is_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryview_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":226
 * 
 *     @cname('get_memview')
 *     cdef get_memview(self):             # <<<<<<<<<<<<<<
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 *         return  memoryview(self, flags, self.dtype_is_object)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.array.get_memview", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":230
 *         return  memoryview(self, flags, self.dtype_is_object)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self._shape[0]
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_array___len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_array___len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__len__(((struct __pyx_array_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__len__(struct __pyx_array_obj *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "View.MemoryView":231
 * 
 *     def __len__(self):
 *         return self._shape[0]             # <<<<<<<<<<<<<<
 * 
 *     def __getattr__(self, attr):
 */
  __pyx_r = (__pyx_v_self->_shape[0]);
  goto __pyx_L0;

  /* "View.MemoryView":230
 *         return  memoryview(self, flags, self.dtype_is_object)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self._shape[0]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":233
 *         return self._shape[0]
 * 
 *     def __getattr__(self, attr):             # <<<<<<<<<<<<<<
 *         return getattr(self.memview, attr)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_array___getattr__(PyObject *__pyx_v_self, PyObject *__pyx_v_attr); /*proto*/
static PyObject *__pyx_array___getattr__(PyObject *__pyx_v_self, PyObject *__pyx_v_attr) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getattr__ (wrapper)", 0);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getattr__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_attr));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getattr__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_attr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getattr__", 0);

  /* "View.MemoryView":234
 * 
 *     def __getattr__(self, attr):
 *         return getattr(self.memview, attr)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, item):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetAttr(__pyx_t_1, __pyx_v_attr); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":233
 *         return self._shape[0]
 * 
 *     def __getattr__(self, attr):             # <<<<<<<<<<<<<<
 *         return getattr(self.memview, attr)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.array.__getattr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":236
 *         return getattr(self.memview, attr)
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         return self.memview[item]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_array___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/
static PyObject *__pyx_array___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__getitem__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_item));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__getitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "View.MemoryView":237
 * 
 *     def __getitem__(self, item):
 *         return self.memview[item]             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, item, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_item); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":236
 *         return getattr(self.memview, attr)
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         return self.memview[item]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.array.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":239
 *         return self.memview[item]
 * 
 *     def __setitem__(self, item, value):             # <<<<<<<<<<<<<<
 *         self.memview[item] = value
 * 
 */

/* Python wrapper */
static int __pyx_array___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /*proto*/
static int __pyx_array___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_12__setitem__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_item), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_12__setitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "View.MemoryView":240
 * 
 *     def __setitem__(self, item, value):
 *         self.memview[item] = value             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_v_item, __pyx_v_value) < 0)) __PYX_ERR(3, 240, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":239
 *         return self.memview[item]
 * 
 *     def __setitem__(self, item, value):             # <<<<<<<<<<<<<<
 *         self.memview[item] = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.array.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw___pyx_array_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw___pyx_array_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf___pyx_array___reduce_cython__(((struct __pyx_array_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_array___reduce_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(3, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.array.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw___pyx_array_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw___pyx_array_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf___pyx_array_2__setstate_cython__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_array_2__setstate_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(3, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.array.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":244
 * 
 * @cname("__pyx_array_new")
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format,             # <<<<<<<<<<<<<<
 *                           char *mode, char *buf):
 *     cdef array result
 */

static struct __pyx_array_obj *__pyx_array_new(PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, char *__pyx_v_format, char *__pyx_v_mode, char *__pyx_v_buf) {
  struct __pyx_array_obj *__pyx_v_result = 0;
  struct __pyx_array_obj *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("array_cwrapper", 0);

  /* "View.MemoryView":248
 *     cdef array result
 * 
 *     if buf == NULL:             # <<<<<<<<<<<<<<
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_buf == NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":249
 * 
 *     if buf == NULL:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))             # <<<<<<<<<<<<<<
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 */
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_v_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_decode_c_string(__pyx_v_mode, 0, strlen(__pyx_v_mode), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_shape);
    __Pyx_GIVEREF(__pyx_v_shape);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_shape);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_array_type), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_result = ((struct __pyx_array_obj *)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "View.MemoryView":248
 *     cdef array result
 * 
 *     if buf == NULL:             # <<<<<<<<<<<<<<
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":251
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),             # <<<<<<<<<<<<<<
 *                        allocate_buffer=False)
 *         result.data = buf
 */
  /*else*/ {
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_decode_c_string(__pyx_v_mode, 0, strlen(__pyx_v_mode), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_shape);
    __Pyx_GIVEREF(__pyx_v_shape);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_shape);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_3);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_3 = 0;

    /* "View.MemoryView":252
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 *                        allocate_buffer=False)             # <<<<<<<<<<<<<<
 *         result.data = buf
 * 
 */
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_allocate_buffer, Py_False) < 0) __PYX_ERR(3, 252, __pyx_L1_error)

    /* "View.MemoryView":251
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),             # <<<<<<<<<<<<<<
 *                        allocate_buffer=False)
 *         result.data = buf
 */
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_array_type), __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_result = ((struct __pyx_array_obj *)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "View.MemoryView":253
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 *                        allocate_buffer=False)
 *         result.data = buf             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_v_result->data = __pyx_v_buf;
  }
  __pyx_L3:;

  /* "View.MemoryView":255
 *         result.data = buf
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "View.MemoryView":244
 * 
 * @cname("__pyx_array_new")
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format,             # <<<<<<<<<<<<<<
 *                           char *mode, char *buf):
 *     cdef array result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.array_cwrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":281
 * cdef class Enum(object):
 *     cdef object name
 *     def __init__(self, name):             # <<<<<<<<<<<<<<
 *         self.name = name
 *     def __repr__(self):
 */

/* Python wrapper */
static int __pyx_MemviewEnum___init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_MemviewEnum___init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(3, 281, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_name = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(3, 281, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("View.MemoryView.Enum.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self), __pyx_v_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v_name) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "View.MemoryView":282
 *     cdef object name
 *     def __init__(self, name):
 *         self.name = name             # <<<<<<<<<<<<<<
 *     def __repr__(self):
 *         return self.name
 */
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = __pyx_v_name;

  /* "View.MemoryView":281
 * cdef class Enum(object):
 *     cdef object name
 *     def __init__(self, name):             # <<<<<<<<<<<<<<
 *         self.name = name
 *     def __repr__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":283
 *     def __init__(self, name):
 *         self.name = name
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return self.name
 * 
 */

/* Python wrapper */
static PyObject *__pyx_MemviewEnum___repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_MemviewEnum___repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(struct __pyx_MemviewEnum_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "View.MemoryView":284
 *         self.name = name
 *     def __repr__(self):
 *         return self.name             # <<<<<<<<<<<<<<
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* "View.MemoryView":283
 *     def __init__(self, name):
 *         self.name = name
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return self.name
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw___pyx_MemviewEnum_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw___pyx_MemviewEnum_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf___pyx_MemviewEnum___reduce_cython__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_MemviewEnum___reduce_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.name,)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->name);
  __Pyx_GIVEREF(__pyx_v_self->name);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->name);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.name is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.name is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state
 */
  /*else*/ {
    __pyx_t_3 = (__pyx_v_self->name != Py_None);
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":13
 *         use_setstate = self.name is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_Enum); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_184977713);
    __Pyx_GIVEREF(__pyx_int_184977713);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_184977713);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_Enum); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_184977713);
    __Pyx_GIVEREF(__pyx_int_184977713);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_184977713);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.Enum.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw___pyx_MemviewEnum_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw___pyx_MemviewEnum_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf___pyx_MemviewEnum_2__setstate_cython__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_MemviewEnum_2__setstate_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(3, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_unpickle_Enum__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.Enum.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":298
 * 
 * @cname('__pyx_align_pointer')
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:             # <<<<<<<<<<<<<<
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory
 */

static void *__pyx_align_pointer(void *__pyx_v_memory, size_t __pyx_v_alignment) {
  Py_intptr_t __pyx_v_aligned_p;
  size_t __pyx_v_offset;
  void *__pyx_r;
  int __pyx_t_1;

  /* "View.MemoryView":300
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory             # <<<<<<<<<<<<<<
 *     cdef size_t offset
 * 
 */
  __pyx_v_aligned_p = ((Py_intptr_t)__pyx_v_memory);

  /* "View.MemoryView":304
 * 
 *     with cython.cdivision(True):
 *         offset = aligned_p % alignment             # <<<<<<<<<<<<<<
 * 
 *     if offset > 0:
 */
  __pyx_v_offset = (__pyx_v_aligned_p % __pyx_v_alignment);

  /* "View.MemoryView":306
 *         offset = aligned_p % alignment
 * 
 *     if offset > 0:             # <<<<<<<<<<<<<<
 *         aligned_p += alignment - offset
 * 
 */
  __pyx_t_1 = ((__pyx_v_offset > 0) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":307
 * 
 *     if offset > 0:
 *         aligned_p += alignment - offset             # <<<<<<<<<<<<<<
 * 
 *     return <void *> aligned_p
 */
    __pyx_v_aligned_p = (__pyx_v_aligned_p + (__pyx_v_alignment - __pyx_v_offset));

    /* "View.MemoryView":306
 *         offset = aligned_p % alignment
 * 
 *     if offset > 0:             # <<<<<<<<<<<<<<
 *         aligned_p += alignment - offset
 * 
 */
  }

  /* "View.MemoryView":309
 *         aligned_p += alignment - offset
 * 
 *     return <void *> aligned_p             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((void *)__pyx_v_aligned_p);
  goto __pyx_L0;

  /* "View.MemoryView":298
 * 
 * @cname('__pyx_align_pointer')
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:             # <<<<<<<<<<<<<<
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":345
 *     cdef __Pyx_TypeInfo *typeinfo
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):             # <<<<<<<<<<<<<<
 *         self.obj = obj
 *         self.flags = flags
 */

/* Python wrapper */
static int __pyx_memoryview___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_memoryview___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_obj = 0;
  int __pyx_v_flags;
  int __pyx_v_dtype_is_object;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_obj,&__pyx_n_s_flags,&__pyx_n_s_dtype_is_object,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_obj)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, 1); __PYX_ERR(3, 345, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dtype_is_object);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(3, 345, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_obj = values[0];
    __pyx_v_flags = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_flags == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 345, __pyx_L3_error)
    if (values[2]) {
      __pyx_v_dtype_is_object = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_dtype_is_object == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 345, __pyx_L3_error)
    } else {
      __pyx_v_dtype_is_object = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(3, 345, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("View.MemoryView.memoryview.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_obj, __pyx_v_flags, __pyx_v_dtype_is_object);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_v_flags, int __pyx_v_dtype_is_object) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "View.MemoryView":346
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):
 *         self.obj = obj             # <<<<<<<<<<<<<<
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:
 */
  __Pyx_INCREF(__pyx_v_obj);
  __Pyx_GIVEREF(__pyx_v_obj);
  __Pyx_GOTREF(__pyx_v_self->obj);
  __Pyx_DECREF(__pyx_v_self->obj);
  __pyx_v_self->obj = __pyx_v_obj;

  /* "View.MemoryView":347
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):
 *         self.obj = obj
 *         self.flags = flags             # <<<<<<<<<<<<<<
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 */
  __pyx_v_self->flags = __pyx_v_flags;

  /* "View.MemoryView":348
 *         self.obj = obj
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 */
  __pyx_t_2 = (((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))) == ((PyObject *)__pyx_memoryview_type));
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_obj != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "View.MemoryView":349
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)             # <<<<<<<<<<<<<<
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 */
    __pyx_t_4 = __Pyx_GetBuffer(__pyx_v_obj, (&__pyx_v_self->view), __pyx_v_flags); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(3, 349, __pyx_L1_error)

    /* "View.MemoryView":350
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:             # <<<<<<<<<<<<<<
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)
 */
    __pyx_t_1 = ((((PyObject *)__pyx_v_self->view.obj) == NULL) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":351
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None             # <<<<<<<<<<<<<<
 *                 Py_INCREF(Py_None)
 * 
 */
      ((Py_buffer *)(&__pyx_v_self->view))->obj = Py_None;

      /* "View.MemoryView":352
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *         global __pyx_memoryview_thread_locks_used
 */
      Py_INCREF(Py_None);

      /* "View.MemoryView":350
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:             # <<<<<<<<<<<<<<
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)
 */
    }

    /* "View.MemoryView":348
 *         self.obj = obj
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 */
  }

  /* "View.MemoryView":355
 * 
 *         global __pyx_memoryview_thread_locks_used
 *         if __pyx_memoryview_thread_locks_used < THREAD_LOCKS_PREALLOCATED:             # <<<<<<<<<<<<<<
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1
 */
  __pyx_t_1 = ((__pyx_memoryview_thread_locks_used < 8) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":356
 *         global __pyx_memoryview_thread_locks_used
 *         if __pyx_memoryview_thread_locks_used < THREAD_LOCKS_PREALLOCATED:
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]             # <<<<<<<<<<<<<<
 *             __pyx_memoryview_thread_locks_used += 1
 *         if self.lock is NULL:
 */
    __pyx_v_self->lock = (__pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]);

    /* "View.MemoryView":357
 *         if __pyx_memoryview_thread_locks_used < THREAD_LOCKS_PREALLOCATED:
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1             # <<<<<<<<<<<<<<
 *         if self.lock is NULL:
 *             self.lock = PyThread_allocate_lock()
 */
    __pyx_memoryview_thread_locks_used = (__pyx_memoryview_thread_locks_used + 1);

    /* "View.MemoryView":355
 * 
 *         global __pyx_memoryview_thread_locks_used
 *         if __pyx_memoryview_thread_locks_used < THREAD_LOCKS_PREALLOCATED:             # <<<<<<<<<<<<<<
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1
 */
  }

  /* "View.MemoryView":358
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1
 *         if self.lock is NULL:             # <<<<<<<<<<<<<<
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:
 */
  __pyx_t_1 = ((__pyx_v_self->lock == NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":359
 *             __pyx_memoryview_thread_locks_used += 1
 *         if self.lock is NULL:
 *             self.lock = PyThread_allocate_lock()             # <<<<<<<<<<<<<<
 *             if self.lock is NULL:
 *                 raise MemoryError
 */
    __pyx_v_self->lock = PyThread_allocate_lock();

    /* "View.MemoryView":360
 *         if self.lock is NULL:
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 * 
 */
    __pyx_t_1 = ((__pyx_v_self->lock == NULL) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "View.MemoryView":361
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:
 *                 raise MemoryError             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */
      PyErr_NoMemory(); __PYX_ERR(3, 361, __pyx_L1_error)

      /* "View.MemoryView":360
 *         if self.lock is NULL:
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 * 
 */
    }

    /* "View.MemoryView":358
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1
 *         if self.lock is NULL:             # <<<<<<<<<<<<<<
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:
 */
  }

  /* "View.MemoryView":363
 *                 raise MemoryError
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":364
 * 
 *         if flags & PyBUF_FORMAT:
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')             # <<<<<<<<<<<<<<
 *         else:
 *             self.dtype_is_object = dtype_is_object
 */
    __pyx_t_2 = (((__pyx_v_self->view.format[0]) == 'O') != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L11_bool_binop_done;
    }
    __pyx_t_2 = (((__pyx_v_self->view.format[1]) == '\x00') != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L11_bool_binop_done:;
    __pyx_v_self->dtype_is_object = __pyx_t_1;

    /* "View.MemoryView":363
 *                 raise MemoryError
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')
 *         else:
 */
    goto __pyx_L10;
  }

  /* "View.MemoryView":366
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')
 *         else:
 *             self.dtype_is_object = dtype_is_object             # <<<<<<<<<<<<<<
 * 
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(
 */
  /*else*/ {
    __pyx_v_self->dtype_is_object = __pyx_v_dtype_is_object;
  }
  __pyx_L10:;

  /* "View.MemoryView":368
 *             self.dtype_is_object = dtype_is_object
 * 
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(             # <<<<<<<<<<<<<<
 *                   <void *> &self.acquisition_count[0], sizeof(__pyx_atomic_int))
 *         self.typeinfo = NULL
 */
  __pyx_v_self->acquisition_count_aligned_p = ((__pyx_atomic_int *)__pyx_align_pointer(((void *)(&(__pyx_v_self->acquisition_count[0]))), (sizeof(__pyx_atomic_int))));

  /* "View.MemoryView":370
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(
 *                   <void *> &self.acquisition_count[0], sizeof(__pyx_atomic_int))
 *         self.typeinfo = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(memoryview self):
 */
  __pyx_v_self->typeinfo = NULL;

  /* "View.MemoryView":345
 *     cdef __Pyx_TypeInfo *typeinfo
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):             # <<<<<<<<<<<<<<
 *         self.obj = obj
 *         self.flags = flags
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView.memoryview.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":372
 *         self.typeinfo = NULL
 * 
 *     def __dealloc__(memoryview self):             # <<<<<<<<<<<<<<
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 */

/* Python wrapper */
static void __pyx_memoryview___dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_memoryview___dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(struct __pyx_memoryview_obj *__pyx_v_self) {
  int __pyx_v_i;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyThread_type_lock __pyx_t_6;
  PyThread_type_lock __pyx_t_7;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":373
 * 
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_ReleaseBuffer(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 */
  __pyx_t_1 = (__pyx_v_self->obj != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":374
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)             # <<<<<<<<<<<<<<
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 * 
 */
    __Pyx_ReleaseBuffer((&__pyx_v_self->view));

    /* "View.MemoryView":373
 * 
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_ReleaseBuffer(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":375
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:             # <<<<<<<<<<<<<<
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL
 */
  __pyx_t_2 = ((((Py_buffer *)(&__pyx_v_self->view))->obj == Py_None) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":377
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL             # <<<<<<<<<<<<<<
 *             Py_DECREF(Py_None)
 * 
 */
    ((Py_buffer *)(&__pyx_v_self->view))->obj = NULL;

    /* "View.MemoryView":378
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL
 *             Py_DECREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *         cdef int i
 */
    Py_DECREF(Py_None);

    /* "View.MemoryView":375
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:             # <<<<<<<<<<<<<<
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL
 */
  }
  __pyx_L3:;

  /* "View.MemoryView":382
 *         cdef int i
 *         global __pyx_memoryview_thread_locks_used
 *         if self.lock != NULL:             # <<<<<<<<<<<<<<
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 */
  __pyx_t_2 = ((__pyx_v_self->lock != NULL) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":383
 *         global __pyx_memoryview_thread_locks_used
 *         if self.lock != NULL:
 *             for i in range(__pyx_memoryview_thread_locks_used):             # <<<<<<<<<<<<<<
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1
 */
    __pyx_t_3 = __pyx_memoryview_thread_locks_used;
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "View.MemoryView":384
 *         if self.lock != NULL:
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:             # <<<<<<<<<<<<<<
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:
 */
      __pyx_t_2 = (((__pyx_memoryview_thread_locks[__pyx_v_i]) == __pyx_v_self->lock) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":385
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1             # <<<<<<<<<<<<<<
 *                     if i != __pyx_memoryview_thread_locks_used:
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 */
        __pyx_memoryview_thread_locks_used = (__pyx_memoryview_thread_locks_used - 1);

        /* "View.MemoryView":386
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:             # <<<<<<<<<<<<<<
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 */
        __pyx_t_2 = ((__pyx_v_i != __pyx_memoryview_thread_locks_used) != 0);
        if (__pyx_t_2) {

          /* "View.MemoryView":388
 *                     if i != __pyx_memoryview_thread_locks_used:
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])             # <<<<<<<<<<<<<<
 *                     break
 *             else:
 */
          __pyx_t_6 = (__pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]);
          __pyx_t_7 = (__pyx_memoryview_thread_locks[__pyx_v_i]);

          /* "View.MemoryView":387
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (             # <<<<<<<<<<<<<<
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 *                     break
 */
          (__pyx_memoryview_thread_locks[__pyx_v_i]) = __pyx_t_6;
          (__pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]) = __pyx_t_7;

          /* "View.MemoryView":386
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:             # <<<<<<<<<<<<<<
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 */
        }

        /* "View.MemoryView":389
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 *                     break             # <<<<<<<<<<<<<<
 *             else:
 *                 PyThread_free_lock(self.lock)
 */
        goto __pyx_L6_break;

        /* "View.MemoryView":384
 *         if self.lock != NULL:
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:             # <<<<<<<<<<<<<<
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:
 */
      }
    }
    /*else*/ {

      /* "View.MemoryView":391
 *                     break
 *             else:
 *                 PyThread_free_lock(self.lock)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:
 */
      PyThread_free_lock(__pyx_v_self->lock);
    }
    __pyx_L6_break:;

    /* "View.MemoryView":382
 *         cdef int i
 *         global __pyx_memoryview_thread_locks_used
 *         if self.lock != NULL:             # <<<<<<<<<<<<<<
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 */
  }

  /* "View.MemoryView":372
 *         self.typeinfo = NULL
 * 
 *     def __dealloc__(memoryview self):             # <<<<<<<<<<<<<<
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":393
 *                 PyThread_free_lock(self.lock)
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf
 */

static char *__pyx_memoryview_get_item_pointer(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index) {
  Py_ssize_t __pyx_v_dim;
  char *__pyx_v_itemp;
  PyObject *__pyx_v_idx = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_item_pointer", 0);

  /* "View.MemoryView":395
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf             # <<<<<<<<<<<<<<
 * 
 *         for dim, idx in enumerate(index):
 */
  __pyx_v_itemp = ((char *)__pyx_v_self->view.buf);

  /* "View.MemoryView":397
 *         cdef char *itemp = <char *> self.view.buf
 * 
 *         for dim, idx in enumerate(index):             # <<<<<<<<<<<<<<
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 */
  __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_v_index)) || PyTuple_CheckExact(__pyx_v_index)) {
    __pyx_t_2 = __pyx_v_index; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 397, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 397, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(3, 397, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 397, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(3, 397, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 397, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(3, 397, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_v_dim = __pyx_t_1;
    __pyx_t_1 = (__pyx_t_1 + 1);

    /* "View.MemoryView":398
 * 
 *         for dim, idx in enumerate(index):
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)             # <<<<<<<<<<<<<<
 * 
 *         return itemp
 */
    __pyx_t_6 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_6 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 398, __pyx_L1_error)
    __pyx_t_7 = __pyx_pybuffer_index((&__pyx_v_self->view), __pyx_v_itemp, __pyx_t_6, __pyx_v_dim); if (unlikely(__pyx_t_7 == ((char *)NULL))) __PYX_ERR(3, 398, __pyx_L1_error)
    __pyx_v_itemp = __pyx_t_7;

    /* "View.MemoryView":397
 *         cdef char *itemp = <char *> self.view.buf
 * 
 *         for dim, idx in enumerate(index):             # <<<<<<<<<<<<<<
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":400
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 *         return itemp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_itemp;
  goto __pyx_L0;

  /* "View.MemoryView":393
 *                 PyThread_free_lock(self.lock)
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.get_item_pointer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_idx);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":403
 * 
 * 
 *     def __getitem__(memoryview self, object index):             # <<<<<<<<<<<<<<
 *         if index is Ellipsis:
 *             return self
 */

/* Python wrapper */
static PyObject *__pyx_memoryview___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index); /*proto*/
static PyObject *__pyx_memoryview___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v_index));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_v_have_slices = NULL;
  PyObject *__pyx_v_indices = NULL;
  char *__pyx_v_itemp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "View.MemoryView":404
 * 
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:             # <<<<<<<<<<<<<<
 *             return self
 * 
 */
  __pyx_t_1 = (__pyx_v_index == __pyx_builtin_Ellipsis);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":405
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:
 *             return self             # <<<<<<<<<<<<<<
 * 
 *         have_slices, indices = _unellipsify(index, self.view.ndim)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __pyx_r = ((PyObject *)__pyx_v_self);
    goto __pyx_L0;

    /* "View.MemoryView":404
 * 
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:             # <<<<<<<<<<<<<<
 *             return self
 * 
 */
  }

  /* "View.MemoryView":407
 *             return self
 * 
 *         have_slices, indices = _unellipsify(index, self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *         cdef char *itemp
 */
  __pyx_t_3 = _unellipsify(__pyx_v_index, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (likely(__pyx_t_3 != Py_None)) {
    PyObject* sequence = __pyx_t_3;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(3, 407, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
    __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    #else
    __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 407, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 407, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    #endif
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(3, 407, __pyx_L1_error)
  }
  __pyx_v_have_slices = __pyx_t_4;
  __pyx_t_4 = 0;
  __pyx_v_indices = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "View.MemoryView":410
 * 
 *         cdef char *itemp
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             return memview_slice(self, indices)
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_have_slices); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(3, 410, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "View.MemoryView":411
 *         cdef char *itemp
 *         if have_slices:
 *             return memview_slice(self, indices)             # <<<<<<<<<<<<<<
 *         else:
 *             itemp = self.get_item_pointer(indices)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = ((PyObject *)__pyx_memview_slice(__pyx_v_self, __pyx_v_indices)); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "View.MemoryView":410
 * 
 *         cdef char *itemp
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             return memview_slice(self, indices)
 *         else:
 */
  }

  /* "View.MemoryView":413
 *             return memview_slice(self, indices)
 *         else:
 *             itemp = self.get_item_pointer(indices)             # <<<<<<<<<<<<<<
 *             return self.convert_item_to_object(itemp)
 * 
 */
  /*else*/ {
    __pyx_t_6 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->get_item_pointer(__pyx_v_self, __pyx_v_indices); if (unlikely(__pyx_t_6 == ((char *)NULL))) __PYX_ERR(3, 413, __pyx_L1_error)
    __pyx_v_itemp = __pyx_t_6;

    /* "View.MemoryView":414
 *         else:
 *             itemp = self.get_item_pointer(indices)
 *             return self.convert_item_to_object(itemp)             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(memoryview self, object index, object value):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->convert_item_to_object(__pyx_v_self, __pyx_v_itemp); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 414, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":403
 * 
 * 
 *     def __getitem__(memoryview self, object index):             # <<<<<<<<<<<<<<
 *         if index is Ellipsis:
 *             return self
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_have_slices);
  __Pyx_XDECREF(__pyx_v_indices);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":416
 *             return self.convert_item_to_object(itemp)
 * 
 *     def __setitem__(memoryview self, object index, object value):             # <<<<<<<<<<<<<<
 *         if self.view.readonly:
 *             raise TypeError("Cannot assign to read-only memoryview")
 */

/* Python wrapper */
static int __pyx_memoryview___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /*proto*/
static int __pyx_memoryview___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v_index), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_have_slices = NULL;
  PyObject *__pyx_v_obj = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);
  __Pyx_INCREF(__pyx_v_index);

  /* "View.MemoryView":417
 * 
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise TypeError("Cannot assign to read-only memoryview")
 * 
 */
  __pyx_t_1 = (__pyx_v_self->view.readonly != 0);
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":418
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:
 *             raise TypeError("Cannot assign to read-only memoryview")             # <<<<<<<<<<<<<<
 * 
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 418, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(3, 418, __pyx_L1_error)

    /* "View.MemoryView":417
 * 
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise TypeError("Cannot assign to read-only memoryview")
 * 
 */
  }

  /* "View.MemoryView":420
 *             raise TypeError("Cannot assign to read-only memoryview")
 * 
 *         have_slices, index = _unellipsify(index, self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *         if have_slices:
 */
  __pyx_t_2 = _unellipsify(__pyx_v_index, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 420, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (likely(__pyx_t_2 != Py_None)) {
    PyObject* sequence = __pyx_t_2;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(3, 420, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    #else
    __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 420, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 420, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(3, 420, __pyx_L1_error)
  }
  __pyx_v_have_slices = __pyx_t_3;
  __pyx_t_3 = 0;
  __Pyx_DECREF_SET(__pyx_v_index, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "View.MemoryView":422
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             obj = self.is_slice(value)
 *             if obj:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_have_slices); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(3, 422, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "View.MemoryView":423
 * 
 *         if have_slices:
 *             obj = self.is_slice(value)             # <<<<<<<<<<<<<<
 *             if obj:
 *                 self.setitem_slice_assignment(self[index], obj)
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->is_slice(__pyx_v_self, __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 423, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_obj = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "View.MemoryView":424
 *         if have_slices:
 *             obj = self.is_slice(value)
 *             if obj:             # <<<<<<<<<<<<<<
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 */
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_obj); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(3, 424, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "View.MemoryView":425
 *             obj = self.is_slice(value)
 *             if obj:
 *                 self.setitem_slice_assignment(self[index], obj)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.setitem_slice_assign_scalar(self[index], value)
 */
      __pyx_t_2 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 425, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_slice_assignment(__pyx_v_self, __pyx_t_2, __pyx_v_obj); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 425, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "View.MemoryView":424
 *         if have_slices:
 *             obj = self.is_slice(value)
 *             if obj:             # <<<<<<<<<<<<<<
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 */
      goto __pyx_L5;
    }

    /* "View.MemoryView":427
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 *                 self.setitem_slice_assign_scalar(self[index], value)             # <<<<<<<<<<<<<<
 *         else:
 *             self.setitem_indexed(index, value)
 */
    /*else*/ {
      __pyx_t_4 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_memoryview_type))))) __PYX_ERR(3, 427, __pyx_L1_error)
      __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_slice_assign_scalar(__pyx_v_self, ((struct __pyx_memoryview_obj *)__pyx_t_4), __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L5:;

    /* "View.MemoryView":422
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             obj = self.is_slice(value)
 *             if obj:
 */
    goto __pyx_L4;
  }

  /* "View.MemoryView":429
 *                 self.setitem_slice_assign_scalar(self[index], value)
 *         else:
 *             self.setitem_indexed(index, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef is_slice(self, obj):
 */
  /*else*/ {
    __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_indexed(__pyx_v_self, __pyx_v_index, __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 429, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_L4:;

  /* "View.MemoryView":416
 *             return self.convert_item_to_object(itemp)
 * 
 *     def __setitem__(memoryview self, object index, object value):             # <<<<<<<<<<<<<<
 *         if self.view.readonly:
 *             raise TypeError("Cannot assign to read-only memoryview")
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_have_slices);
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":431
 *             self.setitem_indexed(index, value)
 * 
 *     cdef is_slice(self, obj):             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, memoryview):
 *             try:
 */

static PyObject *__pyx_memoryview_is_slice(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_slice", 0);
  __Pyx_INCREF(__pyx_v_obj);

  /* "View.MemoryView":432
 * 
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):             # <<<<<<<<<<<<<<
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_obj, __pyx_memoryview_type); 
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":433
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "View.MemoryView":434
 *         if not isinstance(obj, memoryview):
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,             # <<<<<<<<<<<<<<
 *                                  self.dtype_is_object)
 *             except TypeError:
 */
        __pyx_t_6 = __Pyx_PyInt_From_int(((__pyx_v_self->flags & (~PyBUF_WRITABLE)) | PyBUF_ANY_CONTIGUOUS)); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 434, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_6);

        /* "View.MemoryView":435
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)             # <<<<<<<<<<<<<<
 *             except TypeError:
 *                 return None
 */
        __pyx_t_7 = __Pyx_PyBool_FromLong(__pyx_v_self->dtype_is_object); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 435, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_7);

        /* "View.MemoryView":434
 *         if not isinstance(obj, memoryview):
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,             # <<<<<<<<<<<<<<
 *                                  self.dtype_is_object)
 *             except TypeError:
 */
        __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 434, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_INCREF(__pyx_v_obj);
        __Pyx_GIVEREF(__pyx_v_obj);
        PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v_obj);
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_7);
        PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);
        __pyx_t_6 = 0;
        __pyx_t_7 = 0;
        __pyx_t_7 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryview_type), __pyx_t_8, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 434, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF_SET(__pyx_v_obj, __pyx_t_7);
        __pyx_t_7 = 0;

        /* "View.MemoryView":433
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */
      }
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L9_try_end;
      __pyx_L4_error:;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "View.MemoryView":436
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 *             except TypeError:             # <<<<<<<<<<<<<<
 *                 return None
 * 
 */
      __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
      if (__pyx_t_9) {
        __Pyx_AddTraceback("View.MemoryView.memoryview.is_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_6) < 0) __PYX_ERR(3, 436, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GOTREF(__pyx_t_6);

        /* "View.MemoryView":437
 *                                  self.dtype_is_object)
 *             except TypeError:
 *                 return None             # <<<<<<<<<<<<<<
 * 
 *         return obj
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_r = Py_None; __Pyx_INCREF(Py_None);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L7_except_return;
      }
      goto __pyx_L6_except_error;
      __pyx_L6_except_error:;

      /* "View.MemoryView":433
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L7_except_return:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L0;
      __pyx_L9_try_end:;
    }

    /* "View.MemoryView":432
 * 
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):             # <<<<<<<<<<<<<<
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 */
  }

  /* "View.MemoryView":439
 *                 return None
 * 
 *         return obj             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_slice_assignment(self, dst, src):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_obj);
  __pyx_r = __pyx_v_obj;
  goto __pyx_L0;

  /* "View.MemoryView":431
 *             self.setitem_indexed(index, value)
 * 
 *     cdef is_slice(self, obj):             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, memoryview):
 *             try:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":441
 *         return obj
 * 
 *     cdef setitem_slice_assignment(self, dst, src):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
 */

static PyObject *__pyx_memoryview_setitem_slice_assignment(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_dst, PyObject *__pyx_v_src) {
  __Pyx_memviewslice __pyx_v_dst_slice;
  __Pyx_memviewslice __pyx_v_src_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice *__pyx_t_1;
  __Pyx_memviewslice *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setitem_slice_assignment", 0);

  /* "View.MemoryView":445
 *         cdef __Pyx_memviewslice src_slice
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],             # <<<<<<<<<<<<<<
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 */
  if (!(likely(((__pyx_v_src) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_src, __pyx_memoryview_type))))) __PYX_ERR(3, 445, __pyx_L1_error)
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(((struct __pyx_memoryview_obj *)__pyx_v_src), (&__pyx_v_src_slice)); if (unlikely(__pyx_t_1 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(3, 445, __pyx_L1_error)

  /* "View.MemoryView":446
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],
 *                                  get_slice_from_memview(dst, &dst_slice)[0],             # <<<<<<<<<<<<<<
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 */
  if (!(likely(((__pyx_v_dst) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dst, __pyx_memoryview_type))))) __PYX_ERR(3, 446, __pyx_L1_error)
  __pyx_t_2 = __pyx_memoryview_get_slice_from_memoryview(((struct __pyx_memoryview_obj *)__pyx_v_dst), (&__pyx_v_dst_slice)); if (unlikely(__pyx_t_2 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(3, 446, __pyx_L1_error)

  /* "View.MemoryView":447
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_src, __pyx_n_s_ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 447, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dst, __pyx_n_s_ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 447, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":445
 *         cdef __Pyx_memviewslice src_slice
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],             # <<<<<<<<<<<<<<
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 */
  __pyx_t_6 = __pyx_memoryview_copy_contents((__pyx_t_1[0]), (__pyx_t_2[0]), __pyx_t_4, __pyx_t_5, __pyx_v_self->dtype_is_object); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(3, 445, __pyx_L1_error)

  /* "View.MemoryView":441
 *         return obj
 * 
 *     cdef setitem_slice_assignment(self, dst, src):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_slice_assignment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":449
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):             # <<<<<<<<<<<<<<
 *         cdef int array[128]
 *         cdef void *tmp = NULL
 */

static PyObject *__pyx_memoryview_setitem_slice_assign_scalar(struct __pyx_memoryview_obj *__pyx_v_self, struct __pyx_memoryview_obj *__pyx_v_dst, PyObject *__pyx_v_value) {
  int __pyx_v_array[0x80];
  void *__pyx_v_tmp;
  void *__pyx_v_item;
  __Pyx_memviewslice *__pyx_v_dst_slice;
  __Pyx_memviewslice __pyx_v_tmp_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice *__pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  char const *__pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setitem_slice_assign_scalar", 0);

  /* "View.MemoryView":451
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):
 *         cdef int array[128]
 *         cdef void *tmp = NULL             # <<<<<<<<<<<<<<
 *         cdef void *item
 * 
 */
  __pyx_v_tmp = NULL;

  /* "View.MemoryView":456
 *         cdef __Pyx_memviewslice *dst_slice
 *         cdef __Pyx_memviewslice tmp_slice
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)             # <<<<<<<<<<<<<<
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):
 */
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_dst, (&__pyx_v_tmp_slice)); if (unlikely(__pyx_t_1 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(3, 456, __pyx_L1_error)
  __pyx_v_dst_slice = __pyx_t_1;

  /* "View.MemoryView":458
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):             # <<<<<<<<<<<<<<
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 */
  __pyx_t_2 = ((((size_t)__pyx_v_self->view.itemsize) > (sizeof(__pyx_v_array))) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":459
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)             # <<<<<<<<<<<<<<
 *             if tmp == NULL:
 *                 raise MemoryError
 */
    __pyx_v_tmp = PyMem_Malloc(__pyx_v_self->view.itemsize);

    /* "View.MemoryView":460
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             item = tmp
 */
    __pyx_t_2 = ((__pyx_v_tmp == NULL) != 0);
    if (unlikely(__pyx_t_2)) {

      /* "View.MemoryView":461
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 *                 raise MemoryError             # <<<<<<<<<<<<<<
 *             item = tmp
 *         else:
 */
      PyErr_NoMemory(); __PYX_ERR(3, 461, __pyx_L1_error)

      /* "View.MemoryView":460
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             item = tmp
 */
    }

    /* "View.MemoryView":462
 *             if tmp == NULL:
 *                 raise MemoryError
 *             item = tmp             # <<<<<<<<<<<<<<
 *         else:
 *             item = <void *> array
 */
    __pyx_v_item = __pyx_v_tmp;

    /* "View.MemoryView":458
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):             # <<<<<<<<<<<<<<
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":464
 *             item = tmp
 *         else:
 *             item = <void *> array             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  /*else*/ {
    __pyx_v_item = ((void *)__pyx_v_array);
  }
  __pyx_L3:;

  /* "View.MemoryView":466
 *             item = <void *> array
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 (<PyObject **> item)[0] = <PyObject *> value
 */
  /*try:*/ {

    /* "View.MemoryView":467
 * 
 *         try:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 */
    __pyx_t_2 = (__pyx_v_self->dtype_is_object != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":468
 *         try:
 *             if self.dtype_is_object:
 *                 (<PyObject **> item)[0] = <PyObject *> value             # <<<<<<<<<<<<<<
 *             else:
 *                 self.assign_item_from_object(<char *> item, value)
 */
      (((PyObject **)__pyx_v_item)[0]) = ((PyObject *)__pyx_v_value);

      /* "View.MemoryView":467
 * 
 *         try:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 */
      goto __pyx_L8;
    }

    /* "View.MemoryView":470
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 *                 self.assign_item_from_object(<char *> item, value)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    /*else*/ {
      __pyx_t_3 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->assign_item_from_object(__pyx_v_self, ((char *)__pyx_v_item), __pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 470, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_L8:;

    /* "View.MemoryView":474
 * 
 * 
 *             if self.view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 */
    __pyx_t_2 = ((__pyx_v_self->view.suboffsets != NULL) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":475
 * 
 *             if self.view.suboffsets != NULL:
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)             # <<<<<<<<<<<<<<
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 *                                 item, self.dtype_is_object)
 */
      __pyx_t_3 = assert_direct_dimensions(__pyx_v_self->view.suboffsets, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 475, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "View.MemoryView":474
 * 
 * 
 *             if self.view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 */
    }

    /* "View.MemoryView":476
 *             if self.view.suboffsets != NULL:
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,             # <<<<<<<<<<<<<<
 *                                 item, self.dtype_is_object)
 *         finally:
 */
    __pyx_memoryview_slice_assign_scalar(__pyx_v_dst_slice, __pyx_v_dst->view.ndim, __pyx_v_self->view.itemsize, __pyx_v_item, __pyx_v_self->dtype_is_object);
  }

  /* "View.MemoryView":479
 *                                 item, self.dtype_is_object)
 *         finally:
 *             PyMem_Free(tmp)             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_indexed(self, index, value):
 */
  /*finally:*/ {
    /*normal exit:*/{
      PyMem_Free(__pyx_v_tmp);
      goto __pyx_L7;
    }
    __pyx_L6_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9) < 0)) __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __pyx_t_4 = __pyx_lineno; __pyx_t_5 = __pyx_clineno; __pyx_t_6 = __pyx_filename;
      {
        PyMem_Free(__pyx_v_tmp);
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
      }
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_ErrRestore(__pyx_t_7, __pyx_t_8, __pyx_t_9);
      __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;
      __pyx_lineno = __pyx_t_4; __pyx_clineno = __pyx_t_5; __pyx_filename = __pyx_t_6;
      goto __pyx_L1_error;
    }
    __pyx_L7:;
  }

  /* "View.MemoryView":449
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):             # <<<<<<<<<<<<<<
 *         cdef int array[128]
 *         cdef void *tmp = NULL
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_slice_assign_scalar", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":481
 *             PyMem_Free(tmp)
 * 
 *     cdef setitem_indexed(self, index, value):             # <<<<<<<<<<<<<<
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)
 */

static PyObject *__pyx_memoryview_setitem_indexed(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  char *__pyx_v_itemp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setitem_indexed", 0);

  /* "View.MemoryView":482
 * 
 *     cdef setitem_indexed(self, index, value):
 *         cdef char *itemp = self.get_item_pointer(index)             # <<<<<<<<<<<<<<
 *         self.assign_item_from_object(itemp, value)
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->get_item_pointer(__pyx_v_self, __pyx_v_index); if (unlikely(__pyx_t_1 == ((char *)NULL))) __PYX_ERR(3, 482, __pyx_L1_error)
  __pyx_v_itemp = __pyx_t_1;

  /* "View.MemoryView":483
 *     cdef setitem_indexed(self, index, value):
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->assign_item_from_object(__pyx_v_self, __pyx_v_itemp, __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":481
 *             PyMem_Free(tmp)
 * 
 *     cdef setitem_indexed(self, index, value):             # <<<<<<<<<<<<<<
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_indexed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":485
 *         self.assign_item_from_object(itemp, value)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

static PyObject *__pyx_memoryview_convert_item_to_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_bytesitem = 0;
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("convert_item_to_object", 0);

  /* "View.MemoryView":488
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 *         import struct             # <<<<<<<<<<<<<<
 *         cdef bytes bytesitem
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 488, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "View.MemoryView":491
 *         cdef bytes bytesitem
 * 
 *         bytesitem = itemp[:self.view.itemsize]             # <<<<<<<<<<<<<<
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)
 */
  __pyx_t_1 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_itemp + 0, __pyx_v_self->view.itemsize - 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_bytesitem = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":492
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "View.MemoryView":493
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)             # <<<<<<<<<<<<<<
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_unpack); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 493, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 493, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_8 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_t_6, __pyx_v_bytesitem};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 493, __pyx_L3_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_t_6, __pyx_v_bytesitem};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 493, __pyx_L3_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 493, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_7) {
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_6);
        __Pyx_INCREF(__pyx_v_bytesitem);
        __Pyx_GIVEREF(__pyx_v_bytesitem);
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_v_bytesitem);
        __pyx_t_6 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 493, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_v_result = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "View.MemoryView":492
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */
    }

    /* "View.MemoryView":497
 *             raise ValueError("Unable to convert item to object")
 *         else:
 *             if len(self.view.format) == 1:             # <<<<<<<<<<<<<<
 *                 return result[0]
 *             return result
 */
    /*else:*/ {
      __pyx_t_10 = strlen(__pyx_v_self->view.format); 
      __pyx_t_11 = ((__pyx_t_10 == 1) != 0);
      if (__pyx_t_11) {

        /* "View.MemoryView":498
 *         else:
 *             if len(self.view.format) == 1:
 *                 return result[0]             # <<<<<<<<<<<<<<
 *             return result
 * 
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_result, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 498, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_r = __pyx_t_1;
        __pyx_t_1 = 0;
        goto __pyx_L6_except_return;

        /* "View.MemoryView":497
 *             raise ValueError("Unable to convert item to object")
 *         else:
 *             if len(self.view.format) == 1:             # <<<<<<<<<<<<<<
 *                 return result[0]
 *             return result
 */
      }

      /* "View.MemoryView":499
 *             if len(self.view.format) == 1:
 *                 return result[0]
 *             return result             # <<<<<<<<<<<<<<
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_result);
      __pyx_r = __pyx_v_result;
      goto __pyx_L6_except_return;
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "View.MemoryView":494
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:             # <<<<<<<<<<<<<<
 *             raise ValueError("Unable to convert item to object")
 *         else:
 */
    __Pyx_ErrFetch(&__pyx_t_1, &__pyx_t_5, &__pyx_t_9);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_error); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 494, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_1, __pyx_t_6);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_ErrRestore(__pyx_t_1, __pyx_t_5, __pyx_t_9);
    __pyx_t_1 = 0; __pyx_t_5 = 0; __pyx_t_9 = 0;
    if (__pyx_t_8) {
      __Pyx_AddTraceback("View.MemoryView.memoryview.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_9, &__pyx_t_5, &__pyx_t_1) < 0) __PYX_ERR(3, 494, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_1);

      /* "View.MemoryView":495
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")             # <<<<<<<<<<<<<<
 *         else:
 *             if len(self.view.format) == 1:
 */
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 495, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __PYX_ERR(3, 495, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "View.MemoryView":492
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L0;
  }

  /* "View.MemoryView":485
 *         self.assign_item_from_object(itemp, value)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("View.MemoryView.memoryview.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_bytesitem);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":501
 *             return result
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

static PyObject *__pyx_memoryview_assign_item_from_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_struct = NULL;
  char __pyx_v_c;
  PyObject *__pyx_v_bytesvalue = 0;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  char *__pyx_t_11;
  char *__pyx_t_12;
  char *__pyx_t_13;
  char *__pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("assign_item_from_object", 0);

  /* "View.MemoryView":504
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 *         import struct             # <<<<<<<<<<<<<<
 *         cdef char c
 *         cdef bytes bytesvalue
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 504, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "View.MemoryView":509
 *         cdef Py_ssize_t i
 * 
 *         if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 */
  __pyx_t_2 = PyTuple_Check(__pyx_v_value); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "View.MemoryView":510
 * 
 *         if isinstance(value, tuple):
 *             bytesvalue = struct.pack(self.view.format, *value)             # <<<<<<<<<<<<<<
 *         else:
 *             bytesvalue = struct.pack(self.view.format, value)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PySequence_Tuple(__pyx_v_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyNumber_Add(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(3, 510, __pyx_L1_error)
    __pyx_v_bytesvalue = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "View.MemoryView":509
 *         cdef Py_ssize_t i
 * 
 *         if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":512
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 *             bytesvalue = struct.pack(self.view.format, value)             # <<<<<<<<<<<<<<
 * 
 *         for i, c in enumerate(bytesvalue):
 */
  /*else*/ {
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 512, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 512, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_1, __pyx_v_value};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 512, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_1, __pyx_v_value};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 512, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 512, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_1);
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_v_value);
      __pyx_t_1 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 512, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(3, 512, __pyx_L1_error)
    __pyx_v_bytesvalue = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;
  }
  __pyx_L3:;

  /* "View.MemoryView":514
 *             bytesvalue = struct.pack(self.view.format, value)
 * 
 *         for i, c in enumerate(bytesvalue):             # <<<<<<<<<<<<<<
 *             itemp[i] = c
 * 
 */
  __pyx_t_9 = 0;
  if (unlikely(__pyx_v_bytesvalue == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' is not iterable");
    __PYX_ERR(3, 514, __pyx_L1_error)
  }
  __Pyx_INCREF(__pyx_v_bytesvalue);
  __pyx_t_10 = __pyx_v_bytesvalue;
  __pyx_t_12 = PyBytes_AS_STRING(__pyx_t_10);
  __pyx_t_13 = (__pyx_t_12 + PyBytes_GET_SIZE(__pyx_t_10));
  for (__pyx_t_14 = __pyx_t_12; __pyx_t_14 < __pyx_t_13; __pyx_t_14++) {
    __pyx_t_11 = __pyx_t_14;
    __pyx_v_c = (__pyx_t_11[0]);

    /* "View.MemoryView":515
 * 
 *         for i, c in enumerate(bytesvalue):
 *             itemp[i] = c             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */
    __pyx_v_i = __pyx_t_9;

    /* "View.MemoryView":514
 *             bytesvalue = struct.pack(self.view.format, value)
 * 
 *         for i, c in enumerate(bytesvalue):             # <<<<<<<<<<<<<<
 *             itemp[i] = c
 * 
 */
    __pyx_t_9 = (__pyx_t_9 + 1);

    /* "View.MemoryView":515
 * 
 *         for i, c in enumerate(bytesvalue):
 *             itemp[i] = c             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */
    (__pyx_v_itemp[__pyx_v_i]) = __pyx_v_c;
  }
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "View.MemoryView":501
 *             return result
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("View.MemoryView.memoryview.assign_item_from_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_bytesvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":518
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(struct __pyx_memoryview_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t *__pyx_t_4;
  char *__pyx_t_5;
  void *__pyx_t_6;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  if (__pyx_v_info == NULL) {
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");
    return -1;
  }
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "View.MemoryView":519
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 * 
 */
  __pyx_t_2 = ((__pyx_v_flags & PyBUF_WRITABLE) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->view.readonly != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":520
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_ND:
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 520, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 520, __pyx_L1_error)

    /* "View.MemoryView":519
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 * 
 */
  }

  /* "View.MemoryView":522
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 * 
 *         if flags & PyBUF_ND:             # <<<<<<<<<<<<<<
 *             info.shape = self.view.shape
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_ND) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":523
 * 
 *         if flags & PyBUF_ND:
 *             info.shape = self.view.shape             # <<<<<<<<<<<<<<
 *         else:
 *             info.shape = NULL
 */
    __pyx_t_4 = __pyx_v_self->view.shape;
    __pyx_v_info->shape = __pyx_t_4;

    /* "View.MemoryView":522
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 * 
 *         if flags & PyBUF_ND:             # <<<<<<<<<<<<<<
 *             info.shape = self.view.shape
 *         else:
 */
    goto __pyx_L6;
  }

  /* "View.MemoryView":525
 *             info.shape = self.view.shape
 *         else:
 *             info.shape = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_STRIDES:
 */
  /*else*/ {
    __pyx_v_info->shape = NULL;
  }
  __pyx_L6:;

  /* "View.MemoryView":527
 *             info.shape = NULL
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.strides = self.view.strides
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_STRIDES) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":528
 * 
 *         if flags & PyBUF_STRIDES:
 *             info.strides = self.view.strides             # <<<<<<<<<<<<<<
 *         else:
 *             info.strides = NULL
 */
    __pyx_t_4 = __pyx_v_self->view.strides;
    __pyx_v_info->strides = __pyx_t_4;

    /* "View.MemoryView":527
 *             info.shape = NULL
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.strides = self.view.strides
 *         else:
 */
    goto __pyx_L7;
  }

  /* "View.MemoryView":530
 *             info.strides = self.view.strides
 *         else:
 *             info.strides = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_INDIRECT:
 */
  /*else*/ {
    __pyx_v_info->strides = NULL;
  }
  __pyx_L7:;

  /* "View.MemoryView":532
 *             info.strides = NULL
 * 
 *         if flags & PyBUF_INDIRECT:             # <<<<<<<<<<<<<<
 *             info.suboffsets = self.view.suboffsets
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_INDIRECT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":533
 * 
 *         if flags & PyBUF_INDIRECT:
 *             info.suboffsets = self.view.suboffsets             # <<<<<<<<<<<<<<
 *         else:
 *             info.suboffsets = NULL
 */
    __pyx_t_4 = __pyx_v_self->view.suboffsets;
    __pyx_v_info->suboffsets = __pyx_t_4;

    /* "View.MemoryView":532
 *             info.strides = NULL
 * 
 *         if flags & PyBUF_INDIRECT:             # <<<<<<<<<<<<<<
 *             info.suboffsets = self.view.suboffsets
 *         else:
 */
    goto __pyx_L8;
  }

  /* "View.MemoryView":535
 *             info.suboffsets = self.view.suboffsets
 *         else:
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */
  /*else*/ {
    __pyx_v_info->suboffsets = NULL;
  }
  __pyx_L8:;

  /* "View.MemoryView":537
 *             info.suboffsets = NULL
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.view.format
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":538
 * 
 *         if flags & PyBUF_FORMAT:
 *             info.format = self.view.format             # <<<<<<<<<<<<<<
 *         else:
 *             info.format = NULL
 */
    __pyx_t_5 = __pyx_v_self->view.format;
    __pyx_v_info->format = __pyx_t_5;

    /* "View.MemoryView":537
 *             info.suboffsets = NULL
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.view.format
 *         else:
 */
    goto __pyx_L9;
  }

  /* "View.MemoryView":540
 *             info.format = self.view.format
 *         else:
 *             info.format = NULL             # <<<<<<<<<<<<<<
 * 
 *         info.buf = self.view.buf
 */
  /*else*/ {
    __pyx_v_info->format = NULL;
  }
  __pyx_L9:;

  /* "View.MemoryView":542
 *             info.format = NULL
 * 
 *         info.buf = self.view.buf             # <<<<<<<<<<<<<<
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize
 */
  __pyx_t_6 = __pyx_v_self->view.buf;
  __pyx_v_info->buf = __pyx_t_6;

  /* "View.MemoryView":543
 * 
 *         info.buf = self.view.buf
 *         info.ndim = self.view.ndim             # <<<<<<<<<<<<<<
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len
 */
  __pyx_t_7 = __pyx_v_self->view.ndim;
  __pyx_v_info->ndim = __pyx_t_7;

  /* "View.MemoryView":544
 *         info.buf = self.view.buf
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize             # <<<<<<<<<<<<<<
 *         info.len = self.view.len
 *         info.readonly = self.view.readonly
 */
  __pyx_t_8 = __pyx_v_self->view.itemsize;
  __pyx_v_info->itemsize = __pyx_t_8;

  /* "View.MemoryView":545
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len             # <<<<<<<<<<<<<<
 *         info.readonly = self.view.readonly
 *         info.obj = self
 */
  __pyx_t_8 = __pyx_v_self->view.len;
  __pyx_v_info->len = __pyx_t_8;

  /* "View.MemoryView":546
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len
 *         info.readonly = self.view.readonly             # <<<<<<<<<<<<<<
 *         info.obj = self
 * 
 */
  __pyx_t_1 = __pyx_v_self->view.readonly;
  __pyx_v_info->readonly = __pyx_t_1;

  /* "View.MemoryView":547
 *         info.len = self.view.len
 *         info.readonly = self.view.readonly
 *         info.obj = self             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "View.MemoryView":518
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":553
 * 
 *     @property
 *     def T(self):             # <<<<<<<<<<<<<<
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)
 */

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_1T_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_1T_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  struct __pyx_memoryviewslice_obj *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":554
 *     @property
 *     def T(self):
 *         cdef _memoryviewslice result = memoryview_copy(self)             # <<<<<<<<<<<<<<
 *         transpose_memslice(&result.from_slice)
 *         return result
 */
  __pyx_t_1 = __pyx_memoryview_copy_object(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 554, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_memoryviewslice_type))))) __PYX_ERR(3, 554, __pyx_L1_error)
  __pyx_v_result = ((struct __pyx_memoryviewslice_obj *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":555
 *     def T(self):
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
  __pyx_t_2 = __pyx_memslice_transpose((&__pyx_v_result->from_slice)); if (unlikely(__pyx_t_2 == ((int)0))) __PYX_ERR(3, 555, __pyx_L1_error)

  /* "View.MemoryView":556
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "View.MemoryView":553
 * 
 *     @property
 *     def T(self):             # <<<<<<<<<<<<<<
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.T.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":559
 * 
 *     @property
 *     def base(self):             # <<<<<<<<<<<<<<
 *         return self.obj
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4base_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4base_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":560
 *     @property
 *     def base(self):
 *         return self.obj             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->obj);
  __pyx_r = __pyx_v_self->obj;
  goto __pyx_L0;

  /* "View.MemoryView":559
 * 
 *     @property
 *     def base(self):             # <<<<<<<<<<<<<<
 *         return self.obj
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":563
 * 
 *     @property
 *     def shape(self):             # <<<<<<<<<<<<<<
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_5shape_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_5shape_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_length;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t *__pyx_t_2;
  Py_ssize_t *__pyx_t_3;
  Py_ssize_t *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":564
 *     @property
 *     def shape(self):
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 564, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = (__pyx_v_self->view.shape + __pyx_v_self->view.ndim);
  for (__pyx_t_4 = __pyx_v_self->view.shape; __pyx_t_4 < __pyx_t_3; __pyx_t_4++) {
    __pyx_t_2 = __pyx_t_4;
    __pyx_v_length = (__pyx_t_2[0]);
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_length); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(3, 564, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __pyx_t_5 = PyList_AsTuple(((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 564, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":563
 * 
 *     @property
 *     def shape(self):             # <<<<<<<<<<<<<<
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.shape.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":567
 * 
 *     @property
 *     def strides(self):             # <<<<<<<<<<<<<<
 *         if self.view.strides == NULL:
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_7strides_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_7strides_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_stride;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t *__pyx_t_3;
  Py_ssize_t *__pyx_t_4;
  Py_ssize_t *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":568
 *     @property
 *     def strides(self):
 *         if self.view.strides == NULL:             # <<<<<<<<<<<<<<
 * 
 *             raise ValueError("Buffer view does not expose strides")
 */
  __pyx_t_1 = ((__pyx_v_self->view.strides == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":570
 *         if self.view.strides == NULL:
 * 
 *             raise ValueError("Buffer view does not expose strides")             # <<<<<<<<<<<<<<
 * 
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 570, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(3, 570, __pyx_L1_error)

    /* "View.MemoryView":568
 *     @property
 *     def strides(self):
 *         if self.view.strides == NULL:             # <<<<<<<<<<<<<<
 * 
 *             raise ValueError("Buffer view does not expose strides")
 */
  }

  /* "View.MemoryView":572
 *             raise ValueError("Buffer view does not expose strides")
 * 
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = (__pyx_v_self->view.strides + __pyx_v_self->view.ndim);
  for (__pyx_t_5 = __pyx_v_self->view.strides; __pyx_t_5 < __pyx_t_4; __pyx_t_5++) {
    __pyx_t_3 = __pyx_t_5;
    __pyx_v_stride = (__pyx_t_3[0]);
    __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_stride); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 572, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_6))) __PYX_ERR(3, 572, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __pyx_t_6 = PyList_AsTuple(((PyObject*)__pyx_t_2)); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":567
 * 
 *     @property
 *     def strides(self):             # <<<<<<<<<<<<<<
 *         if self.view.strides == NULL:
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("View.MemoryView.memoryview.strides.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":575
 * 
 *     @property
 *     def suboffsets(self):             # <<<<<<<<<<<<<<
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim
 */

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_10suboffsets_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_10suboffsets_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_suboffset;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t *__pyx_t_4;
  Py_ssize_t *__pyx_t_5;
  Py_ssize_t *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":576
 *     @property
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:             # <<<<<<<<<<<<<<
 *             return (-1,) * self.view.ndim
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->view.suboffsets == NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":577
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->view.ndim); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 577, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyNumber_Multiply(__pyx_tuple__21, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 577, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "View.MemoryView":576
 *     @property
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:             # <<<<<<<<<<<<<<
 *             return (-1,) * self.view.ndim
 * 
 */
  }

  /* "View.MemoryView":579
 *             return (-1,) * self.view.ndim
 * 
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = (__pyx_v_self->view.suboffsets + __pyx_v_self->view.ndim);
  for (__pyx_t_6 = __pyx_v_self->view.suboffsets; __pyx_t_6 < __pyx_t_5; __pyx_t_6++) {
    __pyx_t_4 = __pyx_t_6;
    __pyx_v_suboffset = (__pyx_t_4[0]);
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_suboffset); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 579, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_2))) __PYX_ERR(3, 579, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_t_2 = PyList_AsTuple(((PyObject*)__pyx_t_3)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":575
 * 
 *     @property
 *     def suboffsets(self):             # <<<<<<<<<<<<<<
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.suboffsets.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":582
 * 
 *     @property
 *     def ndim(self):             # <<<<<<<<<<<<<<
 *         return self.view.ndim
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4ndim_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4ndim_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":583
 *     @property
 *     def ndim(self):
 *         return self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->view.ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":582
 * 
 *     @property
 *     def ndim(self):             # <<<<<<<<<<<<<<
 *         return self.view.ndim
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.ndim.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":586
 * 
 *     @property
 *     def itemsize(self):             # <<<<<<<<<<<<<<
 *         return self.view.itemsize
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_8itemsize_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_8itemsize_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":587
 *     @property
 *     def itemsize(self):
 *         return self.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_self->view.itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":586
 * 
 *     @property
 *     def itemsize(self):             # <<<<<<<<<<<<<<
 *         return self.view.itemsize
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.itemsize.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":590
 * 
 *     @property
 *     def nbytes(self):             # <<<<<<<<<<<<<<
 *         return self.size * self.view.itemsize
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_6nbytes_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_6nbytes_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":591
 *     @property
 *     def nbytes(self):
 *         return self.size * self.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 591, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_self->view.itemsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 591, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 591, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":590
 * 
 *     @property
 *     def nbytes(self):             # <<<<<<<<<<<<<<
 *         return self.size * self.view.itemsize
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.nbytes.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":594
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         if self._size is None:
 *             result = 1
 */

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_v_length = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t *__pyx_t_3;
  Py_ssize_t *__pyx_t_4;
  Py_ssize_t *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":595
 *     @property
 *     def size(self):
 *         if self._size is None:             # <<<<<<<<<<<<<<
 *             result = 1
 * 
 */
  __pyx_t_1 = (__pyx_v_self->_size == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":596
 *     def size(self):
 *         if self._size is None:
 *             result = 1             # <<<<<<<<<<<<<<
 * 
 *             for length in self.view.shape[:self.view.ndim]:
 */
    __Pyx_INCREF(__pyx_int_1);
    __pyx_v_result = __pyx_int_1;

    /* "View.MemoryView":598
 *             result = 1
 * 
 *             for length in self.view.shape[:self.view.ndim]:             # <<<<<<<<<<<<<<
 *                 result *= length
 * 
 */
    __pyx_t_4 = (__pyx_v_self->view.shape + __pyx_v_self->view.ndim);
    for (__pyx_t_5 = __pyx_v_self->view.shape; __pyx_t_5 < __pyx_t_4; __pyx_t_5++) {
      __pyx_t_3 = __pyx_t_5;
      __pyx_t_6 = PyInt_FromSsize_t((__pyx_t_3[0])); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 598, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_XDECREF_SET(__pyx_v_length, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "View.MemoryView":599
 * 
 *             for length in self.view.shape[:self.view.ndim]:
 *                 result *= length             # <<<<<<<<<<<<<<
 * 
 *             self._size = result
 */
      __pyx_t_6 = PyNumber_InPlaceMultiply(__pyx_v_result, __pyx_v_length); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 599, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_6);
      __pyx_t_6 = 0;
    }

    /* "View.MemoryView":601
 *                 result *= length
 * 
 *             self._size = result             # <<<<<<<<<<<<<<
 * 
 *         return self._size
 */
    __Pyx_INCREF(__pyx_v_result);
    __Pyx_GIVEREF(__pyx_v_result);
    __Pyx_GOTREF(__pyx_v_self->_size);
    __Pyx_DECREF(__pyx_v_self->_size);
    __pyx_v_self->_size = __pyx_v_result;

    /* "View.MemoryView":595
 *     @property
 *     def size(self):
 *         if self._size is None:             # <<<<<<<<<<<<<<
 *             result = 1
 * 
 */
  }

  /* "View.MemoryView":603
 *             self._size = result
 * 
 *         return self._size             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_size);
  __pyx_r = __pyx_v_self->_size;
  goto __pyx_L0;

  /* "View.MemoryView":594
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         if self._size is None:
 *             result = 1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("View.MemoryView.memoryview.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_length);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":605
 *         return self._size
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]
 */

/* Python wrapper */
static Py_ssize_t __pyx_memoryview___len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_memoryview___len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "View.MemoryView":606
 * 
 *     def __len__(self):
 *         if self.view.ndim >= 1:             # <<<<<<<<<<<<<<
 *             return self.view.shape[0]
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->view.ndim >= 1) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":607
 *     def __len__(self):
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
    __pyx_r = (__pyx_v_self->view.shape[0]);
    goto __pyx_L0;

    /* "View.MemoryView":606
 * 
 *     def __len__(self):
 *         if self.view.ndim >= 1:             # <<<<<<<<<<<<<<
 *             return self.view.shape[0]
 * 
 */
  }

  /* "View.MemoryView":609
 *             return self.view.shape[0]
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":605
 *         return self._size
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":611
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))
 */

/* Python wrapper */
static PyObject *__pyx_memoryview___repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview___repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "View.MemoryView":612
 * 
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,             # <<<<<<<<<<<<<<
 *                                                id(self))
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":613
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_id, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 613, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "View.MemoryView":612
 * 
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,             # <<<<<<<<<<<<<<
 *                                                id(self))
 * 
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_MemoryView_of_r_at_0x_x, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":611
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":615
 *                                                id(self))
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview___str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview___str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "View.MemoryView":616
 * 
 *     def __str__(self):
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_MemoryView_of_r_object, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":615
 *                                                id(self))
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":619
 * 
 * 
 *     def is_c_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_is_c_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_memoryview_is_c_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_c_contig (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice *__pyx_v_mslice;
  __Pyx_memviewslice __pyx_v_tmp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_c_contig", 0);

  /* "View.MemoryView":622
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)             # <<<<<<<<<<<<<<
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)
 * 
 */
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_self, (&__pyx_v_tmp)); if (unlikely(__pyx_t_1 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(3, 622, __pyx_L1_error)
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":623
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *     def is_f_contig(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_memviewslice_is_contig((__pyx_v_mslice[0]), 'C', __pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":619
 * 
 * 
 *     def is_c_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_c_contig", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":625
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)
 * 
 *     def is_f_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_is_f_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_memoryview_is_f_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_f_contig (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice *__pyx_v_mslice;
  __Pyx_memviewslice __pyx_v_tmp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_f_contig", 0);

  /* "View.MemoryView":628
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)             # <<<<<<<<<<<<<<
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)
 * 
 */
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_self, (&__pyx_v_tmp)); if (unlikely(__pyx_t_1 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(3, 628, __pyx_L1_error)
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":629
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *     def copy(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_memviewslice_is_contig((__pyx_v_mslice[0]), 'F', __pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":625
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)
 * 
 *     def is_f_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_f_contig", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":631
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_memoryview_copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_mslice;
  int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);

  /* "View.MemoryView":633
 *     def copy(self):
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS             # <<<<<<<<<<<<<<
 * 
 *         slice_copy(self, &mslice)
 */
  __pyx_v_flags = (__pyx_v_self->flags & (~PyBUF_F_CONTIGUOUS));

  /* "View.MemoryView":635
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 * 
 *         slice_copy(self, &mslice)             # <<<<<<<<<<<<<<
 *         mslice = slice_copy_contig(&mslice, "c", self.view.ndim,
 *                                    self.view.itemsize,
 */
  __pyx_memoryview_slice_copy(__pyx_v_self, (&__pyx_v_mslice));

  /* "View.MemoryView":636
 * 
 *         slice_copy(self, &mslice)
 *         mslice = slice_copy_contig(&mslice, "c", self.view.ndim,             # <<<<<<<<<<<<<<
 *                                    self.view.itemsize,
 *                                    flags|PyBUF_C_CONTIGUOUS,
 */
  __pyx_t_1 = __pyx_memoryview_copy_new_contig((&__pyx_v_mslice), ((char *)"c"), __pyx_v_self->view.ndim, __pyx_v_self->view.itemsize, (__pyx_v_flags | PyBUF_C_CONTIGUOUS), __pyx_v_self->dtype_is_object); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 636, __pyx_L1_error)
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":641
 *                                    self.dtype_is_object)
 * 
 *         return memoryview_copy_from_slice(self, &mslice)             # <<<<<<<<<<<<<<
 * 
 *     def copy_fortran(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_memoryview_copy_object_from_slice(__pyx_v_self, (&__pyx_v_mslice)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":631
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":643
 *         return memoryview_copy_from_slice(self, &mslice)
 * 
 *     def copy_fortran(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_copy_fortran(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_memoryview_copy_fortran(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_fortran (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_src;
  __Pyx_memviewslice __pyx_v_dst;
  int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_fortran", 0);

  /* "View.MemoryView":645
 *     def copy_fortran(self):
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS             # <<<<<<<<<<<<<<
 * 
 *         slice_copy(self, &src)
 */
  __pyx_v_flags = (__pyx_v_self->flags & (~PyBUF_C_CONTIGUOUS));

  /* "View.MemoryView":647
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 * 
 *         slice_copy(self, &src)             # <<<<<<<<<<<<<<
 *         dst = slice_copy_contig(&src, "fortran", self.view.ndim,
 *                                 self.view.itemsize,
 */
  __pyx_memoryview_slice_copy(__pyx_v_self, (&__pyx_v_src));

  /* "View.MemoryView":648
 * 
 *         slice_copy(self, &src)
 *         dst = slice_copy_contig(&src, "fortran", self.view.ndim,             # <<<<<<<<<<<<<<
 *                                 self.view.itemsize,
 *                                 flags|PyBUF_F_CONTIGUOUS,
 */
  __pyx_t_1 = __pyx_memoryview_copy_new_contig((&__pyx_v_src), ((char *)"fortran"), __pyx_v_self->view.ndim, __pyx_v_self->view.itemsize, (__pyx_v_flags | PyBUF_F_CONTIGUOUS), __pyx_v_self->dtype_is_object); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 648, __pyx_L1_error)
  __pyx_v_dst = __pyx_t_1;

  /* "View.MemoryView":653
 *                                 self.dtype_is_object)
 * 
 *         return memoryview_copy_from_slice(self, &dst)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_memoryview_copy_object_from_slice(__pyx_v_self, (&__pyx_v_dst)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":643
 *         return memoryview_copy_from_slice(self, &mslice)
 * 
 *     def copy_fortran(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.copy_fortran", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw___pyx_memoryview_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw___pyx_memoryview_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf___pyx_memoryview___reduce_cython__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_memoryview___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(3, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw___pyx_memoryview_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw___pyx_memoryview_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf___pyx_memoryview_2__setstate_cython__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_memoryview_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__23, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(3, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":657
 * 
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):             # <<<<<<<<<<<<<<
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 */

static PyObject *__pyx_memoryview_new(PyObject *__pyx_v_o, int __pyx_v_flags, int __pyx_v_dtype_is_object, __Pyx_TypeInfo *__pyx_v_typeinfo) {
  struct __pyx_memoryview_obj *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_cwrapper", 0);

  /* "View.MemoryView":658
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)             # <<<<<<<<<<<<<<
 *     result.typeinfo = typeinfo
 *     return result
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 658, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_dtype_is_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 658, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 658, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_o);
  __Pyx_GIVEREF(__pyx_v_o);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_o);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryview_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 658, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_result = ((struct __pyx_memoryview_obj *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "View.MemoryView":659
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo             # <<<<<<<<<<<<<<
 *     return result
 * 
 */
  __pyx_v_result->typeinfo = __pyx_v_typeinfo;

  /* "View.MemoryView":660
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 *     return result             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_check')
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "View.MemoryView":657
 * 
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):             # <<<<<<<<<<<<<<
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview_cwrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":663
 * 
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):             # <<<<<<<<<<<<<<
 *     return isinstance(o, memoryview)
 * 
 */

static CYTHON_INLINE int __pyx_memoryview_check(PyObject *__pyx_v_o) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("memoryview_check", 0);

  /* "View.MemoryView":664
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):
 *     return isinstance(o, memoryview)             # <<<<<<<<<<<<<<
 * 
 * cdef tuple _unellipsify(object index, int ndim):
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_o, __pyx_memoryview_type); 
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "View.MemoryView":663
 * 
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):             # <<<<<<<<<<<<<<
 *     return isinstance(o, memoryview)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":666
 *     return isinstance(o, memoryview)
 * 
 * cdef tuple _unellipsify(object index, int ndim):             # <<<<<<<<<<<<<<
 *     """
 *     Replace all ellipses with full slices and fill incomplete indices with
 */

static PyObject *_unellipsify(PyObject *__pyx_v_index, int __pyx_v_ndim) {
  PyObject *__pyx_v_tup = NULL;
  PyObject *__pyx_v_result = NULL;
  int __pyx_v_have_slices;
  int __pyx_v_seen_ellipsis;
  CYTHON_UNUSED PyObject *__pyx_v_idx = NULL;
  PyObject *__pyx_v_item = NULL;
  Py_ssize_t __pyx_v_nslices;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_unellipsify", 0);

  /* "View.MemoryView":671
 *     full slices.
 *     """
 *     if not isinstance(index, tuple):             # <<<<<<<<<<<<<<
 *         tup = (index,)
 *     else:
 */
  __pyx_t_1 = PyTuple_Check(__pyx_v_index); 
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":672
 *     """
 *     if not isinstance(index, tuple):
 *         tup = (index,)             # <<<<<<<<<<<<<<
 *     else:
 *         tup = index
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 672, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_index);
    __Pyx_GIVEREF(__pyx_v_index);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_index);
    __pyx_v_tup = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "View.MemoryView":671
 *     full slices.
 *     """
 *     if not isinstance(index, tuple):             # <<<<<<<<<<<<<<
 *         tup = (index,)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":674
 *         tup = (index,)
 *     else:
 *         tup = index             # <<<<<<<<<<<<<<
 * 
 *     result = []
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_index);
    __pyx_v_tup = __pyx_v_index;
  }
  __pyx_L3:;

  /* "View.MemoryView":676
 *         tup = index
 * 
 *     result = []             # <<<<<<<<<<<<<<
 *     have_slices = False
 *     seen_ellipsis = False
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 676, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_result = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "View.MemoryView":677
 * 
 *     result = []
 *     have_slices = False             # <<<<<<<<<<<<<<
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 */
  __pyx_v_have_slices = 0;

  /* "View.MemoryView":678
 *     result = []
 *     have_slices = False
 *     seen_ellipsis = False             # <<<<<<<<<<<<<<
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 */
  __pyx_v_seen_ellipsis = 0;

  /* "View.MemoryView":679
 *     have_slices = False
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):             # <<<<<<<<<<<<<<
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_t_3 = __pyx_int_0;
  if (likely(PyList_CheckExact(__pyx_v_tup)) || PyTuple_CheckExact(__pyx_v_tup)) {
    __pyx_t_4 = __pyx_v_tup; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_v_tup); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 679, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 679, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(3, 679, __pyx_L1_error)
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 679, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(3, 679, __pyx_L1_error)
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 679, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      }
    } else {
      __pyx_t_7 = __pyx_t_6(__pyx_t_4);
      if (unlikely(!__pyx_t_7)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(3, 679, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_7);
    __pyx_t_7 = 0;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_3);
    __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_t_3, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 679, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3);
    __pyx_t_3 = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "View.MemoryView":680
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:             # <<<<<<<<<<<<<<
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 */
    __pyx_t_2 = (__pyx_v_item == __pyx_builtin_Ellipsis);
    __pyx_t_1 = (__pyx_t_2 != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":681
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 *             if not seen_ellipsis:             # <<<<<<<<<<<<<<
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True
 */
      __pyx_t_1 = ((!(__pyx_v_seen_ellipsis != 0)) != 0);
      if (__pyx_t_1) {

        /* "View.MemoryView":682
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))             # <<<<<<<<<<<<<<
 *                 seen_ellipsis = True
 *             else:
 */
        __pyx_t_8 = PyObject_Length(__pyx_v_tup); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(3, 682, __pyx_L1_error)
        __pyx_t_7 = PyList_New(1 * ((((__pyx_v_ndim - __pyx_t_8) + 1)<0) ? 0:((__pyx_v_ndim - __pyx_t_8) + 1))); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 682, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        { Py_ssize_t __pyx_temp;
          for (__pyx_temp=0; __pyx_temp < ((__pyx_v_ndim - __pyx_t_8) + 1); __pyx_temp++) {
            __Pyx_INCREF(__pyx_slice__24);
            __Pyx_GIVEREF(__pyx_slice__24);
            PyList_SET_ITEM(__pyx_t_7, __pyx_temp, __pyx_slice__24);
          }
        }
        __pyx_t_9 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_7); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(3, 682, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "View.MemoryView":683
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True             # <<<<<<<<<<<<<<
 *             else:
 *                 result.append(slice(None))
 */
        __pyx_v_seen_ellipsis = 1;

        /* "View.MemoryView":681
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 *             if not seen_ellipsis:             # <<<<<<<<<<<<<<
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True
 */
        goto __pyx_L7;
      }

      /* "View.MemoryView":685
 *                 seen_ellipsis = True
 *             else:
 *                 result.append(slice(None))             # <<<<<<<<<<<<<<
 *             have_slices = True
 *         else:
 */
      /*else*/ {
        __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_result, __pyx_slice__24); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(3, 685, __pyx_L1_error)
      }
      __pyx_L7:;

      /* "View.MemoryView":686
 *             else:
 *                 result.append(slice(None))
 *             have_slices = True             # <<<<<<<<<<<<<<
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):
 */
      __pyx_v_have_slices = 1;

      /* "View.MemoryView":680
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:             # <<<<<<<<<<<<<<
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 */
      goto __pyx_L6;
    }

    /* "View.MemoryView":688
 *             have_slices = True
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):             # <<<<<<<<<<<<<<
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 */
    /*else*/ {
      __pyx_t_2 = PySlice_Check(__pyx_v_item); 
      __pyx_t_10 = ((!(__pyx_t_2 != 0)) != 0);
      if (__pyx_t_10) {
      } else {
        __pyx_t_1 = __pyx_t_10;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_10 = ((!(PyIndex_Check(__pyx_v_item) != 0)) != 0);
      __pyx_t_1 = __pyx_t_10;
      __pyx_L9_bool_binop_done:;
      if (unlikely(__pyx_t_1)) {

        /* "View.MemoryView":689
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):
 *                 raise TypeError("Cannot index with type '%s'" % type(item))             # <<<<<<<<<<<<<<
 * 
 *             have_slices = have_slices or isinstance(item, slice)
 */
        __pyx_t_7 = __Pyx_PyString_FormatSafe(__pyx_kp_s_Cannot_index_with_type_s, ((PyObject *)Py_TYPE(__pyx_v_item))); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 689, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 689, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_Raise(__pyx_t_11, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __PYX_ERR(3, 689, __pyx_L1_error)

        /* "View.MemoryView":688
 *             have_slices = True
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):             # <<<<<<<<<<<<<<
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 */
      }

      /* "View.MemoryView":691
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 *             have_slices = have_slices or isinstance(item, slice)             # <<<<<<<<<<<<<<
 *             result.append(item)
 * 
 */
      __pyx_t_10 = (__pyx_v_have_slices != 0);
      if (!__pyx_t_10) {
      } else {
        __pyx_t_1 = __pyx_t_10;
        goto __pyx_L11_bool_binop_done;
      }
      __pyx_t_10 = PySlice_Check(__pyx_v_item); 
      __pyx_t_2 = (__pyx_t_10 != 0);
      __pyx_t_1 = __pyx_t_2;
      __pyx_L11_bool_binop_done:;
      __pyx_v_have_slices = __pyx_t_1;

      /* "View.MemoryView":692
 * 
 *             have_slices = have_slices or isinstance(item, slice)
 *             result.append(item)             # <<<<<<<<<<<<<<
 * 
 *     nslices = ndim - len(result)
 */
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_result, __pyx_v_item); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(3, 692, __pyx_L1_error)
    }
    __pyx_L6:;

    /* "View.MemoryView":679
 *     have_slices = False
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):             # <<<<<<<<<<<<<<
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":694
 *             result.append(item)
 * 
 *     nslices = ndim - len(result)             # <<<<<<<<<<<<<<
 *     if nslices:
 *         result.extend([slice(None)] * nslices)
 */
  __pyx_t_5 = PyList_GET_SIZE(__pyx_v_result); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 694, __pyx_L1_error)
  __pyx_v_nslices = (__pyx_v_ndim - __pyx_t_5);

  /* "View.MemoryView":695
 * 
 *     nslices = ndim - len(result)
 *     if nslices:             # <<<<<<<<<<<<<<
 *         result.extend([slice(None)] * nslices)
 * 
 */
  __pyx_t_1 = (__pyx_v_nslices != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":696
 *     nslices = ndim - len(result)
 *     if nslices:
 *         result.extend([slice(None)] * nslices)             # <<<<<<<<<<<<<<
 * 
 *     return have_slices or nslices, tuple(result)
 */
    __pyx_t_3 = PyList_New(1 * ((__pyx_v_nslices<0) ? 0:__pyx_v_nslices)); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 696, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    { Py_ssize_t __pyx_temp;
      for (__pyx_temp=0; __pyx_temp < __pyx_v_nslices; __pyx_temp++) {
        __Pyx_INCREF(__pyx_slice__24);
        __Pyx_GIVEREF(__pyx_slice__24);
        PyList_SET_ITEM(__pyx_t_3, __pyx_temp, __pyx_slice__24);
      }
    }
    __pyx_t_9 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_3); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(3, 696, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "View.MemoryView":695
 * 
 *     nslices = ndim - len(result)
 *     if nslices:             # <<<<<<<<<<<<<<
 *         result.extend([slice(None)] * nslices)
 * 
 */
  }

  /* "View.MemoryView":698
 *         result.extend([slice(None)] * nslices)
 * 
 *     return have_slices or nslices, tuple(result)             # <<<<<<<<<<<<<<
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 */
  __Pyx_XDECREF(__pyx_r);
  if (!__pyx_v_have_slices) {
  } else {
    __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_have_slices); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 698, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L14_bool_binop_done;
  }
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_nslices); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __pyx_t_4;
  __pyx_t_4 = 0;
  __pyx_L14_bool_binop_done:;
  __pyx_t_4 = PyList_AsTuple(__pyx_v_result); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_r = ((PyObject*)__pyx_t_11);
  __pyx_t_11 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":666
 *     return isinstance(o, memoryview)
 * 
 * cdef tuple _unellipsify(object index, int ndim):             # <<<<<<<<<<<<<<
 *     """
 *     Replace all ellipses with full slices and fill incomplete indices with
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("View.MemoryView._unellipsify", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tup);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_idx);
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":700
 *     return have_slices or nslices, tuple(result)
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):             # <<<<<<<<<<<<<<
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 */

static PyObject *assert_direct_dimensions(Py_ssize_t *__pyx_v_suboffsets, int __pyx_v_ndim) {
  Py_ssize_t __pyx_v_suboffset;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t *__pyx_t_1;
  Py_ssize_t *__pyx_t_2;
  Py_ssize_t *__pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("assert_direct_dimensions", 0);

  /* "View.MemoryView":701
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     for suboffset in suboffsets[:ndim]:             # <<<<<<<<<<<<<<
 *         if suboffset >= 0:
 *             raise ValueError("Indirect dimensions not supported")
 */
  __pyx_t_2 = (__pyx_v_suboffsets + __pyx_v_ndim);
  for (__pyx_t_3 = __pyx_v_suboffsets; __pyx_t_3 < __pyx_t_2; __pyx_t_3++) {
    __pyx_t_1 = __pyx_t_3;
    __pyx_v_suboffset = (__pyx_t_1[0]);

    /* "View.MemoryView":702
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("Indirect dimensions not supported")
 * 
 */
    __pyx_t_4 = ((__pyx_v_suboffset >= 0) != 0);
    if (unlikely(__pyx_t_4)) {

      /* "View.MemoryView":703
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             raise ValueError("Indirect dimensions not supported")             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 703, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(3, 703, __pyx_L1_error)

      /* "View.MemoryView":702
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("Indirect dimensions not supported")
 * 
 */
    }
  }

  /* "View.MemoryView":700
 *     return have_slices or nslices, tuple(result)
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):             # <<<<<<<<<<<<<<
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.assert_direct_dimensions", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":710
 * 
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):             # <<<<<<<<<<<<<<
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim
 *     cdef bint negative_step
 */

static struct __pyx_memoryview_obj *__pyx_memview_slice(struct __pyx_memoryview_obj *__pyx_v_memview, PyObject *__pyx_v_indices) {
  int __pyx_v_new_ndim;
  int __pyx_v_suboffset_dim;
  int __pyx_v_dim;
  __Pyx_memviewslice __pyx_v_src;
  __Pyx_memviewslice __pyx_v_dst;
  __Pyx_memviewslice *__pyx_v_p_src;
  struct __pyx_memoryviewslice_obj *__pyx_v_memviewsliceobj = 0;
  __Pyx_memviewslice *__pyx_v_p_dst;
  int *__pyx_v_p_suboffset_dim;
  Py_ssize_t __pyx_v_start;
  Py_ssize_t __pyx_v_stop;
  Py_ssize_t __pyx_v_step;
  int __pyx_v_have_start;
  int __pyx_v_have_stop;
  int __pyx_v_have_step;
  PyObject *__pyx_v_index = NULL;
  struct __pyx_memoryview_obj *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_memoryview_obj *__pyx_t_4;
  char *__pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memview_slice", 0);

  /* "View.MemoryView":711
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim             # <<<<<<<<<<<<<<
 *     cdef bint negative_step
 *     cdef __Pyx_memviewslice src, dst
 */
  __pyx_v_new_ndim = 0;
  __pyx_v_suboffset_dim = -1;

  /* "View.MemoryView":718
 * 
 * 
 *     memset(&dst, 0, sizeof(dst))             # <<<<<<<<<<<<<<
 * 
 *     cdef _memoryviewslice memviewsliceobj
 */
  (void)(memset((&__pyx_v_dst), 0, (sizeof(__pyx_v_dst))));

  /* "View.MemoryView":722
 *     cdef _memoryviewslice memviewsliceobj
 * 
 *     assert memview.view.ndim > 0             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(memview, _memoryviewslice):
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_memview->view.ndim > 0) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(3, 722, __pyx_L1_error)
    }
  }
  #endif

  /* "View.MemoryView":724
 *     assert memview.view.ndim > 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":725
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         memviewsliceobj = memview             # <<<<<<<<<<<<<<
 *         p_src = &memviewsliceobj.from_slice
 *     else:
 */
    if (!(likely(((((PyObject *)__pyx_v_memview)) == Py_None) || likely(__Pyx_TypeTest(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type))))) __PYX_ERR(3, 725, __pyx_L1_error)
    __pyx_t_3 = ((PyObject *)__pyx_v_memview);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_memviewsliceobj = ((struct __pyx_memoryviewslice_obj *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "View.MemoryView":726
 *     if isinstance(memview, _memoryviewslice):
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice             # <<<<<<<<<<<<<<
 *     else:
 *         slice_copy(memview, &src)
 */
    __pyx_v_p_src = (&__pyx_v_memviewsliceobj->from_slice);

    /* "View.MemoryView":724
 *     assert memview.view.ndim > 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":728
 *         p_src = &memviewsliceobj.from_slice
 *     else:
 *         slice_copy(memview, &src)             # <<<<<<<<<<<<<<
 *         p_src = &src
 * 
 */
  /*else*/ {
    __pyx_memoryview_slice_copy(__pyx_v_memview, (&__pyx_v_src));

    /* "View.MemoryView":729
 *     else:
 *         slice_copy(memview, &src)
 *         p_src = &src             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_p_src = (&__pyx_v_src);
  }
  __pyx_L3:;

  /* "View.MemoryView":735
 * 
 * 
 *     dst.memview = p_src.memview             # <<<<<<<<<<<<<<
 *     dst.data = p_src.data
 * 
 */
  __pyx_t_4 = __pyx_v_p_src->memview;
  __pyx_v_dst.memview = __pyx_t_4;

  /* "View.MemoryView":736
 * 
 *     dst.memview = p_src.memview
 *     dst.data = p_src.data             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __pyx_v_p_src->data;
  __pyx_v_dst.data = __pyx_t_5;

  /* "View.MemoryView":741
 * 
 * 
 *     cdef __Pyx_memviewslice *p_dst = &dst             # <<<<<<<<<<<<<<
 *     cdef int *p_suboffset_dim = &suboffset_dim
 *     cdef Py_ssize_t start, stop, step
 */
  __pyx_v_p_dst = (&__pyx_v_dst);

  /* "View.MemoryView":742
 * 
 *     cdef __Pyx_memviewslice *p_dst = &dst
 *     cdef int *p_suboffset_dim = &suboffset_dim             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t start, stop, step
 *     cdef bint have_start, have_stop, have_step
 */
  __pyx_v_p_suboffset_dim = (&__pyx_v_suboffset_dim);

  /* "View.MemoryView":746
 *     cdef bint have_start, have_stop, have_step
 * 
 *     for dim, index in enumerate(indices):             # <<<<<<<<<<<<<<
 *         if PyIndex_Check(index):
 *             slice_memviewslice(
 */
  __pyx_t_6 = 0;
  if (likely(PyList_CheckExact(__pyx_v_indices)) || PyTuple_CheckExact(__pyx_v_indices)) {
    __pyx_t_3 = __pyx_v_indices; __Pyx_INCREF(__pyx_t_3); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_indices); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 746, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 746, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_9 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_9); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(3, 746, __pyx_L1_error)
        #else
        __pyx_t_9 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 746, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_9 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_9); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(3, 746, __pyx_L1_error)
        #else
        __pyx_t_9 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 746, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        #endif
      }
    } else {
      __pyx_t_9 = __pyx_t_8(__pyx_t_3);
      if (unlikely(!__pyx_t_9)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(3, 746, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_9);
    }
    __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_dim = __pyx_t_6;
    __pyx_t_6 = (__pyx_t_6 + 1);

    /* "View.MemoryView":747
 * 
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):             # <<<<<<<<<<<<<<
 *             slice_memviewslice(
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 */
    __pyx_t_2 = (PyIndex_Check(__pyx_v_index) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":751
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 *                 index, 0, 0, # start, stop, step             # <<<<<<<<<<<<<<
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 */
      __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 751, __pyx_L1_error)

      /* "View.MemoryView":748
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):
 *             slice_memviewslice(             # <<<<<<<<<<<<<<
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 */
      __pyx_t_11 = __pyx_memoryview_slice_memviewslice(__pyx_v_p_dst, (__pyx_v_p_src->shape[__pyx_v_dim]), (__pyx_v_p_src->strides[__pyx_v_dim]), (__pyx_v_p_src->suboffsets[__pyx_v_dim]), __pyx_v_dim, __pyx_v_new_ndim, __pyx_v_p_suboffset_dim, __pyx_t_10, 0, 0, 0, 0, 0, 0); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(3, 748, __pyx_L1_error)

      /* "View.MemoryView":747
 * 
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):             # <<<<<<<<<<<<<<
 *             slice_memviewslice(
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 */
      goto __pyx_L6;
    }

    /* "View.MemoryView":754
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 *         elif index is None:             # <<<<<<<<<<<<<<
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 */
    __pyx_t_2 = (__pyx_v_index == Py_None);
    __pyx_t_1 = (__pyx_t_2 != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":755
 *                 False)
 *         elif index is None:
 *             p_dst.shape[new_ndim] = 1             # <<<<<<<<<<<<<<
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1
 */
      (__pyx_v_p_dst->shape[__pyx_v_new_ndim]) = 1;

      /* "View.MemoryView":756
 *         elif index is None:
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0             # <<<<<<<<<<<<<<
 *             p_dst.suboffsets[new_ndim] = -1
 *             new_ndim += 1
 */
      (__pyx_v_p_dst->strides[__pyx_v_new_ndim]) = 0;

      /* "View.MemoryView":757
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1             # <<<<<<<<<<<<<<
 *             new_ndim += 1
 *         else:
 */
      (__pyx_v_p_dst->suboffsets[__pyx_v_new_ndim]) = -1L;

      /* "View.MemoryView":758
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1
 *             new_ndim += 1             # <<<<<<<<<<<<<<
 *         else:
 *             start = index.start or 0
 */
      __pyx_v_new_ndim = (__pyx_v_new_ndim + 1);

      /* "View.MemoryView":754
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 *         elif index is None:             # <<<<<<<<<<<<<<
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 */
      goto __pyx_L6;
    }

    /* "View.MemoryView":760
 *             new_ndim += 1
 *         else:
 *             start = index.start or 0             # <<<<<<<<<<<<<<
 *             stop = index.stop or 0
 *             step = index.step or 0
 */
    /*else*/ {
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_start); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 760, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(3, 760, __pyx_L1_error)
      if (!__pyx_t_1) {
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 760, __pyx_L1_error)
        __pyx_t_10 = __pyx_t_12;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L7_bool_binop_done;
      }
      __pyx_t_10 = 0;
      __pyx_L7_bool_binop_done:;
      __pyx_v_start = __pyx_t_10;

      /* "View.MemoryView":761
 *         else:
 *             start = index.start or 0
 *             stop = index.stop or 0             # <<<<<<<<<<<<<<
 *             step = index.step or 0
 * 
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_stop); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 761, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(3, 761, __pyx_L1_error)
      if (!__pyx_t_1) {
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 761, __pyx_L1_error)
        __pyx_t_10 = __pyx_t_12;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_10 = 0;
      __pyx_L9_bool_binop_done:;
      __pyx_v_stop = __pyx_t_10;

      /* "View.MemoryView":762
 *             start = index.start or 0
 *             stop = index.stop or 0
 *             step = index.step or 0             # <<<<<<<<<<<<<<
 * 
 *             have_start = index.start is not None
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_step); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 762, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(3, 762, __pyx_L1_error)
      if (!__pyx_t_1) {
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 762, __pyx_L1_error)
        __pyx_t_10 = __pyx_t_12;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L11_bool_binop_done;
      }
      __pyx_t_10 = 0;
      __pyx_L11_bool_binop_done:;
      __pyx_v_step = __pyx_t_10;

      /* "View.MemoryView":764
 *             step = index.step or 0
 * 
 *             have_start = index.start is not None             # <<<<<<<<<<<<<<
 *             have_stop = index.stop is not None
 *             have_step = index.step is not None
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_start); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 764, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = (__pyx_t_9 != Py_None);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_v_have_start = __pyx_t_1;

      /* "View.MemoryView":765
 * 
 *             have_start = index.start is not None
 *             have_stop = index.stop is not None             # <<<<<<<<<<<<<<
 *             have_step = index.step is not None
 * 
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_stop); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 765, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = (__pyx_t_9 != Py_None);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_v_have_stop = __pyx_t_1;

      /* "View.MemoryView":766
 *             have_start = index.start is not None
 *             have_stop = index.stop is not None
 *             have_step = index.step is not None             # <<<<<<<<<<<<<<
 * 
 *             slice_memviewslice(
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_step); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 766, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = (__pyx_t_9 != Py_None);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_v_have_step = __pyx_t_1;

      /* "View.MemoryView":768
 *             have_step = index.step is not None
 * 
 *             slice_memviewslice(             # <<<<<<<<<<<<<<
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 */
      __pyx_t_11 = __pyx_memoryview_slice_memviewslice(__pyx_v_p_dst, (__pyx_v_p_src->shape[__pyx_v_dim]), (__pyx_v_p_src->strides[__pyx_v_dim]), (__pyx_v_p_src->suboffsets[__pyx_v_dim]), __pyx_v_dim, __pyx_v_new_ndim, __pyx_v_p_suboffset_dim, __pyx_v_start, __pyx_v_stop, __pyx_v_step, __pyx_v_have_start, __pyx_v_have_stop, __pyx_v_have_step, 1); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(3, 768, __pyx_L1_error)

      /* "View.MemoryView":774
 *                 have_start, have_stop, have_step,
 *                 True)
 *             new_ndim += 1             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(memview, _memoryviewslice):
 */
      __pyx_v_new_ndim = (__pyx_v_new_ndim + 1);
    }
    __pyx_L6:;

    /* "View.MemoryView":746
 *     cdef bint have_start, have_stop, have_step
 * 
 *     for dim, index in enumerate(indices):             # <<<<<<<<<<<<<<
 *         if PyIndex_Check(index):
 *             slice_memviewslice(
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":776
 *             new_ndim += 1
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":777
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));

    /* "View.MemoryView":778
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_dtype_func,
 *                                     memview.dtype_is_object)
 */
    if (unlikely(!__pyx_v_memviewsliceobj)) { __Pyx_RaiseUnboundLocalError("memviewsliceobj"); __PYX_ERR(3, 778, __pyx_L1_error) }

    /* "View.MemoryView":779
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 *     else:
 */
    if (unlikely(!__pyx_v_memviewsliceobj)) { __Pyx_RaiseUnboundLocalError("memviewsliceobj"); __PYX_ERR(3, 779, __pyx_L1_error) }

    /* "View.MemoryView":777
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,
 */
    __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_dst, __pyx_v_new_ndim, __pyx_v_memviewsliceobj->to_object_func, __pyx_v_memviewsliceobj->to_dtype_func, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 777, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_memoryview_type))))) __PYX_ERR(3, 777, __pyx_L1_error)
    __pyx_r = ((struct __pyx_memoryview_obj *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "View.MemoryView":776
 *             new_ndim += 1
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 */
  }

  /* "View.MemoryView":782
 *                                     memview.dtype_is_object)
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(((PyObject *)__pyx_r));

    /* "View.MemoryView":783
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,
 *                                     memview.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_dst, __pyx_v_new_ndim, NULL, NULL, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 782, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "View.MemoryView":782
 *                                     memview.dtype_is_object)
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 * 
 */
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_memoryview_type))))) __PYX_ERR(3, 782, __pyx_L1_error)
    __pyx_r = ((struct __pyx_memoryview_obj *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":710
 * 
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):             # <<<<<<<<<<<<<<
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim
 *     cdef bint negative_step
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("View.MemoryView.memview_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_memviewsliceobj);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":807
 * 
 * @cname('__pyx_memoryview_slice_memviewslice')
 * cdef int slice_memviewslice(             # <<<<<<<<<<<<<<
 *         __Pyx_memviewslice *dst,
 *         Py_ssize_t shape, Py_ssize_t stride, Py_ssize_t suboffset,
 */

static int __pyx_memoryview_slice_memviewslice(__Pyx_memviewslice *__pyx_v_dst, Py_ssize_t __pyx_v_shape, Py_ssize_t __pyx_v_stride, Py_ssize_t __pyx_v_suboffset, int __pyx_v_dim, int __pyx_v_new_ndim, int *__pyx_v_suboffset_dim, Py_ssize_t __pyx_v_start, Py_ssize_t __pyx_v_stop, Py_ssize_t __pyx_v_step, int __pyx_v_have_start, int __pyx_v_have_stop, int __pyx_v_have_step, int __pyx_v_is_slice) {
  Py_ssize_t __pyx_v_new_shape;
  int __pyx_v_negative_step;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":827
 *     cdef bint negative_step
 * 
 *     if not is_slice:             # <<<<<<<<<<<<<<
 * 
 *         if start < 0:
 */
  __pyx_t_1 = ((!(__pyx_v_is_slice != 0)) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":829
 *     if not is_slice:
 * 
 *         if start < 0:             # <<<<<<<<<<<<<<
 *             start += shape
 *         if not 0 <= start < shape:
 */
    __pyx_t_1 = ((__pyx_v_start < 0) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":830
 * 
 *         if start < 0:
 *             start += shape             # <<<<<<<<<<<<<<
 *         if not 0 <= start < shape:
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 */
      __pyx_v_start = (__pyx_v_start + __pyx_v_shape);

      /* "View.MemoryView":829
 *     if not is_slice:
 * 
 *         if start < 0:             # <<<<<<<<<<<<<<
 *             start += shape
 *         if not 0 <= start < shape:
 */
    }

    /* "View.MemoryView":831
 *         if start < 0:
 *             start += shape
 *         if not 0 <= start < shape:             # <<<<<<<<<<<<<<
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 *     else:
 */
    __pyx_t_1 = (0 <= __pyx_v_start);
    if (__pyx_t_1) {
      __pyx_t_1 = (__pyx_v_start < __pyx_v_shape);
    }
    __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":832
 *             start += shape
 *         if not 0 <= start < shape:
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)             # <<<<<<<<<<<<<<
 *     else:
 * 
 */
      __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_IndexError, ((char *)"Index out of bounds (axis %d)"), __pyx_v_dim); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(3, 832, __pyx_L1_error)

      /* "View.MemoryView":831
 *         if start < 0:
 *             start += shape
 *         if not 0 <= start < shape:             # <<<<<<<<<<<<<<
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 *     else:
 */
    }

    /* "View.MemoryView":827
 *     cdef bint negative_step
 * 
 *     if not is_slice:             # <<<<<<<<<<<<<<
 * 
 *         if start < 0:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":835
 *     else:
 * 
 *         negative_step = have_step != 0 and step < 0             # <<<<<<<<<<<<<<
 * 
 *         if have_step and step == 0:
 */
  /*else*/ {
    __pyx_t_1 = ((__pyx_v_have_step != 0) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_2 = __pyx_t_1;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_step < 0) != 0);
    __pyx_t_2 = __pyx_t_1;
    __pyx_L6_bool_binop_done:;
    __pyx_v_negative_step = __pyx_t_2;

    /* "View.MemoryView":837
 *         negative_step = have_step != 0 and step < 0
 * 
 *         if have_step and step == 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)
 * 
 */
    __pyx_t_1 = (__pyx_v_have_step != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_2 = __pyx_t_1;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_step == 0) != 0);
    __pyx_t_2 = __pyx_t_1;
    __pyx_L9_bool_binop_done:;
    if (__pyx_t_2) {

      /* "View.MemoryView":838
 * 
 *         if have_step and step == 0:
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_ValueError, ((char *)"Step may not be zero (axis %d)"), __pyx_v_dim); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(3, 838, __pyx_L1_error)

      /* "View.MemoryView":837
 *         negative_step = have_step != 0 and step < 0
 * 
 *         if have_step and step == 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)
 * 
 */
    }

    /* "View.MemoryView":841
 * 
 * 
 *         if have_start:             # <<<<<<<<<<<<<<
 *             if start < 0:
 *                 start += shape
 */
    __pyx_t_2 = (__pyx_v_have_start != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":842
 * 
 *         if have_start:
 *             if start < 0:             # <<<<<<<<<<<<<<
 *                 start += shape
 *                 if start < 0:
 */
      __pyx_t_2 = ((__pyx_v_start < 0) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":843
 *         if have_start:
 *             if start < 0:
 *                 start += shape             # <<<<<<<<<<<<<<
 *                 if start < 0:
 *                     start = 0
 */
        __pyx_v_start = (__pyx_v_start + __pyx_v_shape);

        /* "View.MemoryView":844
 *             if start < 0:
 *                 start += shape
 *                 if start < 0:             # <<<<<<<<<<<<<<
 *                     start = 0
 *             elif start >= shape:
 */
        __pyx_t_2 = ((__pyx_v_start < 0) != 0);
        if (__pyx_t_2) {

          /* "View.MemoryView":845
 *                 start += shape
 *                 if start < 0:
 *                     start = 0             # <<<<<<<<<<<<<<
 *             elif start >= shape:
 *                 if negative_step:
 */
          __pyx_v_start = 0;

          /* "View.MemoryView":844
 *             if start < 0:
 *                 start += shape
 *                 if start < 0:             # <<<<<<<<<<<<<<
 *                     start = 0
 *             elif start >= shape:
 */
        }

        /* "View.MemoryView":842
 * 
 *         if have_start:
 *             if start < 0:             # <<<<<<<<<<<<<<
 *                 start += shape
 *                 if start < 0:
 */
        goto __pyx_L12;
      }

      /* "View.MemoryView":846
 *                 if start < 0:
 *                     start = 0
 *             elif start >= shape:             # <<<<<<<<<<<<<<
 *                 if negative_step:
 *                     start = shape - 1
 */
      __pyx_t_2 = ((__pyx_v_start >= __pyx_v_shape) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":847
 *                     start = 0
 *             elif start >= shape:
 *                 if negative_step:             # <<<<<<<<<<<<<<
 *                     start = shape - 1
 *                 else:
 */
        __pyx_t_2 = (__pyx_v_negative_step != 0);
        if (__pyx_t_2) {

          /* "View.MemoryView":848
 *             elif start >= shape:
 *                 if negative_step:
 *                     start = shape - 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     start = shape
 */
          __pyx_v_start = (__pyx_v_shape - 1);

          /* "View.MemoryView":847
 *                     start = 0
 *             elif start >= shape:
 *                 if negative_step:             # <<<<<<<<<<<<<<
 *                     start = shape - 1
 *                 else:
 */
          goto __pyx_L14;
        }

        /* "View.MemoryView":850
 *                     start = shape - 1
 *                 else:
 *                     start = shape             # <<<<<<<<<<<<<<
 *         else:
 *             if negative_step:
 */
        /*else*/ {
          __pyx_v_start = __pyx_v_shape;
        }
        __pyx_L14:;

        /* "View.MemoryView":846
 *                 if start < 0:
 *                     start = 0
 *             elif start >= shape:             # <<<<<<<<<<<<<<
 *                 if negative_step:
 *                     start = shape - 1
 */
      }
      __pyx_L12:;

      /* "View.MemoryView":841
 * 
 * 
 *         if have_start:             # <<<<<<<<<<<<<<
 *             if start < 0:
 *                 start += shape
 */
      goto __pyx_L11;
    }

    /* "View.MemoryView":852
 *                     start = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 start = shape - 1
 *             else:
 */
    /*else*/ {
      __pyx_t_2 = (__pyx_v_negative_step != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":853
 *         else:
 *             if negative_step:
 *                 start = shape - 1             # <<<<<<<<<<<<<<
 *             else:
 *                 start = 0
 */
        __pyx_v_start = (__pyx_v_shape - 1);

        /* "View.MemoryView":852
 *                     start = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 start = shape - 1
 *             else:
 */
        goto __pyx_L15;
      }

      /* "View.MemoryView":855
 *                 start = shape - 1
 *             else:
 *                 start = 0             # <<<<<<<<<<<<<<
 * 
 *         if have_stop:
 */
      /*else*/ {
        __pyx_v_start = 0;
      }
      __pyx_L15:;
    }
    __pyx_L11:;

    /* "View.MemoryView":857
 *                 start = 0
 * 
 *         if have_stop:             # <<<<<<<<<<<<<<
 *             if stop < 0:
 *                 stop += shape
 */
    __pyx_t_2 = (__pyx_v_have_stop != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":858
 * 
 *         if have_stop:
 *             if stop < 0:             # <<<<<<<<<<<<<<
 *                 stop += shape
 *                 if stop < 0:
 */
      __pyx_t_2 = ((__pyx_v_stop < 0) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":859
 *         if have_stop:
 *             if stop < 0:
 *                 stop += shape             # <<<<<<<<<<<<<<
 *                 if stop < 0:
 *                     stop = 0
 */
        __pyx_v_stop = (__pyx_v_stop + __pyx_v_shape);

        /* "View.MemoryView":860
 *             if stop < 0:
 *                 stop += shape
 *                 if stop < 0:             # <<<<<<<<<<<<<<
 *                     stop = 0
 *             elif stop > shape:
 */
        __pyx_t_2 = ((__pyx_v_stop < 0) != 0);
        if (__pyx_t_2) {

          /* "View.MemoryView":861
 *                 stop += shape
 *                 if stop < 0:
 *                     stop = 0             # <<<<<<<<<<<<<<
 *             elif stop > shape:
 *                 stop = shape
 */
          __pyx_v_stop = 0;

          /* "View.MemoryView":860
 *             if stop < 0:
 *                 stop += shape
 *                 if stop < 0:             # <<<<<<<<<<<<<<
 *                     stop = 0
 *             elif stop > shape:
 */
        }

        /* "View.MemoryView":858
 * 
 *         if have_stop:
 *             if stop < 0:             # <<<<<<<<<<<<<<
 *                 stop += shape
 *                 if stop < 0:
 */
        goto __pyx_L17;
      }

      /* "View.MemoryView":862
 *                 if stop < 0:
 *                     stop = 0
 *             elif stop > shape:             # <<<<<<<<<<<<<<
 *                 stop = shape
 *         else:
 */
      __pyx_t_2 = ((__pyx_v_stop > __pyx_v_shape) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":863
 *                     stop = 0
 *             elif stop > shape:
 *                 stop = shape             # <<<<<<<<<<<<<<
 *         else:
 *             if negative_step:
 */
        __pyx_v_stop = __pyx_v_shape;

        /* "View.MemoryView":862
 *                 if stop < 0:
 *                     stop = 0
 *             elif stop > shape:             # <<<<<<<<<<<<<<
 *                 stop = shape
 *         else:
 */
      }
      __pyx_L17:;

      /* "View.MemoryView":857
 *                 start = 0
 * 
 *         if have_stop:             # <<<<<<<<<<<<<<
 *             if stop < 0:
 *                 stop += shape
 */
      goto __pyx_L16;
    }

    /* "View.MemoryView":865
 *                 stop = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 stop = -1
 *             else:
 */
    /*else*/ {
      __pyx_t_2 = (__pyx_v_negative_step != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":866
 *         else:
 *             if negative_step:
 *                 stop = -1             # <<<<<<<<<<<<<<
 *             else:
 *                 stop = shape
 */
        __pyx_v_stop = -1L;

        /* "View.MemoryView":865
 *                 stop = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 stop = -1
 *             else:
 */
        goto __pyx_L19;
      }

      /* "View.MemoryView":868
 *                 stop = -1
 *             else:
 *                 stop = shape             # <<<<<<<<<<<<<<
 * 
 *         if not have_step:
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_shape;
      }
      __pyx_L19:;
    }
    __pyx_L16:;

    /* "View.MemoryView":870
 *                 stop = shape
 * 
 *         if not have_step:             # <<<<<<<<<<<<<<
 *             step = 1
 * 
 */
    __pyx_t_2 = ((!(__pyx_v_have_step != 0)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":871
 * 
 *         if not have_step:
 *             step = 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_v_step = 1;

      /* "View.MemoryView":870
 *                 stop = shape
 * 
 *         if not have_step:             # <<<<<<<<<<<<<<
 *             step = 1
 * 
 */
    }

    /* "View.MemoryView":875
 * 
 *         with cython.cdivision(True):
 *             new_shape = (stop - start) // step             # <<<<<<<<<<<<<<
 * 
 *             if (stop - start) - step * new_shape:
 */
    __pyx_v_new_shape = ((__pyx_v_stop - __pyx_v_start) / __pyx_v_step);

    /* "View.MemoryView":877
 *             new_shape = (stop - start) // step
 * 
 *             if (stop - start) - step * new_shape:             # <<<<<<<<<<<<<<
 *                 new_shape += 1
 * 
 */
    __pyx_t_2 = (((__pyx_v_stop - __pyx_v_start) - (__pyx_v_step * __pyx_v_new_shape)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":878
 * 
 *             if (stop - start) - step * new_shape:
 *                 new_shape += 1             # <<<<<<<<<<<<<<
 * 
 *         if new_shape < 0:
 */
      __pyx_v_new_shape = (__pyx_v_new_shape + 1);

      /* "View.MemoryView":877
 *             new_shape = (stop - start) // step
 * 
 *             if (stop - start) - step * new_shape:             # <<<<<<<<<<<<<<
 *                 new_shape += 1
 * 
 */
    }

    /* "View.MemoryView":880
 *                 new_shape += 1
 * 
 *         if new_shape < 0:             # <<<<<<<<<<<<<<
 *             new_shape = 0
 * 
 */
    __pyx_t_2 = ((__pyx_v_new_shape < 0) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":881
 * 
 *         if new_shape < 0:
 *             new_shape = 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_v_new_shape = 0;

      /* "View.MemoryView":880
 *                 new_shape += 1
 * 
 *         if new_shape < 0:             # <<<<<<<<<<<<<<
 *             new_shape = 0
 * 
 */
    }

    /* "View.MemoryView":884
 * 
 * 
 *         dst.strides[new_ndim] = stride * step             # <<<<<<<<<<<<<<
 *         dst.shape[new_ndim] = new_shape
 *         dst.suboffsets[new_ndim] = suboffset
 */
    (__pyx_v_dst->strides[__pyx_v_new_ndim]) = (__pyx_v_stride * __pyx_v_step);

    /* "View.MemoryView":885
 * 
 *         dst.strides[new_ndim] = stride * step
 *         dst.shape[new_ndim] = new_shape             # <<<<<<<<<<<<<<
 *         dst.suboffsets[new_ndim] = suboffset
 * 
 */
    (__pyx_v_dst->shape[__pyx_v_new_ndim]) = __pyx_v_new_shape;

    /* "View.MemoryView":886
 *         dst.strides[new_ndim] = stride * step
 *         dst.shape[new_ndim] = new_shape
 *         dst.suboffsets[new_ndim] = suboffset             # <<<<<<<<<<<<<<
 * 
 * 
 */
    (__pyx_v_dst->suboffsets[__pyx_v_new_ndim]) = __pyx_v_suboffset;
  }
  __pyx_L3:;

  /* "View.MemoryView":889
 * 
 * 
 *     if suboffset_dim[0] < 0:             # <<<<<<<<<<<<<<
 *         dst.data += start * stride
 *     else:
 */
  __pyx_t_2 = (((__pyx_v_suboffset_dim[0]) < 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":890
 * 
 *     if suboffset_dim[0] < 0:
 *         dst.data += start * stride             # <<<<<<<<<<<<<<
 *     else:
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 */
    __pyx_v_dst->data = (__pyx_v_dst->data + (__pyx_v_start * __pyx_v_stride));

    /* "View.MemoryView":889
 * 
 * 
 *     if suboffset_dim[0] < 0:             # <<<<<<<<<<<<<<
 *         dst.data += start * stride
 *     else:
 */
    goto __pyx_L23;
  }

  /* "View.MemoryView":892
 *         dst.data += start * stride
 *     else:
 *         dst.suboffsets[suboffset_dim[0]] += start * stride             # <<<<<<<<<<<<<<
 * 
 *     if suboffset >= 0:
 */
  /*else*/ {
    __pyx_t_3 = (__pyx_v_suboffset_dim[0]);
    (__pyx_v_dst->suboffsets[__pyx_t_3]) = ((__pyx_v_dst->suboffsets[__pyx_t_3]) + (__pyx_v_start * __pyx_v_stride));
  }
  __pyx_L23:;

  /* "View.MemoryView":894
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 * 
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         if not is_slice:
 *             if new_ndim == 0:
 */
  __pyx_t_2 = ((__pyx_v_suboffset >= 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":895
 * 
 *     if suboffset >= 0:
 *         if not is_slice:             # <<<<<<<<<<<<<<
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 */
    __pyx_t_2 = ((!(__pyx_v_is_slice != 0)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":896
 *     if suboffset >= 0:
 *         if not is_slice:
 *             if new_ndim == 0:             # <<<<<<<<<<<<<<
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 */
      __pyx_t_2 = ((__pyx_v_new_ndim == 0) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":897
 *         if not is_slice:
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset             # <<<<<<<<<<<<<<
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "
 */
        __pyx_v_dst->data = ((((char **)__pyx_v_dst->data)[0]) + __pyx_v_suboffset);

        /* "View.MemoryView":896
 *     if suboffset >= 0:
 *         if not is_slice:
 *             if new_ndim == 0:             # <<<<<<<<<<<<<<
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 */
        goto __pyx_L26;
      }

      /* "View.MemoryView":899
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "             # <<<<<<<<<<<<<<
 *                                      "must be indexed and not sliced", dim)
 *         else:
 */
      /*else*/ {

        /* "View.MemoryView":900
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "
 *                                      "must be indexed and not sliced", dim)             # <<<<<<<<<<<<<<
 *         else:
 *             suboffset_dim[0] = new_ndim
 */
        __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_IndexError, ((char *)"All dimensions preceding dimension %d must be indexed and not sliced"), __pyx_v_dim); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(3, 899, __pyx_L1_error)
      }
      __pyx_L26:;

      /* "View.MemoryView":895
 * 
 *     if suboffset >= 0:
 *         if not is_slice:             # <<<<<<<<<<<<<<
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 */
      goto __pyx_L25;
    }

    /* "View.MemoryView":902
 *                                      "must be indexed and not sliced", dim)
 *         else:
 *             suboffset_dim[0] = new_ndim             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    /*else*/ {
      (__pyx_v_suboffset_dim[0]) = __pyx_v_new_ndim;
    }
    __pyx_L25:;

    /* "View.MemoryView":894
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 * 
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         if not is_slice:
 *             if new_ndim == 0:
 */
  }

  /* "View.MemoryView":904
 *             suboffset_dim[0] = new_ndim
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":807
 * 
 * @cname('__pyx_memoryview_slice_memviewslice')
 * cdef int slice_memviewslice(             # <<<<<<<<<<<<<<
 *         __Pyx_memviewslice *dst,
 *         Py_ssize_t shape, Py_ssize_t stride, Py_ssize_t suboffset,
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("View.MemoryView.slice_memviewslice", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":910
 * 
 * @cname('__pyx_pybuffer_index')
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,             # <<<<<<<<<<<<<<
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 */

static char *__pyx_pybuffer_index(Py_buffer *__pyx_v_view, char *__pyx_v_bufp, Py_ssize_t __pyx_v_index, Py_ssize_t __pyx_v_dim) {
  Py_ssize_t __pyx_v_shape;
  Py_ssize_t __pyx_v_stride;
  Py_ssize_t __pyx_v_suboffset;
  Py_ssize_t __pyx_v_itemsize;
  char *__pyx_v_resultp;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pybuffer_index", 0);

  /* "View.MemoryView":912
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t itemsize = view.itemsize
 *     cdef char *resultp
 */
  __pyx_v_suboffset = -1L;

  /* "View.MemoryView":913
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 *     cdef Py_ssize_t itemsize = view.itemsize             # <<<<<<<<<<<<<<
 *     cdef char *resultp
 * 
 */
  __pyx_t_1 = __pyx_v_view->itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":916
 *     cdef char *resultp
 * 
 *     if view.ndim == 0:             # <<<<<<<<<<<<<<
 *         shape = view.len / itemsize
 *         stride = itemsize
 */
  __pyx_t_2 = ((__pyx_v_view->ndim == 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":917
 * 
 *     if view.ndim == 0:
 *         shape = view.len / itemsize             # <<<<<<<<<<<<<<
 *         stride = itemsize
 *     else:
 */
    if (unlikely(__pyx_v_itemsize == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
      __PYX_ERR(3, 917, __pyx_L1_error)
    }
    else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((Py_ssize_t)-1) > 0)) && unlikely(__pyx_v_itemsize == (Py_ssize_t)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_view->len))) {
      PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
      __PYX_ERR(3, 917, __pyx_L1_error)
    }
    __pyx_v_shape = __Pyx_div_Py_ssize_t(__pyx_v_view->len, __pyx_v_itemsize);

    /* "View.MemoryView":918
 *     if view.ndim == 0:
 *         shape = view.len / itemsize
 *         stride = itemsize             # <<<<<<<<<<<<<<
 *     else:
 *         shape = view.shape[dim]
 */
    __pyx_v_stride = __pyx_v_itemsize;

    /* "View.MemoryView":916
 *     cdef char *resultp
 * 
 *     if view.ndim == 0:             # <<<<<<<<<<<<<<
 *         shape = view.len / itemsize
 *         stride = itemsize
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":920
 *         stride = itemsize
 *     else:
 *         shape = view.shape[dim]             # <<<<<<<<<<<<<<
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:
 */
  /*else*/ {
    __pyx_v_shape = (__pyx_v_view->shape[__pyx_v_dim]);

    /* "View.MemoryView":921
 *     else:
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]             # <<<<<<<<<<<<<<
 *         if view.suboffsets != NULL:
 *             suboffset = view.suboffsets[dim]
 */
    __pyx_v_stride = (__pyx_v_view->strides[__pyx_v_dim]);

    /* "View.MemoryView":922
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *             suboffset = view.suboffsets[dim]
 * 
 */
    __pyx_t_2 = ((__pyx_v_view->suboffsets != NULL) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":923
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:
 *             suboffset = view.suboffsets[dim]             # <<<<<<<<<<<<<<
 * 
 *     if index < 0:
 */
      __pyx_v_suboffset = (__pyx_v_view->suboffsets[__pyx_v_dim]);

      /* "View.MemoryView":922
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *             suboffset = view.suboffsets[dim]
 * 
 */
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":925
 *             suboffset = view.suboffsets[dim]
 * 
 *     if index < 0:             # <<<<<<<<<<<<<<
 *         index += view.shape[dim]
 *         if index < 0:
 */
  __pyx_t_2 = ((__pyx_v_index < 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":926
 * 
 *     if index < 0:
 *         index += view.shape[dim]             # <<<<<<<<<<<<<<
 *         if index < 0:
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 */
    __pyx_v_index = (__pyx_v_index + (__pyx_v_view->shape[__pyx_v_dim]));

    /* "View.MemoryView":927
 *     if index < 0:
 *         index += view.shape[dim]
 *         if index < 0:             # <<<<<<<<<<<<<<
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */
    __pyx_t_2 = ((__pyx_v_index < 0) != 0);
    if (unlikely(__pyx_t_2)) {

      /* "View.MemoryView":928
 *         index += view.shape[dim]
 *         if index < 0:
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)             # <<<<<<<<<<<<<<
 * 
 *     if index >= shape:
 */
      __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 928, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 928, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 928, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(3, 928, __pyx_L1_error)

      /* "View.MemoryView":927
 *     if index < 0:
 *         index += view.shape[dim]
 *         if index < 0:             # <<<<<<<<<<<<<<
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */
    }

    /* "View.MemoryView":925
 *             suboffset = view.suboffsets[dim]
 * 
 *     if index < 0:             # <<<<<<<<<<<<<<
 *         index += view.shape[dim]
 *         if index < 0:
 */
  }

  /* "View.MemoryView":930
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     if index >= shape:             # <<<<<<<<<<<<<<
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */
  __pyx_t_2 = ((__pyx_v_index >= __pyx_v_shape) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "View.MemoryView":931
 * 
 *     if index >= shape:
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)             # <<<<<<<<<<<<<<
 * 
 *     resultp = bufp + index * stride
 */
    __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 931, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 931, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 931, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 931, __pyx_L1_error)

    /* "View.MemoryView":930
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     if index >= shape:             # <<<<<<<<<<<<<<
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */
  }

  /* "View.MemoryView":933
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     resultp = bufp + index * stride             # <<<<<<<<<<<<<<
 *     if suboffset >= 0:
 *         resultp = (<char **> resultp)[0] + suboffset
 */
  __pyx_v_resultp = (__pyx_v_bufp + (__pyx_v_index * __pyx_v_stride));

  /* "View.MemoryView":934
 * 
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 */
  __pyx_t_2 = ((__pyx_v_suboffset >= 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":935
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:
 *         resultp = (<char **> resultp)[0] + suboffset             # <<<<<<<<<<<<<<
 * 
 *     return resultp
 */
    __pyx_v_resultp = ((((char **)__pyx_v_resultp)[0]) + __pyx_v_suboffset);

    /* "View.MemoryView":934
 * 
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 */
  }

  /* "View.MemoryView":937
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 *     return resultp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_resultp;
  goto __pyx_L0;

  /* "View.MemoryView":910
 * 
 * @cname('__pyx_pybuffer_index')
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,             # <<<<<<<<<<<<<<
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView.pybuffer_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":943
 * 
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:             # <<<<<<<<<<<<<<
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 */

static int __pyx_memslice_transpose(__Pyx_memviewslice *__pyx_v_memslice) {
  int __pyx_v_ndim;
  Py_ssize_t *__pyx_v_shape;
  Py_ssize_t *__pyx_v_strides;
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_r;
  int __pyx_t_1;
  Py_ssize_t *__pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":944
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:
 *     cdef int ndim = memslice.memview.view.ndim             # <<<<<<<<<<<<<<
 * 
 *     cdef Py_ssize_t *shape = memslice.shape
 */
  __pyx_t_1 = __pyx_v_memslice->memview->view.ndim;
  __pyx_v_ndim = __pyx_t_1;

  /* "View.MemoryView":946
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 *     cdef Py_ssize_t *shape = memslice.shape             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t *strides = memslice.strides
 * 
 */
  __pyx_t_2 = __pyx_v_memslice->shape;
  __pyx_v_shape = __pyx_t_2;

  /* "View.MemoryView":947
 * 
 *     cdef Py_ssize_t *shape = memslice.shape
 *     cdef Py_ssize_t *strides = memslice.strides             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __pyx_v_memslice->strides;
  __pyx_v_strides = __pyx_t_2;

  /* "View.MemoryView":951
 * 
 *     cdef int i, j
 *     for i in range(ndim / 2):             # <<<<<<<<<<<<<<
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]
 */
  __pyx_t_3 = __Pyx_div_long(__pyx_v_ndim, 2);
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_4; __pyx_t_1+=1) {
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":952
 *     cdef int i, j
 *     for i in range(ndim / 2):
 *         j = ndim - 1 - i             # <<<<<<<<<<<<<<
 *         strides[i], strides[j] = strides[j], strides[i]
 *         shape[i], shape[j] = shape[j], shape[i]
 */
    __pyx_v_j = ((__pyx_v_ndim - 1) - __pyx_v_i);

    /* "View.MemoryView":953
 *     for i in range(ndim / 2):
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]             # <<<<<<<<<<<<<<
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 */
    __pyx_t_5 = (__pyx_v_strides[__pyx_v_j]);
    __pyx_t_6 = (__pyx_v_strides[__pyx_v_i]);
    (__pyx_v_strides[__pyx_v_i]) = __pyx_t_5;
    (__pyx_v_strides[__pyx_v_j]) = __pyx_t_6;

    /* "View.MemoryView":954
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]
 *         shape[i], shape[j] = shape[j], shape[i]             # <<<<<<<<<<<<<<
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:
 */
    __pyx_t_6 = (__pyx_v_shape[__pyx_v_j]);
    __pyx_t_5 = (__pyx_v_shape[__pyx_v_i]);
    (__pyx_v_shape[__pyx_v_i]) = __pyx_t_6;
    (__pyx_v_shape[__pyx_v_j]) = __pyx_t_5;

    /* "View.MemoryView":956
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:             # <<<<<<<<<<<<<<
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 */
    __pyx_t_8 = (((__pyx_v_memslice->suboffsets[__pyx_v_i]) >= 0) != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_8 = (((__pyx_v_memslice->suboffsets[__pyx_v_j]) >= 0) != 0);
    __pyx_t_7 = __pyx_t_8;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_7) {

      /* "View.MemoryView":957
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")             # <<<<<<<<<<<<<<
 * 
 *     return 1
 */
      __pyx_t_9 = __pyx_memoryview_err(__pyx_builtin_ValueError, ((char *)"Cannot transpose memoryview with indirect dimensions")); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(3, 957, __pyx_L1_error)

      /* "View.MemoryView":956
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:             # <<<<<<<<<<<<<<
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 */
    }
  }

  /* "View.MemoryView":959
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "View.MemoryView":943
 * 
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:             # <<<<<<<<<<<<<<
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("View.MemoryView.transpose_memslice", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":976
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 */

/* Python wrapper */
static void __pyx_memoryviewslice___dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_memoryviewslice___dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(struct __pyx_memoryviewslice_obj *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":977
 * 
 *     def __dealloc__(self):
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 */
  __PYX_XDEC_MEMVIEW((&__pyx_v_self->from_slice), 1);

  /* "View.MemoryView":976
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":979
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)
 */

static PyObject *__pyx_memoryviewslice_convert_item_to_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("convert_item_to_object", 0);

  /* "View.MemoryView":980
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:             # <<<<<<<<<<<<<<
 *             return self.to_object_func(itemp)
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_self->to_object_func != NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":981
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)             # <<<<<<<<<<<<<<
 *         else:
 *             return memoryview.convert_item_to_object(self, itemp)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_v_self->to_object_func(__pyx_v_itemp); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "View.MemoryView":980
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:             # <<<<<<<<<<<<<<
 *             return self.to_object_func(itemp)
 *         else:
 */
  }

  /* "View.MemoryView":983
 *             return self.to_object_func(itemp)
 *         else:
 *             return memoryview.convert_item_to_object(self, itemp)             # <<<<<<<<<<<<<<
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_memoryview_convert_item_to_object(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_itemp); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 983, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":979
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":985
 *             return memoryview.convert_item_to_object(self, itemp)
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)
 */

static PyObject *__pyx_memoryviewslice_assign_item_from_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("assign_item_from_object", 0);

  /* "View.MemoryView":986
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:             # <<<<<<<<<<<<<<
 *             self.to_dtype_func(itemp, value)
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_self->to_dtype_func != NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":987
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)             # <<<<<<<<<<<<<<
 *         else:
 *             memoryview.assign_item_from_object(self, itemp, value)
 */
    __pyx_t_2 = __pyx_v_self->to_dtype_func(__pyx_v_itemp, __pyx_v_value); if (unlikely(__pyx_t_2 == ((int)0))) __PYX_ERR(3, 987, __pyx_L1_error)

    /* "View.MemoryView":986
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:             # <<<<<<<<<<<<<<
 *             self.to_dtype_func(itemp, value)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":989
 *             self.to_dtype_func(itemp, value)
 *         else:
 *             memoryview.assign_item_from_object(self, itemp, value)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  /*else*/ {
    __pyx_t_3 = __pyx_memoryview_assign_item_from_object(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_itemp, __pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 989, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "View.MemoryView":985
 *             return memoryview.convert_item_to_object(self, itemp)
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.assign_item_from_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":992
 * 
 *     @property
 *     def base(self):             # <<<<<<<<<<<<<<
 *         return self.from_object
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_16_memoryviewslice_4base_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_16_memoryviewslice_4base_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_16_memoryviewslice_4base___get__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_16_memoryviewslice_4base___get__(struct __pyx_memoryviewslice_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":993
 *     @property
 *     def base(self):
 *         return self.from_object             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->from_object);
  __pyx_r = __pyx_v_self->from_object;
  goto __pyx_L0;

  /* "View.MemoryView":992
 * 
 *     @property
 *     def base(self):             # <<<<<<<<<<<<<<
 *         return self.from_object
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw___pyx_memoryviewslice_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw___pyx_memoryviewslice_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf___pyx_memoryviewslice___reduce_cython__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_memoryviewslice___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(3, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw___pyx_memoryviewslice_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw___pyx_memoryviewslice_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf___pyx_memoryviewslice_2__setstate_cython__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_memoryviewslice_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(3, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":999
 * 
 * @cname('__pyx_memoryview_fromslice')
 * cdef memoryview_fromslice(__Pyx_memviewslice memviewslice,             # <<<<<<<<<<<<<<
 *                           int ndim,
 *                           object (*to_object_func)(char *),
 */

static PyObject *__pyx_memoryview_fromslice(__Pyx_memviewslice __pyx_v_memviewslice, int __pyx_v_ndim, PyObject *(*__pyx_v_to_object_func)(char *), int (*__pyx_v_to_dtype_func)(char *, PyObject *), int __pyx_v_dtype_is_object) {
  struct __pyx_memoryviewslice_obj *__pyx_v_result = 0;
  Py_ssize_t __pyx_v_suboffset;
  PyObject *__pyx_v_length = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_TypeInfo *__pyx_t_4;
  Py_buffer __pyx_t_5;
  Py_ssize_t *__pyx_t_6;
  Py_ssize_t *__pyx_t_7;
  Py_ssize_t *__pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_fromslice", 0);

  /* "View.MemoryView":1007
 *     cdef _memoryviewslice result
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
  __pyx_t_1 = ((((PyObject *)__pyx_v_memviewslice.memview) == Py_None) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1008
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:
 *         return None             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "View.MemoryView":1007
 *     cdef _memoryviewslice result
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
  }

  /* "View.MemoryView":1013
 * 
 * 
 *     result = _memoryviewslice(None, 0, dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     result.from_slice = memviewslice
 */
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_dtype_is_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1013, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 1013, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_3, 0, Py_None);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_0);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryviewslice_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1013, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_result = ((struct __pyx_memoryviewslice_obj *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "View.MemoryView":1015
 *     result = _memoryviewslice(None, 0, dtype_is_object)
 * 
 *     result.from_slice = memviewslice             # <<<<<<<<<<<<<<
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)
 * 
 */
  __pyx_v_result->from_slice = __pyx_v_memviewslice;

  /* "View.MemoryView":1016
 * 
 *     result.from_slice = memviewslice
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)             # <<<<<<<<<<<<<<
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base
 */
  __PYX_INC_MEMVIEW((&__pyx_v_memviewslice), 1);

  /* "View.MemoryView":1018
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base             # <<<<<<<<<<<<<<
 *     result.typeinfo = memviewslice.memview.typeinfo
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_memviewslice.memview), __pyx_n_s_base); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1018, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_result->from_object);
  __Pyx_DECREF(__pyx_v_result->from_object);
  __pyx_v_result->from_object = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "View.MemoryView":1019
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base
 *     result.typeinfo = memviewslice.memview.typeinfo             # <<<<<<<<<<<<<<
 * 
 *     result.view = memviewslice.memview.view
 */
  __pyx_t_4 = __pyx_v_memviewslice.memview->typeinfo;
  __pyx_v_result->__pyx_base.typeinfo = __pyx_t_4;

  /* "View.MemoryView":1021
 *     result.typeinfo = memviewslice.memview.typeinfo
 * 
 *     result.view = memviewslice.memview.view             # <<<<<<<<<<<<<<
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim
 */
  __pyx_t_5 = __pyx_v_memviewslice.memview->view;
  __pyx_v_result->__pyx_base.view = __pyx_t_5;

  /* "View.MemoryView":1022
 * 
 *     result.view = memviewslice.memview.view
 *     result.view.buf = <void *> memviewslice.data             # <<<<<<<<<<<<<<
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 */
  __pyx_v_result->__pyx_base.view.buf = ((void *)__pyx_v_memviewslice.data);

  /* "View.MemoryView":1023
 *     result.view = memviewslice.memview.view
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim             # <<<<<<<<<<<<<<
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 *     Py_INCREF(Py_None)
 */
  __pyx_v_result->__pyx_base.view.ndim = __pyx_v_ndim;

  /* "View.MemoryView":1024
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None             # <<<<<<<<<<<<<<
 *     Py_INCREF(Py_None)
 * 
 */
  ((Py_buffer *)(&__pyx_v_result->__pyx_base.view))->obj = Py_None;

  /* "View.MemoryView":1025
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 *     Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:
 */
  Py_INCREF(Py_None);

  /* "View.MemoryView":1027
 *     Py_INCREF(Py_None)
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:             # <<<<<<<<<<<<<<
 *         result.flags = PyBUF_RECORDS
 *     else:
 */
  __pyx_t_1 = ((((struct __pyx_memoryview_obj *)__pyx_v_memviewslice.memview)->flags & PyBUF_WRITABLE) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1028
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:
 *         result.flags = PyBUF_RECORDS             # <<<<<<<<<<<<<<
 *     else:
 *         result.flags = PyBUF_RECORDS_RO
 */
    __pyx_v_result->__pyx_base.flags = PyBUF_RECORDS;

    /* "View.MemoryView":1027
 *     Py_INCREF(Py_None)
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:             # <<<<<<<<<<<<<<
 *         result.flags = PyBUF_RECORDS
 *     else:
 */
    goto __pyx_L4;
  }

  /* "View.MemoryView":1030
 *         result.flags = PyBUF_RECORDS
 *     else:
 *         result.flags = PyBUF_RECORDS_RO             # <<<<<<<<<<<<<<
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
 */
  /*else*/ {
    __pyx_v_result->__pyx_base.flags = PyBUF_RECORDS_RO;
  }
  __pyx_L4:;

  /* "View.MemoryView":1032
 *         result.flags = PyBUF_RECORDS_RO
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape             # <<<<<<<<<<<<<<
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides
 * 
 */
  __pyx_v_result->__pyx_base.view.shape = ((Py_ssize_t *)__pyx_v_result->from_slice.shape);

  /* "View.MemoryView":1033
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_result->__pyx_base.view.strides = ((Py_ssize_t *)__pyx_v_result->from_slice.strides);

  /* "View.MemoryView":1036
 * 
 * 
 *     result.view.suboffsets = NULL             # <<<<<<<<<<<<<<
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:
 */
  __pyx_v_result->__pyx_base.view.suboffsets = NULL;

  /* "View.MemoryView":1037
 * 
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:             # <<<<<<<<<<<<<<
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 */
  __pyx_t_7 = (__pyx_v_result->from_slice.suboffsets + __pyx_v_ndim);
  for (__pyx_t_8 = __pyx_v_result->from_slice.suboffsets; __pyx_t_8 < __pyx_t_7; __pyx_t_8++) {
    __pyx_t_6 = __pyx_t_8;
    __pyx_v_suboffset = (__pyx_t_6[0]);

    /* "View.MemoryView":1038
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break
 */
    __pyx_t_1 = ((__pyx_v_suboffset >= 0) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":1039
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_v_result->__pyx_base.view.suboffsets = ((Py_ssize_t *)__pyx_v_result->from_slice.suboffsets);

      /* "View.MemoryView":1040
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break             # <<<<<<<<<<<<<<
 * 
 *     result.view.len = result.view.itemsize
 */
      goto __pyx_L6_break;

      /* "View.MemoryView":1038
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break
 */
    }
  }
  __pyx_L6_break:;

  /* "View.MemoryView":1042
 *             break
 * 
 *     result.view.len = result.view.itemsize             # <<<<<<<<<<<<<<
 *     for length in result.view.shape[:ndim]:
 *         result.view.len *= length
 */
  __pyx_t_9 = __pyx_v_result->__pyx_base.view.itemsize;
  __pyx_v_result->__pyx_base.view.len = __pyx_t_9;

  /* "View.MemoryView":1043
 * 
 *     result.view.len = result.view.itemsize
 *     for length in result.view.shape[:ndim]:             # <<<<<<<<<<<<<<
 *         result.view.len *= length
 * 
 */
  __pyx_t_7 = (__pyx_v_result->__pyx_base.view.shape + __pyx_v_ndim);
  for (__pyx_t_8 = __pyx_v_result->__pyx_base.view.shape; __pyx_t_8 < __pyx_t_7; __pyx_t_8++) {
    __pyx_t_6 = __pyx_t_8;
    __pyx_t_2 = PyInt_FromSsize_t((__pyx_t_6[0])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1043, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_length, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "View.MemoryView":1044
 *     result.view.len = result.view.itemsize
 *     for length in result.view.shape[:ndim]:
 *         result.view.len *= length             # <<<<<<<<<<<<<<
 * 
 *     result.to_object_func = to_object_func
 */
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_result->__pyx_base.view.len); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1044, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyNumber_InPlaceMultiply(__pyx_t_2, __pyx_v_length); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 1044, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 1044, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_result->__pyx_base.view.len = __pyx_t_9;
  }

  /* "View.MemoryView":1046
 *         result.view.len *= length
 * 
 *     result.to_object_func = to_object_func             # <<<<<<<<<<<<<<
 *     result.to_dtype_func = to_dtype_func
 * 
 */
  __pyx_v_result->to_object_func = __pyx_v_to_object_func;

  /* "View.MemoryView":1047
 * 
 *     result.to_object_func = to_object_func
 *     result.to_dtype_func = to_dtype_func             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
  __pyx_v_result->to_dtype_func = __pyx_v_to_dtype_func;

  /* "View.MemoryView":1049
 *     result.to_dtype_func = to_dtype_func
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "View.MemoryView":999
 * 
 * @cname('__pyx_memoryview_fromslice')
 * cdef memoryview_fromslice(__Pyx_memviewslice memviewslice,             # <<<<<<<<<<<<<<
 *                           int ndim,
 *                           object (*to_object_func)(char *),
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview_fromslice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_length);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1052
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 * cdef __Pyx_memviewslice *get_slice_from_memview(memoryview memview,             # <<<<<<<<<<<<<<
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 */

static __Pyx_memviewslice *__pyx_memoryview_get_slice_from_memoryview(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_mslice) {
  struct __pyx_memoryviewslice_obj *__pyx_v_obj = 0;
  __Pyx_memviewslice *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_slice_from_memview", 0);

  /* "View.MemoryView":1055
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         obj = memview
 *         return &obj.from_slice
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1056
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):
 *         obj = memview             # <<<<<<<<<<<<<<
 *         return &obj.from_slice
 *     else:
 */
    if (!(likely(((((PyObject *)__pyx_v_memview)) == Py_None) || likely(__Pyx_TypeTest(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type))))) __PYX_ERR(3, 1056, __pyx_L1_error)
    __pyx_t_3 = ((PyObject *)__pyx_v_memview);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_obj = ((struct __pyx_memoryviewslice_obj *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "View.MemoryView":1057
 *     if isinstance(memview, _memoryviewslice):
 *         obj = memview
 *         return &obj.from_slice             # <<<<<<<<<<<<<<
 *     else:
 *         slice_copy(memview, mslice)
 */
    __pyx_r = (&__pyx_v_obj->from_slice);
    goto __pyx_L0;

    /* "View.MemoryView":1055
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         obj = memview
 *         return &obj.from_slice
 */
  }

  /* "View.MemoryView":1059
 *         return &obj.from_slice
 *     else:
 *         slice_copy(memview, mslice)             # <<<<<<<<<<<<<<
 *         return mslice
 * 
 */
  /*else*/ {
    __pyx_memoryview_slice_copy(__pyx_v_memview, __pyx_v_mslice);

    /* "View.MemoryView":1060
 *     else:
 *         slice_copy(memview, mslice)
 *         return mslice             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_slice_copy')
 */
    __pyx_r = __pyx_v_mslice;
    goto __pyx_L0;
  }

  /* "View.MemoryView":1052
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 * cdef __Pyx_memviewslice *get_slice_from_memview(memoryview memview,             # <<<<<<<<<<<<<<
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.get_slice_from_memview", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_obj);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1063
 * 
 * @cname('__pyx_memoryview_slice_copy')
 * cdef void slice_copy(memoryview memview, __Pyx_memviewslice *dst):             # <<<<<<<<<<<<<<
 *     cdef int dim
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 */

static void __pyx_memoryview_slice_copy(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_dst) {
  int __pyx_v_dim;
  Py_ssize_t *__pyx_v_shape;
  Py_ssize_t *__pyx_v_strides;
  Py_ssize_t *__pyx_v_suboffsets;
  __Pyx_RefNannyDeclarations
  Py_ssize_t *__pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  __Pyx_RefNannySetupContext("slice_copy", 0);

  /* "View.MemoryView":1067
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 * 
 *     shape = memview.view.shape             # <<<<<<<<<<<<<<
 *     strides = memview.view.strides
 *     suboffsets = memview.view.suboffsets
 */
  __pyx_t_1 = __pyx_v_memview->view.shape;
  __pyx_v_shape = __pyx_t_1;

  /* "View.MemoryView":1068
 * 
 *     shape = memview.view.shape
 *     strides = memview.view.strides             # <<<<<<<<<<<<<<
 *     suboffsets = memview.view.suboffsets
 * 
 */
  __pyx_t_1 = __pyx_v_memview->view.strides;
  __pyx_v_strides = __pyx_t_1;

  /* "View.MemoryView":1069
 *     shape = memview.view.shape
 *     strides = memview.view.strides
 *     suboffsets = memview.view.suboffsets             # <<<<<<<<<<<<<<
 * 
 *     dst.memview = <__pyx_memoryview *> memview
 */
  __pyx_t_1 = __pyx_v_memview->view.suboffsets;
  __pyx_v_suboffsets = __pyx_t_1;

  /* "View.MemoryView":1071
 *     suboffsets = memview.view.suboffsets
 * 
 *     dst.memview = <__pyx_memoryview *> memview             # <<<<<<<<<<<<<<
 *     dst.data = <char *> memview.view.buf
 * 
 */
  __pyx_v_dst->memview = ((struct __pyx_memoryview_obj *)__pyx_v_memview);

  /* "View.MemoryView":1072
 * 
 *     dst.memview = <__pyx_memoryview *> memview
 *     dst.data = <char *> memview.view.buf             # <<<<<<<<<<<<<<
 * 
 *     for dim in range(memview.view.ndim):
 */
  __pyx_v_dst->data = ((char *)__pyx_v_memview->view.buf);

  /* "View.MemoryView":1074
 *     dst.data = <char *> memview.view.buf
 * 
 *     for dim in range(memview.view.ndim):             # <<<<<<<<<<<<<<
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]
 */
  __pyx_t_2 = __pyx_v_memview->view.ndim;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_dim = __pyx_t_4;

    /* "View.MemoryView":1075
 * 
 *     for dim in range(memview.view.ndim):
 *         dst.shape[dim] = shape[dim]             # <<<<<<<<<<<<<<
 *         dst.strides[dim] = strides[dim]
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1
 */
    (__pyx_v_dst->shape[__pyx_v_dim]) = (__pyx_v_shape[__pyx_v_dim]);

    /* "View.MemoryView":1076
 *     for dim in range(memview.view.ndim):
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]             # <<<<<<<<<<<<<<
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1
 * 
 */
    (__pyx_v_dst->strides[__pyx_v_dim]) = (__pyx_v_strides[__pyx_v_dim]);

    /* "View.MemoryView":1077
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_object')
 */
    if ((__pyx_v_suboffsets != 0)) {
      __pyx_t_5 = (__pyx_v_suboffsets[__pyx_v_dim]);
    } else {
      __pyx_t_5 = -1L;
    }
    (__pyx_v_dst->suboffsets[__pyx_v_dim]) = __pyx_t_5;
  }

  /* "View.MemoryView":1063
 * 
 * @cname('__pyx_memoryview_slice_copy')
 * cdef void slice_copy(memoryview memview, __Pyx_memviewslice *dst):             # <<<<<<<<<<<<<<
 *     cdef int dim
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":1080
 * 
 * @cname('__pyx_memoryview_copy_object')
 * cdef memoryview_copy(memoryview memview):             # <<<<<<<<<<<<<<
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 */

static PyObject *__pyx_memoryview_copy_object(struct __pyx_memoryview_obj *__pyx_v_memview) {
  __Pyx_memviewslice __pyx_v_memviewslice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_copy", 0);

  /* "View.MemoryView":1083
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 *     slice_copy(memview, &memviewslice)             # <<<<<<<<<<<<<<
 *     return memoryview_copy_from_slice(memview, &memviewslice)
 * 
 */
  __pyx_memoryview_slice_copy(__pyx_v_memview, (&__pyx_v_memviewslice));

  /* "View.MemoryView":1084
 *     cdef __Pyx_memviewslice memviewslice
 *     slice_copy(memview, &memviewslice)
 *     return memoryview_copy_from_slice(memview, &memviewslice)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_copy_object_from_slice(__pyx_v_memview, (&__pyx_v_memviewslice)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 1084, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":1080
 * 
 * @cname('__pyx_memoryview_copy_object')
 * cdef memoryview_copy(memoryview memview):             # <<<<<<<<<<<<<<
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1087
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 * cdef memoryview_copy_from_slice(memoryview memview, __Pyx_memviewslice *memviewslice):             # <<<<<<<<<<<<<<
 *     """
 *     Create a new memoryview object from a given memoryview object and slice.
 */

static PyObject *__pyx_memoryview_copy_object_from_slice(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_memviewslice) {
  PyObject *(*__pyx_v_to_object_func)(char *);
  int (*__pyx_v_to_dtype_func)(char *, PyObject *);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *(*__pyx_t_3)(char *);
  int (*__pyx_t_4)(char *, PyObject *);
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_copy_from_slice", 0);

  /* "View.MemoryView":1094
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1095
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         to_object_func = (<_memoryviewslice> memview).to_object_func             # <<<<<<<<<<<<<<
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 *     else:
 */
    __pyx_t_3 = ((struct __pyx_memoryviewslice_obj *)__pyx_v_memview)->to_object_func;
    __pyx_v_to_object_func = __pyx_t_3;

    /* "View.MemoryView":1096
 *     if isinstance(memview, _memoryviewslice):
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func             # <<<<<<<<<<<<<<
 *     else:
 *         to_object_func = NULL
 */
    __pyx_t_4 = ((struct __pyx_memoryviewslice_obj *)__pyx_v_memview)->to_dtype_func;
    __pyx_v_to_dtype_func = __pyx_t_4;

    /* "View.MemoryView":1094
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":1098
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 *     else:
 *         to_object_func = NULL             # <<<<<<<<<<<<<<
 *         to_dtype_func = NULL
 * 
 */
  /*else*/ {
    __pyx_v_to_object_func = NULL;

    /* "View.MemoryView":1099
 *     else:
 *         to_object_func = NULL
 *         to_dtype_func = NULL             # <<<<<<<<<<<<<<
 * 
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,
 */
    __pyx_v_to_dtype_func = NULL;
  }
  __pyx_L3:;

  /* "View.MemoryView":1101
 *         to_dtype_func = NULL
 * 
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,             # <<<<<<<<<<<<<<
 *                                 to_object_func, to_dtype_func,
 *                                 memview.dtype_is_object)
 */
  __Pyx_XDECREF(__pyx_r);

  /* "View.MemoryView":1103
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,
 *                                 to_object_func, to_dtype_func,
 *                                 memview.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __pyx_memoryview_fromslice((__pyx_v_memviewslice[0]), __pyx_v_memview->view.ndim, __pyx_v_to_object_func, __pyx_v_to_dtype_func, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 1101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":1087
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 * cdef memoryview_copy_from_slice(memoryview memview, __Pyx_memviewslice *memviewslice):             # <<<<<<<<<<<<<<
 *     """
 *     Create a new memoryview object from a given memoryview object and slice.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy_from_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1109
 * 
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:             # <<<<<<<<<<<<<<
 *     if arg < 0:
 *         return -arg
 */

static Py_ssize_t abs_py_ssize_t(Py_ssize_t __pyx_v_arg) {
  Py_ssize_t __pyx_r;
  int __pyx_t_1;

  /* "View.MemoryView":1110
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:             # <<<<<<<<<<<<<<
 *         return -arg
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_arg < 0) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1111
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:
 *         return -arg             # <<<<<<<<<<<<<<
 *     else:
 *         return arg
 */
    __pyx_r = (-__pyx_v_arg);
    goto __pyx_L0;

    /* "View.MemoryView":1110
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:             # <<<<<<<<<<<<<<
 *         return -arg
 *     else:
 */
  }

  /* "View.MemoryView":1113
 *         return -arg
 *     else:
 *         return arg             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_get_best_slice_order')
 */
  /*else*/ {
    __pyx_r = __pyx_v_arg;
    goto __pyx_L0;
  }

  /* "View.MemoryView":1109
 * 
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:             # <<<<<<<<<<<<<<
 *     if arg < 0:
 *         return -arg
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1116
 * 
 * @cname('__pyx_get_best_slice_order')
 * cdef char get_best_order(__Pyx_memviewslice *mslice, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Figure out the best memory access order for a given slice.
 */

static char __pyx_get_best_slice_order(__Pyx_memviewslice *__pyx_v_mslice, int __pyx_v_ndim) {
  int __pyx_v_i;
  Py_ssize_t __pyx_v_c_stride;
  Py_ssize_t __pyx_v_f_stride;
  char __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "View.MemoryView":1121
 *     """
 *     cdef int i
 *     cdef Py_ssize_t c_stride = 0             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t f_stride = 0
 * 
 */
  __pyx_v_c_stride = 0;

  /* "View.MemoryView":1122
 *     cdef int i
 *     cdef Py_ssize_t c_stride = 0
 *     cdef Py_ssize_t f_stride = 0             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim - 1, -1, -1):
 */
  __pyx_v_f_stride = 0;

  /* "View.MemoryView":1124
 *     cdef Py_ssize_t f_stride = 0
 * 
 *     for i in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]
 */
  for (__pyx_t_1 = (__pyx_v_ndim - 1); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":1125
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             c_stride = mslice.strides[i]
 *             break
 */
    __pyx_t_2 = (((__pyx_v_mslice->shape[__pyx_v_i]) > 1) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1126
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_v_c_stride = (__pyx_v_mslice->strides[__pyx_v_i]);

      /* "View.MemoryView":1127
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]
 *             break             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
 */
      goto __pyx_L4_break;

      /* "View.MemoryView":1125
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             c_stride = mslice.strides[i]
 *             break
 */
    }
  }
  __pyx_L4_break:;

  /* "View.MemoryView":1129
 *             break
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]
 */
  __pyx_t_1 = __pyx_v_ndim;
  __pyx_t_3 = __pyx_t_1;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "View.MemoryView":1130
 * 
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             f_stride = mslice.strides[i]
 *             break
 */
    __pyx_t_2 = (((__pyx_v_mslice->shape[__pyx_v_i]) > 1) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1131
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_v_f_stride = (__pyx_v_mslice->strides[__pyx_v_i]);

      /* "View.MemoryView":1132
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]
 *             break             # <<<<<<<<<<<<<<
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):
 */
      goto __pyx_L7_break;

      /* "View.MemoryView":1130
 * 
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             f_stride = mslice.strides[i]
 *             break
 */
    }
  }
  __pyx_L7_break:;

  /* "View.MemoryView":1134
 *             break
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):             # <<<<<<<<<<<<<<
 *         return 'C'
 *     else:
 */
  __pyx_t_2 = ((abs_py_ssize_t(__pyx_v_c_stride) <= abs_py_ssize_t(__pyx_v_f_stride)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1135
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):
 *         return 'C'             # <<<<<<<<<<<<<<
 *     else:
 *         return 'F'
 */
    __pyx_r = 'C';
    goto __pyx_L0;

    /* "View.MemoryView":1134
 *             break
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):             # <<<<<<<<<<<<<<
 *         return 'C'
 *     else:
 */
  }

  /* "View.MemoryView":1137
 *         return 'C'
 *     else:
 *         return 'F'             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  /*else*/ {
    __pyx_r = 'F';
    goto __pyx_L0;
  }

  /* "View.MemoryView":1116
 * 
 * @cname('__pyx_get_best_slice_order')
 * cdef char get_best_order(__Pyx_memviewslice *mslice, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Figure out the best memory access order for a given slice.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1140
 * 
 * @cython.cdivision(True)
 * cdef void _copy_strided_to_strided(char *src_data, Py_ssize_t *src_strides,             # <<<<<<<<<<<<<<
 *                                    char *dst_data, Py_ssize_t *dst_strides,
 *                                    Py_ssize_t *src_shape, Py_ssize_t *dst_shape,
 */

static void _copy_strided_to_strided(char *__pyx_v_src_data, Py_ssize_t *__pyx_v_src_strides, char *__pyx_v_dst_data, Py_ssize_t *__pyx_v_dst_strides, Py_ssize_t *__pyx_v_src_shape, Py_ssize_t *__pyx_v_dst_shape, int __pyx_v_ndim, size_t __pyx_v_itemsize) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  CYTHON_UNUSED Py_ssize_t __pyx_v_src_extent;
  Py_ssize_t __pyx_v_dst_extent;
  Py_ssize_t __pyx_v_src_stride;
  Py_ssize_t __pyx_v_dst_stride;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;

  /* "View.MemoryView":1147
 * 
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t src_extent = src_shape[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]
 */
  __pyx_v_src_extent = (__pyx_v_src_shape[0]);

  /* "View.MemoryView":1148
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t src_extent = src_shape[0]
 *     cdef Py_ssize_t dst_extent = dst_shape[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t src_stride = src_strides[0]
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 */
  __pyx_v_dst_extent = (__pyx_v_dst_shape[0]);

  /* "View.MemoryView":1149
 *     cdef Py_ssize_t src_extent = src_shape[0]
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 */
  __pyx_v_src_stride = (__pyx_v_src_strides[0]);

  /* "View.MemoryView":1150
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]
 *     cdef Py_ssize_t dst_stride = dst_strides[0]             # <<<<<<<<<<<<<<
 * 
 *     if ndim == 1:
 */
  __pyx_v_dst_stride = (__pyx_v_dst_strides[0]);

  /* "View.MemoryView":1152
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 */
  __pyx_t_1 = ((__pyx_v_ndim == 1) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1153
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */
    __pyx_t_2 = ((__pyx_v_src_stride > 0) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_dst_stride > 0) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }

    /* "View.MemoryView":1154
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):             # <<<<<<<<<<<<<<
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 *        else:
 */
    __pyx_t_2 = (((size_t)__pyx_v_src_stride) == __pyx_v_itemsize);
    if (__pyx_t_2) {
      __pyx_t_2 = (__pyx_v_itemsize == ((size_t)__pyx_v_dst_stride));
    }
    __pyx_t_3 = (__pyx_t_2 != 0);
    __pyx_t_1 = __pyx_t_3;
    __pyx_L5_bool_binop_done:;

    /* "View.MemoryView":1153
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */
    if (__pyx_t_1) {

      /* "View.MemoryView":1155
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)             # <<<<<<<<<<<<<<
 *        else:
 *            for i in range(dst_extent):
 */
      (void)(memcpy(__pyx_v_dst_data, __pyx_v_src_data, (__pyx_v_itemsize * __pyx_v_dst_extent)));

      /* "View.MemoryView":1153
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */
      goto __pyx_L4;
    }

    /* "View.MemoryView":1157
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 *        else:
 *            for i in range(dst_extent):             # <<<<<<<<<<<<<<
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride
 */
    /*else*/ {
      __pyx_t_4 = __pyx_v_dst_extent;
      __pyx_t_5 = __pyx_t_4;
      for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
        __pyx_v_i = __pyx_t_6;

        /* "View.MemoryView":1158
 *        else:
 *            for i in range(dst_extent):
 *                memcpy(dst_data, src_data, itemsize)             # <<<<<<<<<<<<<<
 *                src_data += src_stride
 *                dst_data += dst_stride
 */
        (void)(memcpy(__pyx_v_dst_data, __pyx_v_src_data, __pyx_v_itemsize));

        /* "View.MemoryView":1159
 *            for i in range(dst_extent):
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride             # <<<<<<<<<<<<<<
 *                dst_data += dst_stride
 *     else:
 */
        __pyx_v_src_data = (__pyx_v_src_data + __pyx_v_src_stride);

        /* "View.MemoryView":1160
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride
 *                dst_data += dst_stride             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(dst_extent):
 */
        __pyx_v_dst_data = (__pyx_v_dst_data + __pyx_v_dst_stride);
      }
    }
    __pyx_L4:;

    /* "View.MemoryView":1152
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":1162
 *                dst_data += dst_stride
 *     else:
 *         for i in range(dst_extent):             # <<<<<<<<<<<<<<
 *             _copy_strided_to_strided(src_data, src_strides + 1,
 *                                      dst_data, dst_strides + 1,
 */
  /*else*/ {
    __pyx_t_4 = __pyx_v_dst_extent;
    __pyx_t_5 = __pyx_t_4;
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
      __pyx_v_i = __pyx_t_6;

      /* "View.MemoryView":1163
 *     else:
 *         for i in range(dst_extent):
 *             _copy_strided_to_strided(src_data, src_strides + 1,             # <<<<<<<<<<<<<<
 *                                      dst_data, dst_strides + 1,
 *                                      src_shape + 1, dst_shape + 1,
 */
      _copy_strided_to_strided(__pyx_v_src_data, (__pyx_v_src_strides + 1), __pyx_v_dst_data, (__pyx_v_dst_strides + 1), (__pyx_v_src_shape + 1), (__pyx_v_dst_shape + 1), (__pyx_v_ndim - 1), __pyx_v_itemsize);

      /* "View.MemoryView":1167
 *                                      src_shape + 1, dst_shape + 1,
 *                                      ndim - 1, itemsize)
 *             src_data += src_stride             # <<<<<<<<<<<<<<
 *             dst_data += dst_stride
 * 
 */
      __pyx_v_src_data = (__pyx_v_src_data + __pyx_v_src_stride);

      /* "View.MemoryView":1168
 *                                      ndim - 1, itemsize)
 *             src_data += src_stride
 *             dst_data += dst_stride             # <<<<<<<<<<<<<<
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,
 */
      __pyx_v_dst_data = (__pyx_v_dst_data + __pyx_v_dst_stride);
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":1140
 * 
 * @cython.cdivision(True)
 * cdef void _copy_strided_to_strided(char *src_data, Py_ssize_t *src_strides,             # <<<<<<<<<<<<<<
 *                                    char *dst_data, Py_ssize_t *dst_strides,
 *                                    Py_ssize_t *src_shape, Py_ssize_t *dst_shape,
 */

  /* function exit code */
}

/* "View.MemoryView":1170
 *             dst_data += dst_stride
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 */

static void copy_strided_to_strided(__Pyx_memviewslice *__pyx_v_src, __Pyx_memviewslice *__pyx_v_dst, int __pyx_v_ndim, size_t __pyx_v_itemsize) {

  /* "View.MemoryView":1173
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 *     _copy_strided_to_strided(src.data, src.strides, dst.data, dst.strides,             # <<<<<<<<<<<<<<
 *                              src.shape, dst.shape, ndim, itemsize)
 * 
 */
  _copy_strided_to_strided(__pyx_v_src->data, __pyx_v_src->strides, __pyx_v_dst->data, __pyx_v_dst->strides, __pyx_v_src->shape, __pyx_v_dst->shape, __pyx_v_ndim, __pyx_v_itemsize);

  /* "View.MemoryView":1170
 *             dst_data += dst_stride
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 */

  /* function exit code */
}

/* "View.MemoryView":1177
 * 
 * @cname('__pyx_memoryview_slice_get_size')
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize
 */

static Py_ssize_t __pyx_memoryview_slice_get_size(__Pyx_memviewslice *__pyx_v_src, int __pyx_v_ndim) {
  Py_ssize_t __pyx_v_shape;
  Py_ssize_t __pyx_v_size;
  Py_ssize_t __pyx_r;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t *__pyx_t_2;
  Py_ssize_t *__pyx_t_3;
  Py_ssize_t *__pyx_t_4;

  /* "View.MemoryView":1179
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     for shape in src.shape[:ndim]:
 */
  __pyx_t_1 = __pyx_v_src->memview->view.itemsize;
  __pyx_v_size = __pyx_t_1;

  /* "View.MemoryView":1181
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize
 * 
 *     for shape in src.shape[:ndim]:             # <<<<<<<<<<<<<<
 *         size *= shape
 * 
 */
  __pyx_t_3 = (__pyx_v_src->shape + __pyx_v_ndim);
  for (__pyx_t_4 = __pyx_v_src->shape; __pyx_t_4 < __pyx_t_3; __pyx_t_4++) {
    __pyx_t_2 = __pyx_t_4;
    __pyx_v_shape = (__pyx_t_2[0]);

    /* "View.MemoryView":1182
 * 
 *     for shape in src.shape[:ndim]:
 *         size *= shape             # <<<<<<<<<<<<<<
 * 
 *     return size
 */
    __pyx_v_size = (__pyx_v_size * __pyx_v_shape);
  }

  /* "View.MemoryView":1184
 *         size *= shape
 * 
 *     return size             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_fill_contig_strides_array')
 */
  __pyx_r = __pyx_v_size;
  goto __pyx_L0;

  /* "View.MemoryView":1177
 * 
 * @cname('__pyx_memoryview_slice_get_size')
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1187
 * 
 * @cname('__pyx_fill_contig_strides_array')
 * cdef Py_ssize_t fill_contig_strides_array(             # <<<<<<<<<<<<<<
 *                 Py_ssize_t *shape, Py_ssize_t *strides, Py_ssize_t stride,
 *                 int ndim, char order) nogil:
 */

static Py_ssize_t __pyx_fill_contig_strides_array(Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, Py_ssize_t __pyx_v_stride, int __pyx_v_ndim, char __pyx_v_order) {
  int __pyx_v_idx;
  Py_ssize_t __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "View.MemoryView":1196
 *     cdef int idx
 * 
 *     if order == 'F':             # <<<<<<<<<<<<<<
 *         for idx in range(ndim):
 *             strides[idx] = stride
 */
  __pyx_t_1 = ((__pyx_v_order == 'F') != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1197
 * 
 *     if order == 'F':
 *         for idx in range(ndim):             # <<<<<<<<<<<<<<
 *             strides[idx] = stride
 *             stride *= shape[idx]
 */
    __pyx_t_2 = __pyx_v_ndim;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_idx = __pyx_t_4;

      /* "View.MemoryView":1198
 *     if order == 'F':
 *         for idx in range(ndim):
 *             strides[idx] = stride             # <<<<<<<<<<<<<<
 *             stride *= shape[idx]
 *     else:
 */
      (__pyx_v_strides[__pyx_v_idx]) = __pyx_v_stride;

      /* "View.MemoryView":1199
 *         for idx in range(ndim):
 *             strides[idx] = stride
 *             stride *= shape[idx]             # <<<<<<<<<<<<<<
 *     else:
 *         for idx in range(ndim - 1, -1, -1):
 */
      __pyx_v_stride = (__pyx_v_stride * (__pyx_v_shape[__pyx_v_idx]));
    }

    /* "View.MemoryView":1196
 *     cdef int idx
 * 
 *     if order == 'F':             # <<<<<<<<<<<<<<
 *         for idx in range(ndim):
 *             strides[idx] = stride
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":1201
 *             stride *= shape[idx]
 *     else:
 *         for idx in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *             strides[idx] = stride
 *             stride *= shape[idx]
 */
  /*else*/ {
    for (__pyx_t_2 = (__pyx_v_ndim - 1); __pyx_t_2 > -1; __pyx_t_2-=1) {
      __pyx_v_idx = __pyx_t_2;

      /* "View.MemoryView":1202
 *     else:
 *         for idx in range(ndim - 1, -1, -1):
 *             strides[idx] = stride             # <<<<<<<<<<<<<<
 *             stride *= shape[idx]
 * 
 */
      (__pyx_v_strides[__pyx_v_idx]) = __pyx_v_stride;

      /* "View.MemoryView":1203
 *         for idx in range(ndim - 1, -1, -1):
 *             strides[idx] = stride
 *             stride *= shape[idx]             # <<<<<<<<<<<<<<
 * 
 *     return stride
 */
      __pyx_v_stride = (__pyx_v_stride * (__pyx_v_shape[__pyx_v_idx]));
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":1205
 *             stride *= shape[idx]
 * 
 *     return stride             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 */
  __pyx_r = __pyx_v_stride;
  goto __pyx_L0;

  /* "View.MemoryView":1187
 * 
 * @cname('__pyx_fill_contig_strides_array')
 * cdef Py_ssize_t fill_contig_strides_array(             # <<<<<<<<<<<<<<
 *                 Py_ssize_t *shape, Py_ssize_t *strides, Py_ssize_t stride,
 *                 int ndim, char order) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1208
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 * cdef void *copy_data_to_temp(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                              __Pyx_memviewslice *tmpslice,
 *                              char order,
 */

static void *__pyx_memoryview_copy_data_to_temp(__Pyx_memviewslice *__pyx_v_src, __Pyx_memviewslice *__pyx_v_tmpslice, char __pyx_v_order, int __pyx_v_ndim) {
  int __pyx_v_i;
  void *__pyx_v_result;
  size_t __pyx_v_itemsize;
  size_t __pyx_v_size;
  void *__pyx_r;
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  struct __pyx_memoryview_obj *__pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":1219
 *     cdef void *result
 * 
 *     cdef size_t itemsize = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 *     cdef size_t size = slice_get_size(src, ndim)
 * 
 */
  __pyx_t_1 = __pyx_v_src->memview->view.itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":1220
 * 
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef size_t size = slice_get_size(src, ndim)             # <<<<<<<<<<<<<<
 * 
 *     result = malloc(size)
 */
  __pyx_v_size = __pyx_memoryview_slice_get_size(__pyx_v_src, __pyx_v_ndim);

  /* "View.MemoryView":1222
 *     cdef size_t size = slice_get_size(src, ndim)
 * 
 *     result = malloc(size)             # <<<<<<<<<<<<<<
 *     if not result:
 *         _err(MemoryError, NULL)
 */
  __pyx_v_result = malloc(__pyx_v_size);

  /* "View.MemoryView":1223
 * 
 *     result = malloc(size)
 *     if not result:             # <<<<<<<<<<<<<<
 *         _err(MemoryError, NULL)
 * 
 */
  __pyx_t_2 = ((!(__pyx_v_result != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1224
 *     result = malloc(size)
 *     if not result:
 *         _err(MemoryError, NULL)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __pyx_memoryview_err(__pyx_builtin_MemoryError, NULL); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(3, 1224, __pyx_L1_error)

    /* "View.MemoryView":1223
 * 
 *     result = malloc(size)
 *     if not result:             # <<<<<<<<<<<<<<
 *         _err(MemoryError, NULL)
 * 
 */
  }

  /* "View.MemoryView":1227
 * 
 * 
 *     tmpslice.data = <char *> result             # <<<<<<<<<<<<<<
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):
 */
  __pyx_v_tmpslice->data = ((char *)__pyx_v_result);

  /* "View.MemoryView":1228
 * 
 *     tmpslice.data = <char *> result
 *     tmpslice.memview = src.memview             # <<<<<<<<<<<<<<
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]
 */
  __pyx_t_4 = __pyx_v_src->memview;
  __pyx_v_tmpslice->memview = __pyx_t_4;

  /* "View.MemoryView":1229
 *     tmpslice.data = <char *> result
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         tmpslice.shape[i] = src.shape[i]
 *         tmpslice.suboffsets[i] = -1
 */
  __pyx_t_3 = __pyx_v_ndim;
  __pyx_t_5 = __pyx_t_3;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
    __pyx_v_i = __pyx_t_6;

    /* "View.MemoryView":1230
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]             # <<<<<<<<<<<<<<
 *         tmpslice.suboffsets[i] = -1
 * 
 */
    (__pyx_v_tmpslice->shape[__pyx_v_i]) = (__pyx_v_src->shape[__pyx_v_i]);

    /* "View.MemoryView":1231
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]
 *         tmpslice.suboffsets[i] = -1             # <<<<<<<<<<<<<<
 * 
 *     fill_contig_strides_array(&tmpslice.shape[0], &tmpslice.strides[0], itemsize,
 */
    (__pyx_v_tmpslice->suboffsets[__pyx_v_i]) = -1L;
  }

  /* "View.MemoryView":1233
 *         tmpslice.suboffsets[i] = -1
 * 
 *     fill_contig_strides_array(&tmpslice.shape[0], &tmpslice.strides[0], itemsize,             # <<<<<<<<<<<<<<
 *                               ndim, order)
 * 
 */
  (void)(__pyx_fill_contig_strides_array((&(__pyx_v_tmpslice->shape[0])), (&(__pyx_v_tmpslice->strides[0])), __pyx_v_itemsize, __pyx_v_ndim, __pyx_v_order));

  /* "View.MemoryView":1237
 * 
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if tmpslice.shape[i] == 1:
 *             tmpslice.strides[i] = 0
 */
  __pyx_t_3 = __pyx_v_ndim;
  __pyx_t_5 = __pyx_t_3;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
    __pyx_v_i = __pyx_t_6;

    /* "View.MemoryView":1238
 * 
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:             # <<<<<<<<<<<<<<
 *             tmpslice.strides[i] = 0
 * 
 */
    __pyx_t_2 = (((__pyx_v_tmpslice->shape[__pyx_v_i]) == 1) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1239
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:
 *             tmpslice.strides[i] = 0             # <<<<<<<<<<<<<<
 * 
 *     if slice_is_contig(src[0], order, ndim):
 */
      (__pyx_v_tmpslice->strides[__pyx_v_i]) = 0;

      /* "View.MemoryView":1238
 * 
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:             # <<<<<<<<<<<<<<
 *             tmpslice.strides[i] = 0
 * 
 */
    }
  }

  /* "View.MemoryView":1241
 *             tmpslice.strides[i] = 0
 * 
 *     if slice_is_contig(src[0], order, ndim):             # <<<<<<<<<<<<<<
 *         memcpy(result, src.data, size)
 *     else:
 */
  __pyx_t_2 = (__pyx_memviewslice_is_contig((__pyx_v_src[0]), __pyx_v_order, __pyx_v_ndim) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1242
 * 
 *     if slice_is_contig(src[0], order, ndim):
 *         memcpy(result, src.data, size)             # <<<<<<<<<<<<<<
 *     else:
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)
 */
    (void)(memcpy(__pyx_v_result, __pyx_v_src->data, __pyx_v_size));

    /* "View.MemoryView":1241
 *             tmpslice.strides[i] = 0
 * 
 *     if slice_is_contig(src[0], order, ndim):             # <<<<<<<<<<<<<<
 *         memcpy(result, src.data, size)
 *     else:
 */
    goto __pyx_L9;
  }

  /* "View.MemoryView":1244
 *         memcpy(result, src.data, size)
 *     else:
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
  /*else*/ {
    copy_strided_to_strided(__pyx_v_src, __pyx_v_tmpslice, __pyx_v_ndim, __pyx_v_itemsize);
  }
  __pyx_L9:;

  /* "View.MemoryView":1246
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "View.MemoryView":1208
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 * cdef void *copy_data_to_temp(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                              __Pyx_memviewslice *tmpslice,
 *                              char order,
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("View.MemoryView.copy_data_to_temp", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = NULL;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1251
 * 
 * @cname('__pyx_memoryview_err_extents')
 * cdef int _err_extents(int i, Py_ssize_t extent1,             # <<<<<<<<<<<<<<
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 */

static int __pyx_memoryview_err_extents(int __pyx_v_i, Py_ssize_t __pyx_v_extent1, Py_ssize_t __pyx_v_extent2) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_err_extents", 0);

  /* "View.MemoryView":1254
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 *                                                         (i, extent1, extent2))             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err_dim')
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 1254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_extent1); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_extent2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 1254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 1254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;

  /* "View.MemoryView":1253
 * cdef int _err_extents(int i, Py_ssize_t extent1,
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %             # <<<<<<<<<<<<<<
 *                                                         (i, extent1, extent2))
 * 
 */
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_got_differing_extents_in_dimensi, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 1253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 1253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_Raise(__pyx_t_4, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_ERR(3, 1253, __pyx_L1_error)

  /* "View.MemoryView":1251
 * 
 * @cname('__pyx_memoryview_err_extents')
 * cdef int _err_extents(int i, Py_ssize_t extent1,             # <<<<<<<<<<<<<<
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView._err_extents", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "View.MemoryView":1257
 * 
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:             # <<<<<<<<<<<<<<
 *     raise error(msg.decode('ascii') % dim)
 * 
 */

static int __pyx_memoryview_err_dim(PyObject *__pyx_v_error, char *__pyx_v_msg, int __pyx_v_dim) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_err_dim", 0);
  __Pyx_INCREF(__pyx_v_error);

  /* "View.MemoryView":1258
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:
 *     raise error(msg.decode('ascii') % dim)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err')
 */
  __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_msg, 0, strlen(__pyx_v_msg), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_dim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 1258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyUnicode_Format(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 1258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_INCREF(__pyx_v_error);
  __pyx_t_3 = __pyx_v_error; __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 1258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(3, 1258, __pyx_L1_error)

  /* "View.MemoryView":1257
 * 
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:             # <<<<<<<<<<<<<<
 *     raise error(msg.decode('ascii') % dim)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView._err_dim", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_XDECREF(__pyx_v_error);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "View.MemoryView":1261
 * 
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:             # <<<<<<<<<<<<<<
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))
 */

static int __pyx_memoryview_err(PyObject *__pyx_v_error, char *__pyx_v_msg) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_err", 0);
  __Pyx_INCREF(__pyx_v_error);

  /* "View.MemoryView":1262
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:             # <<<<<<<<<<<<<<
 *         raise error(msg.decode('ascii'))
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_msg != NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":1263
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))             # <<<<<<<<<<<<<<
 *     else:
 *         raise error
 */
    __pyx_t_3 = __Pyx_decode_c_string(__pyx_v_msg, 0, strlen(__pyx_v_msg), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 1263, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_error);
    __pyx_t_4 = __pyx_v_error; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1263, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(3, 1263, __pyx_L1_error)

    /* "View.MemoryView":1262
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:             # <<<<<<<<<<<<<<
 *         raise error(msg.decode('ascii'))
 *     else:
 */
  }

  /* "View.MemoryView":1265
 *         raise error(msg.decode('ascii'))
 *     else:
 *         raise error             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_contents')
 */
  /*else*/ {
    __Pyx_Raise(__pyx_v_error, 0, 0, 0);
    __PYX_ERR(3, 1265, __pyx_L1_error)
  }

  /* "View.MemoryView":1261
 * 
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:             # <<<<<<<<<<<<<<
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView._err", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_XDECREF(__pyx_v_error);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "View.MemoryView":1268
 * 
 * @cname('__pyx_memoryview_copy_contents')
 * cdef int memoryview_copy_contents(__Pyx_memviewslice src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice dst,
 *                                   int src_ndim, int dst_ndim,
 */

static int __pyx_memoryview_copy_contents(__Pyx_memviewslice __pyx_v_src, __Pyx_memviewslice __pyx_v_dst, int __pyx_v_src_ndim, int __pyx_v_dst_ndim, int __pyx_v_dtype_is_object) {
  void *__pyx_v_tmpdata;
  size_t __pyx_v_itemsize;
  int __pyx_v_i;
  char __pyx_v_order;
  int __pyx_v_broadcasting;
  int __pyx_v_direct_copy;
  __Pyx_memviewslice __pyx_v_tmp;
  int __pyx_v_ndim;
  int __pyx_r;
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  void *__pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":1276
 *     Check for overlapping memory and verify the shapes.
 *     """
 *     cdef void *tmpdata = NULL             # <<<<<<<<<<<<<<
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef int i
 */
  __pyx_v_tmpdata = NULL;

  /* "View.MemoryView":1277
 *     """
 *     cdef void *tmpdata = NULL
 *     cdef size_t itemsize = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)
 */
  __pyx_t_1 = __pyx_v_src.memview->view.itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":1279
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)             # <<<<<<<<<<<<<<
 *     cdef bint broadcasting = False
 *     cdef bint direct_copy = False
 */
  __pyx_v_order = __pyx_get_best_slice_order((&__pyx_v_src), __pyx_v_src_ndim);

  /* "View.MemoryView":1280
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)
 *     cdef bint broadcasting = False             # <<<<<<<<<<<<<<
 *     cdef bint direct_copy = False
 *     cdef __Pyx_memviewslice tmp
 */
  __pyx_v_broadcasting = 0;

  /* "View.MemoryView":1281
 *     cdef char order = get_best_order(&src, src_ndim)
 *     cdef bint broadcasting = False
 *     cdef bint direct_copy = False             # <<<<<<<<<<<<<<
 *     cdef __Pyx_memviewslice tmp
 * 
 */
  __pyx_v_direct_copy = 0;

  /* "View.MemoryView":1284
 *     cdef __Pyx_memviewslice tmp
 * 
 *     if src_ndim < dst_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 */
  __pyx_t_2 = ((__pyx_v_src_ndim < __pyx_v_dst_ndim) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1285
 * 
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)             # <<<<<<<<<<<<<<
 *     elif dst_ndim < src_ndim:
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 */
    __pyx_memoryview_broadcast_leading((&__pyx_v_src), __pyx_v_src_ndim, __pyx_v_dst_ndim);

    /* "View.MemoryView":1284
 *     cdef __Pyx_memviewslice tmp
 * 
 *     if src_ndim < dst_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":1286
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 */
  __pyx_t_2 = ((__pyx_v_dst_ndim < __pyx_v_src_ndim) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1287
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 *         broadcast_leading(&dst, dst_ndim, src_ndim)             # <<<<<<<<<<<<<<
 * 
 *     cdef int ndim = max(src_ndim, dst_ndim)
 */
    __pyx_memoryview_broadcast_leading((&__pyx_v_dst), __pyx_v_dst_ndim, __pyx_v_src_ndim);

    /* "View.MemoryView":1286
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 */
  }
  __pyx_L3:;

  /* "View.MemoryView":1289
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 *     cdef int ndim = max(src_ndim, dst_ndim)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
 */
  __pyx_t_3 = __pyx_v_dst_ndim;
  __pyx_t_4 = __pyx_v_src_ndim;
  if (((__pyx_t_3 > __pyx_t_4) != 0)) {
    __pyx_t_5 = __pyx_t_3;
  } else {
    __pyx_t_5 = __pyx_t_4;
  }
  __pyx_v_ndim = __pyx_t_5;

  /* "View.MemoryView":1291
 *     cdef int ndim = max(src_ndim, dst_ndim)
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:
 */
  __pyx_t_5 = __pyx_v_ndim;
  __pyx_t_3 = __pyx_t_5;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "View.MemoryView":1292
 * 
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:             # <<<<<<<<<<<<<<
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 */
    __pyx_t_2 = (((__pyx_v_src.shape[__pyx_v_i]) != (__pyx_v_dst.shape[__pyx_v_i])) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1293
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:             # <<<<<<<<<<<<<<
 *                 broadcasting = True
 *                 src.strides[i] = 0
 */
      __pyx_t_2 = (((__pyx_v_src.shape[__pyx_v_i]) == 1) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":1294
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:
 *                 broadcasting = True             # <<<<<<<<<<<<<<
 *                 src.strides[i] = 0
 *             else:
 */
        __pyx_v_broadcasting = 1;

        /* "View.MemoryView":1295
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 *                 src.strides[i] = 0             # <<<<<<<<<<<<<<
 *             else:
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 */
        (__pyx_v_src.strides[__pyx_v_i]) = 0;

        /* "View.MemoryView":1293
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:             # <<<<<<<<<<<<<<
 *                 broadcasting = True
 *                 src.strides[i] = 0
 */
        goto __pyx_L7;
      }

      /* "View.MemoryView":1297
 *                 src.strides[i] = 0
 *             else:
 *                 _err_extents(i, dst.shape[i], src.shape[i])             # <<<<<<<<<<<<<<
 * 
 *         if src.suboffsets[i] >= 0:
 */
      /*else*/ {
        __pyx_t_6 = __pyx_memoryview_err_extents(__pyx_v_i, (__pyx_v_dst.shape[__pyx_v_i]), (__pyx_v_src.shape[__pyx_v_i])); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(3, 1297, __pyx_L1_error)
      }
      __pyx_L7:;

      /* "View.MemoryView":1292
 * 
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:             # <<<<<<<<<<<<<<
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 */
    }

    /* "View.MemoryView":1299
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 * 
 *         if src.suboffsets[i] >= 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 */
    __pyx_t_2 = (((__pyx_v_src.suboffsets[__pyx_v_i]) >= 0) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1300
 * 
 *         if src.suboffsets[i] >= 0:
 *             _err_dim(ValueError, "Dimension %d is not direct", i)             # <<<<<<<<<<<<<<
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 */
      __pyx_t_6 = __pyx_memoryview_err_dim(__pyx_builtin_ValueError, ((char *)"Dimension %d is not direct"), __pyx_v_i); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(3, 1300, __pyx_L1_error)

      /* "View.MemoryView":1299
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 * 
 *         if src.suboffsets[i] >= 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 */
    }
  }

  /* "View.MemoryView":1302
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):             # <<<<<<<<<<<<<<
 * 
 *         if not slice_is_contig(src, order, ndim):
 */
  __pyx_t_2 = (__pyx_slices_overlap((&__pyx_v_src), (&__pyx_v_dst), __pyx_v_ndim, __pyx_v_itemsize) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1304
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 * 
 *         if not slice_is_contig(src, order, ndim):             # <<<<<<<<<<<<<<
 *             order = get_best_order(&dst, ndim)
 * 
 */
    __pyx_t_2 = ((!(__pyx_memviewslice_is_contig(__pyx_v_src, __pyx_v_order, __pyx_v_ndim) != 0)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1305
 * 
 *         if not slice_is_contig(src, order, ndim):
 *             order = get_best_order(&dst, ndim)             # <<<<<<<<<<<<<<
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)
 */
      __pyx_v_order = __pyx_get_best_slice_order((&__pyx_v_dst), __pyx_v_ndim);

      /* "View.MemoryView":1304
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 * 
 *         if not slice_is_contig(src, order, ndim):             # <<<<<<<<<<<<<<
 *             order = get_best_order(&dst, ndim)
 * 
 */
    }

    /* "View.MemoryView":1307
 *             order = get_best_order(&dst, ndim)
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)             # <<<<<<<<<<<<<<
 *         src = tmp
 * 
 */
    __pyx_t_7 = __pyx_memoryview_copy_data_to_temp((&__pyx_v_src), (&__pyx_v_tmp), __pyx_v_order, __pyx_v_ndim); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(3, 1307, __pyx_L1_error)
    __pyx_v_tmpdata = __pyx_t_7;

    /* "View.MemoryView":1308
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)
 *         src = tmp             # <<<<<<<<<<<<<<
 * 
 *     if not broadcasting:
 */
    __pyx_v_src = __pyx_v_tmp;

    /* "View.MemoryView":1302
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):             # <<<<<<<<<<<<<<
 * 
 *         if not slice_is_contig(src, order, ndim):
 */
  }

  /* "View.MemoryView":1310
 *         src = tmp
 * 
 *     if not broadcasting:             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = ((!(__pyx_v_broadcasting != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1313
 * 
 * 
 *         if slice_is_contig(src, 'C', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):
 */
    __pyx_t_2 = (__pyx_memviewslice_is_contig(__pyx_v_src, 'C', __pyx_v_ndim) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1314
 * 
 *         if slice_is_contig(src, 'C', ndim):
 *             direct_copy = slice_is_contig(dst, 'C', ndim)             # <<<<<<<<<<<<<<
 *         elif slice_is_contig(src, 'F', ndim):
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 */
      __pyx_v_direct_copy = __pyx_memviewslice_is_contig(__pyx_v_dst, 'C', __pyx_v_ndim);

      /* "View.MemoryView":1313
 * 
 * 
 *         if slice_is_contig(src, 'C', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):
 */
      goto __pyx_L12;
    }

    /* "View.MemoryView":1315
 *         if slice_is_contig(src, 'C', ndim):
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 */
    __pyx_t_2 = (__pyx_memviewslice_is_contig(__pyx_v_src, 'F', __pyx_v_ndim) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1316
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):
 *             direct_copy = slice_is_contig(dst, 'F', ndim)             # <<<<<<<<<<<<<<
 * 
 *         if direct_copy:
 */
      __pyx_v_direct_copy = __pyx_memviewslice_is_contig(__pyx_v_dst, 'F', __pyx_v_ndim);

      /* "View.MemoryView":1315
 *         if slice_is_contig(src, 'C', ndim):
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 */
    }
    __pyx_L12:;

    /* "View.MemoryView":1318
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 *         if direct_copy:             # <<<<<<<<<<<<<<
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 */
    __pyx_t_2 = (__pyx_v_direct_copy != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1320
 *         if direct_copy:
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 */
      __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

      /* "View.MemoryView":1321
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))             # <<<<<<<<<<<<<<
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)
 */
      (void)(memcpy(__pyx_v_dst.data, __pyx_v_src.data, __pyx_memoryview_slice_get_size((&__pyx_v_src), __pyx_v_ndim)));

      /* "View.MemoryView":1322
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 *             free(tmpdata)
 *             return 0
 */
      __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

      /* "View.MemoryView":1323
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */
      free(__pyx_v_tmpdata);

      /* "View.MemoryView":1324
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):
 */
      __pyx_r = 0;
      goto __pyx_L0;

      /* "View.MemoryView":1318
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 *         if direct_copy:             # <<<<<<<<<<<<<<
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 */
    }

    /* "View.MemoryView":1310
 *         src = tmp
 * 
 *     if not broadcasting:             # <<<<<<<<<<<<<<
 * 
 * 
 */
  }

  /* "View.MemoryView":1326
 *             return 0
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = (__pyx_v_order == 'F');
  if (__pyx_t_2) {
    __pyx_t_2 = ('F' == __pyx_get_best_slice_order((&__pyx_v_dst), __pyx_v_ndim));
  }
  __pyx_t_8 = (__pyx_t_2 != 0);
  if (__pyx_t_8) {

    /* "View.MemoryView":1329
 * 
 * 
 *         transpose_memslice(&src)             # <<<<<<<<<<<<<<
 *         transpose_memslice(&dst)
 * 
 */
    __pyx_t_5 = __pyx_memslice_transpose((&__pyx_v_src)); if (unlikely(__pyx_t_5 == ((int)0))) __PYX_ERR(3, 1329, __pyx_L1_error)

    /* "View.MemoryView":1330
 * 
 *         transpose_memslice(&src)
 *         transpose_memslice(&dst)             # <<<<<<<<<<<<<<
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 */
    __pyx_t_5 = __pyx_memslice_transpose((&__pyx_v_dst)); if (unlikely(__pyx_t_5 == ((int)0))) __PYX_ERR(3, 1330, __pyx_L1_error)

    /* "View.MemoryView":1326
 *             return 0
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):             # <<<<<<<<<<<<<<
 * 
 * 
 */
  }

  /* "View.MemoryView":1332
 *         transpose_memslice(&dst)
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 */
  __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

  /* "View.MemoryView":1333
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)             # <<<<<<<<<<<<<<
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 * 
 */
  copy_strided_to_strided((&__pyx_v_src), (&__pyx_v_dst), __pyx_v_ndim, __pyx_v_itemsize);

  /* "View.MemoryView":1334
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)
 *     refcount_copying(&dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 * 
 *     free(tmpdata)
 */
  __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

  /* "View.MemoryView":1336
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 * 
 *     free(tmpdata)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_tmpdata);

  /* "View.MemoryView":1337
 * 
 *     free(tmpdata)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":1268
 * 
 * @cname('__pyx_memoryview_copy_contents')
 * cdef int memoryview_copy_contents(__Pyx_memviewslice src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice dst,
 *                                   int src_ndim, int dst_ndim,
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("View.MemoryView.memoryview_copy_contents", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1340
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 * cdef void broadcast_leading(__Pyx_memviewslice *mslice,             # <<<<<<<<<<<<<<
 *                             int ndim,
 *                             int ndim_other) nogil:
 */

static void __pyx_memoryview_broadcast_leading(__Pyx_memviewslice *__pyx_v_mslice, int __pyx_v_ndim, int __pyx_v_ndim_other) {
  int __pyx_v_i;
  int __pyx_v_offset;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "View.MemoryView":1344
 *                             int ndim_other) nogil:
 *     cdef int i
 *     cdef int offset = ndim_other - ndim             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim - 1, -1, -1):
 */
  __pyx_v_offset = (__pyx_v_ndim_other - __pyx_v_ndim);

  /* "View.MemoryView":1346
 *     cdef int offset = ndim_other - ndim
 * 
 *     for i in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]
 */
  for (__pyx_t_1 = (__pyx_v_ndim - 1); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":1347
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         mslice.shape[i + offset] = mslice.shape[i]             # <<<<<<<<<<<<<<
 *         mslice.strides[i + offset] = mslice.strides[i]
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 */
    (__pyx_v_mslice->shape[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->shape[__pyx_v_i]);

    /* "View.MemoryView":1348
 *     for i in range(ndim - 1, -1, -1):
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]             # <<<<<<<<<<<<<<
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 * 
 */
    (__pyx_v_mslice->strides[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->strides[__pyx_v_i]);

    /* "View.MemoryView":1349
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(offset):
 */
    (__pyx_v_mslice->suboffsets[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->suboffsets[__pyx_v_i]);
  }

  /* "View.MemoryView":1351
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 * 
 *     for i in range(offset):             # <<<<<<<<<<<<<<
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]
 */
  __pyx_t_1 = __pyx_v_offset;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1352
 * 
 *     for i in range(offset):
 *         mslice.shape[i] = 1             # <<<<<<<<<<<<<<
 *         mslice.strides[i] = mslice.strides[0]
 *         mslice.suboffsets[i] = -1
 */
    (__pyx_v_mslice->shape[__pyx_v_i]) = 1;

    /* "View.MemoryView":1353
 *     for i in range(offset):
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]             # <<<<<<<<<<<<<<
 *         mslice.suboffsets[i] = -1
 * 
 */
    (__pyx_v_mslice->strides[__pyx_v_i]) = (__pyx_v_mslice->strides[0]);

    /* "View.MemoryView":1354
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]
 *         mslice.suboffsets[i] = -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
    (__pyx_v_mslice->suboffsets[__pyx_v_i]) = -1L;
  }

  /* "View.MemoryView":1340
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 * cdef void broadcast_leading(__Pyx_memviewslice *mslice,             # <<<<<<<<<<<<<<
 *                             int ndim,
 *                             int ndim_other) nogil:
 */

  /* function exit code */
}

/* "View.MemoryView":1362
 * 
 * @cname('__pyx_memoryview_refcount_copying')
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object,             # <<<<<<<<<<<<<<
 *                            int ndim, bint inc) nogil:
 * 
 */

static void __pyx_memoryview_refcount_copying(__Pyx_memviewslice *__pyx_v_dst, int __pyx_v_dtype_is_object, int __pyx_v_ndim, int __pyx_v_inc) {
  int __pyx_t_1;

  /* "View.MemoryView":1366
 * 
 * 
 *     if dtype_is_object:             # <<<<<<<<<<<<<<
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,
 *                                            dst.strides, ndim, inc)
 */
  __pyx_t_1 = (__pyx_v_dtype_is_object != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1367
 * 
 *     if dtype_is_object:
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,             # <<<<<<<<<<<<<<
 *                                            dst.strides, ndim, inc)
 * 
 */
    __pyx_memoryview_refcount_objects_in_slice_with_gil(__pyx_v_dst->data, __pyx_v_dst->shape, __pyx_v_dst->strides, __pyx_v_ndim, __pyx_v_inc);

    /* "View.MemoryView":1366
 * 
 * 
 *     if dtype_is_object:             # <<<<<<<<<<<<<<
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,
 *                                            dst.strides, ndim, inc)
 */
  }

  /* "View.MemoryView":1362
 * 
 * @cname('__pyx_memoryview_refcount_copying')
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object,             # <<<<<<<<<<<<<<
 *                            int ndim, bint inc) nogil:
 * 
 */

  /* function exit code */
}

/* "View.MemoryView":1371
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')
 * cdef void refcount_objects_in_slice_with_gil(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 */

static void __pyx_memoryview_refcount_objects_in_slice_with_gil(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, int __pyx_v_inc) {
  __Pyx_RefNannyDeclarations
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("refcount_objects_in_slice_with_gil", 0);

  /* "View.MemoryView":1374
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 *     refcount_objects_in_slice(data, shape, strides, ndim, inc)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 */
  __pyx_memoryview_refcount_objects_in_slice(__pyx_v_data, __pyx_v_shape, __pyx_v_strides, __pyx_v_ndim, __pyx_v_inc);

  /* "View.MemoryView":1371
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')
 * cdef void refcount_objects_in_slice_with_gil(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "View.MemoryView":1377
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 * cdef void refcount_objects_in_slice(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                     Py_ssize_t *strides, int ndim, bint inc):
 *     cdef Py_ssize_t i
 */

static void __pyx_memoryview_refcount_objects_in_slice(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, int __pyx_v_inc) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("refcount_objects_in_slice", 0);

  /* "View.MemoryView":1381
 *     cdef Py_ssize_t i
 * 
 *     for i in range(shape[0]):             # <<<<<<<<<<<<<<
 *         if ndim == 1:
 *             if inc:
 */
  __pyx_t_1 = (__pyx_v_shape[0]);
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1382
 * 
 *     for i in range(shape[0]):
 *         if ndim == 1:             # <<<<<<<<<<<<<<
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])
 */
    __pyx_t_4 = ((__pyx_v_ndim == 1) != 0);
    if (__pyx_t_4) {

      /* "View.MemoryView":1383
 *     for i in range(shape[0]):
 *         if ndim == 1:
 *             if inc:             # <<<<<<<<<<<<<<
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 */
      __pyx_t_4 = (__pyx_v_inc != 0);
      if (__pyx_t_4) {

        /* "View.MemoryView":1384
 *         if ndim == 1:
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])             # <<<<<<<<<<<<<<
 *             else:
 *                 Py_DECREF((<PyObject **> data)[0])
 */
        Py_INCREF((((PyObject **)__pyx_v_data)[0]));

        /* "View.MemoryView":1383
 *     for i in range(shape[0]):
 *         if ndim == 1:
 *             if inc:             # <<<<<<<<<<<<<<
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 */
        goto __pyx_L6;
      }

      /* "View.MemoryView":1386
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 *                 Py_DECREF((<PyObject **> data)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,
 */
      /*else*/ {
        Py_DECREF((((PyObject **)__pyx_v_data)[0]));
      }
      __pyx_L6:;

      /* "View.MemoryView":1382
 * 
 *     for i in range(shape[0]):
 *         if ndim == 1:             # <<<<<<<<<<<<<<
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])
 */
      goto __pyx_L5;
    }

    /* "View.MemoryView":1388
 *                 Py_DECREF((<PyObject **> data)[0])
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,             # <<<<<<<<<<<<<<
 *                                       ndim - 1, inc)
 * 
 */
    /*else*/ {

      /* "View.MemoryView":1389
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,
 *                                       ndim - 1, inc)             # <<<<<<<<<<<<<<
 * 
 *         data += strides[0]
 */
      __pyx_memoryview_refcount_objects_in_slice(__pyx_v_data, (__pyx_v_shape + 1), (__pyx_v_strides + 1), (__pyx_v_ndim - 1), __pyx_v_inc);
    }
    __pyx_L5:;

    /* "View.MemoryView":1391
 *                                       ndim - 1, inc)
 * 
 *         data += strides[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_data = (__pyx_v_data + (__pyx_v_strides[0]));
  }

  /* "View.MemoryView":1377
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 * cdef void refcount_objects_in_slice(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                     Py_ssize_t *strides, int ndim, bint inc):
 *     cdef Py_ssize_t i
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":1397
 * 
 * @cname('__pyx_memoryview_slice_assign_scalar')
 * cdef void slice_assign_scalar(__Pyx_memviewslice *dst, int ndim,             # <<<<<<<<<<<<<<
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 */

static void __pyx_memoryview_slice_assign_scalar(__Pyx_memviewslice *__pyx_v_dst, int __pyx_v_ndim, size_t __pyx_v_itemsize, void *__pyx_v_item, int __pyx_v_dtype_is_object) {

  /* "View.MemoryView":1400
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 *     refcount_copying(dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,
 *                          itemsize, item)
 */
  __pyx_memoryview_refcount_copying(__pyx_v_dst, __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

  /* "View.MemoryView":1401
 *                               bint dtype_is_object) nogil:
 *     refcount_copying(dst, dtype_is_object, ndim, False)
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,             # <<<<<<<<<<<<<<
 *                          itemsize, item)
 *     refcount_copying(dst, dtype_is_object, ndim, True)
 */
  __pyx_memoryview__slice_assign_scalar(__pyx_v_dst->data, __pyx_v_dst->shape, __pyx_v_dst->strides, __pyx_v_ndim, __pyx_v_itemsize, __pyx_v_item);

  /* "View.MemoryView":1403
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,
 *                          itemsize, item)
 *     refcount_copying(dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_memoryview_refcount_copying(__pyx_v_dst, __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

  /* "View.MemoryView":1397
 * 
 * @cname('__pyx_memoryview_slice_assign_scalar')
 * cdef void slice_assign_scalar(__Pyx_memviewslice *dst, int ndim,             # <<<<<<<<<<<<<<
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 */

  /* function exit code */
}

/* "View.MemoryView":1407
 * 
 * @cname('__pyx_memoryview__slice_assign_scalar')
 * cdef void _slice_assign_scalar(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                               Py_ssize_t *strides, int ndim,
 *                               size_t itemsize, void *item) nogil:
 */

static void __pyx_memoryview__slice_assign_scalar(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, size_t __pyx_v_itemsize, void *__pyx_v_item) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_stride;
  Py_ssize_t __pyx_v_extent;
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;

  /* "View.MemoryView":1411
 *                               size_t itemsize, void *item) nogil:
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t extent = shape[0]
 * 
 */
  __pyx_v_stride = (__pyx_v_strides[0]);

  /* "View.MemoryView":1412
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]
 *     cdef Py_ssize_t extent = shape[0]             # <<<<<<<<<<<<<<
 * 
 *     if ndim == 1:
 */
  __pyx_v_extent = (__pyx_v_shape[0]);

  /* "View.MemoryView":1414
 *     cdef Py_ssize_t extent = shape[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 */
  __pyx_t_1 = ((__pyx_v_ndim == 1) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1415
 * 
 *     if ndim == 1:
 *         for i in range(extent):             # <<<<<<<<<<<<<<
 *             memcpy(data, item, itemsize)
 *             data += stride
 */
    __pyx_t_2 = __pyx_v_extent;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "View.MemoryView":1416
 *     if ndim == 1:
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)             # <<<<<<<<<<<<<<
 *             data += stride
 *     else:
 */
      (void)(memcpy(__pyx_v_data, __pyx_v_item, __pyx_v_itemsize));

      /* "View.MemoryView":1417
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 *             data += stride             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(extent):
 */
      __pyx_v_data = (__pyx_v_data + __pyx_v_stride);
    }

    /* "View.MemoryView":1414
 *     cdef Py_ssize_t extent = shape[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":1419
 *             data += stride
 *     else:
 *         for i in range(extent):             # <<<<<<<<<<<<<<
 *             _slice_assign_scalar(data, shape + 1, strides + 1,
 *                                 ndim - 1, itemsize, item)
 */
  /*else*/ {
    __pyx_t_2 = __pyx_v_extent;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "View.MemoryView":1420
 *     else:
 *         for i in range(extent):
 *             _slice_assign_scalar(data, shape + 1, strides + 1,             # <<<<<<<<<<<<<<
 *                                 ndim - 1, itemsize, item)
 *             data += stride
 */
      __pyx_memoryview__slice_assign_scalar(__pyx_v_data, (__pyx_v_shape + 1), (__pyx_v_strides + 1), (__pyx_v_ndim - 1), __pyx_v_itemsize, __pyx_v_item);

      /* "View.MemoryView":1422
 *             _slice_assign_scalar(data, shape + 1, strides + 1,
 *                                 ndim - 1, itemsize, item)
 *             data += stride             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_v_data = (__pyx_v_data + __pyx_v_stride);
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":1407
 * 
 * @cname('__pyx_memoryview__slice_assign_scalar')
 * cdef void _slice_assign_scalar(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                               Py_ssize_t *strides, int ndim,
 *                               size_t itemsize, void *item) nogil:
 */

  /* function exit code */
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_1__pyx_unpickle_Enum(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_15View_dot_MemoryView_1__pyx_unpickle_Enum = {"__pyx_unpickle_Enum", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_15View_dot_MemoryView_1__pyx_unpickle_Enum, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_15View_dot_MemoryView_1__pyx_unpickle_Enum(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Enum (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Enum", 1, 3, 3, 1); __PYX_ERR(3, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Enum", 1, 3, 3, 2); __PYX_ERR(3, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_Enum") < 0)) __PYX_ERR(3, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Enum", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(3, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("View.MemoryView.__pyx_unpickle_Enum", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_15View_dot_MemoryView___pyx_unpickle_Enum(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView___pyx_unpickle_Enum(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Enum", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xb068931:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb068931 = (name))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xb068931) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xb068931:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb068931 = (name))" % __pyx_checksum)
 *     __pyx_result = Enum.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xb068931:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb068931 = (name))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xb0, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(3, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xb068931:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb068931 = (name))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb068931 = (name))" % __pyx_checksum)
 *     __pyx_result = Enum.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_MemviewEnum_type), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb068931 = (name))" % __pyx_checksum)
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(3, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_unpickle_Enum__set_state(((struct __pyx_MemviewEnum_obj *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb068931 = (name))" % __pyx_checksum)
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 *     __pyx_result.name = __pyx_state[0]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.__pyx_unpickle_Enum", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_unpickle_Enum__set_state(struct __pyx_MemviewEnum_obj *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Enum__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 *     __pyx_result.name = __pyx_state[0]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(3, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->name);
  __Pyx_DECREF(__pyx_v___pyx_result->name);
  __pyx_v___pyx_result->name = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(3, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(3, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 1) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(3, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("View.MemoryView.__pyx_unpickle_Enum__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set__Index __pyx_vtable_12ms_deisotope_2_c_8peak_set__Index;

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_freelist_12ms_deisotope_2_c_8peak_set__Index[100000];
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set__Index = 0;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set__Index(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *p;
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set__Index > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index)) & ((t->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set__Index[--__pyx_freecount_12ms_deisotope_2_c_8peak_set__Index];
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index));
    (void) PyObject_INIT(o, t);
  } else {
    if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
      o = (*t->tp_alloc)(t, 0);
    } else {
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
    }
    if (unlikely(!o)) return 0;
  }
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)o);
  p->__pyx_vtab = __pyx_vtabptr_12ms_deisotope_2_c_8peak_set__Index;
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set__Index(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set__Index < 100000) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index)) & ((Py_TYPE(o)->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    __pyx_freelist_12ms_deisotope_2_c_8peak_set__Index[__pyx_freecount_12ms_deisotope_2_c_8peak_set__Index++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_6_Index_neutral_mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_6_Index_neutral_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_6_Index_mz(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_2mz_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_6_Index_mz(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_2mz_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set__Index[] = {
  {"clone", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_5clone, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_6_Index_4clone},
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_7__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_6_Index_6__reduce__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set__Index[] = {
  {(char *)"neutral_mass", __pyx_getprop_12ms_deisotope_2_c_8peak_set_6_Index_neutral_mass, __pyx_setprop_12ms_deisotope_2_c_8peak_set_6_Index_neutral_mass, (char *)"neutral_mass: 'size_t'", 0},
  {(char *)"mz", __pyx_getprop_12ms_deisotope_2_c_8peak_set_6_Index_mz, __pyx_setprop_12ms_deisotope_2_c_8peak_set_6_Index_mz, (char *)"mz: 'size_t'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set__Index = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set._Index", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set__Index, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "_Index(neutral_mass=0, mz=0)\n\n    Stores the ordered index of an object under sort by `mz` and\n    sort by `neutral_mass`\n\n    Attributes\n    ----------\n    mz : int\n        Index of this object (or its parent), ordered by mz\n    neutral_mass : int\n        Index of this object (or its parent), ordered by neutral mass\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_12ms_deisotope_2_c_8peak_set__Index, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_12ms_deisotope_2_c_8peak_set__Index, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set__Index, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_EnvelopePair __pyx_vtable_12ms_deisotope_2_c_8peak_set_EnvelopePair;

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_freelist_12ms_deisotope_2_c_8peak_set_EnvelopePair[1000000];
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set_EnvelopePair = 0;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_EnvelopePair(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *p;
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set_EnvelopePair > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair)) & ((t->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set_EnvelopePair[--__pyx_freecount_12ms_deisotope_2_c_8peak_set_EnvelopePair];
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair));
    (void) PyObject_INIT(o, t);
  } else {
    if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
      o = (*t->tp_alloc)(t, 0);
    } else {
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
    }
    if (unlikely(!o)) return 0;
  }
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)o);
  p->__pyx_vtab = __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_EnvelopePair;
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_EnvelopePair(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set_EnvelopePair < 1000000) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair)) & ((Py_TYPE(o)->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    __pyx_freelist_12ms_deisotope_2_c_8peak_set_EnvelopePair[__pyx_freecount_12ms_deisotope_2_c_8peak_set_EnvelopePair++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}
static PyObject *__pyx_sq_item_12ms_deisotope_2_c_8peak_set_EnvelopePair(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_mz(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_mz(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_intensity(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_intensity(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_EnvelopePair[] = {
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_8__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_12EnvelopePair_7__reduce__},
  {"_eq", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_10_eq, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9_eq},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set_EnvelopePair[] = {
  {(char *)"mz", __pyx_getprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_mz, __pyx_setprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_mz, (char *)"mz: 'double'", 0},
  {(char *)"intensity", __pyx_getprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_intensity, __pyx_setprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_intensity, (char *)"intensity: 'double'", 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_EnvelopePair = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_12ms_deisotope_2_c_8peak_set_EnvelopePair, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_EnvelopePair = {
  0, /*mp_length*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_3__getitem__, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.EnvelopePair", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_EnvelopePair, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_14__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_EnvelopePair, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_EnvelopePair, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "EnvelopePair(mz, intensity)", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_12__richcmp__, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_5__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_12ms_deisotope_2_c_8peak_set_EnvelopePair, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_12ms_deisotope_2_c_8peak_set_EnvelopePair, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_EnvelopePair, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_Envelope __pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope;

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_freelist_12ms_deisotope_2_c_8peak_set_Envelope[100000];
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set_Envelope = 0;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_Envelope(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *p;
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set_Envelope > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope)) & ((t->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set_Envelope[--__pyx_freecount_12ms_deisotope_2_c_8peak_set_Envelope];
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
      o = (*t->tp_alloc)(t, 0);
    } else {
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
    }
    if (unlikely(!o)) return 0;
  }
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)o);
  p->__pyx_vtab = __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_Envelope;
  p->pairs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_Envelope(PyObject *o) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->pairs);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set_Envelope < 100000) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope)) & ((Py_TYPE(o)->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    __pyx_freelist_12ms_deisotope_2_c_8peak_set_Envelope[__pyx_freecount_12ms_deisotope_2_c_8peak_set_Envelope++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_Envelope(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)o;
  if (p->pairs) {
    e = (*v)(p->pairs, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_Envelope(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)o;
  tmp = ((PyObject*)p->pairs);
  p->pairs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_12ms_deisotope_2_c_8peak_set_Envelope(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_8Envelope_pairs(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_8Envelope_pairs(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_3__set__(o, v);
  }
  else {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_Envelope[] = {
  {"_eq", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_11_eq, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_10_eq},
  {"clone", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_15clone, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_14clone},
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_17__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_16__reduce__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set_Envelope[] = {
  {(char *)"pairs", __pyx_getprop_12ms_deisotope_2_c_8peak_set_8Envelope_pairs, __pyx_setprop_12ms_deisotope_2_c_8peak_set_8Envelope_pairs, (char *)"pairs: tuple", 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_Envelope = {
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_9__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_12ms_deisotope_2_c_8peak_set_Envelope, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_Envelope = {
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_9__len__, /*mp_length*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_3__getitem__, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_Envelope = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.Envelope", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_Envelope, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_7__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_Envelope, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_Envelope, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Envelope(pairs)\n\n    Represents a sequence of (mz, intensity) pairs which store peak positions\n    that were matched. Since these peaks may later be mutated by subtraction,\n    this structure stores copies of the numbers\n\n    Supports the Sequence protocol, and is read-only.\n\n    Attributes\n    ----------\n    pairs : tuple of tuple of (float, float)\n        list of (mz, intensity) pairs\n    ", /*tp_doc*/
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_Envelope, /*tp_traverse*/
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_Envelope, /*tp_clear*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_13__richcmp__, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_12ms_deisotope_2_c_8peak_set_Envelope, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_12ms_deisotope_2_c_8peak_set_Envelope, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_Envelope, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *p;
  PyObject *o = __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase*)__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  p->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)Py_None); Py_INCREF(Py_None);
  p->envelope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)Py_None); Py_INCREF(Py_None);
  p->fit = ((struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(PyObject *o) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_index);
  Py_CLEAR(p->envelope);
  Py_CLEAR(p->fit);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase)) __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak);
}

static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)o;
  e = ((likely(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase)) ? ((__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_traverse) ? __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak)); if (e) return e;
  if (p->_index) {
    e = (*v)(((PyObject *)p->_index), a); if (e) return e;
  }
  if (p->envelope) {
    e = (*v)(((PyObject *)p->envelope), a); if (e) return e;
  }
  if (p->fit) {
    e = (*v)(((PyObject *)p->fit), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)o;
  if (likely(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase)) { if (__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_clear) __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak);
  tmp = ((PyObject*)p->_index);
  p->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->envelope);
  p->envelope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->fit);
  p->fit = ((struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_index(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_neutral_mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_neutral_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_signal_to_noise(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_signal_to_noise(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__index(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_3__set__(o, v);
  }
  else {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_5__del__(o);
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_full_width_at_half_max(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_full_width_at_half_max(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_charge(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_charge(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_a_to_a2_ratio(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_a_to_a2_ratio(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_most_abundant_mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_most_abundant_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_average_mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_average_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_score(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_score(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_envelope(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_envelope(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_3__set__(o, v);
  }
  else {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_5__del__(o);
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_fit(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_fit(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_3__set__(o, v);
  }
  else {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_5__del__(o);
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_chosen_for_msms(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_chosen_for_msms(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak[] = {
  {"clone", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5clone, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_4clone},
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_7__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6__reduce__},
  {"_eq", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_9_eq, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8_eq},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak[] = {
  {(char *)"index", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_index, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_index, (char *)0, 0},
  {(char *)"neutral_mass", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_neutral_mass, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_neutral_mass, (char *)"neutral_mass: 'double'", 0},
  {(char *)"signal_to_noise", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_signal_to_noise, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_signal_to_noise, (char *)"signal_to_noise: 'double'", 0},
  {(char *)"_index", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__index, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__index, (char *)"_index: ms_deisotope._c.peak_set._Index", 0},
  {(char *)"full_width_at_half_max", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_full_width_at_half_max, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_full_width_at_half_max, (char *)"full_width_at_half_max: 'double'", 0},
  {(char *)"charge", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_charge, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_charge, (char *)"charge: 'int'", 0},
  {(char *)"a_to_a2_ratio", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_a_to_a2_ratio, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_a_to_a2_ratio, (char *)"a_to_a2_ratio: 'double'", 0},
  {(char *)"most_abundant_mass", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_most_abundant_mass, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_most_abundant_mass, (char *)"most_abundant_mass: 'double'", 0},
  {(char *)"average_mass", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_average_mass, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_average_mass, (char *)"average_mass: 'double'", 0},
  {(char *)"score", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_score, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_score, (char *)"score: 'double'", 0},
  {(char *)"envelope", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_envelope, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_envelope, (char *)"envelope: ms_deisotope._c.peak_set.Envelope", 0},
  {(char *)"fit", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_fit, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_fit, (char *)"fit: ms_deisotope._c.scoring.IsotopicFitRecord", 0},
  {(char *)"chosen_for_msms", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_chosen_for_msms, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_chosen_for_msms, (char *)"chosen_for_msms: 'bool'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.DeconvolutedPeak", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3__hash__, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "DeconvolutedPeak(neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max, a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0, envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0)\n\n    Represent a single deconvoluted peak which represents an aggregated isotopic\n    pattern collapsed to its monoisotopic peak, with a known charge state\n\n    Attributes\n    ----------\n    a_to_a2_ratio : float\n        Ratio of intensities of A peak to A+2 peak\n    average_mass : float\n        The averaged neutral mass of the composition, the weighted average\n        of the envelope peaks.\n    charge : int\n        The signed charge state of the isotopic pattern\n    envelope : Envelope\n        The sequence of (mz, intensity) pairs which map to this peak\n    full_width_at_half_max : float\n        The averaged full width at half max of this isotopic pattern\n    index : _Index\n        The position of this peak in different orderings\n    intensity : float\n        The summed height of the peaks in the isotopic pattern this peak captures\n    most_abundant_mass : float\n        The neutral mass of the most abundant peak in the isotopic pattern this peak captures\n    mz : float\n        The mass-charge-ratio of the monoisotopic peak\n    neutral_mass : float\n        The neutral mass of the monoisotopic peak\n    score : float\n        An assigned value describing the quality of this peak's fit. The semantics of this score\n        depends upon the scoring function\n    signal_to_noise : float\n        The average signal-to-noise ratio of the peaks in the isotopic pattern this peak captures\n    ", /*tp_doc*/
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_traverse*/
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_clear*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_11__richcmp__, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak __pyx_vtable_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *p;
  PyObject *o = __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase*)__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak;
  return o;
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_drift_time(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_drift_time(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak[] = {
  {"clone", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_3clone, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_2clone},
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_5__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_4__reduce__},
  {"_eq", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_7_eq, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_6_eq},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak[] = {
  {(char *)"drift_time", __pyx_getprop_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_drift_time, __pyx_setprop_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_drift_time, (char *)"drift_time: 'double'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_9__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "IonMobilityDeconvolutedPeak(neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max, a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0, envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0, drift_time=0)", /*tp_doc*/
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_traverse*/
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *p;
  PyObject *o = __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase*)__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution;
  p->solution = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution(PyObject *o) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->solution);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(o);
}

static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)o;
  e = __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(o, v, a); if (e) return e;
  if (p->solution) {
    e = (*v)(p->solution, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)o;
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(o);
  tmp = ((PyObject*)p->solution);
  p->solution = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_solution(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_solution(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_3__set__(o, v);
  }
  else {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution[] = {
  {"clone", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_3clone, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_2clone},
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_5__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_4__reduce__},
  {"from_peak", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_10from_peak, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_9from_peak},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution[] = {
  {(char *)"solution", __pyx_getprop_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_solution, __pyx_setprop_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_solution, (char *)"solution: object", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.DeconvolutedPeakSolution", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13__repr__, /*tp_repr*/
  #else
  0, /*tp_repr*/
  #endif
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "DeconvolutedPeakSolution(solution, fit, *args, **kwargs)\n\n    Extends :class:`DeconvolutedPeak` to also include a reference to\n    the :class:`IsotopicFitRecord` instance it is derived from, and optionally an\n    object which the fit derives from.\n\n    Attributes\n    ----------\n    fit : IsotpicFitRecord\n        The isotopic pattern fit used to construct this peak\n    solution : object\n        Representation of the \"source\" of the isotopic fit\n    ", /*tp_doc*/
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, /*tp_traverse*/
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_7__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)o);
  p->__pyx_vtab = __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;
  p->peaks = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_mz_ordered = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyObject *o) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->peaks);
  Py_CLEAR(p->_mz_ordered);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)o;
  if (p->peaks) {
    e = (*v)(p->peaks, a); if (e) return e;
  }
  if (p->_mz_ordered) {
    e = (*v)(p->_mz_ordered, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)o;
  tmp = ((PyObject*)p->peaks);
  p->peaks = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_mz_ordered);
  p->_mz_ordered = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static PyObject *__pyx_tp_richcompare_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_13__eq__(o1, o2);
    }
    case Py_NE: {
      return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_15__ne__(o1, o2);
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_peaks(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_peaks(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_3__set__(o, v);
  }
  else {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_5__del__(o);
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__mz_ordered(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__mz_ordered(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_3__set__(o, v);
  }
  else {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_5__del__(o);
  }
}

static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_indexed(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_1__get__(o);
}

static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_indexed(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet[] = {
  {"reindex", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_3reindex, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_2reindex},
  {"clone", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_19clone, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_18clone},
  {"copy", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_21copy, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_20copy},
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_23__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_22__reduce__},
  {"has_peak", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_25has_peak, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_24has_peak},
  {"all_peaks_for", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_27all_peaks_for, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_26all_peaks_for},
  {"get_nearest_peak", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_29get_nearest_peak, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_28get_nearest_peak},
  {"between", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_31between, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_30between},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet[] = {
  {(char *)"peaks", __pyx_getprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_peaks, __pyx_setprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_peaks, (char *)"peaks: tuple", 0},
  {(char *)"_mz_ordered", __pyx_getprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__mz_ordered, __pyx_setprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__mz_ordered, (char *)"_mz_ordered: tuple", 0},
  {(char *)"indexed", __pyx_getprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_indexed, __pyx_setprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_indexed, (char *)"indexed: 'bool'", 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_DeconvolutedPeakSet = {
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_DeconvolutedPeakSet = {
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7__len__, /*mp_length*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem__, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.DeconvolutedPeakSet", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_9__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_DeconvolutedPeakSet, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_DeconvolutedPeakSet, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "DeconvolutedPeakSet(peaks)\n\n    Represents a collection of :class:`DeconvolutedPeak` instances under multiple orderings.\n\n    Attributes\n    ----------\n    peaks : tuple of DeconvolutedPeak\n        Collection of peaks ordered by `neutral_mass`\n    _mz_ordered: tuple of DeconvolutedPeak\n        Collection of peaks ordered by `mz`\n    ", /*tp_doc*/
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_traverse*/
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_clear*/
  __pyx_tp_richcompare_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_17__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *p;
  PyObject *o = __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet*)__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed;
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(o);
}

static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed[] = {
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_5__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_4__reduce__},
  {"__getstate__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_7__getstate__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_6__getstate__},
  {"__setstate__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_9__setstate__, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_8__setstate__},
  {"_release_buffers", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_11_release_buffers, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_10_release_buffers},
  {"set_interval_index_size", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_13set_interval_index_size, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_12set_interval_index_size},
  {"reindex", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_15reindex, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_14reindex},
  {"_test_interval", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_17_test_interval, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_16_test_interval},
  {"all_peaks_for", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_19all_peaks_for, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_18all_peaks_for},
  {"test_interval", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_21test_interval, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_20test_interval},
  {"find_interval_for", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_23find_interval_for, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_22find_interval_for},
  {"check_interval", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_25check_interval, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_24check_interval},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_9__repr__, /*tp_repr*/
  #else
  0, /*tp_repr*/
  #endif
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "DeconvolutedPeakSetIndexed(peaks)", /*tp_doc*/
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_traverse*/
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_17__iter__, /*tp_iter*/
  #else
  0, /*tp_iter*/
  #endif
  0, /*tp_iternext*/
  __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__[8];
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ = 0;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__)))) {
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__[--__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__];
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__(PyObject *o) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_self);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__)))) {
    __pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__[__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *)o;
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.__pyx_scope_struct____iter__", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__[8];
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ = 0;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__)))) {
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__[--__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__];
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__(PyObject *o) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_self);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__)))) {
    __pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__[__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *)o;
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *)o;
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.__pyx_scope_struct_1___repr__", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__, /*tp_traverse*/
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr[8];
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr = 0;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr)))) {
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr[--__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr];
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr(PyObject *o) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  Py_CLEAR(p->__pyx_v_t);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr)))) {
    __pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr[__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *)o;
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;
  }
  if (p->__pyx_v_t) {
    e = (*v)(p->__pyx_v_t, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.__pyx_scope_struct_2_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__[8];
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ = 0;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__)))) {
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__[--__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__];
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__(PyObject *o) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_self);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__)))) {
    __pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__[__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *)o;
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.__pyx_scope_struct_3___iter__", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__[8];
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ = 0;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__)))) {
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__[--__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__];
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__(PyObject *o) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_item);
  Py_CLEAR(p->__pyx_v_self);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__)))) {
    __pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__[__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *)o;
  if (p->__pyx_v_item) {
    e = (*v)(p->__pyx_v_item, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *)o;
  tmp = ((PyObject*)p->__pyx_v_item);
  p->__pyx_v_item = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.__pyx_scope_struct_4___getitem__", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__, /*tp_traverse*/
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr[8];
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr = 0;

static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr)))) {
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr[--__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr];
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr(PyObject *o) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  Py_CLEAR(p->__pyx_v_p);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr)))) {
    __pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr[__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *)o;
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;
  }
  if (p->__pyx_v_p) {
    e = (*v)(p->__pyx_v_p, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.__pyx_scope_struct_5_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_array __pyx_vtable_array;

static PyObject *__pyx_tp_new_array(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_array_obj *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_array_obj *)o);
  p->__pyx_vtab = __pyx_vtabptr_array;
  p->mode = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_format = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_array___cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_array(PyObject *o) {
  struct __pyx_array_obj *p = (struct __pyx_array_obj *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_array___dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->mode);
  Py_CLEAR(p->_format);
  (*Py_TYPE(o)->tp_free)(o);
}
static PyObject *__pyx_sq_item_array(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_array(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_array___setitem__(o, i, v);
  }
  else {
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}

static PyObject *__pyx_tp_getattro_array(PyObject *o, PyObject *n) {
  PyObject *v = __Pyx_PyObject_GenericGetAttr(o, n);
  if (!v && PyErr_ExceptionMatches(PyExc_AttributeError)) {
    PyErr_Clear();
    v = __pyx_array___getattr__(o, n);
  }
  return v;
}

static PyObject *__pyx_getprop___pyx_array_memview(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_5array_7memview_1__get__(o);
}

static PyMethodDef __pyx_methods_array[] = {
  {"__getattr__", (PyCFunction)__pyx_array___getattr__, METH_O|METH_COEXIST, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw___pyx_array_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw___pyx_array_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_array[] = {
  {(char *)"memview", __pyx_getprop___pyx_array_memview, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_array = {
  __pyx_array___len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_array, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_array = {
  __pyx_array___len__, /*mp_length*/
  __pyx_array___getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_array, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_array = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  __pyx_array_getbuffer, /*bf_getbuffer*/
  0, /*bf_releasebuffer*/
};

static PyTypeObject __pyx_type___pyx_array = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.array", /*tp_name*/
  sizeof(struct __pyx_array_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_array, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_array, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_array, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  __pyx_tp_getattro_array, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_array, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_array, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_array, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_array, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};

static PyObject *__pyx_tp_new_Enum(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_MemviewEnum_obj *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_MemviewEnum_obj *)o);
  p->name = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_Enum(PyObject *o) {
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->name);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_Enum(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;
  if (p->name) {
    e = (*v)(p->name, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_Enum(PyObject *o) {
  PyObject* tmp;
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;
  tmp = ((PyObject*)p->name);
  p->name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_Enum[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw___pyx_MemviewEnum_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw___pyx_MemviewEnum_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type___pyx_MemviewEnum = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.Enum", /*tp_name*/
  sizeof(struct __pyx_MemviewEnum_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_Enum, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_MemviewEnum___repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_Enum, /*tp_traverse*/
  __pyx_tp_clear_Enum, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_Enum, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_MemviewEnum___init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_Enum, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_memoryview __pyx_vtable_memoryview;

static PyObject *__pyx_tp_new_memoryview(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_memoryview_obj *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_memoryview_obj *)o);
  p->__pyx_vtab = __pyx_vtabptr_memoryview;
  p->obj = Py_None; Py_INCREF(Py_None);
  p->_size = Py_None; Py_INCREF(Py_None);
  p->_array_interface = Py_None; Py_INCREF(Py_None);
  p->view.obj = NULL;
  if (unlikely(__pyx_memoryview___cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_memoryview(PyObject *o) {
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_memoryview___dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->obj);
  Py_CLEAR(p->_size);
  Py_CLEAR(p->_array_interface);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_memoryview(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;
  if (p->obj) {
    e = (*v)(p->obj, a); if (e) return e;
  }
  if (p->_size) {
    e = (*v)(p->_size, a); if (e) return e;
  }
  if (p->_array_interface) {
    e = (*v)(p->_array_interface, a); if (e) return e;
  }
  if (p->view.obj) {
    e = (*v)(p->view.obj, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_memoryview(PyObject *o) {
  PyObject* tmp;
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;
  tmp = ((PyObject*)p->obj);
  p->obj = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_size);
  p->_size = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_array_interface);
  p->_array_interface = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  Py_CLEAR(p->view.obj);
  return 0;
}
static PyObject *__pyx_sq_item_memoryview(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_memoryview(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_memoryview___setitem__(o, i, v);
  }
  else {
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}

static PyObject *__pyx_getprop___pyx_memoryview_T(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_1T_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_base(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_4base_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_shape(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_5shape_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_strides(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_7strides_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_suboffsets(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_10suboffsets_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_ndim(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_4ndim_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_itemsize(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_8itemsize_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_nbytes(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_6nbytes_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_4size_1__get__(o);
}

static PyMethodDef __pyx_methods_memoryview[] = {
  {"is_c_contig", (PyCFunction)__pyx_memoryview_is_c_contig, METH_NOARGS, 0},
  {"is_f_contig", (PyCFunction)__pyx_memoryview_is_f_contig, METH_NOARGS, 0},
  {"copy", (PyCFunction)__pyx_memoryview_copy, METH_NOARGS, 0},
  {"copy_fortran", (PyCFunction)__pyx_memoryview_copy_fortran, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw___pyx_memoryview_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw___pyx_memoryview_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_memoryview[] = {
  {(char *)"T", __pyx_getprop___pyx_memoryview_T, 0, (char *)0, 0},
  {(char *)"base", __pyx_getprop___pyx_memoryview_base, 0, (char *)0, 0},
  {(char *)"shape", __pyx_getprop___pyx_memoryview_shape, 0, (char *)0, 0},
  {(char *)"strides", __pyx_getprop___pyx_memoryview_strides, 0, (char *)0, 0},
  {(char *)"suboffsets", __pyx_getprop___pyx_memoryview_suboffsets, 0, (char *)0, 0},
  {(char *)"ndim", __pyx_getprop___pyx_memoryview_ndim, 0, (char *)0, 0},
  {(char *)"itemsize", __pyx_getprop___pyx_memoryview_itemsize, 0, (char *)0, 0},
  {(char *)"nbytes", __pyx_getprop___pyx_memoryview_nbytes, 0, (char *)0, 0},
  {(char *)"size", __pyx_getprop___pyx_memoryview_size, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_memoryview = {
  __pyx_memoryview___len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_memoryview, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_memoryview = {
  __pyx_memoryview___len__, /*mp_length*/
  __pyx_memoryview___getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_memoryview, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_memoryview = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  __pyx_memoryview_getbuffer, /*bf_getbuffer*/
  0, /*bf_releasebuffer*/
};

static PyTypeObject __pyx_type___pyx_memoryview = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set.memoryview", /*tp_name*/
  sizeof(struct __pyx_memoryview_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_memoryview, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_memoryview___repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_memoryview, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_memoryview, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_memoryview___str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_memoryview, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_memoryview, /*tp_traverse*/
  __pyx_tp_clear_memoryview, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_memoryview, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_memoryview, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_memoryview, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct__memoryviewslice __pyx_vtable__memoryviewslice;

static PyObject *__pyx_tp_new__memoryviewslice(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_memoryviewslice_obj *p;
  PyObject *o = __pyx_tp_new_memoryview(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_memoryviewslice_obj *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_memoryview*)__pyx_vtabptr__memoryviewslice;
  p->from_object = Py_None; Py_INCREF(Py_None);
  p->from_slice.memview = NULL;
  return o;
}

static void __pyx_tp_dealloc__memoryviewslice(PyObject *o) {
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_memoryviewslice___dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->from_object);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_memoryview(o);
}

static int __pyx_tp_traverse__memoryviewslice(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;
  e = __pyx_tp_traverse_memoryview(o, v, a); if (e) return e;
  if (p->from_object) {
    e = (*v)(p->from_object, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear__memoryviewslice(PyObject *o) {
  PyObject* tmp;
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;
  __pyx_tp_clear_memoryview(o);
  tmp = ((PyObject*)p->from_object);
  p->from_object = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  __PYX_XDEC_MEMVIEW(&p->from_slice, 1);
  return 0;
}

static PyObject *__pyx_getprop___pyx_memoryviewslice_base(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_16_memoryviewslice_4base_1__get__(o);
}

static PyMethodDef __pyx_methods__memoryviewslice[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw___pyx_memoryviewslice_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw___pyx_memoryviewslice_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets__memoryviewslice[] = {
  {(char *)"base", __pyx_getprop___pyx_memoryviewslice_base, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type___pyx_memoryviewslice = {
  PyVarObject_HEAD_INIT(0, 0)
  "ms_deisotope._c.peak_set._memoryviewslice", /*tp_name*/
  sizeof(struct __pyx_memoryviewslice_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc__memoryviewslice, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_memoryview___repr__, /*tp_repr*/
  #else
  0, /*tp_repr*/
  #endif
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_memoryview___str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Internal class for passing memoryview slices to Python", /*tp_doc*/
  __pyx_tp_traverse__memoryviewslice, /*tp_traverse*/
  __pyx_tp_clear__memoryviewslice, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods__memoryviewslice, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets__memoryviewslice, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new__memoryviewslice, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {"_Index_reconstructor", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_1_Index_reconstructor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set__Index_reconstructor},
  {"_Index_to_bytes", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_3_Index_to_bytes, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_2_Index_to_bytes},
  {"_Index_from_bytes", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_5_Index_from_bytes, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_4_Index_from_bytes},
  {"Envelope_to_bytes", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_7Envelope_to_bytes, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_6Envelope_to_bytes},
  {"Envelope_from_bytes", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_9Envelope_from_bytes, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_from_bytes},
  {"Envelope_reconstructor", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_11Envelope_reconstructor, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_10Envelope_reconstructor},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_peak_set(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_peak_set},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "peak_set",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_kp_s_0_4f_0_2f, __pyx_k_0_4f_0_2f, sizeof(__pyx_k_0_4f_0_2f), 0, 0, 1, 0},
  {&__pyx_n_s_ASCII, __pyx_k_ASCII, sizeof(__pyx_k_ASCII), 0, 0, 1, 1},
  {&__pyx_n_s_AttributeError, __pyx_k_AttributeError, sizeof(__pyx_k_AttributeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Buffer_view_does_not_expose_stri, __pyx_k_Buffer_view_does_not_expose_stri, sizeof(__pyx_k_Buffer_view_does_not_expose_stri), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_only_create_a_buffer_that_is, __pyx_k_Can_only_create_a_buffer_that_is, sizeof(__pyx_k_Can_only_create_a_buffer_that_is), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_assign_to_read_only_memor, __pyx_k_Cannot_assign_to_read_only_memor, sizeof(__pyx_k_Cannot_assign_to_read_only_memor), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_create_writable_memory_vi, __pyx_k_Cannot_create_writable_memory_vi, sizeof(__pyx_k_Cannot_create_writable_memory_vi), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_index_with_type_s, __pyx_k_Cannot_index_with_type_s, sizeof(__pyx_k_Cannot_index_with_type_s), 0, 0, 1, 0},
  {&__pyx_n_s_DeconvolutedPeak, __pyx_k_DeconvolutedPeak, sizeof(__pyx_k_DeconvolutedPeak), 0, 0, 1, 1},
  {&__pyx_n_s_DeconvolutedPeakSet, __pyx_k_DeconvolutedPeakSet, sizeof(__pyx_k_DeconvolutedPeakSet), 0, 0, 1, 1},
  {&__pyx_n_s_DeconvolutedPeakSetIndexed, __pyx_k_DeconvolutedPeakSetIndexed, sizeof(__pyx_k_DeconvolutedPeakSetIndexed), 0, 0, 1, 1},
  {&__pyx_kp_s_DeconvolutedPeakSet_d_Peaks, __pyx_k_DeconvolutedPeakSet_d_Peaks, sizeof(__pyx_k_DeconvolutedPeakSet_d_Peaks), 0, 0, 1, 0},
  {&__pyx_n_s_DeconvolutedPeakSolution, __pyx_k_DeconvolutedPeakSolution, sizeof(__pyx_k_DeconvolutedPeakSolution), 0, 0, 1, 1},
  {&__pyx_n_s_DeconvolutedPeakSolution___iter, __pyx_k_DeconvolutedPeakSolution___iter, sizeof(__pyx_k_DeconvolutedPeakSolution___iter), 0, 0, 1, 1},
  {&__pyx_n_s_Ellipsis, __pyx_k_Ellipsis, sizeof(__pyx_k_Ellipsis), 0, 0, 1, 1},
  {&__pyx_kp_s_Empty_shape_tuple_for_cython_arr, __pyx_k_Empty_shape_tuple_for_cython_arr, sizeof(__pyx_k_Empty_shape_tuple_for_cython_arr), 0, 0, 1, 0},
  {&__pyx_n_s_Envelope, __pyx_k_Envelope, sizeof(__pyx_k_Envelope), 0, 0, 1, 1},
  {&__pyx_n_s_EnvelopePair, __pyx_k_EnvelopePair, sizeof(__pyx_k_EnvelopePair), 0, 0, 1, 1},
  {&__pyx_n_s_EnvelopePair___iter, __pyx_k_EnvelopePair___iter, sizeof(__pyx_k_EnvelopePair___iter), 0, 0, 1, 1},
  {&__pyx_n_s_Envelope_reconstructor, __pyx_k_Envelope_reconstructor, sizeof(__pyx_k_Envelope_reconstructor), 0, 0, 1, 1},
  {&__pyx_kp_s_Expected_at_least_d_argument_s_g, __pyx_k_Expected_at_least_d_argument_s_g, sizeof(__pyx_k_Expected_at_least_d_argument_s_g), 0, 0, 1, 0},
  {&__pyx_kp_s_Function_call_with_ambiguous_arg, __pyx_k_Function_call_with_ambiguous_arg, sizeof(__pyx_k_Function_call_with_ambiguous_arg), 0, 0, 1, 0},
  {&__pyx_n_s_INTERVAL_INDEX_SIZE, __pyx_k_INTERVAL_INDEX_SIZE, sizeof(__pyx_k_INTERVAL_INDEX_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xb0, __pyx_k_Incompatible_checksums_s_vs_0xb0, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xb0), 0, 0, 1, 0},
  {&__pyx_n_s_Index, __pyx_k_Index, sizeof(__pyx_k_Index), 0, 0, 1, 1},
  {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
  {&__pyx_n_s_Index_reconstructor, __pyx_k_Index_reconstructor, sizeof(__pyx_k_Index_reconstructor), 0, 0, 1, 1},
  {&__pyx_kp_s_Indirect_dimensions_not_supporte, __pyx_k_Indirect_dimensions_not_supporte, sizeof(__pyx_k_Indirect_dimensions_not_supporte), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_mode_expected_c_or_fortr, __pyx_k_Invalid_mode_expected_c_or_fortr, sizeof(__pyx_k_Invalid_mode_expected_c_or_fortr), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_shape_in_axis_d_d, __pyx_k_Invalid_shape_in_axis_d_d, sizeof(__pyx_k_Invalid_shape_in_axis_d_d), 0, 0, 1, 0},
  {&__pyx_n_s_IonMobilityDeconvolutedPeak, __pyx_k_IonMobilityDeconvolutedPeak, sizeof(__pyx_k_IonMobilityDeconvolutedPeak), 0, 0, 1, 1},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_s_MemoryView_of_r_at_0x_x, __pyx_k_MemoryView_of_r_at_0x_x, sizeof(__pyx_k_MemoryView_of_r_at_0x_x), 0, 0, 1, 0},
  {&__pyx_kp_s_MemoryView_of_r_object, __pyx_k_MemoryView_of_r_object, sizeof(__pyx_k_MemoryView_of_r_object), 0, 0, 1, 0},
  {&__pyx_kp_s_No_matching_signature_found, __pyx_k_No_matching_signature_found, sizeof(__pyx_k_No_matching_signature_found), 0, 0, 1, 0},
  {&__pyx_n_b_O, __pyx_k_O, sizeof(__pyx_k_O), 0, 0, 0, 1},
  {&__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_k_Out_of_bounds_on_buffer_access_a, sizeof(__pyx_k_Out_of_bounds_on_buffer_access_a), 0, 0, 1, 0},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Unable_to_convert_item_to_object, __pyx_k_Unable_to_convert_item_to_object, sizeof(__pyx_k_Unable_to_convert_item_to_object), 0, 0, 1, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_View_MemoryView, __pyx_k_View_MemoryView, sizeof(__pyx_k_View_MemoryView), 0, 0, 1, 1},
  {&__pyx_kp_s__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 1, 0},
  {&__pyx_kp_s__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 0, 1, 0},
  {&__pyx_kp_s__4, __pyx_k__4, sizeof(__pyx_k__4), 0, 0, 1, 0},
  {&__pyx_n_s_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 0, 1, 1},
  {&__pyx_n_s_a_to_a2_ratio, __pyx_k_a_to_a2_ratio, sizeof(__pyx_k_a_to_a2_ratio), 0, 0, 1, 1},
  {&__pyx_n_s_all_peaks_for, __pyx_k_all_peaks_for, sizeof(__pyx_k_all_peaks_for), 0, 0, 1, 1},
  {&__pyx_n_s_allocate_buffer, __pyx_k_allocate_buffer, sizeof(__pyx_k_allocate_buffer), 0, 0, 1, 1},
  {&__pyx_n_s_area, __pyx_k_area, sizeof(__pyx_k_area), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_array, __pyx_k_array, sizeof(__pyx_k_array), 0, 0, 1, 1},
  {&__pyx_n_s_attrgetter, __pyx_k_attrgetter, sizeof(__pyx_k_attrgetter), 0, 0, 1, 1},
  {&__pyx_n_s_average_mass, __pyx_k_average_mass, sizeof(__pyx_k_average_mass), 0, 0, 1, 1},
  {&__pyx_n_s_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 0, 1, 1},
  {&__pyx_n_s_base, __pyx_k_base, sizeof(__pyx_k_base), 0, 0, 1, 1},
  {&__pyx_n_s_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 0, 1, 1},
  {&__pyx_n_u_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 1, 0, 1},
  {&__pyx_n_s_center_value, __pyx_k_center_value, sizeof(__pyx_k_center_value), 0, 0, 1, 1},
  {&__pyx_n_s_charge, __pyx_k_charge, sizeof(__pyx_k_charge), 0, 0, 1, 1},
  {&__pyx_n_s_chosen_for_msms, __pyx_k_chosen_for_msms, sizeof(__pyx_k_chosen_for_msms), 0, 0, 1, 1},
  {&__pyx_n_s_class, __pyx_k_class, sizeof(__pyx_k_class), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_clone, __pyx_k_clone, sizeof(__pyx_k_clone), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_kp_s_contiguous_and_direct, __pyx_k_contiguous_and_direct, sizeof(__pyx_k_contiguous_and_direct), 0, 0, 1, 0},
  {&__pyx_kp_s_contiguous_and_indirect, __pyx_k_contiguous_and_indirect, sizeof(__pyx_k_contiguous_and_indirect), 0, 0, 1, 0},
  {&__pyx_n_s_convert, __pyx_k_convert, sizeof(__pyx_k_convert), 0, 0, 1, 1},
  {&__pyx_n_s_converted, __pyx_k_converted, sizeof(__pyx_k_converted), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_current_envelope, __pyx_k_current_envelope, sizeof(__pyx_k_current_envelope), 0, 0, 1, 1},
  {&__pyx_kp_s_d_d, __pyx_k_d_d, sizeof(__pyx_k_d_d), 0, 0, 1, 0},
  {&__pyx_n_s_decode_envelopes, __pyx_k_decode_envelopes, sizeof(__pyx_k_decode_envelopes), 0, 0, 1, 1},
  {&__pyx_n_s_defaults, __pyx_k_defaults, sizeof(__pyx_k_defaults), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_double, __pyx_k_double, sizeof(__pyx_k_double), 0, 0, 1, 1},
  {&__pyx_n_s_drift_time, __pyx_k_drift_time, sizeof(__pyx_k_drift_time), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_dtype_is_object, __pyx_k_dtype_is_object, sizeof(__pyx_k_dtype_is_object), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_envelope, __pyx_k_envelope, sizeof(__pyx_k_envelope), 0, 0, 1, 1},
  {&__pyx_n_s_envelope_list, __pyx_k_envelope_list, sizeof(__pyx_k_envelope_list), 0, 0, 1, 1},
  {&__pyx_n_s_envelope_obj, __pyx_k_envelope_obj, sizeof(__pyx_k_envelope_obj), 0, 0, 1, 1},
  {&__pyx_n_s_eq, __pyx_k_eq, sizeof(__pyx_k_eq), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_error_tolerance, __pyx_k_error_tolerance, sizeof(__pyx_k_error_tolerance), 0, 0, 1, 1},
  {&__pyx_n_s_fit, __pyx_k_fit, sizeof(__pyx_k_fit), 0, 0, 1, 1},
  {&__pyx_n_s_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 0, 1, 1},
  {&__pyx_n_s_float, __pyx_k_float, sizeof(__pyx_k_float), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_fortran, __pyx_k_fortran, sizeof(__pyx_k_fortran), 0, 0, 1, 1},
  {&__pyx_n_u_fortran, __pyx_k_fortran, sizeof(__pyx_k_fortran), 0, 1, 0, 1},
  {&__pyx_n_s_from_peak, __pyx_k_from_peak, sizeof(__pyx_k_from_peak), 0, 0, 1, 1},
  {&__pyx_n_s_full_width_at_half_max, __pyx_k_full_width_at_half_max, sizeof(__pyx_k_full_width_at_half_max), 0, 0, 1, 1},
  {&__pyx_n_s_genexpr, __pyx_k_genexpr, sizeof(__pyx_k_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_getitem___locals_genexpr, __pyx_k_getitem___locals_genexpr, sizeof(__pyx_k_getitem___locals_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_kp_s_got_differing_extents_in_dimensi, __pyx_k_got_differing_extents_in_dimensi, sizeof(__pyx_k_got_differing_extents_in_dimensi), 0, 0, 1, 0},
  {&__pyx_n_s_has_peak, __pyx_k_has_peak, sizeof(__pyx_k_has_peak), 0, 0, 1, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_indexed, __pyx_k_indexed, sizeof(__pyx_k_indexed), 0, 0, 1, 1},
  {&__pyx_n_s_inf, __pyx_k_inf, sizeof(__pyx_k_inf), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_init___locals_lambda, __pyx_k_init___locals_lambda, sizeof(__pyx_k_init___locals_lambda), 0, 0, 1, 1},
  {&__pyx_n_s_intensity, __pyx_k_intensity, sizeof(__pyx_k_intensity), 0, 0, 1, 1},
  {&__pyx_n_s_itemsize, __pyx_k_itemsize, sizeof(__pyx_k_itemsize), 0, 0, 1, 1},
  {&__pyx_kp_s_itemsize_0_for_cython_array, __pyx_k_itemsize_0_for_cython_array, sizeof(__pyx_k_itemsize_0_for_cython_array), 0, 0, 1, 0},
  {&__pyx_n_s_iter, __pyx_k_iter, sizeof(__pyx_k_iter), 0, 0, 1, 1},
  {&__pyx_n_s_join, __pyx_k_join, sizeof(__pyx_k_join), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_kind, __pyx_k_kind, sizeof(__pyx_k_kind), 0, 0, 1, 1},
  {&__pyx_n_s_kwargs, __pyx_k_kwargs, sizeof(__pyx_k_kwargs), 0, 0, 1, 1},
  {&__pyx_n_s_m1, __pyx_k_m1, sizeof(__pyx_k_m1), 0, 0, 1, 1},
  {&__pyx_n_s_m2, __pyx_k_m2, sizeof(__pyx_k_m2), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_map, __pyx_k_map, sizeof(__pyx_k_map), 0, 0, 1, 1},
  {&__pyx_n_s_memview, __pyx_k_memview, sizeof(__pyx_k_memview), 0, 0, 1, 1},
  {&__pyx_n_s_mode, __pyx_k_mode, sizeof(__pyx_k_mode), 0, 0, 1, 1},
  {&__pyx_n_s_most_abundant_mass, __pyx_k_most_abundant_mass, sizeof(__pyx_k_most_abundant_mass), 0, 0, 1, 1},
  {&__pyx_n_s_ms_deisotope__c_peak_set, __pyx_k_ms_deisotope__c_peak_set, sizeof(__pyx_k_ms_deisotope__c_peak_set), 0, 0, 1, 1},
  {&__pyx_kp_s_ms_deisotope__c_peak_set_pyx, __pyx_k_ms_deisotope__c_peak_set_pyx, sizeof(__pyx_k_ms_deisotope__c_peak_set_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_mz, __pyx_k_mz, sizeof(__pyx_k_mz), 0, 0, 1, 1},
  {&__pyx_n_s_n, __pyx_k_n, sizeof(__pyx_k_n), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_ndim, __pyx_k_ndim, sizeof(__pyx_k_ndim), 0, 0, 1, 1},
  {&__pyx_n_s_neutral_mass, __pyx_k_neutral_mass, sizeof(__pyx_k_neutral_mass), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_s_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 0, 1, 0},
  {&__pyx_kp_s_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 0, 1, 0},
  {&__pyx_n_s_obj, __pyx_k_obj, sizeof(__pyx_k_obj), 0, 0, 1, 1},
  {&__pyx_n_s_operator, __pyx_k_operator, sizeof(__pyx_k_operator), 0, 0, 1, 1},
  {&__pyx_n_s_pack, __pyx_k_pack, sizeof(__pyx_k_pack), 0, 0, 1, 1},
  {&__pyx_n_s_pairs, __pyx_k_pairs, sizeof(__pyx_k_pairs), 0, 0, 1, 1},
  {&__pyx_n_s_peak, __pyx_k_peak, sizeof(__pyx_k_peak), 0, 0, 1, 1},
  {&__pyx_n_s_peaks, __pyx_k_peaks, sizeof(__pyx_k_peaks), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_capi, __pyx_k_pyx_capi, sizeof(__pyx_k_pyx_capi), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_getbuffer, __pyx_k_pyx_getbuffer, sizeof(__pyx_k_pyx_getbuffer), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_Enum, __pyx_k_pyx_unpickle_Enum, sizeof(__pyx_k_pyx_unpickle_Enum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_reindex, __pyx_k_reindex, sizeof(__pyx_k_reindex), 0, 0, 1, 1},
  {&__pyx_n_s_release_buffers, __pyx_k_release_buffers, sizeof(__pyx_k_release_buffers), 0, 0, 1, 1},
  {&__pyx_n_s_repr___locals_genexpr, __pyx_k_repr___locals_genexpr, sizeof(__pyx_k_repr___locals_genexpr), 0, 0, 1, 1},
  {&__pyx_kp_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 0},
  {&__pyx_n_s_score, __pyx_k_score, sizeof(__pyx_k_score), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_kp_s_self___class_____name___a_to_a2, __pyx_k_self___class_____name___a_to_a2, sizeof(__pyx_k_self___class_____name___a_to_a2), 0, 0, 1, 0},
  {&__pyx_kp_s_self___class_____name___a_to_a2_2, __pyx_k_self___class_____name___a_to_a2_2, sizeof(__pyx_k_self___class_____name___a_to_a2_2), 0, 0, 1, 0},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_signal_to_noise, __pyx_k_signal_to_noise, sizeof(__pyx_k_signal_to_noise), 0, 0, 1, 1},
  {&__pyx_n_s_signatures, __pyx_k_signatures, sizeof(__pyx_k_signatures), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_solution, __pyx_k_solution, sizeof(__pyx_k_solution), 0, 0, 1, 1},
  {&__pyx_n_s_sorted, __pyx_k_sorted, sizeof(__pyx_k_sorted), 0, 0, 1, 1},
  {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_step, __pyx_k_step, sizeof(__pyx_k_step), 0, 0, 1, 1},
  {&__pyx_n_s_stop, __pyx_k_stop, sizeof(__pyx_k_stop), 0, 0, 1, 1},
  {&__pyx_kp_s_strided_and_direct, __pyx_k_strided_and_direct, sizeof(__pyx_k_strided_and_direct), 0, 0, 1, 0},
  {&__pyx_kp_s_strided_and_direct_or_indirect, __pyx_k_strided_and_direct_or_indirect, sizeof(__pyx_k_strided_and_direct_or_indirect), 0, 0, 1, 0},
  {&__pyx_kp_s_strided_and_indirect, __pyx_k_strided_and_indirect, sizeof(__pyx_k_strided_and_indirect), 0, 0, 1, 0},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_strip, __pyx_k_strip, sizeof(__pyx_k_strip), 0, 0, 1, 1},
  {&__pyx_n_s_struct, __pyx_k_struct, sizeof(__pyx_k_struct), 0, 0, 1, 1},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_tolerance, __pyx_k_tolerance, sizeof(__pyx_k_tolerance), 0, 0, 1, 1},
  {&__pyx_kp_s_unable_to_allocate_array_data, __pyx_k_unable_to_allocate_array_data, sizeof(__pyx_k_unable_to_allocate_array_data), 0, 0, 1, 0},
  {&__pyx_kp_s_unable_to_allocate_shape_and_str, __pyx_k_unable_to_allocate_shape_and_str, sizeof(__pyx_k_unable_to_allocate_shape_and_str), 0, 0, 1, 0},
  {&__pyx_n_s_unpack, __pyx_k_unpack, sizeof(__pyx_k_unpack), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_use_mz, __pyx_k_use_mz, sizeof(__pyx_k_use_mz), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 51, __pyx_L1_error)
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) __PYX_ERR(0, 141, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 190, __pyx_L1_error)
  __pyx_builtin_map = __Pyx_GetBuiltinName(__pyx_n_s_map); if (!__pyx_builtin_map) __PYX_ERR(0, 251, __pyx_L1_error)
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(0, 459, __pyx_L1_error)
  __pyx_builtin_sorted = __Pyx_GetBuiltinName(__pyx_n_s_sorted); if (!__pyx_builtin_sorted) __PYX_ERR(0, 572, __pyx_L1_error)
  __pyx_builtin_AttributeError = __Pyx_GetBuiltinName(__pyx_n_s_AttributeError); if (!__pyx_builtin_AttributeError) __PYX_ERR(0, 607, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(2, 884, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(3, 133, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(3, 148, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(3, 151, __pyx_L1_error)
  __pyx_builtin_Ellipsis = __Pyx_GetBuiltinName(__pyx_n_s_Ellipsis); if (!__pyx_builtin_Ellipsis) __PYX_ERR(3, 404, __pyx_L1_error)
  __pyx_builtin_id = __Pyx_GetBuiltinName(__pyx_n_s_id); if (!__pyx_builtin_id) __PYX_ERR(3, 613, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1106
 * @cython.boundscheck(False)
 * @cython.wraparound(False)
 * def decode_envelopes(np.ndarray[cython.floating, ndim=1, mode='c'] array):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list envelope_list, current_envelope
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "ms_deisotope/_c/peak_set.pyx":1157
 * 
 *     def __setstate__(self, d):
 *         if d.get("indexed", False):             # <<<<<<<<<<<<<<
 *             self.reindex()
 * 
 */
  __pyx_tuple__7 = PyTuple_Pack(2, __pyx_n_s_indexed, Py_False); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 1157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":884
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(2, 884, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "../../../../../tmp/pip-build-env-2qywma9d/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":890
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(2, 890, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "View.MemoryView":133
 * 
 *         if not self.ndim:
 *             raise ValueError("Empty shape tuple for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if itemsize <= 0:
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_Empty_shape_tuple_for_cython_arr); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(3, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "View.MemoryView":136
 * 
 *         if itemsize <= 0:
 *             raise ValueError("itemsize <= 0 for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(format, bytes):
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_itemsize_0_for_cython_array); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(3, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "View.MemoryView":148
 * 
 *         if not self._shape:
 *             raise MemoryError("unable to allocate shape and strides.")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_unable_to_allocate_shape_and_str); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(3, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "View.MemoryView":176
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")             # <<<<<<<<<<<<<<
 * 
 *             if self.dtype_is_object:
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_unable_to_allocate_array_data); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(3, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "View.MemoryView":192
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")             # <<<<<<<<<<<<<<
 *         info.buf = self.data
 *         info.len = self.len
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_Can_only_create_a_buffer_that_is); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(3, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(3, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "View.MemoryView":418
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:
 *             raise TypeError("Cannot assign to read-only memoryview")             # <<<<<<<<<<<<<<
 * 
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_Cannot_assign_to_read_only_memor); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(3, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "View.MemoryView":495
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")             # <<<<<<<<<<<<<<
 *         else:
 *             if len(self.view.format) == 1:
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s_Unable_to_convert_item_to_object); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(3, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "View.MemoryView":520
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_ND:
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_s_Cannot_create_writable_memory_vi); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(3, 520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "View.MemoryView":570
 *         if self.view.strides == NULL:
 * 
 *             raise ValueError("Buffer view does not expose strides")             # <<<<<<<<<<<<<<
 * 
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_s_Buffer_view_does_not_expose_stri); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(3, 570, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "View.MemoryView":577
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])
 */
  __pyx_tuple__21 = PyTuple_New(1); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(3, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_INCREF(__pyx_int_neg_1);
  __Pyx_GIVEREF(__pyx_int_neg_1);
  PyTuple_SET_ITEM(__pyx_tuple__21, 0, __pyx_int_neg_1);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(3, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "View.MemoryView":682
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))             # <<<<<<<<<<<<<<
 *                 seen_ellipsis = True
 *             else:
 */
  __pyx_slice__24 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__24)) __PYX_ERR(3, 682, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__24);
  __Pyx_GIVEREF(__pyx_slice__24);

  /* "View.MemoryView":703
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             raise ValueError("Indirect dimensions not supported")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_s_Indirect_dimensions_not_supporte); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(3, 703, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(3, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "ms_deisotope/_c/peak_set.pyx":41
 * cdef double epsilon = 1e-3
 * 
 * neutral_mass_getter = operator.attrgetter("neutral_mass")             # <<<<<<<<<<<<<<
 * mz_getter = operator.attrgetter('mz')
 * 
 */
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_n_s_neutral_mass); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "ms_deisotope/_c/peak_set.pyx":42
 * 
 * neutral_mass_getter = operator.attrgetter("neutral_mass")
 * mz_getter = operator.attrgetter('mz')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_n_s_mz); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "ms_deisotope/_c/peak_set.pyx":1095
 * 
 * 
 * def convert(self):             # <<<<<<<<<<<<<<
 *     return DeconvolutedPeak(
 *         self.neutral_mass, self.intensity, self.charge,
 */
  __pyx_tuple__30 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(0, 1095, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);
  __pyx_codeobj__31 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__30, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ms_deisotope__c_peak_set_pyx, __pyx_n_s_convert, 1095, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__31)) __PYX_ERR(0, 1095, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":1106
 * @cython.boundscheck(False)
 * @cython.wraparound(False)
 * def decode_envelopes(np.ndarray[cython.floating, ndim=1, mode='c'] array):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list envelope_list, current_envelope
 */
  __pyx_tuple__32 = PyTuple_Pack(9, __pyx_n_s_array, __pyx_n_s_envelope_list, __pyx_n_s_current_envelope, __pyx_n_s_converted, __pyx_n_s_envelope_obj, __pyx_n_s_i, __pyx_n_s_n, __pyx_n_s_a, __pyx_n_s_b); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);
  __pyx_codeobj__33 = (PyObject*)__Pyx_PyCode_New(1, 0, 9, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__32, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ms_deisotope__c_peak_set_pyx, __pyx_n_s_decode_envelopes, 1106, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__33)) __PYX_ERR(0, 1106, __pyx_L1_error)

  /* "View.MemoryView":286
 *         return self.name
 * 
 * cdef generic = Enum("<strided and direct or indirect>")             # <<<<<<<<<<<<<<
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")
 */
  __pyx_tuple__34 = PyTuple_Pack(1, __pyx_kp_s_strided_and_direct_or_indirect); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(3, 286, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);

  /* "View.MemoryView":287
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default             # <<<<<<<<<<<<<<
 * cdef indirect = Enum("<strided and indirect>")
 * 
 */
  __pyx_tuple__35 = PyTuple_Pack(1, __pyx_kp_s_strided_and_direct); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(3, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);

  /* "View.MemoryView":288
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__36 = PyTuple_Pack(1, __pyx_kp_s_strided_and_indirect); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(3, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);

  /* "View.MemoryView":291
 * 
 * 
 * cdef contiguous = Enum("<contiguous and direct>")             # <<<<<<<<<<<<<<
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")
 * 
 */
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_kp_s_contiguous_and_direct); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(3, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);

  /* "View.MemoryView":292
 * 
 * cdef contiguous = Enum("<contiguous and direct>")
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__38 = PyTuple_Pack(1, __pyx_kp_s_contiguous_and_indirect); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(3, 292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);

  /* "(tree fragment)":1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__39 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(3, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);
  __pyx_codeobj__40 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__39, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_Enum, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__40)) __PYX_ERR(3, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_1eneg_5 = PyFloat_FromDouble(1e-5); if (unlikely(!__pyx_float_1eneg_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_184977713 = PyInt_FromLong(184977713L); if (unlikely(!__pyx_int_184977713)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_12ms_deisotope_2_c_8peak_set_neutral_mass_getter = Py_None; Py_INCREF(Py_None);
  __pyx_v_12ms_deisotope_2_c_8peak_set_mz_getter = Py_None; Py_INCREF(Py_None);
  generic = Py_None; Py_INCREF(Py_None);
  strided = Py_None; Py_INCREF(Py_None);
  indirect = Py_None; Py_INCREF(Py_None);
  contiguous = Py_None; Py_INCREF(Py_None);
  indirect_contiguous = Py_None; Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  if (__Pyx_ExportVoidPtr(__pyx_n_s_INTERVAL_INDEX_SIZE, (void *)&__pyx_v_12ms_deisotope_2_c_8peak_set_INTERVAL_INDEX_SIZE, "size_t") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("_binary_search_interval", (void (*)(void))__pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval, "int (double *, double, double, size_t, size_t *, size_t *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set__Index = &__pyx_vtable_12ms_deisotope_2_c_8peak_set__Index;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set__Index._create = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *(*)(size_t, size_t))__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set__Index) < 0) __PYX_ERR(0, 93, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set__Index.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set__Index.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set__Index.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set__Index.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set__Index.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set__Index) < 0) __PYX_ERR(0, 93, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Index, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set__Index) < 0) __PYX_ERR(0, 93, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set__Index = &__pyx_type_12ms_deisotope_2_c_8peak_set__Index;
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_EnvelopePair = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_EnvelopePair;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_EnvelopePair._eq = (int (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__eq;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_EnvelopePair._create = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *(*)(double, double))__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair) < 0) __PYX_ERR(0, 129, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_EnvelopePair) < 0) __PYX_ERR(0, 129, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_EnvelopePair, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair) < 0) __PYX_ERR(0, 129, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair = &__pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair;
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_Envelope = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope.get_size = (size_t (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *))__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope.getitem = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, size_t))__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope._eq = (int (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__eq;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope.clone = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_clone;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope._create = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *(*)(PyObject *))__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_Envelope) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_Envelope.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_Envelope.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_Envelope.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_Envelope.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_Envelope.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_Envelope) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Envelope, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_Envelope) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope = &__pyx_type_12ms_deisotope_2_c_8peak_set_Envelope;
  __pyx_t_1 = PyImport_ImportModule("ms_peak_picker._c.peak_set"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_set", "PeakBase", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakBase = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakBase)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.__pyx_base = *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakBase;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.__pyx_base.clone = (struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_clone;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak._eq = (int (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__eq;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak._create_simple = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*)(double, double, int, double, double, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *))__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__create_simple;
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.tp_base = __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak) < 0) __PYX_ERR(0, 323, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak) < 0) __PYX_ERR(0, 323, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DeconvolutedPeak, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak) < 0) __PYX_ERR(0, 323, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak = &__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.__pyx_base = *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.__pyx_base.__pyx_base.clone = (struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_clone;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.__pyx_base._eq = (int (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak__eq;
  __pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.tp_base = __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak) < 0) __PYX_ERR(0, 455, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak) < 0) __PYX_ERR(0, 455, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_IonMobilityDeconvolutedPeak, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak) < 0) __PYX_ERR(0, 455, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak = &__pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak;
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.__pyx_base = *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.__pyx_base.__pyx_base.clone = (struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_clone;
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.tp_base = __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution) < 0) __PYX_ERR(0, 497, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution) < 0) __PYX_ERR(0, 497, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DeconvolutedPeakSolution, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution) < 0) __PYX_ERR(0, 497, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution = &__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution;
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.reindex = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_reindex;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.clone = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_clone;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.copy = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_copy;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet._has_peak = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak *__pyx_optional_args))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.has_peak = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak *__pyx_optional_args))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.all_peaks_for = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for *__pyx_optional_args))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet._get_nearest_peak = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, double *))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__get_nearest_peak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.get_size = (size_t (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_get_size;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.getitem = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, size_t))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_getitem;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.getslice = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, size_t, size_t))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_getslice;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet) < 0) __PYX_ERR(0, 544, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet) < 0) __PYX_ERR(0, 544, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DeconvolutedPeakSet, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet) < 0) __PYX_ERR(0, 544, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet = &__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.__pyx_base = *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.__pyx_base.reindex = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_reindex;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.__pyx_base._has_peak = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak *__pyx_optional_args))__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.__pyx_base.all_peaks_for = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for *__pyx_optional_args))__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed._build_index_arrays = (void (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *))__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__build_index_arrays;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed._interval_for = (int (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *, double, double, size_t *, size_t *))__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__interval_for;
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.tp_base = __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed) < 0) __PYX_ERR(0, 1139, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed) < 0) __PYX_ERR(0, 1139, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DeconvolutedPeakSetIndexed, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed) < 0) __PYX_ERR(0, 1139, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed = &__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__) < 0) __PYX_ERR(0, 143, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ = &__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__) < 0) __PYX_ERR(0, 259, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ = &__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr) < 0) __PYX_ERR(0, 260, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr = &__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__) < 0) __PYX_ERR(0, 529, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ = &__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__) < 0) __PYX_ERR(0, 599, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ = &__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr) < 0) __PYX_ERR(0, 601, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr = &__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr;
  __pyx_vtabptr_array = &__pyx_vtable_array;
  __pyx_vtable_array.get_memview = (PyObject *(*)(struct __pyx_array_obj *))__pyx_array_get_memview;
  if (PyType_Ready(&__pyx_type___pyx_array) < 0) __PYX_ERR(3, 105, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type___pyx_array.tp_print = 0;
  #endif
  if (__Pyx_SetVtable(__pyx_type___pyx_array.tp_dict, __pyx_vtabptr_array) < 0) __PYX_ERR(3, 105, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type___pyx_array) < 0) __PYX_ERR(3, 105, __pyx_L1_error)
  __pyx_array_type = &__pyx_type___pyx_array;
  if (PyType_Ready(&__pyx_type___pyx_MemviewEnum) < 0) __PYX_ERR(3, 279, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type___pyx_MemviewEnum.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type___pyx_MemviewEnum.tp_dictoffset && __pyx_type___pyx_MemviewEnum.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type___pyx_MemviewEnum.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type___pyx_MemviewEnum) < 0) __PYX_ERR(3, 279, __pyx_L1_error)
  __pyx_MemviewEnum_type = &__pyx_type___pyx_MemviewEnum;
  __pyx_vtabptr_memoryview = &__pyx_vtable_memoryview;
  __pyx_vtable_memoryview.get_item_pointer = (char *(*)(struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_get_item_pointer;
  __pyx_vtable_memoryview.is_slice = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_is_slice;
  __pyx_vtable_memoryview.setitem_slice_assignment = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *, PyObject *))__pyx_memoryview_setitem_slice_assignment;
  __pyx_vtable_memoryview.setitem_slice_assign_scalar = (PyObject *(*)(struct __pyx_memoryview_obj *, struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_setitem_slice_assign_scalar;
  __pyx_vtable_memoryview.setitem_indexed = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *, PyObject *))__pyx_memoryview_setitem_indexed;
  __pyx_vtable_memoryview.convert_item_to_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *))__pyx_memoryview_convert_item_to_object;
  __pyx_vtable_memoryview.assign_item_from_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *, PyObject *))__pyx_memoryview_assign_item_from_object;
  if (PyType_Ready(&__pyx_type___pyx_memoryview) < 0) __PYX_ERR(3, 330, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type___pyx_memoryview.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type___pyx_memoryview.tp_dictoffset && __pyx_type___pyx_memoryview.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type___pyx_memoryview.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type___pyx_memoryview.tp_dict, __pyx_vtabptr_memoryview) < 0) __PYX_ERR(3, 330, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type___pyx_memoryview) < 0) __PYX_ERR(3, 330, __pyx_L1_error)
  __pyx_memoryview_type = &__pyx_type___pyx_memoryview;
  __pyx_vtabptr__memoryviewslice = &__pyx_vtable__memoryviewslice;
  __pyx_vtable__memoryviewslice.__pyx_base = *__pyx_vtabptr_memoryview;
  __pyx_vtable__memoryviewslice.__pyx_base.convert_item_to_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *))__pyx_memoryviewslice_convert_item_to_object;
  __pyx_vtable__memoryviewslice.__pyx_base.assign_item_from_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *, PyObject *))__pyx_memoryviewslice_assign_item_from_object;
  __pyx_type___pyx_memoryviewslice.tp_base = __pyx_memoryview_type;
  if (PyType_Ready(&__pyx_type___pyx_memoryviewslice) < 0) __PYX_ERR(3, 965, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type___pyx_memoryviewslice.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type___pyx_memoryviewslice.tp_dictoffset && __pyx_type___pyx_memoryviewslice.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type___pyx_memoryviewslice.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type___pyx_memoryviewslice.tp_dict, __pyx_vtabptr__memoryviewslice) < 0) __PYX_ERR(3, 965, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type___pyx_memoryviewslice) < 0) __PYX_ERR(3, 965, __pyx_L1_error)
  __pyx_memoryviewslice_type = &__pyx_type___pyx_memoryviewslice;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("brainpy._c.composition"); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7brainpy_2_c_11composition_PyComposition = __Pyx_ImportType(__pyx_t_1, "brainpy._c.composition", "PyComposition", sizeof(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7brainpy_2_c_11composition_PyComposition) __PYX_ERR(4, 123, __pyx_L1_error)
  __pyx_vtabptr_7brainpy_2_c_11composition_PyComposition = (struct __pyx_vtabstruct_7brainpy_2_c_11composition_PyComposition*)__Pyx_GetVtable(__pyx_ptype_7brainpy_2_c_11composition_PyComposition->tp_dict); if (unlikely(!__pyx_vtabptr_7brainpy_2_c_11composition_PyComposition)) __PYX_ERR(4, 123, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("brainpy._c.isotopic_distribution"); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak = __Pyx_ImportType(__pyx_t_1, "brainpy._c.isotopic_distribution", "TheoreticalPeak", sizeof(struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak) __PYX_ERR(5, 63, __pyx_L1_error)
  __pyx_vtabptr_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak = (struct __pyx_vtabstruct_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak*)__Pyx_GetVtable(__pyx_ptype_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak->tp_dict); if (unlikely(!__pyx_vtabptr_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak)) __PYX_ERR(5, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_peak_picker._c.peak_set"); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14ms_peak_picker_2_c_8peak_set_FittedPeak = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_set", "FittedPeak", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_8peak_set_FittedPeak) __PYX_ERR(6, 12, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_FittedPeak = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_FittedPeak*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_FittedPeak->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_FittedPeak)) __PYX_ERR(6, 12, __pyx_L1_error)
  __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSet = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_set", "PeakSet", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSet) __PYX_ERR(6, 29, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSet = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSet->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSet)) __PYX_ERR(6, 29, __pyx_L1_error)
  __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_set", "PeakSetIndexed", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed) __PYX_ERR(6, 83, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed)) __PYX_ERR(6, 83, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(7, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(2, 199, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(2, 222, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(2, 226, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(2, 238, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(2, 764, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_peak_picker._c.peak_index"); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14ms_peak_picker_2_c_10peak_index_PeakIndex = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_index", "PeakIndex", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_10peak_index_PeakIndex) __PYX_ERR(8, 5, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_10peak_index_PeakIndex = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_10peak_index_PeakIndex*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_10peak_index_PeakIndex->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_10peak_index_PeakIndex)) __PYX_ERR(8, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.averagine"); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_12ms_deisotope_2_c_9averagine_Averagine = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.averagine", "Averagine", sizeof(struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_9averagine_Averagine) __PYX_ERR(9, 12, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_9averagine_Averagine = (struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_Averagine*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_9averagine_Averagine->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_9averagine_Averagine)) __PYX_ERR(9, 12, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.averagine", "TheoreticalIsotopicPattern", sizeof(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern) __PYX_ERR(9, 23, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern = (struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern)) __PYX_ERR(9, 23, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_9averagine_AveragineCache = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.averagine", "AveragineCache", sizeof(struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_9averagine_AveragineCache) __PYX_ERR(9, 79, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_9averagine_AveragineCache = (struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_AveragineCache*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_9averagine_AveragineCache->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_9averagine_AveragineCache)) __PYX_ERR(9, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.deconvoluter_base"); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.deconvoluter_base", "DeconvoluterBase", sizeof(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase) __PYX_ERR(10, 25, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase = (struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase)) __PYX_ERR(10, 25, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.deconvoluter_base", "AveragineDeconvoluterBase", sizeof(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase) __PYX_ERR(10, 57, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase = (struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase)) __PYX_ERR(10, 57, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.deconvoluter_base", "MultiAveragineDeconvoluterBase", sizeof(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase) __PYX_ERR(10, 69, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase = (struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase)) __PYX_ERR(10, 69, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.deconvoluter_base", "ChargeIterator", sizeof(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator) __PYX_ERR(10, 91, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator = (struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator)) __PYX_ERR(10, 91, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.scoring"); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "IsotopicFitRecord", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord) __PYX_ERR(11, 16, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_IsotopicFitRecord = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitRecord*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_IsotopicFitRecord)) __PYX_ERR(11, 16, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_FitSelectorBase = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "FitSelectorBase", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_FitSelectorBase) __PYX_ERR(11, 38, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_FitSelectorBase = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FitSelectorBase*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_FitSelectorBase->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_FitSelectorBase)) __PYX_ERR(11, 38, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitterBase = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "IsotopicFitterBase", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitterBase) __PYX_ERR(11, 50, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_IsotopicFitterBase = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitterBase->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_IsotopicFitterBase)) __PYX_ERR(11, 50, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_LeastSquaresFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "LeastSquaresFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_LeastSquaresFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_LeastSquaresFitter) __PYX_ERR(11, 62, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_LeastSquaresFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_LeastSquaresFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_LeastSquaresFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_LeastSquaresFitter)) __PYX_ERR(11, 62, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_GTestFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "GTestFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_GTestFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_GTestFitter) __PYX_ERR(11, 66, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_GTestFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_GTestFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_GTestFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_GTestFitter)) __PYX_ERR(11, 66, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledGTestFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "ScaledGTestFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledGTestFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledGTestFitter) __PYX_ERR(11, 69, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_ScaledGTestFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_ScaledGTestFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledGTestFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_ScaledGTestFitter)) __PYX_ERR(11, 69, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_MSDeconVFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "MSDeconVFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_MSDeconVFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_MSDeconVFitter) __PYX_ERR(11, 72, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_MSDeconVFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_MSDeconVFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_MSDeconVFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_MSDeconVFitter)) __PYX_ERR(11, 72, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "PenalizedMSDeconVFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter) __PYX_ERR(11, 78, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter)) __PYX_ERR(11, 78, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_FunctionScorer = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "FunctionScorer", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FunctionScorer), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_FunctionScorer) __PYX_ERR(11, 84, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_FunctionScorer = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FunctionScorer*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_FunctionScorer->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_FunctionScorer)) __PYX_ERR(11, 84, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_InterferenceDetection = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "InterferenceDetection", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_InterferenceDetection), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_InterferenceDetection) __PYX_ERR(11, 89, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_InterferenceDetection = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_InterferenceDetection*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_InterferenceDetection->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_InterferenceDetection)) __PYX_ERR(11, 89, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_DistinctPatternFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "DistinctPatternFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_DistinctPatternFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_DistinctPatternFitter) __PYX_ERR(11, 95, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_DistinctPatternFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_DistinctPatternFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_DistinctPatternFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_DistinctPatternFitter)) __PYX_ERR(11, 95, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "ScaledPenalizedMSDeconvFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter) __PYX_ERR(11, 102, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter)) __PYX_ERR(11, 102, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_DotProductFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "DotProductFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_DotProductFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_DotProductFitter) __PYX_ERR(11, 112, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_DotProductFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_DotProductFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_DotProductFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_DotProductFitter)) __PYX_ERR(11, 112, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("brainpy._c.composition"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "nist_mass", (void **)&__pyx_vp_7brainpy_2_c_11composition_nist_mass, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "PROTON", (void **)&__pyx_vp_7brainpy_2_c_11composition_PROTON, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_ElementTable", (void **)&__pyx_vp_7brainpy_2_c_11composition__ElementTable, "struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("brainpy._c.isotopic_constants"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "DEFAULT_ISOTOPIC_CONSTANTS_SIZE", (void **)&__pyx_vp_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE, "size_t") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_peak_picker._c.peak_set"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "INTERVAL_INDEX_SIZE", (void **)&__pyx_vp_14ms_peak_picker_2_c_8peak_set_INTERVAL_INDEX_SIZE, "size_t") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.averagine"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "PROTON", (void **)&__pyx_vp_12ms_deisotope_2_c_9averagine_PROTON, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.scoring"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "INFINITY", (void **)&__pyx_vp_12ms_deisotope_2_c_7scoring_INFINITY, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.averagine"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction(__pyx_t_1, "mass_charge_ratio", (void (**)(void))&__pyx_f_12ms_deisotope_2_c_9averagine_mass_charge_ratio, "double (double, int, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_mass_charge_ratio *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initpeak_set(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initpeak_set(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_peak_set(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_peak_set(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_peak_set(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  double __pyx_t_4;
  static PyThread_type_lock __pyx_t_5[8];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'peak_set' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_peak_set(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("peak_set", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_ms_deisotope___c__peak_set) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name_2, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "ms_deisotope._c.peak_set")) {
      if (unlikely(PyDict_SetItemString(modules, "ms_deisotope._c.peak_set", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  if (unlikely(__Pyx_modinit_variable_export_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_function_export_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_variable_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_function_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "ms_deisotope/_c/peak_set.pyx":3
 * # cython: embedsignature=True
 * 
 * import operator             # <<<<<<<<<<<<<<
 * 
 * cimport cython
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_operator, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_operator, __pyx_t_1) < 0) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":27
 * cimport numpy as np
 * 
 * np.import_array()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __pyx_f_5numpy_import_array(); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 27, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":39
 *     object mz_getter
 * 
 * cdef double epsilon = 1e-3             # <<<<<<<<<<<<<<
 * 
 * neutral_mass_getter = operator.attrgetter("neutral_mass")
 */
  __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon = 1e-3;

  /* "ms_deisotope/_c/peak_set.pyx":41
 * cdef double epsilon = 1e-3
 * 
 * neutral_mass_getter = operator.attrgetter("neutral_mass")             # <<<<<<<<<<<<<<
 * mz_getter = operator.attrgetter('mz')
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_operator); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_attrgetter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__28, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_XGOTREF(__pyx_v_12ms_deisotope_2_c_8peak_set_neutral_mass_getter);
  __Pyx_DECREF_SET(__pyx_v_12ms_deisotope_2_c_8peak_set_neutral_mass_getter, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":42
 * 
 * neutral_mass_getter = operator.attrgetter("neutral_mass")
 * mz_getter = operator.attrgetter('mz')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_operator); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_attrgetter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_XGOTREF(__pyx_v_12ms_deisotope_2_c_8peak_set_mz_getter);
  __Pyx_DECREF_SET(__pyx_v_12ms_deisotope_2_c_8peak_set_mz_getter, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":535
 * 
 *     @classmethod
 *     def from_peak(cls, peak, solution):             # <<<<<<<<<<<<<<
 *         inst = cls(
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,
 */
  __Pyx_GetNameInClass(__pyx_t_1, (PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, __pyx_n_s_from_peak); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "ms_deisotope/_c/peak_set.pyx":534
 *         yield self.fit
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def from_peak(cls, peak, solution):
 *         inst = cls(
 */
  __pyx_t_3 = __Pyx_Method_ClassMethod(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution->tp_dict, __pyx_n_s_from_peak, __pyx_t_3) < 0) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution);

  /* "ms_deisotope/_c/peak_set.pyx":809
 * 
 * cdef double INF
 * INF = float('inf')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_AsDouble(__pyx_n_s_inf); if (unlikely(__pyx_t_4 == ((double)((double)-1)) && PyErr_Occurred())) __PYX_ERR(0, 809, __pyx_L1_error)
  __pyx_v_12ms_deisotope_2_c_8peak_set_INF = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":1095
 * 
 * 
 * def convert(self):             # <<<<<<<<<<<<<<
 *     return DeconvolutedPeak(
 *         self.neutral_mass, self.intensity, self.charge,
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_12ms_deisotope_2_c_8peak_set_13convert, NULL, __pyx_n_s_ms_deisotope__c_peak_set); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1095, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_convert, __pyx_t_3) < 0) __PYX_ERR(0, 1095, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1106
 * @cython.boundscheck(False)
 * @cython.wraparound(False)
 * def decode_envelopes(np.ndarray[cython.floating, ndim=1, mode='c'] array):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list envelope_list, current_envelope
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __pyx_FusedFunction_New(&__pyx_fuse_0__pyx_mdef_12ms_deisotope_2_c_8peak_set_17decode_envelopes, 0, __pyx_n_s_decode_envelopes, NULL, __pyx_n_s_ms_deisotope__c_peak_set, __pyx_d, ((PyObject *)__pyx_codeobj__33)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_float, __pyx_t_1) < 0) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_FusedFunction_New(&__pyx_fuse_1__pyx_mdef_12ms_deisotope_2_c_8peak_set_19decode_envelopes, 0, __pyx_n_s_decode_envelopes, NULL, __pyx_n_s_ms_deisotope__c_peak_set, __pyx_d, ((PyObject *)__pyx_codeobj__33)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_double, __pyx_t_1) < 0) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_FusedFunction_New(&__pyx_mdef_12ms_deisotope_2_c_8peak_set_15decode_envelopes, 0, __pyx_n_s_decode_envelopes, NULL, __pyx_n_s_ms_deisotope__c_peak_set, __pyx_d, ((PyObject *)__pyx_codeobj__33)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_1)->__signatures__ = __pyx_t_3;
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_decode_envelopes, __pyx_t_1) < 0) __PYX_ERR(0, 1106, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1137
 * 
 * 
 * INTERVAL_INDEX_SIZE = 0             # <<<<<<<<<<<<<<
 * 
 * cdef class DeconvolutedPeakSetIndexed(DeconvolutedPeakSet):
 */
  __pyx_v_12ms_deisotope_2_c_8peak_set_INTERVAL_INDEX_SIZE = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1
 * # cython: embedsignature=True             # <<<<<<<<<<<<<<
 * 
 * import operator
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":209
 *         info.obj = self
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(array self):
 */
  __pyx_t_1 = __pyx_capsule_create(((void *)(&__pyx_array_getbuffer)), ((char *)"getbuffer(obj, view, flags)")); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_array_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_1) < 0) __PYX_ERR(3, 209, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_array_type);

  /* "View.MemoryView":286
 *         return self.name
 * 
 * cdef generic = Enum("<strided and direct or indirect>")             # <<<<<<<<<<<<<<
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__34, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 286, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(generic);
  __Pyx_DECREF_SET(generic, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":287
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default             # <<<<<<<<<<<<<<
 * cdef indirect = Enum("<strided and indirect>")
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__35, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(strided);
  __Pyx_DECREF_SET(strided, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":288
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__36, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(indirect);
  __Pyx_DECREF_SET(indirect, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":291
 * 
 * 
 * cdef contiguous = Enum("<contiguous and direct>")             # <<<<<<<<<<<<<<
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__37, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(contiguous);
  __Pyx_DECREF_SET(contiguous, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":292
 * 
 * cdef contiguous = Enum("<contiguous and direct>")
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__38, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(indirect_contiguous);
  __Pyx_DECREF_SET(indirect_contiguous, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":316
 * 
 * DEF THREAD_LOCKS_PREALLOCATED = 8
 * cdef int __pyx_memoryview_thread_locks_used = 0             # <<<<<<<<<<<<<<
 * cdef PyThread_type_lock[THREAD_LOCKS_PREALLOCATED] __pyx_memoryview_thread_locks = [
 *     PyThread_allocate_lock(),
 */
  __pyx_memoryview_thread_locks_used = 0;

  /* "View.MemoryView":317
 * DEF THREAD_LOCKS_PREALLOCATED = 8
 * cdef int __pyx_memoryview_thread_locks_used = 0
 * cdef PyThread_type_lock[THREAD_LOCKS_PREALLOCATED] __pyx_memoryview_thread_locks = [             # <<<<<<<<<<<<<<
 *     PyThread_allocate_lock(),
 *     PyThread_allocate_lock(),
 */
  __pyx_t_5[0] = PyThread_allocate_lock();
  __pyx_t_5[1] = PyThread_allocate_lock();
  __pyx_t_5[2] = PyThread_allocate_lock();
  __pyx_t_5[3] = PyThread_allocate_lock();
  __pyx_t_5[4] = PyThread_allocate_lock();
  __pyx_t_5[5] = PyThread_allocate_lock();
  __pyx_t_5[6] = PyThread_allocate_lock();
  __pyx_t_5[7] = PyThread_allocate_lock();
  memcpy(&(__pyx_memoryview_thread_locks[0]), __pyx_t_5, sizeof(__pyx_memoryview_thread_locks[0]) * (8));

  /* "View.MemoryView":549
 *         info.obj = self
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_capsule_create(((void *)(&__pyx_memoryview_getbuffer)), ((char *)"getbuffer(obj, view, flags)")); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_memoryview_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_1) < 0) __PYX_ERR(3, 549, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_memoryview_type);

  /* "View.MemoryView":995
 *         return self.from_object
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_capsule_create(((void *)(&__pyx_memoryview_getbuffer)), ((char *)"getbuffer(obj, view, flags)")); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 995, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_memoryviewslice_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_1) < 0) __PYX_ERR(3, 995, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_memoryviewslice_type);

  /* "(tree fragment)":1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_15View_dot_MemoryView_1__pyx_unpickle_Enum, NULL, __pyx_n_s_View_MemoryView); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_Enum, __pyx_t_1) < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init ms_deisotope._c.peak_set", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init ms_deisotope._c.peak_set");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
#include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
#if PY_MAJOR_VERSION < 3
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
#endif
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* None */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* StringJoin */
#if !CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values) {
    return PyObject_CallMethodObjArgs(sep, __pyx_n_s_join, values, NULL);
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* py_abs */
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject *__Pyx_PyLong_AbsNeg(PyObject *n) {
    if (likely(Py_SIZE(n) == -1)) {
        return PyLong_FromLong(((PyLongObject*)n)->ob_digit[0]);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    {
        PyObject *copy = _PyLong_Copy((PyLongObject*)n);
        if (likely(copy)) {
            __Pyx_SET_SIZE(copy, -Py_SIZE(copy));
        }
        return copy;
    }
#else
    return PyNumber_Negative(n);
#endif
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* pyobject_as_double */
static double __Pyx__PyObject_AsDouble(PyObject* obj) {
    PyObject* float_value;
#if !CYTHON_USE_TYPE_SLOTS
    float_value = PyNumber_Float(obj);  if ((0)) goto bad;
#else
    PyNumberMethods *nb = Py_TYPE(obj)->tp_as_number;
    if (likely(nb) && likely(nb->nb_float)) {
        float_value = nb->nb_float(obj);
        if (likely(float_value) && unlikely(!PyFloat_Check(float_value))) {
            PyErr_Format(PyExc_TypeError,
                "__float__ returned non-float (type %.200s)",
                Py_TYPE(float_value)->tp_name);
            Py_DECREF(float_value);
            goto bad;
        }
    } else if (PyUnicode_CheckExact(obj) || PyBytes_CheckExact(obj)) {
#if PY_MAJOR_VERSION >= 3
        float_value = PyFloat_FromString(obj);
#else
        float_value = PyFloat_FromString(obj, 0);
#endif
    } else {
        PyObject* args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        PyTuple_SET_ITEM(args, 0, obj);
        float_value = PyObject_Call((PyObject*)&PyFloat_Type, args, 0);
        PyTuple_SET_ITEM(args, 0, 0);
        Py_DECREF(args);
    }
#endif
    if (likely(float_value)) {
        double value = PyFloat_AS_DOUBLE(float_value);
        Py_DECREF(float_value);
        return value;
    }
bad:
    return (double)-1;
}

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* UnicodeAsUCS4 */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject* x) {
   Py_ssize_t length;
   #if CYTHON_PEP393_ENABLED
   length = PyUnicode_GET_LENGTH(x);
   if (likely(length == 1)) {
       return PyUnicode_READ_CHAR(x, 0);
   }
   #else
   length = PyUnicode_GET_SIZE(x);
   if (likely(length == 1)) {
       return PyUnicode_AS_UNICODE(x)[0];
   }
   #if Py_UNICODE_SIZE == 2
   else if (PyUnicode_GET_SIZE(x) == 2) {
       Py_UCS4 high_val = PyUnicode_AS_UNICODE(x)[0];
       if (high_val >= 0xD800 && high_val <= 0xDBFF) {
           Py_UCS4 low_val = PyUnicode_AS_UNICODE(x)[1];
           if (low_val >= 0xDC00 && low_val <= 0xDFFF) {
               return 0x10000 + (((high_val & ((1<<10)-1)) << 10) | (low_val & ((1<<10)-1)));
           }
       }
   }
   #endif
   #endif
   PyErr_Format(PyExc_ValueError,
                "only single character unicode strings can be converted to Py_UCS4, "
                "got length %" CYTHON_FORMAT_SSIZE_T "d", length);
   return (Py_UCS4)-1;
}

/* object_ord */
static long __Pyx__PyObject_Ord(PyObject* c) {
    Py_ssize_t size;
    if (PyBytes_Check(c)) {
        size = PyBytes_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyBytes_AS_STRING(c)[0];
        }
#if PY_MAJOR_VERSION < 3
    } else if (PyUnicode_Check(c)) {
        return (long)__Pyx_PyUnicode_AsPy_UCS4(c);
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    } else if (PyByteArray_Check(c)) {
        size = PyByteArray_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyByteArray_AS_STRING(c)[0];
        }
#endif
    } else {
        PyErr_Format(PyExc_TypeError,
            "ord() expected string of length 1, but %.200s found", Py_TYPE(c)->tp_name);
        return (long)(Py_UCS4)-1;
    }
    PyErr_Format(PyExc_TypeError,
        "ord() expected a character, but string of length %zd found", size);
    return (long)(Py_UCS4)-1;
}

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o)))
#else
    if (is_list || PySequence_Check(o))
#endif
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

/* UnpackTuple2 */
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_COMPILING_IN_PYPY
    value1 = PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_COMPILING_IN_PYPY
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = Py_TYPE(iter)->tp_iternext;
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#elif PY_MAJOR_VERSION >= 3
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* IsLittleEndian */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void)
{
  union {
    uint32_t u32;
    uint8_t u8[4];
  } S;
  S.u32 = 0x01020304;
  return S.u8[0] == 4;
}

/* BufferFormatCheck */
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t <= '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case '?': return "'bool'";
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparseable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case '?': case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static PyObject *
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number, ndim;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return NULL;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
    ndim = ctx->head->field->type->ndim;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return NULL;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i])
            return PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
        if (*ts != ',' && *ts != ')')
            return PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim)
        return PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return NULL;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return Py_None;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
        CYTHON_FALLTHROUGH;
      case '?': case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if ((ctx->enc_type == *ts) && (got_Z == ctx->is_complex) &&
            (ctx->enc_packmode == ctx->new_packmode) && (!ctx->is_valid_array)) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
        CYTHON_FALLTHROUGH;
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (!__pyx_buffmt_parse_array(ctx, &ts)) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}

/* BufferGetAndValidate */
  static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (unlikely(info->buf == NULL)) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}
static void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static int __Pyx__GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  buf->buf = NULL;
  if (unlikely(__Pyx_GetBuffer(obj, buf, flags) == -1)) {
    __Pyx_ZeroBuffer(buf);
    return -1;
  }
  if (unlikely(buf->ndim != nd)) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if (unlikely((size_t)buf->itemsize != dtype->size)) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_SafeReleaseBuffer(buf);
  return -1;
}

/* BytesEquals */
  static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
  static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* None */
  static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t a, Py_ssize_t b) {
    Py_ssize_t q = a / b;
    Py_ssize_t r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* GetAttr */
  static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* decode_c_string */
  static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    Py_ssize_t length;
    if (unlikely((start < 0) | (stop < 0))) {
        size_t slen = strlen(cstring);
        if (unlikely(slen > (size_t) PY_SSIZE_T_MAX)) {
            PyErr_SetString(PyExc_OverflowError,
                            "c-string too long to convert to Python");
            return NULL;
        }
        length = (Py_ssize_t) slen;
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (unlikely(stop <= start))
        return __Pyx_NewRef(__pyx_empty_unicode);
    length = stop - start;
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* GetAttr3 */
  static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* SwapException */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* Import */
  static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* FastTypeChecks */
  #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* PyIntBinop */
  #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* None */
  static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* None */
  static CYTHON_INLINE long __Pyx_div_long(long a, long b) {
    long q = a / b;
    long r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* ImportFrom */
  static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
  static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* CallNextTpDealloc */
  static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

/* CallNextTpTraverse */
  static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
  static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* PyObject_GenericGetAttrNoDict */
  #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
  #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
  static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* TypeImport */
  #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
  static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* PyObjectGetAttrStrNoError */
  static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
  static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* ClassMethod */
  static PyObject* __Pyx_Method_ClassMethod(PyObject *method) {
#if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM <= 0x05080000
    if (PyObject_TypeCheck(method, &PyWrapperDescr_Type)) {
        return PyClassMethod_New(method);
    }
#else
#if CYTHON_COMPILING_IN_PYSTON || CYTHON_COMPILING_IN_PYPY
    if (PyMethodDescr_Check(method))
#else
    #if PY_MAJOR_VERSION == 2
    static PyTypeObject *methoddescr_type = NULL;
    if (methoddescr_type == NULL) {
       PyObject *meth = PyObject_GetAttrString((PyObject*)&PyList_Type, "append");
       if (!meth) return NULL;
       methoddescr_type = Py_TYPE(meth);
       Py_DECREF(meth);
    }
    #else
    PyTypeObject *methoddescr_type = &PyMethodDescr_Type;
    #endif
    if (__Pyx_TypeCheck(method, methoddescr_type))
#endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject *)method;
        #if PY_VERSION_HEX < 0x03020000
        PyTypeObject *d_type = descr->d_type;
        #else
        PyTypeObject *d_type = descr->d_common.d_type;
        #endif
        return PyDescr_NewClassMethod(d_type, descr->d_method);
    }
#endif
    else if (PyMethod_Check(method)) {
        return PyClassMethod_New(PyMethod_GET_FUNCTION(method));
    }
    else {
        return PyClassMethod_New(method);
    }
}

/* GetNameInClass */
  static PyObject *__Pyx_GetGlobalNameAfterAttributeLookup(PyObject *name) {
    PyObject *result;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    __Pyx_GetModuleGlobalNameUncached(result, name);
    return result;
}
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name) {
    PyObject *result;
    result = __Pyx_PyObject_GetAttrStr(nmspace, name);
    if (!result) {
        result = __Pyx_GetGlobalNameAfterAttributeLookup(name);
    }
    return result;
}

/* FusedFunction */
  static PyObject *
__pyx_FusedFunction_New(PyMethodDef *ml, int flags,
                        PyObject *qualname, PyObject *closure,
                        PyObject *module, PyObject *globals,
                        PyObject *code)
{
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_FusedFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        __pyx_FusedFunctionObject *fusedfunc = (__pyx_FusedFunctionObject *) op;
        fusedfunc->__signatures__ = NULL;
        fusedfunc->type = NULL;
        fusedfunc->self = NULL;
        PyObject_GC_Track(op);
    }
    return op;
}
static void
__pyx_FusedFunction_dealloc(__pyx_FusedFunctionObject *self)
{
    PyObject_GC_UnTrack(self);
    Py_CLEAR(self->self);
    Py_CLEAR(self->type);
    Py_CLEAR(self->__signatures__);
    __Pyx__CyFunction_dealloc((__pyx_CyFunctionObject *) self);
}
static int
__pyx_FusedFunction_traverse(__pyx_FusedFunctionObject *self,
                             visitproc visit,
                             void *arg)
{
    Py_VISIT(self->self);
    Py_VISIT(self->type);
    Py_VISIT(self->__signatures__);
    return __Pyx_CyFunction_traverse((__pyx_CyFunctionObject *) self, visit, arg);
}
static int
__pyx_FusedFunction_clear(__pyx_FusedFunctionObject *self)
{
    Py_CLEAR(self->self);
    Py_CLEAR(self->type);
    Py_CLEAR(self->__signatures__);
    return __Pyx_CyFunction_clear((__pyx_CyFunctionObject *) self);
}
static PyObject *
__pyx_FusedFunction_descr_get(PyObject *self, PyObject *obj, PyObject *type)
{
    __pyx_FusedFunctionObject *func, *meth;
    func = (__pyx_FusedFunctionObject *) self;
    if (func->self || func->func.flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(self);
        return self;
    }
    if (obj == Py_None)
        obj = NULL;
    meth = (__pyx_FusedFunctionObject *) __pyx_FusedFunction_New(
                    ((PyCFunctionObject *) func)->m_ml,
                    ((__pyx_CyFunctionObject *) func)->flags,
                    ((__pyx_CyFunctionObject *) func)->func_qualname,
                    ((__pyx_CyFunctionObject *) func)->func_closure,
                    ((PyCFunctionObject *) func)->m_module,
                    ((__pyx_CyFunctionObject *) func)->func_globals,
                    ((__pyx_CyFunctionObject *) func)->func_code);
    if (!meth)
        return NULL;
    if (func->func.defaults) {
        PyObject **pydefaults;
        int i;
        if (!__Pyx_CyFunction_InitDefaults((PyObject*)meth,
                                      func->func.defaults_size,
                                      func->func.defaults_pyobjects)) {
            Py_XDECREF((PyObject*)meth);
            return NULL;
        }
        memcpy(meth->func.defaults, func->func.defaults, func->func.defaults_size);
        pydefaults = __Pyx_CyFunction_Defaults(PyObject *, meth);
        for (i = 0; i < meth->func.defaults_pyobjects; i++)
            Py_XINCREF(pydefaults[i]);
    }
    Py_XINCREF(func->func.func_classobj);
    meth->func.func_classobj = func->func.func_classobj;
    Py_XINCREF(func->__signatures__);
    meth->__signatures__ = func->__signatures__;
    Py_XINCREF(type);
    meth->type = type;
    Py_XINCREF(func->func.defaults_tuple);
    meth->func.defaults_tuple = func->func.defaults_tuple;
    if (func->func.flags & __Pyx_CYFUNCTION_CLASSMETHOD)
        obj = type;
    Py_XINCREF(obj);
    meth->self = obj;
    return (PyObject *) meth;
}
static PyObject *
_obj_to_str(PyObject *obj)
{
    if (PyType_Check(obj))
        return PyObject_GetAttr(obj, __pyx_n_s_name_2);
    else
        return PyObject_Str(obj);
}
static PyObject *
__pyx_FusedFunction_getitem(__pyx_FusedFunctionObject *self, PyObject *idx)
{
    PyObject *signature = NULL;
    PyObject *unbound_result_func;
    PyObject *result_func = NULL;
    if (self->__signatures__ == NULL) {
        PyErr_SetString(PyExc_TypeError, "Function is not fused");
        return NULL;
    }
    if (PyTuple_Check(idx)) {
        PyObject *list = PyList_New(0);
        Py_ssize_t n = PyTuple_GET_SIZE(idx);
        PyObject *sep = NULL;
        int i;
        if (unlikely(!list))
            return NULL;
        for (i = 0; i < n; i++) {
            int ret;
            PyObject *string;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            PyObject *item = PyTuple_GET_ITEM(idx, i);
#else
            PyObject *item = PySequence_ITEM(idx, i);  if (unlikely(!item)) goto __pyx_err;
#endif
            string = _obj_to_str(item);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
            Py_DECREF(item);
#endif
            if (unlikely(!string)) goto __pyx_err;
            ret = PyList_Append(list, string);
            Py_DECREF(string);
            if (unlikely(ret < 0)) goto __pyx_err;
        }
        sep = PyUnicode_FromString("|");
        if (likely(sep))
            signature = PyUnicode_Join(sep, list);
__pyx_err:
;
        Py_DECREF(list);
        Py_XDECREF(sep);
    } else {
        signature = _obj_to_str(idx);
    }
    if (!signature)
        return NULL;
    unbound_result_func = PyObject_GetItem(self->__signatures__, signature);
    if (unbound_result_func) {
        if (self->self || self->type) {
            __pyx_FusedFunctionObject *unbound = (__pyx_FusedFunctionObject *) unbound_result_func;
            Py_CLEAR(unbound->func.func_classobj);
            Py_XINCREF(self->func.func_classobj);
            unbound->func.func_classobj = self->func.func_classobj;
            result_func = __pyx_FusedFunction_descr_get(unbound_result_func,
                                                        self->self, self->type);
        } else {
            result_func = unbound_result_func;
            Py_INCREF(result_func);
        }
    }
    Py_DECREF(signature);
    Py_XDECREF(unbound_result_func);
    return result_func;
}
static PyObject *
__pyx_FusedFunction_callfunction(PyObject *func, PyObject *args, PyObject *kw)
{
     __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    int static_specialized = (cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD &&
                              !((__pyx_FusedFunctionObject *) func)->__signatures__);
    if (cyfunc->flags & __Pyx_CYFUNCTION_CCLASS && !static_specialized) {
        return __Pyx_CyFunction_CallAsMethod(func, args, kw);
    } else {
        return __Pyx_CyFunction_Call(func, args, kw);
    }
}
static PyObject *
__pyx_FusedFunction_call(PyObject *func, PyObject *args, PyObject *kw)
{
    __pyx_FusedFunctionObject *binding_func = (__pyx_FusedFunctionObject *) func;
    Py_ssize_t argc = PyTuple_GET_SIZE(args);
    PyObject *new_args = NULL;
    __pyx_FusedFunctionObject *new_func = NULL;
    PyObject *result = NULL;
    PyObject *self = NULL;
    int is_staticmethod = binding_func->func.flags & __Pyx_CYFUNCTION_STATICMETHOD;
    int is_classmethod = binding_func->func.flags & __Pyx_CYFUNCTION_CLASSMETHOD;
    if (binding_func->self) {
        Py_ssize_t i;
        new_args = PyTuple_New(argc + 1);
        if (!new_args)
            return NULL;
        self = binding_func->self;
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
        Py_INCREF(self);
#endif
        Py_INCREF(self);
        PyTuple_SET_ITEM(new_args, 0, self);
        for (i = 0; i < argc; i++) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            PyObject *item = PyTuple_GET_ITEM(args, i);
            Py_INCREF(item);
#else
            PyObject *item = PySequence_ITEM(args, i);  if (unlikely(!item)) goto bad;
#endif
            PyTuple_SET_ITEM(new_args, i + 1, item);
        }
        args = new_args;
    } else if (binding_func->type) {
        if (argc < 1) {
            PyErr_SetString(PyExc_TypeError, "Need at least one argument, 0 given.");
            return NULL;
        }
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        self = PyTuple_GET_ITEM(args, 0);
#else
        self = PySequence_ITEM(args, 0);  if (unlikely(!self)) return NULL;
#endif
    }
    if (self && !is_classmethod && !is_staticmethod) {
        int is_instance = PyObject_IsInstance(self, binding_func->type);
        if (unlikely(!is_instance)) {
            PyErr_Format(PyExc_TypeError,
                         "First argument should be of type %.200s, got %.200s.",
                         ((PyTypeObject *) binding_func->type)->tp_name,
                         Py_TYPE(self)->tp_name);
            goto bad;
        } else if (unlikely(is_instance == -1)) {
            goto bad;
        }
    }
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
    Py_XDECREF(self);
    self = NULL;
#endif
    if (binding_func->__signatures__) {
        PyObject *tup;
        if (is_staticmethod && binding_func->func.flags & __Pyx_CYFUNCTION_CCLASS) {
            tup = PyTuple_Pack(3, args,
                               kw == NULL ? Py_None : kw,
                               binding_func->func.defaults_tuple);
            if (unlikely(!tup)) goto bad;
            new_func = (__pyx_FusedFunctionObject *) __Pyx_CyFunction_CallMethod(
                func, binding_func->__signatures__, tup, NULL);
        } else {
            tup = PyTuple_Pack(4, binding_func->__signatures__, args,
                               kw == NULL ? Py_None : kw,
                               binding_func->func.defaults_tuple);
            if (unlikely(!tup)) goto bad;
            new_func = (__pyx_FusedFunctionObject *) __pyx_FusedFunction_callfunction(func, tup, NULL);
        }
        Py_DECREF(tup);
        if (unlikely(!new_func))
            goto bad;
        Py_XINCREF(binding_func->func.func_classobj);
        Py_CLEAR(new_func->func.func_classobj);
        new_func->func.func_classobj = binding_func->func.func_classobj;
        func = (PyObject *) new_func;
    }
    result = __pyx_FusedFunction_callfunction(func, args, kw);
bad:
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
    Py_XDECREF(self);
#endif
    Py_XDECREF(new_args);
    Py_XDECREF((PyObject *) new_func);
    return result;
}
static PyMemberDef __pyx_FusedFunction_members[] = {
    {(char *) "__signatures__",
     T_OBJECT,
     offsetof(__pyx_FusedFunctionObject, __signatures__),
     READONLY,
     0},
    {0, 0, 0, 0, 0},
};
static PyMappingMethods __pyx_FusedFunction_mapping_methods = {
    0,
    (binaryfunc) __pyx_FusedFunction_getitem,
    0,
};
static PyTypeObject __pyx_FusedFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "fused_cython_function",
    sizeof(__pyx_FusedFunctionObject),
    0,
    (destructor) __pyx_FusedFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    0,
    0,
    0,
    &__pyx_FusedFunction_mapping_methods,
    0,
    (ternaryfunc) __pyx_FusedFunction_call,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    0,
    (traverseproc) __pyx_FusedFunction_traverse,
    (inquiry) __pyx_FusedFunction_clear,
    0,
    0,
    0,
    0,
    0,
    __pyx_FusedFunction_members,
    __pyx_CyFunction_getsets,
    &__pyx_CyFunctionType_type,
    0,
    __pyx_FusedFunction_descr_get,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
    0,
#endif
};
static int __pyx_FusedFunction_init(void) {
    __pyx_FusedFunctionType_type.tp_base = __pyx_CyFunctionType;
    __pyx_FusedFunctionType = __Pyx_FetchCommonType(&__pyx_FusedFunctionType_type);
    if (__pyx_FusedFunctionType == NULL) {
        return -1;
    }
    return 0;
}

/* CLineInTraceback */
  #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
  static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
  #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
        if (__Pyx_TypeCheck(obj, __pyx_array_type)) return __pyx_array_getbuffer(obj, view, flags);
        if (__Pyx_TypeCheck(obj, __pyx_memoryview_type)) return __pyx_memoryview_getbuffer(obj, view, flags);
    PyErr_Format(PyExc_TypeError, "'%.200s' does not have the buffer interface", Py_TYPE(obj)->tp_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
    if ((0)) {}
    view->obj = NULL;
    Py_DECREF(obj);
}
#endif


  /* MemviewSliceIsContig */
  static int
__pyx_memviewslice_is_contig(const __Pyx_memviewslice mvs, char order, int ndim)
{
    int i, index, step, start;
    Py_ssize_t itemsize = mvs.memview->view.itemsize;
    if (order == 'F') {
        step = 1;
        start = 0;
    } else {
        step = -1;
        start = ndim - 1;
    }
    for (i = 0; i < ndim; i++) {
        index = start + step * i;
        if (mvs.suboffsets[index] >= 0 || mvs.strides[index] != itemsize)
            return 0;
        itemsize *= mvs.shape[index];
    }
    return 1;
}

/* OverlappingSlices */
  static void
__pyx_get_array_memory_extents(__Pyx_memviewslice *slice,
                               void **out_start, void **out_end,
                               int ndim, size_t itemsize)
{
    char *start, *end;
    int i;
    start = end = slice->data;
    for (i = 0; i < ndim; i++) {
        Py_ssize_t stride = slice->strides[i];
        Py_ssize_t extent = slice->shape[i];
        if (extent == 0) {
            *out_start = *out_end = start;
            return;
        } else {
            if (stride > 0)
                end += stride * (extent - 1);
            else
                start += stride * (extent - 1);
        }
    }
    *out_start = start;
    *out_end = end + itemsize;
}
static int
__pyx_slices_overlap(__Pyx_memviewslice *slice1,
                     __Pyx_memviewslice *slice2,
                     int ndim, size_t itemsize)
{
    void *start1, *end1, *start2, *end2;
    __pyx_get_array_memory_extents(slice1, &start1, &end1, ndim, itemsize);
    __pyx_get_array_memory_extents(slice2, &start2, &end2, ndim, itemsize);
    return (start1 < end2) && (start2 < end1);
}

/* Capsule */
  static CYTHON_INLINE PyObject *
__pyx_capsule_create(void *p, CYTHON_UNUSED const char *sig)
{
    PyObject *cobj;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(p, sig, NULL);
#else
    cobj = PyCObject_FromVoidPtr(p, NULL);
#endif
    return cobj;
}

/* TypeInfoCompare */
  static int
__pyx_typeinfo_cmp(__Pyx_TypeInfo *a, __Pyx_TypeInfo *b)
{
    int i;
    if (!a || !b)
        return 0;
    if (a == b)
        return 1;
    if (a->size != b->size || a->typegroup != b->typegroup ||
            a->is_unsigned != b->is_unsigned || a->ndim != b->ndim) {
        if (a->typegroup == 'H' || b->typegroup == 'H') {
            return a->size == b->size;
        } else {
            return 0;
        }
    }
    if (a->ndim) {
        for (i = 0; i < a->ndim; i++)
            if (a->arraysize[i] != b->arraysize[i])
                return 0;
    }
    if (a->typegroup == 'S') {
        if (a->flags != b->flags)
            return 0;
        if (a->fields || b->fields) {
            if (!(a->fields && b->fields))
                return 0;
            for (i = 0; a->fields[i].type && b->fields[i].type; i++) {
                __Pyx_StructField *field_a = a->fields + i;
                __Pyx_StructField *field_b = b->fields + i;
                if (field_a->offset != field_b->offset ||
                    !__pyx_typeinfo_cmp(field_a->type, field_b->type))
                    return 0;
            }
            return !a->fields[i].type && !b->fields[i].type;
        }
    }
    return 1;
}

/* MemviewSliceValidateAndInit */
  static int
__pyx_check_strides(Py_buffer *buf, int dim, int ndim, int spec)
{
    if (buf->shape[dim] <= 1)
        return 1;
    if (buf->strides) {
        if (spec & __Pyx_MEMVIEW_CONTIG) {
            if (spec & (__Pyx_MEMVIEW_PTR|__Pyx_MEMVIEW_FULL)) {
                if (unlikely(buf->strides[dim] != sizeof(void *))) {
                    PyErr_Format(PyExc_ValueError,
                                 "Buffer is not indirectly contiguous "
                                 "in dimension %d.", dim);
                    goto fail;
                }
            } else if (unlikely(buf->strides[dim] != buf->itemsize)) {
                PyErr_SetString(PyExc_ValueError,
                                "Buffer and memoryview are not contiguous "
                                "in the same dimension.");
                goto fail;
            }
        }
        if (spec & __Pyx_MEMVIEW_FOLLOW) {
            Py_ssize_t stride = buf->strides[dim];
            if (stride < 0)
                stride = -stride;
            if (unlikely(stride < buf->itemsize)) {
                PyErr_SetString(PyExc_ValueError,
                                "Buffer and memoryview are not contiguous "
                                "in the same dimension.");
                goto fail;
            }
        }
    } else {
        if (unlikely(spec & __Pyx_MEMVIEW_CONTIG && dim != ndim - 1)) {
            PyErr_Format(PyExc_ValueError,
                         "C-contiguous buffer is not contiguous in "
                         "dimension %d", dim);
            goto fail;
        } else if (unlikely(spec & (__Pyx_MEMVIEW_PTR))) {
            PyErr_Format(PyExc_ValueError,
                         "C-contiguous buffer is not indirect in "
                         "dimension %d", dim);
            goto fail;
        } else if (unlikely(buf->suboffsets)) {
            PyErr_SetString(PyExc_ValueError,
                            "Buffer exposes suboffsets but no strides");
            goto fail;
        }
    }
    return 1;
fail:
    return 0;
}
static int
__pyx_check_suboffsets(Py_buffer *buf, int dim, CYTHON_UNUSED int ndim, int spec)
{
    if (spec & __Pyx_MEMVIEW_DIRECT) {
        if (unlikely(buf->suboffsets && buf->suboffsets[dim] >= 0)) {
            PyErr_Format(PyExc_ValueError,
                         "Buffer not compatible with direct access "
                         "in dimension %d.", dim);
            goto fail;
        }
    }
    if (spec & __Pyx_MEMVIEW_PTR) {
        if (unlikely(!buf->suboffsets || (buf->suboffsets[dim] < 0))) {
            PyErr_Format(PyExc_ValueError,
                         "Buffer is not indirectly accessible "
                         "in dimension %d.", dim);
            goto fail;
        }
    }
    return 1;
fail:
    return 0;
}
static int
__pyx_verify_contig(Py_buffer *buf, int ndim, int c_or_f_flag)
{
    int i;
    if (c_or_f_flag & __Pyx_IS_F_CONTIG) {
        Py_ssize_t stride = 1;
        for (i = 0; i < ndim; i++) {
            if (unlikely(stride * buf->itemsize != buf->strides[i]  &&  buf->shape[i] > 1)) {
                PyErr_SetString(PyExc_ValueError,
                    "Buffer not fortran contiguous.");
                goto fail;
            }
            stride = stride * buf->shape[i];
        }
    } else if (c_or_f_flag & __Pyx_IS_C_CONTIG) {
        Py_ssize_t stride = 1;
        for (i = ndim - 1; i >- 1; i--) {
            if (unlikely(stride * buf->itemsize != buf->strides[i]  &&  buf->shape[i] > 1)) {
                PyErr_SetString(PyExc_ValueError,
                    "Buffer not C contiguous.");
                goto fail;
            }
            stride = stride * buf->shape[i];
        }
    }
    return 1;
fail:
    return 0;
}
static int __Pyx_ValidateAndInit_memviewslice(
                int *axes_specs,
                int c_or_f_flag,
                int buf_flags,
                int ndim,
                __Pyx_TypeInfo *dtype,
                __Pyx_BufFmt_StackElem stack[],
                __Pyx_memviewslice *memviewslice,
                PyObject *original_obj)
{
    struct __pyx_memoryview_obj *memview, *new_memview;
    __Pyx_RefNannyDeclarations
    Py_buffer *buf;
    int i, spec = 0, retval = -1;
    __Pyx_BufFmt_Context ctx;
    int from_memoryview = __pyx_memoryview_check(original_obj);
    __Pyx_RefNannySetupContext("ValidateAndInit_memviewslice", 0);
    if (from_memoryview && __pyx_typeinfo_cmp(dtype, ((struct __pyx_memoryview_obj *)
                                                            original_obj)->typeinfo)) {
        memview = (struct __pyx_memoryview_obj *) original_obj;
        new_memview = NULL;
    } else {
        memview = (struct __pyx_memoryview_obj *) __pyx_memoryview_new(
                                            original_obj, buf_flags, 0, dtype);
        new_memview = memview;
        if (unlikely(!memview))
            goto fail;
    }
    buf = &memview->view;
    if (unlikely(buf->ndim != ndim)) {
        PyErr_Format(PyExc_ValueError,
                "Buffer has wrong number of dimensions (expected %d, got %d)",
                ndim, buf->ndim);
        goto fail;
    }
    if (new_memview) {
        __Pyx_BufFmt_Init(&ctx, stack, dtype);
        if (unlikely(!__Pyx_BufFmt_CheckString(&ctx, buf->format))) goto fail;
    }
    if (unlikely((unsigned) buf->itemsize != dtype->size)) {
        PyErr_Format(PyExc_ValueError,
                     "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "u byte%s) "
                     "does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "u byte%s)",
                     buf->itemsize,
                     (buf->itemsize > 1) ? "s" : "",
                     dtype->name,
                     dtype->size,
                     (dtype->size > 1) ? "s" : "");
        goto fail;
    }
    if (buf->len > 0) {
        for (i = 0; i < ndim; i++) {
            spec = axes_specs[i];
            if (unlikely(!__pyx_check_strides(buf, i, ndim, spec)))
                goto fail;
            if (unlikely(!__pyx_check_suboffsets(buf, i, ndim, spec)))
                goto fail;
        }
        if (unlikely(buf->strides && !__pyx_verify_contig(buf, ndim, c_or_f_flag)))
            goto fail;
    }
    if (unlikely(__Pyx_init_memviewslice(memview, ndim, memviewslice,
                                         new_memview != NULL) == -1)) {
        goto fail;
    }
    retval = 0;
    goto no_fail;
fail:
    Py_XDECREF(new_memview);
    retval = -1;
no_fail:
    __Pyx_RefNannyFinishContext();
    return retval;
}

/* ObjectToMemviewSlice */
  static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_float(PyObject *obj, int writable_flag) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS_RO | writable_flag, 1,
                                                 &__Pyx_TypeInfo_float, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

/* ObjectToMemviewSlice */
  static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_double(PyObject *obj, int writable_flag) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS_RO | writable_flag, 1,
                                                 &__Pyx_TypeInfo_double, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

/* CIntFromPyVerify */
  #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

static PyObject* __pyx_convert__to_py_struct____pyx_t_12ms_deisotope_2_c_8peak_set_index_cell(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell s) {
    PyObject* res;
    PyObject* member;
    res = __Pyx_PyDict_NewPresized(3); if (unlikely(!res)) return NULL;
    member = PyFloat_FromDouble(s.center_value); if (unlikely(!member)) goto bad;
    if (unlikely(PyDict_SetItem(res, __pyx_n_s_center_value, member) < 0)) goto bad;
    Py_DECREF(member);
    member = __Pyx_PyInt_FromSize_t(s.start); if (unlikely(!member)) goto bad;
    if (unlikely(PyDict_SetItem(res, __pyx_n_s_start, member) < 0)) goto bad;
    Py_DECREF(member);
    member = __Pyx_PyInt_FromSize_t(s.end); if (unlikely(!member)) goto bad;
    if (unlikely(PyDict_SetItem(res, __pyx_n_s_end, member) < 0)) goto bad;
    Py_DECREF(member);
    return res;
    bad:
    Py_XDECREF(member);
    Py_DECREF(res);
    return NULL;
  }
  /* Declarations */
  #if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
  #if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
  #if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
  #if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* MemviewSliceCopyTemplate */
  static __Pyx_memviewslice
__pyx_memoryview_copy_new_contig(const __Pyx_memviewslice *from_mvs,
                                 const char *mode, int ndim,
                                 size_t sizeof_dtype, int contig_flag,
                                 int dtype_is_object)
{
    __Pyx_RefNannyDeclarations
    int i;
    __Pyx_memviewslice new_mvs = { 0, 0, { 0 }, { 0 }, { 0 } };
    struct __pyx_memoryview_obj *from_memview = from_mvs->memview;
    Py_buffer *buf = &from_memview->view;
    PyObject *shape_tuple = NULL;
    PyObject *temp_int = NULL;
    struct __pyx_array_obj *array_obj = NULL;
    struct __pyx_memoryview_obj *memview_obj = NULL;
    __Pyx_RefNannySetupContext("__pyx_memoryview_copy_new_contig", 0);
    for (i = 0; i < ndim; i++) {
        if (unlikely(from_mvs->suboffsets[i] >= 0)) {
            PyErr_Format(PyExc_ValueError, "Cannot copy memoryview slice with "
                                           "indirect dimensions (axis %d)", i);
            goto fail;
        }
    }
    shape_tuple = PyTuple_New(ndim);
    if (unlikely(!shape_tuple)) {
        goto fail;
    }
    __Pyx_GOTREF(shape_tuple);
    for(i = 0; i < ndim; i++) {
        temp_int = PyInt_FromSsize_t(from_mvs->shape[i]);
        if(unlikely(!temp_int)) {
            goto fail;
        } else {
            PyTuple_SET_ITEM(shape_tuple, i, temp_int);
            temp_int = NULL;
        }
    }
    array_obj = __pyx_array_new(shape_tuple, sizeof_dtype, buf->format, (char *) mode, NULL);
    if (unlikely(!array_obj)) {
        goto fail;
    }
    __Pyx_GOTREF(array_obj);
    memview_obj = (struct __pyx_memoryview_obj *) __pyx_memoryview_new(
                                    (PyObject *) array_obj, contig_flag,
                                    dtype_is_object,
                                    from_mvs->memview->typeinfo);
    if (unlikely(!memview_obj))
        goto fail;
    if (unlikely(__Pyx_init_memviewslice(memview_obj, ndim, &new_mvs, 1) < 0))
        goto fail;
    if (unlikely(__pyx_memoryview_copy_contents(*from_mvs, new_mvs, ndim, ndim,
                                                dtype_is_object) < 0))
        goto fail;
    goto no_fail;
fail:
    __Pyx_XDECREF(new_mvs.memview);
    new_mvs.memview = NULL;
    new_mvs.data = NULL;
no_fail:
    __Pyx_XDECREF(shape_tuple);
    __Pyx_XDECREF(temp_int);
    __Pyx_XDECREF(array_obj);
    __Pyx_RefNannyFinishContext();
    return new_mvs;
}

/* MemviewSliceInit */
  static int
__Pyx_init_memviewslice(struct __pyx_memoryview_obj *memview,
                        int ndim,
                        __Pyx_memviewslice *memviewslice,
                        int memview_is_new_reference)
{
    __Pyx_RefNannyDeclarations
    int i, retval=-1;
    Py_buffer *buf = &memview->view;
    __Pyx_RefNannySetupContext("init_memviewslice", 0);
    if (unlikely(memviewslice->memview || memviewslice->data)) {
        PyErr_SetString(PyExc_ValueError,
            "memviewslice is already initialized!");
        goto fail;
    }
    if (buf->strides) {
        for (i = 0; i < ndim; i++) {
            memviewslice->strides[i] = buf->strides[i];
        }
    } else {
        Py_ssize_t stride = buf->itemsize;
        for (i = ndim - 1; i >= 0; i--) {
            memviewslice->strides[i] = stride;
            stride *= buf->shape[i];
        }
    }
    for (i = 0; i < ndim; i++) {
        memviewslice->shape[i]   = buf->shape[i];
        if (buf->suboffsets) {
            memviewslice->suboffsets[i] = buf->suboffsets[i];
        } else {
            memviewslice->suboffsets[i] = -1;
        }
    }
    memviewslice->memview = memview;
    memviewslice->data = (char *)buf->buf;
    if (__pyx_add_acquisition_count(memview) == 0 && !memview_is_new_reference) {
        Py_INCREF(memview);
    }
    retval = 0;
    goto no_fail;
fail:
    memviewslice->memview = 0;
    memviewslice->data = 0;
    retval = -1;
no_fail:
    __Pyx_RefNannyFinishContext();
    return retval;
}
#ifndef Py_NO_RETURN
#define Py_NO_RETURN
#endif
static void __pyx_fatalerror(const char *fmt, ...) Py_NO_RETURN {
    va_list vargs;
    char msg[200];
#ifdef HAVE_STDARG_PROTOTYPES
    va_start(vargs, fmt);
#else
    va_start(vargs);
#endif
    vsnprintf(msg, 200, fmt, vargs);
    va_end(vargs);
    Py_FatalError(msg);
}
static CYTHON_INLINE int
__pyx_add_acquisition_count_locked(__pyx_atomic_int *acquisition_count,
                                   PyThread_type_lock lock)
{
    int result;
    PyThread_acquire_lock(lock, 1);
    result = (*acquisition_count)++;
    PyThread_release_lock(lock);
    return result;
}
static CYTHON_INLINE int
__pyx_sub_acquisition_count_locked(__pyx_atomic_int *acquisition_count,
                                   PyThread_type_lock lock)
{
    int result;
    PyThread_acquire_lock(lock, 1);
    result = (*acquisition_count)--;
    PyThread_release_lock(lock);
    return result;
}
static CYTHON_INLINE void
__Pyx_INC_MEMVIEW(__Pyx_memviewslice *memslice, int have_gil, int lineno)
{
    int first_time;
    struct __pyx_memoryview_obj *memview = memslice->memview;
    if (unlikely(!memview || (PyObject *) memview == Py_None))
        return;
    if (unlikely(__pyx_get_slice_count(memview) < 0))
        __pyx_fatalerror("Acquisition count is %d (line %d)",
                         __pyx_get_slice_count(memview), lineno);
    first_time = __pyx_add_acquisition_count(memview) == 0;
    if (unlikely(first_time)) {
        if (have_gil) {
            Py_INCREF((PyObject *) memview);
        } else {
            PyGILState_STATE _gilstate = PyGILState_Ensure();
            Py_INCREF((PyObject *) memview);
            PyGILState_Release(_gilstate);
        }
    }
}
static CYTHON_INLINE void __Pyx_XDEC_MEMVIEW(__Pyx_memviewslice *memslice,
                                             int have_gil, int lineno) {
    int last_time;
    struct __pyx_memoryview_obj *memview = memslice->memview;
    if (unlikely(!memview || (PyObject *) memview == Py_None)) {
        memslice->memview = NULL;
        return;
    }
    if (unlikely(__pyx_get_slice_count(memview) <= 0))
        __pyx_fatalerror("Acquisition count is %d (line %d)",
                         __pyx_get_slice_count(memview), lineno);
    last_time = __pyx_sub_acquisition_count(memview) == 1;
    memslice->data = NULL;
    if (unlikely(last_time)) {
        if (have_gil) {
            Py_CLEAR(memslice->memview);
        } else {
            PyGILState_STATE _gilstate = PyGILState_Ensure();
            Py_CLEAR(memslice->memview);
            PyGILState_Release(_gilstate);
        }
    } else {
        memslice->memview = NULL;
    }
}

/* CIntFromPy */
  static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_uint32(npy_uint32 value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const npy_uint32 neg_one = (npy_uint32) -1, const_zero = (npy_uint32) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(npy_uint32) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_uint32) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(npy_uint32) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(npy_uint32) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(npy_uint32) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(npy_uint32),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* BytesContains */
  static CYTHON_INLINE int __Pyx_BytesContains(PyObject* bytes, char character) {
    const Py_ssize_t length = PyBytes_GET_SIZE(bytes);
    char* char_start = PyBytes_AS_STRING(bytes);
    return memchr(char_start, (unsigned char)character, (size_t)length) != NULL;
}

/* ImportNumPyArray */
  static PyObject* __Pyx__ImportNumPyArray(void) {
    PyObject *numpy_module, *ndarray_object = NULL;
    numpy_module = __Pyx_Import(__pyx_n_s_numpy, NULL, 0);
    if (likely(numpy_module)) {
        ndarray_object = PyObject_GetAttrString(numpy_module, "ndarray");
        Py_DECREF(numpy_module);
    }
    if (unlikely(!ndarray_object)) {
        PyErr_Clear();
    }
    if (unlikely(!ndarray_object || !PyObject_TypeCheck(ndarray_object, &PyType_Type))) {
        Py_XDECREF(ndarray_object);
        Py_INCREF(Py_None);
        ndarray_object = Py_None;
    }
    return ndarray_object;
}
static CYTHON_INLINE PyObject* __Pyx_ImportNumPyArrayTypeIfAvailable(void) {
    if (unlikely(!__pyx_numpy_ndarray)) {
        __pyx_numpy_ndarray = __Pyx__ImportNumPyArray();
    }
    Py_INCREF(__pyx_numpy_ndarray);
    return __pyx_numpy_ndarray;
}

/* CIntFromPy */
  static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE char __Pyx_PyInt_As_char(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const char neg_one = (char) -1, const_zero = (char) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(char) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(char, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (char) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char) 0;
                case  1: __PYX_VERIFY_RETURN_INT(char, digit, digits[0])
                case 2:
                    if (8 * sizeof(char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 2 * PyLong_SHIFT) {
                            return (char) (((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 3 * PyLong_SHIFT) {
                            return (char) (((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 4 * PyLong_SHIFT) {
                            return (char) (((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (char) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(char) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(char) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char) 0;
                case -1: __PYX_VERIFY_RETURN_INT(char, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(char,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(char) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                            return (char) (((char)-1)*(((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                            return (char) ((((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                            return (char) (((char)-1)*(((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                            return (char) ((((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 4 * PyLong_SHIFT) {
                            return (char) (((char)-1)*(((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 4 * PyLong_SHIFT) {
                            return (char) ((((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(char) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(char) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            char val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (char) -1;
        }
    } else {
        char val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (char) -1;
        val = __Pyx_PyInt_As_char(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to char");
    return (char) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to char");
    return (char) -1;
}

/* PyObjectCallMethod1 */
  static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* CoroutineBase */
  #include <structmember.h>
#include <frameobject.h>
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(CYTHON_UNUSED __pyx_CoroutineObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(CYTHON_UNUSED PyObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(CYTHON_UNUSED PyObject *gen, PyObject *value, CYTHON_UNUSED int closing) {
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_type) {
        #if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
        #else
        if (exc_state->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_state->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            assert(f->f_back == NULL);
            #if PY_VERSION_HEX >= 0x030B00A1
            f->f_back = PyThreadState_GetFrame(tstate);
            #else
            Py_XINCREF(tstate->frame);
            f->f_back = tstate->frame;
            #endif
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    self->is_running = 1;
    retval = self->body((PyObject *) self, tstate, value);
    self->is_running = 0;
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *exc_tb = exc_state->exc_traceback;
    if (likely(exc_tb)) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
#endif
    }
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(CYTHON_UNUSED PyObject* gen, PyObject *retval) {
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
            _PyGen_SetStopIterationValue(result);
        }
        Py_CLEAR(result);
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = Py_TYPE(yf)->tp_iternext(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf, NULL);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, CYTHON_UNUSED PyObject *arg) {
    return __Pyx_Coroutine_Close(self);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = PyObject_CallObject(meth, args);
            } else {
                ret = PyObject_CallFunctionObjArgs(meth, typ, val, tb, NULL);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (Py_REFCNT(self) > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    __Pyx_SET_REFCNT(self, 1);
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(Py_REFCNT(self) > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = Py_REFCNT(self);
        _Py_NewReference(self);
        __Pyx_SET_REFCNT(self, refcnt);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(Py_TYPE(self)) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_name;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_qualname;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *frame = self->gi_frame;
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            __pyx_d,                 /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        if (unlikely(!frame))
            return NULL;
        self->gi_frame = frame;
    }
    Py_INCREF(frame);
    return frame;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
  static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
  #ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
  static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {(char *) "gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     (char*) PyDoc_STR("Frame of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
    0,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
  static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* VoidPtrExport */
  static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig) {
    PyObject *d;
    PyObject *cobj = 0;
    d = PyDict_GetItem(__pyx_d, __pyx_n_s_pyx_capi);
    Py_XINCREF(d);
    if (!d) {
        d = PyDict_New();
        if (!d)
            goto bad;
        if (__Pyx_PyObject_SetAttrStr(__pyx_m, __pyx_n_s_pyx_capi, d) < 0)
            goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItem(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* FunctionExport */
  static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* VoidPtrImport */
  #ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* FunctionImport */
  #ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
  static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
